import json
from t2wml.wikification.utility_functions import dict_to_kgtk

def create_kgtk_from_sparql_query_saved_as_json(query_path, output_path):
    with open(query_path, 'r', encoding="utf-8") as f:
        properties = json.load(f)
    properties_dict={}
    for property in properties:
        id=property["id"]
        properties_dict[id]=dict(label=property["itemLabel"],
                                description=property.get("itemDescription", ""),
                                data_type=property["data_type"])
    dict_to_kgtk(properties_dict, output_path)

def query_sparql_for_all_properties():
    query_str="""SELECT DISTINCT (?item as ?id) ?itemLabel ?itemDescription (?type as ?data_type)
                WHERE
                {
                ?item wdt:P31/wdt:P279* wd:Q18616576;
                        wikibase:propertyType ?type;
                SERVICE wikibase:label { bd:serviceParam wikibase:language "en".}
                }"""

preloaded_properties={
    "P5886": {
        "label": "mode of derivation",
        "description": "(qualifier) the way that a term is derived in P5191",
        "data_type": "WikibaseItem"
    },
    "P4153": {
        "label": "offset",
        "description": "qualifier of \"magic number\" for the number of bytes before the magic number to be searched in a file",
        "data_type": "Quantity"
    },
    "P3005": {
        "label": "valid in place",
        "description": "place where a statement is valid",
        "data_type": "WikibaseItem"
    },
    "P5428": {
        "label": "position of devoiced vowel",
        "description": "(qualifier) position of kana with devoiced vowel",
        "data_type": "Quantity"
    },
    "P5447": {
        "label": "lower limit",
        "description": "lower limit of a range or an interval; (qualifier) used with a dummy value which specifies the type of the range or interval",
        "data_type": "Quantity"
    },
    "P2719": {
        "label": "Hungarian-style transcription",
        "description": "transcription of a monolingual text value in a non-latin script according to the guidelines of the Research Institute for Linguistics of the Hungarian Academy of Sciences (Q7315098), Helyesírás (Q1035212), and Hungarian Wikipedia (Q53464)",
        "data_type": "String"
    },
    "P6149": {
        "label": "running mate",
        "description": "second person on a presidential or other electoral ticket",
        "data_type": "WikibaseItem"
    },
    "P6719": {
        "label": "specified by sense",
        "description": "(qualifier) descriptive sense that when attached to the hypernym, evaluates to the definition of the more specific subject sense",
        "data_type": "WikibaseSense"
    },
    "P4353": {
        "label": "nominated by",
        "description": "who nominated a person for an office or award; can be used as a qualifier",
        "data_type": "WikibaseItem"
    },
    "P7221": {
        "label": "link for periphrastic definition",
        "description": "optional qualifier to define the link between the concept and the lexeme",
        "data_type": "WikibaseSense"
    },
    "P2781": {
        "label": "race time",
        "description": "officially recorded time elapsed in a sports race",
        "data_type": "Quantity"
    },
    "P2937": {
        "label": "parliamentary term",
        "description": "term of a parliament or any deliberative assembly",
        "data_type": "WikibaseItem"
    },
    "P5448": {
        "label": "upper limit",
        "description": "upper limit of a range or an interval; (qualifier) used with a dummy value which specifies the type of the range or interval",
        "data_type": "Quantity"
    },
    "P2677": {
        "label": "relative position within image",
        "description": "position of a motif within a larger image, defined by  x-offset (from left), y-offset (down from top), width w, and height h of the crop region (as a percentage of the whole image) values",
        "data_type": "String"
    },
    "P2911": {
        "label": "time gap",
        "description": "time gap to winner time (P2781)",
        "data_type": "Quantity"
    },
    "P5040": {
        "label": "GHS hazard pictogram",
        "description": "qualifier for P4952 for GHS hazard pictograms",
        "data_type": "WikibaseItem"
    },
    "P4775": {
        "label": "lot number",
        "description": "takes a number; used as a qualifier in combination with new property for \"Wikidata:Property proposal/Auction sale\"; Auction sales are a significant event for artworks, generally occurring as part of a larger sale of goods by an art auction house",
        "data_type": "String"
    },
    "P4188": {
        "label": "Tibetan pinyin",
        "description": "official Chinese transliteration method for Tibetan",
        "data_type": "String"
    },
    "P3323": {
        "label": "opponent during disputation",
        "description": "participant in the defense of a doctoral thesis",
        "data_type": "WikibaseItem"
    },
    "P7125": {
        "label": "date of the latest one",
        "description": "qualifier: when was the latest element of quantity appears/took place",
        "data_type": "Time"
    },
    "P4189": {
        "label": "THL Simplified Phonetic Transcription",
        "description": "transliteration from Tibetan to Latin script with the method developed by the Tibetan and Himalayan Library (THL)",
        "data_type": "String"
    },
    "P2868": {
        "label": "subject has role",
        "description": "role/generic identity of the item (\"subject\"), also in the context of a statement. For the role of the value of the statement (\"object\"), use P3831 (\"object has role\"). For acting roles, use P453 (\"character role\"). For persons, use P39.",
        "data_type": "WikibaseItem"
    },
    "P3740": {
        "label": "number of works",
        "description": "qualifier on identifiers, eg for creators or locations, giving the number of works in the external database associated with the subject of the identifier",
        "data_type": "Quantity"
    },
    "P5481": {
        "label": "position of nasal sonant",
        "description": "(qualifier) position of kana with nasal sonant, which occasionally appears at the syllable of \"が\", \"ぎ\", \"ぐ\", \"げ\", or \"ご\" in some Japanese words",
        "data_type": "Quantity"
    },
    "P4240": {
        "label": "regular expression syntax",
        "description": "qualifier to indicate which syntax/variant of regular expressions applies to a particular pattern",
        "data_type": "WikibaseItem"
    },
    "P5042": {
        "label": "GHS precautionary statement",
        "description": "qualifier for P4952 for GHS P-statements",
        "data_type": "WikibaseItem"
    },
    "P6022": {
        "label": "expression, gesture or body pose",
        "description": "qualifier on \"depicts\" (P180) for human position of an individual in an artwork",
        "data_type": "WikibaseItem"
    },
    "P4626": {
        "label": "identity of object in context",
        "description": "qualifier to specify the identity that applies to a statement when the object of the statement has another identity covered by another item",
        "data_type": "WikibaseItem"
    },
    "P3149": {
        "label": "molecule conformation",
        "description": "qualifier to be used with property \"electric dipole moment\" (P2201)",
        "data_type": "WikibaseItem"
    },
    "P3027": {
        "label": "open period from",
        "description": "qualifier to P3025 (open days) for season when open days apply",
        "data_type": "WikibaseItem"
    },
    "P6552": {
        "label": "Twitter user numeric ID",
        "description": "numeric identifier for a user on Twitter; use as qualifier for P2002 \"Twitter username\"",
        "data_type": "ExternalId"
    },
    "P2744": {
        "label": "PASE name",
        "description": "standardised name (including disambiguation number) given to a person in the Prosopography of Anglo-Saxon England",
        "data_type": "String"
    },
    "P5592": {
        "label": "number of works accessible online",
        "description": "qualifier on external identifiers, for the number of works in a collection or items in a database which are freely accessible online (if different from P3740)",
        "data_type": "Quantity"
    },
    "P3831": {
        "label": "object has role",
        "description": "(qualifier) role or generic identity of the value of a statement (\"object\") in the context of that statement; for the role of the item the statement is on (\"subject\"), use P2868",
        "data_type": "WikibaseItem"
    },
    "P6001": {
        "label": "applies to people",
        "description": "qualifier to a specific group of people, members of a group or organization",
        "data_type": "WikibaseItem"
    },
    "P6833": {
        "label": "title in HTML",
        "description": "(qualifier) for title property (P1476), to write title with HTML markup, if the string can't render it normally. Allowed tags are: sup, sub, i",
        "data_type": "Monolingualtext"
    },
    "P6824": {
        "label": "replacement property",
        "description": "another property should be used, with or without the same value, while violating a constraint",
        "data_type": "WikibaseProperty"
    },
    "P3416": {
        "label": "end period",
        "description": "last period in which the statement is still valid (including the period-value)",
        "data_type": "WikibaseItem"
    },
    "P6835": {
        "label": "title in LaTeX",
        "description": "(qualifier) for title property (P1476), to write title in LaTeX/math notation, if the string can't render it normally. Enclose normal text with \"\\text{\" and \"}\"",
        "data_type": "Math"
    },
    "P5051": {
        "label": "towards",
        "description": "qualifier for \"adjacent station\" (P197) to indicate the terminal station(s) of a transportation line or service in that direction",
        "data_type": "WikibaseItem"
    },
    "P5596": {
        "label": "relative humidity",
        "description": "ratio of the partial pressure of water vapor to the equilibrium vapor pressure of water at a given temperature",
        "data_type": "Quantity"
    },
    "P4155": {
        "label": "separator",
        "description": "parameter for single-value constraint. Multiple values are allowed under that constraint if they have different values for this property.",
        "data_type": "WikibaseProperty"
    },
    "P5548": {
        "label": "derived from form",
        "description": "qualifier to indicate the form for \"derived from\" (P5191) or \"compound of\" (P5238)",
        "data_type": "WikibaseForm"
    },
    "P4945": {
        "label": "download link",
        "description": "URL which can be used to download a work",
        "data_type": "Url"
    },
    "P7008": {
        "label": "unabbreviated text",
        "description": "(qualifier) text in inscription with the abbreviations expanded in a fairly reliable way",
        "data_type": "Monolingualtext"
    },
    "P4878": {
        "label": "symbolizes",
        "description": "symbolism of an artistic element, qualifier for \"depicts\" statements",
        "data_type": "WikibaseItem"
    },
    "P3028": {
        "label": "open period to",
        "description": "qualifier to P3025 (open days) for season when open days apply",
        "data_type": "WikibaseItem"
    },
    "P3026": {
        "label": "closed on",
        "description": "exceptions to open days (P3025), usually special dates",
        "data_type": "WikibaseItem"
    },
    "P4900": {
        "label": "broader concept",
        "description": "qualifier to indicate a broader concept that the present item is part of, as mapped by an external source. The statement being qualified should be an exact match.",
        "data_type": "WikibaseItem"
    },
    "P4970": {
        "label": "alternate names",
        "description": "qualifier for alternate names given for a subject in a database entry",
        "data_type": "String"
    },
    "P4649": {
        "label": "identity of subject in context",
        "description": "qualifier for subjects which may have different identities which are covered by different items, the identity to which the qualified statement applies",
        "data_type": "WikibaseItem"
    },
    "P5168": {
        "label": "applies to name of item",
        "description": "(qualifier) label or alias name to which the claim applies (subject of the statement)",
        "data_type": "Monolingualtext"
    },
    "P7380": {
        "label": "identified in image by",
        "description": "qualifier for the identification of a value/object with a number, letter or sign inscribed in the image",
        "data_type": "String"
    },
    "P4070": {
        "label": "identifier shared with",
        "description": "qualifier, to be used on external identifier IDs, indicating another Wikidata item is also matched to this ID",
        "data_type": "WikibaseItem"
    },
    "P5830": {
        "label": "demonstrates form",
        "description": "(qualifier) the form of a lexeme which is demonstrated by a usage example",
        "data_type": "WikibaseForm"
    },
    "P7452": {
        "label": "reason for preferred rank",
        "description": "qualifier to allow the reason to be indicated why a particular statement should be considered preferred",
        "data_type": "WikibaseItem"
    },
    "P4271": {
        "label": "rating",
        "description": "qualifier to indicate a score given by the referenced source indicating the quality or completeness of the statement",
        "data_type": "WikibaseItem"
    },
    "P2836": {
        "label": "highest income threshold",
        "description": "highest income for which the tax rate applies, qualifier for P2834 (individual tax rate)",
        "data_type": "Quantity"
    },
    "P3274": {
        "label": "content deliverer",
        "description": "third party service (e.g. Google Books, Internet Archive, etc.) that made the content available where it was accessed",
        "data_type": "WikibaseItem"
    },
    "P5980": {
        "label": "derived from sense",
        "description": "qualifier to indicate the sense for \"derived from\" (P5191) or \"compound of\" (P5238)",
        "data_type": "WikibaseSense"
    },
    "P7379": {
        "label": "fineness",
        "description": "relative purity of precious metal, use  \"carat\" (Q1072404) or \"millesimal fineness\" (Q1790688) as units",
        "data_type": "Quantity"
    },
    "P6274": {
        "label": "concentration",
        "description": "qualifier: amount of substance in a mixture (solution) during measurement",
        "data_type": "Quantity"
    },
    "P6072": {
        "label": "demonstrates sense",
        "description": "(qualifier) the sense of a lexeme which is demonstrated by a usage example",
        "data_type": "WikibaseSense"
    },
    "P6607": {
        "label": "constraint clarification",
        "description": "qualifier to provide details on the function or purpose of a property constraint on a given property",
        "data_type": "Monolingualtext"
    },
    "P7141": {
        "label": "measure number",
        "description": "musical bars in a published musical work",
        "data_type": "String"
    },
    "P5041": {
        "label": "GHS hazard statement",
        "description": "qualifier for P4952 for GHS H-statements (also EUH-statements, AUH-statements)",
        "data_type": "WikibaseItem"
    },
    "P7367": {
        "label": "content descriptor",
        "description": "content descriptors for this video game or film",
        "data_type": "WikibaseItem"
    },
    "P5237": {
        "label": "pronunciation variety",
        "description": "qualifier for IPA transcription (P898), pronunciation audio (P443) or spoken text audio (P989) to indicate the associated spoken language variant",
        "data_type": "WikibaseItem"
    },
    "P3415": {
        "label": "start period",
        "description": "initial period in which the statement is valid (including the period-value)",
        "data_type": "WikibaseItem"
    },
    "P7018": {
        "label": "lexeme",
        "description": "(qualifier) lexeme version of monolingual text",
        "data_type": "WikibaseSense"
    },
    "P2701": {
        "label": "file format",
        "description": "file format, compression type, or ontology used in a file",
        "data_type": "WikibaseItem"
    },
    "P5800": {
        "label": "narrative role",
        "description": "narrative role of this character (should be used as a qualifier with P674 or restricted to a certain work using P642)",
        "data_type": "WikibaseItem"
    },
    "P2715": {
        "label": "elected in",
        "description": "qualifier for statements in position held to link the election in which a person gained a position from, or reelection in which the position was confirmed",
        "data_type": "WikibaseItem"
    },
    "P2700": {
        "label": "protocol",
        "description": "communication protocol to use to access a dataset or service",
        "data_type": "WikibaseItem"
    },
    "P5474": {
        "label": "section number",
        "description": "number assigned to a section of sportspeople in a sporting competition; e.g. \"3\" for a \"quarter-final 3\"; use together with property \"stage reached\" (P2443)",
        "data_type": "Quantity"
    },
    "P2676": {
        "label": "rating certificate ID",
        "description": "reference certificate identifier of content rating decisions",
        "data_type": "String"
    },
    "P3938": {
        "label": "named by",
        "description": "person or organisation that coined the name; use as qualifier for P2561 and its subproperties",
        "data_type": "WikibaseItem"
    },
    "P6208": {
        "label": "award rationale",
        "description": "qualifier to add short citation explaining the official reason for awarding a prize to a winner by the person or organization who gives the award. Do not use this property to annotate or add comments about a person's award. No quote marked needed.",
        "data_type": "Monolingualtext"
    },
    "P4680": {
        "label": "constraint scope",
        "description": "defines the scope where a constraint is checked – on the main value of a statement, on qualifiers, or on references",
        "data_type": "WikibaseItem"
    },
    "P3629": {
        "label": "age at event",
        "description": "the age of the subject according to the cited source at the time of an event. Used as a qualifier of significant event property",
        "data_type": "Quantity"
    },
    "P2842": {
        "label": "place of marriage",
        "description": "location where the marriage was celebrated. Use as qualifier for property \"spouse\" (P26)",
        "data_type": "WikibaseItem"
    },
    "P2835": {
        "label": "lowest income threshold",
        "description": "lowest income for which the tax rate applies, qualifier for P2834 (individual tax rate)",
        "data_type": "Quantity"
    },
    "P4633": {
        "label": "name of the character role",
        "description": "name by which a character role is recorded in a database or mentioned in the context of a work, use as qualifier for cast; notably a specific performance of a theatrical production of a play",
        "data_type": "String"
    },
    "P3294": {
        "label": "encoding",
        "description": "encoding according to which a code is to be interpreted (use only as a qualifier)",
        "data_type": "WikibaseItem"
    },
    "P6354": {
        "label": "closest approach",
        "description": "qualifier for minimum distance from an astronomical object that this spacecraft flies",
        "data_type": "Quantity"
    },
    "P7124": {
        "label": "date of the first one",
        "description": "qualifier: when was the first element of quantity appears/took place",
        "data_type": "Time"
    },
    "P5961": {
        "label": "depicted part",
        "description": "use as qualifier for \"depicts\" (P180)",
        "data_type": "WikibaseItem"
    },
    "P7469": {
        "label": "orientation",
        "description": "axis of alignment of an item, or direction perpendicular to its primary face (specify with P518 if necessary)",
        "data_type": "WikibaseItem"
    },
    "P7220": {
        "label": "lexeme for periphrastic definition",
        "description": "sense of a related lexeme used in periphrastic definition (such \"slow\" to define \"slowly\")",
        "data_type": "WikibaseSense"
    },
    "P5427": {
        "label": "position of accent nucleus",
        "description": "(qualifier) position of the accented mora in Japanese",
        "data_type": "Quantity"
    },
    "P4714": {
        "label": "title page number",
        "description": "qualifier to indicate the number of the page of a document in which the title page is located. Don't use for the actual cover page.",
        "data_type": "Quantity"
    },
    "P5054": {
        "label": "cabinet",
        "description": "qualifier for \"position held\" (P39) to indicate the cabinet to which belongs a minister",
        "data_type": "WikibaseItem"
    },
    "P4241": {
        "label": "refine date",
        "description": "qualifier to narrow down the precision of a date not fully specified by the Wikidata time datatype precision field",
        "data_type": "WikibaseItem"
    },
    "P7528": {
        "label": "statement is regarded as spoiler for",
        "description": "exposing this information might ruin the experience of receiving this work",
        "data_type": "WikibaseItem"
    },
    "P7338": {
        "label": "regnal ordinal",
        "description": "(qualifier) the ordinal number used with this name. Generally to distinguish it among persons with the same name who held the same office",
        "data_type": "String"
    },
    "P7315": {
        "label": "IP Code",
        "description": "identifier which classifies and rates the degree of protection provided by mechanical casings and electrical enclosures against intrusion, dust, accidental contact, and water",
        "data_type": "String"
    },
    "P3680": {
        "label": "statement supported by",
        "description": "entity that supports a given statement",
        "data_type": "WikibaseItem"
    },
    "P2916": {
        "label": "syntax clarification",
        "description": "qualifier for P1793 (regular expression): to provide a textual description of the regex syntax of a value. Should be displayable after the text \"The value for the property should match\" and be followed by the regex.",
        "data_type": "Monolingualtext"
    },
    "P4390": {
        "label": "mapping relation type",
        "description": "qualifier to more precisely define the relation of the item to the external identifier using SKOS",
        "data_type": "WikibaseItem"
    },
    "P7174": {
        "label": "school class",
        "description": "(qualifier) class of a given year/time period to which the person belonged in an educational institution",
        "data_type": "WikibaseItem"
    },
    "P4515": {
        "label": "Prisma ID",
        "description": "identifier for a film (movie) or person, in prisma.de",
        "data_type": "ExternalId"
    },
    "P4201": {
        "label": "PagesJaunes ID",
        "description": "identifier for a professional in France in the PagesJaunes website",
        "data_type": "ExternalId"
    },
    "P3370": {
        "label": "Géopatronyme ID",
        "description": "identifier of a family name in the Géopatronyme database",
        "data_type": "ExternalId"
    },
    "P1323": {
        "label": "Terminologia Anatomica 98 ID",
        "description": "Terminologia Anatomica (1998 edition) human anatomical terminology identifier",
        "data_type": "ExternalId"
    },
    "P3859": {
        "label": "Environment Ontology ID",
        "description": "ID from OBO Environment Ontology (without prefix)",
        "data_type": "ExternalId"
    },
    "P3289": {
        "label": "Cellosaurus ID",
        "description": "Cellosaurus cell line identifier",
        "data_type": "ExternalId"
    },
    "P4537": {
        "label": "Spider Ontology ID",
        "description": "identifier for the Spider Ontology which provides a controlled vocabulary for spider comparative biology including anatomical parts (e.g. leg, claw), behavior (e.g. courtship, combing) and products (i.g. silk, web, borrow)",
        "data_type": "ExternalId"
    },
    "P4294": {
        "label": "PROSPERO ID",
        "description": "identifier for a study, in the PROSPERO database",
        "data_type": "ExternalId"
    },
    "P4394": {
        "label": "NeuroNames ID (plain mode)",
        "description": "numeric identifier of a brain structure in the BrainInfo (NeuroNames) database in plain mode",
        "data_type": "ExternalId"
    },
    "P2452": {
        "label": "GeoNames feature code",
        "description": "identifier for feature classes in GeoNames",
        "data_type": "ExternalId"
    },
    "P1550": {
        "label": "Orphanet ID",
        "description": "identifier in the Orpha.net database (without ORPHA prefix)",
        "data_type": "ExternalId"
    },
    "P3583": {
        "label": "Surfline ID",
        "description": "identifier of a topic at surfline.com",
        "data_type": "ExternalId"
    },
    "P3895": {
        "label": "INAO product ID",
        "description": "identifier of a geographical indication in Institut national de l'origine et de la qualité",
        "data_type": "ExternalId"
    },
    "P1690": {
        "label": "ICD-10-PCS",
        "description": "identifier in the ICD-10-PCS (Procedure Coding System, International Classification of Diseases, 10th revision)",
        "data_type": "ExternalId"
    },
    "P2671": {
        "label": "Google Knowledge Graph ID",
        "description": "identifier for Google Knowledge Graph API (starting with \"/g/\"). IDs starting with \"/m/\" are Freebase IDs (P646)",
        "data_type": "ExternalId"
    },
    "P1051": {
        "label": "PSH ID",
        "description": "identifier in the authority database of the Czech Technical Library",
        "data_type": "ExternalId"
    },
    "P3743": {
        "label": "ITU/ISO/IEC object identifier",
        "description": "unique global identifier standardized by the ITU and ISO/IEC for naming any object, concept, or thing",
        "data_type": "ExternalId"
    },
    "P2759": {
        "label": "AUSNUT food ID",
        "description": "identifier of a food in the Australian Food, Supplement and Nutrient Database (AUSNUT) managed by Food Standards Australia New Zealand (FSANZ)",
        "data_type": "ExternalId"
    },
    "P3381": {
        "label": "File Format Wiki page ID",
        "description": "identifier on the \"Just Solve the File Format Problem\"/File Format(s) wiki",
        "data_type": "ExternalId"
    },
    "P2367": {
        "label": "Australian Stratigraphic Units Database ID",
        "description": "identifier for a stratigraphic unit listed in the Australian Stratigraphic Units Database",
        "data_type": "ExternalId"
    },
    "P3790": {
        "label": "AnimeCons.com guest ID",
        "description": "identifier for a person or group at AnimeCons.com",
        "data_type": "ExternalId"
    },
    "P3984": {
        "label": "subreddit",
        "description": "name of the Reddit forum associated to this subject",
        "data_type": "ExternalId"
    },
    "P4336": {
        "label": "Lloyd's Register Ship ID",
        "description": "Ship ID in Lloyd's Register of Ships",
        "data_type": "ExternalId"
    },
    "P2355": {
        "label": "UNESCO Atlas of the World's Languages in Danger ID",
        "description": "identifier for endangered languages listed by UNESCO",
        "data_type": "ExternalId"
    },
    "P1987": {
        "label": "MCN code",
        "description": "code for item in the Mercosur Common Nomenclature",
        "data_type": "String"
    },
    "P2760": {
        "label": "NUTTAB food ID",
        "description": "Identifier of a food within the Nutrient Tables for use in Australia (NUTTAB) which is managed by Food Standards Australia New Zealand (FSANZ)",
        "data_type": "ExternalId"
    },
    "P2357": {
        "label": "Classification of Instructional Programs code",
        "description": "code representing academic disciplines in the U.S. Department of Education's Classification of Instructional Programs",
        "data_type": "String"
    },
    "P3798": {
        "label": "Star Wars Databank ID",
        "description": "identifier of a fictional character and organization at Star Wars Databank",
        "data_type": "ExternalId"
    },
    "P1256": {
        "label": "Iconclass notation",
        "description": "Iconclass code that corresponds with an artistic theme or concept. For artworks, use P1257 (depicts Iconclass notation)",
        "data_type": "ExternalId"
    },
    "P2263": {
        "label": "ISOCAT id",
        "description": "numeric ID from the Data Category Registry",
        "data_type": "String"
    },
    "P3811": {
        "label": "Evidence & Conclusion Ontology ID",
        "description": "identifier in the Evidence & Conclusion Ontology for capturing evidence in biological research",
        "data_type": "ExternalId"
    },
    "P2689": {
        "label": "BARTOC ID",
        "description": "identifier in the BARTOC Terminology Registry",
        "data_type": "ExternalId"
    },
    "P3720": {
        "label": "GPnotebook ID",
        "description": "ID of a topic, in the British medical database GPnotebook",
        "data_type": "ExternalId"
    },
    "P4106": {
        "label": "Finnish archaeological heritage ID",
        "description": "identifier for the Archaeological heritage register for items in mainland Finland",
        "data_type": "ExternalId"
    },
    "P3854": {
        "label": "Soundtrack Collector ID",
        "description": "ID that characterizes a soundtrack at Soundtrack Collector (soundtrackcollector.com)",
        "data_type": "ExternalId"
    },
    "P2457": {
        "label": "Australian National Shipwreck ID",
        "description": "identifier for shipwrecks used by the Australian National Shipwreck Database",
        "data_type": "ExternalId"
    },
    "P2812": {
        "label": "MathWorld identifier",
        "description": "identifier for entries in MathWorld, online mathematics reference work",
        "data_type": "ExternalId"
    },
    "P2480": {
        "label": "IHO Hydrographic Dictionary (S-32) Number",
        "description": "Identification number for items in the Hydrographic Dictionary S-32, issued by International Hydrographic Organization (IHO)",
        "data_type": "ExternalId"
    },
    "P3328": {
        "label": "wurvoc.org measure ID",
        "description": "concept in the Ontology of units of Measure and related concepts (OM) 1.8 of wurvoc.org",
        "data_type": "ExternalId"
    },
    "P3885": {
        "label": "History of Modern Biomedicine ID",
        "description": "identifier of a person or topic in the History of Modern Biomedicine database",
        "data_type": "ExternalId"
    },
    "P3827": {
        "label": "JSTOR topic ID",
        "description": "identifier for a topic at JSTOR",
        "data_type": "ExternalId"
    },
    "P3998": {
        "label": "Censo-Guía archive ID",
        "description": "identifier of an archive in Censo-Guía de Archivos de España e Iberoamérica",
        "data_type": "ExternalId"
    },
    "P3986": {
        "label": "Sequence Ontology ID",
        "description": "identifier of a term in the sequence ontology",
        "data_type": "ExternalId"
    },
    "P4389": {
        "label": "Science Museum people ID",
        "description": "identifier of a person or organisation on the website of the Science Museum Group",
        "data_type": "ExternalId"
    },
    "P3222": {
        "label": "NE.se ID",
        "description": "ID of article on the Swedish Nationalencyklopedin (NE.se) site",
        "data_type": "ExternalId"
    },
    "P3347": {
        "label": "PermID",
        "description": "identifier for an entity or concept, in Thomson Reuters Open PermID",
        "data_type": "ExternalId"
    },
    "P3241": {
        "label": "Catholic Encyclopedia ID",
        "description": "page of an article on newadvent.org",
        "data_type": "ExternalId"
    },
    "P2106": {
        "label": "RXNO Ontology",
        "description": "RSC ontology for organic reactions",
        "data_type": "ExternalId"
    },
    "P2620": {
        "label": "ISO 15924 numeric code",
        "description": "numeric code for a writing system in ISO 15924",
        "data_type": "ExternalId"
    },
    "P4051": {
        "label": "Ukrainian regulations base ID",
        "description": "identifier in the Verkhovna Rada database of laws and regulations of Ukraine",
        "data_type": "ExternalId"
    },
    "P2158": {
        "label": "Cell Line Ontology ID",
        "description": "identifier for the Cell Line Ontology which describes anatomic origin and nature of eukaryotic cell lines",
        "data_type": "ExternalId"
    },
    "P3917": {
        "label": "IPA number order",
        "description": "the IPA order number of an IPA phoneme",
        "data_type": "Quantity"
    },
    "P4395": {
        "label": "BrainInfo ID (hierarchical)",
        "description": "numeric identifier of a brain structure in the BrainInfo (NeuroNames) database in hierarchical mode",
        "data_type": "ExternalId"
    },
    "P4229": {
        "label": "ICD-10-CM",
        "description": "identifier in the ICD-10-CM (International Classification of Diseases, 10th Revision, Clinical Modification)",
        "data_type": "ExternalId"
    },
    "P3545": {
        "label": "Theoi Project ID",
        "description": "identifier for an entity in Greek mythology, at the Theoi Project",
        "data_type": "ExternalId"
    },
    "P2479": {
        "label": "SPDX license ID",
        "description": "SPDX license identifier",
        "data_type": "ExternalId"
    },
    "P1760": {
        "label": "Aviation Safety Network Wikibase Occurrence",
        "description": "identifier for an item in the Aviation Safety Network's database",
        "data_type": "ExternalId"
    },
    "P3074": {
        "label": "Grace's Guide ID",
        "description": "identifier of a person or oganisation, on http://www.gracesguide.co.uk/",
        "data_type": "ExternalId"
    },
    "P3941": {
        "label": "Israel Antiquities Authority ID",
        "description": "ID in the Israeli Antiquities Authority archive",
        "data_type": "ExternalId"
    },
    "P1769": {
        "label": "denkXweb identifier",
        "description": "identifier in the online database for cultural heritage monuments in the German state of Hesse",
        "data_type": "ExternalId"
    },
    "P3390": {
        "label": "Consolidated code of the electronic catalog of libraries of Belarus",
        "description": "Belarus library catalog code",
        "data_type": "ExternalId"
    },
    "P3400": {
        "label": "CORDIS Project ID",
        "description": "unique identifier for research projects funded by the European Commission, in the CORDIS database.",
        "data_type": "ExternalId"
    },
    "P1645": {
        "label": "NIST/CODATA ID",
        "description": "identifier for a physical constant",
        "data_type": "ExternalId"
    },
    "P3962": {
        "label": "Global Trade Item Number",
        "description": "GTIN (or EAN, UCC) is used to identify products via their barcodes",
        "data_type": "ExternalId"
    },
    "P3285": {
        "label": "Mathematics Subject Classification ID",
        "description": "identifier of the same topic in the Mathematics Subject Classification (2010)",
        "data_type": "ExternalId"
    },
    "P4460": {
        "label": "Kaitai Struct format gallery ID",
        "description": "identifier in the global Kaitai Struct formats repository",
        "data_type": "ExternalId"
    },
    "P3088": {
        "label": "Catalogue of Life in Taiwan ID",
        "description": "identifier (name code) for a taxon in the Catalogue of Life in Taiwan",
        "data_type": "ExternalId"
    },
    "P1888": {
        "label": "Dictionary of Medieval Names from European Sources entry",
        "description": "entry in the Dictionary of Medieval Names from European Sources",
        "data_type": "ExternalId"
    },
    "P2742": {
        "label": "Australian Geological Provinces ID",
        "description": "identifier of a geologic province in the Australian Geological Provinces Database published by Geoscience Australia",
        "data_type": "ExternalId"
    },
    "P3949": {
        "label": "Juwra.com ID",
        "description": "identifier for a race, a team or an athlete on the Juwra.com website",
        "data_type": "ExternalId"
    },
    "P2748": {
        "label": "PRONOM file format identifier",
        "description": "identifier (PUID) for a file format in the technical registry PRONOM",
        "data_type": "ExternalId"
    },
    "P1402": {
        "label": "Foundational Model of Anatomy ID",
        "description": "identifier for human anatomical terminology",
        "data_type": "String"
    },
    "P1928": {
        "label": "Vaccine Ontology ID",
        "description": "identifier in the Vaccine Ontology database",
        "data_type": "ExternalId"
    },
    "P4549": {
        "label": "ARLIMA ID",
        "description": "identifier for medieval literature",
        "data_type": "ExternalId"
    },
    "P2179": {
        "label": "ACM Classification Code (2012)",
        "description": "ACM Computing Classification Code of 2012 (8 digits)",
        "data_type": "String"
    },
    "P4589": {
        "label": "Dreadnought Project page",
        "description": "page on the Dreadnought Project website",
        "data_type": "ExternalId"
    },
    "P2751": {
        "label": "Roller Coaster Database ID",
        "description": "identifier in Roller Coaster Database",
        "data_type": "ExternalId"
    },
    "P3201": {
        "label": "Medical Dictionary for Regulatory Activities ID",
        "description": "identifier in the Medical Dictionary for Regulatory Activities",
        "data_type": "ExternalId"
    },
    "P1294": {
        "label": "WWF ecoregion code",
        "description": "identifier for ecoregions",
        "data_type": "ExternalId"
    },
    "P3552": {
        "label": "Australian Registered Scheme Number",
        "description": "unique identifier for a managed investment scheme registered in Australia",
        "data_type": "ExternalId"
    },
    "P3982": {
        "label": "TA98 Latin term",
        "description": "Latin name for anatomical subject as described in Terminologia Anatomica 98",
        "data_type": "ExternalId"
    },
    "P4073": {
        "label": "Fandom wiki ID",
        "description": "identifier of a topic's wiki, in Fandom.com",
        "data_type": "ExternalId"
    },
    "P1554": {
        "label": "UBERON ID",
        "description": "identifier from UBERON ontology (without prefix)",
        "data_type": "ExternalId"
    },
    "P2372": {
        "label": "ODIS ID",
        "description": "identifier in the Belgian ODIS database",
        "data_type": "ExternalId"
    },
    "P1693": {
        "label": "Terminologia Embryologica",
        "description": "standardized list of words used in the description of human embryologic and fetal structures",
        "data_type": "ExternalId"
    },
    "P4297": {
        "label": "MIL-STD-2525 Symbol Identification Code",
        "description": "20 or 30 digit code which denotes a military symbol as specified in MIL-STD-2525",
        "data_type": "ExternalId"
    },
    "P1694": {
        "label": "Terminologia Histologica",
        "description": "controlled vocabulary for use in cytology and histology",
        "data_type": "ExternalId"
    },
    "P4553": {
        "label": "RA Collections ID",
        "description": "identifier for an entry in the collections database of the Royal Academy of Arts, London",
        "data_type": "ExternalId"
    },
    "P3246": {
        "label": "OKVED 2 code of the economic activity",
        "description": "economic activity code according to Russian classification ОК 029-2014",
        "data_type": "ExternalId"
    },
    "P1461": {
        "label": "Patientplus ID",
        "description": "identifier of disease at Patient UK",
        "data_type": "String"
    },
    "P2874": {
        "label": "PubChem BioAssay ID (AID)",
        "description": "central identifier in PubChem, used as accession number for biological and biochemical assays",
        "data_type": "ExternalId"
    },
    "P1471": {
        "label": "reporting mark",
        "description": "reporting mark for railroads in North America",
        "data_type": "String"
    },
    "P4272": {
        "label": "DPLA subject ID",
        "description": "identifier for a subject area in the Digital Public Library of America",
        "data_type": "ExternalId"
    },
    "P3338": {
        "label": "Encyclopedia of Surfing ID",
        "description": "ID for a person, a surfing spot, a subject in the Encyclopedia of Surfing",
        "data_type": "ExternalId"
    },
    "P1755": {
        "label": "Aviation Safety Network accident ID",
        "description": "identifier for an accident description per the Aviation Safety Network's database",
        "data_type": "ExternalId"
    },
    "P1394": {
        "label": "Glottolog code",
        "description": "identifier for a languoid in the Glottolog database",
        "data_type": "ExternalId"
    },
    "P3240": {
        "label": "NBN System Key",
        "description": "identifier of a taxon in the National Biodiversity Network (UK)",
        "data_type": "ExternalId"
    },
    "P3243": {
        "label": "OKVED 1.1 code of the economic activity",
        "description": "economic activity code according to Russian classification ОК 029-2007",
        "data_type": "ExternalId"
    },
    "P4333": {
        "label": "GenBank assembly accession",
        "description": "unique identifier of a genome assembly",
        "data_type": "ExternalId"
    },
    "P4355": {
        "label": "PROSITE documentation ID",
        "description": "identifier for a protein family, domain or functional site, in the PROSITE database",
        "data_type": "ExternalId"
    },
    "P1836": {
        "label": "draft pick number",
        "description": "overall pick number with which a player is selected in a sports draft",
        "data_type": "String"
    },
    "P1778": {
        "label": "forgery after",
        "description": "for an artwork trying to appear to be the work of the named author",
        "data_type": "WikibaseItem"
    },
    "P1777": {
        "label": "manner of",
        "description": "for unknown artists who worked in a similar style as the named author, but not necessary from the same period",
        "data_type": "WikibaseItem"
    },
    "P1779": {
        "label": "possible creator",
        "description": "for a creative work with considerable uncertainty about the author",
        "data_type": "WikibaseItem"
    },
    "P1774": {
        "label": "workshop of",
        "description": "artist whose work group are the likely creator of an artwork",
        "data_type": "WikibaseItem"
    },
    "P1326": {
        "label": "latest date",
        "description": "latest possible time that something could have occurred. Use as qualifier for other date properties",
        "data_type": "Time"
    },
    "P1319": {
        "label": "earliest date",
        "description": "earliest date at which an event could have happened. Use as qualifier for other date properties",
        "data_type": "Time"
    },
    "P2313": {
        "label": "minimum value",
        "description": "qualifier to define a property constraint in combination with P2302",
        "data_type": "Quantity"
    },
    "P1129": {
        "label": "national team caps",
        "description": "total number of games officially played by a sportsman for national team",
        "data_type": "Quantity"
    },
    "P817": {
        "label": "decay mode",
        "description": "type of decay that a radioactive isotope undergoes (should be used as a qualifier for \"decays to\")",
        "data_type": "WikibaseItem"
    },
    "P1355": {
        "label": "number of wins",
        "description": "number of sporting matches, games or events won",
        "data_type": "Quantity"
    },
    "P1065": {
        "label": "archive URL",
        "description": "URL to the archived web page specified with URL property",
        "data_type": "Url"
    },
    "P1721": {
        "label": "pinyin transliteration",
        "description": "hanyu pinyin transliteration of a Mandarin Chinese text (usually to be used as a qualifier)",
        "data_type": "String"
    },
    "P1016": {
        "label": "asteroid taxonomy",
        "description": "type of asteroid classification",
        "data_type": "WikibaseItem"
    },
    "P2311": {
        "label": "maximum date (property constraint)",
        "description": "qualifier to define a property constraint in combination with P2302. Use \"unknown value\" for current date.",
        "data_type": "Time"
    },
    "P697": {
        "label": "ex taxon author",
        "description": "person(s) whom the author(s) of a name credited for the idea, followed in an author citation by \"ex\" (via P405)",
        "data_type": "WikibaseItem"
    },
    "P1135": {
        "label": "nomenclatural status",
        "description": "the status of a scientific name, according to the relevant rules of nomenclature (taxonomy)",
        "data_type": "WikibaseItem"
    },
    "P1353": {
        "label": "original spelling",
        "description": "original spelling of a scientific name",
        "data_type": "String"
    },
    "P2307": {
        "label": "namespace",
        "description": "qualifier to define a property constraint in combination with P2302 (Wikidata property constraint)",
        "data_type": "String"
    },
    "P2312": {
        "label": "maximum value",
        "description": "qualifier to define a property constraint in combination with P2302. \"no value\" can be used to specify no upper bound",
        "data_type": "Quantity"
    },
    "P2077": {
        "label": "pressure",
        "description": "qualifier to indicate at what pressure something took place",
        "data_type": "Quantity"
    },
    "P768": {
        "label": "electoral district",
        "description": "electoral district this person is representing, or of the office that is being contested. Use as qualifier for \"position held\" (P39) or \"office contested\" (P541) or \"candidacy in election\" (P3602)",
        "data_type": "WikibaseItem"
    },
    "P560": {
        "label": "direction",
        "description": "qualifier to indicate the direction of the parent statement's value relative to the subject item",
        "data_type": "WikibaseItem"
    },
    "P2096": {
        "label": "media legend",
        "description": "qualifier to describe the media of the statement. This avoids legend storage in the local infobox when the media is defined at Wikidata. For the date, use P585 instead.",
        "data_type": "Monolingualtext"
    },
    "P81": {
        "label": "connecting line",
        "description": "railway line(s) subject is directly connected to",
        "data_type": "WikibaseItem"
    },
    "P1354": {
        "label": "shown with features",
        "description": "secondary features depicted in a work. Use as qualifier for \"depicts\" (P180)",
        "data_type": "WikibaseItem"
    },
    "P1775": {
        "label": "follower of",
        "description": "for unknown artists who work in the manner of the named artist",
        "data_type": "WikibaseItem"
    },
    "P805": {
        "label": "statement is subject of",
        "description": "(qualifying) item that describes the relation identified in this statement",
        "data_type": "WikibaseItem"
    },
    "P531": {
        "label": "diplomatic mission sent",
        "description": "location of diplomatic mission, i.e. consulate of A in the capital city of B",
        "data_type": "WikibaseItem"
    },
    "P2553": {
        "label": "in work",
        "description": "qualifier of award received and nominated for (P1411) to specify in which creative work the awarded or nominated creative work appeared causing the nomination/award win",
        "data_type": "WikibaseItem"
    },
    "P2308": {
        "label": "class",
        "description": "qualifier to define a property constraint in combination with \"property constraint\" (P2302)",
        "data_type": "WikibaseItem"
    },
    "P574": {
        "label": "year of taxon publication",
        "description": "year when this taxon was formally described (for animals); year when this taxon name was formally established (for plants)",
        "data_type": "Time"
    },
    "P2316": {
        "label": "constraint status",
        "description": "qualifier to define a property constraint in combination with P2302. Use values \"mandatory constraint\" or \"suggestion constraint\"",
        "data_type": "WikibaseItem"
    },
    "P2453": {
        "label": "nominee",
        "description": "qualifier used with «nominated for» to specify which person or organization was nominated",
        "data_type": "WikibaseItem"
    },
    "P642": {
        "label": "of",
        "description": "qualifier stating that a statement applies within the scope of a particular item",
        "data_type": "WikibaseItem"
    },
    "P459": {
        "label": "determination method",
        "description": "how a value is determined, or the standard by which it is declared",
        "data_type": "WikibaseItem"
    },
    "P1643": {
        "label": "departure transaction",
        "description": "how the player was released or transferred from the roster; qualifier for P54 (member of sports team)",
        "data_type": "WikibaseItem"
    },
    "P790": {
        "label": "approved by",
        "description": "item is approved by other item(s) [qualifier: statement is approved by other item(s)]",
        "data_type": "WikibaseItem"
    },
    "P670": {
        "label": "street number",
        "description": "number in the street address. To be used as a qualifier of Property:P669 \"located on street\"",
        "data_type": "String"
    },
    "P2304": {
        "label": "group by",
        "description": "qualifier to define a property constraint in combination with P2302 (Wikidata property constraint) that is used to group constraint violations in constraint violation reports",
        "data_type": "WikibaseProperty"
    },
    "P1264": {
        "label": "valid in period",
        "description": "time period when a statement is valid",
        "data_type": "WikibaseItem"
    },
    "P2126": {
        "label": "Georgian national system of romanization",
        "description": "transliteration of text from Georgian script  to Latin script according to the Georgian national system of romanization",
        "data_type": "String"
    },
    "P2303": {
        "label": "exception to constraint",
        "description": "item that is an exception to the constraint, qualifier to define a property constraint in combination with P2302",
        "data_type": "WikibaseItem"
    },
    "P2076": {
        "label": "temperature",
        "description": "qualifier to indicate at what temperature something took place",
        "data_type": "Quantity"
    },
    "P2241": {
        "label": "reason for deprecation",
        "description": "qualifier to allow the reason to be indicated why a particular statement should be considered deprecated",
        "data_type": "WikibaseItem"
    },
    "P1358": {
        "label": "points for",
        "description": "number of points in a league table or decathlon. (Use P1351 for goals/points in a match)",
        "data_type": "Quantity"
    },
    "P1111": {
        "label": "votes received",
        "description": "qualifier stating the number of votes for a candidate in an election",
        "data_type": "Quantity"
    },
    "P2647": {
        "label": "source of material",
        "description": "place the material used was mined, quarried, found, or produced",
        "data_type": "WikibaseItem"
    },
    "P972": {
        "label": "catalog",
        "description": "catalog for the item, or, as a qualifier of P528 – catalog for which the 'catalog code' is valid",
        "data_type": "WikibaseItem"
    },
    "P2183": {
        "label": "ISO 9:1995",
        "description": "the Latin transliteration of Cyrillic text (used as qualifier)",
        "data_type": "String"
    },
    "P2352": {
        "label": "applies to taxon",
        "description": "qualifier for toxicological concentrations and doses, indicates species in which the drug was tested",
        "data_type": "WikibaseItem"
    },
    "P1780": {
        "label": "school of",
        "description": "for a creative work with author with a style influenced by the known author or circle, active in the same period, but a student or follower",
        "data_type": "WikibaseItem"
    },
    "P1365": {
        "label": "replaces",
        "description": "person, state or item replaced. Use \"structure replaces\" (P1398) for structures. Use \"follows\" (P155) if the previous item was not replaced or predecessor and successor are identical",
        "data_type": "WikibaseItem"
    },
    "P2309": {
        "label": "relation",
        "description": "qualifier to define a property constraint in combination with P2302. Possibly values are: \"instance of\", \"subclass of\" or \"instance or subclass of\". The qualifier to use with the property \"relative\" is \"type of kinship\" (P1039), not this.",
        "data_type": "WikibaseItem"
    },
    "P1026": {
        "label": "academic thesis",
        "description": "thesis or dissertation written for a degree",
        "data_type": "WikibaseItem"
    },
    "P1674": {
        "label": "number confirmed",
        "description": "qualifier; number of instances or occurrences classified as confirmed for reporting purposes",
        "data_type": "Quantity"
    },
    "P656": {
        "label": "RefSeq",
        "description": "qualifier to specify Reference Sequence identifier for the Protein and RNA (either p637 or p639)",
        "data_type": "String"
    },
    "P554": {
        "label": "website username",
        "description": "username on a website that the person or movement has an account on (only use as qualifier of P553)",
        "data_type": "String"
    },
    "P1942": {
        "label": "McCune-Reischauer romanization",
        "description": "romanization of Korean with the McCune-Reischauer system",
        "data_type": "String"
    },
    "P748": {
        "label": "appointed by",
        "description": "who appointed the person to the office, can be used as a qualifier",
        "data_type": "WikibaseItem"
    },
    "P2571": {
        "label": "uncertainty corresponds to",
        "description": "number of standard deviations (sigma) expressing the confidence level of a value",
        "data_type": "WikibaseItem"
    },
    "P654": {
        "label": "direction relative to location",
        "description": "qualifier for geographical locations to express relative direction",
        "data_type": "WikibaseItem"
    },
    "P1642": {
        "label": "acquisition transaction",
        "description": "how the player was acquired; qualifier for P54 (member of sports team)",
        "data_type": "WikibaseItem"
    },
    "P447": {
        "label": "review score by",
        "description": "issuer of a review score",
        "data_type": "WikibaseItem"
    },
    "P1001": {
        "label": "applies to jurisdiction",
        "description": "the item (an institution, law, public office ...) or statement belongs to or has power over or applies to the value (a territorial jurisdiction: a country, state, municipality, ...)",
        "data_type": "WikibaseItem"
    },
    "P1810": {
        "label": "named as",
        "description": "name by which a subject is recorded in a database or mentioned as a contributor of a work",
        "data_type": "String"
    },
    "P453": {
        "label": "character role",
        "description": "specific role played or filled by subject -- use only as qualifier of \"cast member\" (P161), \"voice actor\" (P725)",
        "data_type": "WikibaseItem"
    },
    "P2178": {
        "label": "solvent",
        "description": "qualifier to show which substance is the solvent in a solution-process (use with P2177)",
        "data_type": "WikibaseItem"
    },
    "P2436": {
        "label": "voltage",
        "description": "difference in electric potential between two points (indicated in volt)",
        "data_type": "Quantity"
    },
    "P1545": {
        "label": "series ordinal",
        "description": "position of an item in its parent series (most frequently a 1-based index), generally to be used as a qualifier (different from \"rank\" defined as a class, and from \"ranking\" defined as a property for evaluating a quality).",
        "data_type": "String"
    },
    "P1366": {
        "label": "replaced by",
        "description": "other person or item which continues the item by replacing it in its role. Use P156 (followed by) if the item is not replaced (e.g. books in a series), nor identical, but adds to the series without dropping the role of this item in that series",
        "data_type": "WikibaseItem"
    },
    "P659": {
        "label": "genomic assembly",
        "description": "specify the genome assembly on which the feature is placed",
        "data_type": "WikibaseItem"
    },
    "P1357": {
        "label": "number of draws/ties",
        "description": "number of matches or games drawn or tied in a league or an event",
        "data_type": "Quantity"
    },
    "P1534": {
        "label": "end cause",
        "description": "qualifier to use together with the end date qualifier (P582) to specify the cause",
        "data_type": "WikibaseItem"
    },
    "P2125": {
        "label": "Revised Hepburn romanization",
        "description": "romanized Japanese following the Revised Hepburn romanization system",
        "data_type": "String"
    },
    "P2210": {
        "label": "relative to",
        "description": "qualifier: what a statement value is relative to",
        "data_type": "WikibaseItem"
    },
    "P1033": {
        "label": "GHS signal word",
        "description": "please use with items \"warning\" (Q15350847) and \"danger\" (Q15221217)",
        "data_type": "WikibaseItem"
    },
    "P1686": {
        "label": "for work",
        "description": "qualifier of award received (P166) to specify the work that an award was given to the creator for",
        "data_type": "WikibaseItem"
    },
    "P2001": {
        "label": "Revised Romanization",
        "description": "romanisation following the Revised Romanisation of the Korean language",
        "data_type": "String"
    },
    "P548": {
        "label": "version type",
        "description": "type of version (qualifier for P348, software version), e.g. alpha, beta, stable",
        "data_type": "WikibaseItem"
    },
    "P678": {
        "label": "incertae sedis",
        "description": "qualifier for P171 (parent taxon) to mark which intermediate ranks are incertae sedis",
        "data_type": "WikibaseItem"
    },
    "P1107": {
        "label": "proportion",
        "description": "to be used as a qualifier, value must be between 0 and 1",
        "data_type": "Quantity"
    },
    "P1443": {
        "label": "score method",
        "description": "qualifier of \"points/goal scored by (P1363)\"",
        "data_type": "WikibaseItem"
    },
    "P2148": {
        "label": "distance from river mouth",
        "description": "qualifier for P974 statements on streams",
        "data_type": "Quantity"
    },
    "P1675": {
        "label": "number probable",
        "description": "number of instances or occurrences classified as probable (vs. confirmed) for reporting purposes",
        "data_type": "Quantity"
    },
    "P1676": {
        "label": "number suspected",
        "description": "number of instances or occurrences classified as suspected for reporting purposes",
        "data_type": "Quantity"
    },
    "P2310": {
        "label": "minimum date (property constraint)",
        "description": "qualifier to define a property constraint in combination with P2302",
        "data_type": "Time"
    },
    "P1013": {
        "label": "criterion used",
        "description": "property by which a distinction or classification is made",
        "data_type": "WikibaseItem"
    },
    "P2441": {
        "label": "literal translation",
        "description": "direct or word-for-word translation of a name or phrase (qualifier for name, title, inscription, and quotation properties)",
        "data_type": "Monolingualtext"
    },
    "P2305": {
        "label": "item of property constraint",
        "description": "qualifier to define a property constraint in combination with \"property constraint\" (P2302)",
        "data_type": "WikibaseItem"
    },
    "P1706": {
        "label": "together with",
        "description": "qualifier to specify the item that this property is shared with",
        "data_type": "WikibaseItem"
    },
    "P1310": {
        "label": "statement disputed by",
        "description": "entity that disputes a given statement",
        "data_type": "WikibaseItem"
    },
    "P2443": {
        "label": "stage reached",
        "description": "ultimate point in an event or competition reached by a participant",
        "data_type": "WikibaseItem"
    },
    "P1012": {
        "label": "including",
        "description": "usually used as a qualifier",
        "data_type": "WikibaseItem"
    },
    "P1351": {
        "label": "number of points/goals/set scored",
        "description": "goals / points scored in a match or an event used as qualifier to the participant. Use P1358 for league points.",
        "data_type": "Quantity"
    },
    "P1776": {
        "label": "circle of",
        "description": "for unknown artists who lived in the same time as the named author in a similar style, possibly a follower or someone who had contact with the named artist",
        "data_type": "WikibaseItem"
    },
    "P1262": {
        "label": "RAÄ number",
        "description": "Identificator  for an element in the Database of the cultural heritage in Sweden",
        "data_type": "ExternalId"
    },
    "P405": {
        "label": "taxon author",
        "description": "the author(s) that (optionally) may be cited with the scientific name",
        "data_type": "WikibaseItem"
    },
    "P1039": {
        "label": "kinship to subject",
        "description": "qualifier of \"relative\" (P1038) to indicate less usual family relationships (ancestor, son-in-law, adoptions, etc). Indicate how the qualificator item is related to the main item.",
        "data_type": "WikibaseItem"
    },
    "P1480": {
        "label": "sourcing circumstances",
        "description": "qualification of the truth or accuracy of a source: circa (Q5727902), near (Q21818619), presumably (Q18122778), etc.",
        "data_type": "WikibaseItem"
    },
    "P8330": {
        "label": "file page offset",
        "description": "offset between page numbering of the file (pdf/djvu) and the main numbering used in the reproduced work. Sample: if the  book has page number 2 and that is on page 30 of the pdf: offset is 28 (30-2)",
        "data_type": "Quantity"
    },
    "P484": {
        "label": "IMA Number, broad sense",
        "description": "identifier for a mineral per the International Mineralogical Association - Commission on New Minerals, Nomenclature and Classification (IMA-CNMNC)",
        "data_type": "ExternalId"
    },
    "P8395": {
        "label": "use with property value",
        "description": "qualifier for \"properties for this type\" (P8397): value to be used with property in a statement. Sample: instances of bell tower have \"parts of the class\" statements with the value \"bell\".",
        "data_type": "WikibaseItem"
    },
    "P8397": {
        "label": "prompt when missing",
        "description": "qualifier for \"properties for this type\" (P1963): item with text to display when statement is missing",
        "data_type": "WikibaseItem"
    },
    "P268": {
        "label": "Bibliothèque nationale de France ID",
        "description": "identifier for the subject issued by BNF (Bibliothèque nationale de France). Format: 8 digits followed by a check-digit or letter, do not include the initial 'cb'.",
        "data_type": "ExternalId"
    },
    "P8398": {
        "label": "offset from vertical",
        "description": "distance from a vertical line at a given height, e.g. overhang in metres of a leaning tower at its top",
        "data_type": "Quantity"
    },
    "P8770": {
        "label": "format as language specific regular expression",
        "description": "regex describing a string in a given language; use the language code to define which language it applies to",
        "data_type": "Monolingualtext"
    },
    "P123": {
        "label": "publisher",
        "description": "organization or person responsible for publishing books, periodicals, printed music, podcasts, games or software",
        "data_type": "WikibaseItem"
    },
    "P7930": {
        "label": "Wikimedia community discussion URL",
        "description": "(usually used as reference) a community discussion (in Wikidata or other Wikimedia) about a specific statement, or (rarely) an entity in general which may affect the content of the entity",
        "data_type": "Url"
    },
    "P577": {
        "label": "publication date",
        "description": "date or point in time when a work was first published or released",
        "data_type": "Time"
    },
    "P7668": {
        "label": "file page",
        "description": "property to indicate a page number of a pdf file or similar. The page number is sometimes different from the numbering of the text reproduced in the document. Should allow to link the page when combined with the file URL",
        "data_type": "Quantity"
    },
    "P377": {
        "label": "SCN",
        "description": "Satellite Catalog Number, 5-digit-number including leading zeros (e.g. '00266')",
        "data_type": "ExternalId"
    },
    "P8396": {
        "label": "use with qualifier",
        "description": "qualifier for \"properties for this type\" (P8397): indicate the qualifier to be used with the property in a statement. Sample: instances of bell tower have \"parts of the class\" statements with value \"bell\" and these are to be qualified with \"quantity\"",
        "data_type": "WikibaseProperty"
    },
    "P5017": {
        "label": "last update",
        "description": "date a reference was modified, revised, or updated",
        "data_type": "Time"
    },
    "P557": {
        "label": "DiseasesDB",
        "description": "identifier sourced on the Diseases Database",
        "data_type": "ExternalId"
    },
    "P6678": {
        "label": "WoRMS source ID",
        "description": "identifier of a source in the WoRMS database",
        "data_type": "ExternalId"
    },
    "P4656": {
        "label": "Wikimedia import URL",
        "description": "URL of source to indicate the page or revision of an import source from another Wikimedia project (except actual references, such as Wikisource source texts). Use instead of \"reference URL\" (P854)",
        "data_type": "Url"
    },
    "P7416": {
        "label": "folio(s)",
        "description": "folio number of reference for statement",
        "data_type": "String"
    },
    "P18": {
        "label": "image",
        "description": "image of relevant illustration of the subject; if available, use more specific properties (sample: coat of arms image, locator map, flag image, signature image, logo image, collage image)",
        "data_type": "CommonsMedia"
    },
    "P563": {
        "label": "ICD-O",
        "description": "International Classification of Diseases for Oncology",
        "data_type": "ExternalId"
    },
    "P179": {
        "label": "part of the series",
        "description": "series which contains the subject",
        "data_type": "WikibaseItem"
    },
    "P494": {
        "label": "ICD-10",
        "description": "identifier in the ICD catalogue codes for diseases - Version 10",
        "data_type": "ExternalId"
    },
    "P6184": {
        "label": "reference has role",
        "description": "role, or specific nature, of the given reference",
        "data_type": "WikibaseItem"
    },
    "P506": {
        "label": "ISO 15924 alpha-4 code",
        "description": "4-letter identifier for script or writing system of a language",
        "data_type": "ExternalId"
    },
    "P7421": {
        "label": "line(s)",
        "description": "line number(s) of reference for claim",
        "data_type": "String"
    },
    "P8428": {
        "label": "interactive elements",
        "description": "qualifier on ESRB rating (P852) to describe interactive elements",
        "data_type": "WikibaseItem"
    },
    "P887": {
        "label": "based on heuristic",
        "description": "indicates that the property value is determined based on some heuristic (Q201413); to be used as source",
        "data_type": "WikibaseItem"
    },
    "P5564": {
        "label": "registration district",
        "description": "reference property indicating location for references to a civil or ecclesiastical registry",
        "data_type": "WikibaseItem"
    },
    "P8884": {
        "label": "number of points/goals attempted",
        "description": "shots or other attempts to score points in a sporting event",
        "data_type": "Quantity"
    },
    "P7776": {
        "label": "SSNE person ID",
        "description": "identifier of The Scotland, Scandinavia and Northern European Biographical Database",
        "data_type": "ExternalId"
    },
    "P1433": {
        "label": "published in",
        "description": "larger work that a given work was published in, like a book, journal or music album",
        "data_type": "WikibaseItem"
    },
    "P667": {
        "label": "ICPC 2 ID",
        "description": "classification method for primary care encounters",
        "data_type": "String"
    },
    "P663": {
        "label": "DSM-IV classification",
        "description": "identifier for a mental disorder in the 4th edition of Diagnostic and Statistical Manual of Mental Disorders",
        "data_type": "ExternalId"
    },
    "P1476": {
        "label": "title",
        "description": "published title of a work, such as a newspaper article, a literary work, a website, or a performance work",
        "data_type": "Monolingualtext"
    },
    "P304": {
        "label": "page(s)",
        "description": "page number of source referenced for statement",
        "data_type": "String"
    },
    "P8554": {
        "label": "earliest end date",
        "description": "(qualifier) earliest date on which the statement could have begun to no longer be true",
        "data_type": "Time"
    },
    "P8555": {
        "label": "latest start date",
        "description": "(qualifier) latest date on which the statement could have started to be true",
        "data_type": "Time"
    },
    "P686": {
        "label": "Gene Ontology ID",
        "description": "identifier in the Gene Ontology",
        "data_type": "ExternalId"
    },
    "P7787": {
        "label": "heading",
        "description": "direction (yaw) of the camera (or the viewer), measured as clockwise angle from the North, given in degrees, radians or other units",
        "data_type": "Quantity"
    },
    "P5683": {
        "label": "iNaturalist observation ID",
        "description": "observation identifier in iNaturalist",
        "data_type": "ExternalId"
    },
    "P407": {
        "label": "language of work or name",
        "description": "language associated with this creative work (such as books, shows, songs, or websites) or a name (for persons use \"native language\" (P103) and \"languages spoken, written or signed\" (P1412))",
        "data_type": "WikibaseItem"
    },
    "P1683": {
        "label": "quotation",
        "description": "quotation supporting the statement claim (to be used in the reference or qualifier field only, no quote marks)",
        "data_type": "Monolingualtext"
    },
    "P3452": {
        "label": "inferred from",
        "description": "statement added based on related statement found on the following item, not the entity described by the item (to be used in a reference field)",
        "data_type": "WikibaseItem"
    },
    "P291": {
        "label": "place of publication",
        "description": "geographical place of publication of the edition (use 1st edition when referring to works)",
        "data_type": "WikibaseItem"
    },
    "P7903": {
        "label": "gained territory from",
        "description": "(qualifier) territorial entity from which the subject item gained territory as a result of the event in the statement",
        "data_type": "WikibaseItem"
    },
    "P7904": {
        "label": "gave up territory to",
        "description": "(qualifier) territorial entity to which the subject item gave up some of its territory as a result of the event in the statement",
        "data_type": "WikibaseItem"
    },
    "P872": {
        "label": "printed by",
        "description": "organization or person who printed the creative work (if different from \"publisher\")",
        "data_type": "WikibaseItem"
    },
    "P918": {
        "label": "NOC Occupation Code",
        "description": "NOC/CNP Code for a given occupation in Canada and Québec",
        "data_type": "ExternalId"
    },
    "P9043": {
        "label": "Internet Game Database numeric game ID",
        "description": "numeric identifier for a game on IGDB; use as qualifier for P5794 \"Internet Game Database game ID\"",
        "data_type": "ExternalId"
    },
    "P6333": {
        "label": "title of broader work",
        "description": "title of website, journal or other work in which an article (or other publication) is published in. Use \"published in\" (P1433) instead if possible.",
        "data_type": "Monolingualtext"
    },
    "P2960": {
        "label": "archive date",
        "description": "date a document was archived",
        "data_type": "Time"
    },
    "P8338": {
        "label": "applies to name of value",
        "description": "(qualifier) label or alias (of the claim object) to which the claim applies",
        "data_type": "Monolingualtext"
    },
    "P699": {
        "label": "Disease Ontology ID",
        "description": "identifier in the Disease Ontology database",
        "data_type": "ExternalId"
    },
    "P486": {
        "label": "MeSH descriptor ID",
        "description": "identifier for Descriptor or Supplementary concept in the Medical Subject Headings controlled vocabulary",
        "data_type": "ExternalId"
    },
    "P958": {
        "label": "section, verse, paragraph, or clause",
        "description": "paragraph, or other kind of special indication to find information on a page or on a document (legal texts etc.)",
        "data_type": "String"
    },
    "P587": {
        "label": "MMSI",
        "description": "Maritime Mobile Service Identity of a ship station. Format 8 or 9 digits",
        "data_type": "ExternalId"
    },
    "P7887": {
        "label": "number of reviews/ratings",
        "description": "qualifier on review score to state the number of user reviews, user ratings or critic reviews",
        "data_type": "Quantity"
    },
    "P3865": {
        "label": "type of reference",
        "description": "used to specify the type of a reference",
        "data_type": "WikibaseItem"
    },
    "P863": {
        "label": "InPhO ID",
        "description": "identifier in the Indiana Philosophy Ontology project",
        "data_type": "ExternalId"
    },
    "P8241": {
        "label": "type of age limit",
        "description": "qualifier on  minimum age (P2899) to describe the type of age limit",
        "data_type": "WikibaseItem"
    },
    "P9269": {
        "label": "Odnoklassniki user numeric ID",
        "description": "numeric identifier for a user on Odnoklassniki; use as qualifier for P5163 \"Odnoklassniki ID\"",
        "data_type": "ExternalId"
    },
    "P493": {
        "label": "ICD-9",
        "description": "identifier in the ICD catalogue codes for diseases – Version 9",
        "data_type": "ExternalId"
    },
    "P6655": {
        "label": "stroke order",
        "description": "media file showing the order of strokes in a CJKV character",
        "data_type": "CommonsMedia"
    },
    "P1343": {
        "label": "described by source",
        "description": "work where this item is described",
        "data_type": "WikibaseItem"
    },
    "P8276": {
        "label": "region within image",
        "description": "qualifier to identify a polygon within an image that the main statement applies to",
        "data_type": "String"
    },
    "P8275": {
        "label": "position in biological sequence",
        "description": "index or position of a nucleotide in a genomic sequence, or of an amino acid in a protein's amino acid sequence; used as qualifier",
        "data_type": "Quantity"
    },
    "P503": {
        "label": "ISO standard",
        "description": "number of the ISO standard which normalizes the object",
        "data_type": "ExternalId"
    },
    "P8626": {
        "label": "opening time",
        "description": "time when an establishment or attraction opens, qualifier for P3025",
        "data_type": "WikibaseItem"
    },
    "P8627": {
        "label": "closing time",
        "description": "time when an establishment or attraction closes, qualifier for P3025",
        "data_type": "WikibaseItem"
    },
    "P502": {
        "label": "HURDAT identifier",
        "description": "identifier per HURDAT (North Atlantic hurricane database)",
        "data_type": "String"
    },
    "P248": {
        "label": "stated in",
        "description": "to be used in the references field to refer to the information document or database in which a claim is made; for qualifiers use P805",
        "data_type": "WikibaseItem"
    },
    "P731": {
        "label": "Litholex ID",
        "description": "identifier in the Lithostratigraphic database of Germany maintained by the BGR",
        "data_type": "ExternalId"
    },
    "P7569": {
        "label": "Wikimedia page-version URL",
        "description": "(qualifier) URL of revision of Wikipedia article or other page on a WMF site being used, site specific URL to a page version ID (oldid parameter). Sample: nlwiki URL starts with https://nl.wikipedia.org/w/index.php?oldid=",
        "data_type": "Url"
    },
    "P2093": {
        "label": "author name string",
        "description": "string to store unspecified author or editor name for publications; use if Wikidata item for author (P50) or editor (P98) does not exist or is not known",
        "data_type": "String"
    },
    "P8208": {
        "label": "tilt",
        "description": "direction of the camera, measured as an angle from the Nadir to the Zenit",
        "data_type": "Quantity"
    },
    "P919": {
        "label": "SOC Code (2010)",
        "description": "Standard Occupational Classification code for US jobs (2010 version)",
        "data_type": "ExternalId"
    },
    "P847": {
        "label": "United States Navy aircraft designation",
        "description": "identifier for an aircraft of the United States Navy",
        "data_type": "String"
    },
    "P854": {
        "label": "reference URL",
        "description": "should be used for Internet URLs as references. Use \"Wikimedia import URL\" (P4656) for interwiki links",
        "data_type": "Url"
    },
    "P7793": {
        "label": "filename in archive",
        "description": "qualifier and referencing property to identify the specific file within a compressed archive that is relevant to a statement",
        "data_type": "String"
    },
    "P9140": {
        "label": "time played",
        "description": "time (usually minutes) played by a participant in a sporting event",
        "data_type": "Quantity"
    },
    "P792": {
        "label": "chapter",
        "description": "title or number of the chapter where a claim is made",
        "data_type": "String"
    },
    "P646": {
        "label": "Freebase ID",
        "description": "identifier for a page in the Freebase database. Format: \"/m/0\" followed by 2 to 7 characters. For those starting with \"/g/\", use Google Knowledge Graph identifier (P2671)",
        "data_type": "ExternalId"
    },
    "P3903": {
        "label": "column",
        "description": "typographical column in the page of a document",
        "data_type": "String"
    },
    "P862": {
        "label": "Operational Requirement of the UK Air Ministry",
        "description": "specification for future military aircraft by the UK Air Ministry",
        "data_type": "ExternalId"
    },
    "P1680": {
        "label": "subtitle",
        "description": "for works, when the title is followed by a subtitle",
        "data_type": "Monolingualtext"
    },
    "P849": {
        "label": "Japanese military aircraft designation",
        "description": "identifier for an aircraft per the Japanese military",
        "data_type": "ExternalId"
    },
    "P50": {
        "label": "author",
        "description": "main creator(s) of a written work (use on works, not humans); use P2093 when Wikidata item is unknown or does not exist",
        "data_type": "WikibaseItem"
    },
    "P952": {
        "label": "ISCO-88 occupation code",
        "description": "International Standard Classification of Occupations code according to the International Labour Organization (ILO) classification structure as of 1988",
        "data_type": "String"
    },
    "P8967": {
        "label": "URL match replacement value",
        "description": "(qualifier only) optional qualifier to overwrite the default \\1",
        "data_type": "String"
    },
    "P478": {
        "label": "volume",
        "description": "volume of a book or music release in a collection/series or a published collection of journal issues in a serial publication",
        "data_type": "String"
    },
    "P143": {
        "label": "imported from Wikimedia project",
        "description": "source of this claim's value; used in references section by bots importing data from Wikimedia projects; also allowed to be used for manual imports",
        "data_type": "WikibaseItem"
    },
    "P7855": {
        "label": "attested as",
        "description": "how the word is attested in a text, if different for the current way to write it. Should be used with \"attested in\" (P5323)",
        "data_type": "String"
    },
    "P498": {
        "label": "ISO 4217 code",
        "description": "identifier for a currency per ISO 4217",
        "data_type": "ExternalId"
    },
    "P920": {
        "label": "LEM ID",
        "description": "subject headings for public libraries maintained by the Spanish Ministry of Education, Culture and Sport",
        "data_type": "ExternalId"
    },
    "P305": {
        "label": "IETF language tag",
        "description": "identifier for language or languoid per the Internet Engineering Task Force; can include a primary language subtag,  subtags for script, region, variant, extension, or private-use. Format: 2 or 3 letters, followed by \"-\" if subtags present",
        "data_type": "ExternalId"
    },
    "P433": {
        "label": "issue",
        "description": "issue of a newspaper, a scientific journal or magazine for reference purpose",
        "data_type": "String"
    },
    "P5565": {
        "label": "CRAN project",
        "description": "name of a project in CRAN",
        "data_type": "ExternalId"
    },
    "P8734": {
        "label": "O*NET OnLine code",
        "description": "code for an occupation in O*NET OnLine",
        "data_type": "ExternalId"
    },
    "P155": {
        "label": "follows",
        "description": "immediately prior item in a series of which the subject is a part [if the subject has replaced the preceding item, e.g. political offices, use \"replaces\" (P1365)]",
        "data_type": "WikibaseItem"
    },
    "P9278": {
        "label": "Pandektis ID",
        "description": "identifier for an entity in the collections of the Greek website Pandektis",
        "data_type": "ExternalId"
    },
    "P6783": {
        "label": "speedrun.com game ID",
        "description": "identifier for games on speedrun.com",
        "data_type": "ExternalId"
    },
    "P6823": {
        "label": "GNU ELPA package ID",
        "description": "identifier for a software package in the GNU ELPA archive",
        "data_type": "ExternalId"
    },
    "P6817": {
        "label": "SVT Open archive",
        "description": "identifier for subjects in the archives of Svt, the Swedish national public television broadcaster",
        "data_type": "ExternalId"
    },
    "P8087": {
        "label": "GameTDB game ID",
        "description": "identifier of a video game in the GameTDB database",
        "data_type": "ExternalId"
    },
    "P6771": {
        "label": "Arabic Ontology ID",
        "description": "identifier for external link ontology.birzeit.edu, item equivalent on Q63107058 in English and/or Arabic",
        "data_type": "ExternalId"
    },
    "P4618": {
        "label": "AUSNUT 2011–13 Food Group ID",
        "description": "identifier of a food group in AUSNUT 2011–13 (AUStralian Food and NUTrient Database)",
        "data_type": "ExternalId"
    },
    "P7963": {
        "label": "Cell Ontology ID",
        "description": "ID in the Cell Ontology",
        "data_type": "ExternalId"
    },
    "P8809": {
        "label": "AIWARDS ID",
        "description": "identifier for a pornographic actor, award, film or film studio in the Adult Industry Awards Database",
        "data_type": "ExternalId"
    },
    "P156": {
        "label": "followed by",
        "description": "immediately following item in a series of which the subject is a part [if the subject has been replaced, e.g. political offices, use \"replaced by\" (P1366)]",
        "data_type": "WikibaseItem"
    },
    "P1114": {
        "label": "quantity",
        "description": "number of instances of this subject",
        "data_type": "Quantity"
    },
    "P6672": {
        "label": "Placar UOL Eleições ID",
        "description": "identifier to results of Brazilian municipal and state elections in the Placar UOL Eleições database",
        "data_type": "ExternalId"
    },
    "P5638": {
        "label": "IEEE standard",
        "description": "identifier of an IEEE standard",
        "data_type": "ExternalId"
    },
    "P9037": {
        "label": "BHCL UUID",
        "description": "UUID of authorities used in the Bibliography of the History of the Czech Lands (BHCL)",
        "data_type": "ExternalId"
    },
    "P6867": {
        "label": "Gamepedia wiki ID",
        "description": "wiki name for this topic on Gamepedia",
        "data_type": "ExternalId"
    },
    "P4662": {
        "label": "SAN archive conservator ID",
        "description": "identifier for archive conservators in the Italian national archive system (SAN - Sistema archivistico nazionale)",
        "data_type": "ExternalId"
    },
    "P8679": {
        "label": "Dictionary of Occupational Titles Code (fourth edition, revised)",
        "description": "occupation code from the Dictionary of Occupational Titles, fourth edition, revised",
        "data_type": "ExternalId"
    },
    "P6898": {
        "label": "NeoGeoSoft ID",
        "description": "identifier in the NeoGeoSoft database of Neo Geo video games",
        "data_type": "ExternalId"
    },
    "P9395": {
        "label": "Verified Handles VHID",
        "description": "identifier for an entry in the Verified Handles database",
        "data_type": "ExternalId"
    },
    "P5568": {
        "label": "Python Package Index project",
        "description": "name of a project in Python Package Index",
        "data_type": "ExternalId"
    },
    "P5566": {
        "label": "RubyGems gem",
        "description": "name of a project in RubyGems",
        "data_type": "ExternalId"
    },
    "P31": {
        "label": "instance of",
        "description": "that class of which this subject is a particular example and member",
        "data_type": "WikibaseItem"
    },
    "P9356": {
        "label": "Hymenoptera Anatomy Ontology ID",
        "description": "identifier for an anatomical entity in Hymenoptera Anatomy Ontology, a structured controlled vocabulary of the anatomy of the Hymenoptera (bees, wasps, sawflies and ants)",
        "data_type": "ExternalId"
    },
    "P4726": {
        "label": "Sign@l journal ID",
        "description": "identifier for a journal in the Sign@l database",
        "data_type": "ExternalId"
    },
    "P4636": {
        "label": "Sherpa Juliet ID",
        "description": "identifier on the Juliet directory of research funding policies",
        "data_type": "ExternalId"
    },
    "P968": {
        "label": "e-mail address",
        "description": "e-mail address of the organisation. Format: prefixed with mailto:",
        "data_type": "Url"
    },
    "P5910": {
        "label": "ENI number",
        "description": "European Vessel Identification Number for ships capable of navigating on inland European waters",
        "data_type": "String"
    },
    "P5620": {
        "label": "FFF character ID",
        "description": "identifier for a character on the Franco Fossati Foundation website",
        "data_type": "ExternalId"
    },
    "P6816": {
        "label": "The Dragon Archive ID",
        "description": "ID for the videogames database \"The Dragon Archive\", for Dragon computers",
        "data_type": "ExternalId"
    },
    "P8709": {
        "label": "CTAN package ID",
        "description": "identifier for a package on the Comprehensive TeX Archive Network",
        "data_type": "ExternalId"
    },
    "P5569": {
        "label": "Tebeosfera character ID",
        "description": "identifier for a character on the Tebeosfera website",
        "data_type": "ExternalId"
    },
    "P6002": {
        "label": "WikiArt ID",
        "description": "entry of a people or artwork at WikiArt",
        "data_type": "ExternalId"
    },
    "P6821": {
        "label": "Uppsala University Alvin ID",
        "description": "platform for digital collections and digitized cultural heritage in Uppsala about, managed by Uppsala university library",
        "data_type": "ExternalId"
    },
    "P6656": {
        "label": "BHCL ID",
        "description": "identifier of authorities used in the Bibliography of the History of the Czech Lands (BHCL)",
        "data_type": "ExternalId"
    },
    "P973": {
        "label": "described at URL",
        "description": "item is described at the following URL",
        "data_type": "Url"
    },
    "P5610": {
        "label": "OpenMath ID",
        "description": "OpenMath content dictionary symbol",
        "data_type": "ExternalId"
    },
    "P279": {
        "label": "subclass of",
        "description": "next higher class or type; all instances of these items are instances of those items; this item is a class (subset) of that item. Not to be confused with P31 (instance of)",
        "data_type": "WikibaseItem"
    },
    "P6839": {
        "label": "TV Tropes identifier",
        "description": "identifier in the TV Tropes online wiki",
        "data_type": "ExternalId"
    },
    "P4695": {
        "label": "GEMS Code",
        "description": "WHO Global Environment Monitoring System/ Food Contamination Monitoring and Assessment Programme (GEMS/Food) code for a food-type",
        "data_type": "ExternalId"
    },
    "P8008": {
        "label": "Google Scholar case ID",
        "description": "ID for legal cases cataloged by Google Scholar",
        "data_type": "ExternalId"
    },
    "P8106": {
        "label": "SOCAN work number",
        "description": "identifier for a musical work in SOCAN's repertory",
        "data_type": "ExternalId"
    },
    "P5690": {
        "label": "Saskatchewan Sports Hall of Fame ID",
        "description": "identifier for a sportssubject (athlete, sportsbuilder or a team) on the Saskatchewan Sports Hall of Fame",
        "data_type": "ExternalId"
    },
    "P4729": {
        "label": "INRAN Italian Food ID",
        "description": "identifier in the Italian national nutriment database",
        "data_type": "ExternalId"
    },
    "P6888": {
        "label": "MELPA package ID",
        "description": "identifier for a software package in the MELPA archive",
        "data_type": "ExternalId"
    },
    "P5740": {
        "label": "So Foot ID",
        "description": "identifier for a person, a club or a competition on the So Foot website",
        "data_type": "ExternalId"
    },
    "P7258": {
        "label": "Toons Mag ID",
        "description": "official identifier for a cartoonist, caricaturist, illustrator, and author in Toons Mag",
        "data_type": "ExternalId"
    },
    "P5712": {
        "label": "IMEC ID",
        "description": "identifier on the 'Institut mémoires de l'édition contemporaine's website",
        "data_type": "ExternalId"
    },
    "P5429": {
        "label": "IPTC NewsCode",
        "description": "NewsCode for an item, as defined by the International Press Telecommunications Council (IPTC)",
        "data_type": "ExternalId"
    },
    "P8028": {
        "label": "GameRevolution game ID",
        "description": "identifier of a video game in the GameRevolution database",
        "data_type": "ExternalId"
    },
    "P6683": {
        "label": "Alexander Turnbull Library ID",
        "description": "identifier for names in the Alexander Turnbull Library thesaurus module for unpublished collections",
        "data_type": "ExternalId"
    },
    "P8230": {
        "label": "Bookogs credit ID",
        "description": "a page in Bookogs database which connects to books and magazines authored by a person, or describing a person or thing or idea",
        "data_type": "ExternalId"
    },
    "P4696": {
        "label": "CIQUAL2017 ID",
        "description": "identifier for a food category in the French CIQUAL 2017 nutritional database",
        "data_type": "ExternalId"
    },
    "P8400": {
        "label": "House Divided ID",
        "description": "identifier of a topic in House Divided: The Civil War Research Engine at Dickinson College",
        "data_type": "ExternalId"
    },
    "P6681": {
        "label": "EUNIS ID for habitats",
        "description": "identifier for a habitats on the European Nature Information System website (EUNIS)",
        "data_type": "ExternalId"
    },
    "P6680": {
        "label": "MeSH term ID",
        "description": "identifier of a \"MeSH term\" (Medical Subject Headings) of National Library of Medicine (National Institutes of Health - U.S. Department of Health)",
        "data_type": "ExternalId"
    },
    "P1136": {
        "label": "solved by",
        "description": "person that solved a scientific question",
        "data_type": "WikibaseItem"
    },
    "P8238": {
        "label": "Bibliography of the History of Slovakia ID",
        "description": "identifier of authorities used in Slovak history bibliography BHS",
        "data_type": "ExternalId"
    },
    "P8647": {
        "label": "Library of Congress Children's Subject Headings ID",
        "description": "identifier for a heading in the Library of Congress Children's Subject Headigns",
        "data_type": "ExternalId"
    },
    "P7068": {
        "label": "Gotlands museum",
        "description": "ID for persons or places connected to objects at Gotlands museums web a local museum in Sweden",
        "data_type": "ExternalId"
    },
    "P6950": {
        "label": "Scandipop topic ID",
        "description": "identifier for an topic on Scandipop site",
        "data_type": "ExternalId"
    },
    "P8242": {
        "label": "Niden INE",
        "description": "identifier in the National Statistics Institute",
        "data_type": "ExternalId"
    },
    "P9382": {
        "label": "Unicode character name",
        "description": "official character name assigned by the Unicode Consortium",
        "data_type": "String"
    },
    "P9350": {
        "label": "PeriodO period ID",
        "description": "identifier for a period in the PeriodO time period gazetteer",
        "data_type": "ExternalId"
    },
    "P6795": {
        "label": "Unified Social Credit Identifier",
        "description": "18-digit permanent unique legal identifier for all juridical persons and other organizations in China",
        "data_type": "ExternalId"
    },
    "P8075": {
        "label": "ScreenScraper group ID",
        "description": "identifier of a group in the ScreenScraper database",
        "data_type": "ExternalId"
    },
    "P8076": {
        "label": "The Cover Project game ID",
        "description": "identifier of a video game in The Cover Project database",
        "data_type": "ExternalId"
    },
    "P6845": {
        "label": "abART document ID",
        "description": "identifier of document in czech database abART",
        "data_type": "ExternalId"
    },
    "P7173": {
        "label": "TA2 ID",
        "description": "Terminologia Anatomica second edition (2019) identifier. Unique identifier for human anatomical terminology.",
        "data_type": "ExternalId"
    },
    "P9206": {
        "label": "Swissvote ID",
        "description": "number of a federal vote in Switzerland on the Swissvotes website",
        "data_type": "ExternalId"
    },
    "P6944": {
        "label": "Bionomia ID",
        "description": "identifier for a collector and/or determiner of natural history specimens, in the Bionomia database",
        "data_type": "ExternalId"
    },
    "P9272": {
        "label": "DeCS ID",
        "description": "identifier in the Health Sciences Descriptors thesaurus",
        "data_type": "ExternalId"
    },
    "P6861": {
        "label": "dbSNP ID",
        "description": "identifier used in dbSNP to uniquely identify a genetic variant",
        "data_type": "ExternalId"
    },
    "P527": {
        "label": "has part",
        "description": "part of this subject; inverse property of \"part of\" (P361). See also \"has parts of the class\" (P2670).",
        "data_type": "WikibaseItem"
    },
    "P610": {
        "label": "highest point",
        "description": "point with highest elevation in a region, or on the path of a race or route",
        "data_type": "WikibaseItem"
    },
    "P8077": {
        "label": "FHF establishment ID",
        "description": "identifier for a French medical facility on the Fédération hospitalière de France website",
        "data_type": "ExternalId"
    },
    "P8895": {
        "label": "All the Tropes identifier",
        "description": "identifier in All the Tropes",
        "data_type": "ExternalId"
    },
    "P4750": {
        "label": "National Historic Ships certificate number",
        "description": "certificate number for a vessel, as found on the National Historic Ships website",
        "data_type": "ExternalId"
    },
    "P6694": {
        "label": "MeSH concept ID",
        "description": "identifier of a Medical Subject Headings concept",
        "data_type": "ExternalId"
    },
    "P7131": {
        "label": "Bureau des longitudes ID",
        "description": "identifier for a member of the 'Bureau des longitudes' between 1795 and 1932",
        "data_type": "ExternalId"
    },
    "P7132": {
        "label": "ComingSoon.it film ID",
        "description": "identifier for a film in the website ComingSoon.it",
        "data_type": "ExternalId"
    },
    "P6654": {
        "label": "Periscope ID",
        "description": "identifier for Periscope live video streaming service",
        "data_type": "ExternalId"
    },
    "P6752": {
        "label": "RGALI ID",
        "description": "identifier for the Russian State Archive of Literature and Art (RGALI)",
        "data_type": "ExternalId"
    },
    "P9334": {
        "label": "Cephalopod Ontology ID",
        "description": "identifier for an anatomical entity in Cephalopod Ontology, an anatomical and developmental ontology for cephalopods",
        "data_type": "ExternalId"
    },
    "P6698": {
        "label": "Japan Search name ID",
        "description": "identifier in the Japan Search portal",
        "data_type": "ExternalId"
    },
    "P9340": {
        "label": "MeSH descriptor/qualifier ID",
        "description": "identifier for a MeSH (Medical Subject Headings) descriptor/qualifier combination",
        "data_type": "ExternalId"
    },
    "P9341": {
        "label": "MeSH qualifier ID",
        "description": "identifier of a MeSH (Medical Subject Headings) qualifier",
        "data_type": "ExternalId"
    },
    "P963": {
        "label": "streaming media URL",
        "description": "URL where the subject's media stream exists",
        "data_type": "Url"
    },
    "P9309": {
        "label": "TaDiRAH ID",
        "description": "identifier for a concept in the Taxonomy of Digital Research Activities in the Humanities (TaDiRAH)",
        "data_type": "ExternalId"
    },
    "P6671": {
        "label": "French public service directory ID",
        "description": "identifier for French public services",
        "data_type": "ExternalId"
    },
    "P6132": {
        "label": "Playbill person ID",
        "description": "identifier for people and organizations at Playbill.com",
        "data_type": "ExternalId"
    },
    "P9027": {
        "label": "e-GEDSH ID",
        "description": "identifier for an article in the Gorgias Encyclopedic Dictionary of the Syriac Heritage: Electronic Edition",
        "data_type": "ExternalId"
    },
    "P7895": {
        "label": "Whaling History ID",
        "description": "identifier for people or ships related to whaling",
        "data_type": "ExternalId"
    },
    "P8283": {
        "label": "ISCO-08 occupation code",
        "description": "International Standard Classification of Occupations code according to the International Labour Organization (ILO) classification structure as of 2008",
        "data_type": "String"
    },
    "P6416": {
        "label": "sport-strana.ru article ID",
        "description": "identifier for an article on site sport-strana.ru",
        "data_type": "ExternalId"
    },
    "P9322": {
        "label": "SVKKL authority ID",
        "description": "identifier in the regional authority database of Central Bohemian Research Library",
        "data_type": "ExternalId"
    },
    "P6077": {
        "label": "British Museum bioID",
        "description": "identifier for a person, organisation, or mythological figure on the British Museum main public website",
        "data_type": "ExternalId"
    },
    "P6689": {
        "label": "MassBank accession ID",
        "description": "accession number for entries in the MassBank database (records of mass spectrometry)",
        "data_type": "ExternalId"
    },
    "P4536": {
        "label": "EThOS thesis ID",
        "description": "identifier of a doctoral thesis, in the British Library's EThOS database",
        "data_type": "ExternalId"
    },
    "P2477": {
        "label": "BDRC Resource ID",
        "description": "Unique identifier from the Buddhist Digital Resource Center",
        "data_type": "ExternalId"
    },
    "P3675": {
        "label": "Hans Christian Andersen Centre work ID",
        "description": "Identifier for a work by Hans Christian Andersen in the Digterens danske Værker catalogue",
        "data_type": "ExternalId"
    },
    "P3705": {
        "label": "RITVA Program ID",
        "description": "Radio and Television Archive Program ID of The National Audiovisual Institute of Finland",
        "data_type": "ExternalId"
    },
    "P4300": {
        "label": "YouTube playlist ID",
        "description": "identifier for a playlist (official) on YouTube containing videos for this item",
        "data_type": "ExternalId"
    },
    "P3894": {
        "label": "OSTI article ID",
        "description": "identifier of a scientific article at Office of Scientific and Technical Information",
        "data_type": "ExternalId"
    },
    "P4793": {
        "label": "identifiers.org prefix",
        "description": "code used in front of an identifier, as supported by identifiers.org and n2t.net",
        "data_type": "ExternalId"
    },
    "P2683": {
        "label": "Bekker Number",
        "description": "reference to works in the Corpus Aristotelicum",
        "data_type": "ExternalId"
    },
    "P2703": {
        "label": "BFI National Archive work ID",
        "description": "identifier for (creative) works and objects (moving image, documents, stills, posters, designs, books, articles) in the BFI's Collections Information Database (CID)",
        "data_type": "ExternalId"
    },
    "P4011": {
        "label": "Semantic Scholar paper ID",
        "description": "identifier for an article in the Semantic Scholar database (40 hexadecimal digits; for numeric ID see P8299)",
        "data_type": "ExternalId"
    },
    "P4327": {
        "label": "BHL bibliography ID",
        "description": "identifier for a journal or book (\"bibliography\") in the Biodiversity Heritage Library (BHL)",
        "data_type": "ExternalId"
    },
    "P4337": {
        "label": "digitised page from Lloyd's Register of Ships from 1930 to 1945",
        "description": "ID of a scanned page in Plimsoll ShipData online copy of Lloyd's Register of Ships 1930-1945",
        "data_type": "ExternalId"
    },
    "P3589": {
        "label": "GCD series ID",
        "description": "identifier of a comic book series in the Grand Comics Database (GCD)",
        "data_type": "ExternalId"
    },
    "P3069": {
        "label": "UN document symbol",
        "description": "unique number assigned to United Nations  documents",
        "data_type": "ExternalId"
    },
    "P4430": {
        "label": "New York City Parks monument ID",
        "description": "identifier for monuments and public art in the spaces administered by the New York City Department of Parks and Recreation",
        "data_type": "ExternalId"
    },
    "P3802": {
        "label": "Launchpad.net project ID",
        "description": "identifier for an official Launchpad repository (\"project\") for a software application",
        "data_type": "ExternalId"
    },
    "P3805": {
        "label": "Tax-exempt heritage asset ID",
        "description": "identifier of an artwork, in the UK government's HMRC database of tax-exempt heritage assets",
        "data_type": "ExternalId"
    },
    "P3442": {
        "label": "Debian stable package",
        "description": "name of the official Debian stable package",
        "data_type": "ExternalId"
    },
    "P2666": {
        "label": "Datahub page",
        "description": "Datahub page of a dataset",
        "data_type": "ExternalId"
    },
    "P5711": {
        "label": "Revues Littéraires ID",
        "description": "identifier for a literary magazine on 'Revues Littéraires'",
        "data_type": "ExternalId"
    },
    "P4485": {
        "label": "INDUCKS miniseries ID",
        "description": "identifier for a miniseries of a printed publication of the Disney Universe",
        "data_type": "ExternalId"
    },
    "P2858": {
        "label": "Killer List of Videogames ID",
        "description": "identifier in Killer List of Videogames",
        "data_type": "ExternalId"
    },
    "P4763": {
        "label": "crates.io ID",
        "description": "identifier on crates.io, the Rust package manager (used by cargo)",
        "data_type": "ExternalId"
    },
    "P2613": {
        "label": "TED talk ID",
        "description": "identifier of a talk, in the TED database",
        "data_type": "ExternalId"
    },
    "P2558": {
        "label": "autores.uy ID",
        "description": "unique identifier of Uruguayan authors and artistic, scientific or literary works",
        "data_type": "ExternalId"
    },
    "P4816": {
        "label": "Lemon 64 identifier",
        "description": "identifier in the Lemon 64 database of Commodore 64 videogames",
        "data_type": "ExternalId"
    },
    "P3951": {
        "label": "BioRxiv ID",
        "description": "identifier of a document in bioRxiv, a preprint repository for the biological sciences launched in November 2013",
        "data_type": "ExternalId"
    },
    "P2864": {
        "label": "OpenCritic ID",
        "description": "identifier in OpenCritic",
        "data_type": "ExternalId"
    },
    "P3784": {
        "label": "CiteSeerX article ID",
        "description": "identifier string for a scientific article available from CiteSeer",
        "data_type": "ExternalId"
    },
    "P4157": {
        "label": "MEG ID",
        "description": "ID number for the online catalogue of Musée d'ethnographie de Genève",
        "data_type": "ExternalId"
    },
    "P3116": {
        "label": "ISzDb dub ID",
        "description": "identifier of a dub version for a film in the ISzDb (a Hungarian dub database)",
        "data_type": "ExternalId"
    },
    "P3959": {
        "label": "NNL work ID",
        "description": "ID for a record in the National Library of Israel (NLI) Online Catalog (NNL)",
        "data_type": "ExternalId"
    },
    "P4710": {
        "label": "Guardiana ID",
        "description": "identifier in the SEGA videogame database Guardiana",
        "data_type": "ExternalId"
    },
    "P2540": {
        "label": "Aarne–Thompson–Uther Tale Type Index",
        "description": "index used to classify folktales",
        "data_type": "String"
    },
    "P5680": {
        "label": "Hoopla title ID",
        "description": "an audiobook, film, music album, comic, ebook, or television series on Hoopla",
        "data_type": "ExternalId"
    },
    "P5865": {
        "label": "IGN film ID",
        "description": "identifier for films at the IGN website",
        "data_type": "ExternalId"
    },
    "P5878": {
        "label": "Taxonomic Literature 2 number",
        "description": "catalogue number in Taxonomic Literature 2",
        "data_type": "String"
    },
    "P4483": {
        "label": "INDUCKS story ID",
        "description": "identifier for a comic book story related to the Disney Universe",
        "data_type": "ExternalId"
    },
    "P2687": {
        "label": "NDL JPNO",
        "description": "Japanese National Bibliography Number of the National Diet Library",
        "data_type": "ExternalId"
    },
    "P3212": {
        "label": "ISAN",
        "description": "unique identifier for audiovisual works and related versions, similar to ISBN for books",
        "data_type": "ExternalId"
    },
    "P2961": {
        "label": "BVPH publication ID",
        "description": "identifier in Biblioteca Virtual de Prensa Histórica",
        "data_type": "ExternalId"
    },
    "P5691": {
        "label": "BND ID",
        "description": "digital collection identifier for notorious creative works within the Biblioteca Nacional Digital of the Biblioteca Nacional de Portugal",
        "data_type": "ExternalId"
    },
    "P4028": {
        "label": "Google Scholar paper ID",
        "description": "identifier for a paper in Google Scholar",
        "data_type": "ExternalId"
    },
    "P5733": {
        "label": "bilibili ID",
        "description": "identifier for an anime or drama topic on bilibili",
        "data_type": "ExternalId"
    },
    "P2475": {
        "label": "NAVA ID",
        "description": "identifier of a work, in the National Audiovisual Archive of Hungary",
        "data_type": "ExternalId"
    },
    "P3889": {
        "label": "Swiss Federal Archives ID",
        "description": "Identifier for an entity (portfolio, dossier, document) of the Swiss Federal Archives",
        "data_type": "ExternalId"
    },
    "P4705": {
        "label": "World of Spectrum ID",
        "description": "identifier in the videogame database World of Spectrum",
        "data_type": "ExternalId"
    },
    "P2749": {
        "label": "PRONOM software identifier",
        "description": "identifier (PUID) for software in the technical registry PRONOM",
        "data_type": "ExternalId"
    },
    "P4256": {
        "label": "BOE ID",
        "description": "external and unique identifier which represents a publication in the Spanish Official Gazette",
        "data_type": "ExternalId"
    },
    "P5116": {
        "label": "LGDB game ID",
        "description": "identifier for a game in the Linux Game Database",
        "data_type": "ExternalId"
    },
    "P2879": {
        "label": "Library of the National Congress of Argentina ID",
        "description": "identifier of a work in the National Library of Argentine Congress",
        "data_type": "ExternalId"
    },
    "P5833": {
        "label": "Playbill production ID",
        "description": "identifier for a theatrical production in the Playbill.com database",
        "data_type": "ExternalId"
    },
    "P3418": {
        "label": "Google Play Store app ID",
        "description": "package name of an app registered on Google Play",
        "data_type": "ExternalId"
    },
    "P3528": {
        "label": "Luding game ID",
        "description": "identifier for a game at the Luding database",
        "data_type": "ExternalId"
    },
    "P3414": {
        "label": "Yle Areena item ID",
        "description": "program or series ID in Yle Areena service",
        "data_type": "ExternalId"
    },
    "P3431": {
        "label": "Publons Publication ID",
        "description": "Publons is a website where academics share pre-publication and post-publication peer reviews; any article with a DOI can have a Publons Publication ID, which can then link to a page showing the peer review history of the publication",
        "data_type": "ExternalId"
    },
    "P3939": {
        "label": "ESTC citation number",
        "description": "identifier in the English Short Title Catalogue",
        "data_type": "ExternalId"
    },
    "P4435": {
        "label": "snap package",
        "description": "Snapcraft-based Linux software install",
        "data_type": "ExternalId"
    },
    "P3570": {
        "label": "European Case Law ID",
        "description": "identifier for case law in Europe",
        "data_type": "ExternalId"
    },
    "P5117": {
        "label": "LGDB emulator ID",
        "description": "identifier for an emulator in the Linux Game Database",
        "data_type": "ExternalId"
    },
    "P3089": {
        "label": "Flags of the World ID",
        "description": "identifier for a flag in the Flags of the World database",
        "data_type": "ExternalId"
    },
    "P3098": {
        "label": "ClinicalTrials.gov Identifier",
        "description": "identifier in the ClinicalTrials.gov database",
        "data_type": "ExternalId"
    },
    "P3788": {
        "label": "BNMM authority ID",
        "description": "authority control identifier used at the National Library of Argentina",
        "data_type": "ExternalId"
    },
    "P5808": {
        "label": "Alternativa Teatral work ID",
        "data_type": "ExternalId"
    },
    "P3597": {
        "label": "F-Droid package",
        "description": "Android package in the F-Droid official repository",
        "data_type": "ExternalId"
    },
    "P2735": {
        "label": "Unz Review journal ID",
        "description": "journal identifier at The Unz Review (unz.org), a content-archiving website",
        "data_type": "ExternalId"
    },
    "P5385": {
        "label": "IGN game ID",
        "description": "External identifier for video games at IGN",
        "data_type": "ExternalId"
    },
    "P5119": {
        "label": "LGDB engine ID",
        "description": "identifier for a game engine in the Linux Game Database",
        "data_type": "ExternalId"
    },
    "P4455": {
        "label": "Coinage of the Roman Republic Online ID",
        "description": "identifier of a coin in the Coinage of the Roman Republic Online platform, a digital version of Michael Crawford's 1974 publication Roman Republican Coinage (RRC)",
        "data_type": "ExternalId"
    },
    "P4465": {
        "label": "Ecole des chartes thesis abstract ID",
        "description": "identifier for the abstract of the diploma thesis of an archivist paleographer (graduate student) at the Ecole Nationale des Chartes",
        "data_type": "ExternalId"
    },
    "P3147": {
        "label": "United States Reports ID",
        "description": "ID for US Supreme Court Opinions on OpenJurist website",
        "data_type": "ExternalId"
    },
    "P3732": {
        "label": "PhilPapers record",
        "description": "identifier for papers on PhilPapers",
        "data_type": "ExternalId"
    },
    "P3976": {
        "label": "BVMC work ID",
        "description": "identifier of a work on the Biblioteca Virtual Miguel de Cervantes",
        "data_type": "ExternalId"
    },
    "P2606": {
        "label": "PlayStation ID",
        "description": "unique identifier of PlayStation software and hardware assigned by Sony Computer Entertainment Inc.",
        "data_type": "ExternalId"
    },
    "P3112": {
        "label": "DistroWatch ID",
        "description": "identifier for an operating system at DistroWatch.com",
        "data_type": "ExternalId"
    },
    "P3861": {
        "label": "App Store app ID (global)",
        "description": "identifier for a mobile application in iTunes App Store",
        "data_type": "ExternalId"
    },
    "P4165": {
        "label": "CODECS ID",
        "description": "identifier in CODECS database for Celtic studies",
        "data_type": "ExternalId"
    },
    "P4107": {
        "label": "Framalibre ID",
        "description": "unique identifier in the Framalibre free software directory",
        "data_type": "ExternalId"
    },
    "P4178": {
        "label": "Beazley Archive Pottery Database ID",
        "description": "identifier for an artefact in the Beazley Archive Pottery Database",
        "data_type": "ExternalId"
    },
    "P3825": {
        "label": "United States Statutes at Large citation",
        "description": "citation of an Act of the United States Congress to the United States Statutes at Large",
        "data_type": "ExternalId"
    },
    "P4760": {
        "label": "LTI Korea Library ID",
        "description": "identifier for a Korean writer, book, and event in Literature Translation Institute of Korea Library (library.klti.or.kr)",
        "data_type": "ExternalId"
    },
    "P2704": {
        "label": "EIDR content ID",
        "description": "identifier for a film or television work, edit or manifestation, in the Entertainment Identifier Registry",
        "data_type": "ExternalId"
    },
    "P5315": {
        "label": "BioStor work ID",
        "description": "identifier for a work in the BioStor digital library",
        "data_type": "ExternalId"
    },
    "P3783": {
        "label": "Christie's object ID",
        "description": "identifier for an object offered for sale by Christies",
        "data_type": "ExternalId"
    },
    "P5118": {
        "label": "LGDB tool ID",
        "description": "identifier for a software tool in the Linux Game Database",
        "data_type": "ExternalId"
    },
    "P3495": {
        "label": "FilmPolski.pl ID",
        "description": "identifier for person, film (movie), or TV series, in the filmpolski.pl database",
        "data_type": "ExternalId"
    },
    "P3837": {
        "label": "United States Public Law",
        "description": "citation to United States Public Law",
        "data_type": "ExternalId"
    },
    "P4231": {
        "label": "United Nations Treaty Series Registration Number",
        "description": "registration number of a treaty within the United Nations Treaty Series",
        "data_type": "ExternalId"
    },
    "P4162": {
        "label": "AUR package",
        "description": "official name of the package on the Arch User Repository",
        "data_type": "ExternalId"
    },
    "P4258": {
        "label": "Gallica ID",
        "description": "Gallica ID of a creative work (eg book scan)",
        "data_type": "ExternalId"
    },
    "P4230": {
        "label": "Sefaria ID",
        "description": "identifier for Sefaria, an online library of Jewish texts with freely licensed content",
        "data_type": "ExternalId"
    },
    "P3332": {
        "label": "ACM Digital Library citation ID",
        "description": "unique identifier for a bibliographic record in the Association for Computing Machinery (ACM) Digital Library",
        "data_type": "ExternalId"
    },
    "P4280": {
        "label": "International Standard Text Code",
        "description": "ISO standard unique identifier for text-based works",
        "data_type": "ExternalId"
    },
    "P2537": {
        "label": "Free Software Directory entry",
        "description": "Link to the FSD page on a given software or license",
        "data_type": "ExternalId"
    },
    "P2880": {
        "label": "NIOSHTIC-2 ID",
        "description": "identifier in the NIOSHTIC-2 database run by National Institute for Occupational Safety and Health (part of US CDC)",
        "data_type": "ExternalId"
    },
    "P3499": {
        "label": "Gentoo package",
        "description": "name of the official Gentoo package of this application",
        "data_type": "ExternalId"
    },
    "P5331": {
        "label": "OCLC work ID",
        "description": "identifier for a work-level item in the OCLC linked data system",
        "data_type": "ExternalId"
    },
    "P5792": {
        "label": "NooSFere series ID",
        "description": "identifier for a series in NooSFere Database",
        "data_type": "ExternalId"
    },
    "P3343": {
        "label": "legislation.gov.uk ID",
        "description": "identifier of this legislation on the legislation.gov.uk website",
        "data_type": "ExternalId"
    },
    "P4360": {
        "label": "Monumentos de São Paulo ID",
        "description": "identifier for a monument in São Paulo city, Brazil, on the Monumentos de São Paulo website",
        "data_type": "ExternalId"
    },
    "P4218": {
        "label": "shelf life",
        "description": "length of time that a commodity may be stored without becoming unfit for use or consumption",
        "data_type": "Quantity"
    },
    "P5423": {
        "label": "floor number",
        "description": "floor number in a street address. To be used as a qualifier of Property:P669 \"located on street\" or as a property of a place",
        "data_type": "String"
    },
    "P3295": {
        "label": "code",
        "description": "code used to represent a specific concept in a given encoding",
        "data_type": "String"
    },
    "P3643": {
        "label": "significant environmental impact",
        "description": "types of environmental issues determined to have a significant impact on the object",
        "data_type": "WikibaseItem"
    },
    "P4295": {
        "label": "readership",
        "description": "average number of readers of a newspaper or other periodical, per issue",
        "data_type": "Quantity"
    },
    "P4401": {
        "label": "ID Museus Brazil",
        "description": "identifier for information on Brazilian museums from museus.cultura.gov.br (Museusbr)",
        "data_type": "ExternalId"
    },
    "P2737": {
        "label": "union of",
        "description": "every instance of this class is an instance of at least one class in that list of classes",
        "data_type": "WikibaseItem"
    },
    "P5677": {
        "label": "flexural strength",
        "description": "the stress in a material just before it yields in a flexure test",
        "data_type": "Quantity"
    },
    "P2353": {
        "label": "statistical unit",
        "description": "member of a dataset",
        "data_type": "WikibaseItem"
    },
    "P1687": {
        "label": "Wikidata property",
        "description": "main Wikidata property for this item",
        "data_type": "WikibaseProperty"
    },
    "P2578": {
        "label": "studies",
        "description": "subject item is the academic field studying the object item of this property",
        "data_type": "WikibaseItem"
    },
    "P2702": {
        "label": "dataset distribution",
        "description": "particular manner of distribution of a data set (database or file) that is publicly available",
        "data_type": "WikibaseItem"
    },
    "P5678": {
        "label": "dissipation factor",
        "description": "measure of loss-rate of energy of a mode of oscillation (mechanical, electrical, or electromechanical) in a dissipative system",
        "data_type": "Quantity"
    },
    "P1557": {
        "label": "manifestation of",
        "description": "inherent and characteristic embodiment of a given concept",
        "data_type": "WikibaseItem"
    },
    "P5444": {
        "label": "model year",
        "description": "year a specific model of a product (such as a car) was released, likely different from the actual year of production",
        "data_type": "WikibaseItem"
    },
    "P5022": {
        "label": "test score",
        "description": "score that the item achieved in a test; use as qualifier for \"test taken\" (P5021)",
        "data_type": "Quantity"
    },
    "P2198": {
        "label": "average gradient",
        "description": "gradient expressed as a percentage (between 0 and 1)",
        "data_type": "Quantity"
    },
    "P4330": {
        "label": "contains",
        "description": "item or substance located within this item but not part of it",
        "data_type": "WikibaseItem"
    },
    "P5471": {
        "label": "Harmonized System Code",
        "description": "code in the Harmonized System for a category of object traded internationally",
        "data_type": "String"
    },
    "P4968": {
        "label": "relevant qualification",
        "description": "practitioners of this industry get this degree, licence or certification after specialist education, apprenticeship, or professional review. This includes qualifications that one needs to obtain in order to join the industry or qualifications that one obtains after a certain level of experience in the industry in order to progress further in their career.",
        "data_type": "WikibaseItem"
    },
    "P3113": {
        "label": "does not have part",
        "description": "expected part that the item does not have (for qualities, use P6477)",
        "data_type": "WikibaseItem"
    },
    "P5012": {
        "label": "special rank",
        "description": "non-military rank given to a civil office holder in Russia",
        "data_type": "WikibaseItem"
    },
    "P2341": {
        "label": "indigenous to",
        "description": "place that a language, folk dance, cooking style, food, species or other cultural expression is found (or was originally found)",
        "data_type": "WikibaseItem"
    },
    "P5061": {
        "label": "unit symbol",
        "description": "abbreviation of a unit for each language; if not provided, then it should default to English",
        "data_type": "Monolingualtext"
    },
    "P1817": {
        "label": "addressee",
        "description": "person or organization to whom a letter or note is addressed",
        "data_type": "WikibaseItem"
    },
    "P5030": {
        "label": "talk show guest",
        "description": "person invited in a talk show",
        "data_type": "WikibaseItem"
    },
    "P2918": {
        "label": "PO Box",
        "description": "Post Office box number, as part of an address",
        "data_type": "String"
    },
    "P5166": {
        "label": "established from medical condition",
        "description": "cell line or xenograft established from an individual person or animal with a specific medical condition",
        "data_type": "WikibaseItem"
    },
    "P2184": {
        "label": "history of topic",
        "description": "historical development of an subject's topic",
        "data_type": "WikibaseItem"
    },
    "P3093": {
        "label": "recovered by",
        "description": "person, organisation or vehicle that recovered the item. Use the most specific value known.",
        "data_type": "WikibaseItem"
    },
    "P5027": {
        "label": "number of representations",
        "description": "number of shows (including the premiere) of a concert tour or a performing arts production",
        "data_type": "Quantity"
    },
    "P5591": {
        "label": "normalized Unicode character",
        "description": "Unicode character that represents the given item in NFC (Normalization Form Canonical Composition)",
        "data_type": "WikibaseItem"
    },
    "P1435": {
        "label": "heritage designation",
        "description": "heritage designation of a cultural or natural site",
        "data_type": "WikibaseItem"
    },
    "P5673": {
        "label": "shear modulus",
        "description": "ratio of shear stress to the shear strain",
        "data_type": "Quantity"
    },
    "P5624": {
        "label": "pitting resistance equivalent number",
        "description": "predictive measurement of a stainless steels resistance to localized pitting corrosion",
        "data_type": "Quantity"
    },
    "P5623": {
        "label": "type of water supply",
        "description": "the type of water supply for fountains, mountain huts, and other entities where water can be obtained",
        "data_type": "WikibaseItem"
    },
    "P4934": {
        "label": "calculated from",
        "description": "value of the subject item cannot be measured directly or can be calculated from the following measurands or attributes",
        "data_type": "WikibaseItem"
    },
    "P5670": {
        "label": "glass transition temperature",
        "description": "transition in amorphous materials from a hard and relatively brittle \"glassy\" state into a viscous or rubbery state as the temperature is increased",
        "data_type": "Quantity"
    },
    "P5676": {
        "label": "relative permeability",
        "description": "measure of the ability of a material to support the formation of a magnetic field within itself",
        "data_type": "Quantity"
    },
    "P1542": {
        "label": "has effect",
        "description": "effect of this item",
        "data_type": "WikibaseItem"
    },
    "P5674": {
        "label": "thermal diffusivity",
        "description": "measures the rate of transfer of heat of a material from the hot side to the cold side",
        "data_type": "Quantity"
    },
    "P5066": {
        "label": "operating temperature",
        "description": "temperature at which a device operates. Use qualifier \"criterion used\" (P1013) and values such as \"maximum\" (Q10578722), \"minimum\" (Q10585806), etc.",
        "data_type": "Quantity"
    },
    "P1581": {
        "label": "official blog",
        "description": "URL to the blog of this person or organization",
        "data_type": "Url"
    },
    "P5067": {
        "label": "non-operating temperature",
        "description": "temperature that a device can bear when it's turned off",
        "data_type": "Quantity"
    },
    "P5672": {
        "label": "linear thermal expansion coefficient",
        "description": "measures the fractional change in size per degree change in temperature at a constant pressure",
        "data_type": "Quantity"
    },
    "P4500": {
        "label": "family relationship degree",
        "description": "kinship degree, degree of consanguinity, generation of people with this kinship type when clearly identifiable.",
        "data_type": "Quantity"
    },
    "P1268": {
        "label": "represents",
        "description": "organization, individual, or concept that an entity represents",
        "data_type": "WikibaseItem"
    },
    "P5178": {
        "label": "glossary entry at Wikipedia",
        "description": "description of a concept in a glossary at Wikipedia. Use sitelinks for Wikipedia articles.  Format: use full URL including anchor (#)",
        "data_type": "Url"
    },
    "P5195": {
        "label": "Wikidata Dataset Imports page",
        "description": "URL for the import page for a dataset on Wikidata:Dataset Imports",
        "data_type": "Url"
    },
    "P1880": {
        "label": "measurement scale",
        "description": "scale by which a phenomenon is measured",
        "data_type": "WikibaseItem"
    },
    "P3447": {
        "label": "mirrors data from",
        "description": "the website automatically crawls or mirrors another website as its data source",
        "data_type": "WikibaseItem"
    },
    "P5202": {
        "label": "adapted by",
        "description": "person responsible for the arrangement or adaptation resulting in this version of the original work",
        "data_type": "WikibaseItem"
    },
    "P5439": {
        "label": "research measurement",
        "description": "a measurement of an experimentally observed variable",
        "data_type": "WikibaseItem"
    },
    "P5436": {
        "label": "number of viewers/listeners",
        "description": "number of viewers of a television or broadcasting program; web traffic on websites",
        "data_type": "Quantity"
    },
    "P3095": {
        "label": "practiced by",
        "description": "type of agents that study this subject or work in this profession",
        "data_type": "WikibaseItem"
    },
    "P5046": {
        "label": "International Classification for Standards",
        "description": "classification system for technical standards",
        "data_type": "String"
    },
    "P5004": {
        "label": "in opposition to",
        "description": "qualifier to indicate the purpose of a social action",
        "data_type": "WikibaseItem"
    },
    "P1546": {
        "label": "motto",
        "description": "description of the motto of the subject",
        "data_type": "WikibaseItem"
    },
    "P5028": {
        "label": "sound designer",
        "description": "person responsible for the sound on a stage, in a film or in a radio drama",
        "data_type": "WikibaseItem"
    },
    "P5069": {
        "label": "service life",
        "description": "product's total life in use from the point of sale to the point of discard",
        "data_type": "Quantity"
    },
    "P5675": {
        "label": "relative permittivity",
        "description": "measure of capacitance that is encountered when forming an electric field in a particular medium expressed as a ratio relative to the permittivity of vacuum",
        "data_type": "Quantity"
    },
    "P2361": {
        "label": "online service",
        "description": "online service or online service provider tied to an item",
        "data_type": "WikibaseItem"
    },
    "P5348": {
        "label": "angular diameter",
        "description": "angular measurement describing how large a sphere or circle appears from a given point of view",
        "data_type": "Quantity"
    },
    "P1325": {
        "label": "external data available at",
        "description": "URL where external data on this item can be found",
        "data_type": "Url"
    },
    "P5102": {
        "label": "nature of statement",
        "description": "(qualifier only) the underlying circumstances of this statement",
        "data_type": "WikibaseItem"
    },
    "P1552": {
        "label": "has quality",
        "description": "the entity has an inherent or distinguishing non-material characteristic",
        "data_type": "WikibaseItem"
    },
    "P1451": {
        "label": "motto text",
        "description": "short motivation sentence associated to item",
        "data_type": "Monolingualtext"
    },
    "P2579": {
        "label": "studied by",
        "description": "subject is studied by this science or domain or by this type of student",
        "data_type": "WikibaseItem"
    },
    "P5125": {
        "label": "Wikimedia outline",
        "description": "item about an outline of the topic at Wikimedia sites",
        "data_type": "WikibaseItem"
    },
    "P1269": {
        "label": "facet of",
        "description": "topic of which this item is an aspect, item that offers a broader perspective on the same topic",
        "data_type": "WikibaseItem"
    },
    "P2596": {
        "label": "culture",
        "description": "human culture or people (or several cultures) associated with this item",
        "data_type": "WikibaseItem"
    },
    "P1299": {
        "label": "depicted by",
        "description": "object depicting this subject (creative works, books of the bible)",
        "data_type": "WikibaseItem"
    },
    "P5282": {
        "label": "ground level 360 degree view",
        "description": "URL to access a service that allows to visually navigate through images at ground level in the proximity of the item",
        "data_type": "Url"
    },
    "P5096": {
        "label": "member of the crew of",
        "description": "person who has been a member of a crew associated with the vessel or spacecraft. For spacecraft, inverse of crew member (P1029), backup or reserve team or crew (P3015)",
        "data_type": "WikibaseItem"
    },
    "P5021": {
        "label": "test taken",
        "description": "subject took the test or exam, or was assessed per test or method",
        "data_type": "WikibaseItem"
    },
    "P2852": {
        "label": "emergency phone number",
        "description": "telephone number to contact the emergency services",
        "data_type": "WikibaseItem"
    },
    "P5307": {
        "label": "display technology",
        "description": "technology used by the display of this device",
        "data_type": "WikibaseItem"
    },
    "P2445": {
        "label": "metasubclass of",
        "description": "relation between two metaclasses: instances of this metaclass are likely to be subclasses of classes that are instances of the target metaclass",
        "data_type": "WikibaseItem"
    },
    "P4967": {
        "label": "prerequisite",
        "description": "prior event or achievement that a person or team needs to complete before joining or obtaining the item topic, e.g. a qualifying round of a sports event, a lower-level course in an educational program, or a lower level of a professional qualification",
        "data_type": "WikibaseItem"
    },
    "P2738": {
        "label": "disjoint union of",
        "description": "every instance of this class is an instance of exactly one class in that list of classes",
        "data_type": "WikibaseItem"
    },
    "P5203": {
        "label": "topographic map",
        "description": "topographic map covering this geographic object",
        "data_type": "WikibaseItem"
    },
    "P3712": {
        "label": "objective of project or action",
        "description": "desired result or outcome",
        "data_type": "WikibaseItem"
    },
    "P5422": {
        "label": "has pattern",
        "description": "pattern, design, or motif intrinsic to, incorporated into, or applied to the surface of the subject",
        "data_type": "WikibaseItem"
    },
    "P5304": {
        "label": "type locality (biology)",
        "description": "geographical place of capture, collection or observation of the name-bearing type",
        "data_type": "WikibaseItem"
    },
    "P5280": {
        "label": "radical",
        "description": "radical of Chinese character",
        "data_type": "WikibaseItem"
    },
    "P5244": {
        "label": "reading pattern of Han character",
        "description": "https://www.wikidata.org/wiki/Wikidata:Property_proposal/reading_pattern_of_kanji",
        "data_type": "WikibaseItem"
    },
    "P5328": {
        "label": "revival or touring version of",
        "description": "indicates the original performing arts production which served as a basis for the given production (which uses the same performance plan)",
        "data_type": "WikibaseItem"
    },
    "P3730": {
        "label": "next higher rank",
        "description": "higher rank or level in a ranked hierarchy like sport league, military ranks. If there are several possible, list each one and qualify with \"criterion used\" (P1013), avoid using ranks and date qualifiers. For sports leagues/taxa, use specific properties instead.",
        "data_type": "WikibaseItem"
    },
    "P3176": {
        "label": "uses property",
        "description": "Wikidata property used, discussed, or otherwise analyzed in this work (not the meaning of the property itself)",
        "data_type": "WikibaseProperty"
    },
    "P3729": {
        "label": "next lower rank",
        "description": "lower rank or level in a ranked hierarchy like sport league, military ranks. If there are several possible, list each one and qualify with \"criterion used\" (P1013), avoid using ranks and date qualifiers. For sports leagues/taxa, use specific properties instead.",
        "data_type": "WikibaseItem"
    },
    "P3037": {
        "label": "spatial reference system",
        "description": "frame of reference used to produce a map",
        "data_type": "WikibaseItem"
    },
    "P2212": {
        "label": "angular distance",
        "description": "size of the angle between the two directions originating from the observer and pointing towards these two objects (use with P2210)",
        "data_type": "Quantity"
    },
    "P2559": {
        "label": "Wikidata usage instructions",
        "description": "text describing how to use a property or item. Eventually, this statement is to replace usage instructions currently found in description. Until the corresponding feature is implemented, do not remove it from description",
        "data_type": "Monolingualtext"
    },
    "P1661": {
        "label": "Alexa rank",
        "description": "the website's Alexa ranking with P585 as a qualifier for each value; the current value should be marked 'preferred'",
        "data_type": "Quantity"
    },
    "P5204": {
        "label": "date of commercialization",
        "description": "date when a product was first commercialized",
        "data_type": "Time"
    },
    "P3301": {
        "label": "broadcast by",
        "description": "channel, network, website or service that broadcast this item over radio, TV or the Internet",
        "data_type": "WikibaseItem"
    },
    "P1921": {
        "label": "formatter URI for RDF resource",
        "description": "formatter URL for RDF resource: URI template from which \"$1\" can be automatically replaced with the effective property value on items (it is the URI of the resources, not the URI of the RDF file describing it)",
        "data_type": "String"
    },
    "P1293": {
        "label": "Royal Aero Club Aviator's Certificate ID",
        "description": "identifier",
        "data_type": "ExternalId"
    },
    "P1183": {
        "label": "Gewässerkennzahl",
        "description": "a number to hydrographically order rivers and streams, use more specific properties where possible",
        "data_type": "String"
    },
    "P1261": {
        "label": "Scandinavian Runic-text Database",
        "description": "database about runic inscriptions (including runestones), mainly in Sweden, but stones in all of Europe, Greenland and U.S. can be found here",
        "data_type": "ExternalId"
    },
    "P1838": {
        "label": "PSS-archi ID",
        "description": "identifier for a building in the PSS-archi database",
        "data_type": "ExternalId"
    },
    "P696": {
        "label": "Interlex ID",
        "description": "identifier in the Interlex database (used to be Neurolex)",
        "data_type": "ExternalId"
    },
    "P1692": {
        "label": "ICD-9-CM",
        "description": "identifier in the ICD adaption assigning diagnostic and procedure codes",
        "data_type": "String"
    },
    "P2273": {
        "label": "Heidelberg Academy for Sciences and Humanities member ID",
        "description": "entry in the list of members of the Heidelberg Academy for Sciences and Humanities",
        "data_type": "ExternalId"
    },
    "P2272": {
        "label": "Hederich encyclopedia article ID",
        "description": "entry in Hederich’s encyclopedia of mythology, 3rd edition (1770), via zeno.org",
        "data_type": "ExternalId"
    },
    "P1396": {
        "label": "Linguasphere code",
        "description": "identifier for a language from Linguasphere Observatory",
        "data_type": "ExternalId"
    },
    "P1022": {
        "label": "CNO-11 occupation code",
        "description": "Spanish classification of occupations CNO-11 maintained by the Spanish INE (Instituto Nacional de Estadística)",
        "data_type": "ExternalId"
    },
    "P2153": {
        "label": "PubChem Substance ID (SID)",
        "description": "substance (e.g. mixtures, undefined stereochemistry, ...) identifier in the PubChem database.",
        "data_type": "ExternalId"
    },
    "P1662": {
        "label": "DOI prefix",
        "description": "identifier specific to a DOI registrant",
        "data_type": "ExternalId"
    },
    "P2792": {
        "label": "ASF KID Cave Tag Number",
        "description": "unique identifier for caves within the Karst Index Database of the Australian Speleological Federation",
        "data_type": "ExternalId"
    },
    "P1738": {
        "label": "Merck Index monograph",
        "description": "URL stem of page in The Merck Index Online",
        "data_type": "ExternalId"
    },
    "P1748": {
        "label": "NCI Thesaurus ID",
        "description": "identifier in the United States National Cancer Institute Thesaurus, vocabulary for clinical care, translational and basic research, etc",
        "data_type": "String"
    },
    "P662": {
        "label": "PubChem CID",
        "description": "identifier from database of chemical molecules and their activities in biological assays (Compound ID number)",
        "data_type": "ExternalId"
    },
    "P829": {
        "label": "OEIS ID",
        "description": "identifer on the On-Line Encyclopedia of Integer Sequences",
        "data_type": "ExternalId"
    },
    "P1163": {
        "label": "media type",
        "description": "IANA-registered identifier for a file type (MIME)",
        "data_type": "String"
    },
    "P1300": {
        "label": "bibcode",
        "description": "bibcode for a journal",
        "data_type": "ExternalId"
    },
    "P2572": {
        "label": "hashtag",
        "description": "hashtag associated with this item. Format: do not include the \"#\" symbol",
        "data_type": "String"
    },
    "P679": {
        "label": "ZVG number",
        "description": "identifier of GESTIS database",
        "data_type": "ExternalId"
    },
    "P1837": {
        "label": "Gaoloumi ID",
        "description": "identifier for a building in the Gaoloumi database",
        "data_type": "ExternalId"
    },
    "P760": {
        "label": "DPLA ID",
        "description": "identifier for books, paintings, films, museum objects and archival records that have been digitised throughout United States",
        "data_type": "ExternalId"
    },
    "P1607": {
        "label": "Dialnet author ID",
        "description": "identifier of an author in Dialnet",
        "data_type": "ExternalId"
    },
    "P1329": {
        "label": "phone number",
        "description": "telephone number in standard format (RFC3966), without 'tel:' prefix",
        "data_type": "String"
    },
    "P1438": {
        "label": "Jewish Encyclopedia ID (Russian)",
        "description": "Electronic Jewish Encyclopedia in Russian",
        "data_type": "ExternalId"
    },
    "P1966": {
        "label": "Biblioteca Nacional de Chile catalogue number",
        "description": "catalogue number for books (editions), periodical, magazine, map, photography and other media in the Biblioteca Nacional de Chile",
        "data_type": "ExternalId"
    },
    "P2410": {
        "label": "WikiPathways ID",
        "description": "describes in which biological pathways a biological entity occurs",
        "data_type": "String"
    },
    "P897": {
        "label": "United States Army and Air Force aircraft designation",
        "description": "identifier for a US Army and Air Force aircraft until 1962",
        "data_type": "String"
    },
    "P1991": {
        "label": "LPSN URL",
        "description": "URL for the website List of Prokaryotic names with Standing in Nomenclature (LPSN)",
        "data_type": "Url"
    },
    "P1929": {
        "label": "ClinVar Variation ID",
        "description": "identifier in the ClinVar database for human genomic variation",
        "data_type": "ExternalId"
    },
    "P2720": {
        "label": "embed URL template",
        "description": "formatter URL for embeddable content: URI template from which \"$1\" can be automatically replaced with the effective property value on items",
        "data_type": "String"
    },
    "P1947": {
        "label": "Mapillary ID",
        "description": "ID for a photo on Mapillary, a service for crowdsourcing map photos",
        "data_type": "ExternalId"
    },
    "P2378": {
        "label": "issued by",
        "description": "organisation that issues or allocates an identifier",
        "data_type": "WikibaseItem"
    },
    "P836": {
        "label": "GSS code (2011)",
        "description": "nine-character UK Government Statistical Service code, introduced in 2009 to replace older ONS codes",
        "data_type": "ExternalId"
    },
    "P999": {
        "label": "ARICNS",
        "description": "identifier for stars in ARICNS",
        "data_type": "ExternalId"
    },
    "P2483": {
        "label": "NCES District ID",
        "description": "identifier for a school district or education agency in the United States",
        "data_type": "ExternalId"
    },
    "P2468": {
        "label": "Theatricalia theatre ID",
        "description": "identifier for a theatre, in the theatricalia.com database",
        "data_type": "ExternalId"
    },
    "P2015": {
        "label": "Hansard (1803–2005) ID",
        "description": "identifier of a person in the Hansard 1803-2005 database, indexing speeches in the British parliament (both Commons and Lords)",
        "data_type": "ExternalId"
    },
    "P2264": {
        "label": "mix'n'match catalog ID",
        "description": "ID of the catalog corresponding to the property in the mix'n'match tool",
        "data_type": "ExternalId"
    },
    "P2576": {
        "label": "UCSC Genome Browser assembly ID",
        "description": "identifier of a UCSC's release of a genome",
        "data_type": "ExternalId"
    },
    "P1245": {
        "label": "OmegaWiki Defined Meaning",
        "description": "\"Defined Meaning\" on the site",
        "data_type": "ExternalId"
    },
    "P2590": {
        "label": "BPS language code",
        "description": "language code in Indonesia issued by Statistics Indonesia",
        "data_type": "ExternalId"
    },
    "P1733": {
        "label": "Steam application ID",
        "description": "identifier for an application available from the Steam distribution platform",
        "data_type": "ExternalId"
    },
    "P1467": {
        "label": "WALS genus code",
        "description": "identifier for a language genus",
        "data_type": "ExternalId"
    },
    "P998": {
        "label": "Curlie ID",
        "description": "category path at Open Directory Project",
        "data_type": "ExternalId"
    },
    "P2080": {
        "label": "AcademiaNet ID",
        "description": "identifier in the AcademiaNet database for excellent female scientists",
        "data_type": "ExternalId"
    },
    "P917": {
        "label": "GRAU index",
        "description": "index of Russian (and Soviet) weapons and rockets",
        "data_type": "ExternalId"
    },
    "P894": {
        "label": "zbMATH work ID",
        "description": "identifier in the zbMath database",
        "data_type": "ExternalId"
    },
    "P1980": {
        "label": "PolSys ID",
        "description": "identifier for a person in the PolSys database of Norwegian politicians",
        "data_type": "ExternalId"
    },
    "P1229": {
        "label": "Openpolis ID",
        "description": "identifier in Openpolis, a database of Italian politicians",
        "data_type": "ExternalId"
    },
    "P1627": {
        "label": "Ethnologue.com language code",
        "description": "identifier for a language in ethnologue.com",
        "data_type": "ExternalId"
    },
    "P2192": {
        "label": "endangeredlanguages.com ID",
        "description": "identifier of a language on the website endangeredlanguages.com",
        "data_type": "ExternalId"
    },
    "P1583": {
        "label": "MalaCards ID",
        "description": "identifier in the Malacards database of diseases",
        "data_type": "ExternalId"
    },
    "P980": {
        "label": "code for weekend and holiday homes (Sweden)",
        "description": "identifier for a for weekend or holiday home in Sweden, assigned by Statistics Sweden",
        "data_type": "ExternalId"
    },
    "P1612": {
        "label": "Commons Institution page",
        "description": "name of the institutions's page on Wikimedia Commons (without the prefix \"Institution\")",
        "data_type": "String"
    },
    "P1630": {
        "label": "formatter URL",
        "description": "web page URL; URI template from which \"$1\" can be automatically replaced with the effective property value on items. If the site goes offline, set it to deprecated rank. If the formatter url changes, add a new statement with preferred rank.",
        "data_type": "String"
    },
    "P714": {
        "label": "Dana 8th edition",
        "description": "mineral classification Dana 8th edition",
        "data_type": "ExternalId"
    },
    "P704": {
        "label": "Ensembl transcript ID",
        "description": "transcript ID issued by Ensembl database",
        "data_type": "ExternalId"
    },
    "P732": {
        "label": "BGS Lexicon of Named Rock Units ID",
        "description": "identifier for a stratigraphic unit given by the British Geological Survey",
        "data_type": "ExternalId"
    },
    "P1289": {
        "label": "Kritisches Lexikon zur fremdsprachigen Gegenwartsliteratur ID",
        "description": "identifier in the critical dictionary of foreign contemporary literature",
        "data_type": "ExternalId"
    },
    "P1052": {
        "label": "Portuguese Job Code CPP-2010",
        "data_type": "ExternalId"
    },
    "P1395": {
        "label": "National Cancer Institute ID",
        "description": "identifier at www.cancer.gov",
        "data_type": "ExternalId"
    },
    "P1925": {
        "label": "VIOLIN ID",
        "description": "identifier in the VIOLIN database for vaccines and related subjects",
        "data_type": "ExternalId"
    },
    "P1232": {
        "label": "Linguist list code",
        "description": "identifier for a language per Linguist list",
        "data_type": "ExternalId"
    },
    "P2298": {
        "label": "NSDAP membership number (1925–1945)",
        "data_type": "ExternalId"
    },
    "P818": {
        "label": "arXiv ID",
        "description": "identifier of a document in arXiv pre-print archive",
        "data_type": "ExternalId"
    },
    "P1251": {
        "label": "ABS ASCL 2011 code",
        "description": "Australian Standard Classification of Languages ID for languages",
        "data_type": "ExternalId"
    },
    "P1726": {
        "label": "Florentine musea Inventario 1890 ID",
        "description": "national florentine musea inventory from 1890 identifier",
        "data_type": "ExternalId"
    },
    "P902": {
        "label": "HDS ID",
        "description": "identifier in HDS/HLS/DHS/DSS: Historical Dictionary of Switzerland (Q642074), a national encyclopedia",
        "data_type": "ExternalId"
    },
    "P1468": {
        "label": "WALS family code",
        "description": "identifier for a language family",
        "data_type": "ExternalId"
    },
    "P824": {
        "label": "Meteoritical Bulletin Database ID",
        "description": "identifier for a meteorite in the database of the Meteoritical Society",
        "data_type": "ExternalId"
    },
    "P1271": {
        "label": "Norway Database for Statistics on Higher education publisher ID",
        "description": "identifier",
        "data_type": "ExternalId"
    },
    "P701": {
        "label": "Dodis ID",
        "description": "identifier in the Dodis database (Diplomatic Documents of Switzerland 1945-1969), see Q661051",
        "data_type": "ExternalId"
    },
    "P2074": {
        "label": "internetmedicin.se ID",
        "description": "ID in the Swedish database about medical topics",
        "data_type": "ExternalId"
    },
    "P2161": {
        "label": "Guthrie code",
        "description": "Guthrie code of a Bantu language",
        "data_type": "ExternalId"
    },
    "P1665": {
        "label": "ChessGames.com player ID",
        "description": "identifier on the website Chess Games (www.chessgames.com)",
        "data_type": "ExternalId"
    },
    "P1651": {
        "label": "YouTube video ID",
        "description": "identifier of a video on YouTube; qualify trailers with \"object has role\" (P3831)=\"trailer\" (Q622550). For channels use P2397; for playlists use P4300",
        "data_type": "ExternalId"
    },
    "P1793": {
        "label": "format as a regular expression",
        "description": "regex describing an identifier or a Wikidata property. When using on property constraints, ensure syntax is a PCRE",
        "data_type": "String"
    },
    "P1466": {
        "label": "WALS lect code",
        "description": "identifier for a language",
        "data_type": "ExternalId"
    },
    "P867": {
        "label": "ROME Occupation Code (v3)",
        "description": "ROME Code for a given occupation in France (V3, 1 letter, 4 digits)",
        "data_type": "ExternalId"
    },
    "P844": {
        "label": "UBIGEO code",
        "description": "identifier for a geographical location in Peru used by the National Statistics Institute",
        "data_type": "ExternalId"
    },
    "P1287": {
        "label": "KDG Komponisten der Gegenwart ID",
        "description": "identifier on the Munzinger Archiv",
        "data_type": "ExternalId"
    },
    "P886": {
        "label": "Lexicon istoric retic ID",
        "description": "identifier in the Lexicon istoric retic (LIR), Romansh variant of Historical Dictionary of Switzerland",
        "data_type": "ExternalId"
    },
    "P1828": {
        "label": "IPI name number",
        "description": "identifier for names of a composer, author and other relevant parties",
        "data_type": "ExternalId"
    },
    "P1275": {
        "label": "Norway Import Service and Registration Authority publisher code",
        "description": "identifier",
        "data_type": "ExternalId"
    },
    "P1225": {
        "label": "U.S. National Archives Identifier",
        "description": "identifier for the United States National Archives and Records Administration's online catalog",
        "data_type": "ExternalId"
    },
    "P1751": {
        "label": "Art UK collection ID",
        "description": "identifier for art collections, in the UK",
        "data_type": "ExternalId"
    },
    "P1763": {
        "label": "National Pipe Organ Register ID",
        "description": "identifier in the (United Kingdom) National Pipe Organ Register",
        "data_type": "ExternalId"
    },
    "P1338": {
        "label": "EPSG CRS",
        "description": "identifier for coordinate reference systems per European Petroleum Survey Group Geodetic Parameter Dataset",
        "data_type": "ExternalId"
    },
    "P2493": {
        "label": "OM institution ID",
        "description": "identifier of primary and secondary schools at the Hungarian Ministry of Education",
        "data_type": "ExternalId"
    },
    "P2478": {
        "label": "Railways Archive event ID",
        "description": "Event ID in the Railways Archive database",
        "data_type": "ExternalId"
    },
    "P1236": {
        "label": "Parsons code",
        "description": "identifier for melodic contours of music",
        "data_type": "ExternalId"
    },
    "P884": {
        "label": "State Water Register Code (Russia)",
        "description": "identifier of a body of water in Russia",
        "data_type": "ExternalId"
    },
    "P911": {
        "label": "South African municipality code",
        "description": "identifier for a municipality in South Africa",
        "data_type": "ExternalId"
    },
    "P6201": {
        "label": "OBV editions ID",
        "description": "identifier for entries in the catalogue of the Austrian library network",
        "data_type": "ExternalId"
    },
    "P8732": {
        "label": "Archivio Storico Ricordi opera ID",
        "description": "identifier of an operatic work in the Archivio Storico Ricordi digital collection",
        "data_type": "ExternalId"
    },
    "P8064": {
        "label": "Chinese Clinical Trial Registry ID",
        "description": "identifier for the clinical trials registry in China",
        "data_type": "ExternalId"
    },
    "P6211": {
        "label": "BIBSYS work ID",
        "description": "identifier for works in BIBSYS",
        "data_type": "ExternalId"
    },
    "P6679": {
        "label": "Encyklopedia Teatru Polskiego play ID",
        "description": "theatrical play ID in e-teatr.pl",
        "data_type": "ExternalId"
    },
    "P5968": {
        "label": "CPPAP ID",
        "description": "ID issued by the French \"Commission paritaire des publications et des agences de presse\" for a registered periodical",
        "data_type": "ExternalId"
    },
    "P8070": {
        "label": "ToposText work ID",
        "description": "identifier for a written work in the website ToposText",
        "data_type": "ExternalId"
    },
    "P9051": {
        "label": "ICCD cultural heritage ID",
        "description": "identifier for a cultural heritage object of Italian cultural heritage in the General catalog of Italian cultural heritage",
        "data_type": "ExternalId"
    },
    "P8103": {
        "label": "Paperity article ID",
        "description": "identifier for a scholarly article in the Paperity bibliographic database",
        "data_type": "ExternalId"
    },
    "P653": {
        "label": "PubMed Health",
        "description": "identifier for a physiological condition, in the PubMed Health website",
        "data_type": "ExternalId"
    },
    "P8524": {
        "label": "Offizielle Deutsche Charts song ID",
        "description": "identifier for an song on Offizielle Deutsche Charts site",
        "data_type": "ExternalId"
    },
    "P604": {
        "label": "MedlinePlus ID",
        "description": "health information from U.S. government agencies, and health-related organizations",
        "data_type": "ExternalId"
    },
    "P8299": {
        "label": "Semantic Scholar corpus ID",
        "description": "identifier for an article in the Semantic Scholar database (for hexadecimal ID see P4011)",
        "data_type": "ExternalId"
    },
    "P6901": {
        "label": "NooSFere edition ID",
        "description": "identifier for a edition in NooSFere Database",
        "data_type": "ExternalId"
    },
    "P8619": {
        "label": "BD Gest' series ID",
        "description": "numerical identifier for a comic book series on the BD Gest' website",
        "data_type": "ExternalId"
    },
    "P247": {
        "label": "COSPAR ID",
        "description": "international satellite designation, administered by the UN Committee on Space Research (COSPAR), similar but not synonymous with the NSSDCA ID (P8913)",
        "data_type": "ExternalId"
    },
    "P6373": {
        "label": "Online Catalog of Biblioteca Nacional de Portugal ID",
        "description": "identifier for the full set of bibliographic records from the National Library of Portugal online catalogue",
        "data_type": "ExternalId"
    },
    "P6786": {
        "label": "National Library of Wales Catalogue ID",
        "description": "identifier for an item in the main catalogue of the National Library of Wales",
        "data_type": "ExternalId"
    },
    "P8088": {
        "label": "Elephind.com ID",
        "description": "identifier for a newspaper on the Elephind.com website",
        "data_type": "ExternalId"
    },
    "P6175": {
        "label": "Bitraga work ID",
        "description": "identifier for a work in the Digital Library of Galician Literary Translation database (Bitraga) of the University of Vigo",
        "data_type": "ExternalId"
    },
    "P7609": {
        "label": "SHARE Catalogue work ID",
        "description": "identifier for works in SHARE Catalogue, a linked data federated catalogue of several Southern Italy universities",
        "data_type": "ExternalId"
    },
    "P8532": {
        "label": "Trismegistos text ID",
        "description": "identifier of a papyrus or an epigraphic text on Tresmegistos",
        "data_type": "ExternalId"
    },
    "P8680": {
        "label": "FVLB work ID",
        "description": "identifier for a film or video game on FVLB's official website",
        "data_type": "ExternalId"
    },
    "P7720": {
        "label": "Fossilworks ID for journal article",
        "data_type": "ExternalId"
    },
    "P6805": {
        "label": "Art Gallery of South Australia work ID",
        "description": "identifier of a work in the collection of the Art Gallery of South Australia",
        "data_type": "ExternalId"
    },
    "P8294": {
        "label": "AusStage work ID",
        "description": "identifier for a work at AusStage",
        "data_type": "ExternalId"
    },
    "P5999": {
        "label": "Japan PlayStation Store ID",
        "description": "identifier for a game available on the PlayStation Store (JP region)",
        "data_type": "ExternalId"
    },
    "P9301": {
        "label": "National Gallery of Art Library Bibliographic ID",
        "description": "identifier assigned to a bibliographic record for the library collection at the National Gallery of Art Library",
        "data_type": "ExternalId"
    },
    "P8298": {
        "label": "HBO Max ID",
        "description": "identifier for a creative work on  HBO Max",
        "data_type": "ExternalId"
    },
    "P6491": {
        "label": "Board Game Atlas ID",
        "description": "identifier for a board game in the Board Game Atlas database",
        "data_type": "ExternalId"
    },
    "P8579": {
        "label": "Firefox add-on ID",
        "description": "unique identifier for a Firefox add-on",
        "data_type": "ExternalId"
    },
    "P8184": {
        "label": "DART-Europe thesis ID",
        "description": "identifier for a doctoral thesis on DART-Europe",
        "data_type": "ExternalId"
    },
    "P6221": {
        "label": "NooSFere story ID",
        "description": "identifier for a story in NooSFere Database",
        "data_type": "ExternalId"
    },
    "P7710": {
        "label": "ScienceOpen publication ID",
        "description": "unique identifier of an work in ScienceOpen database",
        "data_type": "ExternalId"
    },
    "P8940": {
        "label": "AppGallery app ID",
        "description": "identifier for a mobile application in AppGallery Store",
        "data_type": "ExternalId"
    },
    "P8978": {
        "label": "DBLP publication ID",
        "description": "identifier of publications in the DBLP database",
        "data_type": "ExternalId"
    },
    "P8123": {
        "label": "DLL Catalog work ID",
        "description": "identifier for a Latin work in the catalog Digital Latin Library",
        "data_type": "ExternalId"
    },
    "P424": {
        "label": "Wikimedia language code",
        "description": "identifier for a language or variant as used by Wikimedia projects",
        "data_type": "String"
    },
    "P9177": {
        "label": "playDB play ID",
        "description": "identifier for theatrical and musical productions in playDB",
        "data_type": "ExternalId"
    },
    "P9262": {
        "label": "ChemRxiv ID",
        "description": "identifier of a document in ChemRxiv, a preprint server for the chemical sciences launched in 2016",
        "data_type": "ExternalId"
    },
    "P8187": {
        "label": "Epistemonikos ID",
        "description": "identifier in the Epistemonikos database of systematic reviews",
        "data_type": "ExternalId"
    },
    "P8071": {
        "label": "BDSP ID",
        "description": "identifier for a scholarly publication in the BDSP bibliographic database",
        "data_type": "ExternalId"
    },
    "P7980": {
        "label": "Clavis Patrum Latinorum ID",
        "description": "identifier for Latin patristic works",
        "data_type": "ExternalId"
    },
    "P6877": {
        "label": "MAHG ID",
        "description": "identifier for items in the catalogue of the Musées d'Art et d'Histoire de Genève (MAHG)",
        "data_type": "ExternalId"
    },
    "P356": {
        "label": "DOI",
        "description": "serial code used to uniquely identify digital objects like academic papers (use upper case letters only)",
        "data_type": "ExternalId"
    },
    "P5932": {
        "label": "IGN TV series ID",
        "description": "identifier for television series on the IGN website",
        "data_type": "ExternalId"
    },
    "P8518": {
        "label": "Henrik Ibsen writings ID",
        "description": "identifier for an item in Henrik Ibsen writings",
        "data_type": "ExternalId"
    },
    "P7924": {
        "label": "Bibliotheca Hagiographica Latina ID",
        "description": "identifier for Latin hagiographical works",
        "data_type": "ExternalId"
    },
    "P8059": {
        "label": "coinop.org game ID",
        "description": "identifier of a video game in the coinop.org database",
        "data_type": "ExternalId"
    },
    "P8442": {
        "label": "Digital Library of the Caribbean ID",
        "description": "identifier of a document in the Digital Library of the Caribbean",
        "data_type": "ExternalId"
    },
    "P7381": {
        "label": "PubPeer article ID",
        "description": "Identifier of an scientific article at PubPeer",
        "data_type": "ExternalId"
    },
    "P6614": {
        "label": "Libreflix ID",
        "description": "identifier for a creative work on Libreflix",
        "data_type": "ExternalId"
    },
    "P8201": {
        "label": "ACUM work ID",
        "description": "identifier for a musical work in Society of Authors, Composers and Music Publishers in Israel (ACUM)",
        "data_type": "ExternalId"
    },
    "P6693": {
        "label": "NGMDb Prod ID",
        "description": "ID for every publication in the United States Geological Survey's National Geologic Map Database (NGMDb)",
        "data_type": "ExternalId"
    },
    "P600": {
        "label": "Wine AppDB ID",
        "description": "identifier for an application in the AppDB of WineHQ",
        "data_type": "ExternalId"
    },
    "P6366": {
        "label": "Microsoft Academic ID",
        "description": "identifier for an object or topic in the Microsoft Academic Graph",
        "data_type": "ExternalId"
    },
    "P7998": {
        "label": "Podchaser podcast ID",
        "description": "identifier of a podcast in the Podchaser database",
        "data_type": "ExternalId"
    },
    "P6769": {
        "label": "CJFD journal article ID",
        "description": "identifier for journal articles in China National Knowledge Infrastructure (CNKI)'s China Journal Full-text Database (CJFD)",
        "data_type": "ExternalId"
    },
    "P458": {
        "label": "IMO ship number",
        "description": "identifier for a ship, ship owner or ship manager per the  International Maritime Organization",
        "data_type": "ExternalId"
    },
    "P7768": {
        "label": "Australian National Maritime Museum object ID",
        "description": "identifier for an object in the Australian National Maritime Museum collection",
        "data_type": "ExternalId"
    },
    "P8607": {
        "label": "The Cutting Room Floor ID",
        "description": "identifier in The Cutting Room Floor wiki",
        "data_type": "ExternalId"
    },
    "P9016": {
        "label": "Medieval Manuscripts in Oxford Libraries work ID",
        "description": "identifier of a work in the online database Medieval Manuscripts in Oxford Libraries",
        "data_type": "ExternalId"
    },
    "P8074": {
        "label": "Geschichtsquellen des deutschen Mittelalters work ID",
        "description": "identifier for medieval German literary works",
        "data_type": "ExternalId"
    },
    "P8311": {
        "label": "PHI Latin Texts work ID",
        "description": "identifier for a work in the PHI Latin Texts digital library",
        "data_type": "ExternalId"
    },
    "P8310": {
        "label": "digilibLT work ID",
        "description": "identifier for a work in the digilibLT digital library",
        "data_type": "ExternalId"
    },
    "P8312": {
        "label": "Musisque Deoque work ID",
        "description": "identifier for a work in the Musisque Deoque digital library",
        "data_type": "ExternalId"
    },
    "P8925": {
        "label": "CQVIP article ID",
        "description": "identifier of an article in CQVIP",
        "data_type": "ExternalId"
    },
    "P6663": {
        "label": "Utpictura18 artwork ID",
        "description": "identifier for an artwork on the Utpictura18 website",
        "data_type": "ExternalId"
    },
    "P8559": {
        "label": "Chrome Webstore extension ID",
        "description": "unique identifier for a Google Chrome extension",
        "data_type": "ExternalId"
    },
    "P8211": {
        "label": "Bookogs work ID",
        "description": "identifier for a written work in the Bookogs database",
        "data_type": "ExternalId"
    },
    "P8752": {
        "label": "Kramerius of Moravian Library UUID",
        "description": "UUID identifier for scanned item (book edition/periodical/page) in Moravian Library",
        "data_type": "ExternalId"
    },
    "P5934": {
        "label": "Short Title Catalogue Flanders (STCV)",
        "description": "identifier for a hand press book in the 'Short Title Catalogus Vlaanderen' (STCV) database of Flanders before 1801",
        "data_type": "ExternalId"
    },
    "P6323": {
        "label": "Centre Pompidou ID",
        "description": "identifier for an entry on the 'Centre national d'art et de culture Georges-Pompidou' website",
        "data_type": "ExternalId"
    },
    "P6668": {
        "label": "theatre-contemporain.net ID",
        "description": "unique identifier of a name or work in the French database theatre-contemporain.net",
        "data_type": "ExternalId"
    },
    "P5935": {
        "label": "Flanders Arts Institute production ID",
        "description": "identifier of an production in the Flanders Arts Institute database for performing arts",
        "data_type": "ExternalId"
    },
    "P6736": {
        "label": "Drobné památky ID",
        "description": "identifier in the database of small-scale public artwork in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P7034": {
        "label": "Syriaca work ID",
        "description": "identifier of a work on Syriaca.org",
        "data_type": "ExternalId"
    },
    "P7985": {
        "label": "Kanopy ID",
        "description": "identifier for a creative work on Kanopy",
        "data_type": "ExternalId"
    },
    "P6676": {
        "label": "OpenEdition text ID",
        "description": "identifier for a book or a chapter on OpenEdition",
        "data_type": "ExternalId"
    },
    "P638": {
        "label": "PDB structure ID",
        "description": "identifier for 3D structural data as per the PDB (Protein Data Bank) database",
        "data_type": "ExternalId"
    },
    "P7788": {
        "label": "openSUSE package",
        "description": "name of the official openSUSE package",
        "data_type": "ExternalId"
    },
    "P8694": {
        "label": "Science Museum Group ID",
        "description": "external identifier for the Science Museum Group",
        "data_type": "ExternalId"
    },
    "P7110": {
        "label": "Textile Connect publication ID",
        "description": "identifier for a scholarly, scientific, or trade publication in the Textile Connect Database",
        "data_type": "ExternalId"
    },
    "P7987": {
        "label": "Bibliotheca Hagiographica Graeca ID",
        "description": "identifier for Greek hagiographical works",
        "data_type": "ExternalId"
    },
    "P7988": {
        "label": "Clavis Patrum Graecorum ID",
        "description": "identifier for Greek patristic works",
        "data_type": "ExternalId"
    },
    "P8150": {
        "label": "COVIDWHO ID",
        "description": "identifier of an article in the WHO COVID-19 Global literature on coronavirus disease database",
        "data_type": "ExternalId"
    },
    "P7991": {
        "label": "Mirabile title ID",
        "description": "identifier for medieval written works",
        "data_type": "ExternalId"
    },
    "P6709": {
        "label": "Løøv classification",
        "description": "classification system for literature on Sami matters",
        "data_type": "String"
    },
    "P9116": {
        "label": "Musica Brasilis score ID",
        "description": "unique identifier for a score in the Musica Brasilis website",
        "data_type": "ExternalId"
    },
    "P6480": {
        "label": "IEEE Xplore document ID",
        "description": "identifier for a document in IEEE Xplore",
        "data_type": "ExternalId"
    },
    "P8589": {
        "label": "Carl-Maria-von-Weber-Gesamtausgabe ID",
        "description": "identifier in the online database Carl-Maria-von-Weber-Gesamtausgabe (WeGA)",
        "data_type": "ExternalId"
    },
    "P591": {
        "label": "EC enzyme number",
        "description": "classification scheme for enzymes",
        "data_type": "String"
    },
    "P628": {
        "label": "E number",
        "description": "number for food additives that are legal in the European Union",
        "data_type": "ExternalId"
    },
    "P7932": {
        "label": "Irish playography play ID",
        "description": "identifier for a play, in the 'Irish Playography' database of theatre plays",
        "data_type": "ExternalId"
    },
    "P8726": {
        "label": "Irish Statute Book ID",
        "description": "identifier of legislation on the irishstatutebook.ie website",
        "data_type": "ExternalId"
    },
    "P7591": {
        "label": "VideoGameGeek game ID",
        "description": "identifier of a video game in the VideoGameGeek database",
        "data_type": "ExternalId"
    },
    "P278": {
        "label": "GOST 7.75–97 code",
        "description": "identifier for a language according to GOST 7.75–97",
        "data_type": "ExternalId"
    },
    "P6661": {
        "label": "Rxivist preprint ID",
        "description": "identifier for a scientific or academic preprint of a paper, in the Rxivist database",
        "data_type": "ExternalId"
    },
    "P1610": {
        "label": "Dialnet article ID",
        "description": "identifier of an article in Dialnet",
        "data_type": "ExternalId"
    },
    "P2535": {
        "label": "Sandbox-Mathematical expression",
        "description": "Sandbox property for value of type \"Mathematical expression\"",
        "data_type": "Math"
    },
    "P460": {
        "label": "said to be the same as",
        "description": "this item is said to be the same as that item, but it's uncertain or disputed",
        "data_type": "WikibaseItem"
    },
    "P2451": {
        "label": "MAME ROM name",
        "description": "name of Multiple Arcade Machine Emulator ROM for a software release",
        "data_type": "ExternalId"
    },
    "P1712": {
        "label": "Metacritic ID",
        "description": "identifier for Metacritic",
        "data_type": "ExternalId"
    },
    "P1806": {
        "label": "ABoK number",
        "description": "identifier for a knot in the Ashley Book of Knots",
        "data_type": "ExternalId"
    },
    "P361": {
        "label": "part of",
        "description": "object of which the subject is a part (if this subject is already part of object A which is a part of object B, then please only make the subject part of object A). Inverse property of \"has part\" (P527, see also \"has parts of the class\" (P2670)).",
        "data_type": "WikibaseItem"
    },
    "P464": {
        "label": "NOR",
        "description": "identifier for French official texts",
        "data_type": "ExternalId"
    },
    "P1106": {
        "label": "Sandbox-Quantity",
        "description": "property to be used for testing the quantity datatype",
        "data_type": "Quantity"
    },
    "P2185": {
        "label": "DLI ID",
        "description": "identifier for a work in the Digital Library of India",
        "data_type": "ExternalId"
    },
    "P2368": {
        "label": "Sandbox-Property",
        "description": "Sandbox property for value of property type \"Property\"",
        "data_type": "WikibaseProperty"
    },
    "P1316": {
        "label": "Swedish Media Database ID",
        "description": "identifier in the Swedish Media Database",
        "data_type": "ExternalId"
    },
    "P2339": {
        "label": "BoardGameGeek ID",
        "description": "identifier for a board game in the BoardGameGeek database",
        "data_type": "ExternalId"
    },
    "P1003": {
        "label": "National Library of Romania ID",
        "description": "identifier for authority control used at the National Library of Romania",
        "data_type": "ExternalId"
    },
    "P2474": {
        "label": "CDLI ID",
        "description": "unique identifier of an object in the Cuneiform Digital Library Initiative",
        "data_type": "ExternalId"
    },
    "P370": {
        "label": "Sandbox-String",
        "description": "Sandbox property for value of type \"String\"",
        "data_type": "String"
    },
    "P3373": {
        "label": "sibling",
        "description": "the subject has the object as their sibling (brother, sister, etc.). Use \"relative\" (P1038) for siblings-in-law (brother-in-law, sister-in-law, etc.) and step-siblings (step-brothers, step-sisters, etc.)",
        "data_type": "WikibaseItem"
    },
    "P1015": {
        "label": "BIBSYS ID",
        "description": "identifier in the Norwegian Authority File, formerly Bibsys Authority File",
        "data_type": "ExternalId"
    },
    "P1647": {
        "label": "subproperty of",
        "description": "all resources related by this property are also related by that property",
        "data_type": "WikibaseProperty"
    },
    "P2670": {
        "label": "has parts of the class",
        "description": "the subject instance has parts of the object class (the subject is usually not a class)",
        "data_type": "WikibaseItem"
    },
    "P819": {
        "label": "ADS bibcode",
        "description": "bibcode in the Astrophysics Data System",
        "data_type": "ExternalId"
    },
    "P2187": {
        "label": "BiblioNet publication ID",
        "description": "identifier in the BiblioNet database of book editions",
        "data_type": "ExternalId"
    },
    "P1573": {
        "label": "BBC Genome ID",
        "description": "identifier for a single episode/programme in the BBC Genome database of Radio Times programme listings",
        "data_type": "ExternalId"
    },
    "P5979": {
        "label": "Sandbox-Sense",
        "description": "sandbox property for value of type \"sense\"",
        "data_type": "WikibaseSense"
    },
    "P5973": {
        "label": "synonym",
        "description": "sense of another lexeme with the same meaning as this sense, in the same language",
        "data_type": "WikibaseSense"
    },
    "P569": {
        "label": "date of birth",
        "description": "date on which the subject was born",
        "data_type": "Time"
    },
    "P1234": {
        "label": "ISFDB publication ID",
        "description": "identifier for an edition of a publication in the Internet Speculative Fiction Database",
        "data_type": "ExternalId"
    },
    "P1219": {
        "label": "Internet Broadway Database show ID",
        "description": "identifier for a play or other work (which may have had 1 or more productions)",
        "data_type": "ExternalId"
    },
    "P2413": {
        "label": "FMD magazine ID",
        "description": "external link to a magazine profile at the FMD (Fashion Model Directory) website",
        "data_type": "ExternalId"
    },
    "P1246": {
        "label": "patent number",
        "description": "identifier for a patented invention",
        "data_type": "ExternalId"
    },
    "P1893": {
        "label": "OpenPlaques plaque ID",
        "description": "identifier for a commemorative plaque, on a building or other structure in OpenPlaques",
        "data_type": "ExternalId"
    },
    "P888": {
        "label": "JSTOR article ID",
        "description": "identifier for an article or a book in JSTOR",
        "data_type": "ExternalId"
    },
    "P626": {
        "label": "Sandbox-GeoCoordinateValue",
        "description": "Sandbox property for value of type \"GeoCoordinateValue\"",
        "data_type": "GlobeCoordinate"
    },
    "P2000": {
        "label": "CPDL ID",
        "description": "identifier of a work or person, in the Choral Public Domain Library",
        "data_type": "ExternalId"
    },
    "P2461": {
        "label": "ComLaw ID",
        "description": "identifier for Commonwealth of Australia legislation, bills, regulations, etc, in the ComLaw database",
        "data_type": "ExternalId"
    },
    "P5972": {
        "label": "translation",
        "description": "word (specifically a Sense) in another language that corresponds exactly to this meaning of the lexeme",
        "data_type": "WikibaseSense"
    },
    "P5188": {
        "label": "Sandbox-Lexeme",
        "description": "sandbox property, use this to experiment linking to Lexeme entities",
        "data_type": "WikibaseLexeme"
    },
    "P5607": {
        "label": "located in the ecclesiastical territorial entity",
        "description": "the item is located on the territory of the following ecclesiastical entity. Use P708 (diocese) for dioceses",
        "data_type": "WikibaseItem"
    },
    "P827": {
        "label": "BBC programme ID",
        "description": "identifier for the corresponding item on the BBC website and internal systems",
        "data_type": "ExternalId"
    },
    "P1933": {
        "label": "MobyGames game ID",
        "description": "identifier for MobyGames",
        "data_type": "ExternalId"
    },
    "P2473": {
        "label": "IGPCV ID",
        "description": "cultural heritage identifier in the Inventario General del Patrimonio Cultural Valenciano",
        "data_type": "ExternalId"
    },
    "P2411": {
        "label": "Artsy gene",
        "description": "generalization of artwork type, technique, material, genre, movement, etc. from artsy.net",
        "data_type": "String"
    },
    "P2409": {
        "label": "CiNii article ID",
        "description": "identifier for an article in CiNii",
        "data_type": "ExternalId"
    },
    "P1450": {
        "label": "Sandbox-Monolingual text",
        "description": "sandbox for testing the monolingual text datatype",
        "data_type": "Monolingualtext"
    },
    "P1844": {
        "label": "HathiTrust ID",
        "description": "identifier from the HathiTrust Digital Library",
        "data_type": "ExternalId"
    },
    "P1085": {
        "label": "LibraryThing work ID",
        "description": "LibraryThing (LT) control number",
        "data_type": "ExternalId"
    },
    "P1144": {
        "label": "Library of Congress Control Number (LCCN) (bibliographic)",
        "description": "record number for entries in the LoC bibliographic catalog (for  authority records use P244)",
        "data_type": "ExternalId"
    },
    "P1560": {
        "label": "name version for other gender",
        "description": "equivalent name (with respect to the meaning of the name) in the same language: female version of a male first name, male version of a female first name. Add primarily the closest matching one",
        "data_type": "WikibaseItem"
    },
    "P3403": {
        "label": "coextensive with",
        "description": "this item has the same boundary as the target item; area associated with (this) entity is identical with the area associated with that entity",
        "data_type": "WikibaseItem"
    },
    "P2959": {
        "label": "permanent duplicated item",
        "description": "this item duplicates another item and the two can't be merged, as one Wikimedia project includes two pages, e. g. in different scripts or languages (applies to some wiki, e.g.: cdowiki, gomwiki). Use \"duplicate item\" for other wikis.",
        "data_type": "WikibaseItem"
    },
    "P2743": {
        "label": "this zoological name is coordinate with",
        "description": "links coordinate zoological names",
        "data_type": "WikibaseItem"
    },
    "P1154": {
        "label": "Scopus EID",
        "description": "unique academic work identifier assigned in Scopus bibliographic database",
        "data_type": "ExternalId"
    },
    "P1209": {
        "label": "CN",
        "description": "identifier for a newspaper or magazine issued by State Administration of Press, Publication, Radio, Film and Television of the People’s Republic of China",
        "data_type": "ExternalId"
    },
    "P889": {
        "label": "Mathematical Reviews ID",
        "description": "identifier in Mathematical Reviews, a journal and online database",
        "data_type": "ExternalId"
    },
    "P932": {
        "label": "PMCID",
        "description": "identifier for a scientific work issued by PubMed Central (without \"PMC\" prefix)",
        "data_type": "ExternalId"
    },
    "P855": {
        "label": "Sandbox-URL",
        "description": "Sandbox property for value of type \"URL\"",
        "data_type": "Url"
    },
    "P1218": {
        "label": "Internet Broadway Database production ID",
        "description": "identifier for a specific run of a show",
        "data_type": "ExternalId"
    },
    "P6604": {
        "label": "Sandbox-Musical Notation",
        "description": "Sandbox property for value of type \"Musical Notation\"",
        "data_type": "MusicalNotation"
    },
    "P1948": {
        "label": "BerlPap identifier",
        "description": "link to the Berliner Papyrusdatenbank",
        "data_type": "ExternalId"
    },
    "P1985": {
        "label": "Anime News Network anime ID",
        "description": "identifier of an anime in Anime News Network",
        "data_type": "ExternalId"
    },
    "P1973": {
        "label": "RSL editions",
        "description": "catalogue number of the Russian State Library of Moscow (see also P1815 and P947)",
        "data_type": "ExternalId"
    },
    "P893": {
        "label": "SSRN article ID",
        "description": "identifier for an article/paper at the SSRN",
        "data_type": "ExternalId"
    },
    "P227": {
        "label": "GND ID",
        "description": "identifier from an international authority file of names, subjects, and organizations (please don't use type n = name, disambiguation) - Deutsche Nationalbibliothek",
        "data_type": "ExternalId"
    },
    "P1162": {
        "label": "Bluebook abbreviation",
        "description": "Bluebook citation style includes abbreviations",
        "data_type": "ExternalId"
    },
    "P1544": {
        "label": "Federal Register Document Number",
        "description": "Authority file ID in the United States Federal Register",
        "data_type": "ExternalId"
    },
    "P1382": {
        "label": "partially coincident with",
        "description": "object that is partially part of, but not fully part of (P361), the subject",
        "data_type": "WikibaseItem"
    },
    "P1235": {
        "label": "ISFDB series ID",
        "description": "identifier for a series, periodical or franchise in the Internet Speculative Fiction Database",
        "data_type": "ExternalId"
    },
    "P949": {
        "label": "National Library of Israel ID",
        "description": "identifier for authority control used at the National Library of Israel",
        "data_type": "ExternalId"
    },
    "P5189": {
        "label": "Sandbox-Form",
        "description": "sandbox property, use this to experiment linking to a Form of a Lexeme entity",
        "data_type": "WikibaseForm"
    },
    "P2536": {
        "label": "Sandbox-External identifier",
        "description": "Sandbox property for value of type \"External identifier\"",
        "data_type": "ExternalId"
    },
    "P1184": {
        "label": "Handle ID",
        "description": "Handle System identifier, which is a superset of the DOI",
        "data_type": "ExternalId"
    },
    "P368": {
        "label": "Sandbox-CommonsMediaFile",
        "description": "Sandbox property for value of type \"Commons Media File\"",
        "data_type": "CommonsMedia"
    },
    "P236": {
        "label": "ISSN",
        "description": "International Standard Serial Number (print or electronic)",
        "data_type": "ExternalId"
    },
    "P5135": {
        "label": "greater than",
        "description": "instances of the item have a greater value than corresponding instances of the object, for the given measure",
        "data_type": "WikibaseItem"
    },
    "P131": {
        "label": "located in the administrative territorial entity",
        "description": "the item is located on the territory of the following administrative entity. Use P276 (location) for specifying locations that are non-administrative places and for items about events",
        "data_type": "WikibaseItem"
    },
    "P1669": {
        "label": "Cultural Objects Names Authority ID",
        "description": "identifier from Cultural Objects Name Authority",
        "data_type": "ExternalId"
    },
    "P1874": {
        "label": "Netflix ID",
        "description": "identifier for a creative work on Netflix",
        "data_type": "ExternalId"
    },
    "P2047": {
        "label": "duration",
        "description": "length of time of an event or process",
        "data_type": "Quantity"
    },
    "P2583": {
        "label": "distance from Earth",
        "description": "estimated distance to astronomical objects",
        "data_type": "Quantity"
    },
    "P1242": {
        "label": "Theatricalia play ID",
        "description": "identifier for a theatrical work, in the Theatricalia database",
        "data_type": "ExternalId"
    },
    "P2209": {
        "label": "SourceForge project",
        "description": "identifier for an official SourceForge repository (\"project\") for a software product",
        "data_type": "ExternalId"
    },
    "P369": {
        "label": "Sandbox-Item",
        "description": "sandbox property for value of type \"Item\"",
        "data_type": "WikibaseItem"
    },
    "P3893": {
        "label": "public domain date",
        "description": "date the item enters into the public domain in a jurisdiction",
        "data_type": "Time"
    },
    "P892": {
        "label": "RfC ID",
        "description": "identifier for an item in Request for Comments, a publication of IETF and the Internet Society (without \"RFC\" prefix)",
        "data_type": "ExternalId"
    },
    "P698": {
        "label": "PubMed ID",
        "description": "identifier for journal articles/abstracts in PubMed",
        "data_type": "ExternalId"
    },
    "P1274": {
        "label": "ISFDB title ID",
        "description": "identifier for a title in the Internet Speculative Fiction Database",
        "data_type": "ExternalId"
    },
    "P4047": {
        "label": "Sandbox-Geographic shape",
        "description": "A sandbox type per available data type",
        "data_type": "GeoShape"
    },
    "P5136": {
        "label": "less than",
        "description": "instances of the item have a lesser value than corresponding instances of the object, for the given measure",
        "data_type": "WikibaseItem"
    },
    "P1972": {
        "label": "Open Hub ID",
        "description": "identifier for free software at OpenHub.net",
        "data_type": "ExternalId"
    },
    "P745": {
        "label": "Low German Bibliography and Biography ID",
        "description": "identifier",
        "data_type": "ExternalId"
    },
    "P1852": {
        "label": "Perry Index",
        "description": "item's value on the Perry Index primarily for fables attributed to Aesop",
        "data_type": "ExternalId"
    },
    "P578": {
        "label": "Sandbox-TimeValue",
        "description": "Sandbox property for value of type \"TimeValue\"",
        "data_type": "Time"
    },
    "P243": {
        "label": "OCLC control number",
        "description": "identifier for a unique bibliographic record in OCLC WorldCat",
        "data_type": "ExternalId"
    },
    "P4045": {
        "label": "Sandbox-Tabular data",
        "description": "A sandbox type per available data type",
        "data_type": "TabularData"
    },
    "P648": {
        "label": "Open Library ID",
        "description": "identifier for a work (\"W\"), edition (\"M\") or author (\"A\") for book data of the Internet Archive",
        "data_type": "ExternalId"
    },
    "P1889": {
        "label": "different from",
        "description": "item that is different from another item, with which it is often confused",
        "data_type": "WikibaseItem"
    },
    "P724": {
        "label": "Internet Archive ID",
        "description": "identifier for an item on Internet Archive",
        "data_type": "ExternalId"
    },
    "P4203": {
        "label": "ROARMAP ID",
        "description": "identifier in the Registry of Open Access Repository Mandates and Policies",
        "data_type": "ExternalId"
    },
    "P3641": {
        "label": "Uniform Type Identifier",
        "description": "identifier used on software provided by Apple Inc. to uniquely identify a given class or type of item",
        "data_type": "ExternalId"
    },
    "P3624": {
        "label": "CWE ID",
        "description": "identifier of a software weakness type identified in the Common Weakness Enumeration (CWE) list",
        "data_type": "ExternalId"
    },
    "P3860": {
        "label": "Wormbase Gene ID",
        "description": "Identifier for a genetic element in Wormbase",
        "data_type": "ExternalId"
    },
    "P3587": {
        "label": "Common Vulnerabilities and Exposures ID",
        "description": "identifier of an information security vulnerability",
        "data_type": "ExternalId"
    },
    "P2892": {
        "label": "UMLS CUI",
        "description": "NLM Unified Medical Language System (UMLS) controlled biomedical vocabulary unique identifier",
        "data_type": "ExternalId"
    },
    "P4081": {
        "label": "BHL creator ID",
        "description": "identifier for an author (\"creator\") in the Biodiversity Heritage Library database",
        "data_type": "ExternalId"
    },
    "P3904": {
        "label": "VIVC grape variety ID",
        "description": "identifier in Vitis International Variety Catalogue",
        "data_type": "ExternalId"
    },
    "P3801": {
        "label": "INEGI municipality ID",
        "description": "identifier for a municipality in Mexico published by INEGI",
        "data_type": "ExternalId"
    },
    "P3003": {
        "label": "zanikleobce.cz abandoned objects ID",
        "description": "identifier of abandoned objects from zanikleobce.cz",
        "data_type": "ExternalId"
    },
    "P4207": {
        "label": "Italian National Earthquake Center ID",
        "description": "identifier for an earthquake in the Italian National Earthquake Centre database",
        "data_type": "ExternalId"
    },
    "P3006": {
        "label": "Marine Regions Geographic ID",
        "description": "identifier for marine geographic objects, in the marineregions.org database",
        "data_type": "ExternalId"
    },
    "P3611": {
        "label": "Borden Code",
        "description": "unique identifier of a archeological site in Canada",
        "data_type": "ExternalId"
    },
    "P3333": {
        "label": "ACM Digital Library event ID",
        "description": "unique identifier of an event or conference series of the Association for Computing Machinery (ACM)",
        "data_type": "ExternalId"
    },
    "P3378": {
        "label": "Merck Index reaction ID",
        "description": "identifier for a chemical reaction, in the Merck Index",
        "data_type": "ExternalId"
    },
    "P4215": {
        "label": "nLab ID",
        "description": "name of a page in nLab wiki",
        "data_type": "ExternalId"
    },
    "P3634": {
        "label": "The Met object ID",
        "description": "object identifier for artworks at the Metropolitan Museum of Art",
        "data_type": "ExternalId"
    },
    "P3471": {
        "label": "WikiSkripta ID",
        "description": "numeric identifier of a page on wikiskripta.eu",
        "data_type": "ExternalId"
    },
    "P4233": {
        "label": "PatientsLikeMe condition ID",
        "description": "identifier for a condition on PatientsLikeMe, a website where patients can share health information",
        "data_type": "ExternalId"
    },
    "P4235": {
        "label": "PatientsLikeMe treatment ID",
        "description": "identifier for a treatment on PatientsLikeMe, a website where patients can share health information",
        "data_type": "ExternalId"
    },
    "P4236": {
        "label": "PatientsLikeMe symptom ID",
        "description": "identifier for a symptom on PatientsLikeMe, a website where patients can share health information",
        "data_type": "ExternalId"
    },
    "P3329": {
        "label": "CIViC variant ID",
        "description": "Identifier used in the CIViC database to identify specific variant",
        "data_type": "ExternalId"
    },
    "P2965": {
        "label": "EU River Basin District code",
        "description": "unique code for an EU River Basin District",
        "data_type": "ExternalId"
    },
    "P2968": {
        "label": "QUDT unit ID",
        "description": "identifier for unit of measure definition according to QUDT ontology",
        "data_type": "ExternalId"
    },
    "P3937": {
        "label": "Reactome ID",
        "description": "Identifier used in Reactome to uniquely identify a biological pathway",
        "data_type": "ExternalId"
    },
    "P3345": {
        "label": "RxNorm ID",
        "description": "identifier for the normalized clinical drug dictionary of the Unified Medical Language System",
        "data_type": "ExternalId"
    },
    "P4089": {
        "label": "Global Terrorism Database ID",
        "description": "identifier in the Global Terrorism Database by the National Consortium for the Study of Terrorism and Responses to Terrorism (START)",
        "data_type": "ExternalId"
    },
    "P2865": {
        "label": "band number",
        "description": "identifier of First Nation in Canada",
        "data_type": "ExternalId"
    },
    "P4254": {
        "label": "Bengali Banglapedia ID",
        "description": "identifier for a Bengali article on Banglapedia, the national online encyclopedia of Bangladesh",
        "data_type": "ExternalId"
    },
    "P4112": {
        "label": "danskfilmogtv person ID",
        "description": "identifier for a person in danskfilmogtv.dk",
        "data_type": "ExternalId"
    },
    "P3968": {
        "label": "CETS number",
        "description": "Council of Europe treaty number",
        "data_type": "ExternalId"
    },
    "P3454": {
        "label": "Arch Linux package",
        "description": "name of the official Arch Linux package",
        "data_type": "ExternalId"
    },
    "P3462": {
        "label": "FAMA work ID",
        "description": "identifier for written work in medieval Latin",
        "data_type": "ExternalId"
    },
    "P3476": {
        "label": "PSA World Tour player ID",
        "description": "PSA World Tour squash player identifier",
        "data_type": "ExternalId"
    },
    "P3472": {
        "label": "VICNAMES Place ID",
        "description": "identifier for a place in Victoria, Australia, in the VICNAMES database",
        "data_type": "ExternalId"
    },
    "P4221": {
        "label": "National Criminal Justice ID",
        "description": "identifier for a publication, report, article or audiovisual product, in the United States' National Criminal Justice Reference Service database",
        "data_type": "ExternalId"
    },
    "P3477": {
        "label": "Nihon Tarento Meikan ID",
        "description": "identifier of person in Nihon Tarento Meikan",
        "data_type": "ExternalId"
    },
    "P3219": {
        "label": "Encyclopædia Universalis ID",
        "description": "identifer for an article in the online version of Encyclopædia Universalis",
        "data_type": "ExternalId"
    },
    "P3463": {
        "label": "Fedora package",
        "description": "name of the official Fedora package",
        "data_type": "ExternalId"
    },
    "P3399": {
        "label": "JTWC tropical cyclone ID",
        "description": "identifier for a tropical cyclone, issued by the Joint Typhoon Warning Center",
        "data_type": "ExternalId"
    },
    "P4211": {
        "label": "Bashkir encyclopedia (Russian version) ID",
        "description": "Bashkir Encyclopedia",
        "data_type": "ExternalId"
    },
    "P4251": {
        "label": "TSE number",
        "description": "number assigned by the Brazilian Superior Electoral Court to registered political parties",
        "data_type": "ExternalId"
    },
    "P3196": {
        "label": "USGS earthquake ID",
        "description": "identifier for an earthquake or other seismic event, in the United States Geological Survey database",
        "data_type": "ExternalId"
    },
    "P3382": {
        "label": "GeneDB ID",
        "description": "gene identifier in GeneDB",
        "data_type": "ExternalId"
    },
    "P3870": {
        "label": "ZFIN Gene ID",
        "description": "identifier for a genetic element in the Zebrafish Model Organism Database",
        "data_type": "ExternalId"
    },
    "P4161": {
        "label": "Michelin Voyages ID",
        "description": "identifier for a place in the website Michelin Voyages",
        "data_type": "ExternalId"
    },
    "P3538": {
        "label": "Fussballdaten.de player ID",
        "description": "identifier at Fussballdaten.de, a German language website which predominantly collects comprehensive statistics on the top five tiers of German football",
        "data_type": "ExternalId"
    },
    "P3406": {
        "label": "Saccharomyces Genome Database ID",
        "description": "identifier for a genetic element in the Saccharomyces Genome Database",
        "data_type": "ExternalId"
    },
    "P2988": {
        "label": "GOST 7.67 cyrillic",
        "description": "country codes in Cyrillic from the GOST standards committee",
        "data_type": "ExternalId"
    },
    "P3254": {
        "label": "property proposal discussion",
        "description": "URL of the page (or page section) on which the creation of the property was discussed",
        "data_type": "Url"
    },
    "P4210": {
        "label": "Bashkir encyclopedia (Bashkir version) ID",
        "description": "Encyclopedia",
        "data_type": "ExternalId"
    },
    "P3473": {
        "label": "Ubuntu package",
        "description": "name of the official Ubuntu package",
        "data_type": "ExternalId"
    },
    "P3469": {
        "label": "WTA tennis tournament ID",
        "data_type": "ExternalId"
    },
    "P3614": {
        "label": "DigDag ID",
        "description": "identifier of a place, in the Digital Atlas of Denmark's historical-administrative geography",
        "data_type": "ExternalId"
    },
    "P2871": {
        "label": "miRBase mature miRNA ID",
        "description": "identifier for mature miRNAs in miRBase, the reference database for microRNAs",
        "data_type": "ExternalId"
    },
    "P2870": {
        "label": "miRBase pre-miRNA ID",
        "description": "identifier for pre-miRNAs in miRBase, the reference database for microRNAs",
        "data_type": "ExternalId"
    },
    "P2878": {
        "label": "Minitel code",
        "description": "access code of a service on the Minitel Videotex",
        "data_type": "ExternalId"
    },
    "P2938": {
        "label": "Pleiades category identifier",
        "description": "category (former place type) identifier in the Pleiades gazetteer",
        "data_type": "ExternalId"
    },
    "P3291": {
        "label": "DocCheck Flexikon En ID",
        "description": "identifier for an article in the English section of DocCheck Flexikon wiki",
        "data_type": "ExternalId"
    },
    "P3292": {
        "label": "DocCheck Flexikon De ID",
        "description": "identifier for an article in the German section of the DocCheck wiki",
        "data_type": "ExternalId"
    },
    "P4168": {
        "label": "IEDB Epitope ID",
        "description": "identifier for an Epitope in the Immune Epitope Database",
        "data_type": "ExternalId"
    },
    "P2796": {
        "label": "3DMet ID",
        "description": "identifier of chemical compounds in 3DMet database",
        "data_type": "ExternalId"
    },
    "P4113": {
        "label": "FRED time-series ID",
        "description": "identifier for an economic data set provided by St. Louis Fed's Federal Reserve Economic Data (FRED)",
        "data_type": "ExternalId"
    },
    "P4115": {
        "label": "INSPIRE ID",
        "description": "universal identifier from Infrastructure for Spatial Information in the European Community (INSPIRE), used across EU databases",
        "data_type": "ExternalId"
    },
    "P4109": {
        "label": "URN-NBN",
        "description": "URI in the urn:nbn: namespace as specified by RFC 3188",
        "data_type": "ExternalId"
    },
    "P4121": {
        "label": "openAIP ID",
        "description": "identifier of an aerodrome, in the openAIP (open aeronautical information platform) database",
        "data_type": "ExternalId"
    },
    "P4111": {
        "label": "danskefilm TV Christmas calendar",
        "description": "identifier for a TV Christmas calendar in danskefilm.dk",
        "data_type": "ExternalId"
    },
    "P3303": {
        "label": "third-party formatter URL",
        "description": "URI template from which \"$1\" can be automatically replaced with the effective property value on items; for sites other than the primary issuing body of the identifier concerned",
        "data_type": "String"
    },
    "P4114": {
        "label": "ADK member ID",
        "description": "Akademie der Künste in Berlin member ID",
        "data_type": "ExternalId"
    },
    "P2816": {
        "label": "HowLongToBeat ID",
        "description": "identifier of a video game at the website HowLongToBeat",
        "data_type": "ExternalId"
    },
    "P2832": {
        "label": "Joint Electronics Type Designation Automated System designation",
        "description": "unclassified designator for United States military electronic equipment",
        "data_type": "ExternalId"
    },
    "P3638": {
        "label": "Oorlogsmonument ID",
        "description": "identifier for a war memorial in the database maintained by the Dutch Nationaal Comité 4 en 5 mei",
        "data_type": "ExternalId"
    },
    "P3456": {
        "label": "ATP tennis tournament ID",
        "data_type": "ExternalId"
    },
    "P4222": {
        "label": "United Nations Treaty Series Volume Number",
        "description": "volume number within the United Nations Treaty Series",
        "data_type": "ExternalId"
    },
    "P3455": {
        "label": "CircleID writer ID",
        "description": "identifier for writers at CircleID",
        "data_type": "ExternalId"
    },
    "P3467": {
        "label": "Inventario Sculture - Polo Museale Fiorentino",
        "description": "identifier of an artwork in the inventory of sculptures of Florentine museums",
        "data_type": "ExternalId"
    },
    "P3480": {
        "label": "base Mémoire reference",
        "description": "reference in the base Mémoire for a French iconographic heritage element",
        "data_type": "ExternalId"
    },
    "P3482": {
        "label": "Europeana Fashion creator ID",
        "description": "identifier for a creator on Europeana Fashion",
        "data_type": "ExternalId"
    },
    "P2931": {
        "label": "Encyclopedia of Triangle Centers identifier",
        "description": "identifier for triangle centers used in the Encyclopedia of Triangle Centers, founded by Clark Kimberling",
        "data_type": "ExternalId"
    },
    "P3470": {
        "label": "Woodland Trust wood ID",
        "description": "identifier for a wood recorded by the Woodland Trust",
        "data_type": "ExternalId"
    },
    "P3481": {
        "label": "Parks & Gardens UK Record ID",
        "description": "identifier for an open space, in the Parks & Gardens UK database",
        "data_type": "ExternalId"
    },
    "P4276": {
        "label": "Cinémathèque québécoise work identifier",
        "description": "Cinémathèque québécoise identifier for a work",
        "data_type": "ExternalId"
    },
    "P4133": {
        "label": "Tourer.it ID",
        "description": "identifier for a cultural heritage site in Emilia-Romagna region in Italy, in the Tourer.it open data portal",
        "data_type": "ExternalId"
    },
    "P4152": {
        "label": "file format identification pattern",
        "description": "pattern or string which is used to identify a file as having a particular known format",
        "data_type": "String"
    },
    "P3841": {
        "label": "Human Phenotype Ontology ID",
        "description": "The Human Phenotype Ontology (HPO) is a widely used vocabulary of phenotypic abnormalities encountered in human disease",
        "data_type": "ExternalId"
    },
    "P3475": {
        "label": "SANU member ID",
        "description": "identifier for a member of the Serbian Academy of Sciences and Arts",
        "data_type": "ExternalId"
    },
    "P4018": {
        "label": "The Arabidopsis Information Resource Accession",
        "description": "Identifier for an object (e.g. clone, gene, genetic marker, sequence, etc.) in TAIR",
        "data_type": "ExternalId"
    },
    "P4197": {
        "label": "IBM graphic character set global ID",
        "description": "identifier of a character set assigned by IBM",
        "data_type": "ExternalId"
    },
    "P3569": {
        "label": "Cultureel Woordenboek ID",
        "description": "identifier for a concept in the Dutch Cultureel Woordenboek ('Cultural Dictionary')",
        "data_type": "ExternalId"
    },
    "P3793": {
        "label": "IPv6 routing prefix",
        "description": "range of IPv6 addresses",
        "data_type": "String"
    },
    "P3523": {
        "label": "Rfam ID",
        "description": "identifier in the database of non-coding RNA (ncRNA) families and other structured RNA elements",
        "data_type": "ExternalId"
    },
    "P3636": {
        "label": "PDB ligand ID",
        "description": "identifier for small molecules and ligands in the Protein Data Bank (PDB)",
        "data_type": "ExternalId"
    },
    "P3599": {
        "label": "archival creator authority record at the Archives nationales",
        "description": "identifier of the Archives Nationales (French National Archives)",
        "data_type": "ExternalId"
    },
    "P3034": {
        "label": "Indonesian ethnicity code",
        "description": "code for ethnic groups in Indonesia, defined in Ensiklopedi Suku Bangsa di Indonesia",
        "data_type": "ExternalId"
    },
    "P3072": {
        "label": "Storting person ID",
        "description": "identifier for a Norwegian parliamentarian in the official Storting database",
        "data_type": "ExternalId"
    },
    "P4069": {
        "label": "Italian National Olympic Committee athlete ID",
        "description": "identifier for an Italian athlete on the website of the Italian National Olympic Committee",
        "data_type": "ExternalId"
    },
    "P4056": {
        "label": "New Zealand Olympic Committee athlete ID",
        "description": "identifier for a Kiwi athlete on the New Zealand Olympic Committee website",
        "data_type": "ExternalId"
    },
    "P4063": {
        "label": "United States Olympic & Paralympic Committee athlete ID",
        "description": "identifier for an American athlete at the United States Olympic  & Paralympic Committee (USOPC) TeamUSA.org website",
        "data_type": "ExternalId"
    },
    "P4303": {
        "label": "Lega Pallavolo Serie A player ID",
        "description": "identifier for a volleyball player on the Lega Pallavolo Serie A website",
        "data_type": "ExternalId"
    },
    "P4285": {
        "label": "Theses.fr person ID",
        "description": "identifier of a PhD holder or thesis jury member, on the French thesis database",
        "data_type": "ExternalId"
    },
    "P4159": {
        "label": "WeRelate person ID",
        "description": "identifier for a person in the Creative Commons-licensed genealogical database WeRelate",
        "data_type": "ExternalId"
    },
    "P4260": {
        "label": "NPB player ID",
        "description": "identifier for a player on the Nippon Professional Baseball website",
        "data_type": "ExternalId"
    },
    "P4318": {
        "label": "Scottish Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Scottish Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4314": {
        "label": "SpeedskatingResults.com speed skater ID",
        "description": "identifier for a speed skater on the SpeedskatingResults.com",
        "data_type": "ExternalId"
    },
    "P4076": {
        "label": "WorldSBK.com racer identifier",
        "description": "identifier for a motorcycle racer on the WorldSBK.com website",
        "data_type": "ExternalId"
    },
    "P4298": {
        "label": "VBL player ID",
        "description": "identifier for a beach volleyball player on the website of the German Volleyball Federation",
        "data_type": "ExternalId"
    },
    "P4077": {
        "label": "Pizmonet ID",
        "description": "ID of an artist or a song in the israeli database for hit parades, Pizmonet",
        "data_type": "ExternalId"
    },
    "P4326": {
        "label": "BFI Filmography person ID",
        "description": "identifier for a person in the British Film Institute (BFI) 'Filmography' database",
        "data_type": "ExternalId"
    },
    "P4289": {
        "label": "LFH player ID",
        "description": "identifier for a player on the website of the French Ligue Féminine de Handball (LFH)",
        "data_type": "ExternalId"
    },
    "P3910": {
        "label": "Bollywood Hungama person numeric ID",
        "description": "identifier of an actor at Bollywood Hungama in the former scheme. Format: up to 9 digits",
        "data_type": "ExternalId"
    },
    "P3988": {
        "label": "National Library Board Singapore ID",
        "description": "Identifier of a person, organisation or place from the name authorities of National Library Board",
        "data_type": "ExternalId"
    },
    "P4302": {
        "label": "Royal Dutch Athletics Federation athlete ID",
        "description": "identifier for a Dutch athletics competitor on the Royal Dutch Athletics Federation website",
        "data_type": "ExternalId"
    },
    "P3908": {
        "label": "Reprezentacija player ID",
        "description": "identifier for a Serbian association football player at Reprezentacija.rs",
        "data_type": "ExternalId"
    },
    "P4319": {
        "label": "Elite Prospects staff ID",
        "description": "identifier for an ice hockey team staff member, on Eliteprospects.com",
        "data_type": "ExternalId"
    },
    "P4349": {
        "label": "Lotsawa House Indian author ID",
        "description": "identifier for Indian authors, in the Lotsawa House library of translations of Tibetan Buddhist religious texts",
        "data_type": "ExternalId"
    },
    "P4357": {
        "label": "Musikverket person ID",
        "description": "identifier for a person in the Swedish Musikverket database of the performing arts",
        "data_type": "ExternalId"
    },
    "P4315": {
        "label": "SwimSwam ID",
        "description": "identifier for a swimmer in the SwimSwam website database",
        "data_type": "ExternalId"
    },
    "P4186": {
        "label": "Australian Women's Register ID",
        "description": "identifier for a person or organisation in the Australian Women's Register",
        "data_type": "ExternalId"
    },
    "P4191": {
        "label": "Alljudo judoka ID",
        "description": "identifier for a judoka on the Alljudo.net website",
        "data_type": "ExternalId"
    },
    "P4192": {
        "label": "LNH player ID",
        "description": "identifier for a player on the Ligue Nationale de Handball (LNH) website",
        "data_type": "ExternalId"
    },
    "P3933": {
        "label": "Cinema.de ID",
        "description": "identifier for a film or person in the German film website cinema.de",
        "data_type": "ExternalId"
    },
    "P3926": {
        "label": "USA Track & Field athlete ID",
        "description": "identifier for an athlete on the USA Track & Field website",
        "data_type": "ExternalId"
    },
    "P3936": {
        "label": "NFF person ID",
        "description": "identifier for person's profile at fotball.no, the official website of the Norwegian Football Federation",
        "data_type": "ExternalId"
    },
    "P3958": {
        "label": "ENARD athlete ID",
        "description": "ID for argentinian athletes from the Ente Nacional de Alto Rendimiento Deportivo (High Sport Performance National Entity)",
        "data_type": "ExternalId"
    },
    "P4145": {
        "label": "Athenaeum person ID",
        "description": "artist id in the Athenaeum artworks website",
        "data_type": "ExternalId"
    },
    "P3940": {
        "label": "OlimpBase Chess Olympiad player ID",
        "description": "identifier for a player at www.olimpbase.org who took part in the Chess Olympiad",
        "data_type": "ExternalId"
    },
    "P3948": {
        "label": "MLL player ID",
        "description": "identifier for a player on the Major League Lacrosse website",
        "data_type": "ExternalId"
    },
    "P3945": {
        "label": "RANM member ID",
        "description": "identifier of a member of the Spanish Royal Academy of Medicine",
        "data_type": "ExternalId"
    },
    "P3981": {
        "label": "Misjonsarkiv person ID",
        "description": "identifier in the missionary archives of VID Specialized University",
        "data_type": "ExternalId"
    },
    "P3964": {
        "label": "BDCYL authority ID",
        "description": "identifier of an author, topic or place in Biblioteca Digital de Castilla y León",
        "data_type": "ExternalId"
    },
    "P3971": {
        "label": "PASE Domesday person ID",
        "description": "identifier for a land holder in 1066 or 1087 on the PASE Domesday website",
        "data_type": "ExternalId"
    },
    "P4232": {
        "label": "Figshare author ID",
        "description": "identifier for an author on Figshare",
        "data_type": "ExternalId"
    },
    "P4169": {
        "label": "YCBA agent ID",
        "description": "identifier for a person in the Yale Center for British Art database",
        "data_type": "ExternalId"
    },
    "P4116": {
        "label": "JewAge person ID",
        "description": "identifier for a person on JewAge, a genealogical database for Jewish people",
        "data_type": "ExternalId"
    },
    "P4008": {
        "label": "Early Aviators people ID",
        "description": "identifier for a person by Early Birds of Aviation, at www.earlyaviators.com",
        "data_type": "ExternalId"
    },
    "P4158": {
        "label": "autores.ar ID",
        "description": "identifier for an Argentinian literary author",
        "data_type": "ExternalId"
    },
    "P4023": {
        "label": "German Football Association person ID",
        "description": "identifier for players, manager and referees in the datacenter of the German Football Association (''Deutscher Fußball-Bund'', DFB)",
        "data_type": "ExternalId"
    },
    "P3915": {
        "label": "Australian Athletics Historical Results athlete ID",
        "description": "identifier for an athlete at the Australian Athletics Historical Results website",
        "data_type": "ExternalId"
    },
    "P4248": {
        "label": "The Black Book ID",
        "description": "ID for a person wanted by the Nazi administration, in the case of an invasion of the United Kingdom, as listed in the \"Black Book\" (\"Sonderfahndungsliste GB\")",
        "data_type": "ExternalId"
    },
    "P4012": {
        "label": "Semantic Scholar author ID",
        "description": "identifier for an author in the Semantic Scholar database",
        "data_type": "ExternalId"
    },
    "P4040": {
        "label": "Rock.com.ar artist ID",
        "description": "identifier for an artist at Rock.com.ar",
        "data_type": "ExternalId"
    },
    "P4126": {
        "label": "ESEC person ID",
        "description": "identifier of a member of the French Economic, Social and Environmental Council",
        "data_type": "ExternalId"
    },
    "P4050": {
        "label": "French Olympic Committee athlete ID",
        "description": "identifier for a French athlete on the French Olympic Committee's EspritBleu website",
        "data_type": "ExternalId"
    },
    "P4054": {
        "label": "Canadian Olympic Committee athlete ID",
        "description": "identifier for a Canadian athlete at Olympic.ca, the official website of the Canadian Olympic Committee",
        "data_type": "ExternalId"
    },
    "P4252": {
        "label": "All-Russian Mathematical Portal ID",
        "description": "identifier for a mathematician in the All-Russian Mathematical Portal",
        "data_type": "ExternalId"
    },
    "P4062": {
        "label": "Czech Olympic Committee athlete ID",
        "description": "identifier for a Czech athlete on the website of the Czech Olympic Committee (Český olympijský výbor – ČOV)",
        "data_type": "ExternalId"
    },
    "P4066": {
        "label": "Hungarian Olympic Committee athlete ID",
        "description": "identifier for an athlete on the website of the Hungarian Olympic Committee",
        "data_type": "ExternalId"
    },
    "P4263": {
        "label": "THW Kiel player ID",
        "description": "identifier for a handball player on the THW Kiel website",
        "data_type": "ExternalId"
    },
    "P3923": {
        "label": "Diamond League athlete ID",
        "description": "identifier for an athlete on the Diamond League website",
        "data_type": "ExternalId"
    },
    "P4278": {
        "label": "FCI rider ID",
        "description": "identifier for a rider on the Italian Cycling Federation website",
        "data_type": "ExternalId"
    },
    "P4283": {
        "label": "LUMIERE director ID",
        "description": "ID of a film director in the LUMIERE database of the European Audiovisual Observatory",
        "data_type": "ExternalId"
    },
    "P4164": {
        "label": "National Baseball Hall of Fame and Museum ID",
        "description": "identifier for a player on the United States' National Baseball Hall of Fame and Museum website",
        "data_type": "ExternalId"
    },
    "P3953": {
        "label": "ALPG Tour golf player ID",
        "description": "historical identifier for a female golf player on a discontinued Australian Ladies Professional Golf website",
        "data_type": "ExternalId"
    },
    "P3932": {
        "label": "Digital Valencian Library author ID",
        "description": "identifier of a person in the database of the Valencian Library, the regional library in the Land of Valencia, Spain",
        "data_type": "ExternalId"
    },
    "P3957": {
        "label": "RealGM basketball player ID",
        "description": "identifier for a basketball player on the RealGM website",
        "data_type": "ExternalId"
    },
    "P3995": {
        "label": "Filmweb.pl ID",
        "description": "identifier for person, film (movie), TV series or videogame, in the filmweb.pl database",
        "data_type": "ExternalId"
    },
    "P4193": {
        "label": "Familypedia person ID",
        "description": "identifier for a person on Familypedia, a genealogical database hosted at Wikia",
        "data_type": "ExternalId"
    },
    "P3980": {
        "label": "Unifrance person ID",
        "description": "identifier of a person on unifrance.org",
        "data_type": "ExternalId"
    },
    "P4332": {
        "label": "Deutsche Synchronkartei actor ID",
        "description": "Identifier for actors in the synchronisation (dubbing) database Deutsche Synchronkartei. Format: digits",
        "data_type": "ExternalId"
    },
    "P3973": {
        "label": "PIM authority ID",
        "description": "ID for an authority record (mostly persons) in the PIM (Petőfi Literary Museum, Hungary)",
        "data_type": "ExternalId"
    },
    "P4261": {
        "label": "FFVoile sailor ID",
        "description": "licence number of a sailor for the French Sailing Federation, corresponding identifier on its website",
        "data_type": "ExternalId"
    },
    "P3956": {
        "label": "National Academy of Medicine (France) Member ID",
        "data_type": "ExternalId"
    },
    "P4026": {
        "label": "Cullum number",
        "description": "identifier for a graduate of the US Military Academy (West Point)",
        "data_type": "ExternalId"
    },
    "P4216": {
        "label": "LNB Pro A player ID",
        "description": "identifier for a player in the Ligue Nationale de Basket-ball (LNB) Pro A webpages",
        "data_type": "ExternalId"
    },
    "P4108": {
        "label": "Gedbas genealogy person ID",
        "description": "identifier for a person in the genealogical database of the Verein für Computergenealogie e.V. on gedbas.genealogy.net",
        "data_type": "ExternalId"
    },
    "P3927": {
        "label": "eWRC-results.com racer ID",
        "description": "identifier for a driver or co-driver on the eWRC-results.com website",
        "data_type": "ExternalId"
    },
    "P3930": {
        "label": "Rallye-info.com driver or co-driver ID",
        "description": "identifier for a driver or co-driver on the Rallye-info.com website",
        "data_type": "ExternalId"
    },
    "P3924": {
        "label": "Track and Field Statistics female athlete ID",
        "description": "identifier for a female athlete on the Track and Field Statistics website",
        "data_type": "ExternalId"
    },
    "P4348": {
        "label": "Lotsawa House Tibetan author ID",
        "description": "identifier for Tibetan authors, in the Lotsawa House library of translations of Tibetan Buddhist religious texts",
        "data_type": "ExternalId"
    },
    "P4281": {
        "label": "LNV player ID",
        "description": "identifier for a player on the French Ligue nationale de volley websiste",
        "data_type": "ExternalId"
    },
    "P3942": {
        "label": "Bmx-results.com rider ID",
        "description": "identifier for a rider on the Bmx-results.com website",
        "data_type": "ExternalId"
    },
    "P4042": {
        "label": "ESBL athlete ID",
        "description": "identifier for a sportsperson in the Estonian sportsbiographical lexicon",
        "data_type": "ExternalId"
    },
    "P4299": {
        "label": "ITF wheelchair player ID",
        "description": "identifier for a wheelchair tennis player on the International Tennis Federation website",
        "data_type": "ExternalId"
    },
    "P3987": {
        "label": "SHARE Catalogue author ID",
        "description": "identifier for authors in SHARE Catalogue, a linked data federated catalogue of several Southern Italy universities",
        "data_type": "ExternalId"
    },
    "P3928": {
        "label": "MotoGP racer ID",
        "description": "identifier for a motorcycle racer on the MotoGP website",
        "data_type": "ExternalId"
    },
    "P3925": {
        "label": "Track and Field Statistics male athlete ID",
        "description": "identifier for a male athlete on the Track and Field Statistics website",
        "data_type": "ExternalId"
    },
    "P4019": {
        "label": "USL Championship player ID",
        "description": "identifier for soccer player profile on website of USL Championship, formerly known as United Soccer League (USL) and USL Pro",
        "data_type": "ExternalId"
    },
    "P4130": {
        "label": "USHMM person ID",
        "description": "identifier for a person in the UHSMM, a database of Holocaust survivors and victims",
        "data_type": "ExternalId"
    },
    "P3955": {
        "label": "NLL player ID",
        "description": "identifier for a player on the National Lacrosse League website",
        "data_type": "ExternalId"
    },
    "P4068": {
        "label": "Chinese Olympic Committee athlete ID",
        "description": "identifier for a Chinese athlete at the Chinese Olympic Committee's CIS Chinese Athletes Database website",
        "data_type": "ExternalId"
    },
    "P4262": {
        "label": "Footoféminin.fr player ID",
        "description": "identifier for a female association football player on the Footoféminin.fr website",
        "data_type": "ExternalId"
    },
    "P4347": {
        "label": "Florida Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Florida Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4362": {
        "label": "Alabama Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Alabama Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4053": {
        "label": "German Olympic Sports Confederation athlete ID",
        "description": "identifier for a German athlete on the website of the German Olympic Sports Confederation (DOSB: Deutscher Olympischer Sportbund)",
        "data_type": "ExternalId"
    },
    "P4060": {
        "label": "Brazilian Olympic Committee athlete ID",
        "description": "identifier for a Brazilian athlete at the Brazilian Olympic Committee (Portuguese: Comitê Olímpico do Brasil) website",
        "data_type": "ExternalId"
    },
    "P4259": {
        "label": "Lega Pallavolo Serie A Femminile player ID",
        "description": "identifier for a volleyball player on the Lega Pallavolo Serie A  Femminile website",
        "data_type": "ExternalId"
    },
    "P4061": {
        "label": "Lithuanian National Olympic Committee athlete ID",
        "description": "identifier for athlete on website of the Lithuanian National Olympic Committee (in Lithuanian: Lietuvos tautinis olimpinis komitetas, LTOK)",
        "data_type": "ExternalId"
    },
    "P4208": {
        "label": "Billboard artist ID",
        "description": "identifier for an artist at the Billboard website",
        "data_type": "ExternalId"
    },
    "P4065": {
        "label": "Argentine Olympic Committee athlete ID",
        "description": "identifier for an Argentine athlete on the website of the Argentine Olympic Committee (Comité Olímpico Argentino)",
        "data_type": "ExternalId"
    },
    "P4074": {
        "label": "FFN swimmer ID",
        "description": "unique federal identifier (IUF) of a French swimmer in extraNat.fr, database of the French Swimming Federation (FFN)",
        "data_type": "ExternalId"
    },
    "P4067": {
        "label": "Romanian Olympic and Sports Committee athlete ID",
        "description": "identifier for an athlete on the website of the Romanian Olympic and Sports Committee",
        "data_type": "ExternalId"
    },
    "P4048": {
        "label": "J.League manager ID",
        "description": "identifier for a manager (coach) at website of J.League, Japan's professional association football (soccer) league",
        "data_type": "ExternalId"
    },
    "P4361": {
        "label": "ExecutedToday ID",
        "description": "identifier for a person or group of individuals, in the ExecutedToday database",
        "data_type": "ExternalId"
    },
    "P4343": {
        "label": "WBPLN author ID",
        "description": "identifier for authors listed in the West Bengal Public Library Network",
        "data_type": "ExternalId"
    },
    "P7397": {
        "label": "247Sports ID",
        "description": "247Sports ID for Football and Basketball Prospects",
        "data_type": "ExternalId"
    },
    "P7438": {
        "label": "FantLab publisher ID",
        "description": "identifier for the publisher in FantLab",
        "data_type": "ExternalId"
    },
    "P7286": {
        "label": "Writers Guild of America writer ID",
        "description": "identifier for a writer in the Writers Guild of America's Find a Writer database",
        "data_type": "ExternalId"
    },
    "P7485": {
        "label": "Kinematoscope writer ID",
        "description": "Database about Films adaptations of literary work (18th and 19th century)",
        "data_type": "ExternalId"
    },
    "P7403": {
        "label": "Publons publisher ID",
        "description": "identifier for publishers on the Publons website",
        "data_type": "ExternalId"
    },
    "P7352": {
        "label": "GENUKI ID",
        "description": "identifier for a UK or Irish place on the GENUKI.org.uk website",
        "data_type": "ExternalId"
    },
    "P7434": {
        "label": "NAD place ID",
        "description": "the topographic database at the Swedish National Archive",
        "data_type": "ExternalId"
    },
    "P7458": {
        "label": "Kinematoscope film ID",
        "description": "Database about Films adaptations of literary work (18th and 19th century)",
        "data_type": "ExternalId"
    },
    "P7348": {
        "label": "Kivid.info ID",
        "description": "identifier for minerals, rocks, mineral resources at kivid.info",
        "data_type": "ExternalId"
    },
    "P7428": {
        "label": "Bluetooth Declaration ID",
        "description": "unique identifier of a certification granted by Bluetooth SIG for product conformance to Bluetooth standards",
        "data_type": "ExternalId"
    },
    "P7296": {
        "label": "Lower Austrian Museum ID",
        "description": "identifier to link with the Lower Austrian database of museums",
        "data_type": "ExternalId"
    },
    "P7446": {
        "label": "Basis Wien institution ID",
        "description": "identifier of an institution in the Basis Wien database",
        "data_type": "ExternalId"
    },
    "P7363": {
        "label": "ISSN-L",
        "description": "linking ISSN (ISSN-L) is a specific ISSN that groups the different media of the same serial publication",
        "data_type": "ExternalId"
    },
    "P7335": {
        "label": "Mixer game ID (former scheme)",
        "description": "identifier for a video game on the Mixer live streaming video platform",
        "data_type": "ExternalId"
    },
    "P7411": {
        "label": "Pubs Galore ID",
        "description": "external identifier for pubs in the United Kingdom",
        "data_type": "ExternalId"
    },
    "P7412": {
        "label": "ForaDeJogo.net team ID",
        "description": "identifier for a association football (soccer) team in the ForaDeJogo.net database",
        "data_type": "ExternalId"
    },
    "P7410": {
        "label": "Healthcare Common Procedure Coding System Identifier",
        "description": "health care procedure codes based on the American Medical Association's Current Procedural Terminology",
        "data_type": "ExternalId"
    },
    "P7437": {
        "label": "FantLab award ID",
        "description": "identifier for the award in FantLab",
        "data_type": "ExternalId"
    },
    "P7439": {
        "label": "FantLab work ID",
        "description": "identifier for the literary work in FantLab",
        "data_type": "ExternalId"
    },
    "P7358": {
        "label": "FootballFacts.ru team ID",
        "description": "identifier for a association football (soccer) team in the FootballFacts.ru database",
        "data_type": "ExternalId"
    },
    "P7492": {
        "label": "EU-Football.info match ID",
        "description": "identifier for an association football match on the EU-Football.info website",
        "data_type": "ExternalId"
    },
    "P7370": {
        "label": "PCEngine Catalog Project ID",
        "description": "identifier of a video game in the PCEngine Catalog Project",
        "data_type": "ExternalId"
    },
    "P7493": {
        "label": "PlaymakerStats.com match ID",
        "description": "identifier for an association football (soccer) match at PlaymakerStats.com (formerly TheFinalBall.com)",
        "data_type": "ExternalId"
    },
    "P7454": {
        "label": "Soccerbase team ID",
        "description": "identifier for a association football (soccer) team in the Soccerbase database",
        "data_type": "ExternalId"
    },
    "P7468": {
        "label": "MOHW HospID",
        "description": "ID of hospital registered under Ministry of Health and Welfare (Taiwan)",
        "data_type": "ExternalId"
    },
    "P7372": {
        "label": "memoriademadrid publication ID",
        "description": "identifier for a journal in the journals archive of \"Biblioteca Digital memoriademadrid\", an open access digital library of Spanish-language periodical publications, established by the city of Madrid",
        "data_type": "ExternalId"
    },
    "P7456": {
        "label": "AAAS keyword ID",
        "description": "identifier for a topic, used by the American Association for the Advancement of Science (AAAS)",
        "data_type": "ExternalId"
    },
    "P7322": {
        "label": "Marine Regions Geographic IDentifier (MRGID)",
        "description": "unique ID created to help name and locate geographical features by the Marine Regions website",
        "data_type": "ExternalId"
    },
    "P7425": {
        "label": "Rotterdam City Archives actor ID",
        "description": "persons and organisations in the database of the media archive of the city of Rotterdam",
        "data_type": "ExternalId"
    },
    "P7346": {
        "label": "Fossiilid.info ID",
        "description": "identifier for fossils at fossiilid.info",
        "data_type": "ExternalId"
    },
    "P7326": {
        "label": "MobyGames genre ID",
        "description": "identifier of a video game genre in the MobyGames database",
        "data_type": "ExternalId"
    },
    "P7429": {
        "label": "Bundle ID",
        "description": "unique identifier for an application used in iOS, iPadOS, macOS, watchOS and tvOS systems",
        "data_type": "ExternalId"
    },
    "P7357": {
        "label": "Diccionari de la Literatura Catalana ID",
        "description": "identifier for an item in the Diccionari de la Literatura Catalana",
        "data_type": "ExternalId"
    },
    "P7351": {
        "label": "FootballDatabase.eu team ID",
        "description": "identifier for a association football (soccer) team in the FootballDatabase.eu database",
        "data_type": "ExternalId"
    },
    "P7431": {
        "label": "Wolfram Language quantity ID",
        "description": "identifier for a (physical, chemical, ...) quantity in the Wolfram Language",
        "data_type": "ExternalId"
    },
    "P7302": {
        "label": "Digital Giza ID",
        "description": "identifier of a Pyramid or other funeral site on Egypt’s Giza Plateau in the American Database Digital Giza",
        "data_type": "ExternalId"
    },
    "P7432": {
        "label": "Steam bundle ID",
        "description": "identification number for Steam platform bundle",
        "data_type": "ExternalId"
    },
    "P7476": {
        "label": "iFixit teardown ID",
        "description": "identifier for a disassembly review of a device on the iFixit website",
        "data_type": "ExternalId"
    },
    "P7325": {
        "label": "DANE code",
        "description": "identification code for Colombian municipalities according to the Departamento Administrativo Nacional de Estadística (DANE)",
        "data_type": "ExternalId"
    },
    "P7455": {
        "label": "Transfermarkt match ID",
        "description": "identifier for an association football (soccer) match in the transfermarkt.com database",
        "data_type": "ExternalId"
    },
    "P7427": {
        "label": "FreeBSD port",
        "description": "name of the official FreeBSD port of this application",
        "data_type": "ExternalId"
    },
    "P7292": {
        "label": "Arnet Miner publication ID",
        "description": "ID of a publication in Arnet Miner",
        "data_type": "ExternalId"
    },
    "P7291": {
        "label": "L-number",
        "description": "semi-scientific classification system for catfish from the Loricariidae family",
        "data_type": "ExternalId"
    },
    "P7350": {
        "label": "Gazetteer for Scotland place ID",
        "description": "identifier for a geographical item on the Gazetteer for Scotland website",
        "data_type": "ExternalId"
    },
    "P7287": {
        "label": "WorldFootball.net team ID",
        "description": "identifier for a association football (soccer) team in the WorldFootball.net database",
        "data_type": "ExternalId"
    },
    "P7453": {
        "label": "90minut.pl team ID",
        "description": "identifier for a association football (soccer) team in the 90minut.pl database",
        "data_type": "ExternalId"
    },
    "P7441": {
        "label": "WorldFootball.net competition ID",
        "description": "identifier for a association football (soccer) competition in the WorldFootball.net database",
        "data_type": "ExternalId"
    },
    "P7298": {
        "label": "Auckland Museum ID",
        "description": "identifier assigned to a person by the Auckland War Memorial Museum, New Zealand",
        "data_type": "ExternalId"
    },
    "P7288": {
        "label": "Bursa Malaysia stock code",
        "description": "stock code on Bursa Malaysia",
        "data_type": "ExternalId"
    },
    "P7307": {
        "label": "TOP500 System ID",
        "description": "unique identifier of a supercomputer in the TOP500 performance list",
        "data_type": "ExternalId"
    },
    "P7448": {
        "label": "Basis Wien object ID",
        "description": "identifier of an object in the Basis Wien database",
        "data_type": "ExternalId"
    },
    "P7314": {
        "label": "TDV İslam Ansiklopedisi ID",
        "description": "identifier for the Islamic Encyclopedia created by the Turkish religion authority Diyanet",
        "data_type": "ExternalId"
    },
    "P7382": {
        "label": "Turkish Football Federation match ID",
        "description": "identifier association football match at TFF database",
        "data_type": "ExternalId"
    },
    "P7480": {
        "label": "Brazilian federal deputy ID",
        "description": "identifier for a member of the Chamber of Deputies of Brazil",
        "data_type": "ExternalId"
    },
    "P7365": {
        "label": "China Martyrs ID",
        "description": "ID for someone killed or sacrificed themself in China",
        "data_type": "ExternalId"
    },
    "P7398": {
        "label": "Internet Off-Broadway Database production ID",
        "description": "identifier for a production in Lortel's Internet Off-Broadway Database",
        "data_type": "ExternalId"
    },
    "P7401": {
        "label": "SMHI drainage basin ID",
        "description": "ID of drainage basin according to the Swedish Meteorological and Hydrological Institute",
        "data_type": "ExternalId"
    },
    "P7323": {
        "label": "Archnet site ID",
        "description": "site ID of archnet.org related to architecture, urban issues, landscape design, visual culture and conservation issues",
        "data_type": "ExternalId"
    },
    "P7402": {
        "label": "Turkish Football Federation stadium ID",
        "description": "identifier association football venue at TFF database",
        "data_type": "ExternalId"
    },
    "P7494": {
        "label": "Soccerway match ID",
        "description": "identifier for an association football (soccer) match in the Soccerway database",
        "data_type": "ExternalId"
    },
    "P7490": {
        "label": "Teams.by team ID",
        "description": "identifier for a association football (soccer) team in the teams.by database",
        "data_type": "ExternalId"
    },
    "P7304": {
        "label": "Highland Historic Environment Record ID",
        "description": "ID for a heritage feature in the Highland Historic Environment Record website and database",
        "data_type": "ExternalId"
    },
    "P7289": {
        "label": "CPUID",
        "description": "x86/x86-64 processor type information returned by the CPUID instruction into the EAX register",
        "data_type": "ExternalId"
    },
    "P7299": {
        "label": "MUBI film ID",
        "description": "identifier for a film at MUBI",
        "data_type": "ExternalId"
    },
    "P7390": {
        "label": "Knesset Law ID",
        "description": "identifier for law in Knesset site",
        "data_type": "ExternalId"
    },
    "P7306": {
        "label": "Downdetector ID",
        "description": "identifier for a webpage on the Downdetector website",
        "data_type": "ExternalId"
    },
    "P7495": {
        "label": "Fussballdaten.de team ID",
        "description": "identifier for a association football (soccer) team in the Fussballdaten.de database",
        "data_type": "ExternalId"
    },
    "P7470": {
        "label": "URN formatter",
        "description": "formatter to generate Uniform Resource Name (URN) from property value. Include $1 to be replaced with property value",
        "data_type": "String"
    },
    "P7353": {
        "label": "NTS Radio artist ID",
        "description": "identifier for an artist on the NTS Radio website",
        "data_type": "ExternalId"
    },
    "P7413": {
        "label": "Ancient Tree Inventory ID",
        "description": "identifier for ancient and notable trees in the United Kingdom",
        "data_type": "ExternalId"
    },
    "P7484": {
        "label": "Kinematoscope film director ID",
        "description": "database of film adaptations of literary work (18th and 19th century)",
        "data_type": "ExternalId"
    },
    "P7460": {
        "label": "FlashScore.com match ID",
        "description": "identifier for a match (association football, basketball, rugby, ice hockey, etc.) in the FlashScore.com database",
        "data_type": "ExternalId"
    },
    "P7364": {
        "label": "FIDAL team ID",
        "description": "identifier for an athletic team in Italy by the Italian athletics federation (FIDAL)",
        "data_type": "ExternalId"
    },
    "P7361": {
        "label": "UEFA team ID",
        "description": "identifier for a association football (soccer) team in the UEFA database",
        "data_type": "ExternalId"
    },
    "P7312": {
        "label": "Aozora Bunko work ID",
        "description": "identifier of an introduction page of work on Aozora Bunko",
        "data_type": "ExternalId"
    },
    "P7387": {
        "label": "Q-Codes ID",
        "description": "Q-Codes ID is for indexing Family medicine related docs; with the ICPC 2, both form the Core Content Classification in General Practice/ Family Medicine (3CGP)",
        "data_type": "ExternalId"
    },
    "P7464": {
        "label": "Genetics Home Reference Conditions ID",
        "description": "identifier for disease in the United States National Institutes of Health's Genetics Home Reference",
        "data_type": "ExternalId"
    },
    "P7318": {
        "label": "Behind The Voice Actors short ID",
        "description": "identifier of a short on the website Behind The Voice Actors",
        "data_type": "ExternalId"
    },
    "P7333": {
        "label": "Pathway Ontology ID",
        "description": "identifier in the Pathway Ontology database",
        "data_type": "ExternalId"
    },
    "P7414": {
        "label": "State Catalogue of the Museum Fund of Russia museum ID",
        "data_type": "ExternalId"
    },
    "P7404": {
        "label": "ADL Hate Symbols Database ID",
        "description": "ID for a item in the Anti-Defamation League Hate Symbols Database",
        "data_type": "ExternalId"
    },
    "P7487": {
        "label": "Kinematoscope literary work ID",
        "description": "Database about Films adaptations of literary work (18th and 19th century)",
        "data_type": "ExternalId"
    },
    "P7294": {
        "label": "itch.io URL",
        "description": "URL of a project hosted on itch.io",
        "data_type": "Url"
    },
    "P7336": {
        "label": "GCF Reference",
        "description": "unique identifier of a product which has been certified by the Global Certification Forum",
        "data_type": "ExternalId"
    },
    "P7337": {
        "label": "Wi-Fi Certification ID",
        "description": "unique identifier of a certification granted by the Wi-Fi Alliance for product conformance to Wi-Fi Alliance standards",
        "data_type": "ExternalId"
    },
    "P7339": {
        "label": "Rivals.com ID",
        "description": "Rivals.com ID for Football and Basketball Prospects",
        "data_type": "ExternalId"
    },
    "P7488": {
        "label": "BDFutbol team ID",
        "description": "identifier for a association football (soccer) team in the BDFutbol database",
        "data_type": "ExternalId"
    },
    "P7445": {
        "label": "Basis Wien person ID",
        "description": "identifier of a person in the Basis Wien database",
        "data_type": "ExternalId"
    },
    "P7447": {
        "label": "Basis Wien event ID",
        "description": "identifier of an event in the Basis Wien database",
        "data_type": "ExternalId"
    },
    "P7449": {
        "label": "NARCIS researcher ID",
        "description": "Dutch researchers with information about publications",
        "data_type": "ExternalId"
    },
    "P7450": {
        "label": "Turkish Football Federation team ID",
        "description": "identifier association football team at TFF database",
        "data_type": "ExternalId"
    },
    "P7388": {
        "label": "Great Encyclopedia of Navarre ID",
        "description": "identifier of an item in the Great Encyclopaedia of Navarre",
        "data_type": "ExternalId"
    },
    "P7329": {
        "label": "ICD-11 ID (MMS)",
        "description": "identifier in ICD terminology of diseases - Version 11 (Mortality and Morbidity Statistics) linearization (not the foundation ID)",
        "data_type": "ExternalId"
    },
    "P7477": {
        "label": "Internet Content Provider Registration Record ID",
        "description": "permit issued by the Chinese Ministry of Industry and Information Technology to permit China-based websites to operate in China",
        "data_type": "ExternalId"
    },
    "P7366": {
        "label": "FAIMER school ID",
        "data_type": "ExternalId"
    },
    "P7320": {
        "label": "Labelcode",
        "description": "record label identifier code issued by Gesellschaft zur Verwertung von Leistungsschutzrechten (GVL)",
        "data_type": "ExternalId"
    },
    "P7389": {
        "label": "Three Decks people ID",
        "description": "naval personnel identifier at Three Decks",
        "data_type": "ExternalId"
    },
    "P7433": {
        "label": "FantLab author ID",
        "description": "identifier for the author in FantLab",
        "data_type": "ExternalId"
    },
    "P7375": {
        "label": "MWNF URL",
        "description": "url in the database of the Museum With No Frontiers (MWNF) virtual museum for the transmission of knowledge",
        "data_type": "Url"
    },
    "P7344": {
        "label": "The Wind Power farm ID",
        "description": "identifier for a wind farm sur le site web The Wind Power",
        "data_type": "ExternalId"
    },
    "P5380": {
        "label": "National Academy of Sciences member ID",
        "description": "identifier for a member or foreign associate on the American National Academy of Sciences website",
        "data_type": "ExternalId"
    },
    "P5211": {
        "label": "GONIAT author ID",
        "description": "identifier for a taxonomic author, in the GONIAT database",
        "data_type": "ExternalId"
    },
    "P5158": {
        "label": "OnsOranje player ID",
        "description": "identifier for a player in the database of OnsOranje",
        "data_type": "ExternalId"
    },
    "P5509": {
        "label": "MAPS poet ID",
        "description": "identifier for a poet on the Modern American Poetry website",
        "data_type": "ExternalId"
    },
    "P5545": {
        "label": "Paris Faculty of Letters professor ID (1809–1908)",
        "description": "identifier of a professor on the online version of La Faculté des lettres de Paris (1809–1908)",
        "data_type": "ExternalId"
    },
    "P5406": {
        "label": "Rendez-vous de l'histoire ID",
        "description": "identifier of a participant of the 'Rendez-vous de l'histoire' on the website of this festival",
        "data_type": "ExternalId"
    },
    "P5418": {
        "label": "Pasteur Institute person ID",
        "description": "identifier for a person on the Pasteur Institute website",
        "data_type": "ExternalId"
    },
    "P5463": {
        "label": "AE member ID",
        "description": "identifier for a member of the Academy of Europe",
        "data_type": "ExternalId"
    },
    "P5462": {
        "label": "RHE professor ID",
        "description": "identifier of a doctor on the database of professor of French Faculties of art and science from 1808 to 1880",
        "data_type": "ExternalId"
    },
    "P5502": {
        "label": "LRB contributor ID",
        "description": "identifier for a contributor on the London Review of Books website",
        "data_type": "ExternalId"
    },
    "P5619": {
        "label": "FFF author ID",
        "description": "identifier for an author on the Franco Fossati Foundation website",
        "data_type": "ExternalId"
    },
    "P5496": {
        "label": "Médicos históricos doctor ID",
        "description": "identifier for a doctor in the Médicos históricos database of the Complutense University of Madrid",
        "data_type": "ExternalId"
    },
    "P5466": {
        "label": "French Academy in Rome resident ID",
        "description": "identifier for a resident on the French Academy in Rome website",
        "data_type": "ExternalId"
    },
    "P5468": {
        "label": "Historia de la Medicina person ID",
        "description": "identifier for a person on the Historia de la Medicina website",
        "data_type": "ExternalId"
    },
    "P5488": {
        "label": "artist-info curator ID",
        "description": "identifier in the artist-info curator database of exhibition records",
        "data_type": "ExternalId"
    },
    "P5491": {
        "label": "BD Gest' author ID",
        "description": "identifier for an author on the BD Gest' website",
        "data_type": "ExternalId"
    },
    "P5469": {
        "label": "Mormon Literature and Creative Arts Database artist ID",
        "description": "identifier for a person listed in the Mormon Literature and Creative Arts Database",
        "data_type": "ExternalId"
    },
    "P5546": {
        "label": "College de France professor ID (1909-1939)",
        "description": "identifier of a professor on the online version of Les Professeurs du Collège de France",
        "data_type": "ExternalId"
    },
    "P5312": {
        "label": "Kino-teatr.ua person ID",
        "description": "identifier for a person at Kino-teatr.ua",
        "data_type": "ExternalId"
    },
    "P5550": {
        "label": "La Vie des idées ID",
        "description": "identifier of an author on La Vie des idées",
        "data_type": "ExternalId"
    },
    "P5544": {
        "label": "Paris Faculty of Letters professor ID (1909-1939)",
        "description": "identifier of a professor on the online version of Les Professeurs de la Faculté des lettres de Paris",
        "data_type": "ExternalId"
    },
    "P5492": {
        "label": "EDIT16 catalogue author ID",
        "description": "identifier of the EDIT16 catalogue about authors cited in 16th-century books or editions",
        "data_type": "ExternalId"
    },
    "P5563": {
        "label": "CSDE Lynching Database ID",
        "description": "identifier for a lynching victim in the CSDE Lynching Database",
        "data_type": "ExternalId"
    },
    "P5534": {
        "label": "Open Media Database person ID",
        "description": "identifier for a person in the Open Media Database",
        "data_type": "ExternalId"
    },
    "P5540": {
        "label": "RedTube ID",
        "description": "identifier for an adult performer in the database of the RedTube website",
        "data_type": "ExternalId"
    },
    "P5516": {
        "label": "Virtual Laboratory person ID",
        "description": "identifier for a person in the Virtual Laboratory database of the Max Planck Institute for the History of Science",
        "data_type": "ExternalId"
    },
    "P5319": {
        "label": "César Award person ID",
        "description": "identifier for a person on the César Awards website",
        "data_type": "ExternalId"
    },
    "P5622": {
        "label": "National Cartoonists Society member ID",
        "description": "member identifier on the National Cartoonists Society on website",
        "data_type": "ExternalId"
    },
    "P5077": {
        "label": "Russian Athletics ID",
        "description": "identifier for athletes in the English version of the Russian Athletics database and website",
        "data_type": "ExternalId"
    },
    "P5553": {
        "label": "IDIH ID",
        "description": "identifier on a researcher on the International Dictionary of Intellectual Historians",
        "data_type": "ExternalId"
    },
    "P5541": {
        "label": "Paris Faculty of Science professor ID",
        "description": "identifier of a professor on the online version of Les Professeurs de la Faculté des sciences de Paris",
        "data_type": "ExternalId"
    },
    "P5587": {
        "label": "Libris-URI",
        "description": "identifier for an item in the catalogue of the Swedish National Library",
        "data_type": "ExternalId"
    },
    "P5346": {
        "label": "Playmates Playmate ID",
        "description": "identifier for a Playmate on the Playmates website",
        "data_type": "ExternalId"
    },
    "P5175": {
        "label": "Univ-droit jurist ID",
        "description": "identifier of a legal scholar on Univ-droit",
        "data_type": "ExternalId"
    },
    "P5612": {
        "label": "LegaBasket.it coach ID",
        "description": "identifier for a coach at legabasket.it",
        "data_type": "ExternalId"
    },
    "P5562": {
        "label": "Tebeosfera ID",
        "description": "identifier for an author on the Tebeosfera website",
        "data_type": "ExternalId"
    },
    "P5361": {
        "label": "BNB person ID",
        "description": "identifier of a person in the British National Bibliography (bnb.data.bl.uk)",
        "data_type": "ExternalId"
    },
    "P5614": {
        "label": "Aosdána ID",
        "description": "identifier for a member on the Aosdána website",
        "data_type": "ExternalId"
    },
    "P5631": {
        "label": "Canal Académie ID",
        "description": "identifier of an academician on Canal Académie's website",
        "data_type": "ExternalId"
    },
    "P5636": {
        "label": "Black Ferns player ID",
        "description": "ID of a female rugby union player of the Black Ferns, the New Zealand women's national rugby union team",
        "data_type": "ExternalId"
    },
    "P5644": {
        "label": "World Rugby Women's World Cup player ID",
        "description": "identifier for a female rugby union player at the 15-a-side World Cup",
        "data_type": "ExternalId"
    },
    "P5629": {
        "label": "Brasseur ID",
        "description": "identifier of a mathematics teacher on the online Dictionnaire des professeurs de mathématiques spéciales",
        "data_type": "ExternalId"
    },
    "P5616": {
        "label": "Comédie-Française ID",
        "description": "identifier for a stage actor on the Comédie-Française website",
        "data_type": "ExternalId"
    },
    "P5617": {
        "label": "Evene ID",
        "description": "identifier for a person on the Evene website",
        "data_type": "ExternalId"
    },
    "P5632": {
        "label": "Persons of Indian Studies ID",
        "description": "identifier for an Indologist on the 'Persons of Indian Studies' website",
        "data_type": "ExternalId"
    },
    "P5075": {
        "label": "Polish Athletics Federation ID",
        "description": "identifier for an athlete on the Polish Athletics Federation website",
        "data_type": "ExternalId"
    },
    "P5442": {
        "label": "Library of Congress of Chile person ID",
        "description": "Biographical Articles of the Library of Congress of Chile ID",
        "data_type": "ExternalId"
    },
    "P5212": {
        "label": "Armenian National Academy of Sciences ID",
        "description": "identifier for a member of the Armenian National Academy of Sciences",
        "data_type": "ExternalId"
    },
    "P5035": {
        "label": "Lambiek Comiclopedia artist ID",
        "description": "identifier for an article on the online biographical reference work about global cartoonists",
        "data_type": "ExternalId"
    },
    "P5419": {
        "label": "NYRB contributor ID",
        "description": "identifier for a contributor on the New York Review of Books website",
        "data_type": "ExternalId"
    },
    "P5057": {
        "label": "Handball-Bundesliga player ID",
        "description": "identifier for a player on the Handball-Bundesliga (HBL) website",
        "data_type": "ExternalId"
    },
    "P5254": {
        "label": "FilmTv.it person ID",
        "description": "identifier for a person at FilmTv.it",
        "data_type": "ExternalId"
    },
    "P5246": {
        "label": "Pornhub ID",
        "description": "identifier for a porn performer in the database of the Pornhub website",
        "data_type": "ExternalId"
    },
    "P5552": {
        "label": "CNRS Talent page",
        "description": "page on the Centre national de la recherche scientifique website presenting a person who has received certain prizes or medals",
        "data_type": "ExternalId"
    },
    "P5264": {
        "label": "Relationship Science person ID",
        "description": "identifier for a person in the Relationship Science database",
        "data_type": "ExternalId"
    },
    "P5068": {
        "label": "Flanders Arts Institute person ID",
        "description": "identifier of a person in the Flanders Arts Institute database for performing arts",
        "data_type": "ExternalId"
    },
    "P5073": {
        "label": "JAAF athlete ID",
        "description": "identifier for athletes in Japan Association of Athletics Federations (JAAF) database and website",
        "data_type": "ExternalId"
    },
    "P5441": {
        "label": "Encyclopaedia Herder person ID",
        "data_type": "ExternalId"
    },
    "P5078": {
        "label": "Athletics Canada ID",
        "description": "identifier for an athlete on the Athletics Canada website",
        "data_type": "ExternalId"
    },
    "P5459": {
        "label": "RHE doctor ID",
        "description": "identifier of a doctor on the database of doctoral thesis submitted in French faculties of Arts from 1808 to 1940",
        "data_type": "ExternalId"
    },
    "P5628": {
        "label": "Football.it female player ID",
        "description": "identifier for a female association football (soccer) player on the femminile.football.it website",
        "data_type": "ExternalId"
    },
    "P5259": {
        "label": "Swedish Gravestone ID",
        "description": "register of +348 000 Swedish gravestones for +759,000 Swedish people",
        "data_type": "ExternalId"
    },
    "P5325": {
        "label": "Swedish Academy member ID",
        "description": "identifier of a person, in the register of members of the Swedish Academy",
        "data_type": "ExternalId"
    },
    "P5493": {
        "label": "EDIT16 catalogue printer ID",
        "description": "identifier of the EDIT16 catalogue about 16th-century printers",
        "data_type": "ExternalId"
    },
    "P5476": {
        "label": "LegaBasket.it player ID",
        "description": "identifier for a player at legabasket.it",
        "data_type": "ExternalId"
    },
    "P5222": {
        "label": "National Women's Soccer League player ID",
        "description": "identifier for a National Women's Soccer League player at nwslsoccer.com",
        "data_type": "ExternalId"
    },
    "P5293": {
        "label": "Akout ID",
        "description": "identifier for an artist on the Akout website",
        "data_type": "ExternalId"
    },
    "P5297": {
        "label": "Companies House officer ID",
        "description": "identifier for an officer (natural person or legal person) of companies registered with Companies House in the United Kingdom",
        "data_type": "ExternalId"
    },
    "P5267": {
        "label": "YouPorn ID",
        "description": "identifier for a porn performer in the database of the YouPorn website",
        "data_type": "ExternalId"
    },
    "P5503": {
        "label": "Poetry Society of America poet ID",
        "description": "identifier for a poet on the Poetry Society of America website",
        "data_type": "ExternalId"
    },
    "P5378": {
        "label": "Akadem person ID",
        "description": "identifier of a person on Akadem",
        "data_type": "ExternalId"
    },
    "P5320": {
        "label": "IUF member ID",
        "data_type": "ExternalId"
    },
    "P5224": {
        "label": "OlimpBase Women's Chess Olympiad player ID",
        "description": "identifier for a player at www.olimpbase.org who took part in the Women's Chess Olympiad",
        "data_type": "ExternalId"
    },
    "P5103": {
        "label": "FIS speed skier ID",
        "description": "identifier for a speed skier, in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P5301": {
        "label": "France Culture person ID",
        "description": "identifier for a person on the France Culture website",
        "data_type": "ExternalId"
    },
    "P5120": {
        "label": "TFRRS athlete ID",
        "description": "identifier for an athlete on the TFRRS track and field and cross country results website",
        "data_type": "ExternalId"
    },
    "P5316": {
        "label": "Swedish Signaturer.se ID",
        "description": "the creator of ceramics and porcelain is described on signature.se and what stamps were used",
        "data_type": "ExternalId"
    },
    "P5298": {
        "label": "Webb-site person ID",
        "description": "identifier for a person on \"Webb-site Who's Who\"",
        "data_type": "ExternalId"
    },
    "P5340": {
        "label": "American Film Institute person ID",
        "description": "identifier for a person at American Film Institute",
        "data_type": "ExternalId"
    },
    "P5421": {
        "label": "Trading Card Database person ID",
        "description": "identifier for a person or fictional character in the Trading Card Database (TCDb)",
        "data_type": "ExternalId"
    },
    "P5339": {
        "label": "Lega Serie A soccer player ID",
        "description": "identifier of a soccer player in the Lega Serie A website",
        "data_type": "ExternalId"
    },
    "P5283": {
        "label": "Strava ID of a professional sport person",
        "description": "Wikidata property for authority control for sports people",
        "data_type": "ExternalId"
    },
    "P5615": {
        "label": "Centre national du théâtre ID",
        "description": "identifier for a person on the Centre national du théâtre website",
        "data_type": "ExternalId"
    },
    "P5284": {
        "label": "Douban movie celebrity ID",
        "description": "identifier for a celebrity at the Douban movie website",
        "data_type": "ExternalId"
    },
    "P5415": {
        "label": "Whonamedit? doctor ID",
        "description": "identifier for a doctor on the Whonamedit? website",
        "data_type": "ExternalId"
    },
    "P5376": {
        "label": "Medicina author ID",
        "description": "identifier for an author in the Medicina database",
        "data_type": "ExternalId"
    },
    "P5243": {
        "label": "Canal-U person ID",
        "description": "identifier of a person on Canal-U",
        "data_type": "ExternalId"
    },
    "P5306": {
        "label": "LONSEA ID",
        "description": "identifier for a person on the LONSEA site",
        "data_type": "ExternalId"
    },
    "P5309": {
        "label": "Drouot artist ID",
        "description": "identifier for an artist on the Drouot website",
        "data_type": "ExternalId"
    },
    "P5375": {
        "label": "BIU Santé person ID",
        "description": "identifier for a person on the Bibliothèque interuniversitaire de santé website",
        "data_type": "ExternalId"
    },
    "P5058": {
        "label": "Encyklopedia Teatru Polskiego person ID",
        "description": "identifier of a person in Polish theatre database e-teatr.pl",
        "data_type": "ExternalId"
    },
    "P5443": {
        "label": "Collège de France professor ID",
        "description": "identifier of a professor at the 'Collège de France' on its website",
        "data_type": "ExternalId"
    },
    "P5370": {
        "label": "Entomologists of the World ID",
        "description": "identifier for an entry in the Biographies of the Entomologists of the World online database",
        "data_type": "ExternalId"
    },
    "P5329": {
        "label": "ARMB member ID",
        "description": "Royal Academy of Medicine of Belgium member ID",
        "data_type": "ExternalId"
    },
    "P5374": {
        "label": "Académie des sciences d'outre-mer member ID",
        "description": "identifier for a member on the Académie des sciences d'outre-mer website",
        "data_type": "ExternalId"
    },
    "P5372": {
        "label": "Dictionnaire critique des historiens de l'art ID",
        "description": "identifier on an art historian on the online Dictionnaire critique des historiens de l'art actifs en France de la Révolution à la Première Guerre mondiale",
        "data_type": "ExternalId"
    },
    "P5390": {
        "label": "Classiques des sciences sociales ID",
        "description": "identifier on the Classiques des sciences sociales website",
        "data_type": "ExternalId"
    },
    "P5362": {
        "label": "ASMP member ID",
        "description": "identifier of a member of the 'Académie des Sciences Morales et Politiques' on its website",
        "data_type": "ExternalId"
    },
    "P5038": {
        "label": "Fogis.se player ID",
        "description": "identifier for a football (soccer) player's page at Fogis.se, a Swedish Football Association database",
        "data_type": "ExternalId"
    },
    "P5373": {
        "label": "École normale alumnus ID",
        "description": "identifier of an alumnus on the Dictionnaire prosopographique des élèves nommés à l'École normale de l'an III",
        "data_type": "ExternalId"
    },
    "P4862": {
        "label": "Amazon author ID",
        "description": "author identifier on Amazon.com",
        "data_type": "ExternalId"
    },
    "P4400": {
        "label": "Memória Globo ID",
        "description": "identifier for pages on the history of Brazilian TV network Rede Globo, researched by a team of journalists, historians and anthropologists",
        "data_type": "ExternalId"
    },
    "P4284": {
        "label": "Nominis given name ID",
        "description": "given name identifier in Nominis database",
        "data_type": "ExternalId"
    },
    "P4801": {
        "label": "LoC and MARC vocabularies ID",
        "description": "identifier for an item in one of the controlled vocabularies maintained by the Library of Congress",
        "data_type": "ExternalId"
    },
    "P4735": {
        "label": "IBM coded character set ID",
        "description": "identifier of a coded character set assigned by IBM",
        "data_type": "ExternalId"
    },
    "P4676": {
        "label": "MAVISE TV channel ID",
        "description": "identifier for a TV channel in the MAVISE database of the European Audiovisual Observatory",
        "data_type": "ExternalId"
    },
    "P4859": {
        "label": "Atarimania identifier",
        "description": "identifier in the Atarimania database of Atari videogames",
        "data_type": "ExternalId"
    },
    "P4932": {
        "label": "Todotango.com work ID",
        "description": "identifier for a song or composition from Todotango.com",
        "data_type": "ExternalId"
    },
    "P4842": {
        "label": "ASHRAE refrigerant number",
        "description": "identifier of a refrigerant assigned in ANSI/ASHRAE standard 34",
        "data_type": "ExternalId"
    },
    "P4858": {
        "label": "Atari Legend identifier",
        "description": "identifier in the Atari Legend database of Atari videogames",
        "data_type": "ExternalId"
    },
    "P4540": {
        "label": "IBTrACS cyclone ID",
        "description": "identifier for a tropical cyclone, issued by the World Data Center for Meteorology",
        "data_type": "ExternalId"
    },
    "P4342": {
        "label": "Store norske leksikon ID",
        "description": "identifier of an article in the online encyclopedia snl.no",
        "data_type": "ExternalId"
    },
    "P4528": {
        "label": "NVE Drainage Basin (REGINE) ID",
        "description": "identifier for water system areas in Norway by the Norwegian Water Resources and Energy Directorate",
        "data_type": "ExternalId"
    },
    "P4354": {
        "label": "search formatter URL",
        "description": "web page search URL; URI template from which \"$1\" can be automatically replaced with the string to be searched for",
        "data_type": "String"
    },
    "P4827": {
        "label": "Swimrankings.net meet ID",
        "description": "Property for swimmeets from swimrankings.net",
        "data_type": "ExternalId"
    },
    "P4532": {
        "label": "Survey of Scottish Witchcraft - Trial ID",
        "description": "іdentifier for trials in the Survey of Scottish Witchcraft database.",
        "data_type": "ExternalId"
    },
    "P4596": {
        "label": "NIOSH Numbered Publication ID",
        "description": "identifier for an official publication of the U.S. National Institute of Safety and Health (NIOSH)",
        "data_type": "ExternalId"
    },
    "P4344": {
        "label": "QEdu ID",
        "description": "identifier for a Brazilian educational institution, in the QEdu database",
        "data_type": "ExternalId"
    },
    "P4723": {
        "label": "GSMArena phone ID",
        "description": "identifier for a telephone model on GSMArena",
        "data_type": "ExternalId"
    },
    "P4484": {
        "label": "INDUCKS issue ID",
        "description": "identifier for an issue of a printed publication of the Disney Universe",
        "data_type": "ExternalId"
    },
    "P4806": {
        "label": "Gaming-History ID",
        "description": "identifier for the Gaming-History videogame database",
        "data_type": "ExternalId"
    },
    "P4845": {
        "label": "Bulbapedia article",
        "description": "article name on Bulbapedia, a Pokémon encyclopedia",
        "data_type": "ExternalId"
    },
    "P4870": {
        "label": "GEPRIS project ID",
        "description": "Identifier of a project in GEPRIS database of funded research projects",
        "data_type": "ExternalId"
    },
    "P4644": {
        "label": "ESCO skill ID",
        "description": "identifier for a Skill, in the European multilingual database ESCO v1",
        "data_type": "ExternalId"
    },
    "P4901": {
        "label": "Zenodo ID",
        "description": "ID of a work in Zenodo repository",
        "data_type": "ExternalId"
    },
    "P4894": {
        "label": "ACE Repertory work ID",
        "description": "registration identifier for a composition at ACE Repertory",
        "data_type": "ExternalId"
    },
    "P4926": {
        "label": "PRotein Ontology ID",
        "description": "Identifier for an entry in the PRotein Ontology (PRO)",
        "data_type": "ExternalId"
    },
    "P4921": {
        "label": "Snooker.org tournament ID",
        "description": "ID about a specific snooker tournament at the Snooker.org database",
        "data_type": "ExternalId"
    },
    "P4899": {
        "label": "Brooklyn Museum Exhibition ID",
        "description": "id of an exhibition in the Brooklyn Museum's \"opencollection\" subwebsite",
        "data_type": "ExternalId"
    },
    "P4832": {
        "label": "World Sailing regatta ID",
        "description": "Property for sailing regattas ID from sailing.com",
        "data_type": "ExternalId"
    },
    "P4670": {
        "label": "Sjukvårdsrådgivningen Category ID",
        "description": "identifier for a disease or medical condition, in the Swedish government's 'Sjukvårdsrådgivningen' database",
        "data_type": "ExternalId"
    },
    "P4652": {
        "label": "ESCO Occupation ID",
        "description": "identifier for an occupation, in the European multilingual database ESCO v1",
        "data_type": "ExternalId"
    },
    "P4888": {
        "label": "AFL Tables umpire ID",
        "description": "ID for a VFL/AFL umpire at afltables.com",
        "data_type": "ExternalId"
    },
    "P4964": {
        "label": "SPLASH",
        "description": "hashed identifier for mass spectra",
        "data_type": "ExternalId"
    },
    "P4917": {
        "label": "Gamebase64 identifier",
        "description": "identifier in the Gamebase64 database of Commodore 64 videogames",
        "data_type": "ExternalId"
    },
    "P4961": {
        "label": "Sega8bit.com identifier",
        "description": "identifier in the Sega8bit.com database of Master System video games",
        "data_type": "ExternalId"
    },
    "P4935": {
        "label": "Small Monuments of Plzeň Catalogue ID",
        "description": "identifier for small monuments in Pilsen, Czech Republic",
        "data_type": "ExternalId"
    },
    "P4833": {
        "label": "Sailboatdata ID",
        "description": "identifier for a sailboat design on sailboatdata.com",
        "data_type": "ExternalId"
    },
    "P4477": {
        "label": "Humble Store ID",
        "description": "identifier for an application sold on the Humble Store",
        "data_type": "ExternalId"
    },
    "P4598": {
        "label": "FAPESP researcher ID",
        "description": "identifier for researchers funded by the Brazilian research education and innovation foundation, FAPESP",
        "data_type": "ExternalId"
    },
    "P4637": {
        "label": "FoodEx2 code",
        "description": "identifier within the FoodEx2 (version 2 of the EFSA Food classification and description system for exposure assessment) food taxonomy",
        "data_type": "ExternalId"
    },
    "P4655": {
        "label": "Flathub ID",
        "description": "ID of an application on Flathub, the Flatpak repository.",
        "data_type": "ExternalId"
    },
    "P4956": {
        "label": "BDH edition ID",
        "description": "identifier of an edition on the Biblioteca Digital Hispánica, including the prefix \"bdh\"",
        "data_type": "ExternalId"
    },
    "P4919": {
        "label": "Plus/4 World ID",
        "description": "identifier in the Plus/4 World database of Commodore videogames",
        "data_type": "ExternalId"
    },
    "P4840": {
        "label": "Directory of Czech publishers ID",
        "description": "identifier of a Czech publisher, in the Czech National Library database",
        "data_type": "ExternalId"
    },
    "P4973": {
        "label": "Wikitribune category",
        "description": "name of a topic (category) on Wikitribune",
        "data_type": "ExternalId"
    },
    "P4860": {
        "label": "BMI work ID",
        "description": "registration identifier for a composition at BMI",
        "data_type": "ExternalId"
    },
    "P4746": {
        "label": "Elhuyar ZTH ID",
        "description": "Elhuyar Science and Technology Dictionary has a wide range of terms in four languages (en, es, fr, eu) that can easily be linked to and from Wikidata",
        "data_type": "ExternalId"
    },
    "P4924": {
        "label": "CueTracker tournament ID",
        "description": "ID about a specific snooker tournament at the CueTracker.net database",
        "data_type": "ExternalId"
    },
    "P4916": {
        "label": "Ready64 ID",
        "description": "identifier in the Ready64 database of Commodore 64 videogames",
        "data_type": "ExternalId"
    },
    "P4495": {
        "label": "Xenopus Anatomical Ontology ID",
        "description": "identifier for an anatomical structure in the Xenopus Anatomical Ontology, a controlled vocubulary for describing Xenopus anatomy and embryological development",
        "data_type": "ExternalId"
    },
    "P4959": {
        "label": "paris.fr equipment ID",
        "description": "identifier for an equipment in Paris on the equipement.paris.fr website",
        "data_type": "ExternalId"
    },
    "P4847": {
        "label": "CPC-Power ID",
        "description": "identifier in the CPC-Power database of Amstrad CPC videogames",
        "data_type": "ExternalId"
    },
    "P4736": {
        "label": "IBM graphic character global ID",
        "description": "identifier of a character/grapheme in the IBM graphic character identification system",
        "data_type": "ExternalId"
    },
    "P4948": {
        "label": "DDB ID",
        "description": "identifier for items in Deutsche Digitale Bibliothek",
        "data_type": "ExternalId"
    },
    "P4834": {
        "label": "Deutsche Synchronkartei series ID",
        "description": "identifier for a TV series in the synchronisation (dubbing) database Deutsche Synchronkartei",
        "data_type": "ExternalId"
    },
    "P4766": {
        "label": "Australian honours ID",
        "description": "ID for an instance of an Australian honour being awarded to an Australian citizen",
        "data_type": "ExternalId"
    },
    "P4776": {
        "label": "MAC Address Block Large ID",
        "description": "unique identifier of an organisation registered with the IEEE",
        "data_type": "String"
    },
    "P4960": {
        "label": "Generation MSX identifier",
        "description": "identifier in the Generation MSX database of MSX videogames",
        "data_type": "ExternalId"
    },
    "P4480": {
        "label": "INDUCKS publication ID",
        "description": "identifier for printed publications of the Disney Universe",
        "data_type": "ExternalId"
    },
    "P4482": {
        "label": "INDUCKS publisher ID",
        "description": "identifier for publishers of printed publications related to the Disney Universe",
        "data_type": "ExternalId"
    },
    "P4953": {
        "label": "Library of Congress Genre/Form Terms ID",
        "description": "ID in the Library of Congress controlled vocabulary for genres and forms",
        "data_type": "ExternalId"
    },
    "P4986": {
        "label": "Routard.com place ID",
        "description": "identifier for a place on the Guide du routard website",
        "data_type": "ExternalId"
    },
    "P4506": {
        "label": "Formal Public Identifier",
        "description": "string that identifies uniquely an item using a Formal Public Identifier according to the ISO/IEC 9070:1991 standard",
        "data_type": "String"
    },
    "P4734": {
        "label": "IBM code page ID",
        "description": "identifier of a code page of characters assigned by IBM",
        "data_type": "ExternalId"
    },
    "P4853": {
        "label": "JMPR database ID",
        "description": "identifier of a pesticide in the JMPR database",
        "data_type": "ExternalId"
    },
    "P4359": {
        "label": "gravsted.dk ID",
        "description": "identifier for a person giving their burial place, in gravsted.dk",
        "data_type": "ExternalId"
    },
    "P4965": {
        "label": "Behind The Voice Actors video game ID",
        "description": "identifier for video games on the website Behind The Voice Actors",
        "data_type": "ExternalId"
    },
    "P4962": {
        "label": "Abandonia identifier",
        "description": "identifier in the Abandonia database of DOS video games",
        "data_type": "ExternalId"
    },
    "P4756": {
        "label": "V Live channel ID",
        "description": "identifier for the V Live channel of a person, musical group or organisation",
        "data_type": "ExternalId"
    },
    "P4399": {
        "label": "Itaú Cultural ID",
        "description": "unique identifier for a entity in the Itaú Cultural Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P4287": {
        "label": "Riigikogu ID",
        "description": "identifier for a member of the Estonian Parliament",
        "data_type": "ExternalId"
    },
    "P4288": {
        "label": "UK National Fruit Collection ID",
        "description": "identifier of a fruit (plant) variety or cultivar, in the United Kingdom National Fruit Collection",
        "data_type": "ExternalId"
    },
    "P4677": {
        "label": "MAVISE on-demand audiovisual service ID",
        "description": "identifier for an on-demand audiovisual service in the MAVISE database of the European Audiovisual Observatory",
        "data_type": "ExternalId"
    },
    "P4685": {
        "label": "Nintendo of America Game Store ID (former scheme)",
        "description": "identifier for software sold on the Nintendo of America Game Store",
        "data_type": "ExternalId"
    },
    "P4597": {
        "label": "FAPESP institution ID",
        "description": "identifier for institutions funded by the Brazilian research education and innovation foundation, FAPESP",
        "data_type": "ExternalId"
    },
    "P4857": {
        "label": "AtariAge ID",
        "description": "identifier in the AtariAge database of Atari videogames",
        "data_type": "ExternalId"
    },
    "P4671": {
        "label": "Hall of Light ID",
        "description": "identifier in the database of Amiga video games Hall of Light",
        "data_type": "ExternalId"
    },
    "P4846": {
        "label": "Lemon Amiga identifier",
        "description": "identifier in the Lemon Amiga database of Amiga videogames",
        "data_type": "ExternalId"
    },
    "P4732": {
        "label": "IUPAC Gold Book ID",
        "description": "identifier of definition in ''Compendium of Chemical Terminology'' (''IUPAC GoldBook'')",
        "data_type": "ExternalId"
    },
    "P4487": {
        "label": "Camera Decision ID",
        "description": "identifier for a camera, on the Camera Decision website",
        "data_type": "ExternalId"
    },
    "P4313": {
        "label": "musée de Bretagne identifiant collections ID",
        "description": "identifier for an object in the collection of the Brittany Museum, France",
        "data_type": "ExternalId"
    },
    "P4338": {
        "label": "LOINC ID",
        "description": "identifier for medical observations, measurements, and documents in the Regenstrief Institute's Logical Observation Identifiers Names and Codes (LOINC), a database of internationalized medical terminology",
        "data_type": "ExternalId"
    },
    "P4889": {
        "label": "MuIS person or group ID",
        "description": "identifier for a person or group in the Estonian museum database MuIS",
        "data_type": "ExternalId"
    },
    "P5001": {
        "label": "METAR code",
        "description": "METeorological Aerodrome Report",
        "data_type": "ExternalId"
    },
    "P4699": {
        "label": "Canal-U channel ID",
        "description": "ID of a channel on Canal-U, the online video library of French Higher Education",
        "data_type": "ExternalId"
    },
    "P4517": {
        "label": "ctext work ID",
        "description": "identifier for a work, in the ctext database. ctext is a database of pre-modern texts in Chinese languages (works and editions have separate identifiers)",
        "data_type": "ExternalId"
    },
    "P4989": {
        "label": "RKDlibrary ID",
        "description": "record for a publication held by the RKD-Netherlands Institute for Art History in the Hague",
        "data_type": "ExternalId"
    },
    "P4946": {
        "label": "Library of Congress Demographic Group Terms ID",
        "description": "identifier for a term in the Library of Congress Demographic Group Terms controlled vocabulary",
        "data_type": "ExternalId"
    },
    "P4849": {
        "label": "International Numbering System number",
        "description": "unique identifier of a food additive in the International Numbering System",
        "data_type": "ExternalId"
    },
    "P4852": {
        "label": "JECFA database ID",
        "description": "identifier of a flavour, food additive, contaminant, toxicant or veterinary drug in the JECFA database",
        "data_type": "ExternalId"
    },
    "P4471": {
        "label": "Rush Parliamentary Archive ID",
        "description": "identifier in the database of British members of parliament, 1832 onwards",
        "data_type": "ExternalId"
    },
    "P4903": {
        "label": "New Georgia Encyclopedia ID",
        "description": "identifier of a topic in the online New Georgia Encyclopedia",
        "data_type": "ExternalId"
    },
    "P4317": {
        "label": "GARD rare disease ID",
        "description": "identifier for a rare disease in the United States National Institutes of Health's Genetic and Rare Diseases (GARD) Information Center database",
        "data_type": "ExternalId"
    },
    "P4854": {
        "label": "Uppslagsverket Finland ID",
        "description": "identifier of an article in the online encyclopedia Uppslagsverket Finland",
        "data_type": "ExternalId"
    },
    "P6532": {
        "label": "has phenotype",
        "description": "a relationship that holds between a biological entity and a phenotype, equivalent to \"has property\" in the relation ontology http://purl.obolibrary.org/obo/RO_0002200",
        "data_type": "WikibaseItem"
    },
    "P6533": {
        "label": "type of front suspension",
        "description": "system of tires, tire air, springs, shock absorbers and linkages that connects a vehicle to its front wheels and allows relative motion between the two",
        "data_type": "WikibaseItem"
    },
    "P6540": {
        "label": "multi-channel network",
        "description": "multi-channel network with which works YouTube channel",
        "data_type": "WikibaseItem"
    },
    "P6524": {
        "label": "expression of",
        "description": "realization of a work through words, sound, image etc. in the FRBR model",
        "data_type": "WikibaseItem"
    },
    "P5869": {
        "label": "model item",
        "description": "defines which item is a best practice example of modelling a subject, which is described by the value of this property, usage instructions at Wikidata:Model item",
        "data_type": "WikibaseItem"
    },
    "P6339": {
        "label": "reports periodicity",
        "description": "physical quantity corresponding to the interval of time between two measurements or reported data",
        "data_type": "WikibaseItem"
    },
    "P6426": {
        "label": "RightsStatement status according to source website",
        "description": "standardized RightsStatements.org rights statement that owning institution or similar authority has applied to this work",
        "data_type": "WikibaseItem"
    },
    "P6563": {
        "label": "key performance indicator",
        "description": "measure used to track progress towards a target",
        "data_type": "WikibaseItem"
    },
    "P5706": {
        "label": "shear strength",
        "description": "strength of a material or component against the type of yield or structural failure when the material or component fails in shear",
        "data_type": "Quantity"
    },
    "P7079": {
        "label": "shrinkage",
        "description": "reduction in the size of something, or the process of becoming smaller, typically when a material return to room temperature after heating",
        "data_type": "Quantity"
    },
    "P6586": {
        "label": "Cook Partisan Voting Index",
        "description": "a measurement of how strongly a United States congressional district or state leans toward the Democratic or Republican Party, compared to the nation as a whole",
        "data_type": "String"
    },
    "P7101": {
        "label": "privacy policy URL",
        "description": "privacy policy of subject",
        "data_type": "Url"
    },
    "P5967": {
        "label": "reference vocabulary",
        "description": "the controlled vocabulary used to supply the form in which the value of a statement was given in the source being referenced",
        "data_type": "WikibaseItem"
    },
    "P6251": {
        "label": "catchphrase",
        "description": "commonly used phrase by which someone or something may be recognized",
        "data_type": "Monolingualtext"
    },
    "P6437": {
        "label": "day of regular release",
        "description": "day(s) when parts of this creative work are released, aired, published, etc",
        "data_type": "WikibaseItem"
    },
    "P7083": {
        "label": "surface roughness",
        "description": "component of surface texture quantified by the deviations in the direction of the normal vector of a real surface from its ideal form",
        "data_type": "Quantity"
    },
    "P6108": {
        "label": "IIIF manifest",
        "description": "JSON-LD formatted document URL reference described by the IIIF Presentation API",
        "data_type": "Url"
    },
    "P6497": {
        "label": "drag coefficient",
        "description": "dimensionless quantity that is used to quantify the drag or resistance of an object in a fluid environment, such as air or water",
        "data_type": "Quantity"
    },
    "P6166": {
        "label": "quotes work",
        "description": "other creative work which this work quotes from textually, verbally or lyrically",
        "data_type": "WikibaseItem"
    },
    "P5692": {
        "label": "dummy value",
        "description": "known test or sample values for identifier or scheme. Do not use for ranges reserved for internal use",
        "data_type": "String"
    },
    "P6477": {
        "label": "does not have quality",
        "description": "expected quality that the item does not possess",
        "data_type": "WikibaseItem"
    },
    "P5896": {
        "label": "trading fee",
        "description": "a trading fee or brokerage commision is fee charged when buying stock or commodities at exchanges or at a bank, use qualifier applies to part for maker and taker fee",
        "data_type": "Quantity"
    },
    "P6938": {
        "label": "seconded by",
        "description": "a person who seconds a motion as parliamentary procedure",
        "data_type": "WikibaseItem"
    },
    "P6531": {
        "label": "observed in",
        "description": "to be used as reference to location, condition, variant where upon the subject property object statement is based",
        "data_type": "WikibaseItem"
    },
    "P6875": {
        "label": "motif represents",
        "description": "object or concept from which this motif is derived, or which it represents",
        "data_type": "WikibaseItem"
    },
    "P6365": {
        "label": "member category",
        "description": "category item for members of the organization",
        "data_type": "WikibaseItem"
    },
    "P6534": {
        "label": "type of rear suspension",
        "description": "system of tires, tire air, springs, shock absorbers and linkages that connects a vehicle to its rear wheels and allows relative motion between the two",
        "data_type": "WikibaseItem"
    },
    "P6757": {
        "label": "exposure time",
        "description": "the length of time a image sensor or photographic film is exposed to light.",
        "data_type": "Quantity"
    },
    "P7160": {
        "label": "manner of inhumane treatment",
        "description": "manner of torture and other cruel, inhumane or degrading treatment or punishments as covered by the UN Convention experienced by the subject",
        "data_type": "WikibaseItem"
    },
    "P5997": {
        "label": "stated in reference as",
        "description": "for use in a reference section, to indicate how the object of the statement was stated in reference, if this is different to how the object of statement would have been referred to in the statement's original context",
        "data_type": "String"
    },
    "P5991": {
        "label": "carbon footprint",
        "description": "Total emissions caused by an individual, event, organisation, or product, expressed as carbon dioxide equivalent. Has to be calculated via a scientificly proven methodology.",
        "data_type": "Quantity"
    },
    "P5994": {
        "label": "recycling code",
        "description": "used to identify the material from which an item is made, to facilitate easier recycling or other reprocessing",
        "data_type": "String"
    },
    "P5824": {
        "label": "is retracted by",
        "description": "(article) is retracted by (retraction notice)",
        "data_type": "WikibaseItem"
    },
    "P5817": {
        "label": "state of use",
        "description": "to indicate state of use of a building, monument, etc. It is not related with its heritage protection (P1435), but to its present state of use. Ex.: abandoned, in use, temporarily closed, etc.",
        "data_type": "WikibaseItem"
    },
    "P6530": {
        "label": "has role in modeling",
        "description": "relation between a research artefact and an entity that is used to study, in virtue of its replicating or approximating features of the studied entity",
        "data_type": "WikibaseItem"
    },
    "P5753": {
        "label": "ideographic description sequences",
        "description": "method to describe composition of Han characters using ideographic description characters and character components",
        "data_type": "WikibaseItem"
    },
    "P6570": {
        "label": "number of words",
        "description": "number of words in text. Text is available at Wikisource.",
        "data_type": "Quantity"
    },
    "P6803": {
        "label": "taxa found at location",
        "description": "plant or animals taxon described as present in location, not countries or first-level subnational entities. Requires reference. Ref should cover a limited number of taxa for specific location.",
        "data_type": "WikibaseItem"
    },
    "P6541": {
        "label": "Stack Exchange site",
        "description": "link to a Stack Exchange network website for the concept",
        "data_type": "Url"
    },
    "P5993": {
        "label": "compressive modulus of elasticity",
        "description": "the ratio of mechanical stress to strain in an elastic material when that material is being compressed",
        "data_type": "Quantity"
    },
    "P5682": {
        "label": "heat deflection temperature",
        "description": "temperature at which a polymer or plastic sample deforms under a specified load",
        "data_type": "Quantity"
    },
    "P6000": {
        "label": "water footprint",
        "description": "total volume of fresh water used to produce the goods and services consumed by the individual or community or produced by the business",
        "data_type": "Quantity"
    },
    "P5708": {
        "label": "force",
        "description": "quantitative value of the interaction between two objects",
        "data_type": "Quantity"
    },
    "P6789": {
        "label": "ISO speed",
        "description": "light sensitivity of film or digital sensor (e.g. ISO/ASA 100)",
        "data_type": "Quantity"
    },
    "P5681": {
        "label": "flexural modulus",
        "description": "ratio of stress to strain in flexural deformation",
        "data_type": "Quantity"
    },
    "P5685": {
        "label": "tear resistance",
        "description": "measure of how well a material can withstand the effects of tearing",
        "data_type": "Quantity"
    },
    "P7004": {
        "label": "AtlasFor ID",
        "description": "identifier of a location or an architectural structure in the Atlas Landscapefor (Italian database of cultural heritage)",
        "data_type": "ExternalId"
    },
    "P5901": {
        "label": "official number of town quarter",
        "description": "official number for the administrative division or section, region or part of a town",
        "data_type": "String"
    },
    "P6241": {
        "label": "collection creator",
        "description": "entity (person, organization, etc.) that caused a record or collection to be produced or gathered",
        "data_type": "WikibaseItem"
    },
    "P6587": {
        "label": "league system",
        "description": "league system the league belongs to",
        "data_type": "WikibaseItem"
    },
    "P5893": {
        "label": "work hardening strain",
        "description": "the strengthening of a material by the specified ratio of plastic deformation",
        "data_type": "Quantity"
    },
    "P5769": {
        "label": "editor-in-chief",
        "description": "a publication's editorial leader who has final responsibility for its operations and policies and the selection of authors, translators, illustrators, and formats for publishing articles or books in collections or periodical papers",
        "data_type": "WikibaseItem"
    },
    "P6902": {
        "label": "era name",
        "description": "era name for rulers in East Asia",
        "data_type": "WikibaseItem"
    },
    "P6939": {
        "label": "moved by",
        "description": "a person who moves a motion as parliamentary procedure",
        "data_type": "WikibaseItem"
    },
    "P7104": {
        "label": "end of covered period",
        "description": "end of period covered by an archive set, a historical book, or a collection",
        "data_type": "Time"
    },
    "P5811": {
        "label": "elongation at break",
        "description": "material property; the ratio between changed length and initial length after breakage of the test specimen. It expresses the capability of a material to resist changes of shape without crack formation.",
        "data_type": "Quantity"
    },
    "P5778": {
        "label": "Cooperative Patent Classification code",
        "description": "patent classification code used between the European Patent Office and United States Patent and Trademark Office",
        "data_type": "String"
    },
    "P6608": {
        "label": "General Material Designation",
        "description": "Canadian archival term indicating the broad class of material to which the item belongs",
        "data_type": "WikibaseItem"
    },
    "P5810": {
        "label": "cryptocurrency symbol",
        "description": "the symbol or identifier of a cryptocurrency based on the Naming Consistency of CCXT",
        "data_type": "String"
    },
    "P6977": {
        "label": "review of",
        "description": "this property connects a review to the work it reviews",
        "data_type": "WikibaseItem"
    },
    "P7014": {
        "label": "terms of service URL",
        "description": "URL linking to terms of service of the subject",
        "data_type": "Url"
    },
    "P5816": {
        "label": "state of conservation",
        "description": "to indicate state of conservation of a building, monument, etc. It is not related with its heritage protection (P1435), but to its present state of conservation. Ex.: ruinous, demolished, correct, etc.",
        "data_type": "WikibaseItem"
    },
    "P6254": {
        "label": "subject lexeme",
        "description": "lexeme described or discussed",
        "data_type": "WikibaseLexeme"
    },
    "P5897": {
        "label": "account charge / subscription fee",
        "description": "a monthly/yearly fee for using a service or account; use qualifier duration to specify the billing cycle",
        "data_type": "Quantity"
    },
    "P5852": {
        "label": "item inherits value from",
        "description": "this value is inherited from the parent (super)class; parent class has exactly the same statement as this statement (To be used in a source field)",
        "data_type": "WikibaseItem"
    },
    "P5895": {
        "label": "foreign transaction fee",
        "description": "foreign transaction fee applies to payments in foreign currencies",
        "data_type": "Quantity"
    },
    "P5949": {
        "label": "Unicode range",
        "description": "Unicode range this writing system or font is covered by",
        "data_type": "String"
    },
    "P7007": {
        "label": "Wolfram Language unit code",
        "description": "input form for a unit of measurement in the Wolfram Language",
        "data_type": "ExternalId"
    },
    "P5898": {
        "label": "borrowing rate",
        "description": "the p.a. borrowing/lending rate",
        "data_type": "Quantity"
    },
    "P5992": {
        "label": "attenuation coefficient",
        "description": "Characterizes how easily a volume of a material can be penetrated by a beam of light, sound, particles, or other energy or matter. Beam type is to be secified using P1013.",
        "data_type": "Quantity"
    },
    "P5804": {
        "label": "has program committee member",
        "data_type": "WikibaseItem"
    },
    "P6573": {
        "label": "Klexikon article ID",
        "description": "ID of an article in Klexikon",
        "data_type": "ExternalId"
    },
    "P6529": {
        "label": "provisional house number in the Czech Republic",
        "description": "house number used for recreational and provisional buildings in Czechia, formerly also in Slovakia (eg. cottages)",
        "data_type": "String"
    },
    "P6954": {
        "label": "online access status",
        "description": "property or qualifier for an ID property indicating whether linked content is directly readable online",
        "data_type": "WikibaseItem"
    },
    "P7103": {
        "label": "start of covered period",
        "description": "start of period covered by an archive set, a historical book, or a collection",
        "data_type": "Time"
    },
    "P6568": {
        "label": "inscription mentions",
        "description": "item about a person or an object mentioned in the inscription's text. Use on Wikimedia Commons on media files",
        "data_type": "WikibaseItem"
    },
    "P5899": {
        "label": "interest rate",
        "description": "The yearly interest rate",
        "data_type": "Quantity"
    },
    "P5947": {
        "label": "Vicat softening point",
        "description": "the temperature at which the specimen is penetrated to a depth of 1 mm by a flat-ended needle with a 1 mm2 circular or square cross-section",
        "data_type": "Quantity"
    },
    "P5828": {
        "label": "seed",
        "description": "seeding information of a participant of a sports event",
        "data_type": "WikibaseItem"
    },
    "P7137": {
        "label": "acknowledged",
        "description": "persons or institutions acknowledged on a creative work",
        "data_type": "WikibaseItem"
    },
    "P7126": {
        "label": "Mac OS type code",
        "description": "code used formerly by Mac OS to identify file types and programs",
        "data_type": "String"
    },
    "P7069": {
        "label": "IAAF competition category",
        "description": "competition quality ranking assigned by the International Association of Athletics Federations used for comparison and future Olympic qualification",
        "data_type": "String"
    },
    "P6107": {
        "label": "winterthur-glossar URL",
        "description": "URL of the object in the Winterthur Glossar",
        "data_type": "Url"
    },
    "P7122": {
        "label": "opinion joined by",
        "description": "who joins a judicial opinion",
        "data_type": "WikibaseItem"
    },
    "P6237": {
        "label": "real estate developer",
        "description": "person or organization responsible for building this item",
        "data_type": "WikibaseItem"
    },
    "P7080": {
        "label": "specific absorption rate",
        "description": "measure of the maximum energy absorbed by a unit of mass of exposed tissue of a person using a mobile phone, over a given time or more simply the power absorbed per unit mass",
        "data_type": "Quantity"
    },
    "P6440": {
        "label": "maintenance method",
        "description": "method or steps that maintain, clean up, or repair this item and lengthens its lifespan",
        "data_type": "WikibaseItem"
    },
    "P6711": {
        "label": "data transfer speed",
        "description": "transfer speed through a bus or a communication medium",
        "data_type": "Quantity"
    },
    "P5709": {
        "label": "abrasion resistance",
        "description": "resistance to the process of scuffing, scratching, wearing down, marring, or rubbing away",
        "data_type": "Quantity"
    },
    "P6790": {
        "label": "f-number",
        "description": "ratio of the system's focal length to the diameter of the entrance pupil. Sample: 4 (generally written \"ƒ/4\") for a lens with 200 mm focal length using an entrance pupil diameter of 50 mm.",
        "data_type": "Quantity"
    },
    "P7045": {
        "label": "chassis",
        "description": "chassis type for cars, buses, trucks, etc",
        "data_type": "WikibaseItem"
    },
    "P6338": {
        "label": "colorist",
        "description": "artist in charge of adding colors to drawings, comics, movie film, etc.",
        "data_type": "WikibaseItem"
    },
    "P6224": {
        "label": "level of description",
        "description": "position of the unit of description in the hierarchy of the fonds",
        "data_type": "WikibaseItem"
    },
    "P6710": {
        "label": "linear reference",
        "description": "nominal distance commonly used to describe location along a route. Defined by operator/authority, it might not reflect physical distance",
        "data_type": "Quantity"
    },
    "P6720": {
        "label": "interwiki prefix at Wikimedia",
        "description": "prefix used for the website on Wikimedia Foundation wikis. Qualify with \"formatter url\" (P1630) from Special:Interwiki. If the url changes, add the qualifier \"end date\" and create a new statement with new formatter.",
        "data_type": "ExternalId"
    },
    "P5805": {
        "label": "OSI Model layer location",
        "description": "location of a network protocol in one of the OSI Model layers",
        "data_type": "WikibaseItem"
    },
    "P6510": {
        "label": "stall speed",
        "description": "minimum airspeed from which an aircraft can take off and maintain flight",
        "data_type": "Quantity"
    },
    "P6427": {
        "label": "trading name",
        "description": "alternative name under which an entity legally trades",
        "data_type": "Monolingualtext"
    },
    "P6014": {
        "label": "flexural strain at break",
        "description": "the strain of material specimen after break in a flexural test",
        "data_type": "Quantity"
    },
    "P7081": {
        "label": "quotation or excerpt",
        "description": "quotation or excerpt from this work. No quotation marks needed",
        "data_type": "Monolingualtext"
    },
    "P6146": {
        "label": "The Hendon Mob ID",
        "description": "identifier for a poker player in The Hendon Mob database",
        "data_type": "ExternalId"
    },
    "P5909": {
        "label": "HKCAN ID",
        "description": "Hong Kong Chinese Authority (Name) ID",
        "data_type": "ExternalId"
    },
    "P6302": {
        "label": "Dictionnaire de spiritualité ID",
        "description": "identifier for a topic on the ''Dictionnaire de spiritualité''",
        "data_type": "ExternalId"
    },
    "P6308": {
        "label": "Scoresway tennis person ID",
        "description": "identifier for a tennis player at the Scoresway",
        "data_type": "ExternalId"
    },
    "P6178": {
        "label": "Dimensions author ID",
        "description": "unique identifier for an author in Dimensions",
        "data_type": "ExternalId"
    },
    "P6198": {
        "label": "Liga MX Femenil player ID",
        "description": "official profile of a female association football (soccer) player from Liga MX Femenil, the top level Mexican league for women",
        "data_type": "ExternalId"
    },
    "P6100": {
        "label": "YÖK Academic Profile ID",
        "description": "identifer for an academic, in the Turkish 'YÖK' database",
        "data_type": "ExternalId"
    },
    "P6223": {
        "label": "L'Encyclopédie philosophique ID",
        "description": "identifier on 'L'Encyclopédie philosophique'",
        "data_type": "ExternalId"
    },
    "P6154": {
        "label": "r-hockey.ru person ID",
        "description": "identifier for a player of staff member at the r-hockey.ru",
        "data_type": "ExternalId"
    },
    "P6171": {
        "label": "Académie d'Arles member ID",
        "description": "identifier for a member of the 'Académie d'Arles' on its website",
        "data_type": "ExternalId"
    },
    "P5721": {
        "label": "Turkish Basketball Super League ID",
        "description": "identifier for a Basketbol Süper Ligi player",
        "data_type": "ExternalId"
    },
    "P5715": {
        "label": "Academia.edu profile URL",
        "description": "URL for a person's profile on the Academia.edu website",
        "data_type": "Url"
    },
    "P6169": {
        "label": "Welsh Book Trade Info ID",
        "description": "identifier for person, business or organisation involved in the production and distribution of books in Wales",
        "data_type": "ExternalId"
    },
    "P6316": {
        "label": "PlaymakerStats.com coach ID",
        "description": "identifier for an association football (soccer) coach at PlaymakerStats.com (formerly TheFinalBall.com)",
        "data_type": "ExternalId"
    },
    "P5731": {
        "label": "Angelicum ID",
        "description": "identifier for an entity in the online catalogue of the Pontifical University of Saint Thomas Aquinas",
        "data_type": "ExternalId"
    },
    "P6231": {
        "label": "BDELIS ID",
        "description": "identifier for a person on the 'Database of Swiss elites in the 20th century'",
        "data_type": "ExternalId"
    },
    "P6130": {
        "label": "Littera ID",
        "description": "Littera identifier related to medieval Galician-Portuguese music (cantigas)",
        "data_type": "ExternalId"
    },
    "P5952": {
        "label": "Annuaire de la magistrature ID",
        "description": "identifier for a French magistrature on the 'Annuaire rétrospectif de la magistrature'",
        "data_type": "ExternalId"
    },
    "P6109": {
        "label": "Danish Football Union player ID",
        "description": "identifier for Danish national team player's page at the official website of the Danish Football Association (in Danish: Dansk Boldspil-Union or DBU)",
        "data_type": "ExternalId"
    },
    "P5788": {
        "label": "The Gymternet gymnast ID",
        "description": "identifier for a gymnast on the Gymternet website",
        "data_type": "ExternalId"
    },
    "P6062": {
        "label": "Scoresway baseball person ID",
        "description": "identifier for a baseball person at the Scoresway",
        "data_type": "ExternalId"
    },
    "P5724": {
        "label": "Basketball Bundesliga ID",
        "description": "identifier for a basketball player on the Basketball Bundesliga website",
        "data_type": "ExternalId"
    },
    "P6188": {
        "label": "BDFA player ID",
        "description": "player ID on the Argentine Football Database website",
        "data_type": "ExternalId"
    },
    "P6253": {
        "label": "Beachsoccer.ru player ID",
        "description": "identifier for a player at the St. Petersburg Beach Soccer Federation site",
        "data_type": "ExternalId"
    },
    "P5739": {
        "label": "Pontificia Università della Santa Croce ID",
        "description": "identifier for an entity in the online catalogue of the Pontifical University of the Holy Cross",
        "data_type": "ExternalId"
    },
    "P6063": {
        "label": "Scoresway basketball person ID",
        "description": "identifier for a basketball person at the Scoresway",
        "data_type": "ExternalId"
    },
    "P6206": {
        "label": "Guida al Fumetto Italiano ID",
        "description": "identifier for a person on the Guida al Fumetto Italiano website",
        "data_type": "ExternalId"
    },
    "P5772": {
        "label": "coach ID at the Russian Biathlon DB",
        "description": "identifier for a coach at the Russian Biathlon DB",
        "data_type": "ExternalId"
    },
    "P5657": {
        "label": "National Humanities Medal winner ID",
        "description": "identifier for a National Humanities Medal winner on the National Endowment for the Humanities website",
        "data_type": "ExternalId"
    },
    "P6304": {
        "label": "researchportal.helsinki.fi profile ID",
        "data_type": "ExternalId"
    },
    "P6083": {
        "label": "Cineuropa person ID",
        "description": "identifier for a person at Cineuropa",
        "data_type": "ExternalId"
    },
    "P6225": {
        "label": "TrENSmissions person ID",
        "description": "identifier for a person on 'TrENSmissions'",
        "data_type": "ExternalId"
    },
    "P6150": {
        "label": "Academy Awards Database nominee ID",
        "description": "identifier for nominees in the official Academy Awards database",
        "data_type": "ExternalId"
    },
    "P5664": {
        "label": "Savoirs ENS ID",
        "description": "identifier for a lecturer on Savoirs ENS",
        "data_type": "ExternalId"
    },
    "P5732": {
        "label": "bgm.tv subject identifier",
        "description": "identifier for creative works sorted on bgm.tv (Bangumi/Chobits)",
        "data_type": "ExternalId"
    },
    "P5723": {
        "label": "Polish Basketball League ID",
        "description": "identifier for a basketball player on the Polish Basketball League website",
        "data_type": "ExternalId"
    },
    "P5958": {
        "label": "Women's Japan Basketball League ID",
        "description": "Women's Japan Basketball League",
        "data_type": "ExternalId"
    },
    "P6232": {
        "label": "Danske Taler speaker ID",
        "description": "Identifier for speaker on the ''Danske Taler'' website",
        "data_type": "ExternalId"
    },
    "P5663": {
        "label": "Académie de Versailles member ID",
        "description": "identifier for a member of the 'Académie de Versailles' on its website",
        "data_type": "ExternalId"
    },
    "P5716": {
        "label": "2006 Commonwealth Games athlete ID",
        "description": "identifier for an athlete who participated at the 2006 Commonwealth Games",
        "data_type": "ExternalId"
    },
    "P5725": {
        "label": "Baloncesto Superior Nacional ID",
        "description": "identifier for a basketball player on the Baloncesto Superior Nacional website",
        "data_type": "ExternalId"
    },
    "P5717": {
        "label": "TPA person ID",
        "description": "identifier for a person on the 'Théâtres parisiens associés' website",
        "data_type": "ExternalId"
    },
    "P6005": {
        "label": "Muck Rack journalist ID",
        "description": "identifier for a journalist at the website Muck Rack",
        "data_type": "ExternalId"
    },
    "P6290": {
        "label": "Penthouse ID",
        "description": "identifier for a model on the Penthouse website PenthouseGold.com",
        "data_type": "ExternalId"
    },
    "P6313": {
        "label": "Russian Premier League referee ID",
        "description": "identifier for a Russian Premier League referee at official website premierliga.ru",
        "data_type": "ExternalId"
    },
    "P6317": {
        "label": "RusTeam player ID",
        "description": "identifier for football player on site RusTeam.permian.ru",
        "data_type": "ExternalId"
    },
    "P6235": {
        "label": "ARB person ID",
        "description": "identifier for a person (including members) on the website of the 'Académie royale de Belgique'",
        "data_type": "ExternalId"
    },
    "P5661": {
        "label": "Académie de Montpellier member ID",
        "description": "identifier for a member of the 'Académie de Montpellier' on its website",
        "data_type": "ExternalId"
    },
    "P5662": {
        "label": "Académie de Mâcon member ID",
        "description": "identifier for a member of the 'Académie de Mâcon' on its website",
        "data_type": "ExternalId"
    },
    "P5647": {
        "label": "Baidu ScholarID",
        "description": "identifier for a scholar at the Baidu Scholar website",
        "data_type": "ExternalId"
    },
    "P5722": {
        "label": "Greek Basket League ID",
        "description": "identifier for a basketball player on the Greek Basket League website",
        "data_type": "ExternalId"
    },
    "P5776": {
        "label": "Arnet Miner author ID",
        "description": "author identifier from the Arnet Miner research database",
        "data_type": "ExternalId"
    },
    "P5771": {
        "label": "Russian Biathlon DB athlete ID",
        "description": "identifier for an athlete at the Russian Biathlon DB",
        "data_type": "ExternalId"
    },
    "P5964": {
        "label": "Norwegian Nationaltheatret actor ID",
        "description": "identifikator for an actor at the Norwegian national theatre",
        "data_type": "ExternalId"
    },
    "P5756": {
        "label": "ABMC person ID",
        "description": "identifier for a person on the American Battle Monuments Commission website",
        "data_type": "ExternalId"
    },
    "P6264": {
        "label": "Harvard Index of Botanists ID",
        "description": "numerical identifier for a person in the Harvard Index of Botanists",
        "data_type": "ExternalId"
    },
    "P6255": {
        "label": "SinemaTürk person ID",
        "description": "identifier for a person in the SinemaTürk database",
        "data_type": "ExternalId"
    },
    "P6024": {
        "label": "Clé des langues ID",
        "description": "identifier for an author on 'La Clé des langues'",
        "data_type": "ExternalId"
    },
    "P6023": {
        "label": "ResearchGate contributions ID",
        "description": "identifier for a non-registered author on ResearchGate",
        "data_type": "ExternalId"
    },
    "P6065": {
        "label": "Scoresway rugby person ID",
        "description": "identifier for a rugby person at the Scoresway",
        "data_type": "ExternalId"
    },
    "P6252": {
        "label": "BSRussia player ID",
        "description": "identifier for a person at the Beach Soccer Russia site",
        "data_type": "ExternalId"
    },
    "P6037": {
        "label": "ProsopoMaths ID",
        "description": "identifier for a mathematician on Prosopomaths",
        "data_type": "ExternalId"
    },
    "P5784": {
        "label": "DSV person ID",
        "description": "identifier for a person on the German Ski Association website",
        "data_type": "ExternalId"
    },
    "P5815": {
        "label": "Olympic Channel athlete ID",
        "description": "identifier for an athlete at the Olympic Channel website (OlympicChannel.com)",
        "data_type": "ExternalId"
    },
    "P5790": {
        "label": "Skijumping.pl jumper ID",
        "description": "identifier for a ski jumper on the Skijumping.pl website",
        "data_type": "ExternalId"
    },
    "P5789": {
        "label": "WorldofVolley player ID",
        "description": "identifier for a volleyball player on the WorldofVolley website",
        "data_type": "ExternalId"
    },
    "P6311": {
        "label": "Pressball.by football (soccer) player ID",
        "description": "identifier for an association football (soccer) player at Pressball.by website",
        "data_type": "ExternalId"
    },
    "P5770": {
        "label": "Animator.ru person ID",
        "description": "ID of person in Animator.ru database",
        "data_type": "ExternalId"
    },
    "P6038": {
        "label": "AFAS author ID",
        "description": "identifier for an author on the prosopographical website of the 'Association française pour l'avancement des sciences'",
        "data_type": "ExternalId"
    },
    "P5773": {
        "label": "Television Academy Foundation interviewee ID",
        "description": "interviewee identifier on the Television Academy Foundation's The Interviews website",
        "data_type": "ExternalId"
    },
    "P6064": {
        "label": "Scoresway ice hockey person ID",
        "description": "identifier for a ice hockey person at the Scoresway",
        "data_type": "ExternalId"
    },
    "P6066": {
        "label": "Scoresway volleyball person ID",
        "description": "identifier for a volleyball person at the Scoresway",
        "data_type": "ExternalId"
    },
    "P6167": {
        "label": "British Executions ID",
        "description": "identifier for a person executed in Britain between 1100 and 1964",
        "data_type": "ExternalId"
    },
    "P5750": {
        "label": "FBref.com player ID",
        "description": "identifier for a player on the FBref website",
        "data_type": "ExternalId"
    },
    "P5859": {
        "label": "Adelphi author ID",
        "description": "identifier for author in the database of the Italian publisher Adelphi",
        "data_type": "ExternalId"
    },
    "P5839": {
        "label": "Académie de Marseille member ID",
        "description": "identifier for a member of the 'Académie de Marseille' on its website",
        "data_type": "ExternalId"
    },
    "P5819": {
        "label": "International Mathematical Olympiad participant ID",
        "description": "identifier of an International Mathematical Olympiad participant on the official website of IMO",
        "data_type": "ExternalId"
    },
    "P6119": {
        "label": "Letterboxd actor ID",
        "description": "identifier for an actor on the website Letterboxd",
        "data_type": "ExternalId"
    },
    "P5871": {
        "label": "Norwegian war refugees register ID",
        "description": "Norwegian register for refugees at Kjesæter 1940 to 1945 ID",
        "data_type": "ExternalId"
    },
    "P6071": {
        "label": "Polish Olympic Committee athlete ID",
        "description": "identifier portion of URL for athlete profile at the official website of the Polish Olympic Committee",
        "data_type": "ExternalId"
    },
    "P6300": {
        "label": "Hymnary author ID",
        "description": "identifier for hymn authors in the Hymnary.org database",
        "data_type": "ExternalId"
    },
    "P5848": {
        "label": "AnimeClick person ID",
        "description": "identifier of person from AnimeClick database",
        "data_type": "ExternalId"
    },
    "P5921": {
        "label": "Index of Historic Collectors and Dealers of Cubism ID",
        "description": "identifier for a person or an organization in the Index of Historic Collectors and Dealers of Cubism database",
        "data_type": "ExternalId"
    },
    "P5951": {
        "label": "ComiXology creator ID",
        "description": "identifier of comics creators from the ComiXology digital comics storefront",
        "data_type": "ExternalId"
    },
    "P5941": {
        "label": "Film Indonesia person ID",
        "description": "ID number on the Film Indonesia for the person and film studio",
        "data_type": "ExternalId"
    },
    "P5989": {
        "label": "Indiancine.ma person ID",
        "description": "identifier for a person, in the Indiancine.ma archive of Indian film",
        "data_type": "ExternalId"
    },
    "P6282": {
        "label": "French Academy of Sciences member ID",
        "description": "identifier for a member of the French Academy of Sciences on its website",
        "data_type": "ExternalId"
    },
    "P5948": {
        "label": "CMI person ID",
        "description": "identifier for a person on the Clay Mathematics Institute website",
        "data_type": "ExternalId"
    },
    "P5908": {
        "label": "Geheugen van de VU person ID",
        "description": "identifier for a person in the database of Geheugen van de VU (Vrije Universiteit Amsterdam)",
        "data_type": "ExternalId"
    },
    "P6312": {
        "label": "Russian Premier League staff ID",
        "description": "identifier for a Russian Premier League staff member at official website premierliga.ru",
        "data_type": "ExternalId"
    },
    "P6214": {
        "label": "Jewish Museum Berlin person ID",
        "description": "Jewish Museum Berlin person ID",
        "data_type": "ExternalId"
    },
    "P5959": {
        "label": "V.League ID (obsolete)",
        "description": "identifier for profile of player on V.League old site. For identifier on new site, use P6527 (for male player) or P6528 (for female player)",
        "data_type": "ExternalId"
    },
    "P6097": {
        "label": "AllPowerlifting.com person ID",
        "description": "identifier for a powerlifter at the AllPowerlifting.com",
        "data_type": "ExternalId"
    },
    "P5742": {
        "label": "Voetbalstats player ID",
        "description": "identifier for a Dutch international player on the Voetbalstats website",
        "data_type": "ExternalId"
    },
    "P6297": {
        "label": "ACB.com coach ID",
        "description": "identifier for a coach at ACB.com",
        "data_type": "ExternalId"
    },
    "P6147": {
        "label": "World Poker Tour player ID",
        "description": "identifier for a poker player on the World Poker Tour website",
        "data_type": "ExternalId"
    },
    "P5807": {
        "label": "Alternativa Teatral person ID",
        "data_type": "ExternalId"
    },
    "P5969": {
        "label": "RBF player ID",
        "description": "identifier for a baketball player at the Russian Basketball Federation website",
        "data_type": "ExternalId"
    },
    "P6174": {
        "label": "TWAS Fellow ID",
        "description": "fellows of The World Academy of Sciences (TWAS)",
        "data_type": "ExternalId"
    },
    "P5960": {
        "label": "Japan Professional Basketball League ID",
        "description": "Japan Professional Basketball League",
        "data_type": "ExternalId"
    },
    "P5002": {
        "label": "Orthodox Encyclopedia ID",
        "description": "identifier for an entry on the official website of the Orthodox Encyclopedia",
        "data_type": "ExternalId"
    },
    "P5006": {
        "label": "DNV GL Vessel register ID",
        "description": "identifier for a ship, in the DNV GL Vessel register",
        "data_type": "ExternalId"
    },
    "P5085": {
        "label": "Pschyrembel Online ID",
        "description": "identifier for an article in Pschyrembel Online",
        "data_type": "ExternalId"
    },
    "P5084": {
        "label": "Yahoo Answers category",
        "description": "category at Yahoo Answers",
        "data_type": "ExternalId"
    },
    "P5143": {
        "label": "amateur radio callsign",
        "description": "callsign assigned to licensed Amateur Radio operator",
        "data_type": "ExternalId"
    },
    "P5149": {
        "label": "Ester ID",
        "description": "identifier for an item in the E-Katalog ester.ee",
        "data_type": "ExternalId"
    },
    "P5156": {
        "label": "Norwegian war sailor register ship-ID",
        "description": "identifikation number for ship in the Norwegian war sailor register",
        "data_type": "ExternalId"
    },
    "P5157": {
        "label": "OnsOranje match ID",
        "description": "identifier for a match in the database of OnsOranje",
        "data_type": "ExternalId"
    },
    "P5164": {
        "label": "Flanders Arts Institute organisation ID",
        "description": "identifier of an organisation in the Flanders Arts Institute database for performing arts",
        "data_type": "ExternalId"
    },
    "P5420": {
        "label": "GS1 Global Product Classification brick code",
        "description": "The brick code is used to classify products in the GS1 Global Product Classification",
        "data_type": "ExternalId"
    },
    "P5227": {
        "label": "Carnegie Hall event ID",
        "description": "identifier for an event in the Carnegie Hall Linked Open Data (LOD) database",
        "data_type": "ExternalId"
    },
    "P5432": {
        "label": "setlist.fm venue ID",
        "description": "identifier for a music venue at Setlist.fm",
        "data_type": "ExternalId"
    },
    "P5241": {
        "label": "Songfacts song ID",
        "description": "identifier for a song on the Songfacts website",
        "data_type": "ExternalId"
    },
    "P5047": {
        "label": "Rosetta Code ID",
        "description": "identifier for a page on rosettacode.org",
        "data_type": "ExternalId"
    },
    "P5270": {
        "label": "MonDO ID",
        "description": "identifier for a disease in the Monarch Disease Ontology",
        "data_type": "ExternalId"
    },
    "P5108": {
        "label": "HanCinema film ID",
        "description": "identifier for a films in the HanCinema database",
        "data_type": "ExternalId"
    },
    "P5172": {
        "label": "Rockipedia artist ID",
        "description": "identifier of an artist biography or a band article in the online service Rockipedia",
        "data_type": "ExternalId"
    },
    "P5148": {
        "label": "e-MEC entry",
        "description": "entry for a Brazilian institution of higher learning in the Ministry of Education",
        "data_type": "ExternalId"
    },
    "P5162": {
        "label": "OnsOranje tournament ID",
        "description": "identifier for a tournament in the database of OnsOranje",
        "data_type": "ExternalId"
    },
    "P5313": {
        "label": "Protected Buildings Register in Finland Building ID",
        "description": "Building ID in the Protected Buildings Register maintained by the Finnish Heritage Agency",
        "data_type": "ExternalId"
    },
    "P5300": {
        "label": "Webb-site organization ID",
        "description": "identifier for an organization on \"Webb-site Who's Who\"",
        "data_type": "ExternalId"
    },
    "P5016": {
        "label": "Digital Typhoon cyclone ID",
        "description": "identifier for general information of a cyclone in South-West Pacific in Digital Typhoon",
        "data_type": "ExternalId"
    },
    "P5031": {
        "label": "Animated Knots ID",
        "description": "identifier for a type of knot, in the Animated Knots website",
        "data_type": "ExternalId"
    },
    "P5015": {
        "label": "Digital Typhoon typhoon ID",
        "description": "identifier for general information of a typhoon in Digital Typhoon",
        "data_type": "ExternalId"
    },
    "P5076": {
        "label": "Römpp online ID",
        "description": "identifier for an article in the online version of Römpp",
        "data_type": "ExternalId"
    },
    "P5159": {
        "label": "The New Fund for Cinema and Television (Israel) ID",
        "description": "ID for movies backed by the New Fund for Cinema and Television (Israel)",
        "data_type": "ExternalId"
    },
    "P5048": {
        "label": "Church of Sweden ID",
        "description": "identifier in the database of dioceses, pastorates and church buildings of Sweden",
        "data_type": "ExternalId"
    },
    "P5049": {
        "label": "Harvard designation",
        "description": "identifier for a pre-1963 spacecraft",
        "data_type": "ExternalId"
    },
    "P5063": {
        "label": "Interlingual Index ID",
        "data_type": "ExternalId"
    },
    "P5387": {
        "label": "Behind The Voice Actors TV show ID",
        "description": "identifier of a television show on the website Behind The Voice Actors",
        "data_type": "ExternalId"
    },
    "P5183": {
        "label": "Eurohockey.com club ID",
        "description": "identifier for an ice hockey team in the database of Eurohockey.com",
        "data_type": "ExternalId"
    },
    "P5184": {
        "label": "Daum Encyclopedia ID",
        "description": "identifier for content in the Daum Encyclopedia",
        "data_type": "ExternalId"
    },
    "P5182": {
        "label": "JMA Seismic Intensity Database ID",
        "description": "identifier for an earthquake event in Japan Meteorological Agency Seismic Intensity Database",
        "data_type": "ExternalId"
    },
    "P5398": {
        "label": "TDKIV term ID",
        "description": "identifier for a term used by the Czech Terminology Database of Library and Information Science (TDKIV) produced by the National Library of the Czech Republic",
        "data_type": "ExternalId"
    },
    "P5235": {
        "label": "Cité de la Musique-Philharmonie de Paris work ID",
        "description": "identifier for a musical work in the Cité de la Musique-Philharmonie de Paris database",
        "data_type": "ExternalId"
    },
    "P5234": {
        "label": "SpectraBase compound ID",
        "description": "identifier of a compound in spectrabase.com, a database with spectra",
        "data_type": "ExternalId"
    },
    "P5176": {
        "label": "KIT Linked Open Numbers ID",
        "description": "the value (without separator) of a positive integer. Various interfaces exist for properties of them.",
        "data_type": "ExternalId"
    },
    "P5062": {
        "label": "DOAB publisher ID",
        "description": "identifier in the Directory of Open Access Books (DOAB)",
        "data_type": "ExternalId"
    },
    "P5064": {
        "label": "World of Physics identifier",
        "description": "identifier for an entity or concept, in Eric Weisstein's 'World of Physics'",
        "data_type": "ExternalId"
    },
    "P5177": {
        "label": "Victorian Heritage Register ID",
        "description": "identifier in Victorian Heritage Register, Australia",
        "data_type": "ExternalId"
    },
    "P5082": {
        "label": "Store medisinske leksikon ID",
        "description": "identifier of an article in the online encyclopedia sml.snl.no",
        "data_type": "ExternalId"
    },
    "P5434": {
        "label": "Gfycat user ID",
        "description": "user name for an entity on Gfycat.com",
        "data_type": "ExternalId"
    },
    "P5424": {
        "label": "Lega Serie A team ID",
        "description": "identifier of a soccer team in the Lega Serie A website",
        "data_type": "ExternalId"
    },
    "P5090": {
        "label": "ARRS race ID",
        "description": "identifier for races held by ARRS (Association of Road Racing Statisticians)",
        "data_type": "ExternalId"
    },
    "P5099": {
        "label": "Il mondo dei doppiatori ID",
        "description": "identifier for a creative work or person at Il mondo dei doppiatori",
        "data_type": "ExternalId"
    },
    "P5219": {
        "label": "BMRB ID",
        "description": "identifier of a compound in bmrb.wisc.edu, an NMR spectra database",
        "data_type": "ExternalId"
    },
    "P5214": {
        "label": "GONIAT paper ID",
        "description": "identifier for a taxonomic paper about Paleozoic ammonoids, in the GONIAT database",
        "data_type": "ExternalId"
    },
    "P5218": {
        "label": "Armenian Cinema ID",
        "description": "identifier for a work or person, in the Armenian Cinema database of the Armenian Association of Film Critics and Cinema Journalists",
        "data_type": "ExternalId"
    },
    "P5092": {
        "label": "ISC event ID",
        "description": "entry in the International Seismological Centre Online Event Bibliography",
        "data_type": "ExternalId"
    },
    "P5097": {
        "label": "Logainm ID",
        "description": "identifier for a placename page on the Placenames Database of Ireland (Logainm) site",
        "data_type": "ExternalId"
    },
    "P5220": {
        "label": "ICSC ID",
        "description": "identifier of a compound in the International Chemical Safety Card database",
        "data_type": "ExternalId"
    },
    "P5087": {
        "label": "FIPS 5-2 numeric code (US states)",
        "description": "Two-digit identifier for US states and other associated areas per former Federal Information Processing Standard FIPS 5-2 standard",
        "data_type": "ExternalId"
    },
    "P5229": {
        "label": "Carnegie Hall work ID",
        "description": "identifier for a work in the Carnegie Hall Linked Open Data (LOD) database",
        "data_type": "ExternalId"
    },
    "P5233": {
        "label": "Filmow ID",
        "description": "identifier at the Filmow database of films and TV shows",
        "data_type": "ExternalId"
    },
    "P5333": {
        "label": "Bérose ID",
        "description": "identifier for a person, a journal or an institution on the website of the Bérose online encyclopedia",
        "data_type": "ExternalId"
    },
    "P5094": {
        "label": "HPIP ID",
        "description": "identifier of an entity of Portuguese influence, including geographic/toponymic, onomastic, original author, chronological references, etc.",
        "data_type": "ExternalId"
    },
    "P5093": {
        "label": "road number (Estonia)",
        "description": "identifier for a road in the Estonian Road Register",
        "data_type": "ExternalId"
    },
    "P5289": {
        "label": "World Stadium Database identifier",
        "description": "sportsarenas/stadium database",
        "data_type": "ExternalId"
    },
    "P5231": {
        "label": "Chromosome numbers of the Flora of Germany database ID",
        "description": "numerical identifier in the \"Chromosome numbers of the Flora of Germany\" database",
        "data_type": "ExternalId"
    },
    "P5122": {
        "label": "Ontario public library ID",
        "description": "identifier in the Ontario public library statistics",
        "data_type": "ExternalId"
    },
    "P5106": {
        "label": "Dictionary of Algorithms and Data Structures ID",
        "description": "identifier for an entry in the Dictionary of Algorithms and Data Structures",
        "data_type": "ExternalId"
    },
    "P5114": {
        "label": "Italian School ID",
        "description": "This property describes the school identifier used by Italian Ministry of Education, Universities and Research for identifying italian schools",
        "data_type": "ExternalId"
    },
    "P5123": {
        "label": "Swedish Literature Bank edition",
        "description": "identifier for an book in the Swedish Literature Bank",
        "data_type": "ExternalId"
    },
    "P5395": {
        "label": "Canadian Encyclopedia article ID",
        "description": "identifier for articles in the Canadian Encyclopedia",
        "data_type": "ExternalId"
    },
    "P5104": {
        "label": "New Zealand Gazetteer place ID",
        "description": "identifier for a place in the New Zealand Gazetteer",
        "data_type": "ExternalId"
    },
    "P5258": {
        "label": "Czech Geomorphological Unit Code",
        "description": "identifier assigned to geomorphological units (regions) in Czechia",
        "data_type": "ExternalId"
    },
    "P5262": {
        "label": "Les Enfoirés song ID",
        "description": "identifier for a song in the Enfoiréthèque, the database of Les Enfoirés website",
        "data_type": "ExternalId"
    },
    "P5260": {
        "label": "Apple App Store developer ID",
        "description": "identifier for an iOS app developer on the Apple App Store",
        "data_type": "ExternalId"
    },
    "P5253": {
        "label": "FilmTv.it movie ID",
        "description": "identifier for a movie at FilmTv.it",
        "data_type": "ExternalId"
    },
    "P5275": {
        "label": "OED Online ID",
        "description": "identifier for an entry in the online version of Oxford English Dictionary",
        "data_type": "ExternalId"
    },
    "P5170": {
        "label": "Norwegian fjord catalog ID",
        "description": "identificator for fjords and sea areas in Norway",
        "data_type": "ExternalId"
    },
    "P5245": {
        "label": "myschool ID",
        "description": "identifier of a school in Australia, in the myschool.edu.au site, a government source of compiled data",
        "data_type": "ExternalId"
    },
    "P5242": {
        "label": "ARWU university ID",
        "description": "identifier for an university on the Academic Ranking of World Universities website",
        "data_type": "ExternalId"
    },
    "P5266": {
        "label": "Finnish national bibliography corporate name ID",
        "description": "Finnish national bibliography corporate names database covers the recommended forms of corporate and creator names that the National Library of Finland has produced in the process of tagging the national bibliography.",
        "data_type": "ExternalId"
    },
    "P5250": {
        "label": "IAB code",
        "description": "code for an Internet Advertising Bureau (IAB) Quality Assurance Guidelines (QAG) segment",
        "data_type": "ExternalId"
    },
    "P5296": {
        "label": "Party Number (Taiwan)",
        "description": "identifier assigned to parties registered in Taiwan by the Ministry of the Interior",
        "data_type": "ExternalId"
    },
    "P5079": {
        "label": "NVE Lake Database ID",
        "description": "national identifying number given by NVE for every lake in Norway with a minimum size of 2500 m²",
        "data_type": "ExternalId"
    },
    "P5417": {
        "label": "Common Procurement Vocabulary",
        "description": "massively multilingual and public domain taxonomy legislated by the EU for goods and services",
        "data_type": "ExternalId"
    },
    "P5324": {
        "label": "Swedish National Archive ID",
        "description": "identifier for an archive for a person/organisation/event... stored at the National Archive of Sweden",
        "data_type": "ExternalId"
    },
    "P5127": {
        "label": "WikiCFP conference series ID",
        "description": "identifier for a conference series, in WikiCFP",
        "data_type": "ExternalId"
    },
    "P5290": {
        "label": "Redump.org ID",
        "description": "identifier at the Redump.org database of video games",
        "data_type": "ExternalId"
    },
    "P5327": {
        "label": "fernsehserien.de ID",
        "description": "identifier for television series on the website fernsehserien.de",
        "data_type": "ExternalId"
    },
    "P5124": {
        "label": "WikiCFP event ID",
        "description": "identifier for an event in WikiCFP, an online database for scientific meetings",
        "data_type": "ExternalId"
    },
    "P5335": {
        "label": "Or Movement settlement ID",
        "description": "the ID of a settlement in the Or Movement database for the Negev and the Galilee",
        "data_type": "ExternalId"
    },
    "P5336": {
        "label": "Or Movement regional council ID",
        "description": "the ID of a regional council in the Or Movement database for the Negev and the Galilee",
        "data_type": "ExternalId"
    },
    "P5357": {
        "label": "The Encyclopedia of Science Fiction ID",
        "description": "identifier for an entry on the Encyclopedia of Science Fiction website",
        "data_type": "ExternalId"
    },
    "P5369": {
        "label": "Encyclopædia Universalis author ID",
        "description": "identifier of an author on the online version of Encyclopædia Universalis",
        "data_type": "ExternalId"
    },
    "P5288": {
        "label": "StadiumDB identifier",
        "data_type": "ExternalId"
    },
    "P5400": {
        "label": "GeoNLP ID",
        "description": "Japan identifier for a place",
        "data_type": "ExternalId"
    },
    "P5391": {
        "label": "Norwegian Polar Institute place name ID",
        "description": "identifier of a location in the Norwegian Polar Institute's \"Place names in Norwegian polar areas\" database",
        "data_type": "ExternalId"
    },
    "P5345": {
        "label": "PC Engine Software Bible ID",
        "description": "identifier in the PC Engine Software Bible database of PC Engine videogames",
        "data_type": "ExternalId"
    },
    "P5334": {
        "label": "Rom Galil settlement ID",
        "description": "ID of a settlement in the Galilee settlement database, Rom Galil",
        "data_type": "ExternalId"
    },
    "P5381": {
        "label": "EGROKN ID",
        "description": "identifier for a monument in the Unified state register of objects of cultural heritage (monuments of history and culture) of the peoples of the Russian Federation",
        "data_type": "ExternalId"
    },
    "P9374": {
        "label": "ODLIS ID",
        "description": "identifier for an entry in ODLIS: Online Dictionary for Library and Information Science",
        "data_type": "ExternalId"
    },
    "P9326": {
        "label": "PatER ID",
        "description": "identifier of cultural heritage in PatER, a website by Italian region Emilia-Romagna",
        "data_type": "ExternalId"
    },
    "P9060": {
        "label": "POSIX locale identifier",
        "description": "identifier for a language per the POSIX standard, as defined in ISO/IEC 9945 and ISO/IEC 15897",
        "data_type": "ExternalId"
    },
    "P9406": {
        "label": "Tacoma Local History & Biography Index ID",
        "description": "identifier for an entry in the Tacoma Public Library's Local History & Biography Index",
        "data_type": "ExternalId"
    },
    "P9391": {
        "label": "Česká divadelní encyklopedie ID",
        "description": "identifier for Czech articles in the Czech encyclopedia of theater",
        "data_type": "ExternalId"
    },
    "P9392": {
        "label": "Tschechische Theaterenzyklopädie ID",
        "description": "identifier for German articles in the Czech encyclopedia of theater",
        "data_type": "ExternalId"
    },
    "P9120": {
        "label": "Pepys Encylopedia ID",
        "description": "identifier for a person, place, or thing in an annotated index to the online diaries of Samuel Pepys",
        "data_type": "ExternalId"
    },
    "P9031": {
        "label": "Barnivore product ID",
        "description": "identifier for a product on Barnivore",
        "data_type": "ExternalId"
    },
    "P9393": {
        "label": "Tacoma-Pierce County Buildings Index ID",
        "description": "identifier for an entry in the Tacoma Public Library's Tacoma-Pierce County Buildings Index",
        "data_type": "ExternalId"
    },
    "P233": {
        "label": "canonical SMILES",
        "description": "Simplified Molecular Input Line Entry Specification (canonical format)",
        "data_type": "String"
    },
    "P416": {
        "label": "quantity symbol (string)",
        "description": "symbol for a mathematical or physical quantity",
        "data_type": "String"
    },
    "P9224": {
        "label": "JedeSchule.de ID",
        "description": "identifier for a school on JedeSchule.de",
        "data_type": "ExternalId"
    },
    "P9065": {
        "label": "Parabola package",
        "description": "name of the official Parabola package",
        "data_type": "ExternalId"
    },
    "P645": {
        "label": "genomic end",
        "description": "genomic ending coordinate of the biological sequence (e.g. a gene)",
        "data_type": "String"
    },
    "P555": {
        "label": "doubles record",
        "description": "win/lose balance for a player in doubles tournaments",
        "data_type": "String"
    },
    "P564": {
        "label": "singles record",
        "description": "win/lose balance for a player in singles tournaments",
        "data_type": "String"
    },
    "P9242": {
        "label": "Whiskybase distillery ID",
        "description": "identifier for a distillery on Whiskybase",
        "data_type": "ExternalId"
    },
    "P9236": {
        "label": "RDA value vocabularies ID",
        "description": "ID for a term in the RDA value vocabularies",
        "data_type": "ExternalId"
    },
    "P9122": {
        "label": "SportsLogos.net team ID",
        "description": "identifier for a sports team on Chris Creamer's Sports Logos Page",
        "data_type": "ExternalId"
    },
    "P9345": {
        "label": "Discord server numeric ID",
        "description": "numeric identifier for a Discord server; use as qualifier for P9078 \"Discord invite ID\"",
        "data_type": "ExternalId"
    },
    "P9364": {
        "label": "Encyclopedia of Mediterranean Humanism ID",
        "description": "identifier for an article in the Encyclopedia of Mediterranean Humanism",
        "data_type": "ExternalId"
    },
    "P9123": {
        "label": "Catalan Vikidia ID",
        "description": "ID of the article in Catalan Vikidia",
        "data_type": "ExternalId"
    },
    "P9100": {
        "label": "GitHub topic",
        "description": "GitHub topic for a repository or subject",
        "data_type": "ExternalId"
    },
    "P9124": {
        "label": "Russian Vikidia ID",
        "description": "ID of the article in Russian Vikidia",
        "data_type": "ExternalId"
    },
    "P9125": {
        "label": "Microsoft MVP profile ID",
        "description": "identifier for a Microsoft Most Valuable Professional Award recipient",
        "data_type": "ExternalId"
    },
    "P9128": {
        "label": "Debian Package Tracker ID",
        "description": "source package identifier in the Debian Package Tracker",
        "data_type": "ExternalId"
    },
    "P9421": {
        "label": "IFVPF ID",
        "description": "identifier for a Brazilian coffee farm in the Inventory of the Vale do Paraíba Fluminense Fazendas",
        "data_type": "ExternalId"
    },
    "P9153": {
        "label": "VAi building ID",
        "description": "identifier for buildings in the building database of Flanders Architecture Institute",
        "data_type": "ExternalId"
    },
    "P215": {
        "label": "spectral class",
        "description": "spectral class of an astronomical object",
        "data_type": "String"
    },
    "P9280": {
        "label": "Museum of the Jewish People at Beit Hatfutsot ID",
        "description": "identifier for an entity in the databases of the Museum of the Jewish People at Beit Hatfutsot",
        "data_type": "ExternalId"
    },
    "P9186": {
        "label": "DicoPolHiS ID",
        "description": "identifier for an entry in the Dictionnaire Politique d'Histoire de la Santé",
        "data_type": "ExternalId"
    },
    "P9331": {
        "label": "Storiaememoriadibologna ID",
        "description": "identifier of the website Storiaememoriadibologna about the most important locations, events and persons of the history of Bologna, especially focused on the Certosa monumental cemetery of Bologna",
        "data_type": "ExternalId"
    },
    "P274": {
        "label": "chemical formula",
        "description": "description of chemical compound  giving element symbols and counts",
        "data_type": "String"
    },
    "P9377": {
        "label": "BMZ lemma ID",
        "description": "identifier for this lexeme in the Middle High German dictionary by Benecke, Müller, Zarncke at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P348": {
        "label": "software version identifier",
        "description": "numeric or nominal identifier of a version of a software program or file format, current or past",
        "data_type": "String"
    },
    "P9378": {
        "label": "FindeB lemma ID",
        "description": "identifier for this lexeme in the Findebuch zum mittelhochdeutschen Wortschatz at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9379": {
        "label": "Lexer lemma ID",
        "description": "identifier for this lexeme in the Middle High German dictionary by Matthias Lexer at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9380": {
        "label": "LexerN lemma ID",
        "description": "identifier for this lexeme in the supplement to the Middle High German dictionary by Matthias Lexer at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9075": {
        "label": "StrategyWiki ID",
        "description": "identifier for this video game in StrategyWiki",
        "data_type": "ExternalId"
    },
    "P9405": {
        "label": "NMRShiftDB structure ID",
        "description": "structure identifiers in the NMRShiftDB",
        "data_type": "ExternalId"
    },
    "P9358": {
        "label": "MWB lemma ID",
        "description": "identifier for this lexeme in the Middle High German dictionary at mhdwb-online.de",
        "data_type": "ExternalId"
    },
    "P9066": {
        "label": "FL number",
        "description": "EU identifier for food flavouring",
        "data_type": "ExternalId"
    },
    "P9335": {
        "label": "Archives West finding aid ID",
        "description": "identifier of a finding aid in Archives West",
        "data_type": "ExternalId"
    },
    "P742": {
        "label": "pseudonym",
        "description": "alias used by someone (for nickname use P1449)",
        "data_type": "String"
    },
    "P246": {
        "label": "element symbol",
        "description": "identifier for a chemical element",
        "data_type": "String"
    },
    "P9398": {
        "label": "Armenian Book ID",
        "description": "identifier for a written work in the Armenian Book database, which is part of the National Library of Armenia",
        "data_type": "ExternalId"
    },
    "P225": {
        "label": "taxon name",
        "description": "correct scientific name of a taxon (according to the reference given)",
        "data_type": "String"
    },
    "P993": {
        "label": "NFPA Health",
        "description": "NFPA rating for a chemical's hazard to health (blue quadrant in fire diamond)",
        "data_type": "String"
    },
    "P994": {
        "label": "NFPA Fire",
        "description": "NFPA rating for a chemical's flammability (red quadrant in fire diamond)",
        "data_type": "String"
    },
    "P9366": {
        "label": "Rombase ID (English)",
        "description": "identifier for an English-language article in the Rombase encyclopedia",
        "data_type": "ExternalId"
    },
    "P9045": {
        "label": "Alpine Linux package",
        "description": "name of the official Alpine Linux package",
        "data_type": "ExternalId"
    },
    "P9400": {
        "label": "Government of Canada Core Subject Thesaurus ID",
        "description": "identifier for an English term in the Government of Canada Core Subject Thesaurus",
        "data_type": "ExternalId"
    },
    "P9401": {
        "label": "Thésaurus des sujets de base du gouvernement du Canada ID",
        "description": "identifier for an French term in the Thésaurus des sujets de base du gouvernement du Canada ID",
        "data_type": "ExternalId"
    },
    "P9090": {
        "label": "Swiss Games Showcase ID",
        "description": "alphabetical identifier for a video game in the Swiss Games Showcase database",
        "data_type": "ExternalId"
    },
    "P9270": {
        "label": "ANSI/NISO standard ID",
        "description": "identifier for the number of the ANSI/NISO standard which normalizes the object",
        "data_type": "ExternalId"
    },
    "P9359": {
        "label": "Encyclopedia of Migration ID",
        "description": "identifier for an English-language article in the Encyclopedia of Migration",
        "data_type": "ExternalId"
    },
    "P9414": {
        "label": "Viasona ID",
        "description": "alphabetical identifier for an artist at the Viasona music archive website",
        "data_type": "ExternalId"
    },
    "P9360": {
        "label": "Ministry of Education New Zealand school ID",
        "description": "Ministry of Education-issued identifier for a school in New Zealand",
        "data_type": "ExternalId"
    },
    "P9273": {
        "label": "RePEc EconPapers ID",
        "description": "identifier for a scholarly article in the RePEc (Research Papers in Economics) database",
        "data_type": "ExternalId"
    },
    "P9306": {
        "label": "EudraCT trial ID",
        "description": "identifier for a trial in the EU Clinical Trials Register, the clinical trial registry of the European Medicines Agency",
        "data_type": "ExternalId"
    },
    "P9143": {
        "label": "Play:Right ID",
        "description": "identifier of a video game or an equipment on playright.dk",
        "data_type": "ExternalId"
    },
    "P465": {
        "label": "sRGB color hex triplet",
        "description": "sRGB hex triplet format for subject color (e.g. 7FFFD4) specifying the 8-bit red, green and blue components",
        "data_type": "String"
    },
    "P393": {
        "label": "edition number",
        "description": "number of an edition (first, second, ... as 1, 2, ...) or event",
        "data_type": "String"
    },
    "P9094": {
        "label": "Pantone color ID",
        "description": "the Pantone color code for a Pantone color",
        "data_type": "ExternalId"
    },
    "P9307": {
        "label": "Fancyclopedia 3 ID",
        "description": "identifier for a page on Fancyclopedia 3",
        "data_type": "ExternalId"
    },
    "P9308": {
        "label": "operator licence number",
        "description": "British heavy goods vehicle (HGV) and public service vehicle (PSV) operator license number",
        "data_type": "ExternalId"
    },
    "P9284": {
        "label": "GB1900 ID",
        "description": "identifier for a place in the GB1900 project transcription of the Ordnance Survey six-inch-to-one-mile map, 1888-1913",
        "data_type": "ExternalId"
    },
    "P9384": {
        "label": "Adelung lemma ID",
        "description": "identifier for this lexeme in the Grammatisch-Kritisches Wörterbuch der Hochdeutschen Mundart at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9385": {
        "label": "DWB lemma ID",
        "description": "identifier for this lexeme in the German dictionary by Jacob and Wilhelm Grimm at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9386": {
        "label": "DWB2 lemma ID",
        "description": "identifier for this lexeme in the revised edition of the German dictionary by Jacob and Wilhelm Grimm at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9387": {
        "label": "GWB lemma ID",
        "description": "identifier for this lexeme in the Goethe dictionary by Jacob and Wilhelm Grimm at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9388": {
        "label": "Meyers lemma ID",
        "description": "identifier for this lexeme in the Meyers Großes Konversationslexikon at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9389": {
        "label": "RDWB1 lemma ID",
        "description": "identifier for this lexeme in the Wörterbuch der russisch-deutschen Übersetzungsschwierigkeiten (German-Russian part) at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P9390": {
        "label": "Wander lemma ID",
        "description": "identifier for this lexeme in the German proverb lexicon by Karl Friedrich Wilhelm Wander at woerterbuchnetz.de",
        "data_type": "ExternalId"
    },
    "P487": {
        "label": "Unicode character",
        "description": "Unicode character representing the item",
        "data_type": "String"
    },
    "P9056": {
        "label": "TheCocktailDB drink ID",
        "description": "identifier of a drink in thecocktaildb.com database",
        "data_type": "ExternalId"
    },
    "P995": {
        "label": "NFPA Instability",
        "description": "NFPA rating for chemical or physical reactivity (yellow quadrant in fire diamond)",
        "data_type": "String"
    },
    "P9057": {
        "label": "TheCocktailDB ingredient ID",
        "description": "identifier of an ingredient in thecocktaildb.com database",
        "data_type": "ExternalId"
    },
    "P9078": {
        "label": "Discord invite ID",
        "description": "identifier for the subject's official Discord server",
        "data_type": "ExternalId"
    },
    "P9118": {
        "label": "Family Video Game Database ID",
        "description": "alphabetical identifier for a video game in the Family Video Game Database",
        "data_type": "ExternalId"
    },
    "P898": {
        "label": "IPA transcription",
        "description": "transcription in the International Phonetic Alphabet",
        "data_type": "String"
    },
    "P877": {
        "label": "NFPA Special",
        "description": "NFPA code for a chemical's other hazards in white (bottom) quadrant",
        "data_type": "String"
    },
    "P9230": {
        "label": "NLS place type ID",
        "description": "place type ID in the Finnish National Land Survey Place Names Register",
        "data_type": "ExternalId"
    },
    "P9064": {
        "label": "Povos Indígenas no Brasil ID",
        "description": "identifier for an indigenous group in the reference work Povos Indígenas no Brasil",
        "data_type": "ExternalId"
    },
    "P644": {
        "label": "genomic start",
        "description": "genomic starting coordinate of the biological sequence (e.g. a gene)",
        "data_type": "String"
    },
    "P9362": {
        "label": "Encyklopedie migrace ID",
        "description": "identifier for a Czech-language article in the Encyklopedie migrace",
        "data_type": "ExternalId"
    },
    "P444": {
        "label": "review score",
        "description": "review score received by a creative work or other entity",
        "data_type": "String"
    },
    "P9096": {
        "label": "Malpedia ID",
        "description": "identity for an item in the Malpedia database",
        "data_type": "ExternalId"
    },
    "P9079": {
        "label": "Red Cross FDRS ID",
        "description": "entry in the Federation-wide Databank and Reporting System (FDRS) of the International Federation of Red Cross and Red Crescent Societies (IFRC)",
        "data_type": "ExternalId"
    },
    "P7725": {
        "label": "litter size",
        "description": "number of young born/laid at the same time by one animal",
        "data_type": "Quantity"
    },
    "P7442": {
        "label": "glacier status",
        "description": "Indicates whether a glacier is retreating, surging, or stable. Qualify each value with \"point in time\" (P585) to indicate when.",
        "data_type": "WikibaseItem"
    },
    "P7481": {
        "label": "variety of form",
        "description": "optional property to link a form to a dialect, or a graphic or phonetic variety",
        "data_type": "WikibaseItem"
    },
    "P217": {
        "label": "inventory number",
        "description": "identifier for a physical object or a set of physical objects in a collection",
        "data_type": "String"
    },
    "P166": {
        "label": "award received",
        "description": "award or recognition received by a person, organisation or creative work",
        "data_type": "WikibaseItem"
    },
    "P88": {
        "label": "commissioned by",
        "description": "person or organization that commissioned this work",
        "data_type": "WikibaseItem"
    },
    "P61": {
        "label": "discoverer or inventor",
        "description": "subject  who discovered, first described, invented, or developed this discovery or invention",
        "data_type": "WikibaseItem"
    },
    "P195": {
        "label": "collection",
        "description": "art, museum, archival, or bibliographic collection the subject is part of",
        "data_type": "WikibaseItem"
    },
    "P178": {
        "label": "developer",
        "description": "organisation or person that developed the item",
        "data_type": "WikibaseItem"
    },
    "P8703": {
        "label": "entry in abbreviations table",
        "description": "abbreviation of name of concept in a publication's list of abbreviations",
        "data_type": "String"
    },
    "P7256": {
        "label": "computer performance",
        "description": "performance of a computer in FLOPS",
        "data_type": "Quantity"
    },
    "P7219": {
        "label": "periphrastic definition",
        "description": "concept used as subject to form a periphrastic definition together with a related lexeme as qualifier (such as \"manner\" to define \"slowly\" with the qualifier \"slow\")",
        "data_type": "WikibaseSense"
    },
    "P7643": {
        "label": "covered period",
        "description": "historical period covered, for instance by a source",
        "data_type": "WikibaseItem"
    },
    "P175": {
        "label": "performer",
        "description": "actor, musician, band or other performer associated with this role or musical work",
        "data_type": "WikibaseItem"
    },
    "P7582": {
        "label": "Wildlife Protection Act 1972 (India) schedule",
        "description": "classification of species found in India as per legal protection status under Wildlife Protection Act 1972",
        "data_type": "WikibaseItem"
    },
    "P9259": {
        "label": "assessment outcome",
        "description": "(qualifier for P5021) outcome of the test or assessment when the test isn't a score (P5022)",
        "data_type": "WikibaseItem"
    },
    "P287": {
        "label": "designed by",
        "description": "person(s) or organization which designed the object",
        "data_type": "WikibaseItem"
    },
    "P7603": {
        "label": "CITES Appendix",
        "description": "appendix of a taxon according to CITES convention",
        "data_type": "WikibaseItem"
    },
    "P7253": {
        "label": "color produced",
        "description": "color or color range produced by a dye, alone or with a mordant",
        "data_type": "WikibaseItem"
    },
    "P7415": {
        "label": "code (image)",
        "description": "Commons image file of code used to represent a specific concept in a given encoding",
        "data_type": "CommonsMedia"
    },
    "P7252": {
        "label": "degree of difficulty",
        "description": "difficulty level of an item",
        "data_type": "WikibaseItem"
    },
    "P7316": {
        "label": "number of decimal digits",
        "description": "number of decimal digits of a natural number",
        "data_type": "Quantity"
    },
    "P7295": {
        "label": "Gregorian calendar start date",
        "description": "date the Gregorian calendar was first used",
        "data_type": "Time"
    },
    "P8693": {
        "label": "fandom",
        "description": "fan group name of a celebrity, musical group, or an artist",
        "data_type": "WikibaseItem"
    },
    "P8739": {
        "label": "prohibits",
        "description": "the norm implies a prohibition of the object in the context of the qualifiers",
        "data_type": "WikibaseItem"
    },
    "P8272": {
        "label": "BaGLAMa GID",
        "description": "unique identifier of category in the BaGLAMa tool",
        "data_type": "ExternalId"
    },
    "P7261": {
        "label": "use restriction status",
        "description": "status of any use restrictions on the object, collection, or materials",
        "data_type": "WikibaseItem"
    },
    "P8017": {
        "label": "generational suffix",
        "description": "suffix used in name",
        "data_type": "WikibaseForm"
    },
    "P415": {
        "label": "radio format",
        "description": "describes the overall content broadcast on a radio station",
        "data_type": "WikibaseItem"
    },
    "P8684": {
        "label": "x-offset",
        "description": "offset on the x-axis or primary axis",
        "data_type": "Quantity"
    },
    "P8685": {
        "label": "y-offset",
        "description": "offset on the y-axis or secondary axis",
        "data_type": "Quantity"
    },
    "P8045": {
        "label": "organized response related to outbreak",
        "description": "specific action taken or system used as a reaction to an outbreak",
        "data_type": "WikibaseItem"
    },
    "P8214": {
        "label": "curriculum vitae URL",
        "description": "URL of the subject's curriculum vitae",
        "data_type": "Url"
    },
    "P7347": {
        "label": "peer review URL",
        "description": "link to the peer review of a scholarly publication",
        "data_type": "Url"
    },
    "P7501": {
        "label": "audio system",
        "description": "audio system hardware used in the item",
        "data_type": "WikibaseItem"
    },
    "P7508": {
        "label": "election called by",
        "description": "person who summoned an election",
        "data_type": "WikibaseItem"
    },
    "P275": {
        "label": "copyright license",
        "description": "license under which this copyrighted work is released",
        "data_type": "WikibaseItem"
    },
    "P84": {
        "label": "architect",
        "description": "person or architectural firm responsible for designing this building",
        "data_type": "WikibaseItem"
    },
    "P8030": {
        "label": "size designation",
        "description": "organizational size classification or designation used to indicate relative size of an entity within an organization (property under development)",
        "data_type": "WikibaseItem"
    },
    "P7510": {
        "label": "XML namespace",
        "description": "URI that identifies uniquely a set of XML elements",
        "data_type": "Url"
    },
    "P7482": {
        "label": "source of file",
        "description": "broad nature of the origin of the file",
        "data_type": "WikibaseItem"
    },
    "P7213": {
        "label": "newspaper archive URL",
        "description": "online repository of newspaper",
        "data_type": "Url"
    },
    "P7309": {
        "label": "climbing route",
        "description": "route used to get closer to a high point",
        "data_type": "WikibaseItem"
    },
    "P7290": {
        "label": "FCC Product Code",
        "description": "identifier of a product assigned by the grantee in an application to the Federal Communications Commission of the United States",
        "data_type": "String"
    },
    "P7378": {
        "label": "item disputed by",
        "description": "the concept described by the item is criticized by other item(s)",
        "data_type": "WikibaseItem"
    },
    "P7376": {
        "label": "worn by",
        "description": "person who wore a garment or object",
        "data_type": "WikibaseItem"
    },
    "P7407": {
        "label": "name (image)",
        "description": "name of something, when it's not fully supported by Unicode or contains characters that are not widely supported",
        "data_type": "CommonsMedia"
    },
    "P8625": {
        "label": "bibliography",
        "description": "Wikidata element for a selection of relevant texts on the given topic",
        "data_type": "WikibaseItem"
    },
    "P149": {
        "label": "architectural style",
        "description": "architectural style of a structure",
        "data_type": "WikibaseItem"
    },
    "P7863": {
        "label": "aperture",
        "description": "aperture of a camera lens",
        "data_type": "Quantity"
    },
    "P7330": {
        "label": "Hardware Version Identification Number",
        "description": "identifier of an electronic device model as registered in the Radio Equipment List of Innovation, Science and Economic Development Canada",
        "data_type": "String"
    },
    "P449": {
        "label": "original broadcaster",
        "description": "network(s) or service(s) that originally broadcasted a radio or television program",
        "data_type": "WikibaseItem"
    },
    "P144": {
        "label": "based on",
        "description": "the work(s) used as the basis for subject item",
        "data_type": "WikibaseItem"
    },
    "P8206": {
        "label": "Gazette of India notification",
        "description": "notification number and date, published in the Gazette of India",
        "data_type": "String"
    },
    "P7377": {
        "label": "tussenvoegsel",
        "description": "component of Dutch family name or similar of other languages, generally an unaffixed prefix. Depending on the language, country and application, it may be ignored in sorting. Note differences in capitalization and affixation (spaces)",
        "data_type": "WikibaseItem"
    },
    "P7260": {
        "label": "Transporter Classification Database ID",
        "description": "classifies transport proteins similar to how EC classifies enzymes",
        "data_type": "ExternalId"
    },
    "P7328": {
        "label": "amount cataloged",
        "description": "amount or size of the collection that has been described or cataloged",
        "data_type": "Quantity"
    },
    "P135": {
        "label": "movement",
        "description": "literary, artistic, scientific or philosophical movement or scene associated with this person or work",
        "data_type": "WikibaseItem"
    },
    "P7531": {
        "label": "MEA Indian Mission ID",
        "description": "identifier for Indian diplomatic missions in foreign countries",
        "data_type": "ExternalId"
    },
    "P8009": {
        "label": "IM channel",
        "description": "address of a multi-user chat, room or group",
        "data_type": "String"
    },
    "P8047": {
        "label": "country of registry",
        "description": "country where a ship is or has been registered",
        "data_type": "WikibaseItem"
    },
    "P371": {
        "label": "presenter",
        "description": "someone who takes a main role in presenting a radio or television program or a performing arts show",
        "data_type": "WikibaseItem"
    },
    "P7479": {
        "label": "key col",
        "description": "highest col connecting to an even higher point",
        "data_type": "WikibaseItem"
    },
    "P7600": {
        "label": "primary note",
        "description": "most prominent note in a raga",
        "data_type": "WikibaseItem"
    },
    "P7604": {
        "label": "supervised by",
        "description": "Is the natural person who has the authority of giving instructions and/or orders to subordinates and is held responsible for the work and actions of subordinates.",
        "data_type": "WikibaseItem"
    },
    "P7443": {
        "label": "number of processor threads",
        "description": "total number of logical cores of a CPU",
        "data_type": "Quantity"
    },
    "P170": {
        "label": "creator",
        "description": "maker of this creative work or other object (where no more specific property exists). Paintings with unknown painters, use \"anonymous\" (Q4233718) as value.",
        "data_type": "WikibaseItem"
    },
    "P7420": {
        "label": "image with frame",
        "description": "media file of painting or other 2D artwork with its frame or more of its surroundings included",
        "data_type": "CommonsMedia"
    },
    "P7163": {
        "label": "typically sells",
        "description": "type of goods particular type of shops typically sell",
        "data_type": "WikibaseItem"
    },
    "P98": {
        "label": "editor",
        "description": "editor of a compiled work such as a book or a periodical (newspaper or an academic journal)",
        "data_type": "WikibaseItem"
    },
    "P161": {
        "label": "cast member",
        "description": "actor in the subject production [use \"character role\" (P453) and/or \"name of the character role\" (P4633) as qualifiers] [use \"voice actor\" (P725) for voice-only role]",
        "data_type": "WikibaseItem"
    },
    "P7209": {
        "label": "ordeal by",
        "description": "manner of judicial practice by which the guilt or innocence of the accused was determined",
        "data_type": "WikibaseItem"
    },
    "P7727": {
        "label": "legislative committee",
        "description": "committee in which the legislation is discussed",
        "data_type": "WikibaseItem"
    },
    "P58": {
        "label": "screenwriter",
        "description": "person(s) who wrote the script for subject item",
        "data_type": "WikibaseItem"
    },
    "P7719": {
        "label": "generalization of",
        "description": "statement that is a particular case of this statement or whose truth is implied by this statement",
        "data_type": "WikibaseItem"
    },
    "P8270": {
        "label": "ID natural monuments in Landkreis Oberspreewald-Lausitz",
        "description": "identifier for natural monuments in Landkreis Oberspreewald-Lausitz",
        "data_type": "ExternalId"
    },
    "P8289": {
        "label": "parish church",
        "description": "church which acts as the religious centre of a parish",
        "data_type": "WikibaseItem"
    },
    "P136": {
        "label": "genre",
        "description": "creative work's genre or an artist's field of work (P101). Use main subject (P921) to relate creative works to their topic",
        "data_type": "WikibaseItem"
    },
    "P8195": {
        "label": "ex-libris",
        "description": "ex-libris (or bookplate) of this person",
        "data_type": "CommonsMedia"
    },
    "P7235": {
        "label": "in defining formula",
        "description": "symbol used in the defining formula (P2534)",
        "data_type": "Math"
    },
    "P8097": {
        "label": "network bands",
        "description": "network bands supported by the device",
        "data_type": "WikibaseItem"
    },
    "P7423": {
        "label": "Professional Football League of Ukraine player ID",
        "description": "identifier for football player on site Professional Football League of Ukraine",
        "data_type": "ExternalId"
    },
    "P7167": {
        "label": "does not have effect",
        "description": "expected effect or usual effect of members of this class absent here",
        "data_type": "WikibaseItem"
    },
    "P7888": {
        "label": "merged into",
        "description": "the subject was dissolved and merged into the object, that existed previously",
        "data_type": "WikibaseItem"
    },
    "P437": {
        "label": "distribution format",
        "description": "method (or type) of distribution for the subject",
        "data_type": "WikibaseItem"
    },
    "P7262": {
        "label": "ESPN.com college football player ID",
        "description": "identifier for a college football player at ESPN.com",
        "data_type": "ExternalId"
    },
    "P7527": {
        "label": "charge conjugation quantum number",
        "description": "charge conjugation (or c-parity) has a multiplicative quantum number that applies when all quantum charges are reversed, i.e. when a particle is transformed into its antiparticle",
        "data_type": "Quantity"
    },
    "P7601": {
        "label": "secondary note",
        "description": "second-most prominent note in a raga; usually a fifth from the primary note, although there are exceptions",
        "data_type": "WikibaseItem"
    },
    "P272": {
        "label": "production company",
        "description": "company that produced this film, audio or performing arts work",
        "data_type": "WikibaseItem"
    },
    "P57": {
        "label": "director",
        "description": "director(s) of film, TV-series, stageplay, video game or similar",
        "data_type": "WikibaseItem"
    },
    "P8738": {
        "label": "permits",
        "description": "the norm or license permits people to engage in the object",
        "data_type": "WikibaseItem"
    },
    "P4367": {
        "label": "Louisiana Sports Hall of Fame ID",
        "description": "identifier for an athlete on the Louisiana Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4383": {
        "label": "Tennessee Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Tennessee Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4505": {
        "label": "IAFD male performer ID",
        "description": "identifier for a male-born performer in the Internet Adult Film Database",
        "data_type": "ExternalId"
    },
    "P4502": {
        "label": "Snooker.org player ID",
        "description": "identifier for a player on the Snooker.org website",
        "data_type": "ExternalId"
    },
    "P4503": {
        "label": "Wimbledon player ID",
        "description": "identifier for a tennis player on the Wimbledon website, in the archive section",
        "data_type": "ExternalId"
    },
    "P4541": {
        "label": "Croatian Olympic Committee athlete ID",
        "description": "identifier for an athlete on the Croatian Olympic Committee (Hrvatski olimpijski odbor / HOO)",
        "data_type": "ExternalId"
    },
    "P4544": {
        "label": "Tennis Temple player ID",
        "description": "identifier for a player on the Tennis Temple website",
        "data_type": "ExternalId"
    },
    "P4413": {
        "label": "Manitoba Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Manitoba Sports Hall of Fame",
        "data_type": "ExternalId"
    },
    "P4418": {
        "label": "New Zealand Sports Hall of Fame ID",
        "description": "identifier for an inductee on the New Zealand Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4410": {
        "label": "Women's Basketball Hall of Fame ID",
        "description": "identifier for a female basketball player on the Women's Basketball Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4422": {
        "label": "U.S. Ski and Snowboard Hall of Fame athlete ID",
        "description": "identifier for an athlete on the U.S. Ski and Snowboard Hall of Fame and Museum website",
        "data_type": "ExternalId"
    },
    "P4516": {
        "label": "Mémoire du cyclisme cyclist ID",
        "description": "identifier for a person on the Mémoire du cyclisme website",
        "data_type": "ExternalId"
    },
    "P4398": {
        "label": "Canada's Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Canada's Sports Hall of Fame webiste",
        "data_type": "ExternalId"
    },
    "P4421": {
        "label": "Sportbox.ru ID",
        "description": "identifier of a sportsperson on the website Sportbox.ru",
        "data_type": "ExternalId"
    },
    "P4538": {
        "label": "Snooker Database player ID",
        "description": "identifier for a player on the Snooker Database website",
        "data_type": "ExternalId"
    },
    "P4522": {
        "label": "Alaska Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Alaska Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4524": {
        "label": "Survey of Scottish Witchcraft - Person ID",
        "description": "identifier for persons associated with the witch trials in the Survey of Scottish Witchcraft database. For accused witches please use the Survey of Scottish Witchcraft - Accused Witch ID",
        "data_type": "ExternalId"
    },
    "P4523": {
        "label": "Vermont Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Vermont Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4406": {
        "label": "Nova Scotia Sport Hall of Fame ID",
        "description": "identifier for an athlete on the Nova Scotia Sport Hall of Fame",
        "data_type": "ExternalId"
    },
    "P4559": {
        "label": "IJF judoka ID",
        "description": "identifier for a judoka on the International Judo Federation website",
        "data_type": "ExternalId"
    },
    "P4560": {
        "label": "National Collegiate Basketball Hall of Fame ID",
        "description": "identifier for a person on the National Collegiate Basketball Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4512": {
        "label": "Scottish Cemetery Kolkata person ID",
        "description": "This website contains the database of Scottish Cemetery in Kolkata, West Bengal, India. The database resource was built by Presidency University in collaboration the University of St. Andrews.",
        "data_type": "ExternalId"
    },
    "P4450": {
        "label": "HAL author ID",
        "description": "identifier of a researcher on HAL, an open archive allowing to deposit scholarly documents freely searchable",
        "data_type": "ExternalId"
    },
    "P4456": {
        "label": "Internet Off-Broadway Database creditable entity ID",
        "description": "identifier for an actor or a company, in Lortel's Internet Off-Broadway Database",
        "data_type": "ExternalId"
    },
    "P4391": {
        "label": "The-Sports.org athlete ID",
        "description": "identifier for an athlete on The-Sports.org",
        "data_type": "ExternalId"
    },
    "P4371": {
        "label": "KBO pitcher ID",
        "description": "identifier for a baseball pitcher in koreabaseball.com",
        "data_type": "ExternalId"
    },
    "P4514": {
        "label": "Africultures person ID",
        "description": "ID of a person at French website Africultures",
        "data_type": "ExternalId"
    },
    "P4491": {
        "label": "Isidore scholar ID",
        "description": "identifier of a scholar on Isidore, a platform that collects links to scholarly documents and official texts",
        "data_type": "ExternalId"
    },
    "P4539": {
        "label": "Collective Biographies of Women ID",
        "description": "identifier for a person, in the Collective Biographies of Women database",
        "data_type": "ExternalId"
    },
    "P4374": {
        "label": "New Mexico Sports Hall of Fame ID",
        "description": "identifier for an athlete on the New Mexico Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4376": {
        "label": "Oklahoma Sports Hall of Fame ID",
        "description": "identifier for an athlete or a team on the Oklahoma Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4555": {
        "label": "Canada Games athlete ID",
        "description": "identifier for an athlete on the Canada Games website",
        "data_type": "ExternalId"
    },
    "P4551": {
        "label": "Everyone Remembered ID",
        "description": "identifier for a Commonwealth military person, who died in World War One, on the Everyone Remembered database",
        "data_type": "ExternalId"
    },
    "P4554": {
        "label": "ICRC PoW ID",
        "description": "identifier for a prisoner of war, in the archives of the International committee of the Red Cross",
        "data_type": "ExternalId"
    },
    "P4366": {
        "label": "Kansas Sports Hall of Fame ID",
        "description": "identifier for an athlete on the Kansas Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4369": {
        "label": "Cairn author ID",
        "description": "identifier for an author in Cairn, an online library of French-language scholarly journals",
        "data_type": "ExternalId"
    },
    "P4547": {
        "label": "Serbian Olympic Committee athlete ID",
        "description": "identifier for an athlete on the Olympic Committee of Serbia website",
        "data_type": "ExternalId"
    },
    "P4414": {
        "label": "New Brunswick Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the New Brunswick Sports Hall of Fame",
        "data_type": "ExternalId"
    },
    "P4556": {
        "label": "FAI ID",
        "description": "identifier for an athlete on the Fédération Aéronautique Internationale website",
        "data_type": "ExternalId"
    },
    "P4562": {
        "label": "Japanese Baseball Hall of Fame ID",
        "description": "identifier for an inductee on the Japanese Baseball Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4574": {
        "label": "Norwegian historical register of persons ID",
        "description": "Identificator for a person in the Norwegian historical register of persons",
        "data_type": "ExternalId"
    },
    "P4571": {
        "label": "West Australian Football League player ID",
        "description": "identifier for a player on the West Australian Football League website",
        "data_type": "ExternalId"
    },
    "P4461": {
        "label": "World Golf Hall of Fame player ID",
        "description": "identifier for a player on the World Golf Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4546": {
        "label": "Gymn Forum athlete ID",
        "description": "identifier for an athlete on the Gymn Forum website",
        "data_type": "ExternalId"
    },
    "P4438": {
        "label": "BFI Films, TV and people ID",
        "description": "identifier for a person or movie at BFI Film & TV Database. Format: 13 digits and lowercase letters",
        "data_type": "ExternalId"
    },
    "P4365": {
        "label": "Hawai‘i Sports Hall of Fame ID",
        "description": "identifier for an athlete on the Hawai‘i Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4478": {
        "label": "Survey of Scottish Witchcraft - Accused witch ID",
        "description": "Accused witch identifier in the Survey of Scottish Witchcraft database",
        "data_type": "ExternalId"
    },
    "P4392": {
        "label": "BC Sports Hall of Fame athlete ID",
        "description": "alphabetical identifier for an athlete on the BC Sports Hall of Fame",
        "data_type": "ExternalId"
    },
    "P4572": {
        "label": "SS KL Auschwitz Garrison ID",
        "description": "identifier for a person, in the Auschwitz camp personnel database",
        "data_type": "ExternalId"
    },
    "P4405": {
        "label": "NLBPA ID",
        "description": "identifier for a player on the Negro League Baseball Players Association website",
        "data_type": "ExternalId"
    },
    "P4402": {
        "label": "Women's Collegiate Tennis Hall of Fame ID",
        "description": "identifier for a player on the Women's Collegiate Tennis Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4368": {
        "label": "Mississippi Sports Hall of Fame ID",
        "description": "identifier for an athlete on the Mississippi Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4396": {
        "label": "Alberta Sports Hall of Fame and Museum athlete ID",
        "description": "identifier for an athlete on the Alberta Sports Hall of Fame and Museum website",
        "data_type": "ExternalId"
    },
    "P4419": {
        "label": "Videolectures ID",
        "description": "identifier for person, meeting or presentation/talk (video) on the Videolectures website",
        "data_type": "ExternalId"
    },
    "P4469": {
        "label": "IGHOF athlete ID",
        "description": "identifier for an inductee on the International Gymnastics Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4381": {
        "label": "Soccerdonna player ID",
        "description": "identifier for a female association football player on the Soccerdonna website",
        "data_type": "ExternalId"
    },
    "P4385": {
        "label": "Missouri Sports Hall of Fame ID",
        "description": "identifier for an athlete on the Missouri Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4481": {
        "label": "INDUCKS creator ID",
        "description": "identifier for creators and authors related to the Disney Universe",
        "data_type": "ExternalId"
    },
    "P4409": {
        "label": "Baseball Almanac ID",
        "description": "identifier for a player on the Baseball Almanac website",
        "data_type": "ExternalId"
    },
    "P4412": {
        "label": "Ontario Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the Ontario Sports Hall of Fame",
        "data_type": "ExternalId"
    },
    "P4451": {
        "label": "Scoresway handball person ID",
        "description": "identifier for a handball player or manager on the Scoresway website",
        "data_type": "ExternalId"
    },
    "P4382": {
        "label": "LFB player ID",
        "description": "identifier for a female basketball player on the Ligue féminine de basket website",
        "data_type": "ExternalId"
    },
    "P4415": {
        "label": "Sport Australia Hall of Fame inductee ID",
        "description": "identifier for an inductee on the Sport Australia Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4386": {
        "label": "Virginia Sports Hall of Fame ID",
        "description": "identifier for an athlete on the Virginia Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4429": {
        "label": "Pro14 player ID",
        "description": "identifier for a rugby union player on the PRO14 website",
        "data_type": "ExternalId"
    },
    "P4397": {
        "label": "IPC.infostradasports.com athlete ID",
        "description": "identifier for an athlete on ipc.infostradasports.com website",
        "data_type": "ExternalId"
    },
    "P4470": {
        "label": "British Bobsleigh & Skeleton Association ID",
        "description": "identifier for sportsperson in the British Bobsleigh & Skeleton Association (BBSA) database",
        "data_type": "ExternalId"
    },
    "P4417": {
        "label": "Russian Premier League player ID",
        "description": "identifier for a Russian Premier League player at official website premierliga.ru",
        "data_type": "ExternalId"
    },
    "P4408": {
        "label": "sports.ru athlete ID",
        "description": "identifier for an athlete on the website sports.ru",
        "data_type": "ExternalId"
    },
    "P4521": {
        "label": "Radio Radicale person ID",
        "description": "identifier for a person in the database of Radio Radicale",
        "data_type": "ExternalId"
    },
    "P4458": {
        "label": "Dutch Cemetery in Chinsurah person ID",
        "description": "identifier for an interred person in the database of the Dutch Cemetery in Chinsurah, West Bengal",
        "data_type": "ExternalId"
    },
    "P4416": {
        "label": "Panthéon des sports du Québec ID",
        "description": "identifier for an athlete or a builder on the Panthéon des sports du Québec",
        "data_type": "ExternalId"
    },
    "P4474": {
        "label": "International Boxing Hall of Fame boxer ID",
        "description": "identifier for a boxer on the International Boxing Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4468": {
        "label": "Mountain Bike Hall of Fame inductee ID",
        "description": "identifier for an inductee on the Mountain Bike Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4561": {
        "label": "Basketball Reference WNBA player ID",
        "description": "identifier for a  Women's National Basketball Association player on the Basketball-Reference.com website",
        "data_type": "ExternalId"
    },
    "P4440": {
        "label": "Biblioteca Nacional de México ID",
        "description": "authority control identifier used at the Biblioteca Nacional de México",
        "data_type": "ExternalId"
    },
    "P4436": {
        "label": "The Coptic Library ID",
        "description": "identifier of author or subject in The Coptic Library",
        "data_type": "ExternalId"
    },
    "P4464": {
        "label": "Rugby League Project player ID",
        "description": "identifier for a player on the Rugby League Project website",
        "data_type": "ExternalId"
    },
    "P4489": {
        "label": "Hellenic Olympic Committee athlete ID",
        "description": "identifier for an athlete on the Hellenic Olympic Committee website",
        "data_type": "ExternalId"
    },
    "P4463": {
        "label": "Canadian Ski Hall of Fame inductee ID",
        "description": "identifier for an inductee on the Canadian Ski Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4486": {
        "label": "Google Play developer ID",
        "description": "numeric identifier for an Android app developer on the Google Play Store",
        "data_type": "ExternalId"
    },
    "P4498": {
        "label": "World Snooker athlete ID",
        "description": "identifier for an athlete on the World Professional Billiards and Snooker Association website",
        "data_type": "ExternalId"
    },
    "P4493": {
        "label": "Barbarian F.C. ID",
        "description": "identifier for a rugby union player on the Barbarians F.C. website (the Barbarians)",
        "data_type": "ExternalId"
    },
    "P4557": {
        "label": "Indiana Basketball Hall of Fame ID",
        "description": "identifier for an athlete on the Indiana Basketball Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4363": {
        "label": "DSMHOF athlete ID",
        "description": "identifier for an athlete on the Delaware Sports Museum and Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4462": {
        "label": "Canadian Baseball Hall of Fame inductee ID",
        "description": "identifier for an inductee on the Canadian Baseball Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4494": {
        "label": "Barbarian R.C. ID",
        "description": "identifier for a rugby union player on the Barbarians R.C. website (the French Barbarians)",
        "data_type": "ExternalId"
    },
    "P4370": {
        "label": "KBO hitter ID",
        "description": "identifier for a baseball hitter in koreabaseball.com",
        "data_type": "ExternalId"
    },
    "P4508": {
        "label": "CycleBase cyclist ID",
        "description": "identifier for a cyclist on the CycleBase website",
        "data_type": "ExternalId"
    },
    "P4548": {
        "label": "Commonwealth Games Federation athlete ID",
        "description": "identifier for an athlete on the Commonwealth Games Federation website",
        "data_type": "ExternalId"
    },
    "P4476": {
        "label": "Australian Baseball League player ID",
        "description": "identifier for a player on the Australian Baseball League website",
        "data_type": "ExternalId"
    },
    "P4475": {
        "label": "European Fencing Confederation fencer ID",
        "description": "identifier on the European Fencing Confederation website",
        "data_type": "ExternalId"
    },
    "P4375": {
        "label": "North Carolina Sports Hall of Fame ID",
        "description": "identifier for an athlete on the North Carolina Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4504": {
        "label": "IWRP athlete ID",
        "description": "identifier for an athlete on the International Weightlifting Results Project website",
        "data_type": "ExternalId"
    },
    "P4542": {
        "label": "World of O athlete ID",
        "description": "identifier for an orienteering athlete on the World of O website",
        "data_type": "ExternalId"
    },
    "P4364": {
        "label": "Georgia Sports Hall of Fame ID",
        "description": "identifier for an athlete on the Georgia Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4377": {
        "label": "Women's Sports Foundation ID",
        "description": "identifier for an athlete on the Women's Sports Foundation website",
        "data_type": "ExternalId"
    },
    "P4490": {
        "label": "British Olympic Association athlete ID",
        "description": "identifier for an athlete on the British Olympic Association's Team GB website",
        "data_type": "ExternalId"
    },
    "P4583": {
        "label": "U.S. Gymnastics Hall of Fame athlete ID",
        "description": "identifier for an inductee on the U.S. Gymnastics Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4507": {
        "label": "California Sports Hall of Fame athlete ID",
        "description": "identifier for an athlete on the California Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P3734": {
        "label": "category for value same as Wikidata",
        "description": "item for categories of pages with template parameter value same as Wikidata",
        "data_type": "WikibaseItem"
    },
    "P1309": {
        "label": "EGAXA ID",
        "description": "identifier in Bibliotheca Alexandrina",
        "data_type": "ExternalId"
    },
    "P1286": {
        "label": "Munzinger Pop ID",
        "description": "identifier on the Munzinger Archiv",
        "data_type": "ExternalId"
    },
    "P947": {
        "label": "RSL ID (person)",
        "description": "person identifier in the Russian State Library",
        "data_type": "ExternalId"
    },
    "P1007": {
        "label": "Lattes Platform number",
        "description": "number for the Lattes Platform entry on the person or group. Relationship between entry and subject must be supported by a source",
        "data_type": "ExternalId"
    },
    "P2875": {
        "label": "property usage tracking category",
        "description": "category tracking a Wikidata property in sister projects",
        "data_type": "WikibaseItem"
    },
    "P1368": {
        "label": "LNB ID",
        "description": "identifier assigned by the National Library of Latvia",
        "data_type": "ExternalId"
    },
    "P1273": {
        "label": "CANTIC ID",
        "description": "identifier for authority control managed by the Library of Catalonia. Format: \"a\", 7 digits, \"x\" or digit.",
        "data_type": "ExternalId"
    },
    "P1207": {
        "label": "NUKAT ID",
        "description": "identifier for authority control in the Center of Warsaw University Library catalog",
        "data_type": "ExternalId"
    },
    "P1047": {
        "label": "Catholic Hierarchy person ID",
        "description": "identifier for the person on catholic-hierarchy.org",
        "data_type": "ExternalId"
    },
    "P1006": {
        "label": "Nationale Thesaurus voor Auteurs ID",
        "description": "identifier for persons (not:works) from the Dutch National Thesaurus for Author names (which also contains non-authors)",
        "data_type": "ExternalId"
    },
    "P651": {
        "label": "Biografisch Portaal van Nederland ID",
        "description": "identifier at Biografisch Portaal van Nederland",
        "data_type": "ExternalId"
    },
    "P536": {
        "label": "ATP player ID",
        "description": "ATP tennis male player identifier",
        "data_type": "ExternalId"
    },
    "P301": {
        "label": "category's main topic",
        "description": "primary topic of the subject Wikimedia category",
        "data_type": "WikibaseItem"
    },
    "P6186": {
        "label": "category for eponymous categories",
        "description": "Wikimedia category that contains subcategories dedicated to specific instances of the said item",
        "data_type": "WikibaseItem"
    },
    "P270": {
        "label": "CALIS ID",
        "description": "identifier for authority control per CALIS (China Academic Library & Information System)",
        "data_type": "ExternalId"
    },
    "P599": {
        "label": "ITF player ID",
        "description": "identifier for a tennis player at the International Tennis Federation (ITF) website",
        "data_type": "ExternalId"
    },
    "P535": {
        "label": "Find A Grave memorial ID",
        "description": "identifier of an individual's burial place in the Find a Grave database",
        "data_type": "ExternalId"
    },
    "P1375": {
        "label": "NSK ID",
        "description": "identifier for an item in the National and University Library in Zagreb (including leading zeroes)",
        "data_type": "ExternalId"
    },
    "P549": {
        "label": "Mathematics Genealogy Project ID",
        "description": "identifier for mathematicians and computer scientists at the Mathematics Genealogy Project",
        "data_type": "ExternalId"
    },
    "P1153": {
        "label": "Scopus author ID",
        "description": "identifier for an author assigned in Scopus bibliographic database",
        "data_type": "ExternalId"
    },
    "P586": {
        "label": "IPNI author ID",
        "description": "numerical identifier for a person in the International Plant Names Index",
        "data_type": "ExternalId"
    },
    "P597": {
        "label": "WTA player ID",
        "description": "identifier for a female tennis player per WTA",
        "data_type": "ExternalId"
    },
    "P409": {
        "label": "Libraries Australia ID",
        "description": "identifier issued by the National Library of Australia (see also P1315 for the newer People Australia identifier). VIAF component. Format: 1-12 digits, removing leading zero-padding.",
        "data_type": "ExternalId"
    },
    "P640": {
        "label": "Léonore ID",
        "description": "identifier of a person in the Léonore database",
        "data_type": "ExternalId"
    },
    "P3709": {
        "label": "category for value different from Wikidata",
        "description": "item for categories of pages with template parameter value different from Wikidata",
        "data_type": "WikibaseItem"
    },
    "P1364": {
        "label": "ITTF table tennis player ID",
        "description": "6-digit identifier for a table tennis player as per International Table Tennis Federation (ITTF)",
        "data_type": "ExternalId"
    },
    "P1465": {
        "label": "category for people who died here",
        "description": "category item for people who died in this location",
        "data_type": "WikibaseItem"
    },
    "P860": {
        "label": "e-archiv.li ID",
        "description": "identifier in e-archiv.li, a directory of sources of Landesarchiv Liechtenstein",
        "data_type": "ExternalId"
    },
    "P910": {
        "label": "topic's main category",
        "description": "main Wikimedia category",
        "data_type": "WikibaseItem"
    },
    "P866": {
        "label": "Perlentaucher ID",
        "description": "identifier in Perlentaucher (Q2071388)",
        "data_type": "ExternalId"
    },
    "P861": {
        "label": "PremiershipRugby.com ID",
        "description": "identifier for each rugby union player who plays - or played - the English Premiership",
        "data_type": "ExternalId"
    },
    "P1422": {
        "label": "Sandrart.net person ID",
        "description": "personal Identification number in Research Platform for Art and Cultural History of the 17th Century (Sandrart.net)",
        "data_type": "ExternalId"
    },
    "P496": {
        "label": "ORCID iD",
        "description": "identifier for a person",
        "data_type": "ExternalId"
    },
    "P2033": {
        "label": "category for pictures taken with camera",
        "description": "Commons category for photos taken with that camera model or model line",
        "data_type": "WikibaseItem"
    },
    "P1280": {
        "label": "CONOR.SI ID",
        "description": "identifier in the National and University Library, Ljubljana database",
        "data_type": "ExternalId"
    },
    "P428": {
        "label": "botanist author abbreviation",
        "description": "standard form (official abbreviation) of a personal name for use in an author citation (only for names of algae, fungi and plants)",
        "data_type": "ExternalId"
    },
    "P7782": {
        "label": "category for ship name",
        "description": "Commons category for a name of a ship",
        "data_type": "WikibaseItem"
    },
    "P1754": {
        "label": "category related to list",
        "description": "Wikimedia category equivalent to a Wikimedia list",
        "data_type": "WikibaseItem"
    },
    "P2517": {
        "label": "category for recipients of this award",
        "description": "link to Wikimedia category for recipients of this award",
        "data_type": "WikibaseItem"
    },
    "P5996": {
        "label": "category for films in this language",
        "description": "link to category entity contains films in this language",
        "data_type": "WikibaseItem"
    },
    "P1238": {
        "label": "Swedish Football Association player ID",
        "description": "identifier for a Swedish association football (soccer) player's page at Svenskfotboll.se, the official website of the Swedish Football Association (in Swedish: Svenska Fotbollförbundet; SvFF)",
        "data_type": "ExternalId"
    },
    "P1464": {
        "label": "category for people born here",
        "description": "category item that groups people born in this place",
        "data_type": "WikibaseItem"
    },
    "P4195": {
        "label": "category for employees of the organization",
        "description": "Wikimedia category for people who work or worked at this organization",
        "data_type": "WikibaseItem"
    },
    "P1285": {
        "label": "Munzinger Sport number",
        "description": "identifer on the Munzinger Archiv",
        "data_type": "ExternalId"
    },
    "P1233": {
        "label": "Internet Speculative Fiction Database author ID",
        "description": "identifier for a person in the Internet Speculative Fiction Database",
        "data_type": "ExternalId"
    },
    "P1187": {
        "label": "Dharma Drum Buddhist College person ID",
        "description": "identifier for a person in DDBC",
        "data_type": "ExternalId"
    },
    "P1157": {
        "label": "US Congress Bio ID",
        "description": "identifier for a person on the Biographical Directory of the United States Congress",
        "data_type": "ExternalId"
    },
    "P1146": {
        "label": "World Athletics athlete ID",
        "description": "identifier for athletes in World Athletics database and website",
        "data_type": "ExternalId"
    },
    "P1213": {
        "label": "NLC authorities",
        "description": "identifier for a person per National Library of China",
        "data_type": "ExternalId"
    },
    "P1792": {
        "label": "category of associated people",
        "description": "Wikimedia category for people associated with this place or organization",
        "data_type": "WikibaseItem"
    },
    "P1409": {
        "label": "Cycling Archives cyclist ID",
        "description": "identifier in the Cycling Archives",
        "data_type": "ExternalId"
    },
    "P3876": {
        "label": "category for alumni of educational institution",
        "description": "category containing people who studied at this institution",
        "data_type": "WikibaseItem"
    },
    "P7867": {
        "label": "category for maps",
        "description": "name of the Wikimedia category specifically for maps or plans of this item",
        "data_type": "WikibaseItem"
    },
    "P971": {
        "label": "category combines topics",
        "description": "this category combines (intersects) these two or more topics",
        "data_type": "WikibaseItem"
    },
    "P1740": {
        "label": "category for films shot at this location",
        "description": "the object is a category for films shot at or in the subject",
        "data_type": "WikibaseItem"
    },
    "P1200": {
        "label": "bodies of water basin category",
        "description": "the Wikimedia category associated with the basin of a bodies of water",
        "data_type": "WikibaseItem"
    },
    "P213": {
        "label": "ISNI",
        "description": "International Standard Name Identifier for an identity. Format: 4 blocks of 4 digits separated by a space, first block is 0000",
        "data_type": "ExternalId"
    },
    "P373": {
        "label": "Commons category",
        "description": "name of the Wikimedia Commons category containing files related to this item (without the prefix \"Category:\")",
        "data_type": "String"
    },
    "P1315": {
        "label": "NLA Trove ID",
        "description": "identifier for people per National Library of Australia (see also P409 for the older Libraries Australia identifier). Format: 5 to 7 digits.",
        "data_type": "ExternalId"
    },
    "P1255": {
        "label": "HelveticArchives ID",
        "description": "identifier in the Swiss National Library's archive database",
        "data_type": "ExternalId"
    },
    "P4224": {
        "label": "category contains",
        "description": "category contains elements that are instances of this item",
        "data_type": "WikibaseItem"
    },
    "P7861": {
        "label": "category for files created with program",
        "description": "Commons category for files created with that program",
        "data_type": "WikibaseItem"
    },
    "P1017": {
        "label": "Vatican Library ID (former scheme)",
        "description": "historical identifier for authority control used at the Vatican Library. Format: \"ADV\" + 8 digits. Use P8034 for newer IDs.",
        "data_type": "ExternalId"
    },
    "P271": {
        "label": "CiNii author ID (books)",
        "description": "identifier for a book author in CiNii (Scholarly and Academic Information Navigator)",
        "data_type": "ExternalId"
    },
    "P7084": {
        "label": "related category",
        "description": "Wikimedia category is related to this item",
        "data_type": "WikibaseItem"
    },
    "P6112": {
        "label": "category for members of a team",
        "description": "Wikimedia category for players playing for a sports team",
        "data_type": "WikibaseItem"
    },
    "P1392": {
        "label": "ComicBookDB ID",
        "description": "identifier for creators at the online database ComicBookDB",
        "data_type": "ExternalId"
    },
    "P3713": {
        "label": "category for value not in Wikidata",
        "description": "item for categories of pages with template parameter value not as property value in  Wikidata",
        "data_type": "WikibaseItem"
    },
    "P214": {
        "label": "VIAF ID",
        "description": "identifier for the Virtual International Authority File database [format: up to 22 digits]",
        "data_type": "ExternalId"
    },
    "P244": {
        "label": "Library of Congress authority ID",
        "description": "Library of Congress identifier for persons, organizations, events, places, titles, and subject headings  [Format: 1-2 specific letters followed by 8-10 digits (see regex). For manifestations of works, use P1144]",
        "data_type": "ExternalId"
    },
    "P8464": {
        "label": "content partnership category",
        "description": "Wikimedia Commons category for media files contributed by this institution",
        "data_type": "WikibaseItem"
    },
    "P691": {
        "label": "NKCR AUT ID",
        "description": "identifier in the Czech National Authority Database (National Library of Czech Republic)",
        "data_type": "ExternalId"
    },
    "P1220": {
        "label": "Internet Broadway Database person ID",
        "description": "identifier for personnel or organizations on Broadway",
        "data_type": "ExternalId"
    },
    "P1185": {
        "label": "Rodovid ID",
        "description": "identifer for a person on rodovid.org",
        "data_type": "ExternalId"
    },
    "P1266": {
        "label": "AlloCiné person ID",
        "description": "identifier for a person on the AlloCiné film database",
        "data_type": "ExternalId"
    },
    "P1053": {
        "label": "ResearcherID",
        "description": "identifier for a researcher in a system for scientific authors, primarily used in Web of Science",
        "data_type": "ExternalId"
    },
    "P835": {
        "label": "author citation (zoology)",
        "description": "short form of the name used after a taxon this person has authored",
        "data_type": "String"
    },
    "P1288": {
        "label": "Kritisches Lexikon der Gegenwartsliteratur ID",
        "description": "identifier on the Munzinger Archiv",
        "data_type": "ExternalId"
    },
    "P1284": {
        "label": "Munzinger person ID",
        "description": "identifier on the Munzinger Archiv",
        "data_type": "ExternalId"
    },
    "P1791": {
        "label": "category of people buried here",
        "description": "Wikimedia category for people with a burial site within this area",
        "data_type": "WikibaseItem"
    },
    "P396": {
        "label": "SBN author ID",
        "description": "identifier issued by National Library Service (SBN) of Italy",
        "data_type": "ExternalId"
    },
    "P345": {
        "label": "IMDb ID",
        "description": "identifier for the IMDb [with prefix 'tt', 'nm', 'co', 'ev', 'ch' or 'ni']",
        "data_type": "ExternalId"
    },
    "P951": {
        "label": "NSZL (VIAF) ID",
        "description": "identifier from the authority file of the National Széchényi Library, Hungary used only by VIAF",
        "data_type": "ExternalId"
    },
    "P1258": {
        "label": "Rotten Tomatoes ID",
        "description": "identifier on Rotten Tomatoes",
        "data_type": "ExternalId"
    },
    "P1276": {
        "label": "Dictionnaire du Jura ID",
        "description": "identifier in the encyclopedia Dictionnaire du Jura",
        "data_type": "ExternalId"
    },
    "P1263": {
        "label": "NNDB people ID",
        "description": "identifier in the Notable Names Database, a biographical database: only for people entries",
        "data_type": "ExternalId"
    },
    "P1048": {
        "label": "NCL ID",
        "description": "identifier for authority control issued by the National Central Library in Taiwan",
        "data_type": "ExternalId"
    },
    "P497": {
        "label": "CBDB ID",
        "description": "identifer for a person in the China Biographical Database",
        "data_type": "ExternalId"
    },
    "P906": {
        "label": "SELIBR ID",
        "description": "identifier per National Library of Sweden Libris library catalog",
        "data_type": "ExternalId"
    },
    "P1753": {
        "label": "list related to category",
        "description": "Wikimedia list equivalent to a Wikimedia category",
        "data_type": "WikibaseItem"
    },
    "P864": {
        "label": "ACM Digital Library author ID",
        "description": "Association for Computing Machinery Digital Library (ACM DL) author identifier",
        "data_type": "ExternalId"
    },
    "P8963": {
        "label": "National Library at Kolkata romanization",
        "description": "transliteration scheme in dictionaries and grammars of Indic languages",
        "data_type": "String"
    },
    "P9311": {
        "label": "Jyutping transliteration",
        "description": "Jyutping transliteration of Cantonese words",
        "data_type": "String"
    },
    "P8183": {
        "label": "general property-based URL formatter",
        "description": "formatter URL to look up information about a class of items in an online resource. Use with properties generally found on an item. Sample: {P478} for volume or {P304} for page(s), {P433} for issue. Special value {page} for the first value of {P304}",
        "data_type": "String"
    },
    "P3721": {
        "label": "public key fingerprint",
        "description": "short sequence of bytes to identify a longer cryptographic public key",
        "data_type": "String"
    },
    "P3625": {
        "label": "Kirshenbaum code",
        "description": "Kirshenbaum symbol for a IPA phoneme",
        "data_type": "String"
    },
    "P1401": {
        "label": "bug tracking system",
        "description": "web page where bugs, issues, and feature requests for a particular software program can be listed or reported",
        "data_type": "Url"
    },
    "P3228": {
        "label": "Schläfli symbol",
        "description": "notation that defines regular polytopes and tessellations",
        "data_type": "String"
    },
    "P8966": {
        "label": "URL match pattern",
        "description": "regex pattern of URL that an external ID may be extracted. Qualifier \"URL match replacement value\" can overwrite the default \\1. Use non-capturing groups when needed \"(?:www)?\".",
        "data_type": "String"
    },
    "P943": {
        "label": "programmer",
        "description": "the programmer that wrote the piece of software",
        "data_type": "WikibaseItem"
    },
    "P2992": {
        "label": "software quality assurance",
        "description": "quality assurance process in place for a particular software",
        "data_type": "WikibaseItem"
    },
    "P5276": {
        "label": "Slavic phonetic alphabet transcription",
        "description": "system of pronunciation recording",
        "data_type": "String"
    },
    "P4213": {
        "label": "Unicode hex codepoint",
        "description": "hexadecimal codepoint in Unicode",
        "data_type": "String"
    },
    "P8533": {
        "label": "SMARTS notation",
        "description": "notation used for specifying substructural patterns in chemical molecules",
        "data_type": "String"
    },
    "P1932": {
        "label": "stated as",
        "description": "use as qualifier to indicate how the value was given in the source",
        "data_type": "String"
    },
    "P9313": {
        "label": "Yale romanization",
        "description": "Yale romanisation of Cantonese words",
        "data_type": "String"
    },
    "P1782": {
        "label": "courtesy name",
        "description": "name bestowed upon a person at adulthood in addition to one's given name, mostly in East Asia",
        "data_type": "String"
    },
    "P2859": {
        "label": "X-SAMPA code",
        "description": "Extended Speech Assessment Methods Phonetic Alphabet (X-SAMPA) code for phoneme",
        "data_type": "String"
    },
    "P2379": {
        "label": "deprecated in version",
        "description": "software version it was deprecated in",
        "data_type": "WikibaseItem"
    },
    "P6507": {
        "label": "taxon author citation",
        "description": "valid author citation for a taxon using the appropriate nomenclature (ICBN, ICZN, ect.)",
        "data_type": "String"
    },
    "P1618": {
        "label": "sport number",
        "description": "number worn on a player's (competitor's) uniform, equipment, etc",
        "data_type": "String"
    },
    "P751": {
        "label": "introduced feature",
        "description": "feature introduced by this version of a product item",
        "data_type": "WikibaseItem"
    },
    "P1673": {
        "label": "general formula",
        "description": "molecular formula of a class of compounds",
        "data_type": "String"
    },
    "P306": {
        "label": "operating system",
        "description": "operating system (OS) on which a software works or the OS installed on hardware",
        "data_type": "WikibaseItem"
    },
    "P2552": {
        "label": "quantitative metrical pattern",
        "description": "description of a quantitative verse's metric",
        "data_type": "String"
    },
    "P2078": {
        "label": "user manual URL",
        "description": "link to the user manual of the object (product, program)",
        "data_type": "Url"
    },
    "P6798": {
        "label": "UPA transcription",
        "description": "transcript in Uralic Phonetic Alphabet. Use as a qualifier for statements with name properties on items, use as main value on forms for lexemes",
        "data_type": "String"
    },
    "P1032": {
        "label": "Digital Rights Management system",
        "description": "technologies  to control the use of digital content and devices after sale",
        "data_type": "WikibaseItem"
    },
    "P1073": {
        "label": "writable file format",
        "description": "file format a program can create and/or write to",
        "data_type": "WikibaseItem"
    },
    "P8942": {
        "label": "word lookup",
        "description": "formatter to look up a word or a name in database or website. If it appears in search results, use \"search formatter URL\" (P4354). Qualify with \"applies to part\" (P518), if applicable; with \"uses\" (P2283) for conversion (ex: \"lowercase\" Q4444253)",
        "data_type": "String"
    },
    "P1414": {
        "label": "GUI toolkit or framework",
        "description": "framework or toolkit a program uses to display the graphical user interface",
        "data_type": "WikibaseItem"
    },
    "P1641": {
        "label": "port",
        "description": "default communication endpoint in TCP, UDP, or other transport protocol",
        "data_type": "Quantity"
    },
    "P400": {
        "label": "platform",
        "description": "platform for which a work was developed or released, or the specific platform version of a software product",
        "data_type": "WikibaseItem"
    },
    "P1993": {
        "label": "TeX string",
        "description": "string to show a concept in TeX or LaTeX",
        "data_type": "String"
    },
    "P1324": {
        "label": "source code repository",
        "description": "public source code repository",
        "data_type": "Url"
    },
    "P479": {
        "label": "input method",
        "description": "input method or device used to interact with a software product",
        "data_type": "WikibaseItem"
    },
    "P1785": {
        "label": "temple name",
        "description": "name bestowed to a monarch after death (East Asia)",
        "data_type": "String"
    },
    "P1195": {
        "label": "file extension",
        "description": "identifier for a file format (e.g. txt for a text file) used as suffix to the file name, don't use dot at start",
        "data_type": "String"
    },
    "P1547": {
        "label": "depends on software",
        "description": "subject software depends on object software",
        "data_type": "WikibaseItem"
    },
    "P1845": {
        "label": "anti-virus alias",
        "description": "alias issued by anti-virus companies to classify malware",
        "data_type": "String"
    },
    "P8000": {
        "label": "electron configuration",
        "description": "distribution of electrons of an atom or ion, using a standard notation",
        "data_type": "String"
    },
    "P4325": {
        "label": "APA phoneme code",
        "description": "symbol of a phoneme in the Americanist Phonetic Notation",
        "data_type": "String"
    },
    "P756": {
        "label": "removed feature",
        "description": "which feature was removed by this version of a product item",
        "data_type": "WikibaseItem"
    },
    "P1787": {
        "label": "art-name",
        "description": "type of pseudonym of modern and historical artists as well as specifically traditionally adopted by writers and artists in East Asia",
        "data_type": "String"
    },
    "P2599": {
        "label": "block size",
        "description": "size of block for this block cipher or hash function",
        "data_type": "Quantity"
    },
    "P1240": {
        "label": "Danish Bibliometric Research Indicator level",
        "description": "Danish scientific level of research publications, coordinated with the nordic list of publication channels. See also item {{q|Q57408668}}",
        "data_type": "String"
    },
    "P5279": {
        "label": "hyphenation",
        "description": "positions where this form can be hyphenated",
        "data_type": "String"
    },
    "P9021": {
        "label": "Bharati Braille",
        "description": "transcription of Indic scripts in a national standard Braille script",
        "data_type": "String"
    },
    "P3033": {
        "label": "package management system",
        "description": "package management system used to publish the software",
        "data_type": "WikibaseItem"
    },
    "P9317": {
        "label": "Cantonese Pinyin",
        "description": "Cantonese Pinyin transliteration of Cantonese words",
        "data_type": "String"
    },
    "P8460": {
        "label": "applies if regular expression matches",
        "description": "the statement is only true, if the id matches this regular expression",
        "data_type": "String"
    },
    "P408": {
        "label": "software engine",
        "description": "software engine employed by the subject item",
        "data_type": "WikibaseItem"
    },
    "P2957": {
        "label": "throughput",
        "description": "volume of units that do or can pass through or be transported by the object",
        "data_type": "Quantity"
    },
    "P6424": {
        "label": "affiliation string",
        "description": "qualifier to provide the published string form of affiliation attached to an author",
        "data_type": "String"
    },
    "P1786": {
        "label": "posthumous name",
        "description": "name given to a person after death (East Asia)",
        "data_type": "String"
    },
    "P1814": {
        "label": "name in kana",
        "description": "the reading of a Japanese name in kana",
        "data_type": "String"
    },
    "P3921": {
        "label": "Wikidata SPARQL query equivalent",
        "description": "SPARQL code that returns a set of entities that correspond with this category or list. Include ?item",
        "data_type": "String"
    },
    "P2440": {
        "label": "transliteration",
        "description": "conversion of text to alternate script (use as a qualifier for monolingual text statements; please use specific property if possible)",
        "data_type": "String"
    },
    "P1072": {
        "label": "readable file format",
        "description": "file format a program can open and read",
        "data_type": "WikibaseItem"
    },
    "P9373": {
        "label": "Ukrainian national romanization",
        "description": "romanized Ukrainian text method for Ukrainian text (transliteration from Ukrainian Cyrillic alphabet to Latin alphabet)",
        "data_type": "String"
    },
    "P7250": {
        "label": "mobile formatter URL",
        "description": "web page URL of mobile version; URI template from which \"$1\" can be automatically replaced with the effective property value on items",
        "data_type": "String"
    },
    "P277": {
        "label": "programming language",
        "description": "the programming language(s) in which the software is developed",
        "data_type": "WikibaseItem"
    },
    "P1372": {
        "label": "binding of software library",
        "description": "software library in another programming language provided by the subject software binding",
        "data_type": "WikibaseItem"
    },
    "P9323": {
        "label": "Cantonese Transliteration Scheme transliteration",
        "description": "Cantonese Transliteration Scheme transliteration of Cantonese words",
        "data_type": "String"
    },
    "P1347": {
        "label": "military casualty classification",
        "description": "allowed values:  killed in action (Q210392), missing in action (Q2344557), died of wounds (Q16861372), prisoner of war (Q179637), killed in flight accident (Q16861407), others used in military casualty classification",
        "data_type": "WikibaseItem"
    },
    "P4896": {
        "label": "3D model",
        "description": "representation of any three-dimensional surface of an object",
        "data_type": "CommonsMedia"
    },
    "P1120": {
        "label": "number of deaths",
        "description": "total (cumulative) number of people who died since start as a direct result of an event or cause",
        "data_type": "Quantity"
    },
    "P6802": {
        "label": "related image",
        "description": "unfitting image to be used because a better alternative does not exist or is not available due to legal restrictions. Don't use if item has P18. Value should not be a generic placeholder",
        "data_type": "CommonsMedia"
    },
    "P1846": {
        "label": "distribution map",
        "description": "distribution of item on a mapped area (for  range map of taxa, use (P181).)",
        "data_type": "CommonsMedia"
    },
    "P8667": {
        "label": "twin town sign",
        "description": "image of sign with twin towns on it",
        "data_type": "CommonsMedia"
    },
    "P4291": {
        "label": "panoramic view",
        "description": "panoramic view of the object",
        "data_type": "CommonsMedia"
    },
    "P692": {
        "label": "Gene Atlas Image",
        "description": "image showing the GeneAtlas expression pattern",
        "data_type": "CommonsMedia"
    },
    "P570": {
        "label": "date of death",
        "description": "date on which the subject died",
        "data_type": "Time"
    },
    "P41": {
        "label": "flag image",
        "description": "image of the item's flag",
        "data_type": "CommonsMedia"
    },
    "P117": {
        "label": "chemical structure",
        "description": "image of a representation of the structure for a chemical compound",
        "data_type": "CommonsMedia"
    },
    "P4004": {
        "label": "escutcheon image",
        "description": "use for the shield part only: use P94 for a coat of arms",
        "data_type": "CommonsMedia"
    },
    "P3896": {
        "label": "geoshape",
        "description": "geographic data from Wikimedia Commons",
        "data_type": "GeoShape"
    },
    "P1590": {
        "label": "number of casualties",
        "description": "number of people dead or injured because of this event.  Used when precise data on number of deaths and number of injured is unavailable.",
        "data_type": "Quantity"
    },
    "P157": {
        "label": "killed by",
        "description": "person who killed the subject",
        "data_type": "WikibaseItem"
    },
    "P9126": {
        "label": "Commons media contributed by",
        "data_type": "WikibaseItem"
    },
    "P8265": {
        "label": "based on tabular data",
        "description": "Commons tabular data file that a Commons file (or statement) derives from",
        "data_type": "TabularData"
    },
    "P381": {
        "label": "PCP reference number",
        "description": "identifier for cultural properties in Switzerland",
        "data_type": "ExternalId"
    },
    "P1636": {
        "label": "date of baptism in early childhood",
        "description": "date when a person was baptized. For times when only baptism records were available, this can be a good substitute for date of birth.",
        "data_type": "Time"
    },
    "P3383": {
        "label": "film poster",
        "description": "poster used to promote and advertise this film (if file is available on Commons). Use P154 for logos, P18 for movie stills and related images. Items about film posters can be linked with the qualifier \"subject of\" (P805).",
        "data_type": "CommonsMedia"
    },
    "P5555": {
        "label": "schematic",
        "description": "illustration of the structure or operation of an object, system, or process",
        "data_type": "CommonsMedia"
    },
    "P51": {
        "label": "audio",
        "description": "relevant sound. If available, use a more specific property. Samples: \"spoken text audio\" (P989), \"pronunciation audio\" (P443)",
        "data_type": "CommonsMedia"
    },
    "P109": {
        "label": "signature",
        "description": "image of a person's signature",
        "data_type": "CommonsMedia"
    },
    "P8766": {
        "label": "rank insignia",
        "description": "image of the insigna for a rank (such as a military rank)",
        "data_type": "CommonsMedia"
    },
    "P4150": {
        "label": "weather history",
        "description": "link to a Commons tabular data file with the weather history of a certain place",
        "data_type": "TabularData"
    },
    "P207": {
        "label": "bathymetry image",
        "description": "image showing bathymetric chart, bathymetric map",
        "data_type": "CommonsMedia"
    },
    "P2343": {
        "label": "playing range image",
        "description": "image showing the playing range of the instrument",
        "data_type": "CommonsMedia"
    },
    "P3909": {
        "label": "last words",
        "description": "last words attributed to a person before their death",
        "data_type": "Monolingualtext"
    },
    "P3150": {
        "label": "birthday",
        "description": "item for day and month on which the subject was born. Used when full \"date of birth\" (P569) isn't known.",
        "data_type": "WikibaseItem"
    },
    "P935": {
        "label": "Commons gallery",
        "description": "name of the Wikimedia Commons gallery page(s) related to this item (is suitable to allow multiple links to more gallery pages)",
        "data_type": "String"
    },
    "P19": {
        "label": "place of birth",
        "description": "most specific known (e.g. city instead of country, or hospital instead of city) birth location of a person, animal or fictional character",
        "data_type": "WikibaseItem"
    },
    "P1543": {
        "label": "monogram",
        "description": "image of a person's monogram",
        "data_type": "CommonsMedia"
    },
    "P181": {
        "label": "taxon range map image",
        "description": "range map of a taxon",
        "data_type": "CommonsMedia"
    },
    "P1196": {
        "label": "manner of death",
        "description": "general circumstances of a person's death; e.g. natural causes, accident, suicide, homicide, etc.  Use 'cause of death' (P509) for the specific physiological mechanism, e.g. heart attack, trauma, pneumonia...",
        "data_type": "WikibaseItem"
    },
    "P154": {
        "label": "logo image",
        "description": "graphic mark or emblem commonly used by commercial enterprises, organizations and products",
        "data_type": "CommonsMedia"
    },
    "P158": {
        "label": "seal image",
        "description": "image of subject's seal (emblem)",
        "data_type": "CommonsMedia"
    },
    "P1317": {
        "label": "floruit",
        "description": "date when the person was known to be active or alive, when birth or death not documented",
        "data_type": "Time"
    },
    "P443": {
        "label": "pronunciation audio",
        "description": "audio file with pronunciation",
        "data_type": "CommonsMedia"
    },
    "P8517": {
        "label": "view",
        "description": "view from the given site",
        "data_type": "CommonsMedia"
    },
    "P5775": {
        "label": "image of interior",
        "description": "image of the inside of an architectural structure, vehicle or other enclosed space",
        "data_type": "CommonsMedia"
    },
    "P477": {
        "label": "Canadian Register of Historic Places ID",
        "description": "identifier in the Canadian Register of Historic Places",
        "data_type": "ExternalId"
    },
    "P380": {
        "label": "Mérimée ID",
        "description": "identifier for a monument in the Mérimée database of French cultural heritage",
        "data_type": "ExternalId"
    },
    "P1766": {
        "label": "place name sign",
        "description": "image of road sign with place name on it",
        "data_type": "CommonsMedia"
    },
    "P990": {
        "label": "audio recording of the subject's spoken voice",
        "description": "audio file representing the speaking voice of a person; or of an  animated cartoon or other fictitious character",
        "data_type": "CommonsMedia"
    },
    "P3451": {
        "label": "nighttime view",
        "description": "image of the subject at night, or at least in twilight",
        "data_type": "CommonsMedia"
    },
    "P1442": {
        "label": "image of grave",
        "description": "picture of a person or animal's grave, gravestone or tomb",
        "data_type": "CommonsMedia"
    },
    "P4640": {
        "label": "photosphere image",
        "description": "image with the field of view 360×180 degrees",
        "data_type": "CommonsMedia"
    },
    "P2713": {
        "label": "sectional view",
        "description": "image file that shows a sectional view of the item",
        "data_type": "CommonsMedia"
    },
    "P10": {
        "label": "video",
        "description": "relevant video. For images, use the property P18. For film trailers, qualify with \"object has role\" (P3831)=\"trailer\" (Q622550)",
        "data_type": "CommonsMedia"
    },
    "P8204": {
        "label": "tabular case data",
        "description": "tabular data on Wikimedia Commons of confirmed cases, recoveries, deaths, etc. due to a medical event; corresponds to P8011, P1603, P8049, P8010, P1120. Do not delete such statements.",
        "data_type": "TabularData"
    },
    "P4602": {
        "label": "date of burial or cremation",
        "description": "date on which the person was buried or cremated soon after death",
        "data_type": "Time"
    },
    "P989": {
        "label": "spoken text audio",
        "description": "audio file for the spoken work or Wikipedia article, including audio descriptions",
        "data_type": "CommonsMedia"
    },
    "P1621": {
        "label": "detail map",
        "description": "map containing details about the entire location",
        "data_type": "CommonsMedia"
    },
    "P2716": {
        "label": "collage image",
        "description": "image file that assembles two or more other images of item",
        "data_type": "CommonsMedia"
    },
    "P746": {
        "label": "date of disappearance",
        "description": "date or point of time a missing person was seen or otherwise known to be alive for the last time",
        "data_type": "Time"
    },
    "P367": {
        "label": "astronomic symbol image",
        "description": "image of the symbol that identify a planet or an asteroid of the solar system",
        "data_type": "CommonsMedia"
    },
    "P7457": {
        "label": "creator's signature",
        "description": "image of the artist's mark on the work that identifies their work",
        "data_type": "CommonsMedia"
    },
    "P1943": {
        "label": "location map",
        "description": "blank geographic map image suitable for overlaying coordinate points, as defined at c:Category:Location maps",
        "data_type": "CommonsMedia"
    },
    "P1944": {
        "label": "relief location map",
        "description": "relief location map of place",
        "data_type": "CommonsMedia"
    },
    "P14": {
        "label": "traffic sign",
        "description": "graphic symbol describing the item, used at the side of or above roads to give instructions or provide information to road users",
        "data_type": "CommonsMedia"
    },
    "P20": {
        "label": "place of death",
        "description": "most specific known (e.g. city instead of country, or hospital instead of city) death location of a person, animal or fictional character",
        "data_type": "WikibaseItem"
    },
    "P2910": {
        "label": "icon",
        "description": "pictogram suitable to represent the item. For logos of a subject, use \"logo image\" (P154); for favicons, use \"small logo or icon\" (P8972)",
        "data_type": "CommonsMedia"
    },
    "P5252": {
        "label": "winter view",
        "description": "image of the subject at winter, preferrable with snow",
        "data_type": "CommonsMedia"
    },
    "P242": {
        "label": "locator map image",
        "description": "geographic map image which highlights the location of the subject within some larger entity",
        "data_type": "CommonsMedia"
    },
    "P7561": {
        "label": "category for the interior of the item",
        "description": "Commons category for media files depicting interior views of this item (object, place)",
        "data_type": "WikibaseItem"
    },
    "P119": {
        "label": "place of burial",
        "description": "location of grave, resting place, place of ash-scattering, etc. (e.g., town/city or cemetery) for a person or animal. There may be several places: e.g., re-burials, parts of body buried separately.",
        "data_type": "WikibaseItem"
    },
    "P948": {
        "label": "page banner",
        "description": "lead image about the topic, mainly used by Wikivoyages and Women in Red",
        "data_type": "CommonsMedia"
    },
    "P1801": {
        "label": "commemorative plaque image",
        "description": "image of plaque or inscription commemorating or mentioning the subject. For graves, use grave picture (P1442)",
        "data_type": "CommonsMedia"
    },
    "P2425": {
        "label": "service ribbon image",
        "description": "an image depicting the ribbon associated with a medal, order, etc.",
        "data_type": "CommonsMedia"
    },
    "P15": {
        "label": "route map",
        "description": "image of route map at Wikimedia Commons",
        "data_type": "CommonsMedia"
    },
    "P491": {
        "label": "orbit diagram",
        "description": "image with the diagram of the orbit of an astronomical body",
        "data_type": "CommonsMedia"
    },
    "P9058": {
        "label": "Fichier des personnes décédées ID",
        "description": "identifier for an entry in an inoffiicial mirror of INSEE's 'Fichier des personnes décédées' (deaths since 1970)",
        "data_type": "ExternalId"
    },
    "P1472": {
        "label": "Commons Creator page",
        "description": "name of Commons Infobox template residing in \"Creator\" namespace on Wikimedia Commons",
        "data_type": "String"
    },
    "P94": {
        "label": "coat of arms image",
        "description": "image of the item's coat of arms - for the shield part only use P4004",
        "data_type": "CommonsMedia"
    },
    "P8592": {
        "label": "aerial view",
        "description": "image of the subject taken from the air",
        "data_type": "CommonsMedia"
    },
    "P8989": {
        "label": "category for the view of the item",
        "description": "category for the view of this item (object, place)",
        "data_type": "WikibaseItem"
    },
    "P8596": {
        "label": "category for the exterior of the item",
        "description": "category for multimedia files depicting exterior views of this item (object, place)",
        "data_type": "WikibaseItem"
    },
    "P359": {
        "label": "Rijksmonument ID",
        "description": "identifier for a monument assigned by the Rijksdienst voor het Cultureel Erfgoed",
        "data_type": "ExternalId"
    },
    "P509": {
        "label": "cause of death",
        "description": "underlying or immediate cause of death.  Underlying cause (e.g. car accident, stomach cancer) preferred.  Use 'manner of death' (P1196) for broadest category, e.g. natural causes, accident, homicide, suicide",
        "data_type": "WikibaseItem"
    },
    "P8933": {
        "label": "category for the view from the item",
        "description": "category for the view from this item (object, place)",
        "data_type": "WikibaseItem"
    },
    "P5962": {
        "label": "sail emblem",
        "description": "image of figure placed on the sail of sailing boats to mark the boat type",
        "data_type": "CommonsMedia"
    },
    "P996": {
        "label": "document file on Wikimedia Commons",
        "description": "file on Wikimedia Commons related to the content of the source/book/report",
        "data_type": "CommonsMedia"
    },
    "P655": {
        "label": "translator",
        "description": "person who adapts any kind of written text from one language to another",
        "data_type": "WikibaseItem"
    },
    "P1071": {
        "label": "location of creation",
        "description": "place where the item was made; where applicable, location of final assembly",
        "data_type": "WikibaseItem"
    },
    "P1811": {
        "label": "list of episodes",
        "description": "link to the article with the list of episodes for this series",
        "data_type": "WikibaseItem"
    },
    "P528": {
        "label": "catalog code",
        "description": "catalog name of an object, use with qualifier P972",
        "data_type": "String"
    },
    "P1028": {
        "label": "donated by",
        "description": "person or organization who donated the object",
        "data_type": "WikibaseItem"
    },
    "P495": {
        "label": "country of origin",
        "description": "country of origin of this item (creative work, food, phrase, product, etc.)",
        "data_type": "WikibaseItem"
    },
    "P1957": {
        "label": "Wikisource index page",
        "description": "page in Wikisource containing digital and paper pagination of the source",
        "data_type": "Url"
    },
    "P2675": {
        "label": "reply to",
        "description": "the intellectual work to which the subsequent work is a direct reply",
        "data_type": "WikibaseItem"
    },
    "P4647": {
        "label": "location of first performance",
        "description": "location where a work was first debuted, performed or broadcasted",
        "data_type": "WikibaseItem"
    },
    "P2364": {
        "label": "production code",
        "description": "production code of a television episode",
        "data_type": "String"
    },
    "P4082": {
        "label": "image captured with",
        "description": "model of camera used to capture this visual work",
        "data_type": "WikibaseItem"
    },
    "P1684": {
        "label": "inscription",
        "description": "inscriptions, markings and signatures on an object",
        "data_type": "Monolingualtext"
    },
    "P2739": {
        "label": "typeface/font used",
        "description": "style of type used in a work",
        "data_type": "WikibaseItem"
    },
    "P5024": {
        "label": "choral conductor",
        "description": "musical conductor of a choir",
        "data_type": "WikibaseItem"
    },
    "P2242": {
        "label": "Florentine musea catalogue ID",
        "description": "identifier for artworks from the catalogue of the national Florentine musea",
        "data_type": "ExternalId"
    },
    "P4908": {
        "label": "season",
        "description": "season of a television show or podcast series",
        "data_type": "WikibaseItem"
    },
    "P2322": {
        "label": "article ID",
        "description": "identifier for an article in an online publication",
        "data_type": "String"
    },
    "P608": {
        "label": "exhibition history",
        "description": "exhibitions where the item is or was displayed",
        "data_type": "WikibaseItem"
    },
    "P4101": {
        "label": "dissertation submitted to",
        "description": "institution to whom a thesis or dissertation is submitted",
        "data_type": "WikibaseItem"
    },
    "P4646": {
        "label": "representation of",
        "description": "performing arts production the item is a representation of",
        "data_type": "WikibaseItem"
    },
    "P2860": {
        "label": "cites work",
        "description": "citation from one creative work to another",
        "data_type": "WikibaseItem"
    },
    "P2681": {
        "label": "is recto of",
        "description": "the two-dimensional artwork that is on the back (verso) side of this artwork",
        "data_type": "WikibaseItem"
    },
    "P2360": {
        "label": "intended public",
        "description": "this work, product, object or event is intended for, or has been designed to that person or group of people, animals, plants, etc",
        "data_type": "WikibaseItem"
    },
    "P750": {
        "label": "distributed by",
        "description": "distributor of a creative work; distributor for a record label; news agency; film distributor",
        "data_type": "WikibaseItem"
    },
    "P2437": {
        "label": "number of seasons",
        "description": "number of seasons a television or radio series has had",
        "data_type": "Quantity"
    },
    "P1040": {
        "label": "film editor",
        "description": "person who works with the raw footage, selecting shots and combining them into sequences to create a finished motion picture",
        "data_type": "WikibaseItem"
    },
    "P953": {
        "label": "full work available at URL",
        "description": "URL of a web page containing the full body of this item",
        "data_type": "Url"
    },
    "P3174": {
        "label": "art director",
        "description": "person credited as the art director/artistic director of this work",
        "data_type": "WikibaseItem"
    },
    "P1113": {
        "label": "number of episodes",
        "description": "number of episodes in a film/TV/radio series",
        "data_type": "Quantity"
    },
    "P2130": {
        "label": "cost",
        "description": "amount of money spent on building or producing an object",
        "data_type": "Quantity"
    },
    "P3931": {
        "label": "copyright holder",
        "description": "person or organisation who holds the copyright of a work according to the Berne Convention",
        "data_type": "WikibaseItem"
    },
    "P4745": {
        "label": "is proceedings from",
        "description": "this work contains articles presented at the given event",
        "data_type": "WikibaseItem"
    },
    "P2507": {
        "label": "corrigendum / erratum",
        "description": "published notice that corrects or amends a previous publication",
        "data_type": "WikibaseItem"
    },
    "P770": {
        "label": "cause of destruction",
        "description": "item which caused the destruction of the subject item",
        "data_type": "WikibaseItem"
    },
    "P4608": {
        "label": "scenographer",
        "description": "person who designed the stage or film set of this item",
        "data_type": "WikibaseItem"
    },
    "P737": {
        "label": "influenced by",
        "description": "this person, idea, etc. is informed by that other person, idea, etc., e.g. “Heidegger was influenced by Aristotle”",
        "data_type": "WikibaseItem"
    },
    "P3275": {
        "label": "storyboard artist",
        "description": "person credited as the storyboard artist of this work",
        "data_type": "WikibaseItem"
    },
    "P1191": {
        "label": "date of first performance",
        "description": "date a work was first debuted, performed or live-broadcasted",
        "data_type": "Time"
    },
    "P856": {
        "label": "official website",
        "description": "URL of the official homepage of an item (current or former) [if the homepage changes, add an additional statement with preferred rank. Do not remove the former URL]",
        "data_type": "Url"
    },
    "P5880": {
        "label": "video designer",
        "description": "person responsible for the creation and integration of film and motion graphics on a stage",
        "data_type": "WikibaseItem"
    },
    "P5059": {
        "label": "modified version of",
        "description": "indicates the work or one of its versions which served as a basis for the adaptation or arrangement resulting in the given version of the work",
        "data_type": "WikibaseItem"
    },
    "P4969": {
        "label": "derivative work",
        "description": "new work of art (film, book, software, etc.) derived from major part of this work",
        "data_type": "WikibaseItem"
    },
    "P5026": {
        "label": "lighting designer",
        "description": "person responsible for lighting on a stage",
        "data_type": "WikibaseItem"
    },
    "P2634": {
        "label": "sitter",
        "description": "person who posed during the creation of a work, whether or not that person is eventually depicted as oneself",
        "data_type": "WikibaseItem"
    },
    "P2551": {
        "label": "used metre",
        "description": "rhythmic structure of the poetic text or musical piece",
        "data_type": "WikibaseItem"
    },
    "P740": {
        "label": "location of formation",
        "description": "location where a group or organization was formed",
        "data_type": "WikibaseItem"
    },
    "P825": {
        "label": "dedicated to",
        "description": "person or organization to whom the subject was dedicated",
        "data_type": "WikibaseItem"
    },
    "P1639": {
        "label": "pendant of",
        "description": "other work in a pair of opposing artworks, such as wedding portraits, commissioned together, but not always",
        "data_type": "WikibaseItem"
    },
    "P4510": {
        "label": "describes a project that uses",
        "description": "essential component or tool that was used in the project described in this publication",
        "data_type": "WikibaseItem"
    },
    "P2913": {
        "label": "date depicted",
        "description": "date represented in a work",
        "data_type": "Time"
    },
    "P2142": {
        "label": "box office",
        "description": "box office takings accumulated by a film",
        "data_type": "Quantity"
    },
    "P2682": {
        "label": "is verso of",
        "description": "the two-dimensional artwork that is on the front (recto) side of this artwork",
        "data_type": "WikibaseItem"
    },
    "P5396": {
        "label": "Online Books Page publication ID",
        "description": "identifier for a publication, at the Online Books Page website with copyright info",
        "data_type": "ExternalId"
    },
    "P4032": {
        "label": "reviewed by",
        "description": "person who reviewed/refereed the work, e.g., scientific peer-reviewer",
        "data_type": "WikibaseItem"
    },
    "P2754": {
        "label": "production date",
        "description": "date the production of a film/audio drama was completed (\"production year\") when much earlier than publication date. Also for some other works (opera?). In general, use \"inception\" (P571). Note \"date of first performance\" (P1191) for theatre/opera.",
        "data_type": "Time"
    },
    "P767": {
        "label": "contributor to the creative work or subject",
        "description": "person or organization that contributed to a subject: co-creator of a creative work or subject",
        "data_type": "WikibaseItem"
    },
    "P1257": {
        "label": "depicts Iconclass notation",
        "description": "Iconclass code depicted in an artwork. For linking Iconclass codes with their corresponding artistic themes or concepts, use P1256 (Iconclass notation).",
        "data_type": "String"
    },
    "P3912": {
        "label": "newspaper format",
        "description": "physical size of a newspaper (berliner, broadsheet, tabloid, etc.)",
        "data_type": "WikibaseItem"
    },
    "P820": {
        "label": "arXiv classification",
        "description": "arXiv classification of pre-print articles",
        "data_type": "String"
    },
    "P3132": {
        "label": "last line",
        "description": "last line (excipit) of a poem, last sentence of a novel, etc.",
        "data_type": "Monolingualtext"
    },
    "P629": {
        "label": "edition or translation of",
        "description": "is an edition or translation of this entity",
        "data_type": "WikibaseItem"
    },
    "P4805": {
        "label": "make-up artist",
        "description": "person responsible for the make-up of cast members in a theatrical production, TV or film",
        "data_type": "WikibaseItem"
    },
    "P2635": {
        "label": "number of parts of this work",
        "description": "the number of parts that a work consists of",
        "data_type": "Quantity"
    },
    "P941": {
        "label": "inspired by",
        "description": "work, human, place or event which inspired this creative work or fictional entity",
        "data_type": "WikibaseItem"
    },
    "P1638": {
        "label": "codename",
        "description": "temporary name of a product or project used during its development",
        "data_type": "Monolingualtext"
    },
    "P747": {
        "label": "has edition or translation",
        "description": "link to an edition of this item",
        "data_type": "WikibaseItem"
    },
    "P2515": {
        "label": "costume designer",
        "description": "person who designed the costumes for a film, television programme, etc",
        "data_type": "WikibaseItem"
    },
    "P2061": {
        "label": "aspect ratio",
        "description": "image width to height ratio of films, photographs, and other images",
        "data_type": "WikibaseItem"
    },
    "P5126": {
        "label": "assistant director",
        "description": "assistant stage director of a performing arts, film or television production",
        "data_type": "WikibaseItem"
    },
    "P725": {
        "label": "voice actor",
        "description": "performer of a spoken role in a creative work such as animation, video game, radio drama, or dubbing over [use \"character role\" (P453) as qualifier] [use \"cast member\" (P161) for live acting]",
        "data_type": "WikibaseItem"
    },
    "P1656": {
        "label": "unveiled by",
        "description": "person who unveils a statue, sculpture, memorial or plaque, etc.",
        "data_type": "WikibaseItem"
    },
    "P1809": {
        "label": "choreographer",
        "description": "person(s) who did the choreography",
        "data_type": "WikibaseItem"
    },
    "P921": {
        "label": "main subject",
        "description": "primary topic of a work (see also P180: depicts)",
        "data_type": "WikibaseItem"
    },
    "P1877": {
        "label": "after a work by",
        "description": "artist whose work strongly inspired/ was copied in this item",
        "data_type": "WikibaseItem"
    },
    "P3080": {
        "label": "game artist",
        "description": "game artist(s) that produced art assets for a role-playing games, collectible card games, video game, etc.",
        "data_type": "WikibaseItem"
    },
    "P1922": {
        "label": "first line",
        "description": "first line (incipit) of a poem, first sentence of a novel, etc.",
        "data_type": "Monolingualtext"
    },
    "P2512": {
        "label": "series spin-off",
        "description": "series' spin-offs",
        "data_type": "WikibaseItem"
    },
    "P5955": {
        "label": "Sekolah Kita ID",
        "description": "identifier for a school in Indonesia (early childhood education, primary education, and  secondary education)",
        "data_type": "ExternalId"
    },
    "P5868": {
        "label": "MobyGames platform ID",
        "description": "identifier for a platform in MobyGames",
        "data_type": "ExternalId"
    },
    "P5590": {
        "label": "oric.org identifier",
        "description": "an identifier in the oric.org database of Oric computer programs",
        "data_type": "ExternalId"
    },
    "P5600": {
        "label": "U-Multirank university ID",
        "description": "identifier for a university, on the U-Multirank website",
        "data_type": "ExternalId"
    },
    "P5456": {
        "label": "TasteAtlas ID",
        "description": "identifier for an item at TasteAtlas",
        "data_type": "ExternalId"
    },
    "P5971": {
        "label": "Europe PlayStation Store ID",
        "description": "identifier for a game available on the PlayStation Store (EU region)",
        "data_type": "ExternalId"
    },
    "P5445": {
        "label": "Language Council of Norways termwiki ID",
        "description": "Identifier for a term in the Norwegian language, as given by an entry in the Language Council of Norways termwiki. This is a terminology database for academic disciplines. The terms are usually given as Bokmål, Nynorsk, and English variants.",
        "data_type": "ExternalId"
    },
    "P5458": {
        "label": "Guide to Pharmacology Target ID",
        "description": "target identifier of the Guide to Pharmacology database",
        "data_type": "ExternalId"
    },
    "P5659": {
        "label": "VGMdb product ID",
        "description": "identifier for a product (game, franchise, video) in the Video Game Music database",
        "data_type": "ExternalId"
    },
    "P5933": {
        "label": "tweet ID",
        "description": "identifier of a status on Twitter",
        "data_type": "ExternalId"
    },
    "P5490": {
        "label": "artist-info location ID",
        "description": "identifier in the artist-info location database of exhibition records",
        "data_type": "ExternalId"
    },
    "P5452": {
        "label": "MyHeritage Surname ID",
        "description": "string identifying a surname (family name) at the MyHeritage genealogy website",
        "data_type": "ExternalId"
    },
    "P6009": {
        "label": "Techopedia ID",
        "description": "identifier for an entry in the techopedia.com technology dictionary",
        "data_type": "ExternalId"
    },
    "P5580": {
        "label": "Norwegian National Road DataBase ID",
        "description": "Database ID in the Norwegian National Road DataBase",
        "data_type": "ExternalId"
    },
    "P5453": {
        "label": "Logicielsmoto identifier",
        "description": "identifier in the Logicielsmoto database of Thomson software",
        "data_type": "ExternalId"
    },
    "P5719": {
        "label": "National Medal of Arts winner ID",
        "description": "identifier for a National Medal of Arts winner on the United States' National Endowment for the Arts website",
        "data_type": "ExternalId"
    },
    "P5699": {
        "label": "Transport Canada LID",
        "description": "unique identifier assigned by Transport Canada for airports, heliports, train stations or other transport/travel-related points of interest",
        "data_type": "ExternalId"
    },
    "P6015": {
        "label": "Handbook of Texas ID",
        "description": "identifier for a subject in the Handbook of Texas Online",
        "data_type": "ExternalId"
    },
    "P6011": {
        "label": "Internet Pinball Database ID",
        "description": "identifier for pinball machines at the Internet Pinball Database",
        "data_type": "ExternalId"
    },
    "P5500": {
        "label": "IPHAN ID",
        "description": "The IPHAN identifier for a historic place or good",
        "data_type": "ExternalId"
    },
    "P5922": {
        "label": "ANZSRC 2008 FoR ID",
        "description": "Australia and New Zealand Standard Research Classification 2008 identifier for a field of research.  1419 nodes at 3 levels: 22 Divisions, 157 Groups, 1340 Fields",
        "data_type": "ExternalId"
    },
    "P5528": {
        "label": "Belgian Heritage in Brazil ID",
        "description": "identifier for notorious individuals, companies and artworks associated to the Belgian heritage in Brazil",
        "data_type": "ExternalId"
    },
    "P5986": {
        "label": "Getty Iconography Authority ID",
        "description": "identifier from the Getty Iconography Authority",
        "data_type": "ExternalId"
    },
    "P5780": {
        "label": "CPCRulez ID",
        "description": "identifier in the CPCRulez database of Amstrad CPC videogames",
        "data_type": "ExternalId"
    },
    "P5530": {
        "label": "Altmetric ID",
        "description": "identifier assigned to a scientific or academic article by altmetrics.com, to track the citation metrics",
        "data_type": "ExternalId"
    },
    "P5884": {
        "label": "Dapodikdasmen ID",
        "description": "identifier for a school in Indonesia",
        "data_type": "ExternalId"
    },
    "P5573": {
        "label": "archINFORM location ID",
        "description": "identifier for a location on the archINFORM website",
        "data_type": "ExternalId"
    },
    "P5574": {
        "label": "CAGE code",
        "description": "identifier for a supplier to government agencies",
        "data_type": "ExternalId"
    },
    "P5585": {
        "label": "SMS Power identifier",
        "description": "identifier of a video game in the SMS Power database of Sega 8-bit console games",
        "data_type": "ExternalId"
    },
    "P5586": {
        "label": "Times Higher Education World University ID",
        "description": "Times Higher Education World University Ranking Website's university identifier",
        "data_type": "ExternalId"
    },
    "P5888": {
        "label": "Arkivportalen archive ID",
        "description": "identifier for an archive in Arkivportalen, a website for Norwegian archives",
        "data_type": "ExternalId"
    },
    "P5749": {
        "label": "Amazon Standard Identification Number",
        "description": "identifier for a product on Amazon.com websites",
        "data_type": "ExternalId"
    },
    "P5737": {
        "label": "Chinese Moegirlpedia ID",
        "description": "Link to entry of Chinese articles on Moegirlpedia",
        "data_type": "ExternalId"
    },
    "P5799": {
        "label": "AustLII ID",
        "description": "identifier in the Australasian Legal Information Institute (AustLII) online database",
        "data_type": "ExternalId"
    },
    "P5794": {
        "label": "Internet Game Database game ID",
        "description": "identifier for a game on the Internet Game Database",
        "data_type": "ExternalId"
    },
    "P5686": {
        "label": "ITU-R Recommendation",
        "description": "identifier of an ITU-R Recommendation",
        "data_type": "ExternalId"
    },
    "P5743": {
        "label": "SR Number",
        "description": "Number for each law in the Systematic Compilation of Federal Legislation of Switzerland",
        "data_type": "ExternalId"
    },
    "P5579": {
        "label": "flight-report.com Airline identifier",
        "data_type": "ExternalId"
    },
    "P5688": {
        "label": "ITU-T Recommendation",
        "description": "identifier of an ITU-T Recommendation",
        "data_type": "ExternalId"
    },
    "P5718": {
        "label": "PAM ID",
        "description": "identifier for a person or organization on the 'Portail des Arts de la Marionnette' website",
        "data_type": "ExternalId"
    },
    "P6013": {
        "label": "Encyclopedia of Appalachia ID",
        "description": "identifier for a subject on the Encyclopedia of Appalachia website",
        "data_type": "ExternalId"
    },
    "P6016": {
        "label": "Tennessee Encyclopedia ID",
        "description": "identifier for a subject on the Tennessee Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P5727": {
        "label": "Partis d'Arcadie ID",
        "description": "identifier for a political party on 'Les Partis d'Arcadie'",
        "data_type": "ExternalId"
    },
    "P5777": {
        "label": "CoinMarketCap cryptocurrency ID",
        "description": "identifier for a cryptocurrency on CoinMarketCap",
        "data_type": "ExternalId"
    },
    "P5846": {
        "label": "AnimeClick novel ID",
        "description": "identifier of novel from AnimeClick database",
        "data_type": "ExternalId"
    },
    "P5887": {
        "label": "Arkivportalen agent ID",
        "description": "identifier for an actor (person or institution) in Arkivportalen, a website for Norwegian archives",
        "data_type": "ExternalId"
    },
    "P6010": {
        "label": "Encyclopedia of Alabama ID",
        "description": "identifier for a subject on the Encyclopedia of Alabama website",
        "data_type": "ExternalId"
    },
    "P5760": {
        "label": "Petit Futé site ID",
        "description": "identifier for a tourist site on the Petit Futé website",
        "data_type": "ExternalId"
    },
    "P5744": {
        "label": "KERIS school ID",
        "description": "identifier for a school in the information system by KERIS",
        "data_type": "ExternalId"
    },
    "P5726": {
        "label": "Pietre della Memoria ID",
        "description": "identifier on the 'Pietre della Memoria' website",
        "data_type": "ExternalId"
    },
    "P5501": {
        "label": "Brenda Tissue Ontology ID",
        "data_type": "ExternalId"
    },
    "P5801": {
        "label": "Academia.edu topic ID",
        "description": "identifier for a research topic on the Academia.edu website",
        "data_type": "ExternalId"
    },
    "P5813": {
        "label": "MusicBrainz release ID",
        "description": "identifier for a release in the MusicBrainz open music encyclopedia (edition)",
        "data_type": "ExternalId"
    },
    "P5803": {
        "label": "VOR/DME identifier for airport beacons",
        "description": "VOR/DME identifier for airport beacons",
        "data_type": "ExternalId"
    },
    "P5902": {
        "label": "Uralonet ID",
        "data_type": "ExternalId"
    },
    "P5755": {
        "label": "EU project RCN",
        "description": "Record Control Number for project under EU Framework Programmes for Research and Technological Development)",
        "data_type": "ExternalId"
    },
    "P5762": {
        "label": "NDB identifier for airport beacons",
        "description": "NDB identifier for airport beacons",
        "data_type": "ExternalId"
    },
    "P5768": {
        "label": "Osservatorio culturale del Cantone Ticino ID",
        "description": "Osservatorio culturale del Cantone Ticino ID for a cultural institution",
        "data_type": "ExternalId"
    },
    "P5785": {
        "label": "EU Research participant ID",
        "description": "ID of organization in EU's Framework Programs for Research",
        "data_type": "ExternalId"
    },
    "P5795": {
        "label": "Internet Game Database platform ID",
        "description": "identifier for a platform on the Internet Game Database",
        "data_type": "ExternalId"
    },
    "P5835": {
        "label": "World Travel Awards ID",
        "description": "identifier for a nominee on the World Travel Awards website",
        "data_type": "ExternalId"
    },
    "P5457": {
        "label": "Conseil constitutionnel ID",
        "description": "identifier of a member of the 'Conseil constitutionnel' on its website",
        "data_type": "ExternalId"
    },
    "P5821": {
        "label": "ArhivX LOD",
        "description": "Croatian art/theater linked open data",
        "data_type": "ExternalId"
    },
    "P5944": {
        "label": "North America PlayStation Store ID",
        "description": "identifier for a game available on the PlayStation Store (NA region)",
        "data_type": "ExternalId"
    },
    "P5950": {
        "label": "São Paulo: os estrangeiros e a construção da cidade ID",
        "description": "unique identifier for an artwork in the database \"São Paulo: os estrangeiros e a construção da cidade\"",
        "data_type": "ExternalId"
    },
    "P5892": {
        "label": "UOL Eleições ID",
        "description": "іdentifier for elections in Brazil containing voting data for each position per State",
        "data_type": "ExternalId"
    },
    "P5843": {
        "label": "Gynopedia ID",
        "description": "identifier for a place (city, region or country) on the Gynopedia wiki about sexual and reproductive health care",
        "data_type": "ExternalId"
    },
    "P5838": {
        "label": "Nintendo GameID",
        "description": "Six-alphanumeric-character Nintendo GameID for a specific game",
        "data_type": "ExternalId"
    },
    "P5860": {
        "label": "AnimeClick drama ID",
        "description": "identifier of drama from AnimeClick database",
        "data_type": "ExternalId"
    },
    "P5874": {
        "label": "re3data repository ID",
        "description": "identifier for a data repository in the re3data registry",
        "data_type": "ExternalId"
    },
    "P5912": {
        "label": "Oqaasileriffik online dictionary ID",
        "description": "ID in the online dictionary of Oqaasileriffik",
        "data_type": "ExternalId"
    },
    "P5507": {
        "label": "artist-info exhibition ID",
        "description": "іdentifier in the artist-info database for exhibition records",
        "data_type": "ExternalId"
    },
    "P5536": {
        "label": "Swedish place name register SOFI",
        "description": "this is a register of names found over time for old places in a parish that is managed by the Swedish Institute for Language and Folklore -  SOFI",
        "data_type": "ExternalId"
    },
    "P5851": {
        "label": "Ex-USSR internal airport code",
        "description": "airport code similar to IATA codes but only for Russian and CEI airports",
        "data_type": "ExternalId"
    },
    "P5870": {
        "label": "Northern Ireland Assembly ID",
        "description": "Identifier for a member of the Northern Ireland Assembly",
        "data_type": "ExternalId"
    },
    "P5875": {
        "label": "ResearchGate publication ID",
        "description": "identifier of a publication in ResearchGate",
        "data_type": "ExternalId"
    },
    "P5866": {
        "label": "LakeNet ID",
        "description": "number of a lake in Global Lake Database",
        "data_type": "ExternalId"
    },
    "P5906": {
        "label": "DxOMark ID",
        "description": "Dxomark article identifier for smartphones and cameras",
        "data_type": "ExternalId"
    },
    "P5903": {
        "label": "Álgu ID",
        "data_type": "ExternalId"
    },
    "P5930": {
        "label": "Open Food Facts ingredient ID",
        "description": "identifier of a food ingredient on Open Food Facts",
        "data_type": "ExternalId"
    },
    "P5936": {
        "label": "old-computers.com ID",
        "description": "identifier in the database of computing platforms old-computers.com",
        "data_type": "ExternalId"
    },
    "P5928": {
        "label": "Ġabra lexeme ID",
        "description": "identifier for a lexeme in the Ġabra lexicon",
        "data_type": "ExternalId"
    },
    "P5914": {
        "label": "IANA Root Zone Database ID",
        "description": "identifier in IANA's Root Zone Database of delegation details for top-level Internet domains",
        "data_type": "ExternalId"
    },
    "P5925": {
        "label": "Moviepilot.de series ID",
        "description": "identifier for TV series on Moviepilot.de",
        "data_type": "ExternalId"
    },
    "P5584": {
        "label": "QS World University ID",
        "description": "identifer for an educational institution, in the QS World University database",
        "data_type": "ExternalId"
    },
    "P5604": {
        "label": "archINFORM keyword ID",
        "description": "identifier for a keyword on the archINFORM website",
        "data_type": "ExternalId"
    },
    "P5954": {
        "label": "Museu Nacional ID",
        "description": "online catalog ID for Museu Nacional",
        "data_type": "ExternalId"
    },
    "P5494": {
        "label": "GameSpot game ID",
        "description": "identifier at the GameSpot database of video games",
        "data_type": "ExternalId"
    },
    "P5656": {
        "label": "MuIS collection ID",
        "description": "identifier for a collection in the Estonian museum database MuIS",
        "data_type": "ExternalId"
    },
    "P5467": {
        "label": "GlyphWiki ID",
        "description": "identifier of GlyphWiki, in which glyphs of Han characters are managed",
        "data_type": "ExternalId"
    },
    "P5581": {
        "label": "NATO Stock Number",
        "description": "13-digit numeric code identifying an object used by NATO member country armed forces",
        "data_type": "ExternalId"
    },
    "P5814": {
        "label": "IANA service name",
        "description": "name of a network service defined in Q30335969",
        "data_type": "ExternalId"
    },
    "P5787": {
        "label": "Ski Jumping Hill Archive ID",
        "description": "identifier for a ski jumping hill on the Ski Jumping Hill Archive website",
        "data_type": "ExternalId"
    },
    "P5806": {
        "label": "SNOMED CT identifier",
        "description": "identifier in the SNOMED CT catalogue codes for diseases, symptoms and procedures",
        "data_type": "ExternalId"
    },
    "P5779": {
        "label": "CPAN project ID",
        "description": "name of a project in CPAN",
        "data_type": "ExternalId"
    },
    "P5495": {
        "label": "Mormon Literature and Creative Arts Database Works ID",
        "description": "identifier for a work listed in the Mormon Literature and Creative Arts Database",
        "data_type": "ExternalId"
    },
    "P5981": {
        "label": "Encyclopedia Virginia ID",
        "description": "identifier of a topic in the online Encyclopedia Virginia",
        "data_type": "ExternalId"
    },
    "P5531": {
        "label": "Central Index Key",
        "description": "number given to an individual or company by the United States Securities and Exchange Commission",
        "data_type": "ExternalId"
    },
    "P4802": {
        "label": "BVPB authority ID",
        "description": "identifier of an author, topic or place in Biblioteca Virtual del Patrimonio Bibliográfico",
        "data_type": "ExternalId"
    },
    "P4799": {
        "label": "Rugby Australia ID",
        "description": "identifier for a rugby union player selected with the Australia national team on the Rugby Australia website",
        "data_type": "ExternalId"
    },
    "P5034": {
        "label": "National Library of Korea ID",
        "description": "identifier for a person in the database of the National Library of Korea",
        "data_type": "ExternalId"
    },
    "P5014": {
        "label": "Welsh Rugby Union men's sevens player ID",
        "description": "identifier for a men's rugby sevens player selected with the Wales national team on the Welsh Rugby Union website",
        "data_type": "ExternalId"
    },
    "P4615": {
        "label": "WAFL FootyFacts ID",
        "description": "identifier for an Australian rules football player on the WAFL FootyFacts website",
        "data_type": "ExternalId"
    },
    "P5007": {
        "label": "Behind The Voice Actors person ID",
        "description": "identifier of a person on the website Behind The Voice Actors",
        "data_type": "ExternalId"
    },
    "P4880": {
        "label": "FFR Sevens player ID",
        "description": "identifier for a rugby sevens player on the French Rugby Federation website",
        "data_type": "ExternalId"
    },
    "P4751": {
        "label": "Sports-Reference.com college basketball coach ID",
        "description": "identifier for a NCAA Division I college men's basketball coach on the Sports-Reference.com college basketball website",
        "data_type": "ExternalId"
    },
    "P4631": {
        "label": "Iowa Sports Hall of Fame ID",
        "description": "identifier for an Iowa Sports Hall of Fame inductee",
        "data_type": "ExternalId"
    },
    "P4698": {
        "label": "Conseil de Presse Luxembourg journalist ID",
        "description": "identifier for a journalist at the Conseil de Presse Luxembourg, the body that governs the professional title of journalist in Luxembourg",
        "data_type": "ExternalId"
    },
    "P4585": {
        "label": "Accademia della Crusca ID",
        "description": "identifier for a member of Accademia della Crusca",
        "data_type": "ExternalId"
    },
    "P4588": {
        "label": "International World Games Association athlete ID",
        "description": "identifier for an athlete on the International World Games Association website",
        "data_type": "ExternalId"
    },
    "P4892": {
        "label": "Siprojuris ID",
        "description": "identifier for a jurist in the database Siprojuris",
        "data_type": "ExternalId"
    },
    "P4795": {
        "label": "Basketball-Reference.com referee ID",
        "description": "identifier for a referee on the Basketball Reference website",
        "data_type": "ExternalId"
    },
    "P4937": {
        "label": "Japan Rugby Football Union men's player ID",
        "description": "identifier for a men's rugby union player selected with the Japan national team on the Japan Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4781": {
        "label": "MYmovies director ID",
        "description": "identifier for a director in MYmovies.it",
        "data_type": "ExternalId"
    },
    "P4782": {
        "label": "Movieplayer person ID",
        "description": "identifier for a character in movieplayer.it",
        "data_type": "ExternalId"
    },
    "P4785": {
        "label": "MYmovies name ID",
        "description": "identifier for producer, cinematographer, screenwriter and company in MYmovies.it",
        "data_type": "ExternalId"
    },
    "P4890": {
        "label": "EPHE ID",
        "description": "identifier of a researcher on the online prosopographical dictionary of the EPHE",
        "data_type": "ExternalId"
    },
    "P4813": {
        "label": "PARES ID",
        "description": "identifier for a person, place, or topic on the Spanish Archives Portal",
        "data_type": "ExternalId"
    },
    "P5018": {
        "label": "Chile Rugby player ID",
        "description": "identifier for a rugby union player selected with the Chile national team on the Chile Rugby website",
        "data_type": "ExternalId"
    },
    "P4642": {
        "label": "Colorado Sports Hall of Fame ID",
        "description": "identifier for an athlete on the Colorado Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4717": {
        "label": "Académie française member ID",
        "description": "identifier for a member of the Académie française on its website",
        "data_type": "ExternalId"
    },
    "P5033": {
        "label": "Filmweb.pl person ID",
        "description": "identifier for a person at filmweb.pl",
        "data_type": "ExternalId"
    },
    "P4925": {
        "label": "Rugby Football Union women's player ID",
        "description": "identifier for a women's rugby union player selected with the England national team on the Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4942": {
        "label": "Societe.com director ID",
        "description": "ID for the director of a French company on Societe.com",
        "data_type": "ExternalId"
    },
    "P4883": {
        "label": "FFF male player ID (former scheme)",
        "description": "identifier for a male player on the French Football Federation website",
        "data_type": "ExternalId"
    },
    "P4822": {
        "label": "La Poste personality ID",
        "description": "identifier for a person in the online biographical dictionary of the French 'Comité pour d'Histoire de la Poste'",
        "data_type": "ExternalId"
    },
    "P4668": {
        "label": "USA Gymnastics Hall of Fame athlete ID",
        "description": "identifier for an inductee on the USA Gymnastics Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4885": {
        "label": "AFL Tables coach ID",
        "description": "ID for a VFL/AFL coach at afltables.com",
        "data_type": "ExternalId"
    },
    "P4893": {
        "label": "X Games athlete ID",
        "description": "identifier for a X Games athlete at xgames.com",
        "data_type": "ExternalId"
    },
    "P4727": {
        "label": "Deutsche Synchronkartei dubbing voice actor ID",
        "description": "identifier for a dubbing voice actor, in the synchronisation (dubbing) database Deutsche Synchronkartei",
        "data_type": "ExternalId"
    },
    "P4731": {
        "label": "The Baseball Cube player ID",
        "description": "identifier of a baseball player at thebaseballcube.com",
        "data_type": "ExternalId"
    },
    "P4836": {
        "label": "Irish Rugby Football Union women's player ID",
        "description": "Identifier for a female rugby union player selected with Ireland women's national team on the Irish Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4707": {
        "label": "Rugby Canada ID",
        "description": "identifier for a rugby union player selected with the Canada national team on the Rugby Canada website",
        "data_type": "ExternalId"
    },
    "P4886": {
        "label": "FFF female player ID (former scheme)",
        "description": "identifier for a female player on the French Football Federation website",
        "data_type": "ExternalId"
    },
    "P4831": {
        "label": "Bandysidan player ID",
        "description": "property for bandy players ID from bandysidan.nu",
        "data_type": "ExternalId"
    },
    "P4601": {
        "label": "Lives of WWI ID",
        "description": "identifier for a British or Commonwealth person who served during the first World War, in the Imperial War Museums' 'Lives of the First World War' database",
        "data_type": "ExternalId"
    },
    "P4682": {
        "label": "Eldoblaje original actor ID",
        "description": "identifier for an original actor in a Spanish dubbed film",
        "data_type": "ExternalId"
    },
    "P4923": {
        "label": "Rugby Football Union men's sevens player ID",
        "description": "identifier for a men's rugby sevens player selected with the England national team on the Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4922": {
        "label": "Rugby Football Union women's sevens player ID",
        "description": "identifier for a women's rugby sevens player selected with the England national team on the Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4891": {
        "label": "Patrons de France ID",
        "description": "identifier of a business executive on Patrons de France",
        "data_type": "ExternalId"
    },
    "P4612": {
        "label": "LKL player ID",
        "description": "identifier for a basketball player on the official website of the Lietuvos krepšinio lyga, the main league in Lithuania",
        "data_type": "ExternalId"
    },
    "P4949": {
        "label": "Irish Rugby Football Union men's sevens player ID",
        "description": "identifier for a men's rugby sevens player selected with the Ireland national team on the Irish Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4976": {
        "label": "Scottish Rugby Union women's player ID",
        "description": "identifier for a women's rugby union player selected with the Scotland national team on the Scottish Rugby Union website",
        "data_type": "ExternalId"
    },
    "P4605": {
        "label": "South Dakota Sports Hall of Fame ID",
        "description": "identifier for an athlete on the South Dakota Sports Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4939": {
        "label": "BFMTV.com director ID",
        "description": "ID for the director of a French company on dirigeants.bfmtv.com",
        "data_type": "ExternalId"
    },
    "P4938": {
        "label": "Japan Rugby Football Union women's player ID",
        "description": "identifier for a women's rugby union player selected with the Japan national team on the Japan Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4941": {
        "label": "Japan Rugby Football Union men's sevens player ID",
        "description": "identifier for a men's rugby sevens player selected with the Japan national team on the Japan Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4944": {
        "label": "Premiers préfets ID",
        "description": "identifier of a prefect on the online version of ''Les Préfets du 11 ventôse an VIII au 4 septembre 1870''",
        "data_type": "ExternalId"
    },
    "P4720": {
        "label": "Basketball-Reference.com WNBA coach ID",
        "description": "identifier for a WNBA coach on the Basketball Reference website",
        "data_type": "ExternalId"
    },
    "P4940": {
        "label": "Japan Rugby Football Union women's sevens player ID",
        "description": "identifier for a women's rugby sevens player selected with the Japan national team on the Japan Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4950": {
        "label": "Irish Rugby Football Union women's sevens player ID",
        "description": "identifier for a women's rugby sevens player selected with the Ireland national team on the Irish Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4759": {
        "label": "Luminous-Lint ID",
        "description": "identifier for a photographer in the Luminous-Lint database",
        "data_type": "ExternalId"
    },
    "P4920": {
        "label": "England Rugby men's player ID",
        "description": "identifier for a men's rugby union player selected with the England national team on the Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P4660": {
        "label": "CPDOC ID",
        "description": "identifier for a bibliographic record in the Center for Research and Documentation of Contemporary History of Brazil (CPDOC)",
        "data_type": "ExternalId"
    },
    "P4830": {
        "label": "Swedish Football Association national team player ID",
        "description": "identifier for a Swedish international association football (soccer) player's page from Landslagsdatabasen at Svenskfotboll.se, the official website of the Swedish Football Association (in Swedish: Svenska Fotbollförbundet; SvFF)",
        "data_type": "ExternalId"
    },
    "P4955": {
        "label": "MR Author ID",
        "description": "Mathematical Reviews ID in MathSciNet",
        "data_type": "ExternalId"
    },
    "P4982": {
        "label": "Women’s Elite Rugby ID",
        "description": "identifier for a female rugby union player on the Women’s Elite Rugby ID",
        "data_type": "ExternalId"
    },
    "P4975": {
        "label": "Scottish Rugby Union men's sevens player ID",
        "description": "identifier for a men's rugby sevens player selected with the Scotland national team on the Scottish Rugby Union website",
        "data_type": "ExternalId"
    },
    "P4957": {
        "label": "AIDA freediver ID",
        "description": "identifier for a freediver on the Association internationale pour le développement de l'apnée website",
        "data_type": "ExternalId"
    },
    "P4790": {
        "label": "Basketball-Reference.com international player ID",
        "description": "identifier for a basketball player in a non-U.S. league on Basketball-Reference.com",
        "data_type": "ExternalId"
    },
    "P4611": {
        "label": "LACMA ID",
        "description": "identifier for an artist or an artwork on the Los Angeles County Museum of Art website",
        "data_type": "ExternalId"
    },
    "P4609": {
        "label": "Queensland Australian Football Hall of Fame inductee ID",
        "description": "identifier for a Queensland Australian Football Hall of Fame inductee on the Queensland Australian Football League website",
        "data_type": "ExternalId"
    },
    "P4744": {
        "label": "Basketball-Reference.com NBA G League player ID",
        "description": "identifier for an NBA G League (formerly NBDL) player on the Basketball Reference website",
        "data_type": "ExternalId"
    },
    "P4985": {
        "label": "TMDb person ID",
        "description": "identifier for a person at The Movie Database",
        "data_type": "ExternalId"
    },
    "P4779": {
        "label": "MYmovies actor ID",
        "description": "identifier for an actor in MYmovies.it",
        "data_type": "ExternalId"
    },
    "P4593": {
        "label": "Spanish Paralympic Committee athlete ID",
        "description": "identifier of an athlete on the website of the Spanish Paralympic Committeee (Comité Paralímpico Español)",
        "data_type": "ExternalId"
    },
    "P4804": {
        "label": "Compagnon de la Libération ID",
        "description": "identifier of a Companion of the Liberation on the website of that French order",
        "data_type": "ExternalId"
    },
    "P4590": {
        "label": "Atomic Heritage Foundation ID",
        "description": "profile of a person on the website of the Atomic Heritage Foundation",
        "data_type": "ExternalId"
    },
    "P4994": {
        "label": "ruchess ID",
        "description": "identifier for a Russian chess player, at ruchess.ru",
        "data_type": "ExternalId"
    },
    "P5029": {
        "label": "Researchmap ID",
        "description": "іdentifier for a researcher on researchmap.jp",
        "data_type": "ExternalId"
    },
    "P4778": {
        "label": "USA Rugby player ID",
        "description": "identifier for a rugby union player selected with the national team on the USA Rugby website",
        "data_type": "ExternalId"
    },
    "P4974": {
        "label": "Scottish Rugby Union men's player ID",
        "description": "identifier for a men's rugby union player selected with the Scotland national team on the Scottish Rugby Union website",
        "data_type": "ExternalId"
    },
    "P4987": {
        "label": "Scottish Rugby Union women's sevens player ID",
        "description": "identifier for a women's rugby sevens player selected with the Scotland national team on the Scottish Rugby Union website",
        "data_type": "ExternalId"
    },
    "P4879": {
        "label": "World Rugby Women's Sevens Series player ID",
        "description": "identifier for a female rugby sevens player on the World Rugby Women's Sevens Series website",
        "data_type": "ExternalId"
    },
    "P4768": {
        "label": "cinematografo.it name or company ID",
        "description": "identifier for a person or company in cinematografo.it",
        "data_type": "ExternalId"
    },
    "P4594": {
        "label": "arXiv author ID",
        "description": "identifier for an author on arXiv",
        "data_type": "ExternalId"
    },
    "P4787": {
        "label": "CiNii author ID (articles)",
        "description": "identifier for a journal article author in CiNii (Scholarly and Academic Information Navigator)",
        "data_type": "ExternalId"
    },
    "P4619": {
        "label": "National Library of Brazil ID",
        "description": "identifier for an element in the database of the National Library of Brazil",
        "data_type": "ExternalId"
    },
    "P4972": {
        "label": "Norwegian war sailor register person ID",
        "description": "identifikation number in the Norwegian war sailor register",
        "data_type": "ExternalId"
    },
    "P4650": {
        "label": "BSD Portal athlete ID",
        "description": "identifier for bobsledder, luger and skeleton racer in the Bob- und Schlittenverband für Deutschland (BSD) database",
        "data_type": "ExternalId"
    },
    "P4638": {
        "label": "The Peerage person ID",
        "description": "identifier for a person on The Peerage genealogical website",
        "data_type": "ExternalId"
    },
    "P4620": {
        "label": "Merkelstiftung person ID",
        "description": "identifier for the geneological record of a person in the database of the Merkelstiftung",
        "data_type": "ExternalId"
    },
    "P4819": {
        "label": "Swedish Portrait Archive ID",
        "description": "identifier for a person in the Swedish portrait archive",
        "data_type": "ExternalId"
    },
    "P4716": {
        "label": "Académie des Inscriptions et Belles-Lettres member ID",
        "description": "identifier for a member of the Académie des Inscriptions et Belles-Lettres on its website",
        "data_type": "ExternalId"
    },
    "P4718": {
        "label": "Basketball-Reference.com NBA coach ID",
        "description": "identifier for a NBA coach on the Basketball Reference website",
        "data_type": "ExternalId"
    },
    "P5013": {
        "label": "Welsh Rugby Union women's player ID",
        "description": "identifier for a women's rugby union player selected with the Wales national team on the Welsh Rugby Union website",
        "data_type": "ExternalId"
    },
    "P4621": {
        "label": "Württembergische Kirchengeschichte person ID",
        "description": "identifier for a person in the database of the Evangelic church in Württemberg",
        "data_type": "ExternalId"
    },
    "P4617": {
        "label": "DVV player ID",
        "description": "identifier for a beach volleyball player on the website of the German Volleyball Federation",
        "data_type": "ExternalId"
    },
    "P4996": {
        "label": "World Para Snowboard athlete ID",
        "description": "identifier of a snowboard athlete, in the World Para database",
        "data_type": "ExternalId"
    },
    "P4681": {
        "label": "Eldoblaje dubbing actor ID",
        "description": "identifier for a dubbing actor in a Spanish dubbed film",
        "data_type": "ExternalId"
    },
    "P4623": {
        "label": "South Australian Football Hall of Fame inductee ID",
        "description": "identifier for a South Australian Football Hall of Fame inductee on the South Australian Football League website",
        "data_type": "ExternalId"
    },
    "P4789": {
        "label": "Who's Who UK ID",
        "description": "identifier in \"Who's Who\" or \"Who Was Who\", UK edition",
        "data_type": "ExternalId"
    },
    "P4872": {
        "label": "GEPRIS person ID",
        "description": "identifier of a person in GEPRIS database of funded research projects",
        "data_type": "ExternalId"
    },
    "P4604": {
        "label": "World Rugby Hall of Fame ID",
        "description": "identifier for a player on the World Rugby Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P4927": {
        "label": "Invaluable.com person ID",
        "description": "identifier for an artist or other person at Invaluable.com",
        "data_type": "ExternalId"
    },
    "P4796": {
        "label": "Basketball-Reference.com NBL player ID",
        "description": "identifier for a NBL player on the Basketball Reference website",
        "data_type": "ExternalId"
    },
    "P7614": {
        "label": "Dictionary of Wisconsin History ID",
        "description": "article identifier for the Dictionary of Wisconsin History",
        "data_type": "ExternalId"
    },
    "P7687": {
        "label": "Kansaspedia ID",
        "description": "identifier for an entry on the Kansaspedia website",
        "data_type": "ExternalId"
    },
    "P7504": {
        "label": "Jeugdliteratuur ID",
        "description": "identifier for an author on the website Jeugdliteratuur.org",
        "data_type": "ExternalId"
    },
    "P7563": {
        "label": "Home of the Underdogs game ID",
        "description": "identifier of a video game in the Home of the Underdogs database",
        "data_type": "ExternalId"
    },
    "P7568": {
        "label": "CBSE affiliation number",
        "description": "code for schools affiliated to Central Board of Secondary Education",
        "data_type": "ExternalId"
    },
    "P7512": {
        "label": "startrek.com Database ID",
        "description": "ID in the Database of startrek.com",
        "data_type": "ExternalId"
    },
    "P7683": {
        "label": "OpenRetro Game Database ID",
        "description": "identifier in the OpenRetro Game Database",
        "data_type": "ExternalId"
    },
    "P7625": {
        "label": "WPI ID",
        "description": "port ID in World Port Index database",
        "data_type": "ExternalId"
    },
    "P7577": {
        "label": "Czech city district ID",
        "description": "identifier for a city district in the Czech Republic, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7635": {
        "label": "administrative district of Czech municipality with authorized municipal office ID",
        "description": "identifier for a municipality with authorized municipal office, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7636": {
        "label": "administrative district of Czech municipality with expanded powers ID",
        "description": "identifier for a municipality with expanded powers, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7631": {
        "label": "MPPDA Digital Archive film ID",
        "description": "link to a film's entry at the MPPDA Digital Archive",
        "data_type": "ExternalId"
    },
    "P7518": {
        "label": "classicamiga ID",
        "description": "identifier in the classicamiga database",
        "data_type": "ExternalId"
    },
    "P7524": {
        "label": "CA PROP 65 ID",
        "description": "ID for substance in California Proposition 65 list of chemicals known to cause cancer or reproductive harm",
        "data_type": "ExternalId"
    },
    "P7525": {
        "label": "ePSD identifier",
        "description": "identier for the electronic Pennsylvania Sumerian Dictionary",
        "data_type": "ExternalId"
    },
    "P7592": {
        "label": "VideoGameGeek platform ID",
        "description": "identifier of a platform in the VideoGameGeek database",
        "data_type": "ExternalId"
    },
    "P7607": {
        "label": "WikiTree category or space",
        "description": "Categories in Wikitree for things that also exist in Wikidata like cemeteries",
        "data_type": "ExternalId"
    },
    "P7649": {
        "label": "Vectrex Game Database ID",
        "description": "identifier of a video game in the Vectrex Game Database",
        "data_type": "ExternalId"
    },
    "P7585": {
        "label": "NinDB game ID",
        "description": "identifier of a video game in the NinDB database",
        "data_type": "ExternalId"
    },
    "P7637": {
        "label": "IDU theatre company ID",
        "description": "identifier for theatre names in Czechia in the Theatre Institute database",
        "data_type": "ExternalId"
    },
    "P7677": {
        "label": "www.gesetze-im-internet.de ID",
        "data_type": "ExternalId"
    },
    "P7617": {
        "label": "Lutris platform ID",
        "description": "identifier of a video game platform in the Lutris database",
        "data_type": "ExternalId"
    },
    "P7611": {
        "label": "Taiwan Cinema film ID",
        "description": "identifier for a film on Taiwan Cinema",
        "data_type": "ExternalId"
    },
    "P7619": {
        "label": "The Cardinals of the Holy Roman Church ID",
        "description": "identifier for a cardinal in the website The Cardinals of the Holy Roman Church",
        "data_type": "ExternalId"
    },
    "P7506": {
        "label": "Naver Encyclopedia ID",
        "description": "ID of an document in Naver Encyclopedia",
        "data_type": "ExternalId"
    },
    "P7502": {
        "label": "Golden ID",
        "description": "page ID for the Golden knowledge base",
        "data_type": "ExternalId"
    },
    "P7536": {
        "label": "Birds of India ID",
        "description": "identifier for birds found in India maintained by the 'Birds of India' database",
        "data_type": "ExternalId"
    },
    "P7549": {
        "label": "Biodiversity of West Bengal species ID",
        "description": "identifier for species found in West Bengal maintained by the 'Biodiversity of West Bengal' database",
        "data_type": "ExternalId"
    },
    "P7544": {
        "label": "Indian Medicinal Plants Database ID",
        "description": "identifier for medicinal plants found in India maintained by the 'Indian Medicinal Plants Database'",
        "data_type": "ExternalId"
    },
    "P7539": {
        "label": "NBAIR pest ID",
        "description": "identifіer for crop pests found in India",
        "data_type": "ExternalId"
    },
    "P7542": {
        "label": "Reptiles of India ID",
        "description": "identifier for reptiles found in India, in the 'Reptiles of India' database",
        "data_type": "ExternalId"
    },
    "P7538": {
        "label": "Moths of India ID",
        "description": "identifier for moths found in India maintained by the 'Moths of India' database",
        "data_type": "ExternalId"
    },
    "P7541": {
        "label": "Planète Aventure ID",
        "description": "identifier of a game in the Planète Aventure database",
        "data_type": "ExternalId"
    },
    "P7685": {
        "label": "CALS Encyclopedia of Arkansas ID",
        "description": "identifier for an entry on the CALS Encyclopedia of Arkansas website",
        "data_type": "ExternalId"
    },
    "P7515": {
        "label": "Chinese Professional Baseball League player ID",
        "description": "ID of a player in Chinese Professional Baseball League website of Taiwan",
        "data_type": "ExternalId"
    },
    "P7554": {
        "label": "Encyclopedia of Mathematics wiki ID",
        "description": "identifier for an article on the Encyclopedia of Mathematics wiki",
        "data_type": "ExternalId"
    },
    "P7632": {
        "label": "MPPDA Digital Archive organisation ID",
        "description": "link to an organisation's records at the MPPDA Digital Archive",
        "data_type": "ExternalId"
    },
    "P7622": {
        "label": "TheGamesDB game ID",
        "description": "identifier of a video game in TheGamesDB database",
        "data_type": "ExternalId"
    },
    "P7666": {
        "label": "Visuotinė lietuvių enciklopedija ID",
        "description": "identifier for an item in the online edition of the Visuotinė lietuvių enciklopedija",
        "data_type": "ExternalId"
    },
    "P7572": {
        "label": "Digital Corpus of Sanskrit ID",
        "description": "property for Sanskrit lemma in IAST transliteration maintained by Digital Corpus of Sanskrit",
        "data_type": "ExternalId"
    },
    "P7574": {
        "label": "Swedish Royal Theater Archive",
        "description": "a database of roles, persons and plays performed at the Swedish Royal theater Royal Dramatic Theatre",
        "data_type": "ExternalId"
    },
    "P7513": {
        "label": "Alfred P. Sloan Foundation grant ID",
        "description": "identifiers of grants in the database of the Alfred P. Sloan Foundation",
        "data_type": "ExternalId"
    },
    "P7597": {
        "label": "Lutris game ID",
        "description": "identifier of a video game in the Lutris database",
        "data_type": "ExternalId"
    },
    "P7647": {
        "label": "Cochrane concept ID",
        "description": "identifier for a concept in the Cochrane linked-data vocabulary, relating to health and medicine",
        "data_type": "ExternalId"
    },
    "P7650": {
        "label": "WeChat ID",
        "description": "account identifier of this item on WeChat",
        "data_type": "ExternalId"
    },
    "P7576": {
        "label": "Broadcast Radio Bearer URI",
        "description": "identifier linking Wikidata item to the broadcast parameters for a transmission",
        "data_type": "ExternalId"
    },
    "P7613": {
        "label": "Biblioteche dei filosofi ID",
        "description": "identifier for a philosopher in the project Biblioteche dei filosofi",
        "data_type": "ExternalId"
    },
    "P7526": {
        "label": "Czech cadastral area ID",
        "description": "identifier for a cadastral area in the Czech Republic, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7595": {
        "label": "Disney+ movie ID",
        "description": "identifier for a film on Disney+",
        "data_type": "ExternalId"
    },
    "P7596": {
        "label": "Disney+ series ID",
        "description": "identifier for television/web series on Disney+",
        "data_type": "ExternalId"
    },
    "P7606": {
        "label": "Czech municipality ID",
        "description": "identifier for a municipality in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P7548": {
        "label": "UK Lakes Portal ID",
        "description": "identifier for a lake or loch in the UK Lakes Portal database and website",
        "data_type": "ExternalId"
    },
    "P7657": {
        "label": "Doctrine court decision ID",
        "description": "identifier for a French court decision on Doctrine",
        "data_type": "ExternalId"
    },
    "P7658": {
        "label": "Juricaf decision ID",
        "description": "decision of a French-speaking supreme court on Juricaf",
        "data_type": "ExternalId"
    },
    "P7679": {
        "label": "Cinépolis KLIC ID",
        "description": "identifier of a movie in Cinépolis KLIC website",
        "data_type": "ExternalId"
    },
    "P7626": {
        "label": "NVE powerplant ID",
        "description": "serial number for a hydroelectric powerplant in  Norwegian Water Resources and Energy Directorate's Hydroelectric database",
        "data_type": "ExternalId"
    },
    "P7639": {
        "label": "OverDrive publisher ID",
        "description": "identifier for a publisher at OverDrive",
        "data_type": "ExternalId"
    },
    "P7517": {
        "label": "Sega Retro ID",
        "description": "identifier in the Sega Retro wiki",
        "data_type": "ExternalId"
    },
    "P7516": {
        "label": "Software Preservation Society ID",
        "description": "identifier of a video game in the Software Preservation Society database",
        "data_type": "ExternalId"
    },
    "P7620": {
        "label": "Treccani's Enciclopedia dei Papi ID",
        "description": "identifier for a pope (or antipope) in the Enciclopedia dei Papi",
        "data_type": "ExternalId"
    },
    "P7555": {
        "label": "UVL game ID",
        "description": "identifier of a video game in the Universal Videogame List",
        "data_type": "ExternalId"
    },
    "P7665": {
        "label": "FMV World ID",
        "description": "identifier of a video game in the FMV World database",
        "data_type": "ExternalId"
    },
    "P7690": {
        "label": "South Carolina Encyclopedia ID",
        "description": "identifier for an entry on the South Carolina Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P7691": {
        "label": "The Oregon Encyclopedia ID",
        "description": "identifier for an entry on the Oregon Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P7497": {
        "label": "Wolfram Language entity type",
        "description": "first part of the Wolfram Language entity code",
        "data_type": "ExternalId"
    },
    "P7686": {
        "label": "Encyclopedia of the Great Plains ID",
        "description": "identifier for an entry on the Encyclopedia of the Great Plains website",
        "data_type": "ExternalId"
    },
    "P7669": {
        "label": "NEC Retro ID",
        "description": "identifier in the NEC Retro wiki",
        "data_type": "ExternalId"
    },
    "P7621": {
        "label": "Church Heritage Cymru ID",
        "description": "identifier for a church building in the Church Heritage Cymru database (not the church code but a separate identifier)",
        "data_type": "ExternalId"
    },
    "P7627": {
        "label": "Artsdata.ca ID",
        "description": "Identifier for resources in the Artsdata.ca Knowledge Graph",
        "data_type": "ExternalId"
    },
    "P7623": {
        "label": "TheGamesDB platform ID",
        "description": "identifier of a video game platform in TheGamesDB database",
        "data_type": "ExternalId"
    },
    "P7505": {
        "label": "ACL Anthology article ID",
        "description": "ID of an article in ACL Anthology",
        "data_type": "ExternalId"
    },
    "P7615": {
        "label": "Finnish generals and admirals in the Imperial Russian Army 1809–1917 ID",
        "description": "The web publication of the Finnish Literature Society presents almost 500 Finnish generals and admirals in the service of the Inperial Russian Army.",
        "data_type": "ExternalId"
    },
    "P7616": {
        "label": "Lutris genre ID",
        "description": "identifier of a video game genre in the Lutris database",
        "data_type": "ExternalId"
    },
    "P7537": {
        "label": "India Biodiversity Portal species ID",
        "description": "identifier for species found in India maintained by the 'India Biodiversity Portal' database",
        "data_type": "ExternalId"
    },
    "P7540": {
        "label": "Odonata of India ID",
        "description": "identifier for odonata species found in India maintained by the 'Odonata of India' database",
        "data_type": "ExternalId"
    },
    "P7651": {
        "label": "Hoopla series ID",
        "description": "identifier for a series in Hoopla",
        "data_type": "ExternalId"
    },
    "P7659": {
        "label": "POWiS ID",
        "description": "identifier in the Places of Worship in Scotland (POWiS) database",
        "data_type": "ExternalId"
    },
    "P7673": {
        "label": "Czech district ID",
        "description": "identifier for a district, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7674": {
        "label": "Czech region ID",
        "description": "identifier for a region, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7507": {
        "label": "Ben Yehuda author ID",
        "description": "author identifier in Project Ben-Yehuda",
        "data_type": "ExternalId"
    },
    "P7575": {
        "label": "Sri Granth Punjabi word ID",
        "description": "property for Punjabi words in Gurmukhi script retrieved from Guru Granth Sahib maintained by Sri Granth",
        "data_type": "ExternalId"
    },
    "P7644": {
        "label": "Encyclopedia of Cleveland History ID",
        "description": "identifier for an entry on the Encyclopedia of Cleveland History website",
        "data_type": "ExternalId"
    },
    "P7556": {
        "label": "UVL platform ID",
        "description": "identifier of a platform in the Universal Videogame List",
        "data_type": "ExternalId"
    },
    "P7559": {
        "label": "Ahotsak Lexeme",
        "description": "Lexeme information at Ahotsak web",
        "data_type": "ExternalId"
    },
    "P7630": {
        "label": "Dictionary of Scottish Architects building ID",
        "description": "identifier for a building in the Dictionary of Scottish Architects database and website",
        "data_type": "ExternalId"
    },
    "P7565": {
        "label": "CISCE school code",
        "description": "code for schools affiliated to the Council for the Indian School Certificate Examinations",
        "data_type": "ExternalId"
    },
    "P7566": {
        "label": "JNAF artist ID",
        "description": "identifier for artists whose arts were collected by Indian private collector Jehangir Nicholson",
        "data_type": "ExternalId"
    },
    "P7567": {
        "label": "OGDB compilation ID",
        "description": "identifier of a video game compilation in the Online Games-Datenbank",
        "data_type": "ExternalId"
    },
    "P7689": {
        "label": "Online Nevada Encyclopedia ID",
        "description": "identifier for an entry on the Online Nevada Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P7648": {
        "label": "OverDrive series ID",
        "description": "identifier for a book series at OverDrive",
        "data_type": "ExternalId"
    },
    "P7587": {
        "label": "eFloraSA ID",
        "description": "Electronic Flora of South Australia: Factsheet ID",
        "data_type": "ExternalId"
    },
    "P7678": {
        "label": "Wikimapia ID",
        "description": "link to Wikimapia item",
        "data_type": "ExternalId"
    },
    "P7680": {
        "label": "Colorado Encyclopedia ID",
        "description": "identifier for an entry on the Colorado Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P7558": {
        "label": "Inguma database ID",
        "description": "Inguma database researcher link",
        "data_type": "ExternalId"
    },
    "P7564": {
        "label": "OGDB game title ID",
        "description": "identifier of a video game in the Online Games-Datenbank",
        "data_type": "ExternalId"
    },
    "P7499": {
        "label": "Van Wijngaarden quarry ID",
        "description": "identifier for an underground quarry assigned by Van Wijngaarden of the National Institute for Field Biological Research for Nature Conservation (RIVON)",
        "data_type": "ExternalId"
    },
    "P7579": {
        "label": "Prague administrative district ID",
        "description": "identifier for a administrative district of Prague, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7660": {
        "label": "super-famicom.jp ID",
        "description": "identifier of a video game in the super-famicom.jp database",
        "data_type": "ExternalId"
    },
    "P7675": {
        "label": "Vie publique intervention ID",
        "description": "identifier for a public intervention on Vie publique",
        "data_type": "ExternalId"
    },
    "P276": {
        "label": "location",
        "description": "location of the object, structure or event. In the case of an administrative entity as containing item use P131 for statistical entities P8138. In the case of a geographic entity use P706. Use P7153 for locations associated with the object.",
        "data_type": "WikibaseItem"
    },
    "P6872": {
        "label": "has written for",
        "description": "publication an author has contributed to",
        "data_type": "WikibaseItem"
    },
    "P9344": {
        "label": "has graphical element",
        "description": "mark, motif, coat of arms, map, or other graphic element added to a creative work",
        "data_type": "WikibaseItem"
    },
    "P575": {
        "label": "time of discovery or invention",
        "description": "date or point in time when the item was discovered or invented",
        "data_type": "Time"
    },
    "P8371": {
        "label": "references work, tradition or theory",
        "description": "creative work, tradition or theory this creative work references by allusion, quote or similar means (for citations in scholarly and legal works use P2860, for other more specific properties have a look at \"see also\" on the property page)",
        "data_type": "WikibaseItem"
    },
    "P7478": {
        "label": "iFixit repairability score (DEPRECATED)",
        "description": "score from 0 to 10 published by the company iFixit",
        "data_type": "String"
    },
    "P1734": {
        "label": "oath of office date",
        "description": "when person swore the oath of office",
        "data_type": "Time"
    },
    "P9234": {
        "label": "supplement to",
        "description": "subject work or edition updates or otherwise complements the object work or edition",
        "data_type": "WikibaseItem"
    },
    "P9202": {
        "label": "teatral.ro ID",
        "description": "identifier for the www.teatral.ro website",
        "data_type": "ExternalId"
    },
    "P7588": {
        "label": "effective date",
        "description": "date on which a statutory or administrative law comes into force. This date may differ for different sections of the law. This may also be retrospective, predating the date of enactment.",
        "data_type": "Time"
    },
    "P576": {
        "label": "dissolved, abolished or demolished date",
        "description": "point in time at which the subject (organisation, building) ceased to exist;  see \"date of official closure\" (P3999) for closing a facility, \"service retirement\" (P730) for retiring equipment,  \"discontinued date\" (P2669) for stopping a product",
        "data_type": "Time"
    },
    "P6819": {
        "label": "calligrapher",
        "description": "calligrapher of a work",
        "data_type": "WikibaseItem"
    },
    "P9161": {
        "label": "thesis committee member",
        "description": "member of the committee for this thesis or dissertation; for any type of thesis (bachelor's, master's, doctoral, etc.). Qualify with \"object has role\" (P3831) to indicate chair",
        "data_type": "WikibaseItem"
    },
    "P582": {
        "label": "end time",
        "description": "time an item ceases to exist or a statement stops being valid",
        "data_type": "Time"
    },
    "P580": {
        "label": "start time",
        "description": "time an item begins to exist or a statement starts being valid",
        "data_type": "Time"
    },
    "P706": {
        "label": "located on terrain feature",
        "description": "located on the specified landform. Should not be used when the value is only political/administrative (P131) or a mountain range (P4552).",
        "data_type": "WikibaseItem"
    },
    "P8883": {
        "label": "BBC sound effects asset ID",
        "description": "unique identifer of a sound effect within the BBC Sound Effects Archive Resource, by RES (Research & Education Space)",
        "data_type": "ExternalId"
    },
    "P7955": {
        "label": "Global Music Rights work ID",
        "description": "identifier for a musical work in Global Music Rights' repertory",
        "data_type": "ExternalId"
    },
    "P6949": {
        "label": "announcement date",
        "description": "time of the first public presentation of a subject by the creator, of information by the media",
        "data_type": "Time"
    },
    "P8565": {
        "label": "British Museum object ID",
        "description": "identifier for a object in the British Museum",
        "data_type": "ExternalId"
    },
    "P7956": {
        "label": "SESAC work number",
        "description": "identifier for a musical work in SESAC's repertory",
        "data_type": "ExternalId"
    },
    "P8791": {
        "label": "digitised by",
        "description": "institution who proceed with digitisation of an exemplar of this item. Qualify with \"criterion used\" (P1013) to indicate which step was done.",
        "data_type": "WikibaseItem"
    },
    "P7514": {
        "label": "landscape architect",
        "description": "person or architectural firm that designed this outdoor area, possibly including landmarks and structures",
        "data_type": "WikibaseItem"
    },
    "P2094": {
        "label": "competition class",
        "description": "official classification by a regulating body under which the subject (events, teams, participants, or equipment) qualifies for inclusion",
        "data_type": "WikibaseItem"
    },
    "P7589": {
        "label": "date of assent",
        "description": "date on which a Bill is assented to by the Head of State (monarch or president) and becomes an Act",
        "data_type": "Time"
    },
    "P598": {
        "label": "commander of (DEPRECATED)",
        "description": "for persons who are notable as commanding officers, the units they commanded",
        "data_type": "WikibaseItem"
    },
    "P6086": {
        "label": "dramaturge",
        "description": "person responsible for dramaturgical aspects in a performing arts production",
        "data_type": "WikibaseItem"
    },
    "P813": {
        "label": "retrieved",
        "description": "date or point in time that information was retrieved from a database or website (for use in online sources)",
        "data_type": "Time"
    },
    "P8345": {
        "label": "media franchise",
        "description": "this creative work belongs to this media franchise",
        "data_type": "WikibaseItem"
    },
    "P585": {
        "label": "point in time",
        "description": "time and date something took place, existed or a statement was true",
        "data_type": "Time"
    },
    "P5881": {
        "label": "projection designer",
        "description": "person responsible for producing all moving and still images that are displayed during a live performance",
        "data_type": "WikibaseItem"
    },
    "P2315": {
        "label": "comment (DEPRECATED)",
        "description": "to be deleted: replace with \"syntax clarification\" (P2916) or add the usage note in the items description.",
        "data_type": "Monolingualtext"
    },
    "P8058": {
        "label": "symbol of",
        "description": "item of which the image is a symbol",
        "data_type": "WikibaseItem"
    },
    "P729": {
        "label": "service entry",
        "description": "date or point in time on which a piece or class of equipment entered operational service",
        "data_type": "Time"
    },
    "P8556": {
        "label": "extinction date",
        "description": "date that a species or language is thought to have gone extinct",
        "data_type": "Time"
    },
    "P8306": {
        "label": "has grouping",
        "description": "section or grouping used in the appendix, list or table",
        "data_type": "WikibaseItem"
    },
    "P8307": {
        "label": "has sorting",
        "description": "entries are sorted by this in the appendix, list or table",
        "data_type": "WikibaseItem"
    },
    "P8308": {
        "label": "has column",
        "description": "columns for entries in the appendix, list or table",
        "data_type": "WikibaseItem"
    },
    "P6592": {
        "label": "OpenStreetMap zoom level",
        "description": "zoom level that would be requested for a 512×512-pixel OSM map that could contain the boundaries of this item",
        "data_type": "String"
    },
    "P8237": {
        "label": "LiederNet text ID",
        "description": "song text identifier used by the LiederNet Archive",
        "data_type": "ExternalId"
    },
    "P7937": {
        "label": "form of creative work",
        "description": "structure of a creative work",
        "data_type": "WikibaseItem"
    },
    "P620": {
        "label": "time of spacecraft landing",
        "description": "date and time of spacecraft landing in UTC",
        "data_type": "Time"
    },
    "P7228": {
        "label": "access restriction status",
        "description": "status of any access restrictions on the object, collection, or materials. Indicates whether the material is available to  the general public or not.",
        "data_type": "WikibaseItem"
    },
    "P9191": {
        "label": "letterer",
        "description": "person responsible for drawing the comic book's text, word balloons, etc.",
        "data_type": "WikibaseItem"
    },
    "P606": {
        "label": "first flight",
        "description": "date or point in time on which aircraft, rocket, or airline first flew",
        "data_type": "Time"
    },
    "P8608": {
        "label": "Fatcat ID",
        "description": "identifier on fatcat.wiki, a repository of scholarly articles maintained by the Internet Archive",
        "data_type": "ExternalId"
    },
    "P1619": {
        "label": "date of official opening",
        "description": "date or point in time an event, museum, theater etc. officially opened",
        "data_type": "Time"
    },
    "P2285": {
        "label": "periapsis date",
        "description": "time for perihelim/perigeum in each orbit for an astronomical object",
        "data_type": "Time"
    },
    "P730": {
        "label": "service retirement",
        "description": "date or point in time on which a piece or class of equipment was retired from operational service",
        "data_type": "Time"
    },
    "P9302": {
        "label": "script style",
        "description": "style of script of a document or typeface",
        "data_type": "WikibaseItem"
    },
    "P8751": {
        "label": "die axis",
        "description": "measurement of a coin that describes how the obverse and reverse dies were aligned to each other when it was struck, either in degrees (0-360) or hours (12-11)",
        "data_type": "Quantity"
    },
    "P1249": {
        "label": "time of earliest written record",
        "description": "first time a subject was mentioned in writing",
        "data_type": "Time"
    },
    "P7010": {
        "label": "imprimitur granted by",
        "description": "person or organization authorizing publication of the book",
        "data_type": "WikibaseItem"
    },
    "P7984": {
        "label": "depicted format",
        "description": "property to help describe the format type of depicted subject, commonly confused with genre",
        "data_type": "WikibaseItem"
    },
    "P2669": {
        "label": "discontinued date",
        "description": "date that the availability of a product was discontinued; see also \"dissolved, abolished or demolished\" (P576)",
        "data_type": "Time"
    },
    "P3999": {
        "label": "date of official closure",
        "description": "date of official closure of a building or event",
        "data_type": "Time"
    },
    "P6216": {
        "label": "copyright status",
        "description": "copyright status for intellectual creations like works of art, publications, software, etc.",
        "data_type": "WikibaseItem"
    },
    "P9052": {
        "label": "date postally canceled",
        "description": "date of a postal marking on a postage stamp or postal stationery to deface the stamp and to prevent its reuse",
        "data_type": "Time"
    },
    "P8513": {
        "label": "LibraryThing series ID",
        "description": "authority control for series on LibraryThing",
        "data_type": "ExternalId"
    },
    "P6695": {
        "label": "number of sentences",
        "description": "number of sentences in text, text is available at Wikisource",
        "data_type": "Quantity"
    },
    "P927": {
        "label": "anatomical location",
        "description": "where in the body does this anatomical feature lie",
        "data_type": "WikibaseItem"
    },
    "P6962": {
        "label": "narrative motif",
        "description": "thematic unit in a narrative or literary work used to develop a theme, mood or plot",
        "data_type": "WikibaseItem"
    },
    "P571": {
        "label": "inception",
        "description": "date or point in time when the subject came into existence as defined",
        "data_type": "Time"
    },
    "P622": {
        "label": "spacecraft docking/undocking date",
        "description": "date and time of spacecraft docking or undocking event, in UTC",
        "data_type": "Time"
    },
    "P621": {
        "label": "time of spacecraft orbit decay",
        "description": "date and time of spacecraft orbit decay in UTC",
        "data_type": "Time"
    },
    "P619": {
        "label": "UTC date of spacecraft launch",
        "description": "year-month-day of spacecraft launch not adjusted by timezone or other offset",
        "data_type": "Time"
    },
    "P7150": {
        "label": "epigraph (literature)",
        "description": "short quotation introducing the work",
        "data_type": "Monolingualtext"
    },
    "P3231": {
        "label": "Media Art Database author ID (former scheme)",
        "description": "identifier for an author in the Media Art database created by the Japanese government department Bunkacho (use P7886 for new identifier with 'C' prefix)",
        "data_type": "ExternalId"
    },
    "P8570": {
        "label": "front and back matter",
        "description": "front and back matter of a book",
        "data_type": "WikibaseItem"
    },
    "P403": {
        "label": "mouth of the watercourse",
        "description": "the body of water to which the watercourse drains",
        "data_type": "WikibaseItem"
    },
    "P6750": {
        "label": "CineChile filmmaker ID",
        "description": "identifier for people listed at CineChile, the Chilean cinema encyclopedia",
        "data_type": "ExternalId"
    },
    "P6746": {
        "label": "Paměť národa ID",
        "description": "identifier in the database of historical witnesses of 20th-century events in the region of today's Czech Republic",
        "data_type": "ExternalId"
    },
    "P6739": {
        "label": "Russia.tv actor profile ID",
        "description": "identifier of an actor or TV-presenter from the site \"Russia-1\"",
        "data_type": "ExternalId"
    },
    "P6732": {
        "label": "VTB United League player ID",
        "description": "identifier for a player at the VTB United League site",
        "data_type": "ExternalId"
    },
    "P6796": {
        "label": "OSZMI ID",
        "description": "identifier of a person or a theatrical production in the Theatrical Database of the Hungarian Theatre Museum and Institute",
        "data_type": "ExternalId"
    },
    "P6715": {
        "label": "SIUSA archive producer person ID",
        "description": "identifier for an archive producer person in the Unified Information System for the Archival Superintendencies",
        "data_type": "ExternalId"
    },
    "P6674": {
        "label": "Desaparecidos Políticos ID",
        "description": "identifier for people who were killed or went missing during the Brazilian military dictatorship (1964-1985)",
        "data_type": "ExternalId"
    },
    "P6410": {
        "label": "LevskiSofia.info coach ID",
        "description": "identifier for a coach at the LevskiSofia.info",
        "data_type": "ExternalId"
    },
    "P6726": {
        "label": "NSW Capital Conviction ID",
        "description": "identifier for people in the NSW Capital Convictions Database",
        "data_type": "ExternalId"
    },
    "P6421": {
        "label": "infosport.ru person ID",
        "description": "identifier for person on site infosport.ru",
        "data_type": "ExternalId"
    },
    "P6682": {
        "label": "Motorsports Hall of Fame of America ID",
        "description": "Mini biographies of the inductees to the Motorsports Hall of Fame of America",
        "data_type": "ExternalId"
    },
    "P6583": {
        "label": "Lucerna person ID",
        "description": "person in the LUCERNA Magic Lantern Web Resource",
        "data_type": "ExternalId"
    },
    "P6475": {
        "label": "CABB player ID",
        "description": "player ID of national teams on the Argentine Basketball Confederation website",
        "data_type": "ExternalId"
    },
    "P6470": {
        "label": "DALFAN ID",
        "description": "identifier for a person the ''Dictionnaire des auteurs de langue française en Amérique du Nord''",
        "data_type": "ExternalId"
    },
    "P6496": {
        "label": "Biblioteca Virtual Andalucía authority ID",
        "description": "authority identifier in the Biblioteca Virtual de Andalucía",
        "data_type": "ExternalId"
    },
    "P6495": {
        "label": "KSI player ID",
        "description": "identifier for player on KSI site",
        "data_type": "ExternalId"
    },
    "P6866": {
        "label": "euroleague.net coach ID",
        "description": "ID for a basketball coach at euroleague.net",
        "data_type": "ExternalId"
    },
    "P6501": {
        "label": "Georgian National Filmography person ID",
        "description": "identifier for a person, in the Georgian National Filmography database",
        "data_type": "ExternalId"
    },
    "P6528": {
        "label": "V.League women's player ID",
        "description": "identifier for profile of female player on V.League site",
        "data_type": "ExternalId"
    },
    "P6878": {
        "label": "Amburger database ID",
        "description": "identifier for the Erik Amburger database of foreigners in pre-revolutionary Russia",
        "data_type": "ExternalId"
    },
    "P6525": {
        "label": "Fondation Maeght artist ID",
        "description": "identifier for an artist on the website of the Fondation Maeght, a French museum",
        "data_type": "ExternalId"
    },
    "P6527": {
        "label": "V.League men's player ID",
        "description": "identifier for profile of male player on V.League site",
        "data_type": "ExternalId"
    },
    "P6357": {
        "label": "Conférence du stage secretary ID",
        "description": "identifier for a secretary of the 'Conférence du stage de Paris' on its website",
        "data_type": "ExternalId"
    },
    "P6502": {
        "label": "JWF Wrestlers Database person ID",
        "description": "identifier for a wrestler in the Japan Wrestling Federation Database",
        "data_type": "ExternalId"
    },
    "P6504": {
        "label": "Lithuanian Sport Encyclopedia (LSE) article ID",
        "description": "identifier for an article on Lithuanian Sport Encyclopedia",
        "data_type": "ExternalId"
    },
    "P6615": {
        "label": "Kicker.de player ID (former scheme)",
        "description": "numeric identifier for an association football player at kicker.de",
        "data_type": "ExternalId"
    },
    "P6388": {
        "label": "sportufo.ru person ID",
        "description": "identifier for person on site sportufo.ru",
        "data_type": "ExternalId"
    },
    "P6613": {
        "label": "CSKA person ID",
        "description": "identifier for person on the website of CSKA Moscow",
        "data_type": "ExternalId"
    },
    "P6734": {
        "label": "Archaeology Data Service person ID",
        "description": "identifier for a person in the Archaeology Data Service library of bibliographic data",
        "data_type": "ExternalId"
    },
    "P6737": {
        "label": "grwar.ru person ID",
        "description": "ID of biographical article at Alexey Likhotvorik's project Russian Army in the Great War",
        "data_type": "ExternalId"
    },
    "P6561": {
        "label": "AiF dossier ID",
        "description": "identifier of people and organizations from the Russian newspaper \"Argumenty i Fakty\"",
        "data_type": "ExternalId"
    },
    "P6401": {
        "label": "sports.md athlete ID",
        "description": "identifier for an athlete at the sports.md",
        "data_type": "ExternalId"
    },
    "P6825": {
        "label": "RFGS person ID",
        "description": "identifier for a person at the Russian Federation of Alpine Skiing",
        "data_type": "ExternalId"
    },
    "P6575": {
        "label": "Académie de Rouen member ID",
        "description": "identifier for a member of the Académie de Rouen on its website",
        "data_type": "ExternalId"
    },
    "P6548": {
        "label": "NZ On Screen person ID",
        "description": "identifier for a person in the NZ On Screen database",
        "data_type": "ExternalId"
    },
    "P6556": {
        "label": "SICRIS researcher ID",
        "description": "ID of a registered researcher in Slovenia from the Slovenian Current Research Information System (SICRIS)",
        "data_type": "ExternalId"
    },
    "P6620": {
        "label": "L'Express person ID",
        "description": "identifier for a person on the website of L'Express",
        "data_type": "ExternalId"
    },
    "P6492": {
        "label": "BVPH authority ID",
        "description": "authority identifier in the Biblioteca Virtual de Prensa Histórica",
        "data_type": "ExternalId"
    },
    "P6804": {
        "label": "Art Gallery of South Australia creator ID",
        "description": "identifier for artists in the Art Gallery of South Australia",
        "data_type": "ExternalId"
    },
    "P6603": {
        "label": "film-documentaire.fr author ID",
        "description": "identifier of an author in the French database film-documentaire.fr",
        "data_type": "ExternalId"
    },
    "P6815": {
        "label": "University of Amsterdam Album Academicum ID",
        "description": "identifier of professors and doctors of the University of Amsterdam and its predecessor, the Athenaeum Illustre, from 1632 up to now",
        "data_type": "ExternalId"
    },
    "P6596": {
        "label": "a-tremasov.ru biography ID",
        "description": "ID of biographical article at Aleksei Tremasov's project Always with me…",
        "data_type": "ExternalId"
    },
    "P6598": {
        "label": "flgr-results.ru athlete ID",
        "description": "identifier for an athletes at the Data accounting system of Cross Country Ski Federation of Russia",
        "data_type": "ExternalId"
    },
    "P6836": {
        "label": "Pro-Football-Reference.com coach ID",
        "description": "identifier for American football coaches at Pro-Football-Reference.com",
        "data_type": "ExternalId"
    },
    "P6415": {
        "label": "Football.Odessa.ua person ID",
        "description": "identifier for person on site football.odessa.ua",
        "data_type": "ExternalId"
    },
    "P6640": {
        "label": "JRC Names id",
        "description": "ID in the JRC Names gazetteer, which provides spelling variants and EMM news about the entity (220k news items per day). Alias: JRCN",
        "data_type": "ExternalId"
    },
    "P6863": {
        "label": "Digital Prosopography of the Roman Republic ID",
        "description": "identifier of an ancient Roman from the Roman Republic, an online prosopographical research",
        "data_type": "ExternalId"
    },
    "P6770": {
        "label": "Swiss National Sound Archives ID",
        "description": "ID in the Swiss National Sound Archives, that is responsible for safeguarding the sound heritage of Switzerland. The SNSA collects and catalogs audio documents (music or spoken word), that have a relationship with the Swiss history and culture.",
        "data_type": "ExternalId"
    },
    "P6865": {
        "label": "eurobasket.com coach ID",
        "description": "ID for a basketball coach at eurobasket.com",
        "data_type": "ExternalId"
    },
    "P6321": {
        "label": "Teams.by player ID",
        "description": "Teams.by identifier for an association football (soccer) player",
        "data_type": "ExternalId"
    },
    "P6320": {
        "label": "Futsal Association of Russia player ID",
        "description": "AMFR identifier for a futsal player",
        "data_type": "ExternalId"
    },
    "P6318": {
        "label": "Crimean Football Union player ID",
        "description": "identifier for football player of Crimean Football Union",
        "data_type": "ExternalId"
    },
    "P6673": {
        "label": "Memórias da Ditadura ID",
        "description": "identifier for people who were killed or went missing during the Brazilian military dictatorship (1964-1985)",
        "data_type": "ExternalId"
    },
    "P6844": {
        "label": "abART person ID",
        "description": "identifier of person in the Czech database abART",
        "data_type": "ExternalId"
    },
    "P6837": {
        "label": "Keybase username",
        "description": "username in the cryptographic social network Keybase",
        "data_type": "ExternalId"
    },
    "P6490": {
        "label": "BIDICAM authority ID",
        "description": "authority identifier in the Biblioteca Digital de Castilla-La Mancha",
        "data_type": "ExternalId"
    },
    "P6692": {
        "label": "CEMDP ID",
        "description": "identifier for people who were killed or went missing during the Brazilian military dictatorship (1964-1985)",
        "data_type": "ExternalId"
    },
    "P6705": {
        "label": "Ming Qing Archive ID",
        "description": "identifier for a person in the Ming Qing Archive Name Authority Database",
        "data_type": "ExternalId"
    },
    "P6701": {
        "label": "Referee.ru person ID",
        "description": "identifier for person at Football referees of Russia site",
        "data_type": "ExternalId"
    },
    "P6330": {
        "label": "England Football Online manager profile",
        "description": "identifier for football manager on England Football Online site",
        "data_type": "ExternalId"
    },
    "P6624": {
        "label": "FSkate.ru skater ID",
        "description": "identifier for a skater at FSkate.ru",
        "data_type": "ExternalId"
    },
    "P6636": {
        "label": "CDEC ID",
        "description": "ID for the Italian victims of the Nazi Holocaust, from the database of CDEC - Italian center for Jewish contemporary documentation",
        "data_type": "ExternalId"
    },
    "P6634": {
        "label": "LinkedIn personal profile ID",
        "description": "identifier for a person, on the LinkedIn website",
        "data_type": "ExternalId"
    },
    "P6612": {
        "label": "FFS athlete ID",
        "description": "identifier for athletes at the Fédération française de ski site",
        "data_type": "ExternalId"
    },
    "P6652": {
        "label": "Sumo Reference ID",
        "description": "identifier for profile of sumo wrestler on Sumo Reference",
        "data_type": "ExternalId"
    },
    "P6728": {
        "label": "Dictionnaire des Vendéens ID",
        "description": "identifier for a person on the online Dictionnaire des Vendéens",
        "data_type": "ExternalId"
    },
    "P6632": {
        "label": "100 bombardirov person ID",
        "description": "identifier for a person 100bombardirov.ru",
        "data_type": "ExternalId"
    },
    "P6628": {
        "label": "FaroeSoccer coach ID",
        "description": "identifier for a coach at FaroeSoccer.com",
        "data_type": "ExternalId"
    },
    "P6650": {
        "label": "Martindale-Hubbell profile",
        "description": "profile of a person at Martindale-Hubbell",
        "data_type": "ExternalId"
    },
    "P6662": {
        "label": "Springboks Rugby Player ID",
        "description": "identifier for a Rugby Player on the official SA Rugby website",
        "data_type": "ExternalId"
    },
    "P6582": {
        "label": "Dutch Instrument Makers ID",
        "description": "biographical data about instrument makers that were active in the Netherlands",
        "data_type": "ExternalId"
    },
    "P6399": {
        "label": "Romanian Soccer player ID",
        "description": "identifier for a player at the Romanian Soccer site",
        "data_type": "ExternalId"
    },
    "P6403": {
        "label": "TheatreOnline ID",
        "description": "identifier for a theater actor/actress in theatreonline.com website",
        "data_type": "ExternalId"
    },
    "P6850": {
        "label": "ABA League ID",
        "description": "identifier for a player in the ABA League website",
        "data_type": "ExternalId"
    },
    "P6384": {
        "label": "Viperson ID",
        "description": "identifier for person on site Viperson",
        "data_type": "ExternalId"
    },
    "P6669": {
        "label": "FIS telemark skier ID",
        "description": "identifier for a telemark skier in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P6331": {
        "label": "England Football Online player profile",
        "description": "identifier for football player on England Football Online site",
        "data_type": "ExternalId"
    },
    "P6329": {
        "label": "Share-VDE author ID",
        "description": "identifier for entities (people and organizations) in the federated linked data platform Share Virtual Discovery Environment",
        "data_type": "ExternalId"
    },
    "P6690": {
        "label": "CNV-SP ID",
        "description": "identifier for people who were killed or went missing during the Brazilian military dictatorship (1964-1985) in the São Paulo State compiled by the São Paulo State Truth Commission",
        "data_type": "ExternalId"
    },
    "P6677": {
        "label": "OpenEdition Books author ID",
        "description": "identifier for an author on OpenEdition",
        "data_type": "ExternalId"
    },
    "P6407": {
        "label": "World Cube Association ID",
        "description": "World Cube Association identifier for a person",
        "data_type": "ExternalId"
    },
    "P6351": {
        "label": "Genius artist numeric ID",
        "description": "numeric identifier for an artist on Genius",
        "data_type": "ExternalId"
    },
    "P6702": {
        "label": "Shanghai Library person ID",
        "description": "identifier for persons in the Shanghai Library Name Authority Database",
        "data_type": "ExternalId"
    },
    "P6336": {
        "label": "JKT48 member ID",
        "description": "ID of JKT48 members at their website",
        "data_type": "ExternalId"
    },
    "P6777": {
        "label": "Danskefilmstemmer.dk person ID",
        "description": "identifier of an voice actor, director or translator in the Danish DanskeFilmStemmer.dk database",
        "data_type": "ExternalId"
    },
    "P6350": {
        "label": "Doctrine ID",
        "description": "identifier for a lawyer on Doctrine",
        "data_type": "ExternalId"
    },
    "P6362": {
        "label": "Litchfield Ledger ID",
        "description": "identifier in Litchfield Ledger database",
        "data_type": "ExternalId"
    },
    "P6730": {
        "label": "CroRec ID",
        "description": "identifier for musicians in the Croatia Records database (www.crorec.hr)",
        "data_type": "ExternalId"
    },
    "P6386": {
        "label": "biathlon.com.ua person ID",
        "description": "identifier for a person at biathlon.com.ua",
        "data_type": "ExternalId"
    },
    "P6722": {
        "label": "FemBio ID",
        "description": "identifier for a woman, in the FemBio database",
        "data_type": "ExternalId"
    },
    "P6371": {
        "label": "Archives of Maryland Biographical Series ID",
        "description": "identifier for the Archives of Maryland Biographical Series",
        "data_type": "ExternalId"
    },
    "P6627": {
        "label": "FaroeSoccer player ID",
        "description": "identifier for a player at FaroeSoccer.com",
        "data_type": "ExternalId"
    },
    "P6521": {
        "label": "Crew United person ID",
        "description": "identifier for persons in the Crew united database of German films, TV series, and related productions",
        "data_type": "ExternalId"
    },
    "P6387": {
        "label": "RBU person ID",
        "description": "identifier for a person at the Russian Biathlon Union site",
        "data_type": "ExternalId"
    },
    "P6688": {
        "label": "National Wrestling Hall of Fame wrestler ID",
        "data_type": "ExternalId"
    },
    "P6411": {
        "label": "LevskiSofia.info player ID",
        "description": "identifier for a player at the LevskiSofia.info",
        "data_type": "ExternalId"
    },
    "P6743": {
        "label": "LNB Coach id",
        "description": "identifier for a coach in the French basketball championship",
        "data_type": "ExternalId"
    },
    "P1818": {
        "label": "Kaiserhof ID",
        "description": "identifier in the personnel database of the courtiers of the Austrian Habsburg imperial line and the branch lines",
        "data_type": "ExternalId"
    },
    "P2458": {
        "label": "Mackolik.com player ID",
        "description": "ID for an association football (soccer) player in the Turkish Mackolik.com database",
        "data_type": "ExternalId"
    },
    "P1899": {
        "label": "LibriVox author ID",
        "description": "author ID for an author represented at LibriVox",
        "data_type": "ExternalId"
    },
    "P1938": {
        "label": "Project Gutenberg author ID",
        "description": "author identifier at Project Gutenberg",
        "data_type": "ExternalId"
    },
    "P1430": {
        "label": "OpenPlaques subject ID",
        "description": "identifier for a person or other subject in the OpenPlaques database — https://openplaques.org/",
        "data_type": "ExternalId"
    },
    "P2028": {
        "label": "United States Armed Forces service number",
        "description": "number assigned by the US military to soldiers and sailors as the primary means of service member identification from 1918 until 1974",
        "data_type": "ExternalId"
    },
    "P1663": {
        "label": "ProCyclingStats cyclist ID",
        "description": "identifier on the website ProCyclingStats (www.procyclingstats.com)",
        "data_type": "ExternalId"
    },
    "P2030": {
        "label": "NASA biographical ID",
        "description": "identifier used by NASA for an astronaut or cosmonaut",
        "data_type": "ExternalId"
    },
    "P2038": {
        "label": "ResearchGate profile ID",
        "description": "identifier for a person, used by ResearchGate profiles",
        "data_type": "ExternalId"
    },
    "P2340": {
        "label": "CESAR person ID",
        "description": "identifier for a person in the CESAR database of French theatre of the seventeenth and eighteenth centuries",
        "data_type": "ExternalId"
    },
    "P1825": {
        "label": "Baseball-Reference.com major league player ID",
        "description": "identifier for a Major League Baseball player assigned by Baseball-Reference.com",
        "data_type": "ExternalId"
    },
    "P1819": {
        "label": "genealogics.org person ID",
        "description": "identifier for a person at genealogics.org. Format: \"I\" followed by 8 digits.",
        "data_type": "ExternalId"
    },
    "P2071": {
        "label": "Mémoire des hommes ID",
        "description": "French government database indexing all french soldier war casualties",
        "data_type": "ExternalId"
    },
    "P1440": {
        "label": "FIDE player ID",
        "description": "identifier on the FIDE database for chess players",
        "data_type": "ExternalId"
    },
    "P1563": {
        "label": "MacTutor biography ID",
        "description": "identifier of the person's biography in the MacTutor History of Mathematics archive",
        "data_type": "ExternalId"
    },
    "P2162": {
        "label": "Deutsche Ultramarathon-Vereinigung ID",
        "description": "identifier for individual runners in the Deutsche Ultramarathon-Vereinigung (German Ultramarathon Association) database",
        "data_type": "ExternalId"
    },
    "P2168": {
        "label": "Swedish Film Database person ID",
        "description": "identifier for a person on the Swedish Film Database (SFDb)",
        "data_type": "ExternalId"
    },
    "P2421": {
        "label": "Prosopographia Attica",
        "description": "identifier for a person in Prosopographia Attica (Q2113277) by Johannes Kirchner (Q1697841)",
        "data_type": "ExternalId"
    },
    "P1803": {
        "label": "Masaryk University person ID",
        "description": "identifer for a person affiliated with Masaryk University (staff, alumnus, student…)",
        "data_type": "ExternalId"
    },
    "P1714": {
        "label": "Journalisted ID",
        "description": "identifier on Journalisted, an independent, not-for-profit website (by Media Standards Trust) listing the published writing of journalists",
        "data_type": "ExternalId"
    },
    "P2383": {
        "label": "CTHS person ID",
        "description": "identifier for a person in the directory of French learned societies",
        "data_type": "ExternalId"
    },
    "P2435": {
        "label": "PORT person ID",
        "description": "PORT-network film database: identifier for a person",
        "data_type": "ExternalId"
    },
    "P1842": {
        "label": "Global Anabaptist Mennonite Encyclopedia Online ID",
        "description": "identifier for Global Anabaptist Mennonite Encyclopedia Online",
        "data_type": "ExternalId"
    },
    "P1463": {
        "label": "Post-Reformation Digital Library author ID",
        "description": "identifier from Post-Reformation Digital Library",
        "data_type": "ExternalId"
    },
    "P1599": {
        "label": "Cambridge Alumni Database ID",
        "description": "identifier in the Cambridge Alumni Database/Alumni Cantabrigienses (ACAD)",
        "data_type": "ExternalId"
    },
    "P2412": {
        "label": "FMD designer ID",
        "description": "external link to a designer's profile at the FMD (Fashion Model Directory) website",
        "data_type": "ExternalId"
    },
    "P2447": {
        "label": "Transfermarkt manager ID",
        "description": "identifier for an association football (soccer) manager or coach in the transfermarkt.com database",
        "data_type": "ExternalId"
    },
    "P2449": {
        "label": "Turkish Football Federation coach  ID",
        "description": "identifier for a coach or manager in the Turkish Football Federation's database",
        "data_type": "ExternalId"
    },
    "P1650": {
        "label": "BBF ID",
        "description": "identifier in the archive database of the German \"Research Library for the History of Education\" (personal data of the teachers of Prussia)",
        "data_type": "ExternalId"
    },
    "P1541": {
        "label": "CQ Ranking male cyclist ID",
        "description": "identifier for a male cyclist at cqranking.com",
        "data_type": "ExternalId"
    },
    "P1735": {
        "label": "Comedien.ch ID",
        "description": "identifier in Comedien.ch database of actors and commedians",
        "data_type": "ExternalId"
    },
    "P1664": {
        "label": "Cycling Database ID",
        "description": "identifier on the website Cycling Database (www.cyclingdatabase.com)",
        "data_type": "ExternalId"
    },
    "P1649": {
        "label": "KMDb person ID",
        "description": "identifier for a person on the Korean Movie Database (KMDb)",
        "data_type": "ExternalId"
    },
    "P2287": {
        "label": "CRIStin ID",
        "description": "ID in the database for Norwegian scientists",
        "data_type": "ExternalId"
    },
    "P2456": {
        "label": "DBLP author ID",
        "description": "identifier for author entries in the DBLP computer science bibliography (use portion of DBLP person key after homepages/)",
        "data_type": "ExternalId"
    },
    "P2460": {
        "label": "Persons of Ancient Athens",
        "description": "identifier for a person in Persons of Ancient Athens (Q21994990) by John S. Traill (Q13422797)",
        "data_type": "ExternalId"
    },
    "P1935": {
        "label": "Database of Classical Scholars ID",
        "description": "identifier for persons in the Database of Classical Scholars",
        "data_type": "ExternalId"
    },
    "P2381": {
        "label": "Academic Tree ID",
        "description": "identifer on academictree.org",
        "data_type": "ExternalId"
    },
    "P2459": {
        "label": "IBU biathlete ID",
        "description": "biathlete's identifier in the International Biathlon Union's database",
        "data_type": "ExternalId"
    },
    "P2454": {
        "label": "KNAW past member ID",
        "description": "identifier for a person in the Royal Netherlands Academy of Arts and Sciences past member database",
        "data_type": "ExternalId"
    },
    "P1788": {
        "label": "DVN ID",
        "description": "identifier in the Dutch Digitaal Vrouwenlexicon van Nederland (Online Dictionary of Dutch Women), a biography portal of prominent women in Dutch history.",
        "data_type": "ExternalId"
    },
    "P2481": {
        "label": "Elite Prospects player ID",
        "description": "ice hockey player ID assigned by Elite Prospects",
        "data_type": "ExternalId"
    },
    "P2482": {
        "label": "SABR person ID",
        "description": "identifier of a person in the Society for American Baseball Research database",
        "data_type": "ExternalId"
    },
    "P2448": {
        "label": "Turkish Football Federation player ID",
        "description": "identifier for a player in the Turkish Football Federation's database",
        "data_type": "ExternalId"
    },
    "P2350": {
        "label": "SpeedSkatingBase.eu ID",
        "description": "identifier for a speed skater in the Speedskatingbase.eu database with personal best distance records",
        "data_type": "ExternalId"
    },
    "P2070": {
        "label": "Fellow of the Royal Society ID",
        "description": "Fellow ID of the Royal Society",
        "data_type": "ExternalId"
    },
    "P2090": {
        "label": "Power of 10 athlete ID",
        "description": "identifier for an athlete in the Power of 10 database",
        "data_type": "ExternalId"
    },
    "P1969": {
        "label": "MovieMeter director ID (former scheme)",
        "description": "identifier for a person on the MovieMeter film database",
        "data_type": "ExternalId"
    },
    "P2450": {
        "label": "Encyclopædia Britannica contributor ID",
        "description": "identifier for a contributor to the 'Encyclopædia Britannica'",
        "data_type": "ExternalId"
    },
    "P2398": {
        "label": "MLS player ID",
        "description": "identifier for soccer player profile on MLSsoccer.com, website of Major League Soccer",
        "data_type": "ExternalId"
    },
    "P2255": {
        "label": "Debrett's People of Today ID",
        "description": "identifier for a person, in Debrett's \"People of Today\"",
        "data_type": "ExternalId"
    },
    "P2016": {
        "label": "Catalogus Professorum Academiae Groninganae ID",
        "description": "identifier for a professor, in the Catalogus Professorum Academiae Groninganae",
        "data_type": "ExternalId"
    },
    "P1908": {
        "label": "CWGC person ID",
        "description": "identifier for a person, in the online database of Commonwealth War Graves Commission",
        "data_type": "ExternalId"
    },
    "P2380": {
        "label": "French Sculpture Census artist ID",
        "description": "identifier in the French Sculpture Census",
        "data_type": "ExternalId"
    },
    "P1977": {
        "label": "Les Archives du Spectacle Person ID",
        "description": "Identifier for an actor/actress/playwright in the Les Archives du spectacle database",
        "data_type": "ExternalId"
    },
    "P2387": {
        "label": "Elonet person ID",
        "description": "identifier for a person, in the Elonet database",
        "data_type": "ExternalId"
    },
    "P2005": {
        "label": "Catalogus Professorum Halensis ID",
        "description": "biographical entry in the Catalogus Professorum Halensis",
        "data_type": "ExternalId"
    },
    "P1556": {
        "label": "zbMATH author ID",
        "description": "identifier of a person in the Zentralblatt MATH database",
        "data_type": "ExternalId"
    },
    "P1847": {
        "label": "Nasjonalbiblioteket photographer ID",
        "description": "identifier used by the Nasjonalbiblioteket (Norwegian National Library) for photographers",
        "data_type": "ExternalId"
    },
    "P2423": {
        "label": "International Fencing Federation fencer ID",
        "description": "Identifier in the database of all fencers holding an international licence at the International Fencing Federation (FIE: Fédération Internationale d'Escrime)",
        "data_type": "ExternalId"
    },
    "P1826": {
        "label": "Baseball-Reference.com minor & foreign league player ID",
        "description": "identifier for a Minor League Baseball player assigned by Baseball-Reference.com",
        "data_type": "ExternalId"
    },
    "P2087": {
        "label": "Crunchbase person ID",
        "description": "Identifier for a person, in the Crunchbase database of companies and start-ups, operated by TechCrunch",
        "data_type": "ExternalId"
    },
    "P1666": {
        "label": "Chess Club ID",
        "description": "identifier on the website Chess Club (www.chessclub.com)",
        "data_type": "ExternalId"
    },
    "P2276": {
        "label": "UEFA player ID",
        "description": "identifier for UEFA players",
        "data_type": "ExternalId"
    },
    "P2470": {
        "label": "Talouselämän vaikuttajat ID",
        "description": "identifer for a person, in the Talouselämän vaikuttajat database",
        "data_type": "ExternalId"
    },
    "P1890": {
        "label": "CCAB ID",
        "description": "Collective Catalog of Bibliographic Authorities of Chile ID",
        "data_type": "ExternalId"
    },
    "P2195": {
        "label": "Soccerbase manager ID",
        "description": "manager ID on the Soccerbase.com website",
        "data_type": "ExternalId"
    },
    "P1580": {
        "label": "University of Barcelona authority ID",
        "description": "register of authorities of the University of Barcelona",
        "data_type": "ExternalId"
    },
    "P2006": {
        "label": "ZooBank author ID",
        "description": "identifier for an author at ZooBank",
        "data_type": "ExternalId"
    },
    "P2469": {
        "label": "Theatricalia person ID",
        "description": "identifier for a person, in the Theatricalia  database",
        "data_type": "ExternalId"
    },
    "P1670": {
        "label": "Canadiana Authorities ID",
        "description": "obsolete identifier for authority control per the Library and Archives Canada. Format: 4 digits + 1 letter + 4 digits + optional F",
        "data_type": "ExternalId"
    },
    "P1473": {
        "label": "Nupill Literatura Digital - Author",
        "description": "identifier for an author from Nupill Literatura Digital",
        "data_type": "ExternalId"
    },
    "P2018": {
        "label": "Teuchos ID",
        "description": "identification String in the Teuchos Database",
        "data_type": "ExternalId"
    },
    "P2020": {
        "label": "WorldFootball.net player ID",
        "description": "identifier for an association football (soccer) player at WorldFootball.net, Weltfussball.com, Weltfussball.de, Weltfussball.at, Voetbal.com, Calcio.com, Mondedufoot.fr, or Livefutbol.com",
        "data_type": "ExternalId"
    },
    "P2193": {
        "label": "Soccerbase player ID",
        "description": "player ID on the Soccerbase.com website",
        "data_type": "ExternalId"
    },
    "P1601": {
        "label": "Esperantist ID",
        "description": "number in the address book of early Esperanto speakers by Zamenhof",
        "data_type": "ExternalId"
    },
    "P1871": {
        "label": "CERL Thesaurus ID",
        "description": "identifier in the Consortium of European Research Libraries Thesaurus",
        "data_type": "ExternalId"
    },
    "P1960": {
        "label": "Google Scholar author ID",
        "description": "identifier of a person, in the Google Scholar academic search service",
        "data_type": "ExternalId"
    },
    "P1967": {
        "label": "BoxRec boxer ID",
        "description": "Identifier for a boxer in the BoxRec database",
        "data_type": "ExternalId"
    },
    "P2323": {
        "label": "Swedish Olympic Committee athlete ID",
        "description": "entry in the bibliographic database of the Swedish Olympic Committee (Swedish: Sveriges Olympiska Kommitté, SOK)",
        "data_type": "ExternalId"
    },
    "P2369": {
        "label": "Soccerway player ID",
        "description": "player id on the Soccerway website",
        "data_type": "ExternalId"
    },
    "P1695": {
        "label": "NLP ID (unique)",
        "description": "former National Library of Poland unique identifier. Format: \"A\", 7 digits, \"X\" or another digit. For the newer 16-digit format, use \"NLP ID (PLWABN record)\" (P7293)",
        "data_type": "ExternalId"
    },
    "P1802": {
        "label": "EMLO person ID",
        "description": "identifier in the Early Modern Letters Online project run by the Bodleian Library",
        "data_type": "ExternalId"
    },
    "P2401": {
        "label": "Six Degrees of Francis Bacon ID",
        "description": "identifier in the Six Degrees of Francis Bacon database",
        "data_type": "ExternalId"
    },
    "P2266": {
        "label": "FMD model ID",
        "description": "external link to a model's profile at the FMD (Fashion Model Directory) website",
        "data_type": "ExternalId"
    },
    "P1869": {
        "label": "Hall of Valor ID",
        "description": "identifier for a medal recipient, in the US Hall of Valor",
        "data_type": "ExternalId"
    },
    "P1711": {
        "label": "British Museum person or institution ID",
        "description": "identifier for a person or institution in the British Museum person-institution thesaurus",
        "data_type": "ExternalId"
    },
    "P1710": {
        "label": "Sächsische Biografie ID",
        "description": "identifier in a biographical dictionary of people from Saxony",
        "data_type": "ExternalId"
    },
    "P2428": {
        "label": "RePEc Short-ID",
        "description": "identifier for a researcher in the RePEc (Research Papers in Economics) and IDEAS database",
        "data_type": "ExternalId"
    },
    "P1469": {
        "label": "FIFA player ID",
        "description": "identifier for a association football player per FIFA",
        "data_type": "ExternalId"
    },
    "P1979": {
        "label": "Righteous Among The Nations ID",
        "description": "identifier in the database of The Righteous Among The Nations",
        "data_type": "ExternalId"
    },
    "P2472": {
        "label": "ACMA Radiocommunications Client Number",
        "description": "identifier of an organisation or person who holds a radiocommunications licence within Australia",
        "data_type": "ExternalId"
    },
    "P2471": {
        "label": "Models.com person ID",
        "description": "identifier of a person in the Models.com website",
        "data_type": "ExternalId"
    },
    "P6144": {
        "label": "ortsnamen.ch ID",
        "description": "identifier for a toponym/geographical object on ortsnamen.ch",
        "data_type": "ExternalId"
    },
    "P7181": {
        "label": "PLIA ID",
        "description": "identifier for a place on the Public Lands Interpretive Association website",
        "data_type": "ExternalId"
    },
    "P6148": {
        "label": "GLAMOS glacier ID",
        "description": "identifier for a Swiss glacier on the GLAMOS website",
        "data_type": "ExternalId"
    },
    "P8068": {
        "label": "ToposText place ID",
        "description": "identifier for a place in the website ToposText",
        "data_type": "ExternalId"
    },
    "P3866": {
        "label": "LAWA waterbody ID",
        "description": "identifier for bodies of water in Germany according to the Bund/Länder-Arbeitsgemeinschaft Wasser (LAWA)",
        "data_type": "ExternalId"
    },
    "P7225": {
        "label": "BEACON ID",
        "description": "identifier for a beach of the United States on the Beach Advisory and Closing On-line Notification website",
        "data_type": "ExternalId"
    },
    "P4388": {
        "label": "Mappy place ID",
        "description": "identifier for place in Mappy",
        "data_type": "ExternalId"
    },
    "P7942": {
        "label": "National Historic Lookout Register ID",
        "description": "identifier for a fire lookout on the National Historic Lookout Register",
        "data_type": "ExternalId"
    },
    "P8564": {
        "label": "Art Bonus ID",
        "description": "identifier for Italian national heritage which experienced interventions registered in the website Art Bonus",
        "data_type": "ExternalId"
    },
    "P6778": {
        "label": "OBO Gazetteer ID",
        "description": "identifier from the Open Bio Ontologies Gazetteer (GAZ) (without prefix)",
        "data_type": "ExternalId"
    },
    "P4075": {
        "label": "Czech Monument Catalogue Number",
        "description": "ID given to protected but also some unprotected monuments and other objects in the Czech Republic on the official Monument Catalogue of the National Heritage Institute",
        "data_type": "ExternalId"
    },
    "P6703": {
        "label": "Shanghai Library place ID",
        "description": "identifier for places in the Shanghai Library Name Authority Database",
        "data_type": "ExternalId"
    },
    "P6484": {
        "label": "Antarctica NZ Digital Asset Manager",
        "description": "identifier in the composite gazetteer for Antarctica run New Zealand",
        "data_type": "ExternalId"
    },
    "P6192": {
        "label": "Hembygdsportalen ID",
        "description": "Hembygdsportalen is a site containing local history material added by +600 local history communities in Sweden",
        "data_type": "ExternalId"
    },
    "P7271": {
        "label": "Hiking Project area ID",
        "description": "identifier for an area on the Hiking Project website",
        "data_type": "ExternalId"
    },
    "P8552": {
        "label": "Opera Online opera house ID",
        "description": "identifier of a opera or theater on Opera Online website",
        "data_type": "ExternalId"
    },
    "P4245": {
        "label": "cultural heritage monument in Serbia ID",
        "description": "identifier for cultural heritage sites in the Republic of Serbia, from the WLM database",
        "data_type": "ExternalId"
    },
    "P4249": {
        "label": "Vincoli in Rete ID",
        "description": "identifier of a building or site in the database of the MIBACT (Ministero per i Beni, le Attività Culturali e il Turismo) concerning regulatory constraints on cultural heritage in Italy",
        "data_type": "ExternalId"
    },
    "P8424": {
        "label": "OpenHistoricalMap relation ID",
        "description": "identifier of a relation in OpenHistoricalMap; see also P402, the corresponding property for OpenStreetMap relations",
        "data_type": "ExternalId"
    },
    "P6807": {
        "label": "racing-reference track ID",
        "description": "identifier of a motor racing track, in the racing-reference.com database",
        "data_type": "ExternalId"
    },
    "P8293": {
        "label": "AusStage venue ID",
        "description": "identifier for a venue at AusStage",
        "data_type": "ExternalId"
    },
    "P4568": {
        "label": "SNCZI-IPE reservoir ID",
        "description": "identifier of a reservoir in Spain, in the SNCZI-Inventario de Presas y Embalses database",
        "data_type": "ExternalId"
    },
    "P4689": {
        "label": "Environmental Register code (Estonia)",
        "description": "identifier for an entry in the Estonian Environmental Register",
        "data_type": "ExternalId"
    },
    "P6842": {
        "label": "Campendium ID",
        "description": "identifier for a place on the Campendium website",
        "data_type": "ExternalId"
    },
    "P4711": {
        "label": "CHGIS ID",
        "description": "identifier for a historical administrative region of China in the China Historical Geographic Information System (CHGIS)",
        "data_type": "ExternalId"
    },
    "P4708": {
        "label": "VOGRIPA ID",
        "description": "identifier for a volcano on the Volcano Global Risk Identification and Analysis Project website",
        "data_type": "ExternalId"
    },
    "P6987": {
        "label": "Hungarian National Namespace place ID",
        "description": "identifier for a place in the Hungarian National Namespace",
        "data_type": "ExternalId"
    },
    "P4820": {
        "label": "TORA ID",
        "description": "identifier for a building, structure or settlement, in the Topographical Register for Historic Buildings in Sweden (TORA)",
        "data_type": "ExternalId"
    },
    "P3977": {
        "label": "Songkick venue ID",
        "description": "identifier for a venue, on Songkick",
        "data_type": "ExternalId"
    },
    "P6747": {
        "label": "Recreation.gov campground ID",
        "description": "identifier for a campground in an American recreation area on the Recreation.gov website",
        "data_type": "ExternalId"
    },
    "P8567": {
        "label": "The Living New Deal ID",
        "description": "identifier for a project on the Living New Deal website",
        "data_type": "ExternalId"
    },
    "P4592": {
        "label": "Mountain Project ID",
        "description": "identifier for a climbing area or route on the 'Mountain Project' website",
        "data_type": "ExternalId"
    },
    "P5173": {
        "label": "Rockipedia area ID",
        "description": "identifier of an area, an artisis homested, in the online service Rockipedia",
        "data_type": "ExternalId"
    },
    "P7396": {
        "label": "Beachsafe Beach Key",
        "description": "unique identifier of an Australian beach in the Beachsafe database operated by Surf Life Saving Australia",
        "data_type": "ExternalId"
    },
    "P4697": {
        "label": "Historic Place Names of Wales ID",
        "description": "identifier for a location in Wales on the List of Historic Place Names of Wales",
        "data_type": "ExternalId"
    },
    "P6233": {
        "label": "Place Names Register ID (Estonia)",
        "description": "identifier for a geographical object in the Estonian Place Names Register",
        "data_type": "ExternalId"
    },
    "P8399": {
        "label": "Unique Property Reference Number",
        "description": "unique identifier for every location that can have an address in the UK",
        "data_type": "ExternalId"
    },
    "P3907": {
        "label": "LoJ peak ID",
        "description": "identifier for a mountain on the Lists of John website",
        "data_type": "ExternalId"
    },
    "P6899": {
        "label": "New York City Neighborhood Tabulation area ID",
        "description": "unique identifier of this neighborhood in NYC Department of City Planning public dataset with geographic shapes (NTA aka NYNTA)",
        "data_type": "ExternalId"
    },
    "P8409": {
        "label": "Swedish Environmental Protection Agency Amenity OBJECTID",
        "description": "identifier from the Swedish Environmental Protection Agency geodata table with amenities related to protection areas",
        "data_type": "ExternalId"
    },
    "P6265": {
        "label": "MinDat Locality ID",
        "description": "ID of the minerals locality in MinDat database",
        "data_type": "ExternalId"
    },
    "P7272": {
        "label": "Hiking Project site ID",
        "description": "identifier for a remarkable site (a 'gem') on the Hiking Project website",
        "data_type": "ExternalId"
    },
    "P8217": {
        "label": "iDAI.gazetteer ID",
        "description": "identifier for a place in the database iDAI.gazetteer compiled by the German Archaeological Institute",
        "data_type": "ExternalId"
    },
    "P8218": {
        "label": "Archaeology in Greece Online place ID",
        "description": "identifier for a place in the database Archaeology in Greece Online compiled by the École française d'Athènes and the British School at Athens",
        "data_type": "ExternalId"
    },
    "P8221": {
        "label": "NMMA built heritage ID",
        "description": "identifier for heritage sites of India which are documented by National Mission on Monuments and Antiquities",
        "data_type": "ExternalId"
    },
    "P6814": {
        "label": "NWBib ID",
        "description": "Authority ID in the spatial classification of the North Rhine-Westphalian Bibliography (NWBib)",
        "data_type": "ExternalId"
    },
    "P8447": {
        "label": "Unique Street Reference Number",
        "description": "unique 8 digit number assigned to streets, roads, footpaths or public rights of way in the UK",
        "data_type": "ExternalId"
    },
    "P5200": {
        "label": "eBird hotspot ID",
        "description": "identifier for a nature reserve or other site (\"hotspot\") in the eBird database",
        "data_type": "ExternalId"
    },
    "P8593": {
        "label": "Hikr waypoint ID",
        "description": "numerical identifier for a waypoint on hikr.org",
        "data_type": "ExternalId"
    },
    "P6060": {
        "label": "MoEML ID",
        "description": "ID for a place, person, organisation, or work at the Map of Early Modern London project of the University of Victoria, Canada",
        "data_type": "ExternalId"
    },
    "P4098": {
        "label": "BVMC place ID",
        "description": "identifier of a place on the Biblioteca Virtual Miguel de Cervantes, data.cervantesvirtual.com",
        "data_type": "ExternalId"
    },
    "P7279": {
        "label": "Peakbagger key col ID",
        "description": "identifier for a key col, on the peakbagger.com website",
        "data_type": "ExternalId"
    },
    "P5879": {
        "label": "Qualité Tourisme ID",
        "description": "identifier for a place on the Qualité Tourisme website",
        "data_type": "ExternalId"
    },
    "P5215": {
        "label": "GONIAT place ID",
        "description": "identifier for a place, in the GONIAT database",
        "data_type": "ExternalId"
    },
    "P8623": {
        "label": "American Battlefield Trust battlefield ID",
        "description": "identifier for a battlefield on the American Battlefield Trust website",
        "data_type": "ExternalId"
    },
    "P5818": {
        "label": "BGCI garden ID",
        "description": "identifier for a botanical garden in the Botanic Gardens Conservation International online database",
        "data_type": "ExternalId"
    },
    "P4173": {
        "label": "Instagram location ID",
        "description": "identifier for a location on Instagram",
        "data_type": "ExternalId"
    },
    "P6515": {
        "label": "DoBIH Number",
        "description": "stable identifying number for hills in the Database of British and Irish Hills",
        "data_type": "ExternalId"
    },
    "P6915": {
        "label": "Gambay ID",
        "description": "identifier for a language or place in the Gambay First Languages of Australia map",
        "data_type": "ExternalId"
    },
    "P4118": {
        "label": "NLS-FI Geographic Name ID",
        "description": "National Land Survey of Finland Geographic Name ID",
        "data_type": "ExternalId"
    },
    "P8348": {
        "label": "NVE River Network (ELVIS) ID",
        "description": "unique identifier for rivers and tributaries in NVE River Network",
        "data_type": "ExternalId"
    },
    "P5601": {
        "label": "TCI destination ID",
        "description": "identifier in the geographical database of the Touring Club Italiano, or Touring Club of Italy",
        "data_type": "ExternalId"
    },
    "P7345": {
        "label": "IGNrando' ID",
        "description": "identifier for a point of interest on the IGNrando' website",
        "data_type": "ExternalId"
    },
    "P4246": {
        "label": "LfD-HB object ID",
        "description": "identifier for cultural heritage monuments issued by the Bremen State Office for the Preservation of Monuments",
        "data_type": "ExternalId"
    },
    "P7471": {
        "label": "iNaturalist place ID",
        "description": "identifier for a place in iNaturalist",
        "data_type": "ExternalId"
    },
    "P6644": {
        "label": "Fandango theater ID",
        "description": "movie theater identifier at Fandango.com",
        "data_type": "ExternalId"
    },
    "P8483": {
        "label": "PM20 geo code",
        "description": "code for a geographical category in the 20th Century Press Archives' \"countries\" category system",
        "data_type": "ExternalId"
    },
    "P3972": {
        "label": "PASE Domesday place",
        "description": "Identifier for a settlement in 1066 or 1087 on the PASE Domesday website",
        "data_type": "ExternalId"
    },
    "P8246": {
        "label": "Natural Atlas ID",
        "description": "identifier for a place on the Natural Atlas website",
        "data_type": "ExternalId"
    },
    "P4672": {
        "label": "EMLO location ID",
        "description": "identifier for a location in the Early Modern Letters Online database maintained by the University of Oxford",
        "data_type": "ExternalId"
    },
    "P4127": {
        "label": "EmbassyPages.com ID",
        "description": "ID of an embassy or a consulate general in the database for diplomatic missions, EmbassyPages.com",
        "data_type": "ExternalId"
    },
    "P8649": {
        "label": "National Park Service place ID",
        "description": "identifier for a place in the United States on the National Park Service website",
        "data_type": "ExternalId"
    },
    "P4005": {
        "label": "Bavarikon ID",
        "description": "identifier of a geographical object in the database of places of Bavarikon",
        "data_type": "ExternalId"
    },
    "P4007": {
        "label": "DBS ID",
        "description": "identifier in the German/Austrian Library Statistics (DBS/ÖBS)",
        "data_type": "ExternalId"
    },
    "P7444": {
        "label": "Rijksmuseum Research Library authority ID",
        "description": "authority control identifier for the Rijksmuseum Research Library",
        "data_type": "ExternalId"
    },
    "P8587": {
        "label": "Legacies of British Slave-ownership place ID",
        "description": "identifier for a physical item in the Legacies of British Slave-Ownership database",
        "data_type": "ExternalId"
    },
    "P4059": {
        "label": "Irish National Monument ID",
        "description": "identifier of a structure, in National Monument database of Ireland, maintained by Heritage Ireland",
        "data_type": "ExternalId"
    },
    "P4266": {
        "label": "Bavarian geotope ID",
        "description": "identifier for geotopes in Bavaria (Germany), issued by the Bayerisches Landesamt für Umwelt",
        "data_type": "ExternalId"
    },
    "P8119": {
        "label": "HASC",
        "description": "codes to represent country subdivisions",
        "data_type": "ExternalId"
    },
    "P3992": {
        "label": "SSB urban settlement number",
        "description": "Numeric ID for urban settlements in Norway assigned by Statistics Norway (SSB) since 1980",
        "data_type": "ExternalId"
    },
    "P4279": {
        "label": "hydrological order number",
        "description": "identifier for water bodies in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P5388": {
        "label": "Bulgarian Antarctic Gazetteer ID",
        "description": "identifier of a location in Antarctica in the Bulgarian Antarctic Gazetteer",
        "data_type": "ExternalId"
    },
    "P4227": {
        "label": "Indonesian Small Islands Directory ID",
        "description": "identifier for DPKI directory of small islands by the Indonesian government",
        "data_type": "ExternalId"
    },
    "P7850": {
        "label": "Joconde location ID",
        "description": "identifier for places from the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P8568": {
        "label": "Jewish Virtual Library ID",
        "description": "identifier in the Jewish Virtual Library",
        "data_type": "ExternalId"
    },
    "P8655": {
        "label": "LoC HABS/HAER/HALS place ID",
        "description": "alphanumerical identifier for a place covered by the Historic American Buildings Survey, Historic American Engineering Record or the Historic American Landscapes Survey on the Library of Congress website",
        "data_type": "ExternalId"
    },
    "P6727": {
        "label": "Dimore Storiche Italiane ID",
        "description": "identifier of an Italian villa, palace, garden or park in the database Dimore Storiche Italiane",
        "data_type": "ExternalId"
    },
    "P7102": {
        "label": "Grand Canyon Trust ID",
        "description": "identifier for a place on the Grand Canyon Trust website",
        "data_type": "ExternalId"
    },
    "P4171": {
        "label": "World Heritage Tentative List ID",
        "description": "identifier for a site on the UNESCO World Heritage Tentative Lists",
        "data_type": "ExternalId"
    },
    "P3353": {
        "label": "National Recreation Trails Database ID",
        "description": "identifier for a trail in the United States' National Recreation Trails Database",
        "data_type": "ExternalId"
    },
    "P2845": {
        "label": "RAN ID",
        "description": "identifier in National archaeological register of Romania",
        "data_type": "ExternalId"
    },
    "P1483": {
        "label": "kulturnoe-nasledie.ru ID",
        "description": "identifier for an item in a Russian cultural heritage register",
        "data_type": "ExternalId"
    },
    "P709": {
        "label": "Historic Scotland ID",
        "description": "identifier for a building in the Historic Scotland database",
        "data_type": "ExternalId"
    },
    "P1715": {
        "label": "RKD/ESD (Slovenia) ID",
        "description": "reference number of intangible cultural heritage of Slovenia (EID)",
        "data_type": "ExternalId"
    },
    "P3728": {
        "label": "New Zealand Heritage List number",
        "description": "list number in the New Zealand Heritage List, issued by the Heritage New Zealand Pouhere Taonga",
        "data_type": "ExternalId"
    },
    "P2967": {
        "label": "Queensland Heritage Register ID",
        "description": "identifier of a property listed on the Queensland Heritage Register",
        "data_type": "ExternalId"
    },
    "P1551": {
        "label": "Exceptional heritage of Wallonia ID",
        "description": "unique identifier of the \"Patrimoine immobilier exceptionnel de la Région wallonne\"",
        "data_type": "ExternalId"
    },
    "P1369": {
        "label": "Iranian National Heritage registration number",
        "description": "identifier in the Iranian National Heritage database",
        "data_type": "ExternalId"
    },
    "P3008": {
        "label": "Australian Heritage Database Place ID",
        "description": "identifier \"Place ID\" on the Australian Heritage Database",
        "data_type": "ExternalId"
    },
    "P1702": {
        "label": "DGPC ID",
        "description": "identifier in the DGPC (Directorate General of Cultural Heritage) database of heritage sites in Portugal",
        "data_type": "ExternalId"
    },
    "P3038": {
        "label": "IWM memorial ID",
        "description": "identifier for a war memorial, in the UK Imperial War Museum's War Memorials Register",
        "data_type": "ExternalId"
    },
    "P1459": {
        "label": "Cadw Building ID",
        "description": "identifier for listed buildings in Wales",
        "data_type": "ExternalId"
    },
    "P4868": {
        "label": "Hispania Nostra Red List code",
        "description": "identifier for an endangered heritage item (sometimes a natural element) as listed in Hispania Nostra's Red List of Endangered Heritage",
        "data_type": "ExternalId"
    },
    "P1794": {
        "label": "bureau du patrimoine de Seine-Saint-Denis ID",
        "description": "identifier in the atlas-patrimoine93.fr website",
        "data_type": "ExternalId"
    },
    "P1770": {
        "label": "Romania LMI code",
        "description": "identifier in 'Lista monumentelor istorice din România' (Romanian heritage sites)",
        "data_type": "ExternalId"
    },
    "P1428": {
        "label": "Lost Art ID",
        "description": "identifier on the German Lost Art Foundation website",
        "data_type": "ExternalId"
    },
    "P3424": {
        "label": "Polish cultural heritage register number",
        "description": "reference in a Polish cultural heritage register",
        "data_type": "ExternalId"
    },
    "P2783": {
        "label": "Danish listed buildings case ID",
        "description": "case number of a listed building in Denmark",
        "data_type": "ExternalId"
    },
    "P845": {
        "label": "Saskatchewan Register of Heritage Property ID",
        "description": "identifier in the Saskatchewan Register of Heritage Property",
        "data_type": "ExternalId"
    },
    "P1626": {
        "label": "Thai cultural heritage ID",
        "description": "identifier for cultural heritage properties in Thailand",
        "data_type": "ExternalId"
    },
    "P1460": {
        "label": "NIEA building ID",
        "description": "identifier for listed buildings in Northern Ireland",
        "data_type": "ExternalId"
    },
    "P633": {
        "label": "Quebec cultural heritage directory ID",
        "description": "identifier in the directory of the cultural heritage of Quebec",
        "data_type": "ExternalId"
    },
    "P4587": {
        "label": "Argentinian Historic Heritage ID",
        "description": "identifier for historic heritage object included in the national index of Argentina",
        "data_type": "ExternalId"
    },
    "P2494": {
        "label": "Latvian cultural heritage register ID",
        "description": "ID of the cultural heritage on https://is.mantojums.lv/",
        "data_type": "ExternalId"
    },
    "P4142": {
        "label": "RIWAQ Registry of Historic Buildings in Palestine ID",
        "description": "identifier for a historic building in Palestine on the RIWAQ database",
        "data_type": "ExternalId"
    },
    "P649": {
        "label": "NRHP reference number",
        "description": "identifier in the National Register of Historic Places assigned by the National Park Service of the USA",
        "data_type": "ExternalId"
    },
    "P3449": {
        "label": "NSW Heritage database ID",
        "description": "identifier for a place with recognised heritage value, in Australia's NSW heritage database",
        "data_type": "ExternalId"
    },
    "P3963": {
        "label": "Clochers de France ID",
        "description": "identifier of a religious building on the clochers.org website",
        "data_type": "ExternalId"
    },
    "P3197": {
        "label": "Federal Heritage Buildings ID (Canada)",
        "description": "identifier of a building designated by the   Federal Heritage Buildings Review Office of Canada",
        "data_type": "ExternalId"
    },
    "P4372": {
        "label": "iPatrimônio ID",
        "description": "identifer for a Brazilian historical Heritage asset, in the Infopatrimônio database",
        "data_type": "ExternalId"
    },
    "P4166": {
        "label": "Georgian National Register of Monuments ID",
        "description": "identifier for a monument in the National Register of Monuments of Georgia",
        "data_type": "ExternalId"
    },
    "P933": {
        "label": "Heritage NL ID",
        "description": "identifier of Heritage Foundation of Newfoundland and Labrador heritage property",
        "data_type": "ExternalId"
    },
    "P3324": {
        "label": "petit-patrimoine.com ID",
        "data_type": "ExternalId"
    },
    "P1799": {
        "label": "Maltese Islands National Inventory of Cultural Property ID",
        "description": "identifier in the National Inventory of Cultural Property of the Maltese Islands",
        "data_type": "ExternalId"
    },
    "P3518": {
        "label": "Smithsonian trinomial",
        "description": "unique identifier assigned to an archaeological site in one of many states in the United States",
        "data_type": "ExternalId"
    },
    "P1529": {
        "label": "Gertrude identifier",
        "description": "identifier for a cultural property per the Glad database of French cultural heritage",
        "data_type": "ExternalId"
    },
    "P4037": {
        "label": "South Australian Heritage Register Database ID",
        "description": "identifier for a place with recognised heritage value, in South Australia's heritage database",
        "data_type": "ExternalId"
    },
    "P1822": {
        "label": "DSH object ID",
        "description": "identifier for cultural monuments in Hamburg, Germany",
        "data_type": "ExternalId"
    },
    "P3007": {
        "label": "Cadw Monument ID",
        "description": "identifier for a scheduled monument in Wales",
        "data_type": "ExternalId"
    },
    "P808": {
        "label": "Bien de Interés Cultural (BIC) code",
        "description": "identifier for an item in the Spanish heritage register (included RI/ARI)",
        "data_type": "ExternalId"
    },
    "P3259": {
        "label": "intangible cultural heritage status",
        "description": "status of an item that is designated as intangible heritage",
        "data_type": "WikibaseItem"
    },
    "P3177": {
        "label": "Patrimonio Web JCyL ID",
        "description": "identifier of a building or site in Patrimonio Web of Junta de Castilla y León",
        "data_type": "ExternalId"
    },
    "P2951": {
        "label": "Cultural heritage database in Austria ObjektID",
        "description": "identifier given by the published BDA (Bundesdenkmalamt) list of monuments",
        "data_type": "ExternalId"
    },
    "P1133": {
        "label": "DGO4 identifier",
        "description": "identifier per Direction générale opérationelle (Wallonia)",
        "data_type": "ExternalId"
    },
    "P481": {
        "label": "Palissy ID",
        "description": "identifier in the Palissy database of moveable objects of French cultural heritage",
        "data_type": "ExternalId"
    },
    "P1586": {
        "label": "Catalan object of cultural interest ID",
        "data_type": "ExternalId"
    },
    "P4244": {
        "label": "Bavarian monument authority ID",
        "description": "identifier for cultural heritage monuments (ensembles, buildings and grounds) in Bavaria, issued by the Bavarian State Office for the Preservation of Monuments",
        "data_type": "ExternalId"
    },
    "P2903": {
        "label": "Molendatabase verdwenen molens ID",
        "description": "identifier of former/disappeared mills in the Netherlands at the Molendatabase verdwenen molens",
        "data_type": "ExternalId"
    },
    "P909": {
        "label": "Nova Scotia Register of Historic Places ID",
        "description": "identifier in the Nova Scotia Register of Historic Places",
        "data_type": "ExternalId"
    },
    "P1764": {
        "label": "Flemish Heritage Object ID",
        "description": "identifier for a \"relict\" in the database of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P718": {
        "label": "Canmore ID",
        "description": "identifier in the Royal Commission on the Ancient and Historical Monuments of Scotland's Canmore database",
        "data_type": "ExternalId"
    },
    "P3170": {
        "label": "Cultural Heritage Armenia ID",
        "description": "identifier for a cultural object in Armenia",
        "data_type": "ExternalId"
    },
    "P758": {
        "label": "Kulturminne ID",
        "description": "Norwegian heritage property identification number of Riksantikvaren (the Directorate for Cultural Heritage)",
        "data_type": "ExternalId"
    },
    "P4334": {
        "label": "Heritage Conservation District of Ontario ID",
        "description": "heritage conservation district of Ontario reference number",
        "data_type": "ExternalId"
    },
    "P1371": {
        "label": "ASI Monument ID",
        "description": "identifier assigned to monuments catalogued by the Archaeological Survey of India",
        "data_type": "ExternalId"
    },
    "P3443": {
        "label": "Victorian Heritage Database ID",
        "description": "identifier for a heritage location, in Australia's Victorian Heritage Database",
        "data_type": "ExternalId"
    },
    "P2081": {
        "label": "BLDAM object ID",
        "description": "Object ID to identify cultural heritage monuments in Brandenburg state, Germany",
        "data_type": "ExternalId"
    },
    "P757": {
        "label": "World Heritage Site ID",
        "description": "identifier for a site as assigned by UNESCO. Use on one item per site, link parts with property \"has part\" (P527)",
        "data_type": "ExternalId"
    },
    "P1587": {
        "label": "Slovene Cultural Heritage Register ID",
        "description": "identifier for a cultural monument in the Slovene Cultural Heritage Register",
        "data_type": "ExternalId"
    },
    "P4120": {
        "label": "Ontario Heritage Act Register ID",
        "description": "identifier of a property listed on the Ontario Heritage Act Register",
        "data_type": "ExternalId"
    },
    "P2614": {
        "label": "World Heritage criteria",
        "description": "selection criteria for UNESCO's cultural and natural 'World Heritage' designation since 2005",
        "data_type": "WikibaseItem"
    },
    "P2491": {
        "label": "Radzima.org ID",
        "description": "identifier in the Radzima.org database of landmarks and tourist attractions of Belarus, Lithuania and the Podlasie region of Poland",
        "data_type": "ExternalId"
    },
    "P762": {
        "label": "Czech cultural heritage ID",
        "description": "identifier for cultural heritage properties in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P763": {
        "label": "PEI Register of Historic Places ID",
        "description": "identifier of a historic place in Prince Edward Island, Canada",
        "data_type": "ExternalId"
    },
    "P3503": {
        "label": "LombardiaBeniCulturali building ID",
        "description": "identifier of a historical building in the cultural heritage database of the Lombardy Region of Italy",
        "data_type": "ExternalId"
    },
    "P1600": {
        "label": "Inventari del Patrimoni Arquitectònic de Catalunya code",
        "description": "identifier for an architectural property in Catalonia. max. 5-digit numeric",
        "data_type": "ExternalId"
    },
    "P3227": {
        "label": "Cultural Heritage Kosovo ID",
        "description": "identifier for a cultural object in Kosovo",
        "data_type": "ExternalId"
    },
    "P632": {
        "label": "cultural properties of Belarus reference number",
        "description": "identifier for a cultural property of Belarus",
        "data_type": "ExternalId"
    },
    "P1949": {
        "label": "CulturaItalia ID",
        "description": "identification number for the CulturaItalia LOD section",
        "data_type": "ExternalId"
    },
    "P1708": {
        "label": "LfDS object ID",
        "description": "identifer for national heritage sites issued by the Landesamt für Denkmalpflege Sachsen",
        "data_type": "ExternalId"
    },
    "P2526": {
        "label": "National Historic Sites of Canada ID",
        "description": "the unique identificator of a National Historic Sites of Canada on the Directory of Federal Heritage Designations",
        "data_type": "ExternalId"
    },
    "P3759": {
        "label": "SAHRA heritage site ID",
        "description": "identifier of heritage sites in South Africa, assigned by the South African Heritage Resources Agency",
        "data_type": "ExternalId"
    },
    "P1644": {
        "label": "messes.info Catholic church ID",
        "description": "identifier",
        "data_type": "ExternalId"
    },
    "P4117": {
        "label": "National Record of the Historic Environment ID",
        "description": "Historic England identifier for archaeological, architectural, and maritime sites",
        "data_type": "ExternalId"
    },
    "P4057": {
        "label": "Irish Sites and Monuments Record ID",
        "description": "identifier in Sites and Monuments Record database of Ireland",
        "data_type": "ExternalId"
    },
    "P3318": {
        "label": "Patrimonio Inmueble de Andalucía ID",
        "description": "identifier for a cultural heritage item in the  Patrimonio Inmueble de Andalucía database (BDI)",
        "data_type": "ExternalId"
    },
    "P4273": {
        "label": "New York City Landmarks Preservation Commission ID",
        "description": "landmarks designated by the New York City Landmarks Preservation Commission",
        "data_type": "ExternalId"
    },
    "P5310": {
        "label": "Protected Buildings Register in Finland ID",
        "description": "Item ID in the Protected Buildings Register maintained by the Finnish Heritage Agency",
        "data_type": "ExternalId"
    },
    "P1260": {
        "label": "Swedish Open Cultural Heritage URI",
        "description": "identifier",
        "data_type": "ExternalId"
    },
    "P2948": {
        "label": "Estonian cultural monument ID",
        "description": "identifier in the Estonian national registry of cultural monuments",
        "data_type": "ExternalId"
    },
    "P3178": {
        "label": "Zaragoza monument ID",
        "description": "ID of a monument catalogued in zaragoza.es",
        "data_type": "ExternalId"
    },
    "P1700": {
        "label": "SIPA ID",
        "description": "identifier in the SIPA (Sistema de Informação para o Património Arquitectónico) database of heritage sites in Portugal",
        "data_type": "ExternalId"
    },
    "P2424": {
        "label": "Berlin cultural heritage ID",
        "description": "identifer for an object in the cultural heritage database of Berlin",
        "data_type": "ExternalId"
    },
    "P1216": {
        "label": "National Heritage List for England number",
        "description": "number on the National Heritage List for England",
        "data_type": "ExternalId"
    },
    "P4275": {
        "label": "Japanese Database of National important cultural properties",
        "description": "identifier of a site in the Japanese Database of National important cultural properties",
        "data_type": "ExternalId"
    },
    "P2952": {
        "label": "boat of heritage interest ID",
        "description": "identifier of a ship",
        "data_type": "ExternalId"
    },
    "P3596": {
        "label": "Danish ancient monument ID",
        "description": "identifier for protected ancient monument site in Denmark, assigned by Heritage Agency of Denmark",
        "data_type": "ExternalId"
    },
    "P3600": {
        "label": "protected heritage site in Brussels ID",
        "description": "identifier of a protected heritage site in Brussels, Belgium",
        "data_type": "ExternalId"
    },
    "P4509": {
        "label": "National Monuments of Namibia Site Reference",
        "description": "reference number used for National Monuments of Namibia by the National Heritage Council of Namibia",
        "data_type": "ExternalId"
    },
    "P5517": {
        "label": "Auvergne-Rhône-Alpes Regional Inventory identifier",
        "data_type": "ExternalId"
    },
    "P759": {
        "label": "Alberta Register of Historic Places ID",
        "description": "identifier of historic resources in the Alberta Register of Historic Places",
        "data_type": "ExternalId"
    },
    "P1126": {
        "label": "isospin quantum number",
        "description": "characteristic property of any particle",
        "data_type": "Quantity"
    },
    "P2102": {
        "label": "boiling point",
        "description": "temperature at which a substance changes its phase from liquid to gas (indicate the corresponding pressure as qualifier)",
        "data_type": "Quantity"
    },
    "P2101": {
        "label": "melting point",
        "description": "temperature at which a solid changes its state from solid to liquid at atmospheric pressure",
        "data_type": "Quantity"
    },
    "P2066": {
        "label": "fusion enthalpy",
        "description": "fusion enthalpy of a substance at the melting temperature",
        "data_type": "Quantity"
    },
    "P2054": {
        "label": "density",
        "description": "density of a substance with phase of matter and temperature as qualifiers",
        "data_type": "Quantity"
    },
    "P2056": {
        "label": "heat capacity",
        "description": "specific heat capacity of a substance, with phase of matter and temperature as qualifiers",
        "data_type": "Quantity"
    },
    "P2300": {
        "label": "minimal lethal dose",
        "description": "lowest concentration of a toxic substance in an environmental medium that kills individual organisms or test species under a defined set of conditions",
        "data_type": "Quantity"
    },
    "P2710": {
        "label": "minimal lethal concentration",
        "description": "lowest concentration of a toxic substance in an environmental medium that kills individual organisms or test species under a defined set of conditions",
        "data_type": "Quantity"
    },
    "P2073": {
        "label": "vehicle range",
        "description": "distance a vehicle can travel without refuelling",
        "data_type": "Quantity"
    },
    "P2216": {
        "label": "radial velocity",
        "description": "component of the object's velocity that points in the direction of the radius connecting the object and the point",
        "data_type": "Quantity"
    },
    "P2149": {
        "label": "clock speed",
        "description": "CPU's specified clock frequency (use with determination method = Q73207925 or Q73208059 as appropriate)",
        "data_type": "Quantity"
    },
    "P1295": {
        "label": "emissivity",
        "description": "ability of a substance to radiate thermal energy from its surface",
        "data_type": "Quantity"
    },
    "P2250": {
        "label": "life expectancy",
        "description": "average life expectancy for this group or species",
        "data_type": "Quantity"
    },
    "P2151": {
        "label": "focal length",
        "description": "focal length of lens (telescope, etc.)",
        "data_type": "Quantity"
    },
    "P2230": {
        "label": "torque",
        "description": "torque of a machine, a motor/engine/powerplant, or a vehicle",
        "data_type": "Quantity"
    },
    "P2793": {
        "label": "clearance",
        "description": "distance between surface under bridge and bottom of a bridge deck",
        "data_type": "Quantity"
    },
    "P1127": {
        "label": "isospin z-component",
        "description": "characteristic property of any elementary particle",
        "data_type": "Quantity"
    },
    "P2262": {
        "label": "draft",
        "description": "depth below the water line to the bottom of a vessel's hull",
        "data_type": "Quantity"
    },
    "P2118": {
        "label": "kinematic viscosity",
        "description": "viscosity of a substance, with phase of matter (P515) and temperature (P2076) as qualifiers",
        "data_type": "Quantity"
    },
    "P3020": {
        "label": "residence time of water",
        "description": "residence time of water in a lake or sea",
        "data_type": "Quantity"
    },
    "P2261": {
        "label": "beam",
        "description": "width of a ship at its widest point measured at its nominal waterline",
        "data_type": "Quantity"
    },
    "P2404": {
        "label": "time-weighted average exposure limit",
        "description": "recommended or required concentration limit for chemical exposure in a workplace in a given work day",
        "data_type": "Quantity"
    },
    "P2044": {
        "label": "elevation above sea level",
        "description": "height of the item (geographical object) as measured relative to sea level",
        "data_type": "Quantity"
    },
    "P2075": {
        "label": "speed of sound",
        "description": "speed of sound waves in a material. Use qualifiers to specify the measurement conditions.",
        "data_type": "Quantity"
    },
    "P2791": {
        "label": "power consumed",
        "description": "electrical power consumed by an appliance",
        "data_type": "Quantity"
    },
    "P2046": {
        "label": "area",
        "description": "area occupied by an object",
        "data_type": "Quantity"
    },
    "P2160": {
        "label": "mass excess",
        "description": "difference between its actual mass and its mass number in atomic mass units",
        "data_type": "Quantity"
    },
    "P2154": {
        "label": "binding energy",
        "description": "energy required to disassemble a whole system into separate parts",
        "data_type": "Quantity"
    },
    "P2227": {
        "label": "metallicity",
        "description": "abundance of elements that are heavier than hydrogen or helium in an astronomical object",
        "data_type": "Quantity"
    },
    "P2217": {
        "label": "cruise speed",
        "description": "design cruise speed",
        "data_type": "Quantity"
    },
    "P2565": {
        "label": "global warming potential",
        "description": "heat trapped by a certain gas in CO2 equivalents",
        "data_type": "Quantity"
    },
    "P2113": {
        "label": "sublimation temperature",
        "description": "temperature at which a substance changes directly from solid to gaseous (indicate corresponding pressure qualifier)",
        "data_type": "Quantity"
    },
    "P2442": {
        "label": "conversion to standard unit",
        "description": "translation of the value of a measurement to different units",
        "data_type": "Quantity"
    },
    "P1123": {
        "label": "parity quantum number",
        "description": "property of particles",
        "data_type": "Quantity"
    },
    "P2116": {
        "label": "enthalpy of vaporization",
        "description": "vaporization enthalpy of a substance with temperature as qualifier",
        "data_type": "Quantity"
    },
    "P2128": {
        "label": "flash point",
        "description": "lowest temperature at which it can vaporize to form an ignitable mixture in air. Add qualifier for measurement method (P459): open cup or closed cup. Not equal to fire point.",
        "data_type": "Quantity"
    },
    "P2260": {
        "label": "ionization energy",
        "description": "minimum amount of energy required to remove an electron from an atom, ion or molecule in the gaseous state",
        "data_type": "Quantity"
    },
    "P2806": {
        "label": "vibration",
        "description": "level of vibration measured",
        "data_type": "Quantity"
    },
    "P2112": {
        "label": "wing area",
        "description": "area of an aircraft's wing surfaces",
        "data_type": "Quantity"
    },
    "P2051": {
        "label": "M sin i",
        "description": "exoplanet mass multiplied by the inclination angle",
        "data_type": "Quantity"
    },
    "P2718": {
        "label": "lowest-observed-adverse-effect level",
        "description": "lowest concentration or amount of a substance (dose), found by experiment or observation, which causes an adverse effect on morphology, functional capacity, growth, development, or life span of a target organism distinguishable from normal (control) organisms of the same species and strain under defined conditions of exposure",
        "data_type": "Quantity"
    },
    "P2049": {
        "label": "width",
        "description": "width of an object",
        "data_type": "Quantity"
    },
    "P2150": {
        "label": "FSB speed",
        "description": "CPU front-side bus speed",
        "data_type": "Quantity"
    },
    "P2228": {
        "label": "maximum thrust",
        "description": "maximum thrust specified for a jet engine",
        "data_type": "Quantity"
    },
    "P2233": {
        "label": "semi-major axis of an orbit",
        "description": "semi-major axis of a stable orbit (Astronomy)",
        "data_type": "Quantity"
    },
    "P2234": {
        "label": "volume as quantity",
        "description": "quantity of three-dimensional space",
        "data_type": "Quantity"
    },
    "P2370": {
        "label": "conversion to SI unit",
        "description": "conversion of the unit into a coherent SI unit",
        "data_type": "Quantity"
    },
    "P2067": {
        "label": "mass",
        "description": "mass (in colloquial usage also known as weight) of the item",
        "data_type": "Quantity"
    },
    "P2808": {
        "label": "wavelength",
        "description": "spatial period of a wave",
        "data_type": "Quantity"
    },
    "P2386": {
        "label": "diameter",
        "description": "the diameter of a circular or spherical object",
        "data_type": "Quantity"
    },
    "P2069": {
        "label": "magnetic moment",
        "description": "torque a particle will experience in an external magnetic field",
        "data_type": "Quantity"
    },
    "P2144": {
        "label": "frequency",
        "description": "frequency in Hz at which the subject works, for example the frequency a radio station can be received",
        "data_type": "Quantity"
    },
    "P2923": {
        "label": "focal height",
        "description": "height of the lamp of a lighthouse from water level",
        "data_type": "Quantity"
    },
    "P2043": {
        "label": "length",
        "description": "measured dimension of an object",
        "data_type": "Quantity"
    },
    "P2797": {
        "label": "sound power level",
        "description": "level of sound power at a reference value of 1 pW",
        "data_type": "Quantity"
    },
    "P2532": {
        "label": "lowest atmospheric pressure",
        "description": "minimum pressure measured or estimated for a storm (a measure of strength for tropical cyclones)",
        "data_type": "Quantity"
    },
    "P2784": {
        "label": "Mercalli intensity scale",
        "description": "measurement of the intensity of an earthquake",
        "data_type": "WikibaseItem"
    },
    "P2229": {
        "label": "thermal design power",
        "description": "specified amount of heat a computer component generates under normal conditions that the cooling system must dissipate",
        "data_type": "Quantity"
    },
    "P2055": {
        "label": "electrical conductivity",
        "description": "electrical conductivity of a substance with phase of matter and temperature as qualifiers",
        "data_type": "Quantity"
    },
    "P2807": {
        "label": "molar volume",
        "description": "qualifier with e.g. P873",
        "data_type": "Quantity"
    },
    "P2068": {
        "label": "thermal conductivity",
        "description": "property of a material that describes how efficiently heat is conducted",
        "data_type": "Quantity"
    },
    "P2547": {
        "label": "perimeter",
        "description": "linear distance around the outside of an object",
        "data_type": "Quantity"
    },
    "P2231": {
        "label": "explosive velocity",
        "data_type": "Quantity"
    },
    "P2048": {
        "label": "height",
        "description": "vertical length of an entity",
        "data_type": "Quantity"
    },
    "P2362": {
        "label": "time to altitude",
        "description": "time required by the subject to reach the specified altitude, from sea level",
        "data_type": "Quantity"
    },
    "P2107": {
        "label": "decomposition point",
        "description": "decomposition point of a substance",
        "data_type": "Quantity"
    },
    "P2117": {
        "label": "combustion enthalpy",
        "description": "enthalpy difference of a substance for combustion",
        "data_type": "Quantity"
    },
    "P2052": {
        "label": "speed",
        "description": "magnitude of the velocity of the item",
        "data_type": "Quantity"
    },
    "P2053": {
        "label": "watershed area",
        "description": "size of a stream's watershed (drainage basin)",
        "data_type": "Quantity"
    },
    "P2199": {
        "label": "autoignition temperature",
        "description": "lowest temperature at which it will spontaneously ignite in a normal atmosphere without an external source of ignition",
        "data_type": "Quantity"
    },
    "P2114": {
        "label": "half-life",
        "description": "time required for the amount of something to fall to half its initial value",
        "data_type": "Quantity"
    },
    "P2145": {
        "label": "explosive energy equivalent",
        "description": "amount of energy discharged during an explosion",
        "data_type": "Quantity"
    },
    "P2097": {
        "label": "term length of office",
        "description": "length of time in years (unit: Q577) a person (usually a politician) is to serve in a particular office. Do not add bounds.",
        "data_type": "Quantity"
    },
    "P2202": {
        "label": "lower flammable limit",
        "description": "the lower bound of the concentration range over which a flammable mixture of gas or vapour in air can be ignited at a given temperature and pressure (0-1)",
        "data_type": "Quantity"
    },
    "P2211": {
        "label": "position angle",
        "description": "measurement relating to observed visual binary stars (use with P2210)",
        "data_type": "Quantity"
    },
    "P2527": {
        "label": "earthquake magnitude on the moment magnitude scale",
        "description": "magnitude of an earthquake according to the moment magnitude scale",
        "data_type": "Quantity"
    },
    "P2717": {
        "label": "no-observed-adverse-effect level",
        "description": "greatest concentration or amount of a substance, found by experiment or observation, which causes no detectable adverse alteration of morphology, functional capacity, growth, development, or life span of the target organism under defined conditions of exposure",
        "data_type": "Quantity"
    },
    "P2542": {
        "label": "acceptable daily intake",
        "description": "estimate of the amount of a food additive, expressed on a body weight basis, that can be ingested daily over a lifetime without appreciable health risk",
        "data_type": "Quantity"
    },
    "P2129": {
        "label": "immediately dangerous to life or health",
        "description": "Immediately Dangerous to Life and Health value",
        "data_type": "Quantity"
    },
    "P2248": {
        "label": "argument of periapsis",
        "description": "angle from the body's ascending node to its periapsis",
        "data_type": "Quantity"
    },
    "P2254": {
        "label": "maximum operating altitude",
        "description": "ceiling or maximum density altitude at which an aircraft can operate",
        "data_type": "Quantity"
    },
    "P2060": {
        "label": "luminosity",
        "description": "total amount of energy emitted by an astronomical object per unit time",
        "data_type": "Quantity"
    },
    "P2050": {
        "label": "wingspan",
        "description": "distance from one wingtip to the other, of an airplane or an animal",
        "data_type": "Quantity"
    },
    "P2528": {
        "label": "earthquake magnitude on the Richter magnitude scale",
        "description": "magnitude of an earthquake according to the Richter magnitude scale",
        "data_type": "Quantity"
    },
    "P2873": {
        "label": "time in space",
        "description": "time in space by an astronaut or cosmonaut",
        "data_type": "Quantity"
    },
    "P2119": {
        "label": "vapor pressure",
        "description": "pressure exerted by a vapor in thermodynamic equilibrium with its condensed phases",
        "data_type": "Quantity"
    },
    "P2645": {
        "label": "mean lifetime",
        "description": "inverse of exponential decay rate; half-life is ln(2) times this value",
        "data_type": "Quantity"
    },
    "P2177": {
        "label": "solubility",
        "description": "property of a chemical to dissolve in another chemical forming a solution. Provide solvent (P2178) and temperature (P2076) as qualifiers",
        "data_type": "Quantity"
    },
    "P3013": {
        "label": "surface tension",
        "description": "physical property of liquid",
        "data_type": "Quantity"
    },
    "P2430": {
        "label": "takeoff roll",
        "description": "distance required for an aircraft to take off (usually measured to the point where the aircraft reaches 50 feet)",
        "data_type": "Quantity"
    },
    "P2120": {
        "label": "radius",
        "description": "distance between the center and the surface of a circle or sphere",
        "data_type": "Quantity"
    },
    "P3559": {
        "label": "maximum size or capacity",
        "description": "maximum allowed/supported/usable (data) size",
        "data_type": "Quantity"
    },
    "P3374": {
        "label": "endianness",
        "description": "order of the bytes that compose a digital word in computer memory",
        "data_type": "WikibaseItem"
    },
    "P7994": {
        "label": "Colecovision Zone ID",
        "description": "identifier of a video game in the Colecovision Zone database",
        "data_type": "ExternalId"
    },
    "P617": {
        "label": "yard number",
        "description": "identifier of a ship assigned by its builder (shipyard)",
        "data_type": "String"
    },
    "P4092": {
        "label": "checksum",
        "description": "small-sized datum derived from a block of digital data for the purpose of detecting errors. Use qualifier \"determination method\" (P459) to indicate how it's calculated, e.g. MD5.",
        "data_type": "String"
    },
    "P4653": {
        "label": "partition table type",
        "description": "qualifier for \"partition identifier\" that determines what type of partition table is used with the identifier",
        "data_type": "WikibaseItem"
    },
    "P4654": {
        "label": "partition type identifier",
        "description": "string that contains the identifier of a partition type specified in a partition table",
        "data_type": "String"
    },
    "P7966": {
        "label": "NetBSD package",
        "description": "name of the package available in the NetBSD Packages Collection",
        "data_type": "ExternalId"
    },
    "P7967": {
        "label": "OpenBSD port",
        "description": "name of the package available in the OpenBSD Ports Collection",
        "data_type": "ExternalId"
    },
    "P4809": {
        "label": "sets environment variable",
        "description": "an environment variable which a software program sets",
        "data_type": "WikibaseItem"
    },
    "P3966": {
        "label": "programming paradigm",
        "description": "programming paradigm in which a programming language is classified",
        "data_type": "WikibaseItem"
    },
    "P1945": {
        "label": "street key",
        "description": "identification number for a specific street within the street cadastre of a municipality",
        "data_type": "String"
    },
    "P1671": {
        "label": "route number",
        "description": "number of a rail or other public transport line",
        "data_type": "String"
    },
    "P4428": {
        "label": "implementation of",
        "description": "implementation of a standard, program, specification or programming language",
        "data_type": "WikibaseItem"
    },
    "P296": {
        "label": "station code",
        "description": "generic identifier for a railway station, when possible, use specific property on certain coding system (e.g. P1378 for China Railway TMIS codes)",
        "data_type": "String"
    },
    "P1824": {
        "label": "road number",
        "description": "number assigned to a stretch of public roadway",
        "data_type": "String"
    },
    "P4575": {
        "label": "HTML entity",
        "description": "string in HTML source code that represents this character on a page",
        "data_type": "String"
    },
    "P879": {
        "label": "pennant number",
        "description": "the number or designation associated with individual warships, often painted on the hull",
        "data_type": "String"
    },
    "P9238": {
        "label": "staking percentage",
        "description": "percentage of the funds that participates actively in a proof-of-stake network",
        "data_type": "Quantity"
    },
    "P2598": {
        "label": "serial number",
        "description": "an identifier for a specific object among the same product. Not a product code or model number",
        "data_type": "String"
    },
    "P8827": {
        "label": "GitLab username",
        "description": "username of this project, person or organization on GitLab.com",
        "data_type": "ExternalId"
    },
    "P3985": {
        "label": "supports programming language",
        "description": "programming language which is supported by this programming tool",
        "data_type": "WikibaseItem"
    },
    "P4742": {
        "label": "Uniform Resource Identifier Scheme",
        "description": "IANA-assigned uniform resource identifier scheme which identifies items within this class or set",
        "data_type": "ExternalId"
    },
    "P791": {
        "label": "ISIL",
        "description": "identifier for a library or related organization",
        "data_type": "ExternalId"
    },
    "P8778": {
        "label": "Docker Hub repository",
        "description": "Docker repository hosted on Docker Hub",
        "data_type": "ExternalId"
    },
    "P2009": {
        "label": "Exif model",
        "description": "string as it appears in the EXIF generated by a camera",
        "data_type": "String"
    },
    "P2010": {
        "label": "Exif make",
        "description": "string of the manufacturer as it appears in the EXIF generated by a specific digital camera model",
        "data_type": "String"
    },
    "P5196": {
        "label": "output method",
        "description": "output method or device used to interact with a software product",
        "data_type": "WikibaseItem"
    },
    "P3575": {
        "label": "data size",
        "description": "size of a software, dataset, neural network, or individual file",
        "data_type": "Quantity"
    },
    "P4387": {
        "label": "update method",
        "description": "method used by an app/OS to receive updates or self-update",
        "data_type": "WikibaseItem"
    },
    "P6931": {
        "label": "Repology project name",
        "description": "package name and identifier in Repology",
        "data_type": "ExternalId"
    },
    "P6665": {
        "label": "Pro-Linux.de DBApp ID",
        "description": "identifier for an app in the Pro-Linux.de database",
        "data_type": "ExternalId"
    },
    "P395": {
        "label": "licence plate code",
        "description": "distinguishing signs or parts of license plate associated with the subject. For countries: international licence plate country code or distinguishing sign of vehicles",
        "data_type": "String"
    },
    "P8423": {
        "label": "version control system",
        "description": "version control system used by a content repository",
        "data_type": "WikibaseItem"
    },
    "P8443": {
        "label": "Homebrew formula name",
        "description": "identifier of a package (formula) in the Homebrew package manager",
        "data_type": "ExternalId"
    },
    "P4837": {
        "label": "has command line option",
        "description": "an option which is accepted by the software from the command line",
        "data_type": "String"
    },
    "P212": {
        "label": "ISBN-13",
        "description": "identifier for a book (edition), thirteen digit",
        "data_type": "ExternalId"
    },
    "P2259": {
        "label": "mobile network code",
        "description": "code for mobile networks defined by ITU-T Recommendation E.212",
        "data_type": "String"
    },
    "P1685": {
        "label": "Pokédex / Pokémon browser number",
        "description": "identification number of a Pokémon in a Pokédex or Pokémon Browser",
        "data_type": "String"
    },
    "P5885": {
        "label": "Microsoft Store product ID",
        "description": "identifier for a application in Microsoft Store",
        "data_type": "ExternalId"
    },
    "P6948": {
        "label": "video system",
        "description": "video system hardware that the item uses",
        "data_type": "WikibaseItem"
    },
    "P946": {
        "label": "ISIN",
        "description": "identifier for a security",
        "data_type": "ExternalId"
    },
    "P249": {
        "label": "ticker symbol",
        "description": "identifier for a publicly traded share of a particular stock on a particular stock market or that of a cryptocurrency",
        "data_type": "String"
    },
    "P798": {
        "label": "military designation",
        "description": "officially assigned designation for a vehicle in military service",
        "data_type": "String"
    },
    "P8209": {
        "label": "SILL software ID",
        "description": "identifier for a libre software in the Socle interministériel de logiciels libres website",
        "data_type": "ExternalId"
    },
    "P8262": {
        "label": "npm package",
        "description": "name of a package on npm",
        "data_type": "ExternalId"
    },
    "P6765": {
        "label": "Guix Variable Name",
        "description": "identifier for a package record in the GNU Guix system",
        "data_type": "ExternalId"
    },
    "P4839": {
        "label": "Wolfram Language entity code",
        "description": "input form for an entity in Wolfram Language",
        "data_type": "ExternalId"
    },
    "P4998": {
        "label": "key press",
        "description": "keyboard shortcut sequence. See property talk page for applicable syntax",
        "data_type": "String"
    },
    "P4669": {
        "label": "tabular software version",
        "description": "historical list of software versions and their release dates",
        "data_type": "TabularData"
    },
    "P9025": {
        "label": "Mitre ATT&CK ID",
        "description": "identifier for an item in the Mitre ATT&CK knowledge base",
        "data_type": "ExternalId"
    },
    "P4043": {
        "label": "emulates",
        "description": "emulates the identified platform, CPU, or system",
        "data_type": "WikibaseItem"
    },
    "P473": {
        "label": "local dialing code",
        "description": "identifier dedicated to subject city by the area communication network",
        "data_type": "String"
    },
    "P4810": {
        "label": "reads environment variable",
        "description": "an environment variable which a software program reads and may be effected by",
        "data_type": "WikibaseItem"
    },
    "P7832": {
        "label": "Basque Vikidia ID",
        "description": "ID of the article in Basque Vikidia",
        "data_type": "ExternalId"
    },
    "P7841": {
        "label": "Armenian Vikidia ID",
        "description": "name of the article in Armenian Vikidia",
        "data_type": "ExternalId"
    },
    "P7806": {
        "label": "Games Database system ID",
        "description": "identifier of a video game system in the Games Database",
        "data_type": "ExternalId"
    },
    "P7812": {
        "label": "Macintosh Repository ID",
        "description": "identifier of a video game in the Macintosh Repository database",
        "data_type": "ExternalId"
    },
    "P7825": {
        "label": "UCUM code",
        "description": "case-sensitive code from the Unified Code for Units of Measure specification to identify a unit of measurement",
        "data_type": "ExternalId"
    },
    "P7707": {
        "label": "PlayStation DataCenter URL",
        "description": "identifier in the PlayStation DataCenter database",
        "data_type": "Url"
    },
    "P7870": {
        "label": "Analysis & Policy Observatory term ID",
        "description": "identifier for a term in the Analysis & Policy Observatory taxonomy",
        "data_type": "ExternalId"
    },
    "P7866": {
        "label": "marterl.at ID",
        "description": "identifier in the marterl.at database",
        "data_type": "ExternalId"
    },
    "P7845": {
        "label": "MarketScreener business leaders ID",
        "description": "ID of someone in MarketScreener",
        "data_type": "ExternalId"
    },
    "P7716": {
        "label": "ACMA Broadcast Service Licence Number",
        "description": "unique identifier of a Broadcast Service Licence issued by the Australian Communications and Media Authority",
        "data_type": "ExternalId"
    },
    "P7794": {
        "label": "Games Database developer ID",
        "description": "identifier of a video game developer in the Games Database",
        "data_type": "ExternalId"
    },
    "P7749": {
        "label": "Nomenclature for Museum Cataloging",
        "description": "nomenclature for object naming and classification, maintained by Canadian Heritage Information Network (CHIN) and American Association for State and Local History (AASLH)",
        "data_type": "ExternalId"
    },
    "P7751": {
        "label": "portable music history ID",
        "description": "identifier of a video game in the portable music history database",
        "data_type": "ExternalId"
    },
    "P7733": {
        "label": "Computer Emuzone game ID",
        "description": "identifier of a video game in the Computer Emuzone database",
        "data_type": "ExternalId"
    },
    "P7735": {
        "label": "Czech cohesion region ID",
        "description": "identifier for a cohesion region, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7759": {
        "label": "The Video Games Museum system ID",
        "description": "identifier of a video game system in the Video Games Museum database",
        "data_type": "ExternalId"
    },
    "P7795": {
        "label": "National September 11 Memorial ID",
        "description": "identifier of person killed in the attacks of September 11, 2001 and February 26, 1993.",
        "data_type": "ExternalId"
    },
    "P7822": {
        "label": "Italian Vikidia ID",
        "description": "name of the article in Italian Vikidia",
        "data_type": "ExternalId"
    },
    "P7830": {
        "label": "LiverTox ID",
        "description": "LiverTox database ID",
        "data_type": "ExternalId"
    },
    "P7819": {
        "label": "OrthodoxWiki ID (English)",
        "description": "numeric identifier of a page in the English edition of OrthodoxWiki.org",
        "data_type": "ExternalId"
    },
    "P7828": {
        "label": "What is the Apple IIGS? ID",
        "description": "identifier in the What is the Apple IIGS? database",
        "data_type": "ExternalId"
    },
    "P7771": {
        "label": "PersonalData.IO ID",
        "description": "identifier for an entity in the wiki.personaldata.io database",
        "data_type": "ExternalId"
    },
    "P7799": {
        "label": "Apple IIGS France ID",
        "description": "identifier of a software in the Apple IIGS France database",
        "data_type": "ExternalId"
    },
    "P7801": {
        "label": "CEEB K-12 school code",
        "description": "code issued by the College Board for K-12 schools",
        "data_type": "ExternalId"
    },
    "P7840": {
        "label": "ScummVM wiki ID",
        "description": "identifier in the ScummVM wiki",
        "data_type": "ExternalId"
    },
    "P7714": {
        "label": "tDAR creator ID",
        "description": "identifier for a person or organization in the Digital Archaeological Record",
        "data_type": "ExternalId"
    },
    "P7730": {
        "label": "ZX81 Collection publisher ID",
        "description": "identifier of a video game publisher in the ZX81 Collection database",
        "data_type": "ExternalId"
    },
    "P7823": {
        "label": "BookBrainz work ID",
        "description": "identifier for a work on the BookBrainz open book encyclopedia",
        "data_type": "ExternalId"
    },
    "P7834": {
        "label": "Authenticus ID",
        "description": "ID of a researcher in Authenticus website",
        "data_type": "ExternalId"
    },
    "P7826": {
        "label": "VGMPF ID",
        "description": "identifier in the Video Game Music Preservation Foundation wiki",
        "data_type": "ExternalId"
    },
    "P7853": {
        "label": "c64games.de ID",
        "description": "identifier of a video game in the www.c64games.de database",
        "data_type": "ExternalId"
    },
    "P7718": {
        "label": "Complex Portal accession ID",
        "description": "accession number of protein complex at the Complex Portal",
        "data_type": "ExternalId"
    },
    "P7724": {
        "label": "Littré ID",
        "description": "identifier for a Lexeme on the online Littré",
        "data_type": "ExternalId"
    },
    "P7694": {
        "label": "Aberdeenshire HER ID",
        "description": "ID for a heritage feature in the Aberdeenshire Historic Environment Record website and database",
        "data_type": "ExternalId"
    },
    "P7703": {
        "label": "Spanish Quora topic ID",
        "description": "identifier for a topic on Quora (Spanish language version)",
        "data_type": "ExternalId"
    },
    "P7702": {
        "label": "The Sierra Chest ID",
        "description": "identifier in The Sierra Chest database",
        "data_type": "ExternalId"
    },
    "P7704": {
        "label": "Europeana entity",
        "description": "identifier for persons, places and topics at Europeana",
        "data_type": "ExternalId"
    },
    "P7723": {
        "label": "The Encyclopedia of Oklahoma History and Culture ID",
        "description": "identifier for an entry on the Encyclopedia of Oklahoma History and Culture website",
        "data_type": "ExternalId"
    },
    "P7817": {
        "label": "Electronic library Ukrainica ID",
        "description": "identifier at the Electronic library Ukrainica",
        "data_type": "ExternalId"
    },
    "P7813": {
        "label": "Open Churches ID",
        "description": "ID for a church on the website of Open Churches project",
        "data_type": "ExternalId"
    },
    "P7757": {
        "label": "ScreenScraper platform ID",
        "description": "identifier of a video game platform in the ScreenScraper database",
        "data_type": "ExternalId"
    },
    "P7758": {
        "label": "SPIE profile ID",
        "description": "ID of a profile in SPIE and SPIE Digital Library",
        "data_type": "ExternalId"
    },
    "P7775": {
        "label": "RationalWiki ID",
        "description": "identifier for an item at RationalWiki",
        "data_type": "ExternalId"
    },
    "P7774": {
        "label": "The Good Old Days ID",
        "description": "identifier of a video game in The Good Old Days database",
        "data_type": "ExternalId"
    },
    "P7726": {
        "label": "PlanetMath ID",
        "description": "identifier for entries in PlanetMath, online mathematics reference work",
        "data_type": "ExternalId"
    },
    "P7747": {
        "label": "NES Cart Database ID",
        "description": "identifier of a video game in the NES Cart Database",
        "data_type": "ExternalId"
    },
    "P7846": {
        "label": "Beta maṣāḥǝft ID",
        "description": "stable identifier of a resource linked to the Ethiopic manuscript tradition present in the Beta maṣāḥǝft research environment (https://betamasaheft.eu/)",
        "data_type": "ExternalId"
    },
    "P7847": {
        "label": "DigitaltMuseum ID",
        "description": "Identifier at DigitaltMuseum",
        "data_type": "ExternalId"
    },
    "P7715": {
        "label": "World Flora Online ID",
        "description": "identifier for a plant in World Flora Online",
        "data_type": "ExternalId"
    },
    "P7780": {
        "label": "Russian PFL player ID",
        "description": "identifier for a player at the Russian PFL",
        "data_type": "ExternalId"
    },
    "P7731": {
        "label": "ZX81 Collection ID",
        "description": "identifier of a video game in the ZX81 Collection database",
        "data_type": "ExternalId"
    },
    "P7736": {
        "label": "Czech territorial region ID",
        "description": "identifier for a territorial region, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7740": {
        "label": "Garaph group ID",
        "description": "identifier of a video game group in the garaph.info database",
        "data_type": "ExternalId"
    },
    "P7746": {
        "label": "Natural Product Atlas ID",
        "description": "identifier in the Natural Product Atlas",
        "data_type": "ExternalId"
    },
    "P7753": {
        "label": "Projekt Gutenberg-DE ID",
        "description": "identifier for an author on the website Projekt Gutenberg-DE",
        "data_type": "ExternalId"
    },
    "P7701": {
        "label": "SNESmusic.org game ID",
        "description": "identifier of a video game in the SNESmusic.org database",
        "data_type": "ExternalId"
    },
    "P7773": {
        "label": "The Digital Local Culture Encyclopedia of Korea ID",
        "description": "ID of a topic in The Digital Local Culture Encyclopedia of Korea",
        "data_type": "ExternalId"
    },
    "P7752": {
        "label": "Prague municipal district ID",
        "description": "identifier for a municipal district of Prague, conferred by 'Český úřad zeměměřický a katastrální'",
        "data_type": "ExternalId"
    },
    "P7717": {
        "label": "Vaare falne ID",
        "description": "identifiers for Norwegian casualties of world war 2",
        "data_type": "ExternalId"
    },
    "P7818": {
        "label": "French Vikidia ID",
        "description": "ID of the article in French Vikidia",
        "data_type": "ExternalId"
    },
    "P7809": {
        "label": "LaunchBox Games Database platform ID",
        "description": "identifier of a video game platform in the LaunchBox Games Database",
        "data_type": "ExternalId"
    },
    "P7843": {
        "label": "German Vikidia ID",
        "description": "ID of the article in German Vikidia",
        "data_type": "ExternalId"
    },
    "P7821": {
        "label": "Sarvavijnanakosam ID",
        "description": "identifier for a topic in the Sarvavijnanakosam Malayalam language encyclopedia",
        "data_type": "ExternalId"
    },
    "P7772": {
        "label": "Atlas Obscura identifier",
        "description": "identifier for places at Atlas Obscura",
        "data_type": "ExternalId"
    },
    "P7868": {
        "label": "NPDRMI record ID",
        "description": "identifier for digital records in the National Portal and Digital Repository for Museums of India",
        "data_type": "ExternalId"
    },
    "P7852": {
        "label": "C64.COM ID",
        "description": "identifier of a video game in the WWW.C64.COM database",
        "data_type": "ExternalId"
    },
    "P7721": {
        "label": "Wikispore ID",
        "description": "page on Wikispore",
        "data_type": "ExternalId"
    },
    "P7783": {
        "label": "J-GLOBAL ID",
        "description": "identifier of a topic on J-GLOBAL",
        "data_type": "ExternalId"
    },
    "P7741": {
        "label": "IDFA film ID",
        "description": "identifier for a documentary film at the IDFA website",
        "data_type": "ExternalId"
    },
    "P7858": {
        "label": "Upper Austria Museum ID",
        "description": "Identifier to link Wikidata with the Upper Austrian database of museums",
        "data_type": "ExternalId"
    },
    "P7722": {
        "label": "TLFi ID",
        "description": "identifier for a lexeme on the Trésor de la langue française informatisé",
        "data_type": "ExternalId"
    },
    "P7802": {
        "label": "D-MSX ID",
        "description": "identifier of a video game in the D-MSX database",
        "data_type": "ExternalId"
    },
    "P7811": {
        "label": "Macintosh Garden game ID",
        "description": "identifier of a video game in the Macintosh Garden database",
        "data_type": "ExternalId"
    },
    "P7814": {
        "label": "OpenBibArt ID",
        "description": "identifier for a bibliographical reference on the OpenBibArt website",
        "data_type": "ExternalId"
    },
    "P7805": {
        "label": "Games Database publisher ID",
        "description": "identifier of a video game publisher in the Games Database",
        "data_type": "ExternalId"
    },
    "P7827": {
        "label": "Spanish Vikidia ID",
        "description": "name of the article in Spanish Vikidia",
        "data_type": "ExternalId"
    },
    "P7756": {
        "label": "ScreenScraper game ID",
        "description": "identifier of a video game in the ScreenScraper database",
        "data_type": "ExternalId"
    },
    "P7835": {
        "label": "Instituto Moreira Salles ID",
        "description": "identifier for an artwork in the collections of the Instituto Moreira Salles",
        "data_type": "ExternalId"
    },
    "P7842": {
        "label": "Vienna History Wiki ID",
        "description": "numeric identifier of a page in the Vienna History Wiki",
        "data_type": "ExternalId"
    },
    "P7692": {
        "label": "Utah History Encyclopedia ID",
        "description": "identifier for an entry on the Utah History Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P7693": {
        "label": "WyoHistory.org Encyclopedia ID",
        "description": "identifier for an entry on the WyoHistory.org website about the history of Wyoming",
        "data_type": "ExternalId"
    },
    "P7791": {
        "label": "NTIS accession number",
        "description": "identifier in the U.S. National Technical Information Service database",
        "data_type": "ExternalId"
    },
    "P7792": {
        "label": "PubAg ID",
        "description": "identifier in the PubAg database",
        "data_type": "ExternalId"
    },
    "P7838": {
        "label": "Polish Nationwide Railway Database - station ID",
        "description": "identifier of railway station in Polish Nationwide Railway Database (www.bazakolejowa.pl)",
        "data_type": "ExternalId"
    },
    "P7734": {
        "label": "console music history ID",
        "description": "identifier of a video game in the console music history database",
        "data_type": "ExternalId"
    },
    "P7738": {
        "label": "EEPS ID",
        "description": "identifier on the ''Encyclopedia of Evolutionary Psychological Science''",
        "data_type": "ExternalId"
    },
    "P7785": {
        "label": "LaunchBox Games Database game ID",
        "description": "identifier of a video game in the LaunchBox Games Database",
        "data_type": "ExternalId"
    },
    "P7764": {
        "label": "Archives Portal Europe ID",
        "description": "Identifier for an archival institution in the directory of Archives Portal Europe",
        "data_type": "ExternalId"
    },
    "P7739": {
        "label": "Garaph game ID",
        "description": "identifier of a video game in the garaph.info database",
        "data_type": "ExternalId"
    },
    "P7762": {
        "label": "VBProfiles person ID",
        "description": "ID of someone in VBProfiles",
        "data_type": "ExternalId"
    },
    "P7851": {
        "label": "Identifier for a resource held by the Smithsonian Institution",
        "description": "identifier for a resource held by the Smithsonian Institution",
        "data_type": "ExternalId"
    },
    "P7705": {
        "label": "superfamicom.org URL",
        "description": "url of a video game in the superfamicom.org database",
        "data_type": "Url"
    },
    "P7871": {
        "label": "PCBdB game ID",
        "description": "identifier of a video game in the Arcade PCB database",
        "data_type": "ExternalId"
    },
    "P7695": {
        "label": "e-WV: The West Virginia Encyclopedia ID",
        "description": "identifier for an entry on the West Virginia Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P7697": {
        "label": "Maine: An Encyclopedia ID",
        "description": "identifier for an entry on the Maine: An Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P7824": {
        "label": "CPCWiki ID",
        "description": "identifier in the CPCWiki",
        "data_type": "ExternalId"
    },
    "P7829": {
        "label": "English Vikidia ID",
        "description": "ID of the article in English Vikidia",
        "data_type": "ExternalId"
    },
    "P7807": {
        "label": "ICD-11 (foundation)",
        "description": "identifier in ICD terminology of diseases - Version 11 (foundation ID) (not Mortality and Morbidity Statistics ID)",
        "data_type": "ExternalId"
    },
    "P7815": {
        "label": "textove.com artist ID",
        "description": "artist ID at Bulgarian website textove.com",
        "data_type": "ExternalId"
    },
    "P2684": {
        "label": "Kijkwijzer rating",
        "description": "Dutch media content rating system",
        "data_type": "WikibaseItem"
    },
    "P8733": {
        "label": "number of rooms",
        "description": "number of rooms in a building",
        "data_type": "Quantity"
    },
    "P2756": {
        "label": "EIRIN film rating",
        "description": "category in the Japanese film rating system (add EIRIN number with qualifier P2676)",
        "data_type": "WikibaseItem"
    },
    "P4080": {
        "label": "number of houses",
        "description": "number of houses in given territorial entity",
        "data_type": "Quantity"
    },
    "P6116": {
        "label": "chord progression",
        "description": "series of chords used in this musical work",
        "data_type": "WikibaseItem"
    },
    "P1398": {
        "label": "structure replaces",
        "description": "the item this building or structure replaced, at the same geographic location",
        "data_type": "WikibaseItem"
    },
    "P5707": {
        "label": "samples from work",
        "description": "audio or video which this work uses an excerpt of",
        "data_type": "WikibaseItem"
    },
    "P5970": {
        "label": "Medierådet rating",
        "description": "Rating of a film in the Danish movie content rating system",
        "data_type": "WikibaseItem"
    },
    "P2747": {
        "label": "Filmiroda rating",
        "description": "rating of a film in the former (2004–2011) Hungarian film rating system",
        "data_type": "WikibaseItem"
    },
    "P6718": {
        "label": "music video",
        "description": "short film integrating this song with imagery",
        "data_type": "WikibaseItem"
    },
    "P3082": {
        "label": "destroyed",
        "description": "physical items destroyed by this event",
        "data_type": "WikibaseItem"
    },
    "P364": {
        "label": "original language of film or TV show",
        "description": "language in which a film or a performance work was originally created. Deprecated for written works and songs; use P407 (\"language of work or name\") instead.",
        "data_type": "WikibaseItem"
    },
    "P4437": {
        "label": "FPB rating",
        "description": "rating of a film or video game in the South African media content rating system",
        "data_type": "WikibaseItem"
    },
    "P8032": {
        "label": "victim",
        "description": "confirmed victim(s) of this crime, attack, accident or disaster",
        "data_type": "WikibaseItem"
    },
    "P483": {
        "label": "recorded at studio or venue",
        "description": "studio or location where a musical composition was recorded",
        "data_type": "WikibaseItem"
    },
    "P3402": {
        "label": "CNC film rating (Romania)",
        "description": "rating of a movie in the Romanian content rating system",
        "data_type": "WikibaseItem"
    },
    "P3081": {
        "label": "damaged",
        "description": "physical items damaged by this event",
        "data_type": "WikibaseItem"
    },
    "P8004": {
        "label": "ordered by",
        "description": "subject that ordered the act",
        "data_type": "WikibaseItem"
    },
    "P1725": {
        "label": "beats per minute",
        "description": "tempo of a musical work, indicated numerically (use P1558 for qualitative tempo markings)",
        "data_type": "Quantity"
    },
    "P826": {
        "label": "tonality",
        "description": "key of a musical composition",
        "data_type": "WikibaseItem"
    },
    "P3428": {
        "label": "INCAA film rating",
        "description": "rating of a movie in the Argentinian content rating system",
        "data_type": "WikibaseItem"
    },
    "P92": {
        "label": "main regulatory text",
        "description": "text setting the main rules by which the subject is regulated",
        "data_type": "WikibaseItem"
    },
    "P3834": {
        "label": "RTC film rating",
        "description": "rating of a film in the Mexican film classification system",
        "data_type": "WikibaseItem"
    },
    "P1657": {
        "label": "MPAA film rating",
        "description": "US film classification administered by the Motion Picture Association of America",
        "data_type": "WikibaseItem"
    },
    "P1139": {
        "label": "floors below ground",
        "description": "total number of below ground floors of the building",
        "data_type": "Quantity"
    },
    "P2550": {
        "label": "recording or performance of",
        "description": "item is a recording or release of this composition",
        "data_type": "WikibaseItem"
    },
    "P344": {
        "label": "director of photography",
        "description": "person responsible for the framing, lighting, and filtration of the subject work",
        "data_type": "WikibaseItem"
    },
    "P162": {
        "label": "producer",
        "description": "person(s) who produced the film, musical work, theatrical production, etc. (for film, this does not include executive producers, associate producers, etc.) [for production company, use P272, video games - use P178]",
        "data_type": "WikibaseItem"
    },
    "P167": {
        "label": "structure replaced by",
        "description": "the item which replaced this building or structure, at the same geographic location",
        "data_type": "WikibaseItem"
    },
    "P1981": {
        "label": "FSK film rating",
        "description": "permitted audience of a film, according to the German film-rating system (add Prüfnummer with qualifier \"P2676\")",
        "data_type": "WikibaseItem"
    },
    "P915": {
        "label": "filming location",
        "description": "actual place where this scene/film was shot. For the setting, use \"narrative location\" (P840)",
        "data_type": "WikibaseItem"
    },
    "P3157": {
        "label": "event distance",
        "description": "distance over which a race or other event is conducted or was achieved",
        "data_type": "Quantity"
    },
    "P1101": {
        "label": "floors above ground",
        "description": "above ground floor count of a building (ground floor and attic included)",
        "data_type": "Quantity"
    },
    "P7979": {
        "label": "ACM conference ID",
        "description": "identifier for conferences hosted by ACM",
        "data_type": "ExternalId"
    },
    "P2629": {
        "label": "BBFC rating",
        "description": "British media content rating (add BBFC reference with qualifier \"P2676\")",
        "data_type": "WikibaseItem"
    },
    "P209": {
        "label": "highest judicial authority",
        "description": "supreme judicial body within a country, administrative division, or other organization",
        "data_type": "WikibaseItem"
    },
    "P870": {
        "label": "instrumentation",
        "description": "combination of musical instruments employed in a composition or accompanying a (folk) dance",
        "data_type": "WikibaseItem"
    },
    "P942": {
        "label": "theme music",
        "description": "the theme music/song used by the item",
        "data_type": "WikibaseItem"
    },
    "P193": {
        "label": "main building contractor",
        "description": "the main organization responsible for construction of this structure or building",
        "data_type": "WikibaseItem"
    },
    "P1431": {
        "label": "executive producer",
        "description": "executive producer of a movie or TV show",
        "data_type": "WikibaseItem"
    },
    "P1558": {
        "label": "tempo marking",
        "description": "qualitative indication about the speed or pace of a given musical piece (use P1725 for numerical tempo markings)",
        "data_type": "WikibaseItem"
    },
    "P3886": {
        "label": "number of perpetrators",
        "description": "number of perpetrators in an event, as reported by reliable sources",
        "data_type": "Quantity"
    },
    "P7778": {
        "label": "Museum of Modern Art online exhibition ID",
        "description": "online exhibition identifier for a Museum of Modern Art or MoMA PS1 exhibition",
        "data_type": "ExternalId"
    },
    "P87": {
        "label": "librettist",
        "description": "author of the libretto (words) of an opera, operetta, oratorio or cantata, or of the book of a musical",
        "data_type": "WikibaseItem"
    },
    "P2637": {
        "label": "RARS rating",
        "description": "rating in the russian media content rating system",
        "data_type": "WikibaseItem"
    },
    "P5072": {
        "label": "presented in",
        "description": "event at which a talk, film, paper, or poster was presented",
        "data_type": "WikibaseItem"
    },
    "P86": {
        "label": "composer",
        "description": "person(s) who wrote the music [for lyricist, use \"lyrics by\" (P676)]",
        "data_type": "WikibaseItem"
    },
    "P4220": {
        "label": "order of battle",
        "description": "arrangement of units and hierarchical organization of the armed forces involved in the specified military action",
        "data_type": "WikibaseItem"
    },
    "P3158": {
        "label": "enclosure",
        "description": "the type of structure enclosing another structure",
        "data_type": "WikibaseItem"
    },
    "P3816": {
        "label": "film script",
        "description": "script version for subject film is described at",
        "data_type": "WikibaseItem"
    },
    "P9260": {
        "label": "music transcriber",
        "description": "person who transcribed a musical work into musical notation; for composers of music, use P86",
        "data_type": "WikibaseItem"
    },
    "P6658": {
        "label": "RCQ classification",
        "description": "movie classification for the Régie du Cinéma Québécois",
        "data_type": "WikibaseItem"
    },
    "P8031": {
        "label": "perpetrator",
        "description": "person or organization who carried out a harmful, illegal, or immoral act",
        "data_type": "WikibaseItem"
    },
    "P6657": {
        "label": "CHVRS Classification",
        "description": "Canadian movie classification",
        "data_type": "WikibaseItem"
    },
    "P1625": {
        "label": "has melody",
        "description": "this work has the melody of the following work",
        "data_type": "WikibaseItem"
    },
    "P736": {
        "label": "cover art by",
        "description": "name of person or team creating cover artwork for book, record album, single record etc.",
        "data_type": "WikibaseItem"
    },
    "P467": {
        "label": "legislated by",
        "description": "indicates that an act or bill was passed by a legislature.  The value can be a particular session of the legislature",
        "data_type": "WikibaseItem"
    },
    "P3440": {
        "label": "time signature",
        "description": "specification of beats in a musical bar or measure",
        "data_type": "WikibaseItem"
    },
    "P6846": {
        "label": "abART exhibition ID",
        "description": "identifier of exhibition in czech database abART",
        "data_type": "ExternalId"
    },
    "P457": {
        "label": "foundational text",
        "description": "text through which an institution or object has been created or established",
        "data_type": "WikibaseItem"
    },
    "P631": {
        "label": "structural engineer",
        "description": "person, group or organisation responsible for the structural engineering of a building or structure",
        "data_type": "WikibaseItem"
    },
    "P2291": {
        "label": "charted in",
        "description": "chart where the element reached a position",
        "data_type": "WikibaseItem"
    },
    "P1352": {
        "label": "ranking",
        "description": "subject's numbered position within a competition or group of performers",
        "data_type": "Quantity"
    },
    "P2758": {
        "label": "CNC film rating (France)",
        "description": "category assigned to a film by the Board of Film Classification (CNC) which what audiences may view it in France",
        "data_type": "WikibaseItem"
    },
    "P2208": {
        "label": "average shot length",
        "description": "cinemetrical measure",
        "data_type": "Quantity"
    },
    "P85": {
        "label": "anthem",
        "description": "subject's official anthem",
        "data_type": "WikibaseItem"
    },
    "P3650": {
        "label": "JMK film rating",
        "description": "rating of a film in the Austrian film rating system",
        "data_type": "WikibaseItem"
    },
    "P466": {
        "label": "occupant",
        "description": "a person or organization occupying property",
        "data_type": "WikibaseItem"
    },
    "P264": {
        "label": "record label",
        "description": "brand and trademark associated with the marketing of subject music recordings and music videos",
        "data_type": "WikibaseItem"
    },
    "P3030": {
        "label": "sheet music",
        "description": "media file containing the musical score (sheet music) for this item",
        "data_type": "CommonsMedia"
    },
    "P3818": {
        "label": "KMRB film rating",
        "description": "rating of a film in the South Korean film/video rating system",
        "data_type": "WikibaseItem"
    },
    "P3803": {
        "label": "original film format",
        "description": "film format used to create a work (aliases: film gauge, film size)",
        "data_type": "WikibaseItem"
    },
    "P476": {
        "label": "CELEX number",
        "description": "identifier for European legal texts in EUR-Lex database",
        "data_type": "ExternalId"
    },
    "P676": {
        "label": "lyrics by",
        "description": "author of song lyrics",
        "data_type": "WikibaseItem"
    },
    "P2363": {
        "label": "NMHH film rating",
        "description": "rating of a film in the Hungarian film rating system",
        "data_type": "WikibaseItem"
    },
    "P358": {
        "label": "discography",
        "description": "item for list pages with discography of artist or band",
        "data_type": "WikibaseItem"
    },
    "P6439": {
        "label": "has lyrics",
        "description": "work whose text is used as the lyrics of this work",
        "data_type": "WikibaseItem"
    },
    "P1301": {
        "label": "number of elevators",
        "description": "number of elevators in a building",
        "data_type": "Quantity"
    },
    "P2279": {
        "label": "ambitus",
        "description": "musical interval/range of a melody",
        "data_type": "WikibaseItem"
    },
    "P658": {
        "label": "tracklist",
        "description": "audio tracks contained in this release",
        "data_type": "WikibaseItem"
    },
    "P406": {
        "label": "soundtrack release",
        "description": "music release that incorporates music directly recorded from the soundtrack of an audiovisual work",
        "data_type": "WikibaseItem"
    },
    "P7509": {
        "label": "Estonian biographical database ID",
        "description": "ID of someone in Estonian biographical database (Eesti biograafiline andmebaas ISIK)",
        "data_type": "ExternalId"
    },
    "P7293": {
        "label": "PLWABN ID",
        "description": "National Library of Poland record no. identifier. Format: \"981\", followed by 8 digits, then ending with \"05606\"",
        "data_type": "ExternalId"
    },
    "P7300": {
        "label": "MUBI person ID",
        "description": "identifier for a person at MUBI",
        "data_type": "ExternalId"
    },
    "P7303": {
        "label": "Grammy Awards artist ID",
        "description": "identifier for an artist on the Grammy Awards website",
        "data_type": "ExternalId"
    },
    "P7360": {
        "label": "UEFA coach ID",
        "description": "identifier for a coach at the UEFA",
        "data_type": "ExternalId"
    },
    "P7698": {
        "label": "Musée de la Résistance ID",
        "description": "identifier for a person on the online Musée de la Résistance",
        "data_type": "ExternalId"
    },
    "P7154": {
        "label": "Empik author ID",
        "description": "identifier for an author on Empik site",
        "data_type": "ExternalId"
    },
    "P7578": {
        "label": "DUC ID",
        "description": "identifier for a noted woman on the online version of the ''Dictionnaire universel des créatrices''",
        "data_type": "ExternalId"
    },
    "P7491": {
        "label": "Austrian Parliament 1848 - 1918 ID",
        "description": "identifier for an individual between 1848 – 1918, in the Austrian Parliament's \"Who's Who\" database",
        "data_type": "ExternalId"
    },
    "P7463": {
        "label": "EU-Football.info manager ID",
        "description": "identifier for an association football manager on the EU-Football.info website",
        "data_type": "ExternalId"
    },
    "P7676": {
        "label": "Vie publique person ID",
        "description": "identifier for a public personality on Vie publique",
        "data_type": "ExternalId"
    },
    "P7533": {
        "label": "AaRC person ID",
        "description": "identifier for a person on the AaRC site (All about Romanian Cinema)",
        "data_type": "ExternalId"
    },
    "P7343": {
        "label": "Elitefootball player ID",
        "description": "identifier for a player at the elitefootball.com",
        "data_type": "ExternalId"
    },
    "P7341": {
        "label": "sixpackfilmdata person ID",
        "description": "identifier of a person in the cinema sixpackfilmdata database",
        "data_type": "ExternalId"
    },
    "P7178": {
        "label": "Cinestaan person ID",
        "description": "external identifier for people related to Indian movies",
        "data_type": "ExternalId"
    },
    "P7641": {
        "label": "MTV Germany artist ID",
        "description": "identifier for an artist on MTV Germany site",
        "data_type": "ExternalId"
    },
    "P7232": {
        "label": "LongSwimsDB ID",
        "description": "identifier for a swimmer in the LongSwimsDB database",
        "data_type": "ExternalId"
    },
    "P7700": {
        "label": "Slovak National Library ID",
        "description": "authority id in Slovak National Library, a VIAF component (code SKMASNL)",
        "data_type": "ExternalId"
    },
    "P7257": {
        "label": "Guia dos Quadrinhos artist ID",
        "description": "identifier for a comic book artist",
        "data_type": "ExternalId"
    },
    "P7183": {
        "label": "EPW author ID",
        "description": "identifier for authors whose articles are published in Economic and Political Weekly",
        "data_type": "ExternalId"
    },
    "P7176": {
        "label": "XING profile ID",
        "description": "identifier for the social media website XING",
        "data_type": "ExternalId"
    },
    "P7466": {
        "label": "Edvard Munch's correspondance person ID",
        "description": "identity identifier for a person in correspondance with Edvard Munch",
        "data_type": "ExternalId"
    },
    "P7684": {
        "label": "Vie publique report ID",
        "description": "identifier for a public report on Vie publique",
        "data_type": "ExternalId"
    },
    "P7530": {
        "label": "Fortuna liga player ID",
        "description": "identifier for football player on site Slovak Fortuna liga",
        "data_type": "ExternalId"
    },
    "P7571": {
        "label": "Memorial Book Bundesarchiv ID",
        "description": "identifier for the Memorial Book Bundesarchiv",
        "data_type": "ExternalId"
    },
    "P7586": {
        "label": "Cinepub person ID",
        "description": "identifier for a person on the Cinepub site",
        "data_type": "ExternalId"
    },
    "P7277": {
        "label": "Pandora artist ID",
        "description": "identifier for an artist on Pandora music site",
        "data_type": "ExternalId"
    },
    "P6960": {
        "label": "IMVDb artist ID",
        "description": "artist identifier for the IMVDb music video database",
        "data_type": "ExternalId"
    },
    "P7652": {
        "label": "Germany06.Ru person ID",
        "description": "identifier for a participants of 2006 FIFA World Cup at Germany06.Ru site",
        "data_type": "ExternalId"
    },
    "P7399": {
        "label": "Futbolme player ID",
        "description": "identifier for a player at Futbolme website",
        "data_type": "ExternalId"
    },
    "P6975": {
        "label": "RBF professional boxer ID",
        "description": "identifier for an professional boxer at the Boxing Federation of Russia",
        "data_type": "ExternalId"
    },
    "P7171": {
        "label": "Hacker News username",
        "description": "unique identifier of a user account on the Hacker News social media website",
        "data_type": "ExternalId"
    },
    "P7682": {
        "label": "National Library of New Zealand ID",
        "description": "identifier for an entity in the National Library of New Zealand",
        "data_type": "ExternalId"
    },
    "P7405": {
        "label": "Turkish Football Federation referee ID",
        "description": "identifier association football referee at TFF database",
        "data_type": "ExternalId"
    },
    "P7440": {
        "label": "FIG gymnast (no licence) identifier",
        "description": "gymnast's identifier (no licence) at the database of International Federation of Gymnastics",
        "data_type": "ExternalId"
    },
    "P7265": {
        "label": "FNL player ID",
        "description": "identifier for a player at the 1fnl.ru",
        "data_type": "ExternalId"
    },
    "P6980": {
        "label": "LB.ua dossier ID",
        "description": "ID in series of dossier on the company and people by LB.ua",
        "data_type": "ExternalId"
    },
    "P7633": {
        "label": "MPPDA Digital Archive person ID",
        "description": "link to a person's records at the MPPDA Digital Archive",
        "data_type": "ExternalId"
    },
    "P7278": {
        "label": "Peakbagger climber ID",
        "description": "identifier for a climber, on the peakbagger.com website",
        "data_type": "ExternalId"
    },
    "P7709": {
        "label": "ScienceOpen author ID",
        "description": "unique identifier of an author in ScienceOpen database",
        "data_type": "ExternalId"
    },
    "P7612": {
        "label": "Taiwan Cinema person ID",
        "description": "identifier for a person on Taiwan Cinema",
        "data_type": "ExternalId"
    },
    "P7638": {
        "label": "OverDrive creator ID",
        "description": "identifier for an author or creator at OverDrive",
        "data_type": "ExternalId"
    },
    "P7608": {
        "label": "Englandstats.com Soccer ID",
        "description": "identifier for a male player on the Englandstats.com website",
        "data_type": "ExternalId"
    },
    "P7362": {
        "label": "Ukrainian Premier League  player ID",
        "description": "identifier for football player on site Ukrainian Premier League",
        "data_type": "ExternalId"
    },
    "P6891": {
        "label": "National Film Board of Canada director identifier",
        "description": "director identifier used by the National Film Board of Canada website",
        "data_type": "ExternalId"
    },
    "P7451": {
        "label": "Czech Fortuna liga player ID",
        "description": "identifier for football player on site Czech Fortuna liga",
        "data_type": "ExternalId"
    },
    "P6910": {
        "label": "CIN ID",
        "description": "property for Bosnian-Herzegovinian politicians biographies and property status given by Centar za istraživačko novinarstvo (CIN) website",
        "data_type": "ExternalId"
    },
    "P7282": {
        "label": "Slovo i Dilo person ID",
        "description": "identifier for person on Slovo i Dilo site",
        "data_type": "ExternalId"
    },
    "P6934": {
        "label": "Syriac Biographical Dictionary ID",
        "description": "identifier of a person in the Syriac Biographical Dictionary, an online research tool",
        "data_type": "ExternalId"
    },
    "P7645": {
        "label": "RomanianActors person ID",
        "description": "identifier for an actor on the RomanianActors site",
        "data_type": "ExternalId"
    },
    "P7311": {
        "label": "Aozora Bunko author ID",
        "description": "identifier of a list page of works by author on Aozora Bunko",
        "data_type": "ExternalId"
    },
    "P7653": {
        "label": "MovieGe person ID",
        "description": "identifier for a person on the movie.ge site",
        "data_type": "ExternalId"
    },
    "P7550": {
        "label": "International Paralympic Committee athlete alphabetical ID",
        "description": "alphabetical identifier for an athlete on the International Paralympic Committee website",
        "data_type": "ExternalId"
    },
    "P6905": {
        "label": "Bebo profile ID",
        "description": "identifier for an artist on Bebo site",
        "data_type": "ExternalId"
    },
    "P6907": {
        "label": "BVLarramendi ID",
        "description": "identifier for a person on the Polymath Virtual Library",
        "data_type": "ExternalId"
    },
    "P7475": {
        "label": "dp.ru person ID",
        "description": "dp.ru identifier for a person",
        "data_type": "ExternalId"
    },
    "P7467": {
        "label": "Broadway Photographs person ID",
        "description": "identifier for historical Broadway photographers and their subjects",
        "data_type": "ExternalId"
    },
    "P6940": {
        "label": "FPBR person ID",
        "description": "identifier for a person at the Russian Professional Boxing Federation",
        "data_type": "ExternalId"
    },
    "P6953": {
        "label": "2014 Commonwealth Games athlete ID",
        "description": "identifier for an athlete who participated at 2014 Commonwealth Games",
        "data_type": "ExternalId"
    },
    "P7483": {
        "label": "CinemaRX person ID",
        "description": "identifier for a person on the CinemaRX.ro website",
        "data_type": "ExternalId"
    },
    "P7624": {
        "label": "YVNG ID",
        "description": "identifier in the Yad Vashem central database of Holocaust victims' names",
        "data_type": "ExternalId"
    },
    "P7119": {
        "label": "AMPAS collections person ID",
        "description": "identifier for a person or organization represented in the Academy of Motion Picture Arts and Sciences (AMPAS) collections catalog",
        "data_type": "ExternalId"
    },
    "P6993": {
        "label": "Pro Kabaddi League player ID",
        "description": "external identifiers for players participating in Pro Kabaddi League",
        "data_type": "ExternalId"
    },
    "P7670": {
        "label": "Prog Archives artist ID",
        "description": "identifier for an artist on the website Prog Archives",
        "data_type": "ExternalId"
    },
    "P7395": {
        "label": "DynamoMania.com player ID",
        "description": "identifier for a player at the DynamoMania.com",
        "data_type": "ExternalId"
    },
    "P7142": {
        "label": "Poincaré Papers person ID",
        "description": "identifier for a correspondent of Henri Poincaré, or for a person cited on his letters",
        "data_type": "ExternalId"
    },
    "P7474": {
        "label": "Online Cenotaph ID",
        "description": "identifier in the Online Cenotaph database assigned to a military serviceperson by the Auckland War Memorial Museum",
        "data_type": "ExternalId"
    },
    "P7459": {
        "label": "CanadaSoccer.com person ID",
        "description": "identifier for a person at the CanadaSoccer.com",
        "data_type": "ExternalId"
    },
    "P6994": {
        "label": "RBF amateur boxer ID",
        "description": "identifier for an amateur boxer at the Boxing Federation of Russia",
        "data_type": "ExternalId"
    },
    "P7037": {
        "label": "COURAGE ID",
        "description": "COURAGE (“Cultural Opposition – Understanding the CultuRal HeritAGE of Dissent in the Former Socialist Countries”) is a three-year international research project funded by Horizon 2020, the EU Framework Programme for Research and Innovation",
        "data_type": "ExternalId"
    },
    "P7503": {
        "label": "Ovrtur biography ID",
        "description": "Ovrtur.com biography ID",
        "data_type": "ExternalId"
    },
    "P7359": {
        "label": "LaPreferente.com player ID",
        "description": "identifier for a player at the LaPreferente.com site",
        "data_type": "ExternalId"
    },
    "P7027": {
        "label": "MPG ID",
        "description": "identifier for an expert researcher or an institution in the database of the Max-Planck-Gesellschaft",
        "data_type": "ExternalId"
    },
    "P7654": {
        "label": "Ratingraph actress ID",
        "description": "identifier for an actress on the Ratingraph site",
        "data_type": "ExternalId"
    },
    "P6973": {
        "label": "Personnel de l'administration préfectorale ID",
        "description": "identifier for an official on the online version of ''Le Personnel de l'administration préfectorale (1881-1926)''",
        "data_type": "ExternalId"
    },
    "P7489": {
        "label": "Stathletics ID",
        "description": "identifier for stathletics.ch",
        "data_type": "ExternalId"
    },
    "P7696": {
        "label": "Français libres ID",
        "description": "identifier for a French resistance fighter on Français libres",
        "data_type": "ExternalId"
    },
    "P7032": {
        "label": "Repertorium van ambtsdragers en ambtenaren ID",
        "description": "identifier for people on a web site with high level Dutch civil servants between 1428 and 1861",
        "data_type": "ExternalId"
    },
    "P7580": {
        "label": "AboutTheArtists artist ID",
        "description": "identifier for a person on the AboutTheArtists Database",
        "data_type": "ExternalId"
    },
    "P6936": {
        "label": "Who's Who of American Comic Books ID",
        "description": "identifier for creators and publishers in the Who's Who of American Comic Books",
        "data_type": "ExternalId"
    },
    "P7545": {
        "label": "askArt person ID",
        "description": "identifier for the askArt website",
        "data_type": "ExternalId"
    },
    "P7671": {
        "label": "Semion author ID",
        "description": "identifier of an author or reviewer, in Semion",
        "data_type": "ExternalId"
    },
    "P7656": {
        "label": "Arolsen ID",
        "description": "identifier for an Holocaust victim on the Arolsen Archives' website",
        "data_type": "ExternalId"
    },
    "P7472": {
        "label": "National Marine Biological Library  ID",
        "description": "authority control identifier for the National Marine Biological Library",
        "data_type": "ExternalId"
    },
    "P7498": {
        "label": "Women Film Pioneers ID",
        "description": "identifier from the Women Film Pioneers Project",
        "data_type": "ExternalId"
    },
    "P6988": {
        "label": "Hungarian National Namespace person ID",
        "description": "identifier for a person in the Hungarian National Namespace",
        "data_type": "ExternalId"
    },
    "P7520": {
        "label": "Filmový přehled person ID",
        "description": "identifier for a person or organisation in the Filmový přehled database",
        "data_type": "ExternalId"
    },
    "P7672": {
        "label": "Musiikkituottajat artist ID (chart)",
        "description": "identifier for an artist on Musiikkituottajat music site",
        "data_type": "ExternalId"
    },
    "P7594": {
        "label": "CinePT person ID",
        "description": "identifier for a person or organisatin on the CinePT website",
        "data_type": "ExternalId"
    },
    "P7148": {
        "label": "Nauka Ukrainy scientist ID",
        "description": "identifier for scientist on Nauka Ukrainy site",
        "data_type": "ExternalId"
    },
    "P7602": {
        "label": "Ratingraph actor ID",
        "description": "identifier for a person on the Ratingraph site - actor",
        "data_type": "ExternalId"
    },
    "P3679": {
        "label": "stock market index",
        "description": "method of measuring the value of a section of the stock market",
        "data_type": "WikibaseItem"
    },
    "P6195": {
        "label": "funding scheme",
        "description": "is supported under the funding scheme",
        "data_type": "WikibaseItem"
    },
    "P6544": {
        "label": "total points in career",
        "description": "sports statistic",
        "data_type": "Quantity"
    },
    "P2746": {
        "label": "production statistics",
        "description": "amount of a certain good produced in/by the item",
        "data_type": "WikibaseItem"
    },
    "P8876": {
        "label": "number of taxpayers",
        "description": "number of persons liable for a tax",
        "data_type": "Quantity"
    },
    "P3690": {
        "label": "IFSC climber ID",
        "description": "identifier for a climber on the International Federation of Sport Climbing website",
        "data_type": "ExternalId"
    },
    "P4096": {
        "label": "RePEc institute ID",
        "description": "identifier for economics research institutions in the Research Papers in Economics (RePEc) database",
        "data_type": "ExternalId"
    },
    "P3883": {
        "label": "Red Bull athlete ID",
        "description": "identifier for a sponsored athlete on the Red Bull website",
        "data_type": "ExternalId"
    },
    "P2663": {
        "label": "common equity tier 1 capital ratio (CETI)",
        "description": "relation between the core capital or shareholder's equity of a Bank and its risk-weighted assets",
        "data_type": "Quantity"
    },
    "P2664": {
        "label": "units sold",
        "description": "sales figures of an object",
        "data_type": "Quantity"
    },
    "P8093": {
        "label": "donations",
        "description": "Sum of donations an organization receives",
        "data_type": "Quantity"
    },
    "P2769": {
        "label": "budget",
        "description": "assigned monetary amount for a project (for the estimated cost of a film, also commonly referred to as budget, use P2130)",
        "data_type": "Quantity"
    },
    "P3529": {
        "label": "median income",
        "description": "median household income in a place",
        "data_type": "Quantity"
    },
    "P6794": {
        "label": "minimum wage",
        "description": "lowest wage which can be paid legally in a state for working",
        "data_type": "Quantity"
    },
    "P6509": {
        "label": "total goals in career",
        "description": "goalscoring statistic",
        "data_type": "Quantity"
    },
    "P2403": {
        "label": "total assets",
        "description": "value of assets held by a private or public entity",
        "data_type": "Quantity"
    },
    "P8744": {
        "label": "economy of topic",
        "description": "item that deals with the economy of the subject",
        "data_type": "WikibaseItem"
    },
    "P2662": {
        "label": "consumption rate per capita",
        "description": "rate of consumption of a product divided by the population",
        "data_type": "Quantity"
    },
    "P2661": {
        "label": "target interest rate",
        "description": "benchmark interest rate of the central bank",
        "data_type": "Quantity"
    },
    "P5513": {
        "label": "Enciclopèdia de l'Esport Català ID",
        "description": "identifier on Enciclopèdia de l'Esport Català, encyclopedia of Catalan sportspeople and sports clubs",
        "data_type": "ExternalId"
    },
    "P664": {
        "label": "organizer",
        "description": "person or institution organizing an event",
        "data_type": "WikibaseItem"
    },
    "P3087": {
        "label": "fiscal/tax revenue",
        "description": "fiscal revenue of a public entity (not for private entities)",
        "data_type": "Quantity"
    },
    "P4103": {
        "label": "assets under management",
        "description": "total market value of financial assets which a financial institution manages on behalf of its clients and itself",
        "data_type": "Quantity"
    },
    "P3618": {
        "label": "base salary",
        "description": "gross salary of a position or a person (not including bonuses or other forms of remuneration)",
        "data_type": "Quantity"
    },
    "P3494": {
        "label": "points classification",
        "data_type": "WikibaseItem"
    },
    "P8369": {
        "label": "Econlib entry ID",
        "description": "identifier for an entry in encyclopedia of the Library of Economics and Liberty by Liberty Fund, Inc., a private, educational foundation",
        "data_type": "ExternalId"
    },
    "P5995": {
        "label": "kit supplier",
        "description": "official supplier of sports goods to a given club or a national sports team",
        "data_type": "WikibaseItem"
    },
    "P9396": {
        "label": "trade union membership rate",
        "description": "rate of members of trade unions in proportion to the total number of workers",
        "data_type": "Quantity"
    },
    "P2297": {
        "label": "employment by economic sector",
        "description": "employment divided by sector",
        "data_type": "Quantity"
    },
    "P2416": {
        "label": "sports discipline competed in",
        "description": "discipline an athlete competed in within a sport",
        "data_type": "WikibaseItem"
    },
    "P2417": {
        "label": "stage classification",
        "description": "stage results and ranking of a stage race",
        "data_type": "WikibaseItem"
    },
    "P4909": {
        "label": "number of players in region",
        "description": "number of registered players in a sport or people practicing a sport in a region",
        "data_type": "Quantity"
    },
    "P4010": {
        "label": "GDP (PPP)",
        "description": "gross domestic product (GDP) at purchasing power parity (current international $)",
        "data_type": "Quantity"
    },
    "P2855": {
        "label": "VAT-rate",
        "description": "percentage value-added tax in this country or region. Do not use for sales tax. For specialised rates, use qualifiers",
        "data_type": "Quantity"
    },
    "P641": {
        "label": "sport",
        "description": "sport that the subject participates or participated in or is associated with",
        "data_type": "WikibaseItem"
    },
    "P2630": {
        "label": "cost of damage",
        "description": "monetary value of damage caused by this event",
        "data_type": "Quantity"
    },
    "P3496": {
        "label": "teams classification by points",
        "description": "teams classification by points",
        "data_type": "WikibaseItem"
    },
    "P8324": {
        "label": "funder",
        "description": "entity that gives money to a person, organization, or project for a specific purpose",
        "data_type": "WikibaseItem"
    },
    "P6543": {
        "label": "total shots in career",
        "description": "sports statistic",
        "data_type": "Quantity"
    },
    "P3911": {
        "label": "STW Thesaurus for Economics ID",
        "description": "identifier for a descriptor of the STW Thesaurus for Economics, linking to literature on economics and business administration",
        "data_type": "ExternalId"
    },
    "P8476": {
        "label": "BTI Governance Index",
        "description": "measures change of countries towards democracy and market economy",
        "data_type": "Quantity"
    },
    "P8477": {
        "label": "BTI Status Index",
        "description": "measures change of countries towards democracy and market economy",
        "data_type": "Quantity"
    },
    "P2402": {
        "label": "total expenditure",
        "description": "amount of spending by this public or private entity, not to be confused with fiscal expenditure",
        "data_type": "Quantity"
    },
    "P8957": {
        "label": "GS1 GPC code",
        "description": "code used to classify products in the GS1 Global Product Classification: Segment>Family>Class>Brick, or Attribute Type: Value",
        "data_type": "ExternalId"
    },
    "P3279": {
        "label": "statistical leader",
        "description": "leader of a sports tournament in one of statistical qualities (points, assists, rebounds etc.). Don't use for overall winner. Use a qualifier to link to the item about the quality.",
        "data_type": "WikibaseItem"
    },
    "P9274": {
        "label": "subsidy",
        "description": "fiscal tool a government uses to encourage economic development, help disadvantaged groups, or advance other national objectives",
        "data_type": "Quantity"
    },
    "P9087": {
        "label": "Econlib person ID",
        "description": "identifier for an entry about a person in encyclopedia of the Library of Economics and Liberty by Liberty Fund, Inc., a private, educational right-leaning foundation",
        "data_type": "ExternalId"
    },
    "P8368": {
        "label": "number of branches",
        "description": "number of branches or stores affiliated with a chain, brand or company",
        "data_type": "Quantity"
    },
    "P8491": {
        "label": "AHECC 2017 ID",
        "description": "identifier in the Australian Harmonized Export Commodity Classification 2017",
        "data_type": "ExternalId"
    },
    "P4999": {
        "label": "number of clubs",
        "description": "number of clubs active in a given sport for a region",
        "data_type": "Quantity"
    },
    "P2555": {
        "label": "fee",
        "description": "fee or toll payable to use, transit or enter the subject (only for one-time fees, do NOT use it for an ongoing fee, tuition fee or trading fee)",
        "data_type": "Quantity"
    },
    "P3934": {
        "label": "face value",
        "description": "value printed on a coin, stamp or banknote by the issuing authority",
        "data_type": "Quantity"
    },
    "P2299": {
        "label": "PPP GDP per capita",
        "description": "GDP divided by the population total and adjusted for CPI",
        "data_type": "Quantity"
    },
    "P2761": {
        "label": "Research Papers in Economics Series handle",
        "description": "identifier of academic journals in Research Papers in Economics (RePEc)",
        "data_type": "ExternalId"
    },
    "P6545": {
        "label": "total assists in career",
        "description": "goalscoring statistic",
        "data_type": "Quantity"
    },
    "P6546": {
        "label": "penalty minutes in career",
        "description": "sports statistic",
        "data_type": "Quantity"
    },
    "P8984": {
        "label": "CPV Supplementary",
        "description": "supplementary code for Common Procurement Vocabulary (CPV, P5417) expanding the description of the subject of a contract by adding further details regarding the nature or destination of the object of the contract in the EU",
        "data_type": "ExternalId"
    },
    "P8843": {
        "label": "poverty incidence",
        "description": "proportion of households with per capita income/expenditure less than the per capita poverty threshold to the total number of households",
        "data_type": "Quantity"
    },
    "P6826": {
        "label": "feed-in tariff",
        "description": "monetary value in a given currency, where a renewable power station gets paid for the amount of power they generate",
        "data_type": "Quantity"
    },
    "P2321": {
        "label": "general classification of race participants",
        "description": "classification of race participants",
        "data_type": "WikibaseItem"
    },
    "P6639": {
        "label": "capacity factor",
        "description": "percentage of maximum power output provided on average by a given power station or power generation unit",
        "data_type": "Quantity"
    },
    "P5065": {
        "label": "wind speed",
        "description": "wind flow velocity recorded during a given moment or period of time",
        "data_type": "Quantity"
    },
    "P3744": {
        "label": "number of subscribers",
        "description": "number of subscribers for subscription based companies, e.g. telecommunication companies, newspapers, pay-TV channels, etc.",
        "data_type": "Quantity"
    },
    "P2770": {
        "label": "source of income",
        "description": "source of income of an organization or person",
        "data_type": "WikibaseItem"
    },
    "P2771": {
        "label": "D-U-N-S number",
        "description": "unique company identifier from Dun & Bradstreet",
        "data_type": "ExternalId"
    },
    "P3742": {
        "label": "FIH player ID",
        "description": "identifier for a field hockey player on the International Hockey Federation website",
        "data_type": "ExternalId"
    },
    "P8461": {
        "label": "energy consumption per transaction",
        "description": "energy footprint per transaction for a cryptocurrency",
        "data_type": "Quantity"
    },
    "P7408": {
        "label": "EconBiz publication ID",
        "description": "ID of a journal, book or article in EconBiz",
        "data_type": "ExternalId"
    },
    "P2834": {
        "label": "individual tax rate",
        "description": "percentage tax rate on individuals by income",
        "data_type": "Quantity"
    },
    "P3822": {
        "label": "rules for classification",
        "description": "the rules for classification of a sports league",
        "data_type": "WikibaseItem"
    },
    "P3450": {
        "label": "sports season of league or competition",
        "description": "property that shows the competition of which the item is a season. Use P5138 for \"season of club or team\".",
        "data_type": "WikibaseItem"
    },
    "P4131": {
        "label": "annual energy output",
        "description": "annual energy output of a power plant",
        "data_type": "Quantity"
    },
    "P2869": {
        "label": "record or record progression",
        "description": "links to item on the record or record progression",
        "data_type": "WikibaseItem"
    },
    "P2955": {
        "label": "point of penalty",
        "description": "point of penalty in a tournament",
        "data_type": "Quantity"
    },
    "P3497": {
        "label": "teams classification by time",
        "description": "teams classification by time",
        "data_type": "WikibaseItem"
    },
    "P8247": {
        "label": "nominal share capital",
        "description": "total of the nominal values of the issued ordinary and preference shares of the entity",
        "data_type": "Quantity"
    },
    "P3260": {
        "label": "points awarded",
        "description": "points awarded to the winning person or team for a win, draw, tie or loss. If no points are awarded for some category of finish (e.g. loss) use value 0 not ''no value''.",
        "data_type": "Quantity"
    },
    "P2296": {
        "label": "money supply",
        "description": "amount of currency the central bank has injected in the economy (MO, M1, M2, M3)",
        "data_type": "Quantity"
    },
    "P8340": {
        "label": "estimated value",
        "description": "estimated value of a house, company, patent, etc",
        "data_type": "Quantity"
    },
    "P6400": {
        "label": "Skimap area ID",
        "description": "Record page on www.Skimap.org for ski resort, with current and historical official published maps of lifts and ski-runs",
        "data_type": "ExternalId"
    },
    "P6140": {
        "label": "DanNet 2.2 word ID",
        "description": "identifier for part of the URI for a word in the DanNet resource",
        "data_type": "ExternalId"
    },
    "P6412": {
        "label": "Gran Enciclopèdia de la Música ID",
        "description": "identifier for an item in the Gran Enciclopèdia de la Música",
        "data_type": "ExternalId"
    },
    "P6298": {
        "label": "bgm.tv person identifier",
        "description": "identifier for a person or group of people on bgm.tv (Bangumi/Chobits)",
        "data_type": "ExternalId"
    },
    "P6091": {
        "label": "MGG Online ID",
        "description": "stable identifier for an entry on the MGG Online website",
        "data_type": "ExternalId"
    },
    "P6456": {
        "label": "Bilibili video ID",
        "description": "identifier of a video on Bilibili",
        "data_type": "ExternalId"
    },
    "P6431": {
        "label": "Hymnary tune ID",
        "description": "identifier for hymn tune in the Hymnary.org database",
        "data_type": "ExternalId"
    },
    "P6451": {
        "label": "Zhihu username",
        "description": "domain name of Zhihu users' personal pages",
        "data_type": "ExternalId"
    },
    "P6454": {
        "label": "Bilibili tag ID",
        "description": "number of a tag of Bilibili, a Chinese video website similar to YouTube or Niconico",
        "data_type": "ExternalId"
    },
    "P6160": {
        "label": "BoardGameGeek game publisher ID",
        "description": "identifier for a publisher on the BoardGameGeek website",
        "data_type": "ExternalId"
    },
    "P6162": {
        "label": "Luding game publisher ID",
        "description": "identifier for a publisher on the Luding website",
        "data_type": "ExternalId"
    },
    "P6187": {
        "label": "Lego set ID",
        "description": "unique identifier of a Lego set",
        "data_type": "ExternalId"
    },
    "P6449": {
        "label": "Douban site name",
        "description": "domain name of Douban site written in Latin alphabet",
        "data_type": "ExternalId"
    },
    "P6450": {
        "label": "Douban username",
        "description": "domain name of Douban users' personal pages",
        "data_type": "ExternalId"
    },
    "P6429": {
        "label": "Hymnary hymnal ID",
        "description": "identifier for hymnal in the Hymnary.org database",
        "data_type": "ExternalId"
    },
    "P6423": {
        "label": "MusicBrainz event ID",
        "description": "identifier for an event in the MusicBrainz open music encyclopedia",
        "data_type": "ExternalId"
    },
    "P6165": {
        "label": "Corpus typographique français ID",
        "description": "identifier for a typeface on the Corpus typographique français website",
        "data_type": "ExternalId"
    },
    "P6170": {
        "label": "System16 identifier",
        "description": "identifier in the System16 database of arcade game systems",
        "data_type": "ExternalId"
    },
    "P6138": {
        "label": "Software Heritage identifier (SWHID)",
        "description": "persistent identifier of a software source code artifact as archived in the Software Heritage Archive",
        "data_type": "ExternalId"
    },
    "P6348": {
        "label": "Hymnary text ID",
        "description": "identifier for hymn text in the Hymnary.org database",
        "data_type": "ExternalId"
    },
    "P6262": {
        "label": "Fandom article ID",
        "description": "identifier of an article on Fandom (Wikia). Format: subdomain:Page_title",
        "data_type": "ExternalId"
    },
    "P6245": {
        "label": "PomBase systematic ID",
        "description": "Identifier for a coding gene in the Schizosaccharomyces pombe model organism database PomBase",
        "data_type": "ExternalId"
    },
    "P6443": {
        "label": "Douban drama ID",
        "description": "identifier for a theatrical production at the website Douban",
        "data_type": "ExternalId"
    },
    "P6442": {
        "label": "Douban book ID",
        "description": "identifier for a book at the website Douban",
        "data_type": "ExternalId"
    },
    "P6202": {
        "label": "Geolex ID",
        "description": "ID for every stratigraphic unit in the United States of America (USA) given by the United States Geological Survey",
        "data_type": "ExternalId"
    },
    "P6199": {
        "label": "member of the Portuguese parliament ID",
        "description": "id in the Portuguese Parliament Database",
        "data_type": "ExternalId"
    },
    "P6266": {
        "label": "Garzanti Linguistica ID",
        "description": "ID for lexemes",
        "data_type": "ExternalId"
    },
    "P6181": {
        "label": "Disney A to Z ID",
        "description": "ID of article in online version of Disney A to Z",
        "data_type": "ExternalId"
    },
    "P6395": {
        "label": "Apple Books book ID",
        "description": "identifier for an e-book in the iTunes Store",
        "data_type": "ExternalId"
    },
    "P6210": {
        "label": "LIGA profile",
        "description": "background information on organizations, political parties, biographies of famous people",
        "data_type": "ExternalId"
    },
    "P6205": {
        "label": "Defined Term ID",
        "description": "identifier for a definition in definedterm.com dictionary",
        "data_type": "ExternalId"
    },
    "P6197": {
        "label": "Badgames ID",
        "description": "identifier of a video game from the website Badgames.it",
        "data_type": "ExternalId"
    },
    "P6220": {
        "label": "OpenTrials ID",
        "description": "identifier for OpenTrials database over clinical trials",
        "data_type": "ExternalId"
    },
    "P6222": {
        "label": "DIR3 ID",
        "description": "official identifier of a public Spanish organization",
        "data_type": "ExternalId"
    },
    "P6287": {
        "label": "ICCD ID - CF form",
        "description": "identifier for the Central Institute for Catalogues and Documentation (form type CF)",
        "data_type": "ExternalId"
    },
    "P6286": {
        "label": "ICCD ID - S form",
        "description": "identifier for the Central Institute for Catalogues and Documentation (form type S)",
        "data_type": "ExternalId"
    },
    "P6288": {
        "label": "ICCD ID - CG form",
        "description": "identifier for the Central Institute for Catalogues and Documentation (form type CG)",
        "data_type": "ExternalId"
    },
    "P6278": {
        "label": "Epic Games Store ID",
        "description": "identifier for a video game available on the Epic Games Store",
        "data_type": "ExternalId"
    },
    "P6281": {
        "label": "IPPDH ID",
        "description": "identifier for archives and funds in the Institute of Public Policies in Human Rights (IPPDH) of Mercosur",
        "data_type": "ExternalId"
    },
    "P6448": {
        "label": "Douban Read eBook ID",
        "description": "identifier for a book at the website Douban Read",
        "data_type": "ExternalId"
    },
    "P6430": {
        "label": "Hymnary instance ID",
        "description": "identifier for an individual edition of a published hymn in the Hymnary.org database",
        "data_type": "ExternalId"
    },
    "P6457": {
        "label": "Eventa Servo ID",
        "description": "unique ID for the event on the website Eventa Servo",
        "data_type": "ExternalId"
    },
    "P6263": {
        "label": "MinDat mineral ID",
        "description": "identifier for a mineral or related material (rock, mixture) in the MinDat database",
        "data_type": "ExternalId"
    },
    "P6247": {
        "label": "Lego design ID",
        "description": "unique identifier of a Lego brick design",
        "data_type": "ExternalId"
    },
    "P6445": {
        "label": "Douban music ID",
        "description": "identifier for an album at the website Douban",
        "data_type": "ExternalId"
    },
    "P6032": {
        "label": "IGF entrant ID",
        "description": "identifier of an entrant in the Independent Games Festival",
        "data_type": "ExternalId"
    },
    "P6299": {
        "label": "PC-9801 DB ID",
        "description": "identifier in the PC-9801 DB database of PC-98 video games",
        "data_type": "ExternalId"
    },
    "P6301": {
        "label": "The Armory Show at 100 ID",
        "description": "identifier for an artwork on the Armory Show at 100 website",
        "data_type": "ExternalId"
    },
    "P6229": {
        "label": "Discord Store game SKU",
        "description": "identifier for a game available on the Discord Store",
        "data_type": "ExternalId"
    },
    "P6242": {
        "label": "Meteorological Service of Canada climate site ID",
        "description": "7 digit number assigned by the Meteorological Service of Canada to a site where official weather observations are taken, and serves as a permanent, unique identifier",
        "data_type": "ExternalId"
    },
    "P6248": {
        "label": "Lego element ID",
        "description": "unique identifier of a Lego brick design with a specific colour",
        "data_type": "ExternalId"
    },
    "P6326": {
        "label": "DVR Number",
        "description": "identifier for data processing Austrian organisations",
        "data_type": "ExternalId"
    },
    "P6164": {
        "label": "Depósito Legal ID",
        "description": "identifier for a publication in Spain",
        "data_type": "ExternalId"
    },
    "P6058": {
        "label": "Larousse ID",
        "description": "identifier for a subject on the 'Encyclopédie Larousse'",
        "data_type": "ExternalId"
    },
    "P6337": {
        "label": "PCGamingWiki ID",
        "description": "identifier for PC video games tracked on PCGamingWiki",
        "data_type": "ExternalId"
    },
    "P6444": {
        "label": "Douban game ID",
        "description": "identifier for a video game at the website Douban",
        "data_type": "ExternalId"
    },
    "P6267": {
        "label": "Multiplayer ID",
        "description": "identifier of a video game from the website Multiplayer.it",
        "data_type": "ExternalId"
    },
    "P6360": {
        "label": "Genius album numeric ID",
        "description": "numeric ID for an album, single, other music release or other publication on Genius",
        "data_type": "ExternalId"
    },
    "P6363": {
        "label": "WordLift URL",
        "description": "a Linked Open Data permanent ID of publicly available entities curated by online publishers and distributed in various formats",
        "data_type": "Url"
    },
    "P6277": {
        "label": "Objectif Gard topic ID",
        "description": "identifier for a topic on ''Objectif Gard'''s website",
        "data_type": "ExternalId"
    },
    "P6078": {
        "label": "GameFAQs platform ID",
        "description": "identifier of a gaming platform in GameFAQs database",
        "data_type": "ExternalId"
    },
    "P6279": {
        "label": "ZVR-Number",
        "description": "ID of registered associations in Austria",
        "data_type": "ExternalId"
    },
    "P6389": {
        "label": "Skiresort.info ID",
        "description": "Record on skiresort.info for any specific ski resort",
        "data_type": "ExternalId"
    },
    "P6391": {
        "label": "U-DISE code",
        "description": "identifier for a primary or secondary school in India issued under the Unified District Information System for Education",
        "data_type": "ExternalId"
    },
    "P6068": {
        "label": "DCMOTO identifier",
        "description": "identifier in the DCMOTO database of Thomson MO/TO software",
        "data_type": "ExternalId"
    },
    "P6292": {
        "label": "Complete BBC Micro Games Archive ID",
        "description": "identifier in the Complete BBC Games Archive (bbcmicro.co.uk)",
        "data_type": "ExternalId"
    },
    "P6303": {
        "label": "Swedish Anbytarforum",
        "description": "Anbytarforum is a discussion group to ask advanced genealogy questions regarding Swedish parishes and people in the parish, help reading",
        "data_type": "ExternalId"
    },
    "P6307": {
        "label": "Wikibase Registry ID",
        "description": "Q-ID in the Wikibase Registry",
        "data_type": "ExternalId"
    },
    "P6352": {
        "label": "OLAC video game genre vocabulary ID",
        "description": "identifier of video game genre vocabulary of the OLAC (Online Audiovisual Catalogers)",
        "data_type": "ExternalId"
    },
    "P6090": {
        "label": "Caselaw Access Project case ID",
        "description": "identifier for single case in Caselaw Access Project",
        "data_type": "ExternalId"
    },
    "P6283": {
        "label": "MNopedia ID",
        "description": "identifier for a subject in the online encyclopedia MNopedia, which covers Minnesota",
        "data_type": "ExternalId"
    },
    "P6392": {
        "label": "AISHE code",
        "description": "identifier for a college, university, or standalone institution in India issued by the All India Survey on Higher Education",
        "data_type": "ExternalId"
    },
    "P6135": {
        "label": "VD 18 ID",
        "description": "publication ID in the bibliography of 18th century printed material in the German speaking areas",
        "data_type": "ExternalId"
    },
    "P6377": {
        "label": "VectorBase taxonomy ID",
        "description": "identifier for a taxon in the NIAID Bioinformatics Resource Center VectorBase",
        "data_type": "ExternalId"
    },
    "P6324": {
        "label": "INE ID (Portugal)",
        "description": "identifier for Portuguese municipalities, districts and parishes, by the Portuguese National Statistical Institute (INE)",
        "data_type": "ExternalId"
    },
    "P6419": {
        "label": "GEOFON earthquake ID",
        "description": "ID of an earthquake in GEOFON earthquake ID",
        "data_type": "ExternalId"
    },
    "P6404": {
        "label": "Treccani's Dizionario di Storia ID",
        "description": "identifier for ''Dizionario di Storia'' (''Dictionary of History'') by Treccani",
        "data_type": "ExternalId"
    },
    "P6306": {
        "label": "AllMusic performance ID",
        "description": "identifier for a recording of a performance in AllMusic",
        "data_type": "ExternalId"
    },
    "P6284": {
        "label": "NCpedia ID",
        "description": "identifier for a subject in the online encyclopedia NCpedia, that covers North Carolina",
        "data_type": "ExternalId"
    },
    "P6305": {
        "label": "Wikimedia OTRS ticket number",
        "description": "identifier for a set of email correspondence, in the Wikimedia volunteer response system",
        "data_type": "ExternalId"
    },
    "P6418": {
        "label": "funk channel ID",
        "description": "ID of the funk channel. Used only for formats with a channel on the funk.net website.",
        "data_type": "ExternalId"
    },
    "P6422": {
        "label": "ManualsLib brand ID",
        "description": "identifier for a brand or manufacturer, in the Manuals Lib library of instruction manuals",
        "data_type": "ExternalId"
    },
    "P6334": {
        "label": "Agence photo RMN package ID",
        "description": "identifier for a museum, exhibition, collection, or theme, on the 'Agence photo de la Réunion des Musées nationaux et du Grand Palais' website",
        "data_type": "ExternalId"
    },
    "P6325": {
        "label": "Critique d'art ID",
        "description": "identifier for an entry on the Critique d'art website",
        "data_type": "ExternalId"
    },
    "P6436": {
        "label": "Code of Natura 2000/FFH habitat",
        "description": "codes used in the Q759874  as identifier for habitats",
        "data_type": "ExternalId"
    },
    "P6455": {
        "label": "Bilibili userID",
        "description": "this item's user ID on Bilibili",
        "data_type": "ExternalId"
    },
    "P6461": {
        "label": "Rock's Backpages artist ID",
        "description": "identifier for a music artist, featured in the Rock's Backpages archive",
        "data_type": "ExternalId"
    },
    "P6172": {
        "label": "Satakore game ID",
        "description": "identifier of a video game in the Satakore database",
        "data_type": "ExternalId"
    },
    "P6123": {
        "label": "B3Kat dataset ID",
        "description": "entry ID in B3Kat, the joint union catalogue of the Bavarian Library Network (Bibliotheksverbund Bayern - BVB) and the Cooperative Library Network Berlin-Brandenburg (KOBV)",
        "data_type": "ExternalId"
    },
    "P6367": {
        "label": "Bahamut Gamer's Community ACG Database",
        "description": "a Taiwanese database for Games, Animation, Comic, Light Novels",
        "data_type": "ExternalId"
    },
    "P6179": {
        "label": "Dimensions Publication ID",
        "description": "unique identifier for a work in the Dimensions database",
        "data_type": "ExternalId"
    },
    "P6390": {
        "label": "Education Institution Identification Number",
        "description": "identifier for an educational institution issued by the Bangladesh Bureau of Educational Information and Statistics",
        "data_type": "ExternalId"
    },
    "P6413": {
        "label": "JORFSearch organization ID",
        "data_type": "ExternalId"
    },
    "P6353": {
        "label": "OSGS-Number",
        "description": "ID for organisations registered with the OSGS",
        "data_type": "ExternalId"
    },
    "P6190": {
        "label": "NSW State Archives and Records Authority Agency ID",
        "description": "identifier for current and historic government agencies in New South Wales, Australia,  allocated by the State Records Authority",
        "data_type": "ExternalId"
    },
    "P6405": {
        "label": "Smarthistory ID",
        "description": "identifier for an artwork, an art movement or period on the Smarthistory website",
        "data_type": "ExternalId"
    },
    "P6409": {
        "label": "CORE ID",
        "description": "ID of an article in CORE (Connecting Repositories)",
        "data_type": "ExternalId"
    },
    "P6406": {
        "label": "vodnimlyny.cz ID",
        "description": "identifier in the database of water mills in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P6370": {
        "label": "REDIZO",
        "description": "identificator of school organization in Czech Republic",
        "data_type": "ExternalId"
    },
    "P2698": {
        "label": "CricketArchive player ID",
        "description": "identifier for a person in the CricketArchive.com database",
        "data_type": "ExternalId"
    },
    "P2694": {
        "label": "ISU figure skater ID",
        "description": "identifier for a figure skater in the International Skating Union database",
        "data_type": "ExternalId"
    },
    "P2972": {
        "label": "SNAP ID",
        "description": "identifier for an ancient person, in the SNAP project",
        "data_type": "ExternalId"
    },
    "P3065": {
        "label": "RERO ID",
        "description": "identifier in the Library network of Western Switzerland's RERO database",
        "data_type": "ExternalId"
    },
    "P2732": {
        "label": "Persée author ID",
        "description": "identifier for an author, in Persée",
        "data_type": "ExternalId"
    },
    "P2990": {
        "label": "FIL athlete ID",
        "description": "identifier for sportsperson in the International Luge Federation (FIL) database",
        "data_type": "ExternalId"
    },
    "P2492": {
        "label": "MTMT author ID",
        "description": "author's identifier in the Hungarian Scientific Bibliography",
        "data_type": "ExternalId"
    },
    "P2767": {
        "label": "JudoInside judoka ID",
        "description": "Identifier for a judoka at the JudoInside.com database",
        "data_type": "ExternalId"
    },
    "P3023": {
        "label": "Legacies of British Slave-ownership person ID",
        "description": "identifier of a person, in University College London's 'Legacies of British Slave-ownership' project",
        "data_type": "ExternalId"
    },
    "P2800": {
        "label": "Beach Volleyball Database ID",
        "description": "identifier for an athlete, in the Beach Volleyball Database",
        "data_type": "ExternalId"
    },
    "P2726": {
        "label": "UIPM athlete ID",
        "description": "identifier for a athlete, in the Union Internationale de Pentathlon Moderne database",
        "data_type": "ExternalId"
    },
    "P3044": {
        "label": "College Football HoF ID",
        "description": "identifier for American football players in the College Football Hall of Fame",
        "data_type": "ExternalId"
    },
    "P3049": {
        "label": "Scottish Football Association player ID",
        "description": "identifier of a player in the records of the Scottish Football Association",
        "data_type": "ExternalId"
    },
    "P3056": {
        "label": "TCM Movie Database person ID",
        "description": "identifier for a person (cast or crew member) in the Turner Classic Movies movie database",
        "data_type": "ExternalId"
    },
    "P3122": {
        "label": "OpenDomesday person ID",
        "description": "Identifier for a person, in the OpenDomesday digitisation of the Domesday Book",
        "data_type": "ExternalId"
    },
    "P2914": {
        "label": "MSBI person ID",
        "description": "identifier of a person, in the Mapping the Practice and Profession of Sculpture in Britain and Ireland database",
        "data_type": "ExternalId"
    },
    "P2818": {
        "label": "Sherdog ID",
        "description": "identifier for a mixed martial arts athlete, in the Sherdog database",
        "data_type": "ExternalId"
    },
    "P2930": {
        "label": "INSPIRE-HEP author ID",
        "description": "identifier for authors in INSPIRE-HEP, a major database for high energy physics",
        "data_type": "ExternalId"
    },
    "P2773": {
        "label": "FIS cross-country skier ID",
        "description": "identifier for a cross-country skier, in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P2772": {
        "label": "FIS alpine skier ID",
        "description": "identifier for an alpine skier, in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P2775": {
        "label": "FIS ski jumper ID",
        "description": "identifier for a ski jumper, in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P3133": {
        "label": "NSZL name authority ID",
        "description": "ID of a person or organisation in the \"Integral Information System of the National Library\" (NEKTÁR) in Hungary",
        "data_type": "ExternalId"
    },
    "P2953": {
        "label": "Estonian Research Portal person ID",
        "description": "identifier for a person, in the Estonian Research Portal",
        "data_type": "ExternalId"
    },
    "P2805": {
        "label": "Goratings ID",
        "description": "goratings.org Go player identifier",
        "data_type": "ExternalId"
    },
    "P2810": {
        "label": "LPGA Tour golf player ID",
        "description": "identifier for a golf player, in the LPGA database",
        "data_type": "ExternalId"
    },
    "P2804": {
        "label": "World Sailing member ID",
        "description": "member identifier for a sailor in the World Sailing database",
        "data_type": "ExternalId"
    },
    "P3140": {
        "label": "SourehCinema person ID",
        "description": "identifier for a person at the Iranian movie database SourehCinema",
        "data_type": "ExternalId"
    },
    "P2639": {
        "label": "Filmportal ID",
        "description": "identifier of the German Filmportal.de",
        "data_type": "ExternalId"
    },
    "P2991": {
        "label": "IBSF athlete ID",
        "description": "identifier for sportsperson in the International Bobsleigh & Skeleton Federation (IBSF) database",
        "data_type": "ExternalId"
    },
    "P2602": {
        "label": "HockeyDB.com player ID",
        "description": "identifier for an ice hockey player in the Internet Hockey Database",
        "data_type": "ExternalId"
    },
    "P2592": {
        "label": "Québec cultural heritage directory people identifier",
        "description": "people listed on the Répertoire du patrimoine culturel du Québec",
        "data_type": "ExternalId"
    },
    "P2611": {
        "label": "TED speaker ID",
        "description": "identifier of a person, in the TED database of talks",
        "data_type": "ExternalId"
    },
    "P2823": {
        "label": "Royal Belgian Football Association player ID",
        "description": "identifier for a player, on the official Belgian football federation website",
        "data_type": "ExternalId"
    },
    "P2829": {
        "label": "Internet Wrestling Database ID",
        "description": "identifier for a professional wrestling performer at the Internet Wrestling Database",
        "data_type": "ExternalId"
    },
    "P3126": {
        "label": "ALCUIN ID",
        "description": "identifier of a philosopher in the ALCUIN Infothek der Scholastik",
        "data_type": "ExternalId"
    },
    "P2722": {
        "label": "Deezer artist ID",
        "description": "identifier for an artist on Deezer",
        "data_type": "ExternalId"
    },
    "P2730": {
        "label": "ISSF athlete ID",
        "description": "identifier for an athlete in the International Shooting Sport Federation database",
        "data_type": "ExternalId"
    },
    "P3124": {
        "label": "Polish scientist ID",
        "description": "identifier for a scientist, in the Polish Government's Information Processing Centre database",
        "data_type": "ExternalId"
    },
    "P2889": {
        "label": "FamilySearch person ID",
        "description": "ID of a person in familysearch.org",
        "data_type": "ExternalId"
    },
    "P3045": {
        "label": "HanCinema person ID",
        "description": "identifier for a person in the HanCinema database",
        "data_type": "ExternalId"
    },
    "P2776": {
        "label": "FIS Nordic combined skier ID",
        "description": "identifier for a Nordic combined skier, in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P2750": {
        "label": "Photographers' Identities Catalog ID",
        "description": "identifier used in Photographers' Identities Catalog",
        "data_type": "ExternalId"
    },
    "P2764": {
        "label": "Wrestlingdata person ID",
        "description": "identifier for a professional wrestling performer on the Wrestlingdata database",
        "data_type": "ExternalId"
    },
    "P2601": {
        "label": "Eurohockey.com player ID",
        "description": "identifier for an ice hockey player in the database of Eurohockey.com",
        "data_type": "ExternalId"
    },
    "P2593": {
        "label": "Latvian Olympic Committee athlete ID",
        "description": "identifier of a person in the database of the Latvian Olympic Committee (Latvian: Latvijas Olimpiskā Komiteja, LOK)",
        "data_type": "ExternalId"
    },
    "P2696": {
        "label": "FIG gymnast licence number",
        "description": "gymnast's identifier at the database of the International Federation of Gymnastics",
        "data_type": "ExternalId"
    },
    "P2941": {
        "label": "Munk's Roll ID",
        "description": "identifier of a person, in the biographical website of the Royal College of Physicians, London",
        "data_type": "ExternalId"
    },
    "P2943": {
        "label": "warheroes.ru ID",
        "description": "identifier of a person at warheroes.ru, awarded with title Hero of the Soviet Union or Hero of the Socialist Labour",
        "data_type": "ExternalId"
    },
    "P2977": {
        "label": "LBT person ID",
        "description": "identifier of a person, on Lord Byron and his Times website",
        "data_type": "ExternalId"
    },
    "P2625": {
        "label": "PASE ID",
        "description": "numerical identifier for a person in the Prosopography of Anglo-Saxon England",
        "data_type": "ExternalId"
    },
    "P2949": {
        "label": "WikiTree person ID",
        "description": "identifier for an person in the WikiTree genealogy website",
        "data_type": "ExternalId"
    },
    "P3053": {
        "label": "K League player ID",
        "description": "identifier for a player at website of K League, South Korea's professional association football (soccer) league",
        "data_type": "ExternalId"
    },
    "P3051": {
        "label": "Kindred Britain ID",
        "description": "identifier of an article on Kindred Britain",
        "data_type": "ExternalId"
    },
    "P3029": {
        "label": "UK National Archives ID",
        "description": "identifier for a person, family or organisation, in the UK's National Archives database",
        "data_type": "ExternalId"
    },
    "P3111": {
        "label": "FEI person ID",
        "description": "identifier of athlete at the FEI website",
        "data_type": "ExternalId"
    },
    "P2778": {
        "label": "IAT triathlete ID",
        "description": "identifier for a triathlete in the IAT (Institute for Applied Training Science / Institut für Angewandte Trainingswissenschaft) database",
        "data_type": "ExternalId"
    },
    "P2780": {
        "label": "IAT diver ID",
        "description": "identifier for a diver in the IAT (Institute for Applied Training Science / Institut für Angewandte Trainingswissenschaft) database",
        "data_type": "ExternalId"
    },
    "P2966": {
        "label": "National Library of Wales Authority ID",
        "description": "identifier of persons, corporate entities and families in the National Library of Wales Archives and Manuscripts database",
        "data_type": "ExternalId"
    },
    "P3046": {
        "label": "ForaDeJogo player ID",
        "description": "ForaDeJogo.net identifier for an association football (soccer) player",
        "data_type": "ExternalId"
    },
    "P3050": {
        "label": "FACR player ID",
        "description": "identifier for an association football player at nv.fotbal.cz, website of the Football Association of the Czech Republic (FAČR: Fotbalová asociace České republiky), formerly the Bohemian-Moravian Football Union (ČMFS: Českomoravský fotbalový svaz)",
        "data_type": "ExternalId"
    },
    "P2642": {
        "label": "Fed Cup player ID",
        "description": "identifier for a women's tennis player, in the Fed Cup database. Format: 9 digits",
        "data_type": "ExternalId"
    },
    "P2811": {
        "label": "PGA Tour golf player ID",
        "description": "identifier for a golf player, in the PGA Tour database",
        "data_type": "ExternalId"
    },
    "P2987": {
        "label": "Taekwondo Data ID",
        "description": "identifier for sportsperson in the Taekwondo Data database",
        "data_type": "ExternalId"
    },
    "P2638": {
        "label": "TV.com ID",
        "description": "identifier for an entry on TV.com for movies, people and TV series",
        "data_type": "ExternalId"
    },
    "P3048": {
        "label": "racing-reference.info driver ID",
        "description": "identifier of a driver, in the racing-reference.info database",
        "data_type": "ExternalId"
    },
    "P3043": {
        "label": "Scoresway soccer person ID",
        "description": "identifier for a association football (soccer) player, manager or referee at the Scoresway website",
        "data_type": "ExternalId"
    },
    "P2728": {
        "label": "CageMatch worker ID",
        "description": "identifier for a professional wrestling performer at www.cagematch.net",
        "data_type": "ExternalId"
    },
    "P2709": {
        "label": "CQ Ranking female cyclist ID",
        "description": "identifier for a female cyclist at cqranking.com",
        "data_type": "ExternalId"
    },
    "P2705": {
        "label": "Karate Records ID",
        "description": "identifier for karatekas in the Karate Records database",
        "data_type": "ExternalId"
    },
    "P2861": {
        "label": "Leidse Hoogleraren ID",
        "description": "identifier in the Leidse Hoogleraren, a catalogue of University Professors of Leiden University since 1575",
        "data_type": "ExternalId"
    },
    "P3114": {
        "label": "ISzDb person ID",
        "description": "identifier of a person in the ISzDb (a Hungarian dub database)",
        "data_type": "ExternalId"
    },
    "P2485": {
        "label": "FMD photographer ID",
        "description": "external link to a photographer's profile at the FMD (Fashion Model Directory) website",
        "data_type": "ExternalId"
    },
    "P2736": {
        "label": "Biographical Directory of Federal Judges ID",
        "description": "identifier for a judge in the Biographical Directory of Federal Judges",
        "data_type": "ExternalId"
    },
    "P2574": {
        "label": "National-Football-Teams.com player ID",
        "description": "identifier for a football player, at National-Football-Teams.com",
        "data_type": "ExternalId"
    },
    "P2774": {
        "label": "FIS freestyle skier ID",
        "description": "identifier for a freestyle skier, in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P2777": {
        "label": "FIS snowboarder ID",
        "description": "identifier for a snowboarder, in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P2519": {
        "label": "Scope.dk person ID",
        "description": "identifier for a person on the website Scope.dk",
        "data_type": "ExternalId"
    },
    "P2940": {
        "label": "Catalogus Professorum Rostochiensium ID",
        "description": "identifier in the Catalogus Professorum Rostochensium database on professors of the Rostock University from 1419 to present",
        "data_type": "ExternalId"
    },
    "P2729": {
        "label": "BWF.tournamentsoftware.com player ID",
        "description": "Badminton World Federation identifier for a player profile at BWFbadminton.org (via BWF.tournamentsoftware.com)",
        "data_type": "ExternalId"
    },
    "P2727": {
        "label": "United World Wrestling ID",
        "description": "identifier for an athlete in the United World Wrestling database",
        "data_type": "ExternalId"
    },
    "P2798": {
        "label": "Loop ID",
        "description": "identifier for a person, in the Loop database of researcher impact",
        "data_type": "ExternalId"
    },
    "P2626": {
        "label": "Danish National Filmography person ID",
        "description": "identifier for a person in the Danish National Filmography",
        "data_type": "ExternalId"
    },
    "P2830": {
        "label": "Online World of Wrestling ID",
        "description": "identifier for a professional wrestling performer or tag team at Online World of Wrestling",
        "data_type": "ExternalId"
    },
    "P3136": {
        "label": "elCinema person ID",
        "description": "identifier for a person at elCinema",
        "data_type": "ExternalId"
    },
    "P2779": {
        "label": "IAT weightlifter ID",
        "description": "identifier for a weightlifter in the IAT (Institute for Applied Training Science / Institut für Angewandte Trainingswissenschaft) database",
        "data_type": "ExternalId"
    },
    "P2801": {
        "label": "FIVB beach volleyball player ID",
        "description": "identifier for a beach volleyball player, in the FIVB database",
        "data_type": "ExternalId"
    },
    "P2862": {
        "label": "Catalogus Professorum Academiae Rheno-Traiectinae ID",
        "description": "identifier for a professor at Utrecht University",
        "data_type": "ExternalId"
    },
    "P2640": {
        "label": "Swimrankings.net swimmer ID",
        "description": "identifier for a swimmer, in the Swimrankings.net database",
        "data_type": "ExternalId"
    },
    "P2973": {
        "label": "Spenserians person ID",
        "description": "identifier of a person on the site of the Spenserians project",
        "data_type": "ExternalId"
    },
    "P3010": {
        "label": "World Archery Federation ID",
        "description": "identifier for sportsperson in the World Archery Federation database",
        "data_type": "ExternalId"
    },
    "P2945": {
        "label": "British Book Trade Index ID",
        "description": "identifier for an individual or business entity listed in the  University of Oxford's Bodleian Libraries' British Book Trade Index database",
        "data_type": "ExternalId"
    },
    "P2641": {
        "label": "Davis Cup player ID",
        "description": "identifier for a tennis player, in the Davis Cup database",
        "data_type": "ExternalId"
    },
    "P3052": {
        "label": "Bloomberg person ID",
        "description": "identifier for a business person, at Bloomberg",
        "data_type": "ExternalId"
    },
    "P2580": {
        "label": "Baltisches Biographisches Lexikon digital ID (former scheme)",
        "description": "person's ID at Baltisches Biographisches Lexikon digital encyclopedia",
        "data_type": "ExternalId"
    },
    "P2605": {
        "label": "ČSFD person ID",
        "description": "identifier for a person in the Czech film database ČSFD",
        "data_type": "ExternalId"
    },
    "P2600": {
        "label": "Geni.com profile ID",
        "description": "profile on the Geni.com genealogy website",
        "data_type": "ExternalId"
    },
    "P3047": {
        "label": "PlaymakerStats.com player ID",
        "description": "identifier for an association football (soccer) or handball player at PlaymakerStats.com (formerly TheFinalBall.com)",
        "data_type": "ExternalId"
    },
    "P2685": {
        "label": "Basketball Reference NBA player ID",
        "description": "identifier for a NBA player on the Basketball Reference website",
        "data_type": "ExternalId"
    },
    "P2884": {
        "label": "mains voltage",
        "description": "voltage of residential mains electricity in a country or region",
        "data_type": "Quantity"
    },
    "P1434": {
        "label": "takes place in fictional universe",
        "description": "the subject is a work describing a fictional universe, i.e. whose plot occurs in this universe.",
        "data_type": "WikibaseItem"
    },
    "P4170": {
        "label": "America's National Parks ID",
        "description": "identifier for a United States National Park System unit on eParks.com",
        "data_type": "ExternalId"
    },
    "P39": {
        "label": "position held",
        "description": "subject currently or formerly holds the object position or public office",
        "data_type": "WikibaseItem"
    },
    "P2438": {
        "label": "narrator",
        "description": "narrator, character or person that tells the story",
        "data_type": "WikibaseItem"
    },
    "P9071": {
        "label": "character type",
        "description": "character type (e.g. character archetype, stock character, character stereotype etc.) this fictional character represents",
        "data_type": "WikibaseItem"
    },
    "P2872": {
        "label": "visitor centre",
        "description": "official tourist office of this destination",
        "data_type": "WikibaseItem"
    },
    "P1412": {
        "label": "languages spoken, written or signed",
        "description": "language(s) that a person speaks, writes or signs, including the native language(s)",
        "data_type": "WikibaseItem"
    },
    "P1449": {
        "label": "nickname",
        "description": "informal name (for a pseudonym use P742)",
        "data_type": "Monolingualtext"
    },
    "P4634": {
        "label": "premiere type",
        "description": "indicates the premiere type (world premiere, language premiere, country premiere) of a performing arts production",
        "data_type": "WikibaseItem"
    },
    "P1441": {
        "label": "present in work",
        "description": "this (fictional or fictionalized) entity or person appears in that work as part of the narration (use P2860 for works citing other works, P361/P1433 for works being part of other works, P1343 for entities described in non-fictional accounts)",
        "data_type": "WikibaseItem"
    },
    "P3092": {
        "label": "film crew member",
        "description": "member of the crew creating an audiovisual work, used for miscellaneous roles qualified with the job title when no specific property exists. Don't use if such a property is available: notably for cast member (P161), director (P57), etc.",
        "data_type": "WikibaseItem"
    },
    "P2408": {
        "label": "set in period",
        "description": "historical, contemporary or future period the work is set in or event featured in the story of the work",
        "data_type": "WikibaseItem"
    },
    "P1881": {
        "label": "list of characters",
        "description": "Wikimedia page with the list of characters for this work",
        "data_type": "WikibaseItem"
    },
    "P1875": {
        "label": "represented by",
        "description": "person or agency that represents or manages the subject",
        "data_type": "WikibaseItem"
    },
    "P53": {
        "label": "family",
        "description": "family, including dynasty and nobility houses. Not family name (use P734 for family name).",
        "data_type": "WikibaseItem"
    },
    "P669": {
        "label": "located on street",
        "description": "street, road, or square, where the item is located. To add the number, use Property:P670 \"street number\" as qualifier. Use property P6375 if there is no item for the street",
        "data_type": "WikibaseItem"
    },
    "P421": {
        "label": "located in time zone",
        "description": "time zone for this item",
        "data_type": "WikibaseItem"
    },
    "P54": {
        "label": "member of sports team",
        "description": "sports teams or clubs that the subject currently represents or formerly represented",
        "data_type": "WikibaseItem"
    },
    "P735": {
        "label": "given name",
        "description": "first name or another given name of this person; values used with the property shouldn't link disambiguations nor family names.",
        "data_type": "WikibaseItem"
    },
    "P1080": {
        "label": "from narrative universe",
        "description": "subject's fictional entity is in the object narrative. See also P1441 (present in work) and P1445 (fictional universe described in)",
        "data_type": "WikibaseItem"
    },
    "P1477": {
        "label": "birth name",
        "description": "full name of a person at birth, if different from their current, generally used name (samples: John Peter Doe for Joe Doe, Ann Smith for Ann Miller)",
        "data_type": "Monolingualtext"
    },
    "P1445": {
        "label": "fictional universe described in",
        "description": "to link a fictional universe with a work that describes it: <universe> \"described in the work:\" <work>",
        "data_type": "WikibaseItem"
    },
    "P108": {
        "label": "employer",
        "description": "person or organization for which the subject works or worked",
        "data_type": "WikibaseItem"
    },
    "P1074": {
        "label": "fictional analog of",
        "description": "used to link an entity or class of entities appearing in a creative work with the analogous entity or class of entities in the real world",
        "data_type": "WikibaseItem"
    },
    "P8670": {
        "label": "character designer",
        "description": "person in charge of developing the style and concept of a character in a video game, anime or other creative work",
        "data_type": "WikibaseItem"
    },
    "P734": {
        "label": "family name",
        "description": "part of full name of person",
        "data_type": "WikibaseItem"
    },
    "P1038": {
        "label": "relative",
        "description": "family member (qualify with \"type of kinship\", P1039; for direct family member please use specific property)",
        "data_type": "WikibaseItem"
    },
    "P8419": {
        "label": "Archive Of Our Own tag",
        "description": "tag for this item on \"Archive of Our Own\"",
        "data_type": "ExternalId"
    },
    "P8411": {
        "label": "set in environment",
        "description": "environment (geographic feature, building type, etc.) the narrative work is set in",
        "data_type": "WikibaseItem"
    },
    "P21": {
        "label": "sex or gender",
        "description": "sex or gender identity of human or animal. For human: male, female, non-binary, intersex, transgender female, transgender male, agender. For animal: male organism, female organism. Groups of same gender use subclass of (P279)",
        "data_type": "WikibaseItem"
    },
    "P9215": {
        "label": "set during recurring event",
        "description": "recurring event (e.g. season, month, holiday) during which this fictional work is set",
        "data_type": "WikibaseItem"
    },
    "P6375": {
        "label": "street address",
        "description": "full street address where subject is located. Include building number, city/locality, post code, but not country. Use also P669 if the street has its own separate item",
        "data_type": "Monolingualtext"
    },
    "P40": {
        "label": "child",
        "description": "subject has object as child. Do not use for stepchildren",
        "data_type": "WikibaseItem"
    },
    "P6992": {
        "label": "IMFDB ID",
        "description": "identifier for the Internet Movie Firearms Database",
        "data_type": "ExternalId"
    },
    "P4584": {
        "label": "first appearance",
        "description": "work in which a fictional/mythical character or entity first appeared",
        "data_type": "WikibaseItem"
    },
    "P463": {
        "label": "member of",
        "description": "organization, club or musical group to which the subject belongs. Do not use for membership in ethnic or social groups, nor for holding a position such as a member of parliament (use P39 for that).",
        "data_type": "WikibaseItem"
    },
    "P2853": {
        "label": "electrical plug type",
        "description": "standard plug type for mains electricity in a country",
        "data_type": "WikibaseItem"
    },
    "P17": {
        "label": "country",
        "description": "sovereign state of this item (not to be used for human beings)",
        "data_type": "WikibaseItem"
    },
    "P2907": {
        "label": "UTC timezone offset",
        "description": "difference between Coordinated Universal Time (UTC) and this timezone",
        "data_type": "Quantity"
    },
    "P5940": {
        "label": "plot expanded in",
        "description": "a narrative of this creative work is expanded in another work while '''not necessarily''' being part of the same series. For series use P179 instead",
        "data_type": "WikibaseItem"
    },
    "P511": {
        "label": "honorific prefix",
        "description": "word or expression used before a name, in addressing or referring to a person",
        "data_type": "WikibaseItem"
    },
    "P2554": {
        "label": "production designer",
        "description": "production designer(s) of this motion picture, play, video game or similar",
        "data_type": "WikibaseItem"
    },
    "P3311": {
        "label": "plan view image",
        "description": "image representing the plan of a building or place",
        "data_type": "CommonsMedia"
    },
    "P26": {
        "label": "spouse",
        "description": "the subject has the object as their spouse (husband, wife, partner, etc.). Use \"unmarried partner\" (P451) for non-married companions",
        "data_type": "WikibaseItem"
    },
    "P281": {
        "label": "postal code",
        "description": "identifier assigned by postal authorities for the subject area or building",
        "data_type": "String"
    },
    "P22": {
        "label": "father",
        "description": "male parent of the subject. For stepfather, use \"stepparent\" (P3448)",
        "data_type": "WikibaseItem"
    },
    "P1611": {
        "label": "NATO code for grade",
        "description": "NATO Code for Grades of Military Personnel",
        "data_type": "WikibaseItem"
    },
    "P937": {
        "label": "work location",
        "description": "location where persons were active",
        "data_type": "WikibaseItem"
    },
    "P841": {
        "label": "feast day",
        "description": "saint's principal feast day",
        "data_type": "WikibaseItem"
    },
    "P1035": {
        "label": "honorific suffix",
        "description": "word or expression with connotations conveying esteem or respect when used, after a name, in addressing or referring to a person",
        "data_type": "WikibaseItem"
    },
    "P1327": {
        "label": "partner in business or sport",
        "description": "professional collaborator",
        "data_type": "WikibaseItem"
    },
    "P800": {
        "label": "notable work",
        "description": "notable scientific, artistic or literary work, or other work of significance among subject's works",
        "data_type": "WikibaseItem"
    },
    "P1598": {
        "label": "consecrator",
        "description": "bishop who presided as consecrator or co-consecrator of this bishop",
        "data_type": "WikibaseItem"
    },
    "P1559": {
        "label": "name in native language",
        "description": "name of a person in their native language. Could be displayed in addition to the label, if language has a different script",
        "data_type": "Monolingualtext"
    },
    "P811": {
        "label": "academic minor",
        "description": "minor someone studied at college/university",
        "data_type": "WikibaseItem"
    },
    "P1049": {
        "label": "worshipped by",
        "description": "religion or group/civilization that worships a given deity",
        "data_type": "WikibaseItem"
    },
    "P1350": {
        "label": "number of matches played/races/starts",
        "description": "matches or games a player or a team played during an event. Also a total number of matches a player officially appeared in during the whole career.",
        "data_type": "Quantity"
    },
    "P1321": {
        "label": "place of origin (Switzerland)",
        "description": "lieu d'origine/Heimatort/luogo d'origine of a Swiss national. Not be confused with place of birth or place of residence",
        "data_type": "WikibaseItem"
    },
    "P1596": {
        "label": "penalty",
        "description": "penalty imposed by an authority",
        "data_type": "WikibaseItem"
    },
    "P172": {
        "label": "ethnic group",
        "description": "subject's ethnicity (consensus is that a VERY high standard of proof is needed for this field to be used. In general this means 1) the subject claims it themself, or 2) it is widely agreed on by scholars, or 3) is fictional and portrayed as such)",
        "data_type": "WikibaseItem"
    },
    "P1429": {
        "label": "has pet",
        "description": "pet that a person owns",
        "data_type": "WikibaseItem"
    },
    "P553": {
        "label": "website account on",
        "description": "website that the person or organization has an account on (use with P554) Note: only used with reliable source or if the person or organization disclosed it.",
        "data_type": "WikibaseItem"
    },
    "P1283": {
        "label": "filmography",
        "description": "item for the list of films a person has contributed to. Don't use it to add film items. Instead, add actors as \"cast member\" (P161) on items for films, directors with \"director\" (P57)",
        "data_type": "WikibaseItem"
    },
    "P412": {
        "label": "voice type",
        "description": "person's voice type. expected values: soprano, mezzo-soprano, contralto, countertenor, tenor, baritone, bass (and derivatives)",
        "data_type": "WikibaseItem"
    },
    "P413": {
        "label": "position played on team / speciality",
        "description": "position or specialism of a player on a team",
        "data_type": "WikibaseItem"
    },
    "P1393": {
        "label": "proxy",
        "description": "person authorized to act for another",
        "data_type": "WikibaseItem"
    },
    "P512": {
        "label": "academic degree",
        "description": "academic degree that the person holds",
        "data_type": "WikibaseItem"
    },
    "P91": {
        "label": "sexual orientation",
        "description": "the sexual orientation of the person — use IF AND ONLY IF they have stated it themselves, unambiguously, or it has been widely agreed upon by historians after their death",
        "data_type": "WikibaseItem"
    },
    "P802": {
        "label": "student",
        "description": "notable student(s) of the subject individual",
        "data_type": "WikibaseItem"
    },
    "P1345": {
        "label": "number of victims of killer",
        "description": "number of victims of a serial killer or murderer",
        "data_type": "Quantity"
    },
    "P1344": {
        "label": "participant in",
        "description": "event in which a person or organization was/is a participant; inverse of P710 or P1923",
        "data_type": "WikibaseItem"
    },
    "P27": {
        "label": "country of citizenship",
        "description": "the object is a country that recognizes the subject as its citizen",
        "data_type": "WikibaseItem"
    },
    "P1066": {
        "label": "student of",
        "description": "person who has taught this person",
        "data_type": "WikibaseItem"
    },
    "P468": {
        "label": "dan/kyu rank",
        "description": "rank system used in several board games (e.g. go, shogi, renju), martial arts (e.g. judo, kendo, wushu) and some other games",
        "data_type": "WikibaseItem"
    },
    "P1340": {
        "label": "eye color",
        "description": "color of the irises of a person's eyes",
        "data_type": "WikibaseItem"
    },
    "P263": {
        "label": "official residence",
        "description": "the residence at which heads of government and other senior figures officially reside",
        "data_type": "WikibaseItem"
    },
    "P1576": {
        "label": "lifestyle",
        "description": "typical way of life of an individual, group, or culture",
        "data_type": "WikibaseItem"
    },
    "P1593": {
        "label": "defender",
        "description": "person representing the defendant, at a trial",
        "data_type": "WikibaseItem"
    },
    "P552": {
        "label": "handedness",
        "description": "handedness of the person",
        "data_type": "WikibaseItem"
    },
    "P106": {
        "label": "occupation",
        "description": "occupation of a person; see also \"field of work\" (Property:P101), \"position held\" (Property:P39)",
        "data_type": "WikibaseItem"
    },
    "P1050": {
        "label": "medical condition",
        "description": "any state relevant to the health of an organism, including diseases and positive conditions",
        "data_type": "WikibaseItem"
    },
    "P184": {
        "label": "doctoral advisor",
        "description": "person who supervised the doctorate or PhD thesis of the subject",
        "data_type": "WikibaseItem"
    },
    "P550": {
        "label": "chivalric order",
        "description": "the chivalric order which a person belongs to",
        "data_type": "WikibaseItem"
    },
    "P1303": {
        "label": "instrument",
        "description": "musical instrument that a person plays or teaches or used in a music occupation",
        "data_type": "WikibaseItem"
    },
    "P140": {
        "label": "religion",
        "description": "religion of a person, organization or religious building, or associated with this subject",
        "data_type": "WikibaseItem"
    },
    "P1731": {
        "label": "Fach",
        "description": "describes the special ablilites of an operatic singers voice",
        "data_type": "WikibaseItem"
    },
    "P607": {
        "label": "conflict",
        "description": "battles, wars or other military engagements in which the person or item participated",
        "data_type": "WikibaseItem"
    },
    "P450": {
        "label": "astronaut mission",
        "description": "space mission that the subject is or has been a member of (do not include future missions)",
        "data_type": "WikibaseItem"
    },
    "P69": {
        "label": "educated at",
        "description": "educational institution attended by subject",
        "data_type": "WikibaseItem"
    },
    "P102": {
        "label": "member of political party",
        "description": "the political party of which this politician is or has been a member or otherwise affiliated",
        "data_type": "WikibaseItem"
    },
    "P410": {
        "label": "military rank",
        "description": "military rank achieved by a person (should usually have a \"start time\" qualifier), or military rank associated with a position",
        "data_type": "WikibaseItem"
    },
    "P470": {
        "label": "Eight Banner register",
        "description": "Manchu household register for people of the Qing Dynasty",
        "data_type": "WikibaseItem"
    },
    "P812": {
        "label": "academic major",
        "description": "subject someone studied at college/university",
        "data_type": "WikibaseItem"
    },
    "P1635": {
        "label": "religious name",
        "description": "name taken or used as a member of a religious community",
        "data_type": "Monolingualtext"
    },
    "P66": {
        "label": "ancestral home",
        "description": "place of origin for ancestors of subject",
        "data_type": "WikibaseItem"
    },
    "P185": {
        "label": "doctoral student",
        "description": "doctoral student(s) of a professor",
        "data_type": "WikibaseItem"
    },
    "P1677": {
        "label": "index case of",
        "description": "primary case, patient zero: initial patient in the population of an epidemiological investigation",
        "data_type": "WikibaseItem"
    },
    "P551": {
        "label": "residence",
        "description": "the place where the person is or has been, resident",
        "data_type": "WikibaseItem"
    },
    "P1308": {
        "label": "officeholder",
        "description": "persons who hold and/or held an office or noble title",
        "data_type": "WikibaseItem"
    },
    "P1318": {
        "label": "proved by",
        "description": "person who proved something",
        "data_type": "WikibaseItem"
    },
    "P803": {
        "label": "professorship",
        "description": "professorship position held by this academic person",
        "data_type": "WikibaseItem"
    },
    "P1455": {
        "label": "list of works",
        "description": "link to the article with the works of a person; use P358 for discographies",
        "data_type": "WikibaseItem"
    },
    "P411": {
        "label": "canonization status",
        "description": "stage in the process of attaining sainthood per the subject's religious organization",
        "data_type": "WikibaseItem"
    },
    "P97": {
        "label": "noble title",
        "description": "titles held by the person",
        "data_type": "WikibaseItem"
    },
    "P103": {
        "label": "native language",
        "description": "language or languages a person has learned from early childhood",
        "data_type": "WikibaseItem"
    },
    "P1000": {
        "label": "record held",
        "description": "notable record achieved by a person or entity, include qualifiers for dates held",
        "data_type": "WikibaseItem"
    },
    "P6748": {
        "label": "Interactive Fiction Database ID",
        "description": "identifier for games on tads.org",
        "data_type": "ExternalId"
    },
    "P6754": {
        "label": "OBIS ID",
        "description": "identifier for a taxon, in the Ocean Biogeographic Information System",
        "data_type": "ExternalId"
    },
    "P6716": {
        "label": "MyNeta.info political party ID",
        "description": "external identifier to link data about Indian political parties",
        "data_type": "ExternalId"
    },
    "P6766": {
        "label": "Who's on First ID",
        "description": "identifier from the Who's on First database",
        "data_type": "ExternalId"
    },
    "P6822": {
        "label": "Tainacan MAI ID",
        "description": "identifier for items in the collection of the Museu Arqueologia de Itaipu, Rio de Janeiro",
        "data_type": "ExternalId"
    },
    "P6464": {
        "label": "Mtime people ID",
        "description": "identifier for a movie/film maker at the website Mtime",
        "data_type": "ExternalId"
    },
    "P6463": {
        "label": "Rock's Backpages publication ID",
        "description": "identifier for a newspaper or magazine, articles from which are featured in the Rock's Backpages archive",
        "data_type": "ExternalId"
    },
    "P6577": {
        "label": "Cour des comptes report ID",
        "description": "identifier for a report about an organization on the 'Cour des comptes's website",
        "data_type": "ExternalId"
    },
    "P6597": {
        "label": "Digital Dictionary of Surnames in Germany ID",
        "description": "ID of corresponding entry in the DFD online dictionary of family names",
        "data_type": "ExternalId"
    },
    "P6599": {
        "label": "HCA Red List of Endangered Crafts ID",
        "description": "identifier for entries in the HCA Red List of Endangered Crafts",
        "data_type": "ExternalId"
    },
    "P6776": {
        "label": "IMTL.org ID",
        "description": "identifier on the website Images de Montréal (imtl.org)",
        "data_type": "ExternalId"
    },
    "P6526": {
        "label": "data.gouv.fr dataset ID",
        "data_type": "ExternalId"
    },
    "P6542": {
        "label": "Inventário dos Monumentos RJ ID",
        "data_type": "ExternalId"
    },
    "P6564": {
        "label": "Brilliant Wiki ID",
        "description": "identifier for a Brilliant.org wiki article",
        "data_type": "ExternalId"
    },
    "P6512": {
        "label": "UN/CEFACT Common Code",
        "description": "unit of measurement identifier as defined by the UN/CEFACT in Recommendation No. 20: Codes for Units of Measure Used in International Trade",
        "data_type": "ExternalId"
    },
    "P6468": {
        "label": "ISA ID",
        "description": "identifier for Brazilian indigenous populations from Instituto Socioambiental",
        "data_type": "ExternalId"
    },
    "P6462": {
        "label": "Rock's Backpages author ID",
        "description": "identifier for a music journalist whose works are featured in the Rock's Backpages archive",
        "data_type": "ExternalId"
    },
    "P6476": {
        "label": "castingvideos ID",
        "description": "identifiers for actors on castingvideos.com",
        "data_type": "ExternalId"
    },
    "P6469": {
        "label": "Fuzhou Architecture Heritage ID",
        "description": "identifier for a building in Fuzhou Architecture Heritage",
        "data_type": "ExternalId"
    },
    "P6919": {
        "label": "Indian gallantry awardee ID",
        "description": "recipients of Indian gallantry awards",
        "data_type": "ExternalId"
    },
    "P6699": {
        "label": "Yu-Gi-Oh! TCG cards ID",
        "description": "unique ID for Yu-Gi-Oh! TCG (Trading Cards Game) cards listed at Yu-Gi-Oh! Card Database",
        "data_type": "ExternalId"
    },
    "P6913": {
        "label": "Facebook Gaming game ID",
        "description": "identifier for a video game on Facebook Gaming",
        "data_type": "ExternalId"
    },
    "P6916": {
        "label": "Heritage Gazetteer of Cyprus",
        "description": "identifier for sites in the Heritage Gazetteer of Cyprus",
        "data_type": "ExternalId"
    },
    "P6774": {
        "label": "Mod DB game ID",
        "description": "identifier of a video game on moddb.com",
        "data_type": "ExternalId"
    },
    "P6536": {
        "label": "Gateway to Research Project ID",
        "description": "identifier for a research project receiving public funding in the UK",
        "data_type": "ExternalId"
    },
    "P6513": {
        "label": "AICTE Permanent ID",
        "description": "identifier number issued  by AICTE, the authorized organization in India which grants approval for starting and accreditation technical institutions in India",
        "data_type": "ExternalId"
    },
    "P6503": {
        "label": "Indian census area code (1991)",
        "description": "code used for an area for calculations in the 1991 census of India",
        "data_type": "ExternalId"
    },
    "P6535": {
        "label": "BHL part ID",
        "description": "identifier of a part-work, in the BHL database",
        "data_type": "ExternalId"
    },
    "P6792": {
        "label": "Find NZ Artists ID",
        "description": "identifier for artists listed in Find New Zealand Artists online database",
        "data_type": "ExternalId"
    },
    "P6797": {
        "label": "MSX Games World ID",
        "description": "identifier of a video game in MSX Games World",
        "data_type": "ExternalId"
    },
    "P6808": {
        "label": "SportSport.ba player ID",
        "description": "property for various sportpersons bios and stats on sportsport.ba website",
        "data_type": "ExternalId"
    },
    "P6572": {
        "label": "ProQuest document ID",
        "description": "an identifier for microfilmed dissertations and other documents on ProQuest",
        "data_type": "ExternalId"
    },
    "P6555": {
        "label": "Brazilian Electoral Unit ID",
        "description": "unique identifier of an brazilian electoral unit, defined by the Brazilian Superior Electoral Court",
        "data_type": "ExternalId"
    },
    "P6810": {
        "label": "Xikao Repertoire ID",
        "description": "identifier for people, events and schools about the Chinese opera in the Xikao online dictionary",
        "data_type": "ExternalId"
    },
    "P6600": {
        "label": "Resident Advisor artist ID",
        "description": "identifier for artists at the Resident Advisor website",
        "data_type": "ExternalId"
    },
    "P6841": {
        "label": "ITF tournament ID",
        "description": "identifier for a tennis tournament edition in the International Tennis Federation database",
        "data_type": "ExternalId"
    },
    "P6557": {
        "label": "Dimensions.guide ID",
        "description": "identifier on Dimensions.guide database",
        "data_type": "ExternalId"
    },
    "P6567": {
        "label": "Florida Historical Marker List ID",
        "description": "identifier for Florida historical markers at Florida Division of Historical Resources website",
        "data_type": "ExternalId"
    },
    "P6653": {
        "label": "ETS Designated Institution code",
        "description": "an administrative 4-digit code that Educational Testing Service uses to uniquely identify institutions programs",
        "data_type": "ExternalId"
    },
    "P6717": {
        "label": "Indie DB game ID",
        "description": "identifier of a video game on indiedb.com",
        "data_type": "ExternalId"
    },
    "P6857": {
        "label": "JIS standard",
        "description": "standard that was developed by the Japanese Industrial Standards",
        "data_type": "ExternalId"
    },
    "P6838": {
        "label": "Elhuyar Dictionary ID",
        "description": "identifier for words in the Elhuyar Dictionary, a Basque multilingual dictionary (English, Spanish, French)",
        "data_type": "ExternalId"
    },
    "P6551": {
        "label": "Physics History Network ID",
        "description": "identifier from the Physics History Network at the American Institute of Physics",
        "data_type": "ExternalId"
    },
    "P6908": {
        "label": "Cameo ID",
        "description": "identifier for people on Cameo",
        "data_type": "ExternalId"
    },
    "P6881": {
        "label": "Rugby League Project ID (general)",
        "description": "a general identifier for items other than players in the Rugby League Project database",
        "data_type": "ExternalId"
    },
    "P6871": {
        "label": "ECI Lok Sabha constituency code",
        "description": "the identifier maintained by Election Commission of India for Lok Sabha constituencies",
        "data_type": "ExternalId"
    },
    "P6767": {
        "label": "FoodOn ID",
        "description": "identifier for FoodOn, an ontology focused on categorization and processing of food for humans",
        "data_type": "ExternalId"
    },
    "P6768": {
        "label": "SportSport.ba news ID",
        "description": "ID for a sports club on sportsport.ba website",
        "data_type": "ExternalId"
    },
    "P6581": {
        "label": "Deportation Database transport ID",
        "description": "online guide that includes most of the deportation transports that originated from cities in central, western and southern European countries between 1939 and 1945",
        "data_type": "ExternalId"
    },
    "P6900": {
        "label": "NicoNicoPedia ID",
        "description": "identifier for an article in NicoNicoPedia",
        "data_type": "ExternalId"
    },
    "P6880": {
        "label": "ATP tennis tournament edition ID",
        "description": "identifier for a particular year's event in the ATP database",
        "data_type": "ExternalId"
    },
    "P6917": {
        "label": "Historical Archives of the European Union ID",
        "description": "identifier for a person or an institution that have created, preserved and used the groups of records that constitute the fonds at the Historical Archives of the European Union",
        "data_type": "ExternalId"
    },
    "P6890": {
        "label": "Nchdb asset ID",
        "description": "identifier of a place or cultural practice in the database of the National cultural database management system of the Republic of China (Taiwan)",
        "data_type": "ExternalId"
    },
    "P6827": {
        "label": "Igromania ID",
        "description": "identifier for a video game at the website Igromania.ru",
        "data_type": "ExternalId"
    },
    "P6793": {
        "label": "WMI code",
        "data_type": "String"
    },
    "P6785": {
        "label": "Pakistan Railways station code",
        "description": "code to identify a railway station operated by Pakistan Railways",
        "data_type": "ExternalId"
    },
    "P6853": {
        "label": "Coco Game List ID",
        "description": "identifier in the Coco Game List of Color Computer video games",
        "data_type": "ExternalId"
    },
    "P6514": {
        "label": "DOGC ID",
        "description": "external and unique identifier which represents a publication in Diari Oficial de la Generalitat de Catalunya",
        "data_type": "ExternalId"
    },
    "P6809": {
        "label": "Xikao History ID",
        "description": "identifier for people, events and schools about the Chinese opera in the Xikao online dictionary",
        "data_type": "ExternalId"
    },
    "P6623": {
        "label": "Gamepedia article ID",
        "description": "identifier for an article on Gamepedia",
        "data_type": "ExternalId"
    },
    "P6906": {
        "label": "National Library of Aruba ID",
        "description": "ID used in the collection of the National Library of Aruba",
        "data_type": "ExternalId"
    },
    "P6928": {
        "label": "PCE Daisakusen ID",
        "description": "identifier in the PCE Daisakusen database of PC-Engine video games",
        "data_type": "ExternalId"
    },
    "P6659": {
        "label": "Baden-Württemberg protected area ID",
        "description": "identifier for nature and landscape protected areas in Baden-Württemberg (Germany), issued by the Landesanstalt für Umwelt Baden-Württemberg",
        "data_type": "ExternalId"
    },
    "P6562": {
        "label": "Google Play Movies & TV ID",
        "description": "identifier of films and television series on Google Play",
        "data_type": "ExternalId"
    },
    "P6566": {
        "label": "Indonesian Cultural Heritage Registration System ID",
        "description": "identifier of a Indonesian cultural heritage in Cultural Heritage National Registration System",
        "data_type": "ExternalId"
    },
    "P6558": {
        "label": "National Wrestling Hall of Fame University team ID",
        "description": "identifier for an American university wrestling team",
        "data_type": "ExternalId"
    },
    "P6649": {
        "label": "Democracy Club election ID",
        "description": "identifier in the Democracy Club database of elections",
        "data_type": "ExternalId"
    },
    "P6616": {
        "label": "Library of Parliament of Canada person ID",
        "description": "identifier for a federal politician in Canada (by the Library of Parliament)",
        "data_type": "ExternalId"
    },
    "P6749": {
        "label": "CanLII ID",
        "description": "identifier for decisions of Canadian court cases",
        "data_type": "ExternalId"
    },
    "P6617": {
        "label": "RoMEO publisher ID",
        "description": "identifier of an academic publisher in the SHERPA/RoMEO database of archiving policies",
        "data_type": "ExternalId"
    },
    "P6479": {
        "label": "IEEE Xplore author ID",
        "description": "identifier for an author in IEEE Xplore",
        "data_type": "ExternalId"
    },
    "P6713": {
        "label": "Australian War Memorial ID",
        "description": "identifier for a person, event, unit, place, or item in the Australian War Memorial collection",
        "data_type": "ExternalId"
    },
    "P6781": {
        "label": "ProofWiki ID",
        "description": "identifier for a ProofWiki article",
        "data_type": "ExternalId"
    },
    "P6482": {
        "label": "Image Archive, Herder Institute",
        "description": "link to the online database (Bildkatalog) of the image archive for the display of image sources",
        "data_type": "ExternalId"
    },
    "P6724": {
        "label": "Trainline ID",
        "description": "ID for a european railway station given by Trainline",
        "data_type": "ExternalId"
    },
    "P6666": {
        "label": "LibreGameWiki ID",
        "description": "identifier for articles on LibreGameWiki",
        "data_type": "ExternalId"
    },
    "P6760": {
        "label": "Know Your Meme ID",
        "description": "identifier for a Know Your Meme article",
        "data_type": "ExternalId"
    },
    "P6704": {
        "label": "FrogMAP ID",
        "description": "identifier of a species in FrogMAP - The Atlas of African Frogs encyclopedia",
        "data_type": "ExternalId"
    },
    "P6723": {
        "label": "BlackPast.org ID",
        "description": "identifier for articles on BlackPast.org",
        "data_type": "ExternalId"
    },
    "P6729": {
        "label": "SportSport.ba club ID",
        "description": "property for various sportclubs on sportsport.ba website",
        "data_type": "ExternalId"
    },
    "P6787": {
        "label": "Arachne building ID",
        "description": "identifier for a building in the Arachne database",
        "data_type": "ExternalId"
    },
    "P6646": {
        "label": "JMdictDB ID",
        "description": "identifier for entries at the Japanese Dictionary Database",
        "data_type": "ExternalId"
    },
    "P6494": {
        "label": "ISTC No.",
        "description": "number or ID in the Incunabula Short Title Catalogue",
        "data_type": "ExternalId"
    },
    "P6483": {
        "label": "WBCHSE code",
        "description": "identifier for a college or school situated in West Bengal",
        "data_type": "ExternalId"
    },
    "P7800": {
        "label": "bauhaus.community ID",
        "description": "identifier in the database https://bauhaus.community/",
        "data_type": "ExternalId"
    },
    "P7839": {
        "label": "Saregama artist ID",
        "description": "identifier to link artists included in Saregama",
        "data_type": "ExternalId"
    },
    "P8024": {
        "label": "Nobel Laureate API ID",
        "description": "REST API service with info about Nobel laureates",
        "data_type": "ExternalId"
    },
    "P8021": {
        "label": "Kooora/Goalzz player ID",
        "description": "identifier for an association football (soccer) or handball player at Kooora.com (in Arabic) and Goalzz.com (in English)",
        "data_type": "ExternalId"
    },
    "P8067": {
        "label": "Macdonald Dictionary ID",
        "description": "identifier in the Macdonald Dictionary",
        "data_type": "ExternalId"
    },
    "P8081": {
        "label": "WBIS ID",
        "description": "identifier for a person in a biographical dictionary comprised in the World Biographical Information System",
        "data_type": "ExternalId"
    },
    "P7869": {
        "label": "Analysis & Policy Observatory node ID",
        "description": "identifier for a node in the Analysis & Policy Observatory database",
        "data_type": "ExternalId"
    },
    "P7874": {
        "label": "EFIS person ID",
        "description": "identifier for a person in the Estonian Film Database",
        "data_type": "ExternalId"
    },
    "P8069": {
        "label": "ToposText person ID",
        "description": "identifier for a person in the website ToposText",
        "data_type": "ExternalId"
    },
    "P8050": {
        "label": "Women of Scotland subject ID",
        "description": "subject identifier for a woman in the Women of Scotland project",
        "data_type": "ExternalId"
    },
    "P7745": {
        "label": "Movie Walker person ID",
        "description": "identifier of person on Movie Walker database",
        "data_type": "ExternalId"
    },
    "P7754": {
        "label": "PS photographer ID",
        "description": "identifier for a photographer on Portrait sépia",
        "data_type": "ExternalId"
    },
    "P8051": {
        "label": "Geschichtsquellen des deutschen Mittelalters author ID",
        "description": "identifier for medieval German writers",
        "data_type": "ExternalId"
    },
    "P7920": {
        "label": "TheTVDB person ID",
        "description": "identifier for a person at TheTVDB",
        "data_type": "ExternalId"
    },
    "P8269": {
        "label": "Norwegian prisoner register person ID",
        "description": "identification number for Norwegians in captivity during World War II 1940 -1945",
        "data_type": "ExternalId"
    },
    "P7803": {
        "label": "Decine21 person ID",
        "description": "identifier for a person on the website decine21.com",
        "data_type": "ExternalId"
    },
    "P7750": {
        "label": "Nonfiction ID",
        "description": "numerical identifier for an author on Nonfiction",
        "data_type": "ExternalId"
    },
    "P7926": {
        "label": "Visual AIDS Artist+ Registry ID",
        "description": "identifier for an artist at the Visual AIDS Artist+ Registry",
        "data_type": "ExternalId"
    },
    "P8148": {
        "label": "re:publica speaker ID",
        "description": "identifier of a person in the re:publica conference database of speakers",
        "data_type": "ExternalId"
    },
    "P8149": {
        "label": "Provenio ID",
        "description": "identifier of a book owner in the Provenance Database of Book Owners and the Book Provenance Records",
        "data_type": "ExternalId"
    },
    "P7837": {
        "label": "Nederlands Fotomuseum photographer ID",
        "description": "authority control identifier for the Netherlands Fotomuseum",
        "data_type": "ExternalId"
    },
    "P7898": {
        "label": "National Aviation Hall of Fame ID",
        "description": "identifier for a person on the National Aviation Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P8179": {
        "label": "Canadiana NCF ID",
        "description": "new ID used by Canadiana. Format: \"ncf\" followed by 8 digits",
        "data_type": "ExternalId"
    },
    "P8215": {
        "label": "CDAPNC author ID",
        "description": "identifier for an author on the catalogue of the \"Centre de documentation et d'archives\" of the \"Parc national des Cévennes\"",
        "data_type": "ExternalId"
    },
    "P7831": {
        "label": "NMVW id",
        "description": "identifier for persons and organisations at the Dutch museum of world cultures",
        "data_type": "ExternalId"
    },
    "P8007": {
        "label": "US Bureau of Prisons Inmate Register Number",
        "description": "unique number assigned to inmates",
        "data_type": "ExternalId"
    },
    "P7711": {
        "label": "Joconde author ID",
        "description": "identifier for a person in the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P8171": {
        "label": "Priset of Archidiecezja Gdańska ID",
        "data_type": "ExternalId"
    },
    "P8223": {
        "label": "K-Scholar ID",
        "description": "identifier issued by the National Assembly Library of Korea",
        "data_type": "ExternalId"
    },
    "P8290": {
        "label": "Archivio Storico Ricordi person ID",
        "description": "identifier of a person in the Archivio Storico Ricordi digital collection",
        "data_type": "ExternalId"
    },
    "P8292": {
        "label": "AusStage person ID",
        "description": "identifier for a person at AusStage",
        "data_type": "ExternalId"
    },
    "P8116": {
        "label": "Encyclopedia of Brno History person ID",
        "description": "identifier for people related to the city of Brno and its history",
        "data_type": "ExternalId"
    },
    "P8296": {
        "label": "Cinema Context ID",
        "description": "identifier of an item in Cinema Context",
        "data_type": "ExternalId"
    },
    "P8297": {
        "label": "New York Times contributor ID",
        "description": "identifier of an author on the New York Times website",
        "data_type": "ExternalId"
    },
    "P8044": {
        "label": "Frankfurter Personenlexikon ID",
        "description": "identifier for an article in the Frankfurter Personenlexikon",
        "data_type": "ExternalId"
    },
    "P7969": {
        "label": "DAR ancestor ID",
        "description": "ID of someone in the ancestor database of Daughters of the American Revolution",
        "data_type": "ExternalId"
    },
    "P8153": {
        "label": "Accademia delle Scienze di Torino ID",
        "description": "identifier for a member of Accademia delle Scienze di Torino",
        "data_type": "ExternalId"
    },
    "P8036": {
        "label": "m3db.com person ID",
        "description": "identifier for a person in the m3db.com archive",
        "data_type": "ExternalId"
    },
    "P8037": {
        "label": "Filmfront person ID",
        "description": "person identifier in Filmfront, a Norwegian movie database",
        "data_type": "ExternalId"
    },
    "P8143": {
        "label": "SAR ancestor ID",
        "description": "ID of someone in the ancestor database of Sons of the American Revolution (not including the P- prefix)",
        "data_type": "ExternalId"
    },
    "P8163": {
        "label": "Diels-Kranz ID",
        "description": "identifier for an author in the fragment collection Die Fragmente der Vorsokratiker (5th edition)",
        "data_type": "ExternalId"
    },
    "P7848": {
        "label": "Frick Art Reference Library Photoarchive ID",
        "description": "identifier from the Frick Art Reference Library Photoarchive",
        "data_type": "ExternalId"
    },
    "P8125": {
        "label": "Fandango person ID",
        "description": "identifier for a person in the Fandango film database",
        "data_type": "ExternalId"
    },
    "P7902": {
        "label": "Deutsche Biographie ID",
        "description": "identifier for a person in the Deutsche Biographie",
        "data_type": "ExternalId"
    },
    "P8159": {
        "label": "SciProfiles ID",
        "description": "numerical identifier for an author in SciProfiles by MDPI",
        "data_type": "ExternalId"
    },
    "P7769": {
        "label": "Australian National Maritime Museum person ID",
        "description": "identifier for a person in the Australian National Maritime Museum collection",
        "data_type": "ExternalId"
    },
    "P8124": {
        "label": "CPAN author ID",
        "description": "ID of a author in CPAN",
        "data_type": "ExternalId"
    },
    "P8301": {
        "label": "Montreux Jazz Festival person ID",
        "description": "identifier of a person in the Montreux Jazz Festival database",
        "data_type": "ExternalId"
    },
    "P8034": {
        "label": "Vatican Library VcBA ID",
        "description": "authority control for a subject on the website opac.vatlib.it of the Vatican Library. Format 3 digits + \"/\" + 1 to 6 digits; \"/\" is sometimes rendered as \"_\"",
        "data_type": "ExternalId"
    },
    "P8305": {
        "label": "LongTake person ID",
        "description": "identifier for a person on the longtake.it site",
        "data_type": "ExternalId"
    },
    "P7947": {
        "label": "WeChangEd ID",
        "description": "identifier for an item in the WeChangEd database of Women Editors in Europe",
        "data_type": "ExternalId"
    },
    "P8189": {
        "label": "National Library of Israel J9U ID",
        "description": "new identifier used by the National Library of Israel",
        "data_type": "ExternalId"
    },
    "P8219": {
        "label": "ASUT ID",
        "description": "identifier for an entity (person, organization, family) in the Archivi storici dell'Università di Torino",
        "data_type": "ExternalId"
    },
    "P8190": {
        "label": "UK Modern House Index architect ID",
        "description": "identifier for an architect or architectural practice, in the UK Modern House Index",
        "data_type": "ExternalId"
    },
    "P8235": {
        "label": "LiederNet composer ID",
        "description": "composer identifier used by the LiederNet Archive",
        "data_type": "ExternalId"
    },
    "P8273": {
        "label": "GMC registration number",
        "description": "identifier for a physician, in the United Kingdom General Medical Council's official register",
        "data_type": "ExternalId"
    },
    "P7713": {
        "label": "SNESmusic.org composer ID",
        "description": "identifier of a composer in the SNESmusic.org database",
        "data_type": "ExternalId"
    },
    "P7836": {
        "label": "Livelib.ru person ID",
        "description": "identifier for a player at the Livelib.ru",
        "data_type": "ExternalId"
    },
    "P7808": {
        "label": "kino-teatr.ru person ID",
        "description": "identifier for a person at Kino-teatr.ru",
        "data_type": "ExternalId"
    },
    "P7918": {
        "label": "Médias 19 ID",
        "description": "identifier for a 19th century French journalist on the Médias 19 website",
        "data_type": "ExternalId"
    },
    "P8139": {
        "label": "Moravian Lives ID",
        "description": "identifier for a person in the Moravian Lives database",
        "data_type": "ExternalId"
    },
    "P8146": {
        "label": "SkiMo Stats ID",
        "description": "alphabetical identifier for ski mountaineers at skimostats.com",
        "data_type": "ExternalId"
    },
    "P8280": {
        "label": "Modelhub ID",
        "description": "identifier for a performer on Modelhub",
        "data_type": "ExternalId"
    },
    "P7742": {
        "label": "Lithuanian Football Encyclopedia player ID",
        "description": "identifier for football player on site Lithuanian Football Encyclopedia",
        "data_type": "ExternalId"
    },
    "P8096": {
        "label": "Pinakes copyist or possessor ID",
        "description": "identifier for a copyist or possessor of a Greek manuscript",
        "data_type": "ExternalId"
    },
    "P8172": {
        "label": "Sejm-Wielki.pl profile ID",
        "description": "identifier for a person in Sejm-Wielki.pl and Wielcy.pl databases",
        "data_type": "ExternalId"
    },
    "P8281": {
        "label": "Liszt Academy Lexikon person ID",
        "data_type": "ExternalId"
    },
    "P8207": {
        "label": "The Conversation author ID",
        "description": "identifier for contributors to the online news outlet The Conversation",
        "data_type": "ExternalId"
    },
    "P8170": {
        "label": "RPPS ID",
        "description": "identifier for a health professional in the Répertoire partagé des professionnels intervenant dans le système de santé",
        "data_type": "ExternalId"
    },
    "P7934": {
        "label": "Irish playography person ID",
        "description": "identifier for a person, in the 'Irish Playography' database of theatre plays",
        "data_type": "ExternalId"
    },
    "P7798": {
        "label": "BeWeb family ID",
        "description": "identifier for a family, in the Italian Episcopal Conference database",
        "data_type": "ExternalId"
    },
    "P8094": {
        "label": "GeneaStar person ID",
        "description": "identifier of a person in the GeneaStar genealogical database",
        "data_type": "ExternalId"
    },
    "P7804": {
        "label": "dovidka.com.ua ID",
        "description": "identifier for a person at the dovidka.com.ua",
        "data_type": "ExternalId"
    },
    "P7939": {
        "label": "Dansk kvindebiografisk leksikon ID",
        "description": "identifier for a person in the Dansk kvindebiografisk leksikon",
        "data_type": "ExternalId"
    },
    "P8173": {
        "label": "Extratime.ie player ID",
        "description": "identifier for a player at the Extratime.ie",
        "data_type": "ExternalId"
    },
    "P7962": {
        "label": "Dictionnaire des femmes de l’ancienne France ID",
        "description": "identifier for an article in the Dictionnaire des Femmes de l'ancienne France",
        "data_type": "ExternalId"
    },
    "P8176": {
        "label": "Itch.io developer profile",
        "description": "profile on itch.io",
        "data_type": "ExternalId"
    },
    "P8175": {
        "label": "Infames Romani ID",
        "description": "ID of a Roman politician in the Infames Romani database",
        "data_type": "ExternalId"
    },
    "P8022": {
        "label": "RFI Musique artist ID",
        "description": "identifier for an artist or a band on the music-focused website of Radio France Internationale",
        "data_type": "ExternalId"
    },
    "P7880": {
        "label": "Voetbal International player ID",
        "description": "identifier for a player at the Voetbal International",
        "data_type": "ExternalId"
    },
    "P8079": {
        "label": "elibrary.ru person ID",
        "description": "identifier for a person in the elibrary.ru database of science, education, and government organisations in Russia",
        "data_type": "ExternalId"
    },
    "P8251": {
        "label": "CTHS author ID",
        "description": "identifier for an author on the website of the Comité des travaux historiques et scientifiques",
        "data_type": "ExternalId"
    },
    "P8041": {
        "label": "DR music artist ID",
        "description": "identifier for an artist on DR site",
        "data_type": "ExternalId"
    },
    "P8271": {
        "label": "LombardiaBeniCulturali architecture author ID",
        "description": "identifier for an architecture author (not artwork author) in LombardiaBeniCulturali database",
        "data_type": "ExternalId"
    },
    "P7992": {
        "label": "Poeti d'Italia in lingua latina author ID",
        "description": "identifier for an author in the Poeti d'Italia in lingua latina digital library",
        "data_type": "ExternalId"
    },
    "P8314": {
        "label": "Noovie person ID",
        "description": "identifier for a person on the noovie.com site",
        "data_type": "ExternalId"
    },
    "P7954": {
        "label": "RealGM basketball coach ID",
        "description": "identifier for a basketball coach on the RealGM website",
        "data_type": "ExternalId"
    },
    "P8002": {
        "label": "Social Security Death Index entry",
        "description": "identifier for a person listed in Social Security Death Index or Death Master File",
        "data_type": "ExternalId"
    },
    "P8210": {
        "label": "LombardiaBeniCulturali artwork author ID",
        "description": "identifier of an artwork author in the cultural heritage database of the Lombardy Region of Italy",
        "data_type": "ExternalId"
    },
    "P7943": {
        "label": "The Washington Post contributor ID",
        "description": "identifier for a person on the Washington Post website",
        "data_type": "ExternalId"
    },
    "P7748": {
        "label": "NIPS Proceedings author ID",
        "description": "identifier for an author publishing or editor at the NIPS/NeurIPS conference",
        "data_type": "ExternalId"
    },
    "P8134": {
        "label": "Soccerdonna coach ID",
        "description": "identifier for a coach at the Soccerdonna",
        "data_type": "ExternalId"
    },
    "P7796": {
        "label": "BeWeb person ID",
        "description": "identifier for a person, in the Italian Episcopal Conference database",
        "data_type": "ExternalId"
    },
    "P7859": {
        "label": "WorldCat Identities ID",
        "description": "entity on WorldCat. Use P243 OCLC control number for books",
        "data_type": "ExternalId"
    },
    "P8085": {
        "label": "Curran Index contributor ID",
        "description": "identifier for a periodical contributor indexed in the Curran Index",
        "data_type": "ExternalId"
    },
    "P7865": {
        "label": "CoBiS author ID",
        "description": "ID for authors in CoBiS, Coordination of the Special and Specialist Libraries of the Turin Metropolitan Area",
        "data_type": "ExternalId"
    },
    "P8140": {
        "label": "motorsportstats.com driver ID",
        "description": "identifier for a racing driver at motorsportstats.com",
        "data_type": "ExternalId"
    },
    "P8086": {
        "label": "ODMP person ID",
        "description": "The Officer Down Memorial Page entry for a police officer killed in the line of duty",
        "data_type": "ExternalId"
    },
    "P2825": {
        "label": "via",
        "description": "intermediate point on a journey - stopover location, waystation or routing point",
        "data_type": "WikibaseItem"
    },
    "P967": {
        "label": "guest of honor",
        "description": "guest(s) of honor invited to an event, e.g. a convention",
        "data_type": "WikibaseItem"
    },
    "P6759": {
        "label": "NARA record group number",
        "description": "the unique number assigned to a National Archives and Records Administration record group in United States",
        "data_type": "ExternalId"
    },
    "P6763": {
        "label": "Dictionary of Anhui Writers ID",
        "description": "identifier for a writer in ''Dictionary of Anhui Writers'' which was edited by Anhui Writers Association",
        "data_type": "ExternalId"
    },
    "P1903": {
        "label": "volcanic explosivity index",
        "description": "scale indicating size of an explosive volcanic eruption",
        "data_type": "WikibaseItem"
    },
    "P7965": {
        "label": "Scilit work ID",
        "description": "ID of a work in Scilit by MDPI",
        "data_type": "ExternalId"
    },
    "P823": {
        "label": "speaker",
        "description": "person who is speaker for this event, ceremony, keynote, presentation or in a literary work",
        "data_type": "WikibaseItem"
    },
    "P6852": {
        "label": "CCDC Number",
        "description": "identifier for a single crystal structure per the Cambridge Crystallographic Data Centre (CCDC)",
        "data_type": "ExternalId"
    },
    "P8404": {
        "label": "DOI formatter",
        "description": "a formatter string that will create an DOI",
        "data_type": "String"
    },
    "P542": {
        "label": "officially opened by",
        "description": "person that officially opened the event or place",
        "data_type": "WikibaseItem"
    },
    "P1444": {
        "label": "destination point",
        "description": "intended destination for this route (journey, flight, sailing, exploration, migration, etc.)",
        "data_type": "WikibaseItem"
    },
    "P1436": {
        "label": "collection or exhibition size",
        "description": "number of items in a collection or exhibition",
        "data_type": "Quantity"
    },
    "P6761": {
        "label": "NARA collection identifier",
        "description": "the unique identifier assigned to a National Archives and Records Administration collection",
        "data_type": "ExternalId"
    },
    "P2318": {
        "label": "debut participant",
        "description": "participant for whom this is their debut appearance in a series of events",
        "data_type": "WikibaseItem"
    },
    "P8566": {
        "label": "monumenta.ch ID",
        "description": "identifier for a Latin author in the website monumenta.ch",
        "data_type": "ExternalId"
    },
    "P9108": {
        "label": "Erudit article ID",
        "description": "unique identifier for an article in Érudit",
        "data_type": "ExternalId"
    },
    "P1346": {
        "label": "winner",
        "description": "winner of an event or an award; on award items use P166/P1346 on the item for the awarded work instead; do not use for wars or battles",
        "data_type": "WikibaseItem"
    },
    "P1419": {
        "label": "shape",
        "description": "shape of an object",
        "data_type": "WikibaseItem"
    },
    "P1478": {
        "label": "has immediate cause",
        "description": "nearest, proximate thing that directly resulted in the subject as outcome or effect. Used in conjunction with 'has cause' (i.e. underlying cause) and 'has contributing factor'. See 'Help:Modeling causes'.",
        "data_type": "WikibaseItem"
    },
    "P9147": {
        "label": "Union Catalog of Armenian Continuing Resources ID",
        "description": "identifier for a periodical in the Union Catalog of Armenian Continuing Resources, which is part of the National Library of Armenia",
        "data_type": "ExternalId"
    },
    "P6609": {
        "label": "value hierarchy property",
        "description": "property which specifies less precise items than the indicated value for which statements using the subject property would still be true",
        "data_type": "WikibaseProperty"
    },
    "P2127": {
        "label": "International Nuclear Event Scale",
        "data_type": "WikibaseItem"
    },
    "P7662": {
        "label": "Scilit journal ID",
        "description": "identifier of a journal in Scilit (scientific literature) by MDPI (Q6715186)",
        "data_type": "ExternalId"
    },
    "P1110": {
        "label": "attendance",
        "description": "number of people (spectators, ticket holders) at an event or exhibition",
        "data_type": "Quantity"
    },
    "P6847": {
        "label": "abART group ID",
        "description": "identifier of group in czech database abART",
        "data_type": "ExternalId"
    },
    "P6848": {
        "label": "abART institution ID",
        "description": "identifier of institution in czech database abART",
        "data_type": "ExternalId"
    },
    "P1446": {
        "label": "number of missing",
        "description": "number of people missing after an event - qualify with \"point in time (P585)\"",
        "data_type": "Quantity"
    },
    "P8054": {
        "label": "ARK formatter",
        "description": "formatter to generate Archival Resource Key.  Include $1 to be replaced with property value",
        "data_type": "String"
    },
    "P2257": {
        "label": "event interval",
        "description": "standard nominal interval between scheduled or regularly recurring events. Include unit of time, e.g. \"year\" (Q577)",
        "data_type": "Quantity"
    },
    "P8038": {
        "label": "SIUSA archive conservator ID",
        "description": "identifier for an archive conservator in the Unified Information System for the Archival Superintendencies",
        "data_type": "ExternalId"
    },
    "P3238": {
        "label": "trunk prefix",
        "description": "a digit or digits to be dialled before a telephone number to initiate a telephone call for the purpose of selecting an appropriate telecommunications circuit by which the call is to be routed",
        "data_type": "String"
    },
    "P462": {
        "label": "color",
        "description": "color of subject",
        "data_type": "WikibaseItem"
    },
    "P929": {
        "label": "color space",
        "description": "mathematical model describing the way colors can be represented as tuples of numbers",
        "data_type": "WikibaseItem"
    },
    "P1132": {
        "label": "number of participants",
        "description": "number of participants of an event, e.g. people or groups of people that take part in the event (NO units)",
        "data_type": "Quantity"
    },
    "P6854": {
        "label": "Dimensions grant ID",
        "description": "identifier of a scientific/research grant in Digital Science Dimensions, as reflected in SpringerNature SciGraph",
        "data_type": "ExternalId"
    },
    "P1427": {
        "label": "start point",
        "description": "starting place of this journey, flight, voyage, trek, migration etc.",
        "data_type": "WikibaseItem"
    },
    "P1537": {
        "label": "contributing factor of",
        "description": "thing that is significantly influenced by this cause, but does not directly result from it.  See 'Help:Modeling causes' for examples and discussion.",
        "data_type": "WikibaseItem"
    },
    "P1536": {
        "label": "immediate cause of",
        "description": "immediate cause of this effect",
        "data_type": "WikibaseItem"
    },
    "P710": {
        "label": "participant",
        "description": "person, group of people or organization (object) that actively takes/took part in an event or process (subject).  Preferably qualify with \"object has role\" (P3831). Use P1923 for participants that are teams.",
        "data_type": "WikibaseItem"
    },
    "P7118": {
        "label": "AMPAS collections film ID",
        "description": "identifier for a film in Academy of Motion Picture Arts and Sciences (AMPAS) collections catalog",
        "data_type": "ExternalId"
    },
    "P8605": {
        "label": "Linked Open Data Cloud identifier",
        "description": "identifier of dataset used by the Linked Open Data Cloud project",
        "data_type": "ExternalId"
    },
    "P5872": {
        "label": "perfume note",
        "description": "the note of a perfume",
        "data_type": "WikibaseItem"
    },
    "P1561": {
        "label": "number of survivors",
        "description": "number of people surviving an event",
        "data_type": "Quantity"
    },
    "P4733": {
        "label": "produced sound",
        "description": "item for the sound generated by the subject, for instance the cry of an animal",
        "data_type": "WikibaseItem"
    },
    "P533": {
        "label": "target",
        "description": "target of an attack or military operation",
        "data_type": "WikibaseItem"
    },
    "P8606": {
        "label": "Stairway To Hell ID",
        "description": "identifier of a video game cover in the Stairway To Hell database",
        "data_type": "Url"
    },
    "P1591": {
        "label": "defendant",
        "description": "person or organization accused, at a trial",
        "data_type": "WikibaseItem"
    },
    "P832": {
        "label": "public holiday",
        "description": "official public holiday that occurs in this place in its honor, usually a non-working day",
        "data_type": "WikibaseItem"
    },
    "P1840": {
        "label": "investigated by",
        "description": "person or organization involved in investigation of the item",
        "data_type": "WikibaseItem"
    },
    "P3590": {
        "label": "Relations Ontology ID",
        "description": "ID in the Relations Ontology",
        "data_type": "ExternalId"
    },
    "P8543": {
        "label": "Monumentbrowser ID",
        "description": "identifier of a protected monument in the Monumentbrowser database",
        "data_type": "ExternalId"
    },
    "P9195": {
        "label": "LombardiaBeniCulturali archive producer family ID",
        "description": "identifier for an archive producer family in the database LombardiaBeniCulturali",
        "data_type": "ExternalId"
    },
    "P9198": {
        "label": "Encyclopédie berbère article ID",
        "description": "identifier for an article in the French encyclopedia about the Berbers",
        "data_type": "ExternalId"
    },
    "P2667": {
        "label": "corresponding template",
        "description": "the one-value-template that has the same meaning as this property",
        "data_type": "WikibaseItem"
    },
    "P6812": {
        "label": "AntWiki article ID",
        "description": "identifier for an article in AntWiki: a wiki about ants taxa; ant systematists, myrmecologists, and ant collectors; publications, and more",
        "data_type": "ExternalId"
    },
    "P6104": {
        "label": "maintained by WikiProject",
        "description": "WikiProject that maintains this property, item, or linked pages. If the WikiProject is hosted outside of Wikidata, define the scope using the qualifier \"of (P642)\"",
        "data_type": "WikibaseItem"
    },
    "P9115": {
        "label": "Jewish Museum Berlin object ID",
        "description": "object identifier of the Jewish Museum in Berlin",
        "data_type": "ExternalId"
    },
    "P7797": {
        "label": "BeWeb entity ID",
        "description": "identifier for a entity, in the Italian Episcopal Conference database",
        "data_type": "ExternalId"
    },
    "P7896": {
        "label": "Academia.edu publication ID",
        "description": "identifier of a publication at Academia.edu",
        "data_type": "ExternalId"
    },
    "P2320": {
        "label": "aftershocks",
        "description": "number of smaller earthquakes which follow a significant earthquake",
        "data_type": "Quantity"
    },
    "P1479": {
        "label": "has contributing factor",
        "description": "thing that significantly influenced, but did not directly cause, this outcome or effect.  Used in conjunction with 'has cause' and 'has immediate cause'.  See '[[Help:Modeling causes]]'.",
        "data_type": "WikibaseItem"
    },
    "P115": {
        "label": "home venue",
        "description": "home stadium or venue of a sports team or applicable performing arts organization",
        "data_type": "WikibaseItem"
    },
    "P7087": {
        "label": "inverse label item",
        "description": "item with label/aliases of the inverse relationship of a property",
        "data_type": "WikibaseItem"
    },
    "P9193": {
        "label": "LombardiaBeniCulturali archive producer person ID",
        "description": "identifier for an archive producer person in the database LombardiaBeniCulturali",
        "data_type": "ExternalId"
    },
    "P4876": {
        "label": "number of records",
        "description": "number of values of an identifier or records in a catalog",
        "data_type": "Quantity"
    },
    "P9194": {
        "label": "LombardiaBeniCulturali archive producer organization ID",
        "description": "identifier for an archive producer organization in the database LombardiaBeniCulturali",
        "data_type": "ExternalId"
    },
    "P8356": {
        "label": "SIUSA archive producer family ID",
        "description": "identifier for an archive producer family in the Unified Information System for the Archival Superintendencies",
        "data_type": "ExternalId"
    },
    "P8357": {
        "label": "SIUSA archive producer organization ID",
        "description": "identifier for an archive producer organization in the Unified Information System for the Archival Superintendencies",
        "data_type": "ExternalId"
    },
    "P793": {
        "label": "significant event",
        "description": "significant or notable events associated with the subject",
        "data_type": "WikibaseItem"
    },
    "P8142": {
        "label": "motorsportstats.com venue ID",
        "description": "identifier for a motorsport venue at motorsportstats.com",
        "data_type": "ExternalId"
    },
    "P8023": {
        "label": "VideoGamer.com game ID",
        "description": "identifier of a video game in the VideoGamer.com database",
        "data_type": "ExternalId"
    },
    "P7877": {
        "label": "GameStar ID",
        "description": "identifier of a video game in the GameStar database",
        "data_type": "ExternalId"
    },
    "P7901": {
        "label": "Encyclopedia of Chicago ID",
        "description": "identifier for an article in the Encyclopedia of Chicago",
        "data_type": "ExternalId"
    },
    "P7931": {
        "label": "Adelsvapen ID",
        "description": "Swedish noble family on Adelsvapen-Wiki",
        "data_type": "ExternalId"
    },
    "P7995": {
        "label": "NHS Health A to Z ID",
        "description": "identifier for a health condition in the NHS Health A to Z",
        "data_type": "ExternalId"
    },
    "P7996": {
        "label": "EH.Net Encyclopedia of Economic and Business History ID",
        "description": "identifier for an article in the EH.Net Encyclopedia of Economic and Business History",
        "data_type": "ExternalId"
    },
    "P8253": {
        "label": "OSM Name Suggestion Index identifier",
        "description": "identifier for a brand in OpenStreetMap’s Name Suggestion Index",
        "data_type": "ExternalId"
    },
    "P7951": {
        "label": "DC Comics talent ID",
        "description": "identifier for writers and artists on DCComics.com",
        "data_type": "ExternalId"
    },
    "P7886": {
        "label": "Media Arts Database ID",
        "description": "identifier for the Media Arts Database created by the Japanese government department Bunkacho",
        "data_type": "ExternalId"
    },
    "P7923": {
        "label": "Bollywood Hungama person alphabetic ID",
        "description": "identifier of an actor on Bollywood Hungama",
        "data_type": "ExternalId"
    },
    "P7908": {
        "label": "Clavis Clavium ID",
        "description": "identifier for Patristic, Medieval and Byzantine authors and texts in Latin and Greek",
        "data_type": "ExternalId"
    },
    "P7914": {
        "label": "Gamekult platform ID",
        "description": "identifier of a video game platform in the Gamekult database",
        "data_type": "ExternalId"
    },
    "P7925": {
        "label": "Jurisdiction List number",
        "description": "distinct number used to identify a mauza or village in Bangladesh and parts of India",
        "data_type": "ExternalId"
    },
    "P8112": {
        "label": "CSS color keyword",
        "description": "keyword that refers to the element, according to the CSS standard",
        "data_type": "String"
    },
    "P8003": {
        "label": "ColecoVision.dk ID",
        "description": "identifier of a video game in the ColecoVision.dk database",
        "data_type": "ExternalId"
    },
    "P7897": {
        "label": "TI-99/4A Videogame House ID",
        "description": "identifier of a video game in the TI-99/4A Videogame House database",
        "data_type": "ExternalId"
    },
    "P8117": {
        "label": "FooDB compound ID",
        "description": "ID of chemical compound in FooDB",
        "data_type": "ExternalId"
    },
    "P7968": {
        "label": "VR GameCritic ID",
        "description": "identifier of a video game in the VR GameCritic database",
        "data_type": "ExternalId"
    },
    "P8155": {
        "label": "Republic of Korea Parliamentarian Society ID",
        "description": "ID for individuals indexed by the Republic of Korea Parliamentarian Society",
        "data_type": "ExternalId"
    },
    "P8057": {
        "label": "Apache Project ID",
        "description": "identifier of Apache Project",
        "data_type": "ExternalId"
    },
    "P8019": {
        "label": "Kickstarter project ID",
        "description": "identifier for a project on the Kickstarter crowdfunding platform",
        "data_type": "ExternalId"
    },
    "P8029": {
        "label": "PIV Online ID",
        "description": "identifier for an entry in the online version of Plena Ilustrita Vortaro de Esperanto",
        "data_type": "ExternalId"
    },
    "P7993": {
        "label": "Treccani's Dizionario di Filosofia ID",
        "description": "identifier for an item in the Dizionario di Filosofia",
        "data_type": "ExternalId"
    },
    "P7952": {
        "label": "The First Amendment Encyclopedia ID",
        "description": "identifier for an article in the The First Amendment Encyclopedia",
        "data_type": "ExternalId"
    },
    "P8043": {
        "label": "ELAR ID",
        "description": "identifier for a resource pertaining to a language in the namesake endangered language digital repository",
        "data_type": "ExternalId"
    },
    "P7910": {
        "label": "DANFS ship ID",
        "description": "identifier for a ship on the Dictionary of American Naval Fighting Ships website",
        "data_type": "ExternalId"
    },
    "P7919": {
        "label": "SerialStation game ID",
        "description": "identifier of a video game in the SerialStation database",
        "data_type": "ExternalId"
    },
    "P7873": {
        "label": "EFIS film festival ID",
        "description": "identifier for a film festival in the Estonian Film Database",
        "data_type": "ExternalId"
    },
    "P7876": {
        "label": "FlashScore.com team ID",
        "description": "identifier for a sports team in the FlashScore.com database",
        "data_type": "ExternalId"
    },
    "P7913": {
        "label": "Gamekult game ID",
        "description": "identifier of a video game in the Gamekult database",
        "data_type": "ExternalId"
    },
    "P7899": {
        "label": "MOCAGH ID",
        "description": "identifier of a video game in the database of the Museum of Computer Adventure Game History",
        "data_type": "ExternalId"
    },
    "P8162": {
        "label": "DGHS facility code",
        "description": "identifier for a facility indexed by Bangladesh's Directorate General of Health Services",
        "data_type": "ExternalId"
    },
    "P8105": {
        "label": "SAN archive producer ID",
        "description": "identifier for archive producers in the Italian national archive system",
        "data_type": "ExternalId"
    },
    "P7883": {
        "label": "Historical Marker Database ID",
        "description": "database entries of historical markers",
        "data_type": "ExternalId"
    },
    "P7890": {
        "label": "8-bits.info URL",
        "description": "identifier of a video game in the 8-bits.info database",
        "data_type": "Url"
    },
    "P8156": {
        "label": "NKAA ID",
        "description": "identifier for an entry in the Notable Kentucky African Americans Database",
        "data_type": "ExternalId"
    },
    "P7953": {
        "label": "Indiegogo project ID",
        "description": "identifier for a project on the Indiegogo crowdfunding platform",
        "data_type": "ExternalId"
    },
    "P7972": {
        "label": "CVR person ID",
        "description": "unique identifier for a persons (owners/directors) in Denmark's Central Business Register (CVR), the official database of Danish businesses.",
        "data_type": "ExternalId"
    },
    "P7981": {
        "label": "ExoticA ID",
        "description": "identifier in the ExoticA wiki",
        "data_type": "ExternalId"
    },
    "P7946": {
        "label": "Museu de Memes ID",
        "description": "identifier for the Memes Museum of the Brazilian Fluminense Federal University",
        "data_type": "ExternalId"
    },
    "P8231": {
        "label": "Wiener Wohnen ID",
        "description": "identifier of municipal housing objects (Wiener Gemeindebauten) administered by Wiener Wohnen, Vienna, Austria",
        "data_type": "ExternalId"
    },
    "P7950": {
        "label": "Marvel Comics creators ID",
        "description": "identifier for artists and writers at Marvel.com",
        "data_type": "ExternalId"
    },
    "P7915": {
        "label": "GBAtemp game ID",
        "description": "identifier of a video game in the GBAtemp database",
        "data_type": "ExternalId"
    },
    "P7872": {
        "label": "Diccionari del cinema a Catalunya ID",
        "description": "identifier for an item in the Diccionari del cinema a Catalunya",
        "data_type": "ExternalId"
    },
    "P8091": {
        "label": "Archival Resource Key",
        "description": "Archival Resource Key for item or image",
        "data_type": "ExternalId"
    },
    "P8205": {
        "label": "Mozilla Developer Network article",
        "description": "article that describes the item on Mozilla Developer Network website",
        "data_type": "ExternalId"
    },
    "P7982": {
        "label": "Hrvatska enciklopedija ID",
        "description": "identifier for an article in the Croatian Encyclopedia (Hrvatska enciklopedija)",
        "data_type": "ExternalId"
    },
    "P7999": {
        "label": "Haz-Map ID",
        "description": "Identifier for the Haz-Map database, an openly accessible resource on hazardous agents and occupational disease licensed to and published by NCBI/NLM from 2000-2019. It is still accessible and updated, but no longer licensed by NCBI/NLM",
        "data_type": "ExternalId"
    },
    "P8229": {
        "label": "Co-Optimus ID",
        "description": "identifier of a video game in the co-optimus.com database",
        "data_type": "ExternalId"
    },
    "P7892": {
        "label": "EFIS film ID",
        "description": "identifier for film or television series in the Estonian Film Database",
        "data_type": "ExternalId"
    },
    "P7974": {
        "label": "Faculté des sciences de Nancy ID",
        "description": "identifier for an employee of the 'Faculté des sciences de Nancy' (1854-1950)",
        "data_type": "ExternalId"
    },
    "P7889": {
        "label": "Cambridge Encyclopedia of Anthropology ID",
        "description": "identifier for an article in the Cambridge Encyclopedia of Anthropology",
        "data_type": "ExternalId"
    },
    "P7891": {
        "label": "EFIS filmfirm ID",
        "description": "identifier for a filmfirm on Estonian Film Database",
        "data_type": "ExternalId"
    },
    "P7878": {
        "label": "Soccerdonna team ID",
        "description": "identifier for a association football (soccer) team in the Soccerdonna database",
        "data_type": "ExternalId"
    },
    "P7894": {
        "label": "Swedish School Registry ID",
        "description": "Identifier used by Statistics Sweden to identify school units",
        "data_type": "ExternalId"
    },
    "P7957": {
        "label": "Directory of Maîtres d'art",
        "description": "Maître d'art (Master of Art) is a title awarded for life by the French Ministry of Culture",
        "data_type": "ExternalId"
    },
    "P8014": {
        "label": "Compendium heroicum ID",
        "description": "identifier for an article in the Compendium heroicum",
        "data_type": "ExternalId"
    },
    "P8015": {
        "label": "Encyclopedia on Early Childhood Development ID (English)",
        "description": "identifier for an article in English in the Encyclopedia on Early Childhood Development",
        "data_type": "ExternalId"
    },
    "P8016": {
        "label": "Encyclopédie sur le développement des jeunes enfants ID",
        "description": "identifier for an article in French in the Encyclopedia on Early Childhood Development",
        "data_type": "ExternalId"
    },
    "P8168": {
        "label": "FactGrid item ID",
        "description": "identifier for an item in FactGrid",
        "data_type": "ExternalId"
    },
    "P8169": {
        "label": "Heritage Gateway ID",
        "description": "identifier for records on the historic environment in England",
        "data_type": "ExternalId"
    },
    "P8020": {
        "label": "Gram.pl game ID",
        "description": "identifier of a game on Gram.pl site",
        "data_type": "ExternalId"
    },
    "P7927": {
        "label": "Open Food Facts label",
        "description": "identifier of a food label on Open Food Facts",
        "data_type": "ExternalId"
    },
    "P7875": {
        "label": "Eurogamer ID",
        "description": "identifier of a video game in the Eurogamer database",
        "data_type": "ExternalId"
    },
    "P8040": {
        "label": "RPGamer game ID",
        "description": "identifier of a video game in the RPGamer database",
        "data_type": "ExternalId"
    },
    "P8042": {
        "label": "PC Games Database.de game ID",
        "description": "identifier of a video game in the PC Games Database.de",
        "data_type": "ExternalId"
    },
    "P8120": {
        "label": "ModelSEED compound ID",
        "description": "ID of chemical compound in ModelSEED",
        "data_type": "ExternalId"
    },
    "P7893": {
        "label": "Ciência ID",
        "description": "identifier of scientific author in www.cienciavitae.pt",
        "data_type": "ExternalId"
    },
    "P8121": {
        "label": "UM-BBD compound ID",
        "description": "ID for chemical compound in The University of Minnesota Biocatalysis/Biodegradation Database",
        "data_type": "ExternalId"
    },
    "P8178": {
        "label": "motorsportstats.com team ID",
        "description": "identifier for a motorsport team at motorsportstats.com",
        "data_type": "ExternalId"
    },
    "P7929": {
        "label": "Roglo person ID",
        "description": "identifier for a person on the Roglo genealogical database",
        "data_type": "ExternalId"
    },
    "P8066": {
        "label": "Colecovision Addict ID",
        "description": "identifier of a video game in the Colecovision Addict database",
        "data_type": "ExternalId"
    },
    "P8083": {
        "label": "Visit Tuscany ID",
        "description": "identifier for a touristic attraction in Tuscany in the website Visit Tuscany, developed by Regione Toscana",
        "data_type": "ExternalId"
    },
    "P8197": {
        "label": "PragerU presenter ID",
        "description": "ID for a person on PragerU website",
        "data_type": "ExternalId"
    },
    "P7909": {
        "label": "Can You Run it ID",
        "description": "identifier of a video game in the database of Can You Run it?",
        "data_type": "ExternalId"
    },
    "P8144": {
        "label": "Ternopil Encyclopedia ID",
        "description": "identifier for an entry on the official website of the Online Encyclopedia about Ternopil oblast",
        "data_type": "ExternalId"
    },
    "P8135": {
        "label": "Encyclopedia of České Budějovice ID",
        "description": "identifier for article related to the city of České Budějovice",
        "data_type": "ExternalId"
    },
    "P7881": {
        "label": "Games Database game ID",
        "description": "identifier of a video game in the Games Database",
        "data_type": "ExternalId"
    },
    "P7879": {
        "label": "The Video Games Museum game ID",
        "description": "identifier of a video game in the Video Games Museum database",
        "data_type": "ExternalId"
    },
    "P8136": {
        "label": "Il Sole 24 Ore ID",
        "description": "identifier for on the website of Il Sole 24 Ore newspaper",
        "data_type": "ExternalId"
    },
    "P7976": {
        "label": "stargate-wiki.de article",
        "description": "article name on stargate-wiki.d",
        "data_type": "ExternalId"
    },
    "P8137": {
        "label": "Inventory of Archaic and Classical Poleis ID",
        "description": "identifier for an ancient city in the Inventory of Archaic and Classical Poleis",
        "data_type": "ExternalId"
    },
    "P7977": {
        "label": "AncientFaces person ID",
        "description": "ID of someone in AncientFaces",
        "data_type": "ExternalId"
    },
    "P8084": {
        "label": "URL slug on Nintendo eShop",
        "description": "videogame URL slug on Nintendo Game Store",
        "data_type": "ExternalId"
    },
    "P8090": {
        "label": "GameReactor game ID",
        "description": "identifier for a video game on the GameReactor video game news website",
        "data_type": "ExternalId"
    },
    "P8133": {
        "label": "Swedish Glaciers ID",
        "description": "identifier for glaciers in Sweden",
        "data_type": "ExternalId"
    },
    "P8092": {
        "label": "EcuRed article",
        "description": "article at the encyclopedia EcuRed",
        "data_type": "ExternalId"
    },
    "P1771": {
        "label": "Integrated Postsecondary Education Data System ID",
        "description": "identifier for colleges and universities in the United States",
        "data_type": "ExternalId"
    },
    "P2041": {
        "label": "National Gallery of Victoria artist ID",
        "description": "identifier assigned to an artist by the National Gallery of Victoria in Australia",
        "data_type": "ExternalId"
    },
    "P8597": {
        "label": "axle track",
        "description": "distance between the centerline of two roadwheels on the same axle",
        "data_type": "Quantity"
    },
    "P4511": {
        "label": "vertical depth",
        "description": "vertical distance from a horizontal area to a point below. Compare with \"horizontal depth\" (P5524)",
        "data_type": "Quantity"
    },
    "P1167": {
        "label": "USB vendor ID",
        "description": "hexadecimal identifier of a manufacturer of a USB device",
        "data_type": "ExternalId"
    },
    "P2476": {
        "label": "HNI person/institution ID",
        "description": "identifier for a person or organization in the database of Het Nieuwe Instituut",
        "data_type": "ExternalId"
    },
    "P5679": {
        "label": "electrical resistivity",
        "description": "quantifies how strongly a given material opposes the flow of electric current",
        "data_type": "Quantity"
    },
    "P1901": {
        "label": "BALaT person/organisation id",
        "description": "identifier for person or organisation in the People & Institutions database of Belgian Art Links & Tools (BALaT), maintained by KIK-IRPA, Belgium's Royal Institute for Cultural Heritage.",
        "data_type": "ExternalId"
    },
    "P432": {
        "label": "callsign of airline",
        "description": "identifier used in radio transmissions to refer to the subject airline",
        "data_type": "ExternalId"
    },
    "P3039": {
        "label": "wheelbase",
        "description": "distance between centers of front wheel and rear wheel",
        "data_type": "Quantity"
    },
    "P1386": {
        "label": "Japanese High School Code",
        "description": "identifier for a high school in Japan",
        "data_type": "ExternalId"
    },
    "P3070": {
        "label": "dynamic viscosity",
        "data_type": "Quantity"
    },
    "P5141": {
        "label": "coastline",
        "description": "length of the coastline (sea shore) of the region. Use qualifier \"determination method\" (P459) to define how it's determined/which granularity is used.",
        "data_type": "Quantity"
    },
    "P1870": {
        "label": "Name Assigning Authority Number",
        "description": "identifier in the Archival Resource Key registry",
        "data_type": "ExternalId"
    },
    "P6272": {
        "label": "specific rotation",
        "description": "angle through which plane-polarized light is rotated by an optically active substance or a solution of that substance",
        "data_type": "Quantity"
    },
    "P2427": {
        "label": "GRID ID",
        "description": "institutional identifier from the GRID.ac global research identifier database",
        "data_type": "ExternalId"
    },
    "P5608": {
        "label": "fatigue limit",
        "description": "amplitude (or range) of cyclic stress that can be applied to the material without causing fatigue failure",
        "data_type": "Quantity"
    },
    "P2302": {
        "label": "property constraint",
        "description": "constraint applicable to a Wikidata property",
        "data_type": "WikibaseItem"
    },
    "P245": {
        "label": "Union List of Artist Names ID",
        "description": "identifier from the Getty Union List of Artist Names",
        "data_type": "ExternalId"
    },
    "P1278": {
        "label": "Legal Entity Identifier",
        "description": "identifier for a legally distinct entity per ISO 17442",
        "data_type": "ExternalId"
    },
    "P4036": {
        "label": "field of view",
        "description": "extent of the observable world that can be seen or sensed by the item",
        "data_type": "Quantity"
    },
    "P3252": {
        "label": "maximum viable temperature",
        "description": "highest operating temperature where the culturing of micro-organisms and/or enzyme activity occurs",
        "data_type": "Quantity"
    },
    "P2331": {
        "label": "Cycling Archives team ID",
        "description": "identifier in the Cycling Archives",
        "data_type": "ExternalId"
    },
    "P2484": {
        "label": "NCES School ID",
        "description": "identifier for a school in the United States",
        "data_type": "ExternalId"
    },
    "P8628": {
        "label": "engine displacement",
        "description": "measure of the cylinder volume of an engine",
        "data_type": "Quantity"
    },
    "P2306": {
        "label": "property",
        "description": "qualifier to define a property constraint in combination with P2302 (property constraint), or to limit the scope of Q44292881 (wikidata statement)",
        "data_type": "WikibaseProperty"
    },
    "P2166": {
        "label": "SIGIC institution ID",
        "description": "SIGIC - Slovenian Music Information Center (property for institution)",
        "data_type": "ExternalId"
    },
    "P3251": {
        "label": "minimum viable temperature",
        "description": "lowest operating temperature where the culturing of micro-organisms and/or enzyme activity occurs",
        "data_type": "Quantity"
    },
    "P4296": {
        "label": "stellar rotational velocity",
        "description": "measured parameter for stars",
        "data_type": "Quantity"
    },
    "P4163": {
        "label": "magnification",
        "description": "amount of optical magnification provided",
        "data_type": "Quantity"
    },
    "P3253": {
        "label": "optimum viable temperature",
        "description": "the operating temperature where the culturing of micro-organisms and/or enzyme activity thrives best",
        "data_type": "Quantity"
    },
    "P1239": {
        "label": "ISFDB publisher ID",
        "description": "identifier for a publisher in the Internet Speculative Fiction Database",
        "data_type": "ExternalId"
    },
    "P230": {
        "label": "ICAO airline designator",
        "description": "three letter identifier for an airline (two letters only until 1982) (for airports, see P239)",
        "data_type": "String"
    },
    "P2011": {
        "label": "Cooper-Hewitt Person ID",
        "description": "identifier for a person or organization in the Cooper-Hewitt (Smithsonian Institution) catalogue",
        "data_type": "ExternalId"
    },
    "P3041": {
        "label": "luminous intensity",
        "description": "measure of the emitted light intensity",
        "data_type": "Quantity"
    },
    "P2432": {
        "label": "J. Paul Getty Museum artist id",
        "description": "identifier assigned to an artist by the J. Paul Getty Museum",
        "data_type": "ExternalId"
    },
    "P2463": {
        "label": "elibrary.ru organisation ID",
        "description": "identifier for an organization in the elibrary.ru database of science, education, and government organisations in Russia",
        "data_type": "ExternalId"
    },
    "P7297": {
        "label": "cumulative elevation gain",
        "description": "measure of the difference in height between two points on the terrain, when following a track",
        "data_type": "Quantity"
    },
    "P3078": {
        "label": "standard enthalpy of formation",
        "description": "change of enthalpy during the formation of 1 mole of the compound from its constituent elements, with all substances in their standard states at 1 bar and 25°C",
        "data_type": "Quantity"
    },
    "P1155": {
        "label": "Scopus affiliation ID",
        "description": "identifier for an organisation in author affiliations per Scopus",
        "data_type": "ExternalId"
    },
    "P3071": {
        "label": "standard molar entropy",
        "description": "entropy content of one mole of substance under standard conditions (25°C, 1 bar)",
        "data_type": "Quantity"
    },
    "P1961": {
        "label": "CTHS society ID",
        "description": "identifier of organization of Comité des travaux historiques et scientifiques",
        "data_type": "ExternalId"
    },
    "P229": {
        "label": "IATA airline designator",
        "description": "two-character identifier for an airline",
        "data_type": "String"
    },
    "P7015": {
        "label": "surface gravity",
        "description": "gravitational acceleration experienced at the equator of an astronomical body surface",
        "data_type": "Quantity"
    },
    "P1919": {
        "label": "Ministry of Education of Chile school ID",
        "description": "ID number (Rol Base de Datos, RBD, in Spanish, meaning database role) of all schools (high, middle, primary) in Chile, as registered by the Ministry of Education of Chile",
        "data_type": "ExternalId"
    },
    "P8497": {
        "label": "height of center of mass",
        "data_type": "Quantity"
    },
    "P2189": {
        "label": "BiblioNet publisher ID",
        "description": "identifier in the BiblioNet database of publishers",
        "data_type": "ExternalId"
    },
    "P2165": {
        "label": "SIGIC group ID",
        "description": "SIGIC - Slovenian Music Information Center (property for music group)",
        "data_type": "ExternalId"
    },
    "P6876": {
        "label": "solar irradiance",
        "description": "power per unit area received from the Sun in the form of electromagnetic radiation",
        "data_type": "Quantity"
    },
    "P2342": {
        "label": "AGORHA person/institution ID",
        "description": "identifier for a person or institution in the Agorha database (INHA)",
        "data_type": "ExternalId"
    },
    "P5314": {
        "label": "property scope",
        "description": "constraint system qualifier to define the scope of a property",
        "data_type": "WikibaseItem"
    },
    "P1241": {
        "label": "Swiss Football Association club number",
        "description": "identifier",
        "data_type": "ExternalId"
    },
    "P2252": {
        "label": "National Gallery of Art artist ID",
        "description": "identifier assigned to an artist by the National Gallery of Art in Washington, DC",
        "data_type": "ExternalId"
    },
    "P2328": {
        "label": "ProCyclingStats team ID",
        "data_type": "ExternalId"
    },
    "P2253": {
        "label": "DfE URN",
        "description": "Department for Education Unique Reference Number; school identifier used by the UK government",
        "data_type": "ExternalId"
    },
    "P2337": {
        "label": "Swedish Film Database group ID",
        "description": "group identifier in the Swedish Film Database (SFDb)",
        "data_type": "ExternalId"
    },
    "P2174": {
        "label": "Museum of Modern Art artist ID",
        "description": "identifier assigned to an artist by the Museum of Modern Art",
        "data_type": "ExternalId"
    },
    "P8111": {
        "label": "recommended unit of measurement",
        "description": "unit in which a quantity is measured as recommended by a standard (SI, ISO, IEC, etc.)",
        "data_type": "WikibaseItem"
    },
    "P1795": {
        "label": "Smithsonian American Art Museum person/institution ID",
        "description": "identifier for a person or institution, in the Smithsonian American Art Museum: person/institution thesaurus",
        "data_type": "ExternalId"
    },
    "P2267": {
        "label": "PolitiFact people and groups ID",
        "description": "person or group whose statements have been fact checked by PolitiFact.com",
        "data_type": "ExternalId"
    },
    "P4250": {
        "label": "defined daily dose",
        "description": "average maintenance dose per day for a medicine used for its main indication in adults",
        "data_type": "Quantity"
    },
    "P9059": {
        "label": "subdivision of this unit",
        "description": "name of subdivision of this unit/currency",
        "data_type": "WikibaseItem"
    },
    "P1749": {
        "label": "Parlement & Politiek ID",
        "description": "entry of described object on Parlement & Politiek, website describing Dutch politics",
        "data_type": "ExternalId"
    },
    "P9419": {
        "label": "personal library at",
        "description": "institution holding the subject’s personal library",
        "data_type": "WikibaseItem"
    },
    "P355": {
        "label": "subsidiary",
        "description": "subsidiary of a company or organization; generally a fully owned separate corporation. Compare with \"business division\" (P199). Opposite of parent organization (P749).",
        "data_type": "WikibaseItem"
    },
    "P452": {
        "label": "industry",
        "description": "specific industry of company or organization",
        "data_type": "WikibaseItem"
    },
    "P2838": {
        "label": "professional name (Japan)",
        "description": "name passed down by generation assumed by a master of a traditional Japanese art or sport such as kabuki or sumo",
        "data_type": "WikibaseItem"
    },
    "P2389": {
        "label": "organization directed by the office or position",
        "description": "organization which is directed by this office",
        "data_type": "WikibaseItem"
    },
    "P418": {
        "label": "seal description",
        "description": "description of a subject's seal",
        "data_type": "WikibaseItem"
    },
    "P7085": {
        "label": "TikTok username",
        "description": "this item's username on TikTok; do not include the “@” symbol",
        "data_type": "ExternalId"
    },
    "P6087": {
        "label": "coach of sports team",
        "description": "sports club or team for which this person is or was on-field manager or coach",
        "data_type": "WikibaseItem"
    },
    "P163": {
        "label": "flag",
        "description": "subject's flag",
        "data_type": "WikibaseItem"
    },
    "P3902": {
        "label": "had as last meal",
        "description": "components of the last meal had by a person before death",
        "data_type": "WikibaseItem"
    },
    "P1853": {
        "label": "blood type",
        "description": "blood type of the human or animal",
        "data_type": "WikibaseItem"
    },
    "P4411": {
        "label": "Quora username",
        "description": "username of an individual or organisation, on the Quora website",
        "data_type": "ExternalId"
    },
    "P2359": {
        "label": "Roman nomen gentilicium",
        "description": "standard part of the name of a Roman, link to items about the Roman gens only",
        "data_type": "WikibaseItem"
    },
    "P8403": {
        "label": "theorised by",
        "description": "subject who suggested or proposed this concept or invention",
        "data_type": "WikibaseItem"
    },
    "P237": {
        "label": "coat of arms",
        "description": "subject's coat of arms",
        "data_type": "WikibaseItem"
    },
    "P7406": {
        "label": "style of karate",
        "description": "style of karate of the element",
        "data_type": "WikibaseItem"
    },
    "P1884": {
        "label": "hair color",
        "description": "person's hair color. Use P585 as qualifier if there's more than one value",
        "data_type": "WikibaseItem"
    },
    "P8213": {
        "label": "Arlington Cemetery person ID",
        "description": "unofficial identifier for a person buried in Arlington Cemetery (the entry contains biographies and photographs)",
        "data_type": "ExternalId"
    },
    "P2831": {
        "label": "totem",
        "description": "in many indigenous cultures an individual or group has a particular totem (e.g. a type of animal)",
        "data_type": "WikibaseItem"
    },
    "P2462": {
        "label": "member of the deme",
        "description": "name of the deme in Ancient Attica to which the item belonged",
        "data_type": "WikibaseItem"
    },
    "P1950": {
        "label": "second family name in Spanish name",
        "description": "second (generally maternal) family name in Spanish names (do not use for other double barrelled names)",
        "data_type": "WikibaseItem"
    },
    "P6553": {
        "label": "personal pronoun",
        "description": "personal pronoun(s) this person goes by",
        "data_type": "WikibaseLexeme"
    },
    "P7590": {
        "label": "eBay username",
        "description": "item's username on eBay",
        "data_type": "ExternalId"
    },
    "P8822": {
        "label": "epithet",
        "description": "has an epithet (byname), for a human, a character or a deity",
        "data_type": "WikibaseItem"
    },
    "P8274": {
        "label": "living people protection class",
        "description": "when used with living people this project falls under the category described in our Living People policy",
        "data_type": "WikibaseItem"
    },
    "P4013": {
        "label": "Giphy username",
        "description": "this item's username on Giphy",
        "data_type": "ExternalId"
    },
    "P127": {
        "label": "owned by",
        "description": "owner of the subject",
        "data_type": "WikibaseItem"
    },
    "P2218": {
        "label": "net worth",
        "description": "totality of wealth possessed by a person",
        "data_type": "Quantity"
    },
    "P5389": {
        "label": "permanent resident of",
        "description": "country or region where a person has the legal status of permanent resident",
        "data_type": "WikibaseItem"
    },
    "P8413": {
        "label": "academic appointment",
        "description": "this person has been appointed to a role within the given higher education institution or department; distinct from employment or affiliation",
        "data_type": "WikibaseItem"
    },
    "P488": {
        "label": "chairperson",
        "description": "presiding member of an organization, group or body",
        "data_type": "WikibaseItem"
    },
    "P112": {
        "label": "founded by",
        "description": "founder or co-founder of this organization, religion or place",
        "data_type": "WikibaseItem"
    },
    "P3106": {
        "label": "Guardian topic ID",
        "description": "identifier for a topic at the Guardian newspaper website",
        "data_type": "ExternalId"
    },
    "P8832": {
        "label": "PAN member",
        "description": "identifier for a member of the Polish Academy of Sciences",
        "data_type": "ExternalId"
    },
    "P8929": {
        "label": "religion or world view",
        "description": "religious or world view affiliation of a person, organization or religious building",
        "data_type": "WikibaseItem"
    },
    "P8839": {
        "label": "hairstyle",
        "description": "style of cutting, arranging, or combing the hair on the subject's scalp",
        "data_type": "WikibaseItem"
    },
    "P9139": {
        "label": "first family name in Portuguese name",
        "description": "first (generally maternal) family name in Portuguese names",
        "data_type": "WikibaseItem"
    },
    "P3205": {
        "label": "patient of",
        "description": "was treated or studied as a patient by this person",
        "data_type": "WikibaseItem"
    },
    "P2358": {
        "label": "Roman praenomen",
        "description": "standard part of the name of a Roman, link to items for specific Roman praenomen only.",
        "data_type": "WikibaseItem"
    },
    "P8479": {
        "label": "National Registry of Exonerations Case ID",
        "description": "Case ID for an individual listed in a database of people who have been exonerated of crimes in the United States",
        "data_type": "ExternalId"
    },
    "P2650": {
        "label": "interested in",
        "description": "item of special or vested interest to this person or organisation",
        "data_type": "WikibaseItem"
    },
    "P6275": {
        "label": "copyright representative",
        "description": "person or organisation who represents the copyright for this person or work of art",
        "data_type": "WikibaseItem"
    },
    "P159": {
        "label": "headquarters location",
        "description": "city, where an organization's headquarters is or has been situated. Use P276 qualifier for specific building",
        "data_type": "WikibaseItem"
    },
    "P3919": {
        "label": "contributed to creative work",
        "description": "person is cited as contributing to some creative or published work or series (qualify with \"subject has role\", P2868)",
        "data_type": "WikibaseItem"
    },
    "P8852": {
        "label": "facial hair",
        "description": "style of cutting, arranging, or combing the hair on the subject's face (usually on the chin, cheeks, and upper lip region); beard",
        "data_type": "WikibaseItem"
    },
    "P8258": {
        "label": "field of training",
        "description": "profession, position or activity someone learnt during a training, traineeship or apprenticeship",
        "data_type": "WikibaseItem"
    },
    "P3438": {
        "label": "vehicle normally used",
        "description": "vehicle the subject normally uses",
        "data_type": "WikibaseItem"
    },
    "P3091": {
        "label": "mount",
        "description": "creature ridden by the subject, for instance a horse",
        "data_type": "WikibaseItem"
    },
    "P3602": {
        "label": "candidacy in election",
        "description": "election where the subject is a candidate",
        "data_type": "WikibaseItem"
    },
    "P7781": {
        "label": "exonerated of",
        "description": "crime of which the subject was found not to be guilty after having previously been found guilty by a court for that crime",
        "data_type": "WikibaseItem"
    },
    "P4292": {
        "label": "possessed by spirit",
        "description": "item which is spiritually possessing this item",
        "data_type": "WikibaseItem"
    },
    "P2021": {
        "label": "Erdős number",
        "description": "the \"collaborative distance\" between mathematician Paul Erdős and another person. Use point in time (P585) as qualifier and should be used with a source.",
        "data_type": "Quantity"
    },
    "P8006": {
        "label": "footedness",
        "description": "dominant foot or preferred stance of this person",
        "data_type": "WikibaseItem"
    },
    "P3265": {
        "label": "Myspace ID",
        "description": "identifier for a person or organisation, at Myspace",
        "data_type": "ExternalId"
    },
    "P8927": {
        "label": "kunya",
        "description": "teknonym in Arabic names, the name of an adult derived from their eldest child or from their characteristics",
        "data_type": "WikibaseItem"
    },
    "P7383": {
        "label": "name in hiero markup",
        "description": "hieroglyphs written in Wikihiero syntax",
        "data_type": "String"
    },
    "P2366": {
        "label": "Roman agnomen",
        "description": "optional part of the name of a Roman, link to items about specific Roman agnomen only.",
        "data_type": "WikibaseItem"
    },
    "P3833": {
        "label": "diaspora",
        "description": "diaspora that a cultural group belongs to",
        "data_type": "WikibaseItem"
    },
    "P2365": {
        "label": "Roman cognomen",
        "description": "standard part of the name of a Roman, link to items about specific Roman cognomen only.",
        "data_type": "WikibaseItem"
    },
    "P4015": {
        "label": "Vimeo identifier",
        "description": "this item's username or ID on Vimeo",
        "data_type": "ExternalId"
    },
    "P8976": {
        "label": "Lichess username",
        "description": "username on the chess website lichess.org",
        "data_type": "ExternalId"
    },
    "P3716": {
        "label": "social classification",
        "description": "social class as recognized in traditional or state law",
        "data_type": "WikibaseItem"
    },
    "P2561": {
        "label": "name",
        "description": "name the subject is known by. If a more specific property is available, use that",
        "data_type": "Monolingualtext"
    },
    "P8500": {
        "label": "Vietnamese middle name",
        "description": "middle name of a Vietnamese person; use P735 qualified by P1545 for the middle name of a partially westernized name",
        "data_type": "WikibaseItem"
    },
    "P6539": {
        "label": "Catálogo de Patrimonio Cultural de Castilla-La Mancha ID",
        "description": "identifier of a cultural heritage monument in the Catálogo de Patrimonio Cultural de Castilla-La Mancha",
        "data_type": "ExternalId"
    },
    "P8165": {
        "label": "Maryland's National Register Properties ID",
        "description": "numerical identifier for a structure or a building listed in the National Register of Historic Places in the Maryland's National Register Properties database",
        "data_type": "ExternalId"
    },
    "P5956": {
        "label": "War Memorials Online ID",
        "description": "identifier for a war memorial on the War Memorials Online website",
        "data_type": "ExternalId"
    },
    "P7006": {
        "label": "BIC of the Canary Islands ID",
        "description": "identifier of a cultural property in the catalog of the Government of the Canary Islands",
        "data_type": "ExternalId"
    },
    "P7618": {
        "label": "Virginia Landmarks Register ID",
        "description": "identifier for a Virginia Historic Landmark in the Virginia Landmarks Register database",
        "data_type": "ExternalId"
    },
    "P8698": {
        "label": "Turkey's Culture Portal ID",
        "description": "identifier for a heritage subject on the Turkey's Culture Portal website",
        "data_type": "ExternalId"
    },
    "P8588": {
        "label": "Wisconsin National or State Registers Record ID",
        "description": "alphanumerical identifier for a structure or building listed in the National Register of Historic Places in the Wisconsin Historical Society database",
        "data_type": "ExternalId"
    },
    "P8226": {
        "label": "Wyoming SHPO ID",
        "description": "numerical identifier for a structure or a building listed in the National Register of Historic Places in the Wyoming State Historic Preservation Office database",
        "data_type": "ExternalId"
    },
    "P8158": {
        "label": "History Colorado ID",
        "description": "identifier for a historic structure or a building on the History Colorado website",
        "data_type": "ExternalId"
    },
    "P7958": {
        "label": "Chicago Landmarks ID",
        "description": "identifier for a protected property on the Chicago Landmarks website",
        "data_type": "ExternalId"
    },
    "P8515": {
        "label": "Swiss Industrial Heritage ID",
        "data_type": "ExternalId"
    },
    "P9168": {
        "label": "Hauts-de-France Inventory identifier",
        "description": "Gertrude identifier for a cultural property in the Hauts-de-France region",
        "data_type": "ExternalId"
    },
    "P6473": {
        "label": "NMSRCP reference number",
        "description": "identifier in the State Register of Cultural Properties assigned by the New Mexico Historic Preservation Division",
        "data_type": "ExternalId"
    },
    "P8425": {
        "label": "IAPH code",
        "description": "identifier for cultural heritage records of Instituto Andaluz del Patrimonio Histórico",
        "data_type": "ExternalId"
    },
    "P7900": {
        "label": "Denkmalatlas Niedersachsen Objekt-ID",
        "description": "Unique identifier for monuments in Lower Saxony",
        "data_type": "ExternalId"
    },
    "P8325": {
        "label": "Indonesian intangible cultural heritage",
        "description": "registration number of  intangible cultural heritage in Indonesia",
        "data_type": "ExternalId"
    },
    "P8681": {
        "label": "Passion Patrimoine ID",
        "description": "identifier on the website passion-patrimoine.fr",
        "data_type": "ExternalId"
    },
    "P5549": {
        "label": "INEPAC ID",
        "description": "ID for cultural heritage in Rio de Janeiro, Brazil",
        "data_type": "ExternalId"
    },
    "P5704": {
        "label": "Tasmanian Heritage Register ID",
        "description": "Tasmanian heritage place identification number",
        "data_type": "ExternalId"
    },
    "P5556": {
        "label": "KLEKs ID",
        "description": "identifier for a building or landscape element in the KLEKs database",
        "data_type": "ExternalId"
    },
    "P65": {
        "label": "site of astronomical discovery",
        "description": "the place where an astronomical object was discovered (observatory, satellite)",
        "data_type": "WikibaseItem"
    },
    "P8747": {
        "label": "DC Historic Sites place ID",
        "description": "identifier for a Washington, D.C. historic site on the DC Historic Sites website",
        "data_type": "ExternalId"
    },
    "P9154": {
        "label": "Heritage Information System ID in the database of cultural heritage in Austria",
        "description": "identifier published in the Bundesdenkmalamt list of monuments since 2021",
        "data_type": "ExternalId"
    },
    "P9040": {
        "label": "viaggiareinpuglia.it ID",
        "description": "identifier for a heritage or natural site in Apulia (Q1447) on the website viaggiareinpuglia.it",
        "data_type": "ExternalId"
    },
    "P9261": {
        "label": "Oklahoma's NRHP ID",
        "description": "identifier for a structure or a building listed in the National Register of Historic Places in the Oklahoma's National Register of Historic Places database",
        "data_type": "ExternalId"
    },
    "P9175": {
        "label": "Centre-Val de Loire Inventory identifier",
        "description": "Gertrude identifier for a cultural property in the Centre-Val de Loire region",
        "data_type": "ExternalId"
    },
    "P189": {
        "label": "location of discovery",
        "description": "where the item was located when discovered",
        "data_type": "WikibaseItem"
    },
    "P9176": {
        "label": "Occitanie Inventory identifier",
        "description": "Gertrude identifier for a cultural property in the Occitanie region",
        "data_type": "ExternalId"
    },
    "P9357": {
        "label": "Encyclopedia of French Cultural Heritage in North America ID",
        "description": "identifier for an article in the Encyclopedia of French Cultural Heritage in North America",
        "data_type": "ExternalId"
    },
    "P6012": {
        "label": "California Office of Historic Preservation ID",
        "description": "identifier for a point of interest on the website of the Office of Historic Preservation of California",
        "data_type": "ExternalId"
    },
    "P8185": {
        "label": "DoME exhibition ID",
        "description": "identifier of an exhibition in the Database of Modern Exhibitions (1905-1915)",
        "data_type": "ExternalId"
    },
    "P9263": {
        "label": "Provence-Alpes-Côte d'Azur Inventory ID",
        "description": "Gertrude identifier for a cultural property in the Provence-Alpes-Côte d'Azur region",
        "data_type": "ExternalId"
    },
    "P9342": {
        "label": "Bavarian Monument Map object ID (ground monument)",
        "description": "unique identifier for ground monuments in Bavaria, used in the Bavarian Monument Map Bavarian Monument Map, published by Bavarian State Office for Monument Protection",
        "data_type": "ExternalId"
    },
    "P6239": {
        "label": "IEC commemorative monument of Catalonia ID",
        "description": "identifier of a commemorative monument or relevant structure in Catalonia in the Institut d'Estudis Catalans (IEC) [Institute for Catalan Studies] database",
        "data_type": "ExternalId"
    },
    "P2632": {
        "label": "place of detention",
        "description": "place where this person is or was detained",
        "data_type": "WikibaseItem"
    },
    "P9148": {
        "label": "Saxony-Anhalt cultural heritage object ID",
        "description": "Object ID in the cultural heritage register of Saxony-Anhalt",
        "data_type": "ExternalId"
    },
    "P1158": {
        "label": "location of landing",
        "description": "location where the craft landed",
        "data_type": "WikibaseItem"
    },
    "P8232": {
        "label": "Arkansas Register of Historic Places ID",
        "description": "alphabetical identifier for a structure or a building in the Arkansas Register of Historic Places database",
        "data_type": "ExternalId"
    },
    "P8239": {
        "label": "MACRIS ID",
        "description": "identifier for a structure or a building in the Massachusetts Cultural Resource Information System database",
        "data_type": "ExternalId"
    },
    "P8240": {
        "label": "National Register Database, Louisiana ID",
        "description": "numerical identifier for a structure or a building listed in the National Register of Historic Places in the National Register Database, Louisiana",
        "data_type": "ExternalId"
    },
    "P8199": {
        "label": "Anagrafe Istituti Culturali Ecclesiastici ID",
        "description": "identifier for a cultural institution in the database Anagrafe Istituti Culturali Ecclesiastici",
        "data_type": "ExternalId"
    },
    "P8342": {
        "label": "SIUSA archival fonds ID",
        "description": "identifier for an archival fonds in the Unified Information System for the Archival Superintendencies",
        "data_type": "ExternalId"
    },
    "P9050": {
        "label": "ICCD site ID",
        "description": "identifier for a place or an organisation in the General catalog of Italian cultural heritage",
        "data_type": "ExternalId"
    },
    "P5827": {
        "label": "National Trust for Historic Preservation ID",
        "description": "identifier for a place on the National Trust for Historic Preservation website",
        "data_type": "ExternalId"
    },
    "P5942": {
        "label": "Protected objects Ostbelgien ID",
        "description": "identifier of a protected heritage site of the German-speaking Community of Belgium",
        "data_type": "ExternalId"
    },
    "P7135": {
        "label": "Rijksmonument complex ID",
        "description": "identifier for a monument complex assigned by the Rijksdienst voor het Cultureel Erfgoed",
        "data_type": "ExternalId"
    },
    "P8039": {
        "label": "National Register of Historic Monuments in Bosnia and Herzegovina ID",
        "description": "identifier of a National Monument of Bosnia and Herzegovina",
        "data_type": "ExternalId"
    },
    "P6102": {
        "label": "Schleswig-Holstein cultural heritage object ID",
        "description": "Object ID in the cultural heritage register of Schleswig-Holstein",
        "data_type": "ExternalId"
    },
    "P8499": {
        "label": "WISAARD resource ID",
        "description": "numerical identifier for a Washington State structure or a building in the WISAARD database",
        "data_type": "ExternalId"
    },
    "P8492": {
        "label": "Tree of Public Interest ID",
        "description": "case name for monumental trees in Portugal",
        "data_type": "ExternalId"
    },
    "P5957": {
        "label": "ID of Inventory of the Archaeological and Paleontological Heritage of Catalonia",
        "description": "unique identifier for Archaeological and Paleontological Heritage in Catalonia",
        "data_type": "ExternalId"
    },
    "P8132": {
        "label": "Texas Historic Sites Atlas ID",
        "description": "10-digit identifier for a structure or a building in the Texas Historic Sites Atlas",
        "data_type": "ExternalId"
    },
    "P8493": {
        "label": "Denkmaldatenbank Thurgau ID",
        "description": "identifier in the database of protected properties maintained by the Kanton of Thurgau",
        "data_type": "ExternalId"
    },
    "P9207": {
        "label": "Bourgogne-Franche-Comté inventory identifier",
        "description": "Gertrude identifier for a cultural property in the Bourgogne-Franche-Comté region",
        "data_type": "ExternalId"
    },
    "P9208": {
        "label": "Île-de-France inventory identifier",
        "description": "Gertrude identifier for a cultural property in the Île-de-France region",
        "data_type": "ExternalId"
    },
    "P9209": {
        "label": "Normandie inventory identifier",
        "description": "Gertrude identifier for a cultural property in the Normandie region",
        "data_type": "ExternalId"
    },
    "P9210": {
        "label": "Pays de la Loire inventory identifier",
        "description": "Gertrude identifier for a cultural property in the Pays de la Loire region",
        "data_type": "ExternalId"
    },
    "P6117": {
        "label": "Envanter.gov.tr Monument ID",
        "data_type": "ExternalId"
    },
    "P9211": {
        "label": "Wisconsin Architecture and History Inventory ID",
        "description": "alphanumerical identifier for a structure that is part of the heritage inventory of Wisconsin",
        "data_type": "ExternalId"
    },
    "P5694": {
        "label": "Moroccan cultural heritage ID",
        "description": "identifier used for cultural patrimoine in Morocco",
        "data_type": "ExternalId"
    },
    "P5918": {
        "label": "Sprockhoff-ID",
        "description": "number assigned to megalithic installations in northern Germany by Ernst Sprockhoff",
        "data_type": "ExternalId"
    },
    "P6751": {
        "label": "Heritage Gazetteer of Libya ID",
        "description": "identifier for sites in the Heritage Gazetteer of Libya",
        "data_type": "ExternalId"
    },
    "P9142": {
        "label": "IPHAE ID",
        "description": "numerical identifier for a historic building in the Instituto do Patrimônio Histórico e Artístico do Estado do Rio Grande do Sul database",
        "data_type": "ExternalId"
    },
    "P8352": {
        "label": "Veldkruus ID",
        "description": "identifier of an object in the Veldkruus database",
        "data_type": "ExternalId"
    },
    "P8177": {
        "label": "Oregon Historic Sites Database ID",
        "description": "numerical identifier for a structure or a building in the Oregon Historic Sites Database",
        "data_type": "ExternalId"
    },
    "P8249": {
        "label": "MDAH's HRI ID",
        "description": "numerical identifier for a structure or a building of interest in the Mississippi Department of Archives and History's Historic Resources Inventory database",
        "data_type": "ExternalId"
    },
    "P8250": {
        "label": "RI National Register Search ID",
        "description": "numerical identifier for a structure or a building listed in the National Register of Historic Places in the Rhode Island-National Register Search database",
        "data_type": "ExternalId"
    },
    "P8514": {
        "label": "TOPCMB ID",
        "description": "unique identifier for a entity Tesauro de Objetos do Patrimônio Cultural nos Museus Brasileiros website",
        "data_type": "ExternalId"
    },
    "P9369": {
        "label": "Fortalezas.org ID",
        "description": "identifier for a fortification in Fortalezas.org",
        "data_type": "ExternalId"
    },
    "P9339": {
        "label": "Bavarian Monument Map object ID (architectural monument)",
        "description": "identifier for an architectural monument in Bavaria, used in the Bavarian Monument Map, published by Bavarian State Office for Monument Protection",
        "data_type": "ExternalId"
    },
    "P8723": {
        "label": "American Heritage place ID",
        "description": "identifier for an American historic site on the American Heritage website",
        "data_type": "ExternalId"
    },
    "P8316": {
        "label": "sobory.ru ID",
        "description": "identifier of a church/monastery/chapel at sobory.ru website",
        "data_type": "ExternalId"
    },
    "P9292": {
        "label": "Encyclopédie du patrimoine culturel de l'Amérique française ID",
        "description": "identifier for an article in the Encyclopédie du patrimoine culturel de l'Amérique française",
        "data_type": "ExternalId"
    },
    "P9257": {
        "label": "CPC-PR ID",
        "description": "Coordenação do Patrimônio Cultural do Paraná identifier for a historic building",
        "data_type": "ExternalId"
    },
    "P8282": {
        "label": "Medusa NRHP ID",
        "description": "numerical identifier for a Maryland structure or building listed in the National Register of Historic Places in the Medusa database",
        "data_type": "ExternalId"
    },
    "P5525": {
        "label": "CONDEPHAAT ID",
        "description": "Conselho de Defesa do Patrimônio Histórico identifier for monuments in São Paulo, Brazil",
        "data_type": "ExternalId"
    },
    "P7170": {
        "label": "register of real estate NKP",
        "description": "identifier of this item in the Národná kultúrna pamiatka Register (NKP, National cultural monument)",
        "data_type": "ExternalId"
    },
    "P3557": {
        "label": "World Curling Federation ID",
        "description": "ID for a person in the World Curling Federation database",
        "data_type": "ExternalId"
    },
    "P3276": {
        "label": "Angel List ID",
        "description": "profile of a person or organisation on Angel List",
        "data_type": "ExternalId"
    },
    "P3363": {
        "label": "Tennis Hall of Fame player ID",
        "description": "International Tennis Hall of Fame",
        "data_type": "ExternalId"
    },
    "P3527": {
        "label": "Eurobasket.com player ID",
        "description": "ID for a basketball player at eurobasket.com",
        "data_type": "ExternalId"
    },
    "P3533": {
        "label": "DraftExpress ID",
        "description": "identifier for a basketball player at draftexpress.com",
        "data_type": "ExternalId"
    },
    "P3564": {
        "label": "Global Poker Index ID",
        "description": "ID for a poker player in Global Poker Index",
        "data_type": "ExternalId"
    },
    "P3368": {
        "label": "Prabook ID",
        "description": "identifier of a person in the Prabook database",
        "data_type": "ExternalId"
    },
    "P3579": {
        "label": "Sina Weibo user ID",
        "description": "Sina Weibo user's identifier",
        "data_type": "ExternalId"
    },
    "P3410": {
        "label": "Clergy of the Church of England database ID",
        "description": "identifier for people in the Church of England database, covering English clergy from 1540–1835",
        "data_type": "ExternalId"
    },
    "P3411": {
        "label": "Saxon Academy of Sciences member ID",
        "description": "identifier in the members' database of the Saxon Academy of Sciences",
        "data_type": "ExternalId"
    },
    "P3385": {
        "label": "Japan Sumo Association ID",
        "description": "identifier of a person in the Japan Sumo Association database",
        "data_type": "ExternalId"
    },
    "P3379": {
        "label": "Model Manual ID",
        "description": "identifier for a fashion model in the 'New York' magazine database",
        "data_type": "ExternalId"
    },
    "P3546": {
        "label": "AustralianFootball.com ID",
        "description": "identifier for an Australian rules footballer at the AustralianFootball.com website",
        "data_type": "ExternalId"
    },
    "P3598": {
        "label": "Hockey-Reference.com player ID",
        "description": "identifier for ice hockey players at Hockey-Reference.com",
        "data_type": "ExternalId"
    },
    "P3388": {
        "label": "LittleSis people ID",
        "description": "entry in the LittleSis who-knows-who of government and business database",
        "data_type": "ExternalId"
    },
    "P3574": {
        "label": "FanGraphs player ID",
        "description": "ID for a baseball player in FanGraphs.com",
        "data_type": "ExternalId"
    },
    "P3315": {
        "label": "chesstempo ID",
        "description": "identifier for chess players at chesstempo.com",
        "data_type": "ExternalId"
    },
    "P3588": {
        "label": "WNBA player ID",
        "description": "identifier for a player on the Women's National Basketball Association's website",
        "data_type": "ExternalId"
    },
    "P3389": {
        "label": "Royal Swedish Academy of Letters member ID",
        "description": "identifier in the list of members of the Royal Swedish Academy of Letters, History and Antiquities",
        "data_type": "ExternalId"
    },
    "P3392": {
        "label": "Surman ID",
        "description": "identifier of a record, in the Surman Index of British nonconformist ministers",
        "data_type": "ExternalId"
    },
    "P3305": {
        "label": "KINENOTE person ID",
        "description": "identifier of a person in the KINENOTE movie database",
        "data_type": "ExternalId"
    },
    "P3526": {
        "label": "wisden.com ID",
        "description": "ID for a cricket player at wisden.com",
        "data_type": "ExternalId"
    },
    "P3299": {
        "label": "student register of the University of Helsinki ID (1640–1852)",
        "description": "ID in the online student register of the University of Helsinki 1640–1852. Format: numbers below ca. 20000, see P3325 for higher numbers",
        "data_type": "ExternalId"
    },
    "P3429": {
        "label": "Electronic Enlightenment ID",
        "description": "identifier in the Electronic Enlightenment database",
        "data_type": "ExternalId"
    },
    "P3446": {
        "label": "FIDAL athlete ID",
        "description": "identifier for athletes in the Italian Athletics Federation (FIDAL) database and website",
        "data_type": "ExternalId"
    },
    "P3351": {
        "label": "Adult Film Database actor ID",
        "description": "ID for an actor in the Adult Film Database",
        "data_type": "ExternalId"
    },
    "P3341": {
        "label": "Kvikmyndir person ID",
        "description": "identifier of a person in the Icelandic film database kvikmyndir.is",
        "data_type": "ExternalId"
    },
    "P3413": {
        "label": "Leopoldina member ID",
        "description": "identifier in the members' database of the Leopoldina – German Academy of Sciences",
        "data_type": "ExternalId"
    },
    "P3366": {
        "label": "GECD person ID",
        "description": "identifier of a person in the German Early Cinema Database",
        "data_type": "ExternalId"
    },
    "P3191": {
        "label": "IMIS person ID",
        "description": "identifier for a person in IMIS, database of Flanders Marine Institute",
        "data_type": "ExternalId"
    },
    "P3325": {
        "label": "student register of the University of Helsinki ID (1853–1899)",
        "description": "ID in the online student register of the University of Helsinki 1853–1899. Format: numbers above ca. 20000, see P3299 for lower numbers",
        "data_type": "ExternalId"
    },
    "P3188": {
        "label": "Nobel prize ID",
        "description": "ID in the Nobel prize organization homepage",
        "data_type": "ExternalId"
    },
    "P3314": {
        "label": "365chess player ID",
        "description": "identifier for players at 365chess.com",
        "data_type": "ExternalId"
    },
    "P3316": {
        "label": "ICCF player ID",
        "description": "identifier for a chess player, at iccf.com",
        "data_type": "ExternalId"
    },
    "P3330": {
        "label": "Supermodels.nl ID",
        "description": "ID of a fashion model on the Supermodels.nl website",
        "data_type": "ExternalId"
    },
    "P3568": {
        "label": "Official World Golf Ranking player ID",
        "description": "identifier for a golf player, in the Official World Golf Ranking database",
        "data_type": "ExternalId"
    },
    "P3237": {
        "label": "KU Leuven person ID",
        "description": "identifier for a person in the Who's Who database of the Catholic University of Leuven",
        "data_type": "ExternalId"
    },
    "P3142": {
        "label": "EDb person ID",
        "description": "identifier for a person at the Israeli movie database EDb",
        "data_type": "ExternalId"
    },
    "P3339": {
        "label": "World Surf League ID",
        "description": "identifier for a surfer on the World Surf League website",
        "data_type": "ExternalId"
    },
    "P3541": {
        "label": "MLB.com player ID",
        "description": "identifier for a Major League Baseball player",
        "data_type": "ExternalId"
    },
    "P3346": {
        "label": "HKMDB person ID",
        "description": "identifier for a person at the HKMDB (Hong Kong Movie Database) website",
        "data_type": "ExternalId"
    },
    "P3360": {
        "label": "Nobel Prize People Nomination ID",
        "description": "identifier for a person who nominated someone, or who was nominated, for a Nobel Prize",
        "data_type": "ExternalId"
    },
    "P3361": {
        "label": "PictoRight ID code",
        "description": "code to identify members of the Dutch collective rights management organisation PictoRight and sister organisations worldwide",
        "data_type": "ExternalId"
    },
    "P3159": {
        "label": "UGentMemorialis ID",
        "description": "identifier for a professor at the Ghent University",
        "data_type": "ExternalId"
    },
    "P3445": {
        "label": "Cinema of Israel ID",
        "description": "Identifier number in \"Cinema of Israel\" website",
        "data_type": "ExternalId"
    },
    "P3187": {
        "label": "Marburger Professorenkatalog ID",
        "description": "identifier for a professor, in the Marburger Professorenkatalog of professors at the Philipps-Universität Marburg, Germany",
        "data_type": "ExternalId"
    },
    "P3535": {
        "label": "Japan Golf Tour player ID",
        "description": "identifier for a golf player, in the Japan Golf Tour database",
        "data_type": "ExternalId"
    },
    "P3521": {
        "label": "European Tour golf player ID",
        "description": "identifier for a golf player, in the European Tour database",
        "data_type": "ExternalId"
    },
    "P3582": {
        "label": "Sunshine Tour golf player ID",
        "description": "identifier for a golf player, in the Sunshine Tour database",
        "data_type": "ExternalId"
    },
    "P3506": {
        "label": "Luding designer ID",
        "description": "identifier for a game designer at the Luding database",
        "data_type": "ExternalId"
    },
    "P3283": {
        "label": "Bandcamp ID",
        "description": "identifier for an artist (person or group) or record label, in the Bandcamp database",
        "data_type": "ExternalId"
    },
    "P3280": {
        "label": "BAnQ author ID",
        "description": "identifier for an author or subject heading in the Bibliothèque et Archives nationales du Québec (BAnQ)",
        "data_type": "ExternalId"
    },
    "P3409": {
        "label": "Catalogus Professorum Lipsiensis ID",
        "description": "biographical entry in the University of Leipzig catalog of professors",
        "data_type": "ExternalId"
    },
    "P3269": {
        "label": "Fotografen.nl ID",
        "description": "identifier for a photographer, in the Dutch fotografen.nl database",
        "data_type": "ExternalId"
    },
    "P3520": {
        "label": "databaseOlympics.com athlete ID",
        "description": "identifier for an Olympic athlete at databaseOlympics.com",
        "data_type": "ExternalId"
    },
    "P3537": {
        "label": "FootballDatabase.eu player ID",
        "description": "identifier for a association football player at FootballDatabase.eu",
        "data_type": "ExternalId"
    },
    "P3558": {
        "label": "USCF ID",
        "description": "identifier for a chess player issued by the US Chess Federation",
        "data_type": "ExternalId"
    },
    "P3146": {
        "label": "CiNetMag person ID",
        "description": "identifier for a person at the Iranian movie database CiNetMag",
        "data_type": "ExternalId"
    },
    "P3144": {
        "label": "elFilm person ID",
        "description": "identifier for a person at the Egypt movie database elFilm",
        "data_type": "ExternalId"
    },
    "P3397": {
        "label": "Guide of the French Church person ID",
        "description": "identifier of a person in the guide of the French Catholic Church, edited by the Bishops' Conference of France",
        "data_type": "ExternalId"
    },
    "P3560": {
        "label": "College Football Data Warehouse ID",
        "description": "ID at College Football Data Warehouse",
        "data_type": "ExternalId"
    },
    "P3561": {
        "label": "Pro-Football-Reference.com player ID",
        "description": "identifier for American football players at Pro-Football-Reference.com",
        "data_type": "ExternalId"
    },
    "P3554": {
        "label": "World Series of Poker ID",
        "description": "ID of a poker player in World Series of Poker",
        "data_type": "ExternalId"
    },
    "P3573": {
        "label": "European Handball Federation player ID",
        "description": "id for players at eurohandball.com, the website of the European Handball Federation (EHF)",
        "data_type": "ExternalId"
    },
    "P3565": {
        "label": "J.League player ID",
        "description": "identifier for a player at website of J.League, Japan's professional association football (soccer) league",
        "data_type": "ExternalId"
    },
    "P3577": {
        "label": "Croatian Football Federation player ID",
        "description": "ID of player at Croatian Football Federation website",
        "data_type": "ExternalId"
    },
    "P3584": {
        "label": "ITRA runner ID",
        "description": "identifier for a runner on the International Trail-Running Association's website",
        "data_type": "ExternalId"
    },
    "P3408": {
        "label": "FINA athlete ID (former scheme)",
        "description": "identifier for an athlete on the International Swimming Federation website",
        "data_type": "ExternalId"
    },
    "P3430": {
        "label": "SNAC Ark ID",
        "description": "identifier for items in the Social Networks and Archival Context system",
        "data_type": "ExternalId"
    },
    "P3427": {
        "label": "AS.com athlete ID",
        "description": "identifier for an athlete on AS.com",
        "data_type": "ExternalId"
    },
    "P3576": {
        "label": "TLG author ID",
        "description": "author identifier in the TLG Canon of Greek Authors and Works (third edition)",
        "data_type": "ExternalId"
    },
    "P3547": {
        "label": "AFL Tables player ID",
        "description": "ID for a VFL/AFL player at afltables.com",
        "data_type": "ExternalId"
    },
    "P3204": {
        "label": "Ciné-Ressources person ID",
        "description": "page of a person on the website \"Ciné-Ressources\"",
        "data_type": "ExternalId"
    },
    "P3192": {
        "label": "Last.fm ID",
        "description": "identifier for an artist, group or work, on Last.fm",
        "data_type": "ExternalId"
    },
    "P3162": {
        "label": "Rock and Roll Hall of Fame ID",
        "description": "identifier for an inductee (individual or group), in the US Rock & Roll Hall of Fame",
        "data_type": "ExternalId"
    },
    "P3556": {
        "label": "CurlingZone ID",
        "description": "ID on the CurlingZone database",
        "data_type": "ExternalId"
    },
    "P3145": {
        "label": "Sratim ID",
        "description": "identifier for a movie or person at the Israeli movie database Sratim (sratim.co.il)",
        "data_type": "ExternalId"
    },
    "P3566": {
        "label": "Just Sports Stats ID",
        "description": "ID for American or Canadian football player in www.justsportsstats.com",
        "data_type": "ExternalId"
    },
    "P3542": {
        "label": "FIBA player ID",
        "description": "identifier for a basketball player at fiba.com",
        "data_type": "ExternalId"
    },
    "P3286": {
        "label": "Squash Info player ID",
        "description": "identifier on the website squashinfo.com",
        "data_type": "ExternalId"
    },
    "P3284": {
        "label": "Yahoo! Japan Talent Database ID",
        "description": "Identifier of person in Yahoo! Japan Talent database",
        "data_type": "ExternalId"
    },
    "P3571": {
        "label": "ESPN.com MLB player ID",
        "description": "ID for a Major League Baseball player at ESPN.com",
        "data_type": "ExternalId"
    },
    "P3581": {
        "label": "Pro Football Hall of Fame ID (old)",
        "description": "identifier for American football players in the Pro Football Hall of Fame",
        "data_type": "ExternalId"
    },
    "P3226": {
        "label": "HAS member ID",
        "description": "ID of the data-sheet of members of the Hungarian Academy of Sciences (Q265058)",
        "data_type": "ExternalId"
    },
    "P3171": {
        "label": "International Olympic Committee athlete ID",
        "description": "identifier for a person at Olympic.org, the official website of the International Olympic Committee (IOC)",
        "data_type": "ExternalId"
    },
    "P3172": {
        "label": "World Bridge Federation ID",
        "description": "identifier for a bridge player, issued by the World Bridge Federation",
        "data_type": "ExternalId"
    },
    "P3505": {
        "label": "BoardGameGeek designer ID",
        "description": "ID for a game designer at BoardGameGeek",
        "data_type": "ExternalId"
    },
    "P3525": {
        "label": "ACB.com player ID",
        "description": "ID for a basketball player at ACB.com",
        "data_type": "ExternalId"
    },
    "P3567": {
        "label": "HHOF.com NHL player ID",
        "description": "identifier of an NHL ice hockey player at the Hockey Hall of Fame website (HHOF.com)",
        "data_type": "ExternalId"
    },
    "P3585": {
        "label": "UltraSignup runner ID",
        "description": "identifier for a runner on UltraSignup's website",
        "data_type": "ExternalId"
    },
    "P3532": {
        "label": "databaseFootball.com ID",
        "description": "ID for an American football player in databasefootball.com",
        "data_type": "ExternalId"
    },
    "P3522": {
        "label": "NHL.com player ID",
        "description": "identifier for an ice hockey player at NHL.com",
        "data_type": "ExternalId"
    },
    "P3539": {
        "label": "NFL.com ID (former scheme)",
        "description": "numeric identifier for American football players at NFL.com",
        "data_type": "ExternalId"
    },
    "P3536": {
        "label": "euroleague.net ID",
        "description": "identifier for a basketball player at euroleague.net",
        "data_type": "ExternalId"
    },
    "P3233": {
        "label": "PhilPeople profile",
        "description": "an identifier for user profiles on PhilPeople",
        "data_type": "ExternalId"
    },
    "P3232": {
        "label": "PhilPapers author ID",
        "description": "identifier for authors of papers on PhilPapers",
        "data_type": "ExternalId"
    },
    "P8730": {
        "label": "National Park Service people ID",
        "description": "identifier for a person on the website of the National Park Service, an American federal agency",
        "data_type": "ExternalId"
    },
    "P8638": {
        "label": "British and Irish Furniture Makers Online ID",
        "description": "identifier for a person or company, in the British and Irish Furniture Makers Online (BIFMO) database",
        "data_type": "ExternalId"
    },
    "P8612": {
        "label": "past Fellow of the Royal Society ID",
        "description": "identifier for past Fellows (deceased) of the Royal Society",
        "data_type": "ExternalId"
    },
    "P8877": {
        "label": "Fellow of the Royal Society of Canada ID",
        "description": "identifier of a fellow of the Royal Society of Canada",
        "data_type": "ExternalId"
    },
    "P8772": {
        "label": "Blogger user profile ID",
        "description": "user profile identifier on the Blogger website",
        "data_type": "ExternalId"
    },
    "P8547": {
        "label": "BHF author ID",
        "description": "identifier for an author on the online Bibliographie de l'histoire de France",
        "data_type": "ExternalId"
    },
    "P8802": {
        "label": "MSRI person ID",
        "description": "identifier for a person on the Mathematical Sciences Research Institute website",
        "data_type": "ExternalId"
    },
    "P8618": {
        "label": "ITF player ID 2020",
        "description": "identifier for a tennis player at the International Tennis Federation (ITF) website as of 2020",
        "data_type": "ExternalId"
    },
    "P8511": {
        "label": "Fototeka person ID",
        "description": "identifier of a person in Fototeka (Polish National Film Archive)",
        "data_type": "ExternalId"
    },
    "P8982": {
        "label": "Pontificio Istituto di Archeologia Cristiana ID",
        "description": "numerical identifier for a person in the catalogue of the Library of the Pontifical Institute of Christian Archaeology (PIAC)",
        "data_type": "ExternalId"
    },
    "P8805": {
        "label": "LIMIS person ID",
        "description": "identifier for a person in the Lithuanian Integral Museum Information System Personalities Dictionary",
        "data_type": "ExternalId"
    },
    "P8977": {
        "label": "CITWF person ID",
        "description": "identifier for a person in the Complete Index to World Film",
        "data_type": "ExternalId"
    },
    "P8317": {
        "label": "Philadelphia Museum of Art entity ID",
        "description": "identifier for a person or organization in the Philadelphia Museum of Art",
        "data_type": "ExternalId"
    },
    "P8880": {
        "label": "AstroGen ID",
        "description": "identifier for a researcher in AstroGen",
        "data_type": "ExternalId"
    },
    "P8818": {
        "label": "Church of Jesus Christ pioneer ID",
        "description": "ID of a missionary in Church of Jesus Christ pioneer database",
        "data_type": "ExternalId"
    },
    "P8821": {
        "label": "Memorial Book for the Victims of National Socialism ID",
        "description": "identifier for a person on the Memorial Book for the Victims of National Socialism at the University of Vienna in 1938 website",
        "data_type": "ExternalId"
    },
    "P8743": {
        "label": "Scholars Strategy Network ID",
        "description": "identifier for a person in the Scholars Strategy Network",
        "data_type": "ExternalId"
    },
    "P8939": {
        "label": "Google Play developer slug",
        "description": "human-readable identifier for an developer on the Google Play Store",
        "data_type": "ExternalId"
    },
    "P8523": {
        "label": "Présence Compositrices composer ID",
        "description": "identifier of a female composer on the website Présence Compositrices",
        "data_type": "ExternalId"
    },
    "P8773": {
        "label": "Malmö Museer ID",
        "description": "identifier for person or organization connected to an object at Malmö Museer",
        "data_type": "ExternalId"
    },
    "P8534": {
        "label": "SPLC Individual ID",
        "description": "identifier for individuals that are on file with the Southern Poverty Law Center",
        "data_type": "ExternalId"
    },
    "P8969": {
        "label": "CINE21 person ID",
        "description": "identifier for a person in the database of the film magazine CINE21",
        "data_type": "ExternalId"
    },
    "P8961": {
        "label": "tribunsdelaplebe.fr ID",
        "description": "identifier of a tribune of the plebs and his gens prior to the Hortensian plebiscite on the site tribunsdelaplebe.fr",
        "data_type": "ExternalId"
    },
    "P8844": {
        "label": "Absolute Games person ID",
        "description": "identifier for a person at the Absolute Games website",
        "data_type": "ExternalId"
    },
    "P8848": {
        "label": "CONOR.AL ID",
        "description": "authority control ID in CONOR.AL (authority control of Albanian libraries)",
        "data_type": "ExternalId"
    },
    "P8849": {
        "label": "CONOR.BG ID",
        "description": "authority control ID in CONOR.BG (authority control of Bulgarian libraries)",
        "data_type": "ExternalId"
    },
    "P8850": {
        "label": "CONOR.KS ID",
        "description": "authority control ID in CONOR.KS (authority control of Kosovan libraries)",
        "data_type": "ExternalId"
    },
    "P8851": {
        "label": "CONOR.SR ID",
        "description": "authority control ID in CONOR.SR (authority control of Serbian libraries)",
        "data_type": "ExternalId"
    },
    "P8578": {
        "label": "TracesOfWar person ID",
        "description": "identifier for historical persons associated with post-18th-century wars",
        "data_type": "ExternalId"
    },
    "P8643": {
        "label": "Dostoyevsky and His Entourage ID",
        "description": "identifier for a person somehow connected to Fyodor Dostoyevsky in the website in fedordostoevsky.ru",
        "data_type": "ExternalId"
    },
    "P8580": {
        "label": "NHK Archives Portal person ID",
        "description": "identifier for a person on the NHK Archives Portal website",
        "data_type": "ExternalId"
    },
    "P8914": {
        "label": "ZOBODAT person ID",
        "description": "numerical identifier for a person in ZOBODAT, a bibliographic database of mostly German-language works on zoology",
        "data_type": "ExternalId"
    },
    "P8604": {
        "label": "OnlyFans ID",
        "description": "identifier for a person on the OnlyFans website",
        "data_type": "ExternalId"
    },
    "P8980": {
        "label": "KANTO ID",
        "description": "identifier for a person or an organization in the database KANTO curated by the National Library of Finland",
        "data_type": "ExternalId"
    },
    "P8941": {
        "label": "The Galileo Project ID",
        "description": "numerical identifier for a person in the online database The Galileo Project",
        "data_type": "ExternalId"
    },
    "P8446": {
        "label": "Gateway to Research person ID",
        "description": "identifier for a person in the UK Research and Innovation (UKRI)'s Gateway to Research (GtR) database of research funding",
        "data_type": "ExternalId"
    },
    "P8748": {
        "label": "Rheinland-Pfälzische Personendatenbank ID",
        "description": "static identifier (\"Zitierlink\") of an entry in Rheinland-Pfälzische Personendatenbank (the \"recnums\" is not stable!)",
        "data_type": "ExternalId"
    },
    "P8358": {
        "label": "Lexicon of the Yiddish Theatre ID",
        "description": "identifier for a biography in the Lexicon of the Yiddish Theatre",
        "data_type": "ExternalId"
    },
    "P8853": {
        "label": "Musica Brasilis composer ID",
        "description": "identifier for a composer on the Musica Brasilis website",
        "data_type": "ExternalId"
    },
    "P8971": {
        "label": "Cinémathèque québécoise person ID",
        "description": "identifier for a person in the website Cinémathèque québécoise",
        "data_type": "ExternalId"
    },
    "P8689": {
        "label": "ITHL author ID",
        "description": "identifier for an author in the website of the Institute for the Translation of Hebrew Literature (ITHL)",
        "data_type": "ExternalId"
    },
    "P8780": {
        "label": "Portuguese Football Federation ID",
        "description": "identifier for an association football (soccer) player in the Portuguese Football Federation database",
        "data_type": "ExternalId"
    },
    "P8783": {
        "label": "AAGM people ID",
        "description": "identifier for a person (artist, architect, sitter, benefactor etc) in the website database of Aberdeen Art Galleries and Museums (AAGM)",
        "data_type": "ExternalId"
    },
    "P8829": {
        "label": "Swiss-Gym Male ID",
        "description": "identifier of male Swiss gymnast on Swiss-Gym",
        "data_type": "ExternalId"
    },
    "P8830": {
        "label": "Swiss-Gym Female ID",
        "description": "identifier of female Swiss gymnast on Swiss-Gym",
        "data_type": "ExternalId"
    },
    "P8440": {
        "label": "Personendatenbank Germania Sacra ID",
        "description": "entry in the Digital Index of Persons of Germania Sacra",
        "data_type": "ExternalId"
    },
    "P8964": {
        "label": "OpenReview.net profile ID",
        "description": "identifier for a person on OpenReview.net",
        "data_type": "ExternalId"
    },
    "P8343": {
        "label": "Odeon ID",
        "description": "identifier for a person or a theatrical production on the Odeon Theatre's website",
        "data_type": "ExternalId"
    },
    "P8857": {
        "label": "CRGPG ID",
        "description": "identifier of a Perche-Gouët personality on CRGPG's website",
        "data_type": "ExternalId"
    },
    "P8995": {
        "label": "Lumières.Lausanne ID",
        "description": "numerical identifier for a person in the website Lumières.Lausanne",
        "data_type": "ExternalId"
    },
    "P8678": {
        "label": "Slovník českých nakladatelství 1848-1949 person ID",
        "description": "identifier of person in Czech database of publishing houses (1848-1949)",
        "data_type": "ExternalId"
    },
    "P8750": {
        "label": "Unione Romana Biblioteche Scientifiche ID",
        "description": "identifier for an authority in the catalog of the Unione Romana Biblioteche Scientifiche",
        "data_type": "ExternalId"
    },
    "P8624": {
        "label": "American Battlefield Trust ID (person)",
        "description": "identifier for a person on the American Battlefield Trust website",
        "data_type": "ExternalId"
    },
    "P8788": {
        "label": "Dignity Memorial person ID",
        "description": "identifier used by Dignity Memorial for its obituaries",
        "data_type": "ExternalId"
    },
    "P9017": {
        "label": "Medieval Manuscripts in Oxford Libraries person ID",
        "description": "identifier of a person in the online database Medieval Manuscripts in Oxford Libraries",
        "data_type": "ExternalId"
    },
    "P8366": {
        "label": "GCatholic person ID",
        "description": "ID of a person on GCatholic.org",
        "data_type": "ExternalId"
    },
    "P8367": {
        "label": "Legacy.com person ID",
        "description": "identifier for a person's obituary on Legacy.com",
        "data_type": "ExternalId"
    },
    "P8557": {
        "label": "edukado.net Panteono ID",
        "description": "unique ID for the person in Panteono of the website edukado.net",
        "data_type": "ExternalId"
    },
    "P8795": {
        "label": "General Diamond Catalogue ID",
        "description": "identifier for a person or an organisation in the General Diamond Catalogue",
        "data_type": "ExternalId"
    },
    "P8833": {
        "label": "Union Catalog of Armenian Libraries authority ID",
        "description": "identifier for an authority in the catalog of the Union Catalog of Armenian Libraries",
        "data_type": "ExternalId"
    },
    "P8380": {
        "label": "Directory of Afrocubanas ID",
        "description": "biographical note on Afrocuban Women",
        "data_type": "ExternalId"
    },
    "P8381": {
        "label": "Emmys person ID",
        "description": "identifer for Emmys person ID",
        "data_type": "ExternalId"
    },
    "P8944": {
        "label": "Biblioteca Marciana owner ID",
        "description": "identifier for an owner of a manuscript (person or organization) in the catalog of Biblioteca Marciana",
        "data_type": "ExternalId"
    },
    "P8482": {
        "label": "Český hudební slovník osob a institucí ID",
        "description": "identifier for a person or institution in the Český hudební slovník osob a institucí (Czech Music Dictionary of People and Institutions)",
        "data_type": "ExternalId"
    },
    "P8385": {
        "label": "IDU person ID",
        "description": "personal identifier for artists in Czechia in Theatre Institute database (IDU)",
        "data_type": "ExternalId"
    },
    "P8945": {
        "label": "Museo Galileo biography ID",
        "description": "alphabetical identifier for a person in the biographies curated by Museo Galileo",
        "data_type": "ExternalId"
    },
    "P8386": {
        "label": "Gramophone composer ID",
        "description": "identifier for a composer, in the database of Gramophone UK magazine",
        "data_type": "ExternalId"
    },
    "P8947": {
        "label": "Museo Galileo authority ID",
        "description": "numerical identifier for an authority record in the catalog of Museo Galileo",
        "data_type": "ExternalId"
    },
    "P8347": {
        "label": "Spotify songwriter ID",
        "description": "identifier for a songwriter credited on Spotify",
        "data_type": "ExternalId"
    },
    "P8585": {
        "label": "FIPRESCI person ID",
        "description": "identifier for a critic or filmmaker on the International Federation of Film Critics website",
        "data_type": "ExternalId"
    },
    "P8951": {
        "label": "Order of Canada recipient ID",
        "description": "identifier for the recipients of the Order of Canada, one of the highest orders conferred by the Canadian governemnt",
        "data_type": "ExternalId"
    },
    "P8595": {
        "label": "DIL ID",
        "description": "numerical identifier for a lithographer in the 'Dictionnaire des imprimeurs-lithographes du XIXe siècle'",
        "data_type": "ExternalId"
    },
    "P8793": {
        "label": "Decorati onorificenze repubblica ID",
        "description": "decorated for honors of the Italian Republic record on the Quirinale site",
        "data_type": "ExternalId"
    },
    "P8953": {
        "label": "Students of Prague Universities ID",
        "description": "numeric identifier for student of Prague Universities 1882–1945",
        "data_type": "ExternalId"
    },
    "P8631": {
        "label": "Canadian Women Artists History Initiative ID",
        "description": "identifier in the artist database of Canadian Women Artists History Initiative",
        "data_type": "ExternalId"
    },
    "P8718": {
        "label": "AVN performer ID",
        "description": "numerical identifier for a pornographic actor on the Adult Video News's official website",
        "data_type": "ExternalId"
    },
    "P8767": {
        "label": "EGAFD ID",
        "description": "alphanumerical identifier for a female porn performer in the European Girls Adult Film Database",
        "data_type": "ExternalId"
    },
    "P8611": {
        "label": "Portugal. Dicionário Histórico ID",
        "description": "identifier for an article in the dictionary Portugal. Dicionário Histórico, Corográfico, Heráldico, Biográfico, Bibliográfico, Numismático e Artístico",
        "data_type": "ExternalId"
    },
    "P8720": {
        "label": "xHamster performer ID",
        "description": "identifier for a porn performer in the database of the xHamster website",
        "data_type": "ExternalId"
    },
    "P8721": {
        "label": "AWMDB performer ID",
        "description": "numerical identifier for a pornographic actor in the Adult Web Movie Database",
        "data_type": "ExternalId"
    },
    "P8975": {
        "label": "Manus Online author ID",
        "description": "identifier for an author of a manuscript held by an Italian library",
        "data_type": "ExternalId"
    },
    "P8544": {
        "label": "Frauen in Bewegung 1848–1938 ID",
        "description": "identifier for entries in the ARIADNE “Frauen in Bewegung 1848–1938” database, published by the Austrian National Library",
        "data_type": "ExternalId"
    },
    "P8496": {
        "label": "Archive Site Trinity College Cambridge ID",
        "description": "identifier for an entity (person, organization, family) in the website of the Archive of Trinity College, Cambridge",
        "data_type": "ExternalId"
    },
    "P8633": {
        "label": "Re-Member ID",
        "description": "identifier of a person in the biographical register for the Parliament of Victoria in Australia",
        "data_type": "ExternalId"
    },
    "P8315": {
        "label": "NTB ID",
        "description": "identifier for a person or a theatrical production on the National Theatre Bucharest website",
        "data_type": "ExternalId"
    },
    "P8870": {
        "label": "Hyperion Records person ID",
        "description": "ID of a person on Hyperion Records website",
        "data_type": "ExternalId"
    },
    "P8835": {
        "label": "Visages du diocèse d'Autun ID",
        "description": "identifier for a Catholic presbyter or laic active on the Autun Diocese from 1962 to 2012",
        "data_type": "ExternalId"
    },
    "P8930": {
        "label": "Church of Jesus Christ missionary ID",
        "description": "identifier of a missionary in Church of Jesus Christ missionary database (1830-1930)",
        "data_type": "ExternalId"
    },
    "P8405": {
        "label": "People's Representative Council of Indonesia ID",
        "data_type": "ExternalId"
    },
    "P8613": {
        "label": "Regesta Ecclesiastica Salisburgensia ID",
        "description": "identifier of the Regesta Ecclesiastica Salisburgensia biographical Wiki",
        "data_type": "ExternalId"
    },
    "P8902": {
        "label": "Archives at Yale names ID",
        "description": "identifier for an person, organization or family in Archives at Yale",
        "data_type": "ExternalId"
    },
    "P8487": {
        "label": "OpenCritic critic ID",
        "description": "identifier for a game reviewer on OpenCritic",
        "data_type": "ExternalId"
    },
    "P8657": {
        "label": "photoLondon ID",
        "description": "identifier for the photoLondon database",
        "data_type": "ExternalId"
    },
    "P8728": {
        "label": "Nachlässe in Austria ID",
        "description": "identifier of a reference work on artistic, academic and cultural Nachlässe in Austria",
        "data_type": "ExternalId"
    },
    "P8637": {
        "label": "L'Officiel des spectacles ID",
        "description": "identifier for an artist on the L'Officiel des spectacles website",
        "data_type": "ExternalId"
    },
    "P8837": {
        "label": "Prêtres du diocèse de Quimper ID",
        "description": "identifier for a priest from the diocese of Quimper on the website of the Roman Catholic Church in Finistère, France",
        "data_type": "ExternalId"
    },
    "P8873": {
        "label": "DriveThruComics numeric publisher identifier",
        "description": "numeric identifier for a publisher/manufacturer on DriveThruComics and sibling site",
        "data_type": "ExternalId"
    },
    "P8696": {
        "label": "Directory of Belgian Photographers ID",
        "description": "alphabetical identifier for a photographer in the Directory of Belgian Photographers",
        "data_type": "ExternalId"
    },
    "P485": {
        "label": "archives at",
        "description": "the institution holding the subject's archives",
        "data_type": "WikibaseItem"
    },
    "P3875": {
        "label": "Justia Patents company ID",
        "description": "ID of a company in Justia Patents",
        "data_type": "ExternalId"
    },
    "P169": {
        "label": "chief executive officer",
        "description": "highest-ranking corporate officer appointed as the CEO within an organization",
        "data_type": "WikibaseItem"
    },
    "P8157": {
        "label": "MémorialGenWeb monument ID",
        "description": "ID of a monument (war memorial) on MémorialGenWeb database",
        "data_type": "ExternalId"
    },
    "P984": {
        "label": "IOC country code",
        "description": "three-letter abbreviation country code by the International Olympic Committee",
        "data_type": "ExternalId"
    },
    "P1068": {
        "label": "instruction set",
        "description": "instruction set on which the processor architecture is based",
        "data_type": "WikibaseItem"
    },
    "P199": {
        "label": "business division",
        "description": "organizational divisions of this organization (which are not independent legal entities)",
        "data_type": "WikibaseItem"
    },
    "P7854": {
        "label": "EU-Football.info club ID",
        "description": "identifier for an association football club on the EU-Football.info website",
        "data_type": "ExternalId"
    },
    "P8620": {
        "label": "construction point",
        "description": "the steepest point on a ski jumping hill",
        "data_type": "Quantity"
    },
    "P8810": {
        "label": "parent (unspecified)",
        "description": "parent of the subject. Only use if neither father (P22) nor mother (P25) is applicable",
        "data_type": "WikibaseItem"
    },
    "P1454": {
        "label": "legal form",
        "description": "legal form of an entity",
        "data_type": "WikibaseItem"
    },
    "P4156": {
        "label": "Czech Registration ID",
        "description": "8-digit identifier for an organization (or self-employed enterpreneur) in the Czech Republic, conferred by the Czech Statistical Office",
        "data_type": "ExternalId"
    },
    "P2522": {
        "label": "victory",
        "description": "competition or event won by the subject",
        "data_type": "WikibaseItem"
    },
    "P2954": {
        "label": "Transferred Account Data Interchange Group Code",
        "data_type": "ExternalId"
    },
    "P3448": {
        "label": "stepparent",
        "description": "subject has the object as their stepparent",
        "data_type": "WikibaseItem"
    },
    "P1356": {
        "label": "number of losses",
        "description": "number of sporting matches, games or events lost",
        "data_type": "Quantity"
    },
    "P8815": {
        "label": "IAFD film distributor ID",
        "description": "identifier for a pornographic film distributor in the Internet Adult Film Database",
        "data_type": "ExternalId"
    },
    "P4446": {
        "label": "reward program",
        "description": "reward program associated with the item",
        "data_type": "WikibaseItem"
    },
    "P114": {
        "label": "airline alliance",
        "description": "alliance the airline belongs to",
        "data_type": "WikibaseItem"
    },
    "P1290": {
        "label": "godparent",
        "description": "person who is the godparent of a given person",
        "data_type": "WikibaseItem"
    },
    "P1128": {
        "label": "employees",
        "description": "total number of employees of a company at a given \"point in time\" (P585). Most recent data would generally have preferred rank; data for previous years normal rank (not deprecated rank). Add data for recent years, don't overwrite",
        "data_type": "Quantity"
    },
    "P1971": {
        "label": "number of children",
        "description": "number of children of the person",
        "data_type": "Quantity"
    },
    "P118": {
        "label": "league",
        "description": "league in which team or player plays or has played in",
        "data_type": "WikibaseItem"
    },
    "P505": {
        "label": "general manager",
        "description": "general manager of a sports team. If they are also an on-field manager use P286 instead",
        "data_type": "WikibaseItem"
    },
    "P9239": {
        "label": "affiliated worker organisation",
        "description": "organization representing workers in a specific company",
        "data_type": "WikibaseItem"
    },
    "P4591": {
        "label": "National Inventory of Canadian Military Memorials ID",
        "description": "identifier of a military monument or memorial in the Canadian National Inventory",
        "data_type": "ExternalId"
    },
    "P4624": {
        "label": "squadron embarked",
        "description": "the aircraft squadrons that served/serving on an aircraft carrier",
        "data_type": "WikibaseItem"
    },
    "P822": {
        "label": "mascot",
        "description": "mascot of an organization, e.g. a sports team or university",
        "data_type": "WikibaseItem"
    },
    "P2782": {
        "label": "Models.com client ID",
        "description": "identifier of a client in models.com",
        "data_type": "ExternalId"
    },
    "P831": {
        "label": "parent club",
        "description": "parent club of this team",
        "data_type": "WikibaseItem"
    },
    "P1041": {
        "label": "socket supported",
        "description": "socket for which the electronic part was made",
        "data_type": "WikibaseItem"
    },
    "P414": {
        "label": "stock exchange",
        "description": "exchange on which this company is traded",
        "data_type": "WikibaseItem"
    },
    "P3460": {
        "label": "colonel-in-chief",
        "description": "ceremonial position in a military regiment",
        "data_type": "WikibaseItem"
    },
    "P7779": {
        "label": "member of military unit",
        "description": "smallest military unit that a person is/was in",
        "data_type": "WikibaseItem"
    },
    "P6859": {
        "label": "Corporate Number (South Korea)",
        "description": "identifier assigned to companies registered in South Korea by the Registration Office, the Court of South Korea",
        "data_type": "ExternalId"
    },
    "P8676": {
        "label": "London Stock Exchange company ID",
        "description": "identifier for a company listed on the London Stock Exchange",
        "data_type": "ExternalId"
    },
    "P3377": {
        "label": "Bloomberg company ID",
        "description": "identifier for a company, in the Bloomberg database",
        "data_type": "ExternalId"
    },
    "P634": {
        "label": "captain",
        "description": "captain of this sports team",
        "data_type": "WikibaseItem"
    },
    "P2105": {
        "label": "size of team at finish",
        "description": "number of players/sled dogs/etc. in a team at the end of the race or match",
        "data_type": "Quantity"
    },
    "P113": {
        "label": "airline hub",
        "description": "airport that serves as a hub for an airline",
        "data_type": "WikibaseItem"
    },
    "P8147": {
        "label": "National-Football-Teams.com club ID",
        "description": "identifier for a association football (soccer) club in the National-Football-Teams.com database",
        "data_type": "ExternalId"
    },
    "P8622": {
        "label": "hill size",
        "description": "ski jumping hill size",
        "data_type": "Quantity"
    },
    "P4573": {
        "label": "Merchant Category Code",
        "description": "four-digit number used to classify businesses by the type of goods or services it provides.",
        "data_type": "String"
    },
    "P451": {
        "label": "unmarried partner",
        "description": "someone with whom the person is in a relationship without being married. Use \"spouse\" (P26) for married couples",
        "data_type": "WikibaseItem"
    },
    "P3979": {
        "label": "Unifrance company ID",
        "description": "identifier of a company on unifrance.org",
        "data_type": "ExternalId"
    },
    "P1359": {
        "label": "number of points/goals conceded",
        "description": "points conceded or goals against (use in league table items)",
        "data_type": "Quantity"
    },
    "P25": {
        "label": "mother",
        "description": "female parent of the subject. For stepmother, use \"stepparent\" (P3448)",
        "data_type": "WikibaseItem"
    },
    "P4105": {
        "label": "EGF rating",
        "description": "rating given by EGF (European Go Federation) to European go players",
        "data_type": "Quantity"
    },
    "P3692": {
        "label": "NCAA.com team ID",
        "description": "identifier for an American college sports team on the National Collegiate Athletic Association's website",
        "data_type": "ExternalId"
    },
    "P6697": {
        "label": "Football Money League rank",
        "description": "rank in the Deloitte Football Money League",
        "data_type": "Quantity"
    },
    "P241": {
        "label": "military branch",
        "description": "branch to which this military unit, award, office, or person belongs, e.g. Royal Navy",
        "data_type": "WikibaseItem"
    },
    "P6131": {
        "label": "Soccerway team ID",
        "description": "team ID on the Soccerway website",
        "data_type": "ExternalId"
    },
    "P859": {
        "label": "sponsor",
        "description": "organization or individual that sponsors this item",
        "data_type": "WikibaseItem"
    },
    "P2103": {
        "label": "size of team at start",
        "description": "number of players/sled dogs/etc. in a team at the start of a match or race",
        "data_type": "Quantity"
    },
    "P8127": {
        "label": "tournament format",
        "description": "format of a sports tournament",
        "data_type": "WikibaseItem"
    },
    "P4791": {
        "label": "commanded by",
        "description": "commander of a military unit/army/security service, operation, etc.",
        "data_type": "WikibaseItem"
    },
    "P3225": {
        "label": "Corporate Number (Japan)",
        "description": "identifier assigned to companies and other organizations by the National Tax Agency of Japan",
        "data_type": "ExternalId"
    },
    "P8571": {
        "label": "external auditor",
        "description": "external company or person appointed to audit a business' or an organization's books, requirement for many companies",
        "data_type": "WikibaseItem"
    },
    "P286": {
        "label": "head coach",
        "description": "on-field manager or head coach of a sports club (not to be confused with a general manager P505, which is not a coaching position) or person",
        "data_type": "WikibaseItem"
    },
    "P1320": {
        "label": "OpenCorporates ID",
        "description": "identifier for a corporation, in the OpenCorporates database. Format: country prefix, optional subnational entity abbrevation, \"/\", alphanumeric id",
        "data_type": "ExternalId"
    },
    "P880": {
        "label": "CPU",
        "description": "central processing unit found within the subject item",
        "data_type": "WikibaseItem"
    },
    "P7172": {
        "label": "shmup.com ID",
        "description": "identifier in the shmup.com database of shoot them up video games",
        "data_type": "ExternalId"
    },
    "P7044": {
        "label": "Colonial Frontier Massacres ID",
        "description": "identifier for a massacre in the database of Colonial Frontier Massacres in Central and Eastern Australia 1788-1930",
        "data_type": "ExternalId"
    },
    "P7030": {
        "label": "National Pollutant Inventory ID",
        "description": "jurisdiction identifier for facilities in the National Pollutant Inventory of Australia",
        "data_type": "ExternalId"
    },
    "P7197": {
        "label": "OpenWeatherMap city ID",
        "description": "identifier for a city at OpenWeatherMap",
        "data_type": "ExternalId"
    },
    "P7227": {
        "label": "DIBAVOD ID",
        "description": "ID of water objects (rivers, ponds, lakes, ...) of the Czech republic referenced in DIgitální BÁze VOdohospodářských Dat (DIBAVOD) of VÚV TGM",
        "data_type": "ExternalId"
    },
    "P7204": {
        "label": "Mafab.hu film ID",
        "description": "film ID in the Mafab.hu movie database",
        "data_type": "ExternalId"
    },
    "P7199": {
        "label": "Scribd publication ID",
        "description": "external identifier for a publication in the Scribd library",
        "data_type": "ExternalId"
    },
    "P7275": {
        "label": "Naver VIBE artist ID",
        "description": "identifier for artist on Naver Music site",
        "data_type": "ExternalId"
    },
    "P7073": {
        "label": "extension on WikiApiary",
        "description": "entry of a MediaWiki extension in WikiApiary",
        "data_type": "ExternalId"
    },
    "P7054": {
        "label": "misas.org Catholic church ID",
        "description": "identifier of a catholic church at misas.org",
        "data_type": "ExternalId"
    },
    "P7070": {
        "label": "Musixmatch writer ID",
        "description": "identifier for writers/authors in Musixmatch database",
        "data_type": "ExternalId"
    },
    "P7092": {
        "label": "Russian Fencing Federation fencer ID",
        "description": "Russian Fencing Federation identifier",
        "data_type": "ExternalId"
    },
    "P7161": {
        "label": "Kinsky–Halm Catalogue",
        "description": "number in the Kinsky–Halm Catalogue of the works of Ludwig van Beethoven, which do not have opus numbers, or are fragmentary",
        "data_type": "ExternalId"
    },
    "P7211": {
        "label": "LINE BLOG user ID",
        "description": "user ID on LINE BLOG",
        "data_type": "ExternalId"
    },
    "P7237": {
        "label": "Chinese Engineering Expert Tank ID",
        "description": "ID of a person in Chinese Engineering Expert Tank, a database of Chinese researchers in science and engineering fields",
        "data_type": "ExternalId"
    },
    "P6929": {
        "label": "CUT code",
        "description": "identifier for regions, provinces, and municipalities in Chile as defined by Ministerio de Bienes Nacionales",
        "data_type": "ExternalId"
    },
    "P7280": {
        "label": "PlaymakerStats.com team ID",
        "description": "identifier for a association football (soccer) team at PlaymakerStats.com (formerly TheFinalBall.com)",
        "data_type": "ExternalId"
    },
    "P7263": {
        "label": "Prime Pages ID",
        "description": "an ID for some (usually) large prime in Prime Pages",
        "data_type": "ExternalId"
    },
    "P7281": {
        "label": "Radio Equipment List Certification Number",
        "description": "unique identifier of an electronic device certification registered with Innovation, Science and Economic Development Canada",
        "data_type": "ExternalId"
    },
    "P6959": {
        "label": "Sindh EMIS code",
        "description": "identifier for a school (usually government-affiliated) in the province of Sindh",
        "data_type": "ExternalId"
    },
    "P6957": {
        "label": "Khyber Pakhtunkhwa EMIS code",
        "description": "identifier for a school (usually government-affiliated) in the province of Khyber Pakhtunkhwa",
        "data_type": "ExternalId"
    },
    "P7180": {
        "label": "Planespotters.net aircraft ID",
        "description": "identifier for an aircraft on the Planespotters.net website",
        "data_type": "ExternalId"
    },
    "P7035": {
        "label": "TuttoCalciatori.net coach ID",
        "description": "identifier of a football coach in the website TuttoCalciatori.net",
        "data_type": "ExternalId"
    },
    "P7231": {
        "label": "eSTUDÁNKY ID",
        "description": "identifier in the eSTUDÁNKY database, which covers water sources and water wells in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P7234": {
        "label": "Scribd item ID",
        "description": "external identifier for a book, audio book, document, magazine issue or article in the Scribd library",
        "data_type": "ExternalId"
    },
    "P7273": {
        "label": "National Equipment Registration System Supplier Number",
        "description": "unique identifier for an organisation supplying electrical equipment listed in the Australian National Equipment Registration System",
        "data_type": "ExternalId"
    },
    "P7276": {
        "label": "New Encyclopedic Dictionary of Czech ID",
        "description": "identifier for an entry in the online release of the New Encyclopedic Dictionary of Czech",
        "data_type": "ExternalId"
    },
    "P6998": {
        "label": "iTunes movie collection ID",
        "description": "identifier for a movie collection in the iTunes Store",
        "data_type": "ExternalId"
    },
    "P6995": {
        "label": "National Transportation Safety Board report ID",
        "description": "identifier for a report about a transport accident from the National Transportation Safety Board",
        "data_type": "ExternalId"
    },
    "P6979": {
        "label": "SLNSW unpublished item ID",
        "data_type": "ExternalId"
    },
    "P7019": {
        "label": "ANZCTR ID",
        "description": "identifier of a clinical trial in the Australian New Zealand Clinical Trials Registry",
        "data_type": "ExternalId"
    },
    "P7020": {
        "label": "Arachne collection ID",
        "description": "identifier for a collection in the Arachne database",
        "data_type": "ExternalId"
    },
    "P7242": {
        "label": "Laws & Regulations Database of the Republic of China ID",
        "description": "the website URL code for the Laws & Regulations Database of the Republic of China",
        "data_type": "ExternalId"
    },
    "P7238": {
        "label": "Open Science Framework ID",
        "description": "ID in osf.io",
        "data_type": "ExternalId"
    },
    "P7226": {
        "label": "RPGGeek ID",
        "description": "ID for a tabletop role-playing games, designer or publisher or a family of TTRPG at RPGGeek",
        "data_type": "ExternalId"
    },
    "P6976": {
        "label": "Retrosheet person ID",
        "description": "identifier for a Major League Baseball player, coach, manager or umpire in retrosheet.org",
        "data_type": "ExternalId"
    },
    "P7036": {
        "label": "TuttoCalciatori.net player ID",
        "description": "identifier of a footballer in the website TuttoCalciatori.net",
        "data_type": "ExternalId"
    },
    "P7021": {
        "label": "Arachne object ID",
        "description": "identifier for an object in the Arachne database",
        "data_type": "ExternalId"
    },
    "P7025": {
        "label": "HCIS ID",
        "description": "identifier for a chemical in the Hazardous Chemical Information System by Safe Work Australia",
        "data_type": "ExternalId"
    },
    "P6956": {
        "label": "Balochistan EMIS code",
        "description": "identifier for a school (usually government-affiliated) in the province of Balochistan",
        "data_type": "ExternalId"
    },
    "P6946": {
        "label": "Find & Connect ID",
        "description": "identifier in the Find & Connect website by the Australian Government about orphanages, children's Homes, and other institutions",
        "data_type": "ExternalId"
    },
    "P7049": {
        "label": "AICS Chemical ID (former scheme)",
        "description": "unique identifier of a chemical in the Australian Inventory of Chemical Substances",
        "data_type": "ExternalId"
    },
    "P7067": {
        "label": "Album of the Year album ID",
        "description": "identifier for an album or other release on the Album of the Year (AOTY) website",
        "data_type": "ExternalId"
    },
    "P7059": {
        "label": "Premier Badminton League player ID",
        "description": "external identifiers for players participating in Premier Badminton League (India)",
        "data_type": "ExternalId"
    },
    "P7094": {
        "label": "WaterNSW Rivers and Streams Site No.",
        "description": "site number of a river or stream in the real time water data published by WaterNSW",
        "data_type": "ExternalId"
    },
    "P7064": {
        "label": "PBS Manufacturer ID",
        "description": "unique identifier for a pharmaceutical manufacturer listed in the Australian Pharmaceutical Benefits Scheme",
        "data_type": "ExternalId"
    },
    "P6947": {
        "label": "Goodreads series ID",
        "description": "identifier of a book series in the Goodreads website",
        "data_type": "ExternalId"
    },
    "P7120": {
        "label": "Douyin ID",
        "description": "username on Douyin",
        "data_type": "ExternalId"
    },
    "P7121": {
        "label": "School code of the Ministry of Education, Taiwan",
        "description": "identified codes for the all level of schools in Taiwan ruled by Ministry of Education, Taiwan",
        "data_type": "ExternalId"
    },
    "P7063": {
        "label": "U. S. Supreme Court docket number",
        "description": "identifier for a case filed at the Supreme Court of the United States",
        "data_type": "ExternalId"
    },
    "P7001": {
        "label": "PharmGKB ID",
        "description": "unique identifier for an entity in the PharmGKB knowledgebase",
        "data_type": "ExternalId"
    },
    "P6930": {
        "label": "Pro Football Hall of Fame ID",
        "description": "identifier of a person in the Pro Football Hall of Fame",
        "data_type": "ExternalId"
    },
    "P6996": {
        "label": "Epitafier.se ID",
        "description": "identifier for an inscribed tablet about a dead person in a Swedish church from the website epitafier.se",
        "data_type": "ExternalId"
    },
    "P7146": {
        "label": "Rollerstory.net skater ID",
        "description": "inline speed skater-database",
        "data_type": "ExternalId"
    },
    "P7005": {
        "label": "Adventure Gamers ID",
        "description": "identifier in the Adventure Gamers database of adventure video games",
        "data_type": "ExternalId"
    },
    "P7040": {
        "label": "NosDéputés.fr identifiant",
        "description": "identifier for a french deputies in NosDéputés.fr",
        "data_type": "ExternalId"
    },
    "P7144": {
        "label": "NOTAM (FAA)",
        "description": "reference of a Notice to Airmen (NOTAM) issued by the FAA in the United States",
        "data_type": "ExternalId"
    },
    "P7149": {
        "label": "OhRanger.com ID",
        "description": "identifier for a place on the OhRanger.com website",
        "data_type": "ExternalId"
    },
    "P7000": {
        "label": "DigitalNZ ID",
        "description": "identifier on the Digitalnz.org website for videos, newspapers, maps, photographs, audio, artworks, and news reports that have been digitised in New Zealand",
        "data_type": "ExternalId"
    },
    "P7207": {
        "label": "Moly.hu book ID",
        "description": "book identifier in the Hungarian social cataloging website Moly.hu",
        "data_type": "ExternalId"
    },
    "P7003": {
        "label": "ACMI web ID",
        "description": "identifier for an object in the Australian Centre for the Moving Image",
        "data_type": "ExternalId"
    },
    "P7089": {
        "label": "AFCD PFKID",
        "description": "unique identifier for a food type in the Australian Food Composition Database",
        "data_type": "ExternalId"
    },
    "P7157": {
        "label": "The Top Tens ID",
        "description": "item's ID on The Top Tens website",
        "data_type": "ExternalId"
    },
    "P7051": {
        "label": "Atlas of Living Australia ID",
        "description": "identifier for a taxon in the Atlas of Living Australia",
        "data_type": "ExternalId"
    },
    "P7065": {
        "label": "Kerala state school code",
        "description": "identifier for a school situated in Kerala",
        "data_type": "ExternalId"
    },
    "P7060": {
        "label": "NosSénateurs.fr identifier",
        "description": "identifier for french deputies in NosSénateurs.fr",
        "data_type": "ExternalId"
    },
    "P7057": {
        "label": "FEC Campaign Committee ID",
        "description": "ID for a campaign committee in the United States",
        "data_type": "ExternalId"
    },
    "P7053": {
        "label": "Indian Super League player ID",
        "description": "external Identifiers for players participating in Indian Super League",
        "data_type": "ExternalId"
    },
    "P7072": {
        "label": "wiki on WikiApiary",
        "description": "entry of a MediaWiki site in WikiApiary",
        "data_type": "ExternalId"
    },
    "P7048": {
        "label": "PBS Drug Code",
        "description": "unique identifier for a drug listed in the Australian Pharmaceutical Benefits Scheme",
        "data_type": "ExternalId"
    },
    "P7177": {
        "label": "Cinestaan film ID",
        "description": "external identifier for Indian movies",
        "data_type": "ExternalId"
    },
    "P7179": {
        "label": "Service d'Information sur les Etudes et les Professions Job ID",
        "description": "job entry in official website Service d'Information sur les Etudes et les Professions",
        "data_type": "ExternalId"
    },
    "P7033": {
        "label": "Australian Educational Vocabulary ID",
        "description": "ID for curriculum term in one of the controlled vocabularies at Australian education vocabularies",
        "data_type": "ExternalId"
    },
    "P6932": {
        "label": "RogerEbert.com film ID",
        "description": "identifier for a film in the website RogerEbert.com",
        "data_type": "ExternalId"
    },
    "P7134": {
        "label": "Mojarto artist ID",
        "description": "external identifiers for artists whose arts are sold through Mojarto",
        "data_type": "ExternalId"
    },
    "P7074": {
        "label": "skin on WikiApiary",
        "description": "entry of a MediaWiki skin in WikiApiary",
        "data_type": "ExternalId"
    },
    "P7093": {
        "label": "ABBBS Taxon ID",
        "description": "unique identifier of a taxon listed in the Australian Bird and Bat Banding Scheme Database",
        "data_type": "ExternalId"
    },
    "P6997": {
        "label": "magazine in BitArkivo.org",
        "description": "unique ID for the magazine on the website BitArkivo",
        "data_type": "ExternalId"
    },
    "P7274": {
        "label": "Naver VIBE album ID",
        "description": "identifier for an album on Naver VIBE site",
        "data_type": "ExternalId"
    },
    "P7077": {
        "label": "Paris Foreign Missions Society ID",
        "description": "identifier of a missionary on the site of the archives of the Foreign Missions of Paris",
        "data_type": "ExternalId"
    },
    "P7223": {
        "label": "Transfermarkt team ID",
        "description": "identifier for an association football (soccer) team in the transfermarkt.com database",
        "data_type": "ExternalId"
    },
    "P7140": {
        "label": "München Wiki article ID",
        "description": "identifier for articles at muenchenwiki.de",
        "data_type": "ExternalId"
    },
    "P7264": {
        "label": "FCC Grantee Code",
        "description": "unique identifier of a grantee assigned by the Federal Communications Commission of the United States",
        "data_type": "ExternalId"
    },
    "P7038": {
        "label": "Documenta Catholica Omnia author ID",
        "description": "identifier for an author in Documenta Catholica Omnia digital library",
        "data_type": "ExternalId"
    },
    "P7046": {
        "label": "HOV-ID",
        "description": "identifier for a place in the Historical Gazetteer of Saxony",
        "data_type": "ExternalId"
    },
    "P6958": {
        "label": "Punjab EMIS code",
        "description": "identifier for a school (usually government-affiliated) in the province of Punjab in Pakistan",
        "data_type": "ExternalId"
    },
    "P7139": {
        "label": "ONF forest national ID",
        "description": "unique identifier of a forest by the Office national des forêts (French governmental agency)",
        "data_type": "ExternalId"
    },
    "P7128": {
        "label": "Archives Directory for the History of Collecting in America ID",
        "description": "identifier from the Archives Directory for the History of Collecting in America",
        "data_type": "ExternalId"
    },
    "P7130": {
        "label": "Vidwan ID",
        "description": "external identifier to describe profiles of Indian academicians and scientists maintained in the database Vidwan",
        "data_type": "ExternalId"
    },
    "P7100": {
        "label": "Lens ID",
        "description": "identifier for documents in the Lens.org catalog (patents, articles)",
        "data_type": "ExternalId"
    },
    "P7076": {
        "label": "Zaragoza public art ID",
        "description": "ID of public art catalogued in zaragoza.es",
        "data_type": "ExternalId"
    },
    "P7168": {
        "label": "FGrHist ID",
        "description": "identifier for an ancient Greek historian in Die Fragmente der griechischen Historiker by Felix Jacoby",
        "data_type": "ExternalId"
    },
    "P7117": {
        "label": "Media Arts Database console games ID (former scheme)",
        "description": "identifier for a console game in the Media Art database created by the Japanese government department Bunkacho (former scheme, use P7886 for new identifier with 'C/M' prefix)",
        "data_type": "ExternalId"
    },
    "P7182": {
        "label": "IBP 4-letter bird species alpha code",
        "description": "Standardized four-letter alphabetic code for bird species by the Institute for Bird Populations",
        "data_type": "ExternalId"
    },
    "P6740": {
        "label": "Justia Patents assignee ID",
        "description": "identifier for the assignee of a patent in Justia Patents",
        "data_type": "ExternalId"
    },
    "P4884": {
        "label": "court",
        "description": "specific court a legal case is/was heard/decided in",
        "data_type": "WikibaseItem"
    },
    "P618": {
        "label": "source of energy",
        "description": "describes the source of energy an animated object (machine or animal) uses",
        "data_type": "WikibaseItem"
    },
    "P3719": {
        "label": "regulated by",
        "description": "organization that acts as regulator of an activity, financial market, or stock exchange",
        "data_type": "WikibaseItem"
    },
    "P3751": {
        "label": "Shoftim BeIsrael judge ID",
        "description": "database about the judges of Israel",
        "data_type": "ExternalId"
    },
    "P1594": {
        "label": "judge",
        "description": "judge, magistrate or equivalent, presiding at a trial",
        "data_type": "WikibaseItem"
    },
    "P1592": {
        "label": "prosecutor",
        "description": "person representing the prosecuting authority, at a trial",
        "data_type": "WikibaseItem"
    },
    "P700": {
        "label": "Kemler code",
        "description": "code describing the hazards of a chemical in transport",
        "data_type": "ExternalId"
    },
    "P5630": {
        "label": "prisoner count",
        "description": "number of inmates or prisioners held in this prison",
        "data_type": "Quantity"
    },
    "P5582": {
        "label": "number of arrests",
        "description": "the number of arrests made by public officials during or resulting from an event",
        "data_type": "Quantity"
    },
    "P521": {
        "label": "scheduled service destination",
        "description": "airport or station connected by regular direct service to the subject; for the destination of a trip see P1444",
        "data_type": "WikibaseItem"
    },
    "P1595": {
        "label": "charge",
        "description": "offence with which someone is charged, at a trial",
        "data_type": "WikibaseItem"
    },
    "P9376": {
        "label": "law digest",
        "description": "short text that summarizes a law, decree and other legal rulings, part of the preamble",
        "data_type": "Monolingualtext"
    },
    "P8550": {
        "label": "law or regulation identifying number",
        "description": "a sequence of numbers assigned to laws/acts that were passed by countries or local governments",
        "data_type": "String"
    },
    "P609": {
        "label": "terminus location",
        "description": "location of the terminus of a linear feature",
        "data_type": "WikibaseItem"
    },
    "P1399": {
        "label": "convicted of",
        "description": "crime a person was convicted of",
        "data_type": "WikibaseItem"
    },
    "P6193": {
        "label": "ratified by",
        "description": "used to specify that a treaty has been ratified by a given state or subject of international law",
        "data_type": "WikibaseItem"
    },
    "P8407": {
        "label": "Supreme Court of Sweden case number",
        "description": "identifier for a case in the Supreme Court of Sweden",
        "data_type": "ExternalId"
    },
    "P3148": {
        "label": "repeals",
        "description": "this document or act repeals that other document or act",
        "data_type": "WikibaseItem"
    },
    "P177": {
        "label": "crosses",
        "description": "obstacle (body of water, road, railway...) which this bridge crosses over or this tunnel goes under",
        "data_type": "WikibaseItem"
    },
    "P8433": {
        "label": "Swedish Riksdag document ID",
        "description": "identifier for documents produced in the Riksdag, the national legislature and the supreme decision-making body of Sweden",
        "data_type": "ExternalId"
    },
    "P4821": {
        "label": "Cour des comptes magistrate ID",
        "description": "identifier of a magistrate in the French Court of Audit online biographical dictionary",
        "data_type": "ExternalId"
    },
    "P3493": {
        "label": "legal status (medicine)",
        "description": "legal status for pharmaceutical drugs, e.g. general sales list for paracetamol in the UK",
        "data_type": "WikibaseItem"
    },
    "P2058": {
        "label": "depositor",
        "description": "depositor/depositaries for the treaty",
        "data_type": "WikibaseItem"
    },
    "P1891": {
        "label": "signatory",
        "description": "person, country, or organization that has signed an official document (use P50 for author)",
        "data_type": "WikibaseItem"
    },
    "P3014": {
        "label": "laws applied",
        "description": "the constitutional and statutory provisions that were the primary basis for a decision or law",
        "data_type": "WikibaseItem"
    },
    "P197": {
        "label": "adjacent station",
        "description": "the stations next to this station, sharing the same line(s)",
        "data_type": "WikibaseItem"
    },
    "P1437": {
        "label": "plea",
        "description": "whether a person pleaded guilty, not guilty, etc.",
        "data_type": "WikibaseItem"
    },
    "P8457": {
        "label": "ANZSOC 2011 ID",
        "description": "identifier for criminal offences in Australian and New Zealand",
        "data_type": "ExternalId"
    },
    "P1031": {
        "label": "legal citation of this text",
        "description": "legal citation of legislation or a court decision",
        "data_type": "String"
    },
    "P4424": {
        "label": "mandates",
        "description": "action or status required or requested by a motion",
        "data_type": "WikibaseItem"
    },
    "P375": {
        "label": "space launch vehicle",
        "description": "type of rocket or other vehicle for launching subject payload into outer space",
        "data_type": "WikibaseItem"
    },
    "P8609": {
        "label": "China Treaty Database ID",
        "description": "identifier for a treaty in the China Treaty Database",
        "data_type": "ExternalId"
    },
    "P2567": {
        "label": "amended by",
        "description": "document is amended by specified other document",
        "data_type": "WikibaseItem"
    },
    "P3944": {
        "label": "Old Bailey Proceedings ID",
        "description": "identifier for a trial, on the Old Bailey Proceedings website",
        "data_type": "ExternalId"
    },
    "P5023": {
        "label": "activity policy in this place",
        "description": "policy for activities in this place, for example no photos, photos allowed, no flash, lighting fires, flying drones, etc.",
        "data_type": "WikibaseItem"
    },
    "P7152": {
        "label": "copyright exemption",
        "description": "copyright exemption this file is covered by",
        "data_type": "WikibaseItem"
    },
    "P8610": {
        "label": "e-GOV law ID",
        "description": "identifier in the e-Gov Law Search",
        "data_type": "ExternalId"
    },
    "P4006": {
        "label": "overrules",
        "description": "case that supersedes another case",
        "data_type": "WikibaseItem"
    },
    "P4310": {
        "label": "FOIH decree types ID",
        "description": "identifier for a decree type in the thesaurus of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P1620": {
        "label": "plaintiff",
        "description": "party who initiates a lawsuit",
        "data_type": "WikibaseItem"
    },
    "P546": {
        "label": "docking port",
        "description": "intended docking port for a spacecraft",
        "data_type": "WikibaseItem"
    },
    "P8722": {
        "label": "Normattiva ID",
        "description": "URL linking to the law in the Normattiva website",
        "data_type": "ExternalId"
    },
    "P8635": {
        "label": "NDL law ID",
        "description": "identifier in the NDL (National Diet Library) Japanese Law Index",
        "data_type": "ExternalId"
    },
    "P7162": {
        "label": "cases consolidated",
        "description": "what other cases this case consolidated",
        "data_type": "WikibaseItem"
    },
    "P9119": {
        "label": "LexML Brazil ID",
        "description": "identifier for laws in the LexML system",
        "data_type": "ExternalId"
    },
    "P2568": {
        "label": "repealed by",
        "description": "document is repealed/inactived by specified other document",
        "data_type": "WikibaseItem"
    },
    "P5826": {
        "label": "majority opinion by",
        "description": "judicial opinion agreed to by more than half of the members of a court",
        "data_type": "WikibaseItem"
    },
    "P559": {
        "label": "terminus",
        "description": "the feature (intersecting road, train station, etc.) at the end of a linear feature",
        "data_type": "WikibaseItem"
    },
    "P3281": {
        "label": "French National Assembly Lobbyist ID",
        "description": "identifier of an individual lobbyist or lobbying organisation, at the French National Assembly",
        "data_type": "ExternalId"
    },
    "P2657": {
        "label": "EU Transparency Register ID",
        "description": "identity code for an organisation, in the transparency register of the European Union",
        "data_type": "ExternalId"
    },
    "P4678": {
        "label": "MAVISE competent authority ID",
        "description": "identifier for a licensing authority/register in the MAVISE database of the European Audiovisual Observatory",
        "data_type": "ExternalId"
    },
    "P3016": {
        "label": "French national research structure identifier",
        "description": "identifier in the French national research structure repertory",
        "data_type": "ExternalId"
    },
    "P3066": {
        "label": "GLAM Identifier",
        "description": "unique identifier for heritage institutions as they have been used in the context of the OpenGLAM Benchmark Survey",
        "data_type": "ExternalId"
    },
    "P3901": {
        "label": "ADAGP artist ID",
        "description": "identifier for an artist as a member of the French collective rights management organisation ADAGP and sister organisations worldwide",
        "data_type": "ExternalId"
    },
    "P4550": {
        "label": "CNRS research group ID",
        "description": "identifier for an academic research group issued by the CNRS",
        "data_type": "ExternalId"
    },
    "P2627": {
        "label": "ISO 9362 SWIFT/BIC code",
        "description": "Identifier ISO 9362 SWIFT/BIC code",
        "data_type": "ExternalId"
    },
    "P3914": {
        "label": "GuideStar Israel organization ID",
        "description": "ID in the Israeli non-profit organizations database GuideStar",
        "data_type": "ExternalId"
    },
    "P4871": {
        "label": "GEPRIS organization ID",
        "description": "Identifier of an organization in GEPRIS database of funded research projects",
        "data_type": "ExternalId"
    },
    "P3918": {
        "label": "Répertoire national des associations identifier",
        "description": "identifier of a French non-profit in the Répertoire national des associations",
        "data_type": "ExternalId"
    },
    "P3642": {
        "label": "ARCHON code",
        "description": "unique number used to identify archival collection locations within the UK and key global repositories holding collections relating to British history",
        "data_type": "ExternalId"
    },
    "P4861": {
        "label": "Charity Navigator ID",
        "description": "identifier for a charitable organisation in the United States, in the Charity Navigator database",
        "data_type": "ExternalId"
    },
    "P4331": {
        "label": "Department of Education and Skills roll number",
        "description": "identifier in official government database of Irish schools",
        "data_type": "ExternalId"
    },
    "P3042": {
        "label": "CageMatch wrestling stable ID",
        "description": "identifier for a professional wrestling stable at CageMatch.net",
        "data_type": "ExternalId"
    },
    "P3715": {
        "label": "NISH Hall of Fame ID",
        "description": "identifier for an inductee in its Hall of Fame on the Niedersächsisches Institut für Sportgeschichte website",
        "data_type": "ExternalId"
    },
    "P5508": {
        "label": "archINFORM person or group ID",
        "description": "identifier for a person or group on the archINFORM website",
        "data_type": "ExternalId"
    },
    "P2655": {
        "label": "Estyn ID",
        "description": "identifier for a school in Wales, in the Welsh Assembly's Estyn database",
        "data_type": "ExternalId"
    },
    "P3797": {
        "label": "autonomous system number",
        "description": "unique identifier for a collection of connected Internet Protocol (IP) routing prefixes under the control of one or more network operators",
        "data_type": "ExternalId"
    },
    "P3441": {
        "label": "FIFA country code",
        "description": "three-letter country code assigned by FIFA",
        "data_type": "String"
    },
    "P4691": {
        "label": "OpenSecrets organization ID",
        "description": "identifier used by Opensecrets for organizations involved in US elections",
        "data_type": "ExternalId"
    },
    "P4339": {
        "label": "Radio Radicale organizer ID",
        "description": "identifier for an organizer in the database of Radio Radicale",
        "data_type": "ExternalId"
    },
    "P4725": {
        "label": "C-SPAN organization ID",
        "description": "identifier for an organization, on C-SPAN",
        "data_type": "ExternalId"
    },
    "P4639": {
        "label": "Estonian Football Association team ID",
        "description": "identifier for a team on the Estonian Football Association website",
        "data_type": "ExternalId"
    },
    "P3819": {
        "label": "FI WarSampo army unit ID",
        "description": "identifier in the WarSampo Finnish WW2 portal",
        "data_type": "ExternalId"
    },
    "P5181": {
        "label": "Relationship Science organization ID",
        "description": "identifier for an organization in the Relationship Science database",
        "data_type": "ExternalId"
    },
    "P4581": {
        "label": "Städel Museum artist ID",
        "description": "artist and organization identifier for the Städel Museum",
        "data_type": "ExternalId"
    },
    "P2651": {
        "label": "CRICOS Provider Code",
        "description": "unique identifier for Australian education providers assigned by the Commonwealth Register of Institutions and Courses for Overseas Students (CRICOS)",
        "data_type": "ExternalId"
    },
    "P3035": {
        "label": "ISBN publisher prefix",
        "description": "part of an ISBN(-13) specific to a publisher",
        "data_type": "ExternalId"
    },
    "P3061": {
        "label": "Basisregistratie Instellingen number",
        "description": "register number, issued by the Dutch ministry of Education, given to schools",
        "data_type": "ExternalId"
    },
    "P4911": {
        "label": "Annuaire des fondations ID",
        "description": "ID of a foundation on the directory of foundations of the French Foundation Centre",
        "data_type": "ExternalId"
    },
    "P2939": {
        "label": "CageMatch tag team ID",
        "description": "identifier for a professional wrestling tag team at www.cagematch.net",
        "data_type": "ExternalId"
    },
    "P4128": {
        "label": "NPSN Indonesian school ID",
        "description": "identifier for a school in Indonesia",
        "data_type": "ExternalId"
    },
    "P3849": {
        "label": "LombardiaBeniCulturali institution ID",
        "description": "identifier of a historical institution in the cultural heritage database of the Lombardy Region of Italy",
        "data_type": "ExternalId"
    },
    "P4635": {
        "label": "Indian Financial System Code",
        "description": "identifier for a bank branch participating in either of the two main Payment and settlement systems in India",
        "data_type": "ExternalId"
    },
    "P4907": {
        "label": "Guide Nicaise ID",
        "data_type": "ExternalId"
    },
    "P3407": {
        "label": "Klosterdatenbank ID",
        "description": "entry in the Germania Sacra Klosterdatenbank",
        "data_type": "ExternalId"
    },
    "P3153": {
        "label": "Crossref funder ID",
        "description": "identifier for an organisation that funds research, in the Crossref registry",
        "data_type": "ExternalId"
    },
    "P2740": {
        "label": "ResearchGate institute ID",
        "description": "identifier of an institute in ResearchGate",
        "data_type": "ExternalId"
    },
    "P4052": {
        "label": "Academia.edu institutional ID",
        "description": "academic institution page on academia.edu",
        "data_type": "ExternalId"
    },
    "P2904": {
        "label": "Beer Advocate brewery ID",
        "description": "identifier for a brewery, in the Beer Advocate database",
        "data_type": "ExternalId"
    },
    "P2905": {
        "label": "RateBeer brewery ID",
        "description": "identifier for a brewery, in the RateBeer database",
        "data_type": "ExternalId"
    },
    "P3199": {
        "label": "charity number (Isle of Man)",
        "description": "Charity number of a charity or charitable organisation registered on the Isle of Man",
        "data_type": "ExternalId"
    },
    "P3551": {
        "label": "Australian Registered Body Number",
        "description": "unique identifier for an organisation body (often a foreign business enterprise) registered in Australia",
        "data_type": "ExternalId"
    },
    "P3193": {
        "label": "GS1 Manufacturer code",
        "description": "codes for a company or organisation, used in GS1 identifiers: GTIN, GLN, GRAI, etc (barcodes and RFID)",
        "data_type": "ExternalId"
    },
    "P3206": {
        "label": "data.gouv.fr organisation ID",
        "description": "identifier of an organization on the French national open-data platform data.gouv.fr",
        "data_type": "ExternalId"
    },
    "P3057": {
        "label": "Charity Commission no.",
        "description": "official number of a charity registered with the Charity Commission of England and Wales",
        "data_type": "ExternalId"
    },
    "P2628": {
        "label": "German tax authority ID",
        "description": "numerical identifier for a local German tax office; forms part of the Steuernummer, assigned to individual taxpayers by that authority",
        "data_type": "ExternalId"
    },
    "P4897": {
        "label": "AICTE institute application ID",
        "description": "identifier number issued annually for institutions who need to re-apply for accreditation by the All India Council for Technical Education",
        "data_type": "ExternalId"
    },
    "P3534": {
        "label": "Australian Government Organisations Register ID",
        "description": "identifier of an Australian government organisation within the now discontinued Australian Government Organisations Register",
        "data_type": "ExternalId"
    },
    "P4264": {
        "label": "LinkedIn company ID",
        "description": "identifier for an official company, school, or organisation page, on LinkedIn",
        "data_type": "ExternalId"
    },
    "P2524": {
        "label": "SEED number",
        "description": "identifier for a school in Scotland, in the Scottish Government's SEED database",
        "data_type": "ExternalId"
    },
    "P4058": {
        "label": "FINESS ID",
        "description": "identifier of a medical facility in France in the FINESS directory",
        "data_type": "ExternalId"
    },
    "P2814": {
        "label": "P-number",
        "description": "Branch identifier from the Danish company register for linking physical sites to an organization",
        "data_type": "ExternalId"
    },
    "P5217": {
        "label": "Spyur ID",
        "description": "identifier for a company or organisation in Armenia, in the Spyur directory",
        "data_type": "ExternalId"
    },
    "P3273": {
        "label": "Actorenregister ID",
        "description": "unique identifier in the Dutch Actorenregister database",
        "data_type": "ExternalId"
    },
    "P3393": {
        "label": "LittleSis organization ID",
        "description": "organisation in the LittleSis who-knows-who of government and business database",
        "data_type": "ExternalId"
    },
    "P5858": {
        "label": "Index Herbariorum code",
        "description": "code in Index Herbariorum for plant collections",
        "data_type": "ExternalId"
    },
    "P3603": {
        "label": "Minneapolis Institute of Art constituent ID",
        "description": "identifier assigned to a person or organization by the Minneapolis Institute of Art",
        "data_type": "ExternalId"
    },
    "P4993": {
        "label": "Hungarian public body ID",
        "description": "unique identified for public bodies part in Hungary",
        "data_type": "ExternalId"
    },
    "P3500": {
        "label": "Ringgold ID",
        "description": "identifier for organisations in the publishing industry supply chain",
        "data_type": "ExternalId"
    },
    "P4771": {
        "label": "compArt institution ID",
        "description": "identifier for an institution in the compArt database Digital Art",
        "data_type": "ExternalId"
    },
    "P4031": {
        "label": "Hungarian NGO ID",
        "description": "registration number for Hungarian NGO",
        "data_type": "ExternalId"
    },
    "P2486": {
        "label": "FMD brand ID",
        "description": "external link to a brand's profile at the FMD (Fashion Model Directory) website",
        "data_type": "ExternalId"
    },
    "P4473": {
        "label": "Melon artist ID",
        "description": "identifier for an artist on Melon",
        "data_type": "ExternalId"
    },
    "P4971": {
        "label": "UK Provider Reference Number",
        "description": "identifier for an educational organization issued by the UK Register of Learning Providers",
        "data_type": "ExternalId"
    },
    "P3200": {
        "label": "Northern Ireland charity number",
        "description": "official number of a charity registered with the Charity Commission for Northern Ireland",
        "data_type": "ExternalId"
    },
    "P4090": {
        "label": "Biodiversity Repository ID",
        "description": "identifier of a repository, in the Biodiversity Repository database",
        "data_type": "ExternalId"
    },
    "P3002": {
        "label": "Untappd brewery ID",
        "description": "numerical identifier for a brewery, in the Untappd database",
        "data_type": "ExternalId"
    },
    "P3163": {
        "label": "Scottish Charity number",
        "description": "official number of a charity registered with the Scottish Charity Regulator",
        "data_type": "ExternalId"
    },
    "P3068": {
        "label": "WIPO ST.3",
        "description": "World Intellectual Property Organization standard for the two-letter codes of countries and certain organizations",
        "data_type": "ExternalId"
    },
    "P4563": {
        "label": "Art Museum of Estonia artist ID",
        "description": "artist identifier for the Art Museum of Estonia",
        "data_type": "ExternalId"
    },
    "P3234": {
        "label": "Library of Congress Cultural Heritage Organizations",
        "description": "a system of classification for Cultural Heritage Organizations developed by the Library of Congress",
        "data_type": "ExternalId"
    },
    "P8417": {
        "label": "Group Properties wiki ID",
        "description": "identifier for an article on the Group Properties wiki",
        "data_type": "ExternalId"
    },
    "P8974": {
        "label": "SAPA ID",
        "description": "identifier used by the SAPA Foundation, Swiss Archive of the Performing Arts",
        "data_type": "ExternalId"
    },
    "P8354": {
        "label": "BitterDB compound ID",
        "description": "identifier of chemical compound in BitterDB",
        "data_type": "ExternalId"
    },
    "P8899": {
        "label": "Swedish National Library Arken ID",
        "description": "identifier of an entry in the Arken website maintained by the National Library of Sweden",
        "data_type": "ExternalId"
    },
    "P8803": {
        "label": "stolpersteine-berlin.de Stolperstein identifier",
        "description": "identifier for Stolpersteine in Berlin issued by stolpersteine-berlin.de",
        "data_type": "ExternalId"
    },
    "P8804": {
        "label": "stolpersteine-hamburg.de Stolperstein identifier",
        "description": "identifier for Stolpersteine in Hamburg issued by stolpersteine-hamburg.de",
        "data_type": "ExternalId"
    },
    "P8806": {
        "label": "NSDOK Stolperstein identifier",
        "description": "identifier for Stolpersteine in Cologne issued by the NS-Documentation Center of the City of Cologne",
        "data_type": "ExternalId"
    },
    "P8807": {
        "label": "stolpersteine-bremen.de Stolperstein identifier",
        "description": "identifier for Stolpersteine in Bremen issued by stolpersteine-bremen.de",
        "data_type": "ExternalId"
    },
    "P8808": {
        "label": "snublestein.no Stolperstein identifier",
        "description": "identifier for Stolpersteine in Norway issued by the Oslo Jewish Museum",
        "data_type": "ExternalId"
    },
    "P8699": {
        "label": "NSW State Archives and Records Authority ID",
        "description": "identifier in the URL for items by the NSW State Archives and Records Authority",
        "data_type": "ExternalId"
    },
    "P8525": {
        "label": "ToS;DR service numerical identifier",
        "description": "number used by Terms of Service; Didn't Read to refer to a web-accessible service with a web-accessible ToS (P7014), privacy policy (P7101), or similar legal document",
        "data_type": "ExternalId"
    },
    "P8742": {
        "label": "PIM award ID",
        "description": "identifier of the Petőfi Literary Museum's database of awards and honors in Hungary",
        "data_type": "ExternalId"
    },
    "P8260": {
        "label": "Aberdeen Built Ships ID",
        "description": "identifier for a maritime vessel, in the Aberdeen Built Ships database",
        "data_type": "ExternalId"
    },
    "P8304": {
        "label": "NexusMods game ID",
        "description": "identifier for a video game on the modding website NexusMods",
        "data_type": "ExternalId"
    },
    "P8332": {
        "label": "Dansk litteraturs historie ID",
        "description": "identifier of an article in the online encyclopedia dansklitteraturshistorie.lex.dk",
        "data_type": "ExternalId"
    },
    "P8334": {
        "label": "MuseScore artist ID",
        "description": "identifier for an artist on MuseScore",
        "data_type": "ExternalId"
    },
    "P8642": {
        "label": "FBref.com squad ID",
        "description": "identifier for an association football team on FBref.com",
        "data_type": "ExternalId"
    },
    "P8814": {
        "label": "WordNet 3.1 Synset ID",
        "description": "Synset identifier in Princeton’s WordNet Version 3.1",
        "data_type": "ExternalId"
    },
    "P8326": {
        "label": "Australian Classification ID",
        "description": "identifier for a creative work on the Australian Classification Board's official website",
        "data_type": "ExternalId"
    },
    "P8775": {
        "label": "Soccerway stadium ID",
        "description": "identifier for an association football (soccer) stadium in the Soccerway database",
        "data_type": "ExternalId"
    },
    "P8295": {
        "label": "AustLit ID",
        "description": "identifier at AustLit",
        "data_type": "ExternalId"
    },
    "P8826": {
        "label": "edition humboldt digital ID",
        "description": "identifier for the register of biographies, places and institutions in edition humboldt digital",
        "data_type": "ExternalId"
    },
    "P8846": {
        "label": "FISG athlete ID",
        "description": "identifier for an athlete at FISG official site",
        "data_type": "ExternalId"
    },
    "P8285": {
        "label": "LGBT Danmark online dictionary ID",
        "description": "identifier in LGBT Danmark's online dictionary",
        "data_type": "ExternalId"
    },
    "P8372": {
        "label": "Web of Science ID",
        "description": "unique academic work identifier assigned in Web of Science bibliographic database (Web of Science Core Collection only and do not include WOS: prefix)",
        "data_type": "ExternalId"
    },
    "P8255": {
        "label": "SNES central game ID",
        "description": "identifier of a video game in the SNES central database",
        "data_type": "ExternalId"
    },
    "P8256": {
        "label": "Super Mario Wiki ID",
        "description": "identifier for a Super Mario Wiki article",
        "data_type": "ExternalId"
    },
    "P8886": {
        "label": "CIP data sheet",
        "description": "CIP data sheet of the object",
        "data_type": "ExternalId"
    },
    "P8526": {
        "label": "Have I Been Pwned breach ID",
        "description": "identifier for a data breach in haveibeenpwned.com",
        "data_type": "ExternalId"
    },
    "P8528": {
        "label": "EIB project ID",
        "description": "identifer of a project either funded by or planned to be funded by the European Investment Bank",
        "data_type": "ExternalId"
    },
    "P8300": {
        "label": "Montreux Jazz Festival concert ID",
        "description": "identifier of a concert in the Montreux Jazz Festival database",
        "data_type": "ExternalId"
    },
    "P8302": {
        "label": "Montreux Jazz Festival song ID",
        "description": "identifier of a song performed during the Montreux Jazz Festival",
        "data_type": "ExternalId"
    },
    "P8408": {
        "label": "KBpedia ID",
        "description": "identifier for the KBpedia knowledge graph, which provides consistent mappings across seven large-scale public knowledge bases including Wikidata, and is used to promote data interoperability and extraction of training sets for machine learning",
        "data_type": "ExternalId"
    },
    "P8420": {
        "label": "GameBanana video game ID",
        "description": "identifier for a video game on GameBanana",
        "data_type": "ExternalId"
    },
    "P8401": {
        "label": "thefreedictionary medical term ID",
        "description": "identifier for a term in the medical-dictionary of TheFreeDictionary by Farlex Inc.",
        "data_type": "ExternalId"
    },
    "P8855": {
        "label": "IEV number",
        "description": "reference number in the International Electrotechnical Vocabulary",
        "data_type": "ExternalId"
    },
    "P8507": {
        "label": "Legacy.com newspaper ID",
        "description": "identifier for newspapers that have their obituaries indexed on Legacy.com",
        "data_type": "ExternalId"
    },
    "P8784": {
        "label": "AAGM site ID",
        "description": "identifier for a site in the AAGM catalogue",
        "data_type": "ExternalId"
    },
    "P8994": {
        "label": "Game UI Database ID",
        "description": "numerical identifier for a video game in the Game UI Database",
        "data_type": "ExternalId"
    },
    "P8711": {
        "label": "Emojipedia ID",
        "description": "ID of a page on Emojipedia",
        "data_type": "ExternalId"
    },
    "P8785": {
        "label": "AniDB tag ID",
        "description": "identifier for a tag in the AniDB database",
        "data_type": "ExternalId"
    },
    "P8364": {
        "label": "Deku Deals ID",
        "description": "identifier of a video game in the Deku Deals website",
        "data_type": "ExternalId"
    },
    "P8478": {
        "label": "Svenska Akademiens Ordbok ID",
        "description": "unique identifier of the lexeme in Svenska Akademiens Ordbok (SAOB)",
        "data_type": "ExternalId"
    },
    "P8762": {
        "label": "NCAA Statistics team season ID",
        "description": "identifier for a year of a college sports team on stats.ncaa.org",
        "data_type": "ExternalId"
    },
    "P8858": {
        "label": "Drizly product ID",
        "description": "identifier of the product in the Drizly database",
        "data_type": "ExternalId"
    },
    "P9013": {
        "label": "Encyclopedia of Saami Culture ID",
        "description": "identifier for an article in the Encyclopedia of Saami Culture",
        "data_type": "ExternalId"
    },
    "P8659": {
        "label": "TDD number",
        "description": "TDD number (textphone/teletype for deaf people) in standard format (RFC3966), without 'tel:' prefix",
        "data_type": "String"
    },
    "P8973": {
        "label": "Humanitarian OpenStreetMap Tasking Manager project ID",
        "description": "identifier for a project in the Humanitarian OpenStreetMap Tasking Manager",
        "data_type": "ExternalId"
    },
    "P8965": {
        "label": "OpenReview.net group ID",
        "description": "idenfifier for a (group of) events",
        "data_type": "ExternalId"
    },
    "P8769": {
        "label": "Ontology of units of Measure 2.0 unit ID",
        "data_type": "ExternalId"
    },
    "P8488": {
        "label": "ASCCEG 2019 ID",
        "description": "identifier in the Australian Standard Classification of Cultural and Ethnic Groups 2019",
        "data_type": "ExternalId"
    },
    "P8455": {
        "label": "US Quaternary Fault ID",
        "description": "ID Number of Quaternary Fault in the US as assigned by the United States Geological Survey",
        "data_type": "ExternalId"
    },
    "P8313": {
        "label": "Den Store Danske ID",
        "description": "identifier of an article in the online encyclopedia denstoredanske.lex.dk",
        "data_type": "ExternalId"
    },
    "P8834": {
        "label": "MetaSat ID",
        "description": "identifier for MetaSat, an open metadata vocabulary used to describe space missions",
        "data_type": "ExternalId"
    },
    "P8560": {
        "label": "PARADISEC Catalog",
        "description": "identifier of one of several languages on PARADISEC (the Pacific And Regional Archive for Digital Sources in Endangered Cultures)",
        "data_type": "ExternalId"
    },
    "P8561": {
        "label": "NPR station ID",
        "description": "identifier for a National Public Radio member station",
        "data_type": "ExternalId"
    },
    "P8662": {
        "label": "Les Roses cultivées à L'Haÿ en 1902 ID",
        "description": "identifier in \"Les Roses cultivées à L'Haÿ en 1902: essai de classement\" by Jules Gravereaux",
        "data_type": "ExternalId"
    },
    "P8584": {
        "label": "Wikipedia Library partner ID",
        "description": "numerical identifier for a resource in the Wikipedia Library Card Platform",
        "data_type": "ExternalId"
    },
    "P8261": {
        "label": "Origin ID",
        "description": "identifier for an game on Origin site",
        "data_type": "ExternalId"
    },
    "P8999": {
        "label": "Lexikon zum Literatur- und Kulturbetrieb im Österreich der Zwischenkriegszeit ID",
        "description": "alphabetical identifier for an article in the Lexikon zum Literatur- und Kulturbetrieb im Österreich der Zwischenkriegszeit",
        "data_type": "ExternalId"
    },
    "P8349": {
        "label": "Proleksis enciklopedija ID",
        "description": "identifier for a topic in Proleksis enciklopedija",
        "data_type": "ExternalId"
    },
    "P9000": {
        "label": "Ancient History Encyclopedia ID",
        "description": "alphabetical identifier for an article in the Ancient History Encyclopedia",
        "data_type": "ExternalId"
    },
    "P8351": {
        "label": "vglist video game ID",
        "description": "identifier for a video game on vglist",
        "data_type": "ExternalId"
    },
    "P8520": {
        "label": "TCLF ID",
        "description": "identifier for a person or an historic site on the Cultural Landscape Foundation website",
        "data_type": "ExternalId"
    },
    "P8521": {
        "label": "Sculptures and cities database ID for sculptures",
        "description": "identifier for sculptures in the Sculptures and cities database",
        "data_type": "ExternalId"
    },
    "P8648": {
        "label": "Ministry of the Interior of Italy ID",
        "description": "identifier of a politician in the Italian Ministry of the Interior's database",
        "data_type": "ExternalId"
    },
    "P8629": {
        "label": "Occupational Outlook Handbook ID",
        "description": "identifier for occupations in the Occupational Outlook Handbook",
        "data_type": "ExternalId"
    },
    "P8494": {
        "label": "DSSTOX compound identifier",
        "description": "identifier of compound in DSSTOX",
        "data_type": "ExternalId"
    },
    "P8266": {
        "label": "FEMA number",
        "description": "identifier for flavor ingredients",
        "data_type": "ExternalId"
    },
    "P8586": {
        "label": "Anais do Museu Paulista article ID",
        "description": "identifier for a scholarly article in the Anais do Museu Paulista",
        "data_type": "ExternalId"
    },
    "P8303": {
        "label": "ESRB game ID",
        "description": "identifier for a video game on ESRB's official website",
        "data_type": "ExternalId"
    },
    "P8309": {
        "label": "Yle topic ID",
        "description": "identifier for topics in Finland's national broadcaster Yle's web services",
        "data_type": "ExternalId"
    },
    "P8954": {
        "label": "vets.cz ID",
        "description": "identifier of a memorial in vets.cz",
        "data_type": "ExternalId"
    },
    "P8708": {
        "label": "Danish educational institution number",
        "description": "number for Danish educational institutions",
        "data_type": "ExternalId"
    },
    "P8279": {
        "label": "Absolute Games game ID",
        "description": "identifier for a videogame at the Absolute Games website",
        "data_type": "ExternalId"
    },
    "P8913": {
        "label": "NSSDCA ID",
        "description": "external identifier of records within the Master Catalog of the NASA Space Science Data Coordinated Archive (NSSDCA), similar but not synonymous with the COSPAR ID (P247)",
        "data_type": "ExternalId"
    },
    "P8799": {
        "label": "Twitch tag ID",
        "description": "identifier for a tag on Twitch",
        "data_type": "ExternalId"
    },
    "P8614": {
        "label": "Slack organization ID",
        "description": "unique portion of the group or organization URL on Slack",
        "data_type": "ExternalId"
    },
    "P8346": {
        "label": "Nomor Pokok Wajib Pajak (organisation or corporation)",
        "description": "Indonesian tax identifier known as NPWP",
        "data_type": "ExternalId"
    },
    "P8392": {
        "label": "Rate Your Music release ID",
        "description": "identifier for a music release on Rate Your Music site",
        "data_type": "ExternalId"
    },
    "P8836": {
        "label": "SPOnG game ID",
        "description": "identifier of a video game in the SPOnG database",
        "data_type": "ExternalId"
    },
    "P8318": {
        "label": "WorldFootball.net stadium ID",
        "description": "identifier for a association football (soccer) stadium in the WorldFootball.net database",
        "data_type": "ExternalId"
    },
    "P8319": {
        "label": "National Football Teams.com match ID",
        "description": "identifier for an association football (soccer) match in the National Football Teams.com database",
        "data_type": "ExternalId"
    },
    "P8320": {
        "label": "EU-Football.info stadium ID",
        "description": "identifier for a association football (soccer) stadium in the EU-Football.info database",
        "data_type": "ExternalId"
    },
    "P8782": {
        "label": "ClassInd game ID",
        "description": "identifier for a video game in the Classificação Indicativa database",
        "data_type": "ExternalId"
    },
    "P9279": {
        "label": "Egapro gender equality index",
        "description": "index value for a company per regulations in France",
        "data_type": "Quantity"
    },
    "P6378": {
        "label": "charter URL",
        "description": "link to an organization's charter documents, including articles or certificate of association, incorporation or organization, or its by-laws",
        "data_type": "Url"
    },
    "P2643": {
        "label": "Carnegie Classification of Institutions of Higher Education",
        "description": "classification of colleges and universities in the United States",
        "data_type": "WikibaseItem"
    },
    "P1027": {
        "label": "conferred by",
        "description": "person or organization who grants an award, certification, or role",
        "data_type": "WikibaseItem"
    },
    "P2137": {
        "label": "total equity",
        "description": "amount of equity value for an entity",
        "data_type": "Quantity"
    },
    "P5514": {
        "label": "accredited by",
        "description": "optionally use together with \"subject has role\" (P2868)",
        "data_type": "WikibaseItem"
    },
    "P1408": {
        "label": "licensed to broadcast to",
        "description": "place that a radio/TV station is licensed/required to broadcast to",
        "data_type": "WikibaseItem"
    },
    "P7169": {
        "label": "substitute director/manager",
        "description": "substitute of the director/manager of an organization",
        "data_type": "WikibaseItem"
    },
    "P1549": {
        "label": "demonym",
        "description": "demonym (proper noun) for people or things associated with a given place, usually based off the placename; multiple entries with qualifiers to distinguish are used to list variant forms by reason of grammatical gender or plurality.",
        "data_type": "Monolingualtext"
    },
    "P1410": {
        "label": "number of representatives in an organization/legislature or won in elections",
        "description": "number of seats a political party, faction, or group has in a given legislature or won in elections",
        "data_type": "Quantity"
    },
    "P2541": {
        "label": "operating area",
        "description": "geographic area or jurisdiction an organisation or industry operates in, serves, or has responsibility for",
        "data_type": "WikibaseItem"
    },
    "P1998": {
        "label": "UCI code",
        "description": "code uniquely identifying a cycling team",
        "data_type": "String"
    },
    "P3300": {
        "label": "musical conductor",
        "description": "the person who directs a musical group, orchestra or chorus",
        "data_type": "WikibaseItem"
    },
    "P1533": {
        "label": "family name identical to this given name",
        "description": "last name that is the same as a given first name. Use on items for given names",
        "data_type": "WikibaseItem"
    },
    "P1462": {
        "label": "standards body",
        "description": "organisation that published or maintains the standard governing an item",
        "data_type": "WikibaseItem"
    },
    "P2238": {
        "label": "official symbol",
        "description": "official symbol of an organisation",
        "data_type": "WikibaseItem"
    },
    "P8938": {
        "label": "artistic director",
        "description": "person responsible for the overall artistic vision of an organization and for its execution",
        "data_type": "WikibaseItem"
    },
    "P6589": {
        "label": "endowment",
        "description": "sum of donations for the ongoing support of a not-for-profit institution",
        "data_type": "Quantity"
    },
    "P3989": {
        "label": "members have occupation",
        "description": "all members of this group share the occupation",
        "data_type": "WikibaseItem"
    },
    "P6840": {
        "label": "student organization of",
        "description": "the members of this organization (club or society) are students of the target college or university",
        "data_type": "WikibaseItem"
    },
    "P1448": {
        "label": "official name",
        "description": "official name of the subject in its official language(s)",
        "data_type": "Monolingualtext"
    },
    "P3461": {
        "label": "designated as terrorist by",
        "description": "country or organization that has officially designated a given group as a terrorist organization (e.g. for India, listed on http://mha.nic.in/BO )",
        "data_type": "WikibaseItem"
    },
    "P3975": {
        "label": "secretary general",
        "description": "leader of a political or international organization, sometimes below the chairperson (P488)",
        "data_type": "WikibaseItem"
    },
    "P3268": {
        "label": "PORT organization URL",
        "description": "URL of an organization (venue, band, etc.) in the PORT-network database",
        "data_type": "Url"
    },
    "P138": {
        "label": "named after",
        "description": "entity or event that inspired the subject's name, or namesake (in at least one language). Qualifier \"applies to name\" (P5168) can be used to indicate which one",
        "data_type": "WikibaseItem"
    },
    "P749": {
        "label": "parent organization",
        "description": "parent organization of an organization, opposite of subsidiaries (P355)",
        "data_type": "WikibaseItem"
    },
    "P2124": {
        "label": "member count",
        "description": "number of people who are part of an organization at a particular time",
        "data_type": "Quantity"
    },
    "P2388": {
        "label": "office held by head of the organization",
        "description": "position of the head of this item",
        "data_type": "WikibaseItem"
    },
    "P1313": {
        "label": "office held by head of government",
        "description": "political office that is fulfilled by the head of the government of this item",
        "data_type": "WikibaseItem"
    },
    "P2196": {
        "label": "students count",
        "description": "number of students of any type in an educational organization. Qualify with \" point in time\" (P585). The most recent count would generally have preferred rank; data for previous years normal rank (not deprecated rank). Don't overwrite.",
        "data_type": "Quantity"
    },
    "P4379": {
        "label": "youth wing",
        "description": "group of younger people associated with this organization",
        "data_type": "WikibaseItem"
    },
    "P5822": {
        "label": "admission rate",
        "description": "ratio of the number of applicants admitted to the total number of applicants",
        "data_type": "Quantity"
    },
    "P3320": {
        "label": "board member",
        "description": "member(s) of the board for the organization",
        "data_type": "WikibaseItem"
    },
    "P2392": {
        "label": "teaching method",
        "description": "teaching method adopted by an educational institution",
        "data_type": "WikibaseItem"
    },
    "P1640": {
        "label": "curator",
        "description": "content specialist responsible for this collection or exhibition",
        "data_type": "WikibaseItem"
    },
    "P8786": {
        "label": "dedicated heritage entity",
        "description": "heritage entity dedicated to or associated with the item's subject",
        "data_type": "WikibaseItem"
    },
    "P2012": {
        "label": "cuisine",
        "description": "type of food served by a restaurant or restaurant chain",
        "data_type": "WikibaseItem"
    },
    "P4290": {
        "label": "official app",
        "description": "official app for this organization or other entity (a downloadable application for Android, iPhone etc.)",
        "data_type": "WikibaseItem"
    },
    "P5052": {
        "label": "supervisory board member",
        "description": "member(s) of the supervisory board for the organization",
        "data_type": "WikibaseItem"
    },
    "P8768": {
        "label": "online catalog",
        "data_type": "Url"
    },
    "P6733": {
        "label": "National Taxonomy of Exempt Entities code",
        "description": "classification code for American nonprofit organizations",
        "data_type": "String"
    },
    "P6291": {
        "label": "advertises",
        "description": "cause, organization or product targeted by the advertising campaign, character, jingle or slogan",
        "data_type": "WikibaseItem"
    },
    "P4586": {
        "label": "type foundry",
        "description": "company releasing or distributing a font or typeface",
        "data_type": "WikibaseItem"
    },
    "P1037": {
        "label": "director / manager",
        "description": "person who manages any kind of group",
        "data_type": "WikibaseItem"
    },
    "P3362": {
        "label": "operating income",
        "description": "private entity operating income",
        "data_type": "Quantity"
    },
    "P1923": {
        "label": "participating team",
        "description": "Like 'Participant' (P710) but for teams. For an event like a cycle race or a football match you can use this property to list the teams and P710 to list the individuals (with 'member of sports team' (P54) as a qualifier for the individuals)",
        "data_type": "WikibaseItem"
    },
    "P2828": {
        "label": "corporate officer",
        "description": "person who holds a specific position",
        "data_type": "WikibaseItem"
    },
    "P2133": {
        "label": "total debt",
        "description": "amount of debt for a company, organization or public entity",
        "data_type": "Quantity"
    },
    "P807": {
        "label": "separated from",
        "description": "subject was founded or started by separating from identified object",
        "data_type": "WikibaseItem"
    },
    "P7936": {
        "label": "business model",
        "description": "business model a business enterprise or creative work operates under",
        "data_type": "WikibaseItem"
    },
    "P6346": {
        "label": "statutory purpose",
        "description": "main statutory purpose of an organization as defined in its articles of association",
        "data_type": "Monolingualtext"
    },
    "P6125": {
        "label": "number of volunteers",
        "description": "number of people who volunteer for an organization at a particular time",
        "data_type": "Quantity"
    },
    "P2139": {
        "label": "total revenue",
        "description": "income gained by an organization during a given time frame. Not to be confused with fiscal revenue",
        "data_type": "Quantity"
    },
    "P797": {
        "label": "authority",
        "description": "entity having executive power on given entity",
        "data_type": "WikibaseItem"
    },
    "P1789": {
        "label": "chief operating officer",
        "description": "the chief operating officer of an organization",
        "data_type": "WikibaseItem"
    },
    "P2652": {
        "label": "partnership with",
        "description": "commercial partnership between this organization and another organization or institution",
        "data_type": "WikibaseItem"
    },
    "P2813": {
        "label": "house publication",
        "description": "media that speaks for an organization or a movement, and that is usually edited by its members",
        "data_type": "WikibaseItem"
    },
    "P7251": {
        "label": "Identification code for Japanese universities and colleges",
        "description": "Identification code for universities and colleges in Japan. This code is defined in Japanese Industrial Standards.",
        "data_type": "ExternalId"
    },
    "P1342": {
        "label": "number of seats",
        "description": "total number of seats/members in an assembly (legislative house or similar)",
        "data_type": "Quantity"
    },
    "P9048": {
        "label": "Biographical Archive of Psychiatry ID",
        "description": "identifier for an article in the Biographical Archive of Psychiatry",
        "data_type": "ExternalId"
    },
    "P9324": {
        "label": "BiographySampo person ID",
        "description": "identifier for a person in the BiographySampo service",
        "data_type": "ExternalId"
    },
    "P9026": {
        "label": "SFLI ID",
        "description": "identifier for a person who is member of the Società dei Filologi della Letteratura Italiana",
        "data_type": "ExternalId"
    },
    "P9223": {
        "label": "Provenio UUID",
        "description": "UUID of a book owner in the Provenance Database of Book Owners and the Book Provenance Records",
        "data_type": "ExternalId"
    },
    "P9233": {
        "label": "XXXBios female performer ID",
        "description": "identifier for a female porn performer in the XXXBios database",
        "data_type": "ExternalId"
    },
    "P7964": {
        "label": "corresponding HTML autocomplete attribute",
        "description": "the appropriate value of the autocomplete attribute in HTML",
        "data_type": "String"
    },
    "P9181": {
        "label": "Germanistenverzeichnis ID",
        "description": "identifier for a person in the international directory of German studies scholars",
        "data_type": "ExternalId"
    },
    "P9036": {
        "label": "I professori dell'Università di Pavia (1859-1961) ID",
        "description": "numerical identifier for a university teacher on the website I professori dell'Università di Pavia",
        "data_type": "ExternalId"
    },
    "P9420": {
        "label": "CNN contributor ID",
        "description": "alphabetical identifier for a person on the CNN website",
        "data_type": "ExternalId"
    },
    "P5977": {
        "label": "Wikidata property example for senses",
        "description": "example where this Wikidata property is used; target sense is one that would use this property, with qualifier the property being described given the associated value",
        "data_type": "WikibaseSense"
    },
    "P9329": {
        "label": "Svoya Igra television player ID",
        "description": "identifier for a Svoya Igra TV show player at the svoya-igra.org",
        "data_type": "ExternalId"
    },
    "P9287": {
        "label": "Cyprus Bibliography ID",
        "description": "identifier for an authority in the Cyprus Bibliography (catalogue of all Cypriot publications after 1980)",
        "data_type": "ExternalId"
    },
    "P9169": {
        "label": "Tree of Life contributor ID",
        "description": "identifier for a contributor to the Tree of Life Web Project",
        "data_type": "ExternalId"
    },
    "P5192": {
        "label": "Wikidata property example for lexemes",
        "description": "example where this Wikidata property is used; target lexeme is one that would use this property, with qualifier the property being described given the associated value",
        "data_type": "WikibaseLexeme"
    },
    "P9129": {
        "label": "Data Collection of the Hungarian Peerage ID",
        "description": "identifier of a person in the database of the Hungarian Society of Family History Research",
        "data_type": "ExternalId"
    },
    "P1659": {
        "label": "see also",
        "description": "used to indicate another property that might provide additional information about the Wikidata-property",
        "data_type": "WikibaseProperty"
    },
    "P9081": {
        "label": "SEARCH on line catalogue ID",
        "description": "identifier for an authority in the SEARCH on line catalogue (union-catalog of four libraries in Rome)",
        "data_type": "ExternalId"
    },
    "P9130": {
        "label": "CompositionToday composers showcase ID",
        "description": "identifier for a composer showcase on the CompositionToday website",
        "data_type": "ExternalId"
    },
    "P9246": {
        "label": "Indigenous Australia ID",
        "description": "identifier for a person in Indigenous Australia",
        "data_type": "ExternalId"
    },
    "P9110": {
        "label": "Le Lettere di Theodor Mommsen agli Italiani ID",
        "description": "identifier for a person cited in a letter by Theodor Mommsen in the online database Le Lettere di Theodor Mommsen agli Italiani",
        "data_type": "ExternalId"
    },
    "P9097": {
        "label": "American Academy in Rome ID",
        "description": "identifier for an authority in the library catalogue of the American Academy in Rome",
        "data_type": "ExternalId"
    },
    "P9155": {
        "label": "Joods Monument ID",
        "description": "identifier in the Joods Monument database of Dutch Jews killed in the Holocaust",
        "data_type": "ExternalId"
    },
    "P9039": {
        "label": "Studium Parisiense ID",
        "description": "identifier for a person connected to the University of Paris between the 12th and 16th century in the database Studium Parisiense",
        "data_type": "ExternalId"
    },
    "P9289": {
        "label": "Filmitalia person ID",
        "description": "identifier used on filmitalia.org for the filmography of individuals",
        "data_type": "ExternalId"
    },
    "P2429": {
        "label": "expected completeness",
        "description": "describes whether a property is intended to represent a complete set of real-world items having that property",
        "data_type": "WikibaseItem"
    },
    "P9174": {
        "label": "XXXBios transgender performer ID",
        "description": "identifier for a transgender porn performer in the XXXBios database",
        "data_type": "ExternalId"
    },
    "P5193": {
        "label": "Wikidata property example for forms",
        "description": "example where this Wikidata property is used; target form is one that would use this property, with qualifier the property being described given the associated value",
        "data_type": "WikibaseForm"
    },
    "P9187": {
        "label": "EuroBabeIndex.com ID",
        "description": "alphabetical identifier for a porn actress on the EuroBabeIndex.com website",
        "data_type": "ExternalId"
    },
    "P9156": {
        "label": "Lumni person ID",
        "description": "identifier for a person on the Lumni website, an online French platform for audiovisual teaching materials",
        "data_type": "ExternalId"
    },
    "P9111": {
        "label": "SISSCO ID",
        "description": "identifier for a person who is member of the Società Italiana per lo Studio della Storia Contemporanea",
        "data_type": "ExternalId"
    },
    "P9041": {
        "label": "DiVo person ID",
        "description": "identifier of a person in the Dizionario dei Volgarizzamenti",
        "data_type": "ExternalId"
    },
    "P9265": {
        "label": "Physics Magazine author ID",
        "description": "identifier for an author in the online magazine Physics",
        "data_type": "ExternalId"
    },
    "P9333": {
        "label": "LFH staff ID",
        "description": "identifier for staff on the website of the French Ligue féminine de handball",
        "data_type": "ExternalId"
    },
    "P9315": {
        "label": "Enslaved person ID",
        "description": "alphanumerical identifier for a person on the Enslaved website",
        "data_type": "ExternalId"
    },
    "P9054": {
        "label": "National Historic People ID",
        "description": "identifier for Persons of National Historic Significance used by Parks Canada",
        "data_type": "ExternalId"
    },
    "P9049": {
        "label": "ICCD agent ID",
        "description": "identifier for a person or an organisation in the General catalog of Italian cultural heritage",
        "data_type": "ExternalId"
    },
    "P9044": {
        "label": "GEPRIS-Historisch ID (Person)",
        "description": "identifier of a person in GEPRIS Historisch database of funded research projects of Deutsche Forschungsgemeinschaft (DFG, German Research Foundation)",
        "data_type": "ExternalId"
    },
    "P9134": {
        "label": "Studio di Fonologia Musicale RAI person ID",
        "description": "identifier for a person in the website of the Studio di Fonologia Musicale RAI",
        "data_type": "ExternalId"
    },
    "P9179": {
        "label": "Biblioteca Iglesia Nacional Española en Roma ID",
        "description": "identifier for an authority in the catalogue of the Biblioteca Iglesia Nacional Española en Roma",
        "data_type": "ExternalId"
    },
    "P9247": {
        "label": "Pontifical University of Salamanca ID",
        "description": "identifier for an authority in the catalogue of the library of the Pontifical University of Salamanca",
        "data_type": "ExternalId"
    },
    "P9178": {
        "label": "Biblioteca Franco Serantini ID",
        "description": "identifier for an authority in the catalogue of the Italian library Biblioteca Franco Serantini",
        "data_type": "ExternalId"
    },
    "P9267": {
        "label": "ANPI person numeric ID",
        "description": "numeric identifier of a person of ANPI's Web portal",
        "data_type": "ExternalId"
    },
    "P9268": {
        "label": "Science Magazine author ID",
        "description": "identifier for a Science Magazine author",
        "data_type": "ExternalId"
    },
    "P9226": {
        "label": "ARAE ID",
        "description": "identifier for an entity (person, organization, family) in the Archivo de la Real Academia Española",
        "data_type": "ExternalId"
    },
    "P9298": {
        "label": "Encyclopedia of Plzeň person ID",
        "description": "identifier for people related to Plzeň City and its history",
        "data_type": "ExternalId"
    },
    "P9299": {
        "label": "OsobnostiRegionu.cz ID",
        "description": "identifier for OsobnostiRegionů.cz, a database of regionally important people in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P9300": {
        "label": "Terezín Memorial Database ID",
        "description": "identifier of The Terezín Memorial – Database of politically and racially persecuted persons",
        "data_type": "ExternalId"
    },
    "P9113": {
        "label": "Patrinum ID",
        "description": "identifier for a subject (person, organisation, place, etc.) in the Swiss database Patrinum regarding the Canton of Vaud",
        "data_type": "ExternalId"
    },
    "P9337": {
        "label": "Keratsini-Drapetsona libraries' catalogue authority ID",
        "description": "identifier for an authority in the catalogue of the libraries of Keratsini-Drapetsona Municipality (Piraeus regional unit, Athens, Attica, Greece)",
        "data_type": "ExternalId"
    },
    "P9114": {
        "label": "Mathematica Italiana person ID",
        "description": "identifier for a mathematician in the website Mathematica Italiana",
        "data_type": "ExternalId"
    },
    "P9319": {
        "label": "Dark Horse creator ID",
        "description": "identifier for creators (authors, artists, etc.) from Dark Horse Comics",
        "data_type": "ExternalId"
    },
    "P9412": {
        "label": "Arca author ID",
        "description": "identifier for a Medieval Portuguese author in the database Arca",
        "data_type": "ExternalId"
    },
    "P9413": {
        "label": "BLPL author ID",
        "description": "identifier for an author in BLPL (Digital Library of Literature from Lusophone Countries)",
        "data_type": "ExternalId"
    },
    "P1855": {
        "label": "Wikidata property example",
        "description": "example where this Wikidata property is used; target item is one that would use this property, with qualifier the property being described given the associated value",
        "data_type": "WikibaseItem"
    },
    "P8379": {
        "label": "qualifier for this property",
        "description": "qualifier that is commonly used with this property. Note the difference to \"allowed qualifiers constraint\" (all that can be used) and \"mandatory qualifier constraint\" (all that should be used)",
        "data_type": "WikibaseProperty"
    },
    "P2668": {
        "label": "stability of property value",
        "description": "likelihood that statements with this property will change",
        "data_type": "WikibaseItem"
    },
    "P1629": {
        "label": "subject item of this property",
        "description": "item corresponding to the concept represented by the property",
        "data_type": "WikibaseItem"
    },
    "P9199": {
        "label": "Protagonisti della storia delle scienze della mente ID",
        "description": "identifier for a psychologist or psychiatrist in the website Protagonisti della storia delle scienze della mente",
        "data_type": "ExternalId"
    },
    "P9164": {
        "label": "Svenska Institutet i Rom ID",
        "description": "identifier for an authority in the library catalogue of the Svenska Institutet i Rom",
        "data_type": "ExternalId"
    },
    "P9249": {
        "label": "Nauvoo Community Project ID",
        "description": "identifier for a person in the Nauvoo Community Project database",
        "data_type": "ExternalId"
    },
    "P1896": {
        "label": "source website for the property",
        "description": "URL of website acting as source of data for this Wikidata property",
        "data_type": "Url"
    },
    "P9305": {
        "label": "Max Movie person ID",
        "description": "identifier for a person on the Max Movie website",
        "data_type": "ExternalId"
    },
    "P6685": {
        "label": "Wikidata property example for media",
        "description": "example for the property applicable to media files on Wikimedia Commons (structured data for Commons). See Wikimedia Commons for full usage rules",
        "data_type": "CommonsMedia"
    },
    "P9136": {
        "label": "SOUNZ contributor ID",
        "description": "numerical identifier for persons or organisations listed in the SOUNZ database of New Zealand Music",
        "data_type": "ExternalId"
    },
    "P9250": {
        "label": "Cyprus Library ID",
        "description": "identifier for an authority in the catalogue of Cyprus Library (National Library of Cyprus)",
        "data_type": "ExternalId"
    },
    "P9251": {
        "label": "Cyprus University of Technology ID",
        "description": "identifier for an authority in the catalogue of the library of the Cyprus University of Technology",
        "data_type": "ExternalId"
    },
    "P9160": {
        "label": "Biographical Dictionary of the Czech Lands ID",
        "description": "identifier for a person in the Biographical Dictionary of the Czech Lands",
        "data_type": "ExternalId"
    },
    "P9252": {
        "label": "Amathus ID",
        "description": "identifier for an authority in the Cypriot union catalogue Amathus",
        "data_type": "ExternalId"
    },
    "P9253": {
        "label": "European University Cyprus ID",
        "description": "identifier for an authority in the catalogue of the library of the European University Cyprus",
        "data_type": "ExternalId"
    },
    "P9020": {
        "label": "AV Production person ID",
        "description": "identifier for a person in the website of AV Production",
        "data_type": "ExternalId"
    },
    "P1696": {
        "label": "inverse property",
        "description": "links a property to its inverse property (which relates the subject and object in reverse order)",
        "data_type": "WikibaseProperty"
    },
    "P9137": {
        "label": "Doktori.hu ID",
        "description": "identifier for Hungarian scientist (based on doctoral degree)",
        "data_type": "ExternalId"
    },
    "P9282": {
        "label": "Catalogue of the Capitular Library of Verona author ID",
        "description": "identifier for an author in the Catalogue of the Capitular Library of Verona",
        "data_type": "ExternalId"
    },
    "P9109": {
        "label": "Holocaust.cz person ID",
        "description": "identifier of a person in the database of holocaust victims (holocaust.cz)",
        "data_type": "ExternalId"
    },
    "P9352": {
        "label": "Portrait Archive ID",
        "description": "identifier for a person in the Portrait Archive",
        "data_type": "ExternalId"
    },
    "P9276": {
        "label": "Encyclopedia of Ostrava History person ID",
        "description": "identifier for people related to the city of Ostrava (Czech Republic) and its history",
        "data_type": "ExternalId"
    },
    "P9371": {
        "label": "FranceArchives agent ID",
        "description": "identifier for a person on FranceArchives",
        "data_type": "ExternalId"
    },
    "P9117": {
        "label": "BVFE author ID",
        "description": "numerical identifier for an author in the Biblioteca Virtual de la Filología Española",
        "data_type": "ExternalId"
    },
    "P1963": {
        "label": "properties for this type",
        "description": "when this subject is used as object of \"instance of\" or \"occupation\", the following properties normally apply",
        "data_type": "WikibaseProperty"
    },
    "P2876": {
        "label": "type of unit for this property",
        "description": "units for this property are all instances of this item",
        "data_type": "WikibaseItem"
    },
    "P9404": {
        "label": "OFDb person ID",
        "description": "identifier for a person in OFDb (Online Filmdatenbank)",
        "data_type": "ExternalId"
    },
    "P2271": {
        "label": "Wikidata property example for properties",
        "description": "if the example entity is a property, not an item",
        "data_type": "WikibaseProperty"
    },
    "P8225": {
        "label": "is metaclass for",
        "description": "all instances of the subject are subclasses of the object",
        "data_type": "WikibaseItem"
    },
    "P3015": {
        "label": "backup or reserve team or crew",
        "description": "team or crew that is kept ready to act as reserve",
        "data_type": "WikibaseItem"
    },
    "P4808": {
        "label": "Royal Academy new identifier",
        "description": "new (2018) identifier for a person (or sometimes a thing) in the collections and archives of the Royal Academy, London",
        "data_type": "ExternalId"
    },
    "P1202": {
        "label": "carries scientific instrument",
        "description": "scientific instruments carried by a vessel, satellite, or device that are not required for propelling or navigating",
        "data_type": "WikibaseItem"
    },
    "P2244": {
        "label": "periapsis",
        "description": "distance, at which a celestial body is the closest to the object it orbits",
        "data_type": "Quantity"
    },
    "P4702": {
        "label": "Google Arts & Culture partner ID",
        "description": "identifier for a museum or other partner on the Google Arts & Culture website",
        "data_type": "ExternalId"
    },
    "P6500": {
        "label": "non-free artwork image URL",
        "description": "URL for image of copyrighted artwork on official collection or artist website",
        "data_type": "Url"
    },
    "P8406": {
        "label": "Grove Art Online ID",
        "description": "identifier of a topic in Grove Art Online",
        "data_type": "ExternalId"
    },
    "P7786": {
        "label": "Joconde epoch ID",
        "description": "identifier for an epoch or a style in the Joconde database of the French Museum Service",
        "data_type": "ExternalId"
    },
    "P8430": {
        "label": "Wien Kulturgut: Kunstwerke im öffentlichen Raum ID",
        "description": "identifier of public art in the \"public art register\" of the municipality of Vienna, Austria",
        "data_type": "ExternalId"
    },
    "P1194": {
        "label": "received signal type",
        "description": "type of signal received by a device",
        "data_type": "WikibaseItem"
    },
    "P6635": {
        "label": "ACA author ID",
        "description": "identifier for an author on the website of the Archives de la critique d'art website",
        "data_type": "ExternalId"
    },
    "P707": {
        "label": "satellite bus",
        "description": "general model on which multiple-production satellite spacecraft is based",
        "data_type": "WikibaseItem"
    },
    "P6243": {
        "label": "digital representation of",
        "description": "faithful digitized representation of the indicated object or work. For use on Commons MediaInfo entities.",
        "data_type": "WikibaseItem"
    },
    "P5070": {
        "label": "French standard size for oil paintings",
        "description": "standard for sizes of canvases for use by artists",
        "data_type": "WikibaseItem"
    },
    "P2243": {
        "label": "apoapsis",
        "description": "distance, at which a celestial body is the farthest to the object it orbits",
        "data_type": "Quantity"
    },
    "P2045": {
        "label": "orbital inclination",
        "description": "orbital inclination of a stable orbit",
        "data_type": "Quantity"
    },
    "P180": {
        "label": "depicts",
        "description": "depicted entity (see also P921: main subject)",
        "data_type": "WikibaseItem"
    },
    "P1170": {
        "label": "transmitted signal type",
        "description": "type of signal transmitted by a device",
        "data_type": "WikibaseItem"
    },
    "P1201": {
        "label": "space tug",
        "description": "spacecraft vehicle designed to move the payload from a reference orbit to the target orbit, or direct it to an interplanetary trajectory",
        "data_type": "WikibaseItem"
    },
    "P461": {
        "label": "opposite of",
        "description": "item that is the opposite of this item",
        "data_type": "WikibaseItem"
    },
    "P5524": {
        "label": "horizontal depth",
        "description": "spatial extent for 3D object along the third axis, what is most commonly referred to as its depth. Compare with \"vertical depth\" (P4511)",
        "data_type": "Quantity"
    },
    "P6294": {
        "label": "ARTIC exhibition ID",
        "description": "identifier for an exhibition on the Art Institute of Chicago website",
        "data_type": "ExternalId"
    },
    "P522": {
        "label": "type of orbit",
        "description": "orbit a satellite has around its central body",
        "data_type": "WikibaseItem"
    },
    "P9092": {
        "label": "Art in the Christian Tradition ID",
        "description": "Identifier for works in the Art in the Christian Tradition database managed by the Vanderbilt Divinity Library",
        "data_type": "ExternalId"
    },
    "P2146": {
        "label": "orbital period",
        "description": "the time taken for a given astronomic object to make one complete orbit about another object",
        "data_type": "Quantity"
    },
    "P1259": {
        "label": "coordinates of the point of view",
        "description": "point from which the scene depicted by the element is seen (element can be a photo, a painting, etc.)",
        "data_type": "GlobeCoordinate"
    },
    "P1418": {
        "label": "orbits completed",
        "description": "number of orbits a spacecraft has done around a body",
        "data_type": "Quantity"
    },
    "P3179": {
        "label": "territory overlaps",
        "description": "part or all of the area associated with (this) entity overlaps part or all of the area associated with that entity",
        "data_type": "WikibaseItem"
    },
    "P8192": {
        "label": "Joconde technique ID",
        "description": "identifier for a creative technique from the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P6606": {
        "label": "study or design for",
        "description": "finished work of art for which the subject is a preliminary study, design, or conceptual drawing",
        "data_type": "WikibaseItem"
    },
    "P4765": {
        "label": "Commons compatible image available at URL",
        "description": "image with Commons compatible copyright status is available at the following URL. It could be uploaded to Commons to illustrate the item",
        "data_type": "Url"
    },
    "P5435": {
        "label": "pixiv user ID",
        "description": "the artist id on pixiv",
        "data_type": "ExternalId"
    },
    "P6379": {
        "label": "has works in the collection",
        "description": "collection that has works of this person or organisation (use archive location P485 for the archives)",
        "data_type": "WikibaseItem"
    },
    "P2888": {
        "label": "exact match",
        "description": "(URLs only) used to link two items, indicating a high degree of confidence that the concepts can be used interchangeably",
        "data_type": "Url"
    },
    "P5403": {
        "label": "CNAP artist ID",
        "description": "identifier for an artist on the Centre national des arts plastiques website",
        "data_type": "ExternalId"
    },
    "P1079": {
        "label": "launch contractor",
        "description": "organization contracted to launch the rocket",
        "data_type": "WikibaseItem"
    },
    "P2534": {
        "label": "defining formula",
        "description": "mathematical formula representing a theorem or law. Maximum length: 400 characters",
        "data_type": "Math"
    },
    "P1312": {
        "label": "has facet polytope",
        "description": "facet of a polytope, in the next-lower dimension",
        "data_type": "WikibaseItem"
    },
    "P1851": {
        "label": "input set",
        "description": "a superset of the domain of a function or relation that may include some inputs for which the function is not defined; to specify the set of only those inputs for which the function is defined use domain (P1568)",
        "data_type": "WikibaseItem"
    },
    "P1571": {
        "label": "codomain",
        "description": "codomain of a function",
        "data_type": "WikibaseItem"
    },
    "P1181": {
        "label": "numeric value",
        "description": "numerical value of a number, a mathematical constant, or a physical constant",
        "data_type": "Quantity"
    },
    "P4873": {
        "label": "isoschizomer",
        "description": "isoschizomers of the restriction restriction enzyme, which have the same recognition sequence and the cutting site.",
        "data_type": "WikibaseItem"
    },
    "P4915": {
        "label": "isocaudomer",
        "description": "isocaudomer of the restriction restriction enzyme, which have the different recognition sequence but produces the same termini",
        "data_type": "WikibaseItem"
    },
    "P5572": {
        "label": "expressed in",
        "description": "gene or protein is expressed during a specific condition/cell cycle/process/form",
        "data_type": "WikibaseItem"
    },
    "P1164": {
        "label": "group cardinality",
        "description": "number of elements in a finite group",
        "data_type": "Quantity"
    },
    "P4864": {
        "label": "cutting site of restriction enzyme",
        "description": "DNA cutting site of restriction enzyme, written from 5' to 3'",
        "data_type": "String"
    },
    "P3741": {
        "label": "seed dispersal",
        "description": "mode used by the item to disperse its seeds",
        "data_type": "WikibaseItem"
    },
    "P3263": {
        "label": "base",
        "description": "configuration of a polytop vertices around the symmetry axis",
        "data_type": "WikibaseItem"
    },
    "P589": {
        "label": "point group",
        "description": "crystal subdivision",
        "data_type": "WikibaseItem"
    },
    "P4628": {
        "label": "ICTV virus genome composition",
        "description": "classification of viruses of the International Committee on Taxonomy of Viruses by the molecular composition of the virus genome (DNA, RNA, double or single stranded and translational polarity)",
        "data_type": "WikibaseItem"
    },
    "P2577": {
        "label": "admissible rule in",
        "description": "this logic inference rule is admissible in that logical system",
        "data_type": "WikibaseItem"
    },
    "P2820": {
        "label": "cardinality of this set",
        "description": "measure of number of elements of a set",
        "data_type": "WikibaseItem"
    },
    "P2384": {
        "label": "statement describes",
        "description": "formalization of the statement contains a bound variable in this class",
        "data_type": "WikibaseItem"
    },
    "P4875": {
        "label": "neoschizomer",
        "description": "neoschizomers of the restriction restriction enzyme, which have the same recognition sequence but a different cutting site.",
        "data_type": "WikibaseItem"
    },
    "P3331": {
        "label": "HGVS nomenclature",
        "description": "Sequence Variant Nomenclature from the Human Genome Variation Society (HGVS)",
        "data_type": "ExternalId"
    },
    "P4882": {
        "label": "segmental innervation",
        "description": "specific level(s) of spinal cord responsible for the function of this muscle or organ",
        "data_type": "WikibaseItem"
    },
    "P3488": {
        "label": "minimal incubation period in humans",
        "description": "minimal time between an infection and the onset of disease symptoms in infected humans",
        "data_type": "Quantity"
    },
    "P4914": {
        "label": "produces cohesive end",
        "description": "overhang DNA sequence generated by restriction enzyme, written from 5' to 3'",
        "data_type": "String"
    },
    "P1568": {
        "label": "definition domain",
        "description": "set of \"input\" or argument values for which a mathematical function is defined",
        "data_type": "WikibaseItem"
    },
    "P1102": {
        "label": "flattening",
        "description": "measure of the compression of an ellipsoid of revolution",
        "data_type": "Quantity"
    },
    "P3491": {
        "label": "muscle insertion",
        "description": "the anatomic entity to which the end of a muscle is anchored",
        "data_type": "WikibaseItem"
    },
    "P8194": {
        "label": "hardiness zone",
        "description": "geographic climate area relevant to plant growth and survival, on USDA scale or similar",
        "data_type": "WikibaseItem"
    },
    "P8193": {
        "label": "hardiness of plant",
        "description": "recommended hardiness zone for plant, ability of plant to survive adverse growing conditions",
        "data_type": "WikibaseItem"
    },
    "P1171": {
        "label": "approximation algorithm",
        "description": "method used to approximate a number",
        "data_type": "WikibaseItem"
    },
    "P8697": {
        "label": "RNACentral ID",
        "data_type": "ExternalId"
    },
    "P913": {
        "label": "notation",
        "description": "mathematical notation or another symbol",
        "data_type": "WikibaseItem"
    },
    "P6982": {
        "label": "Australian Faunal Directory publication ID",
        "description": "identifier of a publication in Australian Faunal Directory (Australian Biological Resources Study) of the Australian Government's Department of Environment and Energy",
        "data_type": "ExternalId"
    },
    "P1322": {
        "label": "dual to",
        "description": "dual of a polytope, graph or curve",
        "data_type": "WikibaseItem"
    },
    "P4774": {
        "label": "biological phase",
        "description": "biological period or stage when this biological process or phenomenon occurs",
        "data_type": "WikibaseItem"
    },
    "P5230": {
        "label": "chromosome count",
        "description": "karyological information on the number of chromosomes typical for a species",
        "data_type": "Quantity"
    },
    "P3063": {
        "label": "gestation period",
        "description": "amount of time needed for gestation for this animal taxon",
        "data_type": "Quantity"
    },
    "P7770": {
        "label": "egg incubation period",
        "description": "amount of time needed for incubation of eggs for this animal taxon (birds, fish, reptiles, monotremes)",
        "data_type": "Quantity"
    },
    "P3433": {
        "label": "biological variant of",
        "description": "a variant of a physical biological entity (e.g., gene sequence, protein sequence, epigenetic mark)",
        "data_type": "WikibaseItem"
    },
    "P4777": {
        "label": "has boundary",
        "description": "element that's on the two dimensional border that surrounds the subject; the limit of an entity",
        "data_type": "WikibaseItem"
    },
    "P1678": {
        "label": "has vertex figure",
        "description": "the figure exposed when a corner of a polytope is sliced off",
        "data_type": "WikibaseItem"
    },
    "P3094": {
        "label": "develops from",
        "description": "this class of items develops from another class of items",
        "data_type": "WikibaseItem"
    },
    "P4269": {
        "label": "half maximal effective concentration (EC50)",
        "description": "concentration of a drug, antibody or toxicant which induces a response halfway between the baseline and maximum after a specified exposure time",
        "data_type": "Quantity"
    },
    "P8789": {
        "label": "defining mutations",
        "description": "genetic mutations that define a haplogroup",
        "data_type": "WikibaseItem"
    },
    "P3490": {
        "label": "muscle origin",
        "description": "the anatomic entity to which the beginning of a muscle is anchored",
        "data_type": "WikibaseItem"
    },
    "P6143": {
        "label": "BioOne journal ID",
        "description": "identifier for an academic journal on the BioOne website",
        "data_type": "ExternalId"
    },
    "P3264": {
        "label": "radix",
        "description": "number of distinct digits in a positional numeral system",
        "data_type": "WikibaseItem"
    },
    "P2396": {
        "label": "image of function",
        "description": "set of values that a mathematical function actually takes",
        "data_type": "WikibaseItem"
    },
    "P4863": {
        "label": "recognition sequence",
        "description": "DNA sequence recognized by a restriction enzyme, DNA binding domain, etc, written from 5' to 3'",
        "data_type": "String"
    },
    "P4196": {
        "label": "cytogenetic location",
        "description": "cytogenetic location of the gene or region",
        "data_type": "String"
    },
    "P3487": {
        "label": "maximal incubation period in humans",
        "description": "maximal time between an infection and the onset of disease symptoms in infected humans",
        "data_type": "Quantity"
    },
    "P7862": {
        "label": "period of lactation",
        "description": "the period during which the animal is fed milk",
        "data_type": "Quantity"
    },
    "P4866": {
        "label": "REBASE Enzyme Number",
        "description": "ID in REBASE (Restriction Enzyme Database)",
        "data_type": "ExternalId"
    },
    "P4268": {
        "label": "half maximal inhibitory concentration (IC50)",
        "description": "measure of the effectiveness of a substance in inhibiting a specific biological or biochemical function",
        "data_type": "Quantity"
    },
    "P8872": {
        "label": "has marker",
        "description": "a gene or a protein published as a marker of a species-specific cell type",
        "data_type": "WikibaseItem"
    },
    "P3846": {
        "label": "DBC author ID",
        "description": "identifier for authors set by the Danish Bibliographic Centre",
        "data_type": "ExternalId"
    },
    "P3693": {
        "label": "short-track speed skater ISU ID",
        "description": "identifier for a short track speed skater on the ShorttrackOnLine.info website, based on ISU ID",
        "data_type": "ExternalId"
    },
    "P3694": {
        "label": "SpeedSkatingNews.info speed skater ID",
        "description": "identifier for a speed skater on the SpeedSkatingNews.info website",
        "data_type": "ExternalId"
    },
    "P3697": {
        "label": "Sports-Reference.com college football player ID",
        "description": "identifier for a college football player on the Sports Reference college football (SRCFB) website",
        "data_type": "ExternalId"
    },
    "P3619": {
        "label": "Ski-DB skier ID",
        "description": "ID for a ski racer at Ski-DB Alpine Ski Database",
        "data_type": "ExternalId"
    },
    "P3867": {
        "label": "Israel Chess Federation player ID",
        "description": "identifier for a chess player from the Israel Chess Federation",
        "data_type": "ExternalId"
    },
    "P3869": {
        "label": "IAFD female performer ID",
        "description": "identifier for a female-born performer in the Internet Adult Film Database",
        "data_type": "ExternalId"
    },
    "P3655": {
        "label": "BDFutbol player ID",
        "description": "ID of a football player at BDFutbol",
        "data_type": "ExternalId"
    },
    "P3661": {
        "label": "ForaDeJogo manager ID",
        "description": "ForaDeJogo.net identifier for an association football (soccer) manager",
        "data_type": "ExternalId"
    },
    "P3666": {
        "label": "EPCR player ID",
        "description": "identifier for a rugby player on the European Professional Club Rugby website",
        "data_type": "ExternalId"
    },
    "P3652": {
        "label": "Kontinental Hockey League player ID",
        "description": "ID of a player at KHL.ru",
        "data_type": "ExternalId"
    },
    "P3703": {
        "label": "JMDb person or company ID",
        "description": "ID of an actor or a company at Japanese Movie Database",
        "data_type": "ExternalId"
    },
    "P3706": {
        "label": "RITVA Person ID",
        "description": "Radio and Television Archive Person ID of The National Audiovisual Institute of Finland",
        "data_type": "ExternalId"
    },
    "P3698": {
        "label": "Tennis Australia player ID",
        "description": "identifier for a player on the Tennis Australia website",
        "data_type": "ExternalId"
    },
    "P3726": {
        "label": "EU-Football.info player ID",
        "description": "identifier for an association football player on the EU-Football.info website",
        "data_type": "ExternalId"
    },
    "P3897": {
        "label": "Ladies European Tour golf player ID",
        "description": "identifier for a golf player, in the Ladies European Tour database",
        "data_type": "ExternalId"
    },
    "P3882": {
        "label": "Tilastopaja female athlete ID",
        "description": "identifier for a female athlete on the Tilastopaja website",
        "data_type": "ExternalId"
    },
    "P3605": {
        "label": "90minut player ID",
        "description": "ID in the 90minut website for a football/soccer player",
        "data_type": "ExternalId"
    },
    "P3887": {
        "label": "KVAB member ID",
        "description": "identifier of member of the Royal Flemish Academy of Belgium for Science and the Arts",
        "data_type": "ExternalId"
    },
    "P3884": {
        "label": "Tilastopaja male athlete ID",
        "description": "identifier for a male athlete on the Tilastopaja website",
        "data_type": "ExternalId"
    },
    "P3745": {
        "label": "World Rugby Sevens Series player ID",
        "description": "identifier for a male rugby sevens player on the World Rugby Sevens Series website",
        "data_type": "ExternalId"
    },
    "P3682": {
        "label": "Australian Olympic Committee athlete ID",
        "description": "identifier for a sportsperson on the Australian Olympic Committee (AOC) website",
        "data_type": "ExternalId"
    },
    "P3644": {
        "label": "FFR player ID",
        "description": "identifier for a rugby union player on the French Rugby Federation website",
        "data_type": "ExternalId"
    },
    "P3663": {
        "label": "Fotbal DNES player ID",
        "description": "ID for a football player at Fotbal DNES",
        "data_type": "ExternalId"
    },
    "P3647": {
        "label": "NBA.com player ID",
        "description": "identifier for a basketball player on the National Basketball Association's website",
        "data_type": "ExternalId"
    },
    "P3623": {
        "label": "BadmintonLink player ID",
        "description": "identifier of a badminton player at BadmintonLink.com",
        "data_type": "ExternalId"
    },
    "P3659": {
        "label": "Estonian Football Association player ID",
        "description": "ID of a player by the Estonian Football Association",
        "data_type": "ExternalId"
    },
    "P3664": {
        "label": "Futsal Planet player ID",
        "description": "ID for a football player at Futsal Planet",
        "data_type": "ExternalId"
    },
    "P3686": {
        "label": "ESPN.com NFL player ID",
        "description": "identifier for a National Football League player at ESPN.com",
        "data_type": "ExternalId"
    },
    "P3668": {
        "label": "Sambafoot player ID",
        "description": "ID of a football player at Sambafoot",
        "data_type": "ExternalId"
    },
    "P3708": {
        "label": "PhDTree person ID",
        "description": "ID of a person at PhDTree",
        "data_type": "ExternalId"
    },
    "P3747": {
        "label": "SSRN author ID",
        "description": "identifier for an author at the Social Science Research Network",
        "data_type": "ExternalId"
    },
    "P3826": {
        "label": "Welsh Rugby Union men's player ID",
        "description": "identifier for a rugby union player selected with the Wales national team on the Welsh Rugby Union website",
        "data_type": "ExternalId"
    },
    "P3765": {
        "label": "All-Athletics.com ID",
        "description": "identifier for an athlete on the All-Athletics.com website",
        "data_type": "ExternalId"
    },
    "P3766": {
        "label": "European Athletics ID",
        "description": "identifier for an athlete on the European Athletic Association website",
        "data_type": "ExternalId"
    },
    "P3800": {
        "label": "Safsal coach ID",
        "description": "basketball coach Id in Safsal website",
        "data_type": "ExternalId"
    },
    "P3620": {
        "label": "BWFbadminton.com player ID",
        "description": "identifier of a player by Badminton World Federation, at bwfbadminton.com",
        "data_type": "ExternalId"
    },
    "P3653": {
        "label": "ARRS runner ID",
        "description": "ID of player by ARRS (Association of Road Racing Statisticians)",
        "data_type": "ExternalId"
    },
    "P3667": {
        "label": "IWF athlete ID",
        "description": "ID of an athlete by International Weightlifting Federation",
        "data_type": "ExternalId"
    },
    "P3691": {
        "label": "International Swimming Hall of Fame ID",
        "description": "identifier for an inductee on the International Swimming Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P3672": {
        "label": "IOF athlete ID",
        "description": "identifier for an athlete by the International Orienteering Federation",
        "data_type": "ExternalId"
    },
    "P3665": {
        "label": "L'Équipe football player ID",
        "description": "ID of a football player by L'Équipe",
        "data_type": "ExternalId"
    },
    "P3622": {
        "label": "Russian Bandy Federation player ID",
        "description": "identifier of a bandy player at rusbandy.ru, the official website of the Russian Bandy Federation",
        "data_type": "ExternalId"
    },
    "P3683": {
        "label": "LFP player ID",
        "description": "identifier for a football player on the Ligue de Football Professionnel website",
        "data_type": "ExternalId"
    },
    "P3812": {
        "label": "Elle.fr person ID",
        "description": "identifier for a person on the Elle.fr website",
        "data_type": "ExternalId"
    },
    "P3814": {
        "label": "BoF person ID",
        "description": "identifier for a person on the Business of Fashion website",
        "data_type": "ExternalId"
    },
    "P3718": {
        "label": "NCMEC person ID",
        "description": "identifier of a missing or unidentified person at the United States' National Center for Missing & Exploited Children",
        "data_type": "ExternalId"
    },
    "P3862": {
        "label": "MyDramaList name ID",
        "description": "identifier for a person in the MyDramaList database",
        "data_type": "ExternalId"
    },
    "P3748": {
        "label": "Israel Football Association player ID",
        "description": "identifier for a player on the Israel Football Association website",
        "data_type": "ExternalId"
    },
    "P3848": {
        "label": "Irish Rugby Football Union men's player ID",
        "description": "identifier for a rugby union player selected with Ireland national team on the Irish Rugby Football Union website",
        "data_type": "ExternalId"
    },
    "P3760": {
        "label": "Iditarod musher ID",
        "description": "identifier for a musher on the Iditarod Trail Sled Dog Race website",
        "data_type": "ExternalId"
    },
    "P3881": {
        "label": "National Track & Field Hall of Fame athlete ID",
        "description": "identifier for an athlete on the National Track & Field Hall of Fame webpages",
        "data_type": "ExternalId"
    },
    "P3892": {
        "label": "PictoRight ID-Droit de suite",
        "description": "identifier for a visual artist for their droit de suite (money collection with artwork transfers) of the Dutch collective rights management organisation PictoRight and sister organisations worldwide",
        "data_type": "ExternalId"
    },
    "P3830": {
        "label": "CueTracker player ID",
        "description": "ID about a specific snooker player at the CueTracker.net database",
        "data_type": "ExternalId"
    },
    "P3817": {
        "label": "FI WarSampo person ID",
        "description": "іdentifier in the WarSampo Finnish WW2 portal",
        "data_type": "ExternalId"
    },
    "P3874": {
        "label": "Justia Patents inventor ID",
        "description": "ID of an inventor in Justia Patents",
        "data_type": "ExternalId"
    },
    "P3857": {
        "label": "Cinenacional.com person ID",
        "description": "identifier for a person at cinenacional.com database (from Argentina)",
        "data_type": "ExternalId"
    },
    "P3900": {
        "label": "CONICET person ID",
        "description": "identifier for a researcher, scholar or support personnel at National Scientific and Technical Research Council (Argentina)",
        "data_type": "ExternalId"
    },
    "P3906": {
        "label": "Ishim ID",
        "description": "identifier in the Israeli cinema, television and radio database",
        "data_type": "ExternalId"
    },
    "P3657": {
        "label": "Croatian Football Statistics player ID",
        "description": "ID of an association football (soccer) player at hrnogomet.com, the Croatian Football Statistics (Statistike hrvatskog nogometa) website",
        "data_type": "ExternalId"
    },
    "P3604": {
        "label": "ITU triathlete ID",
        "description": "identifier for a triathlete on the International Triathlon Union's website",
        "data_type": "ExternalId"
    },
    "P3656": {
        "label": "Cross-tables.com Scrabble player ID",
        "description": "ID of a Scrabble player at cross-tables.com",
        "data_type": "ExternalId"
    },
    "P3658": {
        "label": "DZFoot.com player ID",
        "description": "ID of a football player at DZFoot.com",
        "data_type": "ExternalId"
    },
    "P3646": {
        "label": "Naismith Memorial Basketball Hall of Fame ID",
        "description": "identifier on the Naismith Memorial Basketball Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P3671": {
        "label": "USA Gymnastics athlete ID",
        "description": "ID for an athlete at USA Gymnastics",
        "data_type": "ExternalId"
    },
    "P3677": {
        "label": "Wereld van Oranje player ID",
        "description": "ID of a football player at Wereld van Oranje",
        "data_type": "ExternalId"
    },
    "P3681": {
        "label": "ESPNFC.com player ID",
        "description": "identifier for an association football (soccer) player at ESPNFC.com",
        "data_type": "ExternalId"
    },
    "P3685": {
        "label": "ESPN.com NBA player ID",
        "description": "identifier for a National Basketball Association player at ESPN.com",
        "data_type": "ExternalId"
    },
    "P3687": {
        "label": "ESPN.com NHL player ID",
        "description": "identifier for a National Hockey League player at ESPN.com",
        "data_type": "ExternalId"
    },
    "P3689": {
        "label": "ICF canoer ID",
        "description": "identifier for a canoer on the International Canoe Federation website",
        "data_type": "ExternalId"
    },
    "P3700": {
        "label": "NHF player ID",
        "description": "identifier for a player on the Norwegian Handball Federation (NHF) website",
        "data_type": "ExternalId"
    },
    "P3669": {
        "label": "Swimming Australia swimmer ID",
        "description": "ID of a swimmer at Swimming Australia",
        "data_type": "ExternalId"
    },
    "P3670": {
        "label": "Tennis Archives player ID",
        "description": "ID of a tennis player at the Tennis Archives",
        "data_type": "ExternalId"
    },
    "P3725": {
        "label": "CEV player ID",
        "description": "identifier for a player on the European Volleyball Confederation's website",
        "data_type": "ExternalId"
    },
    "P3695": {
        "label": "SpeedSkatingStats speed skater ID",
        "description": "identifier for a speed skater on the SpeedSkatingStats.com website",
        "data_type": "ExternalId"
    },
    "P3630": {
        "label": "Babelio author ID",
        "description": "identifier for an author on the literature website Babelio",
        "data_type": "ExternalId"
    },
    "P3645": {
        "label": "All Blacks player ID",
        "description": "identifier for a men's rugby union player on the New Zealand Rugby Union website",
        "data_type": "ExternalId"
    },
    "P3621": {
        "label": "Darts Database player ID",
        "description": "identifier of a player at Darts Database",
        "data_type": "ExternalId"
    },
    "P3654": {
        "label": "Chess.com player ID",
        "description": "ID of a player at Chess.com",
        "data_type": "ExternalId"
    },
    "P3662": {
        "label": "Football Federation of Ukraine player ID",
        "description": "ID of a player by Football Federation of Ukraine",
        "data_type": "ExternalId"
    },
    "P3807": {
        "label": "S2A3 Biographical Database ID",
        "description": "identifier of a person in the S2A3 Biographical Database of Southern African Science",
        "data_type": "ExternalId"
    },
    "P3767": {
        "label": "French Athletics Federation athlete ID",
        "description": "identifier for an athlete on the French Athletics Federation website",
        "data_type": "ExternalId"
    },
    "P3769": {
        "label": "It's Rugby player ID",
        "description": "identifier for a rugby player on the It's Rugby website",
        "data_type": "ExternalId"
    },
    "P3762": {
        "label": "openMLOL author ID",
        "description": "identifier of an author in the openMLOL digital library of cultural resources",
        "data_type": "ExternalId"
    },
    "P3799": {
        "label": "Safsal player ID",
        "description": "basketball player Id in Safsal website",
        "data_type": "ExternalId"
    },
    "P3845": {
        "label": "TV Guide person ID (former scheme)",
        "description": "ID of a person at TVGuide.com",
        "data_type": "ExternalId"
    },
    "P3843": {
        "label": "German Athletics Association athlete ID",
        "description": "identifier for an athlete on the German Athletics Association website",
        "data_type": "ExternalId"
    },
    "P3786": {
        "label": "Danskefilm person ID",
        "description": "identifier for a person in danskefilm.dk",
        "data_type": "ExternalId"
    },
    "P3821": {
        "label": "Bangla Movie Database ID",
        "description": "ID of actors or movies at Bangla Movie Database",
        "data_type": "ExternalId"
    },
    "P3829": {
        "label": "Publons author ID",
        "description": "identifier of an author or reviewer, in Publons",
        "data_type": "ExternalId"
    },
    "P3835": {
        "label": "Mendeley person ID",
        "description": "deprecated identifier for an author of scholarly works, at mendeley.com (for numeric ID, see P1153)",
        "data_type": "ExternalId"
    },
    "P3696": {
        "label": "Sports-Reference.com college basketball player ID",
        "description": "identifier for a NCAA Division I college men's basketball player on the Sports-Reference.com college basketball website",
        "data_type": "ExternalId"
    },
    "P3651": {
        "label": "American Hockey League player ID",
        "description": "ID of a player at TheAHL.com",
        "data_type": "ExternalId"
    },
    "P3660": {
        "label": "FootballFacts.ru player ID",
        "description": "ID for a football player at FootballFacts.ru",
        "data_type": "ExternalId"
    },
    "P3678": {
        "label": "SA Rugby player ID",
        "description": "identifier for a rugby player on the South African Rugby Union website",
        "data_type": "ExternalId"
    },
    "P297": {
        "label": "ISO 3166-1 alpha-2 code",
        "description": "identifier for a country in two-letter format per ISO 3166-1",
        "data_type": "ExternalId"
    },
    "P1160": {
        "label": "ISO 4 abbreviation",
        "description": "standard abbreviation for periodicals in the International Organization for Standardization system",
        "data_type": "ExternalId"
    },
    "P6519": {
        "label": "ISO 639-3 Change Request ID",
        "description": "Identifier for requested changes to ISO 639-3 language codes. Identifier is issued by the ISO 639-3 registration authority.",
        "data_type": "ExternalId"
    },
    "P3156": {
        "label": "Australian Classification",
        "description": "content rating of an audiovisual work or video game in the Australian Classification system",
        "data_type": "WikibaseItem"
    },
    "P1602": {
        "label": "Art UK venue ID",
        "description": "authority control identifier for venues housing art, in the UK",
        "data_type": "ExternalId"
    },
    "P957": {
        "label": "ISBN-10",
        "description": "former identifier for a book (edition), ten digits. Used for all publications up to 2006 (convertible to ISBN-13 for some online catalogs; useful for old books or fac-similes not reedited since 2007)",
        "data_type": "ExternalId"
    },
    "P2345": {
        "label": "AGORHA event identifier",
        "data_type": "ExternalId"
    },
    "P298": {
        "label": "ISO 3166-1 alpha-3 code",
        "description": "identifier for a country in three-letter format per ISO 3166-1",
        "data_type": "ExternalId"
    },
    "P3194": {
        "label": "INA video ID",
        "description": "identifier of a video on the INA website",
        "data_type": "ExternalId"
    },
    "P219": {
        "label": "ISO 639-2 code",
        "description": "3-letter identifier for language,  macro-language or language family, defined in ISO 639-2 standard",
        "data_type": "ExternalId"
    },
    "P220": {
        "label": "ISO 639-3 code",
        "description": "3-letter identifier for language defined in ISO 639-3, extension of ISO 639-2",
        "data_type": "ExternalId"
    },
    "P8188": {
        "label": "Lexikon der Filmbegriffe ID",
        "description": "ID for the German encyclopedia of film terms (\"Lexikon der Filmbegriffe\")",
        "data_type": "ExternalId"
    },
    "P8505": {
        "label": "traffic sign template image",
        "description": "image used as a template for creating traffic sign images (P14) for specific instances of this sign or members of this highway system",
        "data_type": "CommonsMedia"
    },
    "P7534": {
        "label": "MIC market code",
        "description": "ISO 10383 market identifier code",
        "data_type": "ExternalId"
    },
    "P8244": {
        "label": "Mediaqueer.ca artist ID",
        "data_type": "ExternalId"
    },
    "P299": {
        "label": "ISO 3166-1 numeric code",
        "description": "identifier for a country in numeric format per ISO 3166-1",
        "data_type": "ExternalId"
    },
    "P5209": {
        "label": "ISO 3950 code",
        "description": "identifier for a tooth per ISO 3950",
        "data_type": "ExternalId"
    },
    "P4312": {
        "label": "camera setup",
        "description": "filmmaking method that the cameras were placed by. Use single-camera (Q2918907) or multiple-camera (Q738160)",
        "data_type": "WikibaseItem"
    },
    "P2538": {
        "label": "Nationalmuseum Sweden artist ID",
        "description": "artist identifier for Nationalmuseum in Sweden",
        "data_type": "ExternalId"
    },
    "P8972": {
        "label": "small logo or icon",
        "description": "logo or icon that represents the item and can be rendered at sizes as small as 16×16 pixels",
        "data_type": "CommonsMedia"
    },
    "P3097": {
        "label": "ISBN identifier group",
        "description": "ISBN prefix for countries or languages",
        "data_type": "ExternalId"
    },
    "P1208": {
        "label": "ISMN",
        "description": "International Standard Music Number",
        "data_type": "ExternalId"
    },
    "P2766": {
        "label": "ISO 4063 process number",
        "description": "identifier for a process, in the ISO-approved classification scheme for welding & soldering processes",
        "data_type": "ExternalId"
    },
    "P8512": {
        "label": "size comparison diagram",
        "description": "property for the diagram file that compares the item size with the size of another known object",
        "data_type": "CommonsMedia"
    },
    "P218": {
        "label": "ISO 639-1 code",
        "description": "2-letter identifier for language or family of languages defined in ISO 639-1 standard",
        "data_type": "ExternalId"
    },
    "P1243": {
        "label": "ISRC",
        "description": "international standard code for uniquely identifying sound recordings and music video recordings",
        "data_type": "ExternalId"
    },
    "P3701": {
        "label": "incarnation of",
        "description": "incarnation of another religious or supernatural being",
        "data_type": "WikibaseItem"
    },
    "P773": {
        "label": "ISO 3166-3",
        "description": "identifier for a country name that has been deleted from ISO 3166-1 since its first publication in 1974",
        "data_type": "ExternalId"
    },
    "P8794": {
        "label": "BBFC reference",
        "description": "reference number for a film or video game (-2012) media content rating decision on BBFC's official website",
        "data_type": "ExternalId"
    },
    "P221": {
        "label": "ISO 639-6 code",
        "description": "4-letter identifier for language per ISO 639-6, standard between 2009-2014",
        "data_type": "ExternalId"
    },
    "P3828": {
        "label": "wears",
        "description": "clothing or accessory worn on subject's body",
        "data_type": "WikibaseItem"
    },
    "P300": {
        "label": "ISO 3166-2 code",
        "description": "identifier for a country subdivision per ISO 3166-2 (include country code)",
        "data_type": "ExternalId"
    },
    "P1798": {
        "label": "ISO 639-5 code",
        "description": "3-letter identifier for language family or collective code per ISO 639-5",
        "data_type": "ExternalId"
    },
    "P1827": {
        "label": "ISWC",
        "description": "identifier for musical works, adopted as international standard ISO 15707",
        "data_type": "ExternalId"
    },
    "P1014": {
        "label": "Art & Architecture Thesaurus ID",
        "description": "identifier in the Art & Architecture Thesaurus by the Getty Research Institute",
        "data_type": "ExternalId"
    },
    "P3507": {
        "label": "Bivouac.com mountain ID",
        "description": "identifier of a North American mountain, at Bivouac.com",
        "data_type": "ExternalId"
    },
    "P981": {
        "label": "BAG residence ID",
        "description": "BAG code for Dutch residencies (\"woonplaats\", mostly municipalities)",
        "data_type": "ExternalId"
    },
    "P1481": {
        "label": "vici.org ID",
        "description": "identifier on vici.org - archaeological atlas of antiquity",
        "data_type": "ExternalId"
    },
    "P2496": {
        "label": "Latvian toponymic names database ID",
        "description": "Latvian Geospatial Information Agency geographical names database identifier",
        "data_type": "ExternalId"
    },
    "P2487": {
        "label": "page at website of Belarus Geocenter",
        "description": "state centre of cartographic and geodetic materials and data of the Republic of Belarus",
        "data_type": "ExternalId"
    },
    "P1886": {
        "label": "Smithsonian volcano ID",
        "description": "identifier of volcano in the Smithsonian Global Volcanism Program. Format: 6 digits.",
        "data_type": "ExternalId"
    },
    "P3120": {
        "label": "TOID",
        "description": "TOpographic IDentifier assigned by the Ordnance Survey to identify a feature in Great Britain",
        "data_type": "ExternalId"
    },
    "P982": {
        "label": "MusicBrainz area ID",
        "description": "identifier for an area in the MusicBrainz open music database",
        "data_type": "ExternalId"
    },
    "P3735": {
        "label": "Yad Vashem Encyclopedia of the Ghettos ID",
        "description": "identifier of a ghetto, or a place containing a ghetto, in the Yad Vashem Encyclopedia of the Ghettos",
        "data_type": "ExternalId"
    },
    "P3616": {
        "label": "Vision of Britain place ID",
        "description": "identifier of a place in the University of Portsmouth's Vision of Britain database",
        "data_type": "ExternalId"
    },
    "P2490": {
        "label": "page at OSTIS Belarus Wiki",
        "data_type": "String"
    },
    "P3635": {
        "label": "Peakbagger area ID",
        "description": "identifier for a mountain range or an area, on the peakbagger.com website",
        "data_type": "ExternalId"
    },
    "P1584": {
        "label": "Pleiades ID",
        "description": "identifier for a place in Pleiades hosted at pleiades.stoa.org",
        "data_type": "ExternalId"
    },
    "P1281": {
        "label": "WOEID",
        "description": "identifier for a location on Yahoo! GeoPlanet and Flickr",
        "data_type": "ExternalId"
    },
    "P1397": {
        "label": "State Catalogue of Geographical Names (Russia) ID",
        "description": "identifier in State Catalogue of Geographical Names (Russia)",
        "data_type": "ExternalId"
    },
    "P3134": {
        "label": "TripAdvisor ID",
        "description": "identifier of a place (region, hotel, restaurant, attraction), in TripAdvisor",
        "data_type": "ExternalId"
    },
    "P1958": {
        "label": "Trismegistos Geo ID",
        "description": "identifier of an ancient place on trismegistos.org",
        "data_type": "ExternalId"
    },
    "P590": {
        "label": "GNIS ID",
        "description": "identifier for geographic objects in the US issued by the USGS. For Antarctica, use Property:P804",
        "data_type": "ExternalId"
    },
    "P8882": {
        "label": "complementary property",
        "description": "property expressing an aspect of the same relation that completes this property (e.g. start date and end date). The two can be seen as opposites. For inverses, use \"inverse property\" (P1696). For relations among items, see \"opposite of\" (P461).",
        "data_type": "WikibaseProperty"
    },
    "P3394": {
        "label": "Finnish Lake ID",
        "description": "ID for a lake in the Järviwiki database of Finnish lakes over 1 ha in extent.",
        "data_type": "ExternalId"
    },
    "P1997": {
        "label": "Facebook Places ID",
        "description": "identifier for a place in Facebook",
        "data_type": "ExternalId"
    },
    "P3118": {
        "label": "OpenDomesday settlement ID",
        "description": "Identifier for a British settlement, in the OpenDomesday digitisation of the Domesday Book",
        "data_type": "ExternalId"
    },
    "P3198": {
        "label": "JewishGen Locality ID",
        "description": "identifier of a town in The JewishGen Communities Database",
        "data_type": "ExternalId"
    },
    "P1937": {
        "label": "UN/LOCODE",
        "description": "geographic location code mantained by UNECE",
        "data_type": "ExternalId"
    },
    "P3813": {
        "label": "Bivouac.com pass ID",
        "description": "identifier of a North American mountain pass, at Bivouac.com",
        "data_type": "ExternalId"
    },
    "P2856": {
        "label": "EU Surface Water Body Code",
        "description": "unique EU identifier for a surface water body",
        "data_type": "ExternalId"
    },
    "P2099": {
        "label": "BC Geographical Names ID",
        "description": "Unique ID of the BC Geographical Names",
        "data_type": "ExternalId"
    },
    "P3257": {
        "label": "Queensland place ID",
        "description": "identifier of place, in the official Queensland Government gazetteer",
        "data_type": "ExternalId"
    },
    "P9073": {
        "label": "applicable 'stated in' value",
        "description": "value suitable for \"stated in\" (P248) when property is used in references to refer to the primary publication/website/database associated with the property",
        "data_type": "WikibaseItem"
    },
    "P1968": {
        "label": "Foursquare venue ID",
        "description": "ID of a place in Foursquare",
        "data_type": "ExternalId"
    },
    "P1010": {
        "label": "Iran statistics ID",
        "description": "identifier for places in Iran",
        "data_type": "ExternalId"
    },
    "P2100": {
        "label": "Banque de noms de lieux du Québec ID",
        "description": "unique identifier for geographical names in Quebec",
        "data_type": "ExternalId"
    },
    "P1282": {
        "label": "OpenStreetMap tag or key",
        "description": "OpenStreetMap tagging schema (a Key:key, Tag:key=value, Relation:type, or Role:role) for classes, properties, and values",
        "data_type": "String"
    },
    "P2497": {
        "label": "Latvian National Address Register ID",
        "description": "National Address Register classifier code (State Land Service of the Republic of Latvia)",
        "data_type": "ExternalId"
    },
    "P3119": {
        "label": "Code for China Reservoir Name",
        "description": "an identifer for Chinese reservoirs",
        "data_type": "ExternalId"
    },
    "P2186": {
        "label": "Wiki Loves Monuments ID",
        "description": "unique ID for the monuments included in Wiki Loves Monument's national lists",
        "data_type": "ExternalId"
    },
    "P3850": {
        "label": "LombardiaBeniCulturali toponym ID",
        "description": "identifier of a historical toponym in the cultural heritage database of the Lombardy Region of Italy",
        "data_type": "ExternalId"
    },
    "P3256": {
        "label": "Cave E-Cadastre ID",
        "description": "ID in the Cave E-Cadastre of Slovenia",
        "data_type": "ExternalId"
    },
    "P3633": {
        "label": "British Museum place ID",
        "description": "identifier for a place in the British Museum thesaurus",
        "data_type": "ExternalId"
    },
    "P2488": {
        "label": "page at Belarus Globe website",
        "data_type": "Url"
    },
    "P778": {
        "label": "Church of Sweden parish code",
        "description": "identifier for a parish of the Church of Sweden",
        "data_type": "ExternalId"
    },
    "P3514": {
        "label": "U.S. National Geodetic Survey ID",
        "description": "Identifier for a place by the U.S. National Geodetic Survey",
        "data_type": "ExternalId"
    },
    "P3396": {
        "label": "French Catholic Church structure ID",
        "description": "identifier of a structure in the guide of the French Catholic Church, edited by the Bishops' Conference of France",
        "data_type": "ExternalId"
    },
    "P3175": {
        "label": "Statoids ID",
        "description": "identifier for a place in the Statoids wiki",
        "data_type": "ExternalId"
    },
    "P761": {
        "label": "Lake ID (Sweden)",
        "description": "Identification code for lakes in Sweden, for lakes important enough also used, with SE- prefix, as EU_CD",
        "data_type": "ExternalId"
    },
    "P3109": {
        "label": "Peakbagger mountain ID",
        "description": "identifier for a mountain, on the www.peakbagger.com website",
        "data_type": "ExternalId"
    },
    "P3108": {
        "label": "Yelp ID",
        "description": "identifier of a place, in Yelp.com",
        "data_type": "ExternalId"
    },
    "P3202": {
        "label": "UAI code",
        "description": "official identifier of a French educational establishment",
        "data_type": "ExternalId"
    },
    "P779": {
        "label": "Church of Sweden Pastoratskod",
        "description": "identifier for a pastoral district of the Church of Sweden",
        "data_type": "ExternalId"
    },
    "P1866": {
        "label": "Catholic Hierarchy diocese ID",
        "description": "identifer for a diocese on catholic-hierarchy.org",
        "data_type": "ExternalId"
    },
    "P2382": {
        "label": "Chemins de mémoire ID",
        "data_type": "ExternalId"
    },
    "P1566": {
        "label": "GeoNames ID",
        "description": "identifier in the GeoNames geographical database",
        "data_type": "ExternalId"
    },
    "P2326": {
        "label": "GNS Unique Feature ID",
        "description": "identifier for geographic entities according to the National Geospatial-Intelligence Agency's GEOnet Names Server",
        "data_type": "ExternalId"
    },
    "P3626": {
        "label": "Australian Antarctic Gazetteer ID",
        "description": "identifier of a place, in the Australian Antarctic Gazetteer",
        "data_type": "ExternalId"
    },
    "P3628": {
        "label": "British History Online VCH ID",
        "description": "identifier of a place, in the British History Online digitisation of the Victoria County History",
        "data_type": "ExternalId"
    },
    "P3627": {
        "label": "Historical Gazetteer of England's Place Names ID",
        "description": "identifier for a place in the Historical Gazetteer of England's Place Names website",
        "data_type": "ExternalId"
    },
    "P2503": {
        "label": "Historical Gazetteer (GOV) ID",
        "description": "Historical Gazetteer",
        "data_type": "ExternalId"
    },
    "P3707": {
        "label": "Gridabase glacier ID",
        "description": "identifier for a glacier on the Glacier Risks Data Base website",
        "data_type": "ExternalId"
    },
    "P3230": {
        "label": "SCAR Composite Gazetteer place ID",
        "description": "identifier in the international composite gazetteer for Antarctica run by the Scientific Committee on Antarctic Research",
        "data_type": "ExternalId"
    },
    "P1936": {
        "label": "Digital Atlas of the Roman Empire ID",
        "description": "Identifier of a Roman site, in the Digital Atlas of the Roman Empire",
        "data_type": "ExternalId"
    },
    "P3517": {
        "label": "Geographical Names Board of NSW ID",
        "description": "historical identifier of a geographical feature on a retired Geographical Names Board of New South Wales website",
        "data_type": "ExternalId"
    },
    "P1404": {
        "label": "World Glacier Inventory ID",
        "description": "identifier in the World Glacier Inventory database",
        "data_type": "ExternalId"
    },
    "P3749": {
        "label": "Google Maps Customer ID",
        "description": "in Google Maps, Customer identifier for a place",
        "data_type": "ExternalId"
    },
    "P2618": {
        "label": "inHerit Place Number",
        "description": "identifier for a heritage place, issued by the State Heritage Office of Western Australia",
        "data_type": "ExternalId"
    },
    "P804": {
        "label": "GNIS Antarctica ID",
        "description": "identifier for geographic objects in Antarctica, used in the US Geological Survey's Geographic Names Information System. For U.S. IDs, use Property:P590",
        "data_type": "ExternalId"
    },
    "P1717": {
        "label": "Sandre river ID",
        "description": "identifer of the French national Water Information System",
        "data_type": "ExternalId"
    },
    "P8952": {
        "label": "inappropriate property for this type",
        "description": "property that should not be present for instances of that element",
        "data_type": "WikibaseProperty"
    },
    "P1667": {
        "label": "Getty Thesaurus of Geographic Names ID",
        "description": "identifier in the Getty Thesaurus of Geographic Names",
        "data_type": "ExternalId"
    },
    "P3770": {
        "label": "PeakFinder ID",
        "description": "identifier for a mountain peak on the PeakFinder website",
        "data_type": "ExternalId"
    },
    "P3806": {
        "label": "Mapa place ID",
        "description": "identifier for a place, in the Mapa database of places in Israel",
        "data_type": "ExternalId"
    },
    "P821": {
        "label": "CGNDB Unique ID",
        "description": "identifier(s) for a geographical feature contained in the Canadian Geographical Names Data Base (CGNDB)",
        "data_type": "ExternalId"
    },
    "P3309": {
        "label": "SummitPost mountain ID",
        "description": "identifer of a mountain on SummitPost",
        "data_type": "ExternalId"
    },
    "P3820": {
        "label": "Flanders Arts Institute venue ID",
        "description": "identifier of a venue in the Flanders Arts Institute database for performing arts",
        "data_type": "ExternalId"
    },
    "P1004": {
        "label": "MusicBrainz place ID",
        "description": "Identifier for a place in the MusicBrainz open music encyclopedia",
        "data_type": "ExternalId"
    },
    "P2887": {
        "label": "reserve number (Canada)",
        "description": "identifier of a Canadian Indian reserve",
        "data_type": "ExternalId"
    },
    "P1849": {
        "label": "SSR WrittenForm ID",
        "description": "identifier for the written form a place name, as used by Sentralt Stedsnavnregister in Norway",
        "data_type": "ExternalId"
    },
    "P3513": {
        "label": "peakware mountain ID",
        "description": "identifier for a mountain at Peakware.com",
        "data_type": "ExternalId"
    },
    "P3580": {
        "label": "SIPCA code",
        "description": "identifier of an item in Sistema de Información del Patrimonio Cultural Aragonés (SIPCA)",
        "data_type": "ExternalId"
    },
    "P402": {
        "label": "OpenStreetMap relation ID",
        "description": "identifier for a relation in OpenStreetMap",
        "data_type": "ExternalId"
    },
    "P1850": {
        "label": "SSR place name number",
        "description": "identifier for a place name in the Norwegian Mapping Authority's place name registry Sentralt stedsnavnregister (SSR)",
        "data_type": "ExternalId"
    },
    "P3326": {
        "label": "World Waterfall Database ID",
        "description": "Waterfall ID in the World Waterfall Database",
        "data_type": "ExternalId"
    },
    "P2548": {
        "label": "strand orientation",
        "description": "orientation of gene on double stranded DNA molecule",
        "data_type": "WikibaseItem"
    },
    "P1912": {
        "label": "deletion association with",
        "description": "This property should link a gene and a disease due to a deletion",
        "data_type": "WikibaseItem"
    },
    "P3737": {
        "label": "maximum wavelength of sensitivity",
        "description": "maximum wavelength that the item can detect",
        "data_type": "Quantity"
    },
    "P684": {
        "label": "ortholog",
        "description": "orthologous gene in another species (use with 'species' qualifier)",
        "data_type": "WikibaseItem"
    },
    "P1360": {
        "label": "Monte Carlo Particle Number",
        "description": "number given to each elementary particle",
        "data_type": "String"
    },
    "P517": {
        "label": "interaction",
        "description": "subset of the four fundamental forces (strong (Q11415), electromagnetic (Q849919), weak (Q11418), and gravitation (Q11412) with which a particle interacts",
        "data_type": "WikibaseItem"
    },
    "P1918": {
        "label": "altered regulation leads to",
        "description": "this property should link a gene and a disease due to altered regulation",
        "data_type": "WikibaseItem"
    },
    "P1531": {
        "label": "parent of this hybrid, breed, or cultivar",
        "data_type": "WikibaseItem"
    },
    "P1141": {
        "label": "number of processor cores",
        "description": "number of independent actual processing units (*not* threads)",
        "data_type": "Quantity"
    },
    "P171": {
        "label": "parent taxon",
        "description": "closest parent taxon of the taxon in question",
        "data_type": "WikibaseItem"
    },
    "P3738": {
        "label": "minimum wavelength of sensitivity",
        "description": "minimum wavelength that the item can detect",
        "data_type": "Quantity"
    },
    "P5480": {
        "label": "tensile modulus of elasticity",
        "description": "material proprety linking the elastic deformation to the yield strengh.",
        "data_type": "Quantity"
    },
    "P4788": {
        "label": "storage device",
        "description": "data storage device supported by a machine (e.g. camera or mobile phone)",
        "data_type": "WikibaseItem"
    },
    "P5520": {
        "label": "toughness",
        "description": "material property measuring the ability of a material to absorb energy and plastically deform without fracturing",
        "data_type": "Quantity"
    },
    "P2293": {
        "label": "genetic association",
        "description": "general link between a disease and the causal genetic entity, if the detailed mechanism is unknown/unavailable",
        "data_type": "WikibaseItem"
    },
    "P702": {
        "label": "encoded by",
        "description": "the gene that encodes some gene product",
        "data_type": "WikibaseItem"
    },
    "P5479": {
        "label": "ultimate tensile strength",
        "description": "material property measuring the tensil strengh at break",
        "data_type": "Quantity"
    },
    "P5529": {
        "label": "yield strength",
        "description": "material proprety measuring the stress above which materials begin to permanently deform",
        "data_type": "Quantity"
    },
    "P4183": {
        "label": "angle from vertical",
        "description": "angle the principal axis of an item makes with the vertical; use degree (Q28390) as unit",
        "data_type": "Quantity"
    },
    "P1097": {
        "label": "g-factor",
        "description": "characteristic property of any elementary particle (Landé factor)",
        "data_type": "Quantity"
    },
    "P5575": {
        "label": "coefficient of friction",
        "description": "dimensionless scalar value which describes the ratio of the force of friction between two bodies and the force pressing them together",
        "data_type": "Quantity"
    },
    "P2560": {
        "label": "GPU",
        "description": "graphics processing unit within a system",
        "data_type": "WikibaseItem"
    },
    "P5593": {
        "label": "Poisson's ratio",
        "description": "the negative of the ratio of (signed) transverse strain to (signed) axial strain",
        "data_type": "Quantity"
    },
    "P6073": {
        "label": "permeation",
        "description": "the penetration of a permeate (such as a liquid, gas, or vapor) through a solid",
        "data_type": "Quantity"
    },
    "P5483": {
        "label": "hardness",
        "description": "measure of how resistant solid matter is to various kinds of permanent shape change when a force is applied",
        "data_type": "Quantity"
    },
    "P1109": {
        "label": "refractive index",
        "description": "refractive index of a substance with wavelength and temperature as qualifiers",
        "data_type": "Quantity"
    },
    "P4020": {
        "label": "ISQ dimension",
        "description": "property of an observable physical quantity being a product of the SI base dimensions (length, mass, time, etc.)",
        "data_type": "Math"
    },
    "P5669": {
        "label": "compressive strength",
        "description": "capacity of a material or structure to withstand loads tending to reduce size",
        "data_type": "Quantity"
    },
    "P816": {
        "label": "decays to",
        "description": "what isotope does this radioactive isotope decay to",
        "data_type": "WikibaseItem"
    },
    "P6212": {
        "label": "heat treating",
        "description": "Industrial and metalworking processes used to alter the physical, and sometimes chemical, properties of a material.",
        "data_type": "WikibaseItem"
    },
    "P2222": {
        "label": "gyromagnetic ratio",
        "description": "characteristic property of any elementary particle",
        "data_type": "Quantity"
    },
    "P351": {
        "label": "Entrez Gene ID",
        "description": "identifier for a gene per the NCBI Entrez database",
        "data_type": "ExternalId"
    },
    "P2928": {
        "label": "memory capacity",
        "description": "memory storage capacity of some data storage device",
        "data_type": "Quantity"
    },
    "P1910": {
        "label": "decreased expression in",
        "description": "indicates that a decreased expression of the subject gene is found in the object disease",
        "data_type": "WikibaseItem"
    },
    "P1914": {
        "label": "gene insertion association with",
        "description": "This property should link a gene and a disease due to an insertion",
        "data_type": "WikibaseItem"
    },
    "P2143": {
        "label": "genome size",
        "description": "size of the genome in base pairs",
        "data_type": "Quantity"
    },
    "P1582": {
        "label": "natural product of taxon",
        "description": "links a natural product with its source (animal, plant, fungal, algal, etc.)",
        "data_type": "WikibaseItem"
    },
    "P2201": {
        "label": "electric dipole moment",
        "data_type": "Quantity"
    },
    "P2935": {
        "label": "connector",
        "description": "connectors which the device has/supports",
        "data_type": "WikibaseItem"
    },
    "P2223": {
        "label": "decay width",
        "description": "characteristic property of any elementary particle",
        "data_type": "Quantity"
    },
    "P3891": {
        "label": "observing time available",
        "description": "amount of observing time available at a telescope (or other instrument)",
        "data_type": "Quantity"
    },
    "P2849": {
        "label": "produced by",
        "description": "links a class of biological features (such as a biochemical compound) with another class of biological entities known to produce it",
        "data_type": "WikibaseItem"
    },
    "P2610": {
        "label": "thickness",
        "description": "extent from one surface to the opposite",
        "data_type": "Quantity"
    },
    "P4184": {
        "label": "slope",
        "description": "angle the surface of an item makes with the horizontal",
        "data_type": "Quantity"
    },
    "P680": {
        "label": "molecular function",
        "description": "represents gene ontology function annotations",
        "data_type": "WikibaseItem"
    },
    "P2597": {
        "label": "Gram staining",
        "description": "Gram stain type of a bacterial strain",
        "data_type": "WikibaseItem"
    },
    "P2575": {
        "label": "measures",
        "description": "physical quantity that this device measures",
        "data_type": "WikibaseItem"
    },
    "P515": {
        "label": "phase of matter",
        "description": "state or phase of the matter at which the measure was made",
        "data_type": "WikibaseItem"
    },
    "P5594": {
        "label": "moisture absorption",
        "description": "quantity of moisture a material would absorb in given conditions",
        "data_type": "Quantity"
    },
    "P922": {
        "label": "magnetic ordering",
        "description": "magnetic ordering of a substance",
        "data_type": "WikibaseItem"
    },
    "P1913": {
        "label": "gene duplication association with",
        "description": "This property should link a gene and a disease due to a duplication",
        "data_type": "WikibaseItem"
    },
    "P1915": {
        "label": "gene inversion association with",
        "description": "This property should link a gene and a disease due to an inversion",
        "data_type": "WikibaseItem"
    },
    "P1911": {
        "label": "increased expression in",
        "description": "this property should link a gene and a disease and indicate that an increased expression of the gene is found in the disease",
        "data_type": "WikibaseItem"
    },
    "P1916": {
        "label": "gene substitution association with",
        "description": "This property should link a gene and a disease due to a substitution",
        "data_type": "WikibaseItem"
    },
    "P2152": {
        "label": "antiparticle",
        "description": "particle with the same rest mass and opposite charges",
        "data_type": "WikibaseItem"
    },
    "P2200": {
        "label": "electric charge",
        "description": "electric charge of a subatomic particle",
        "data_type": "Quantity"
    },
    "P1349": {
        "label": "ploidy",
        "description": "ploidy of a genome",
        "data_type": "WikibaseItem"
    },
    "P8956": {
        "label": "compatible with",
        "description": "this work, product, object or standard can interact with another work, product, object or standard",
        "data_type": "WikibaseItem"
    },
    "P4501": {
        "label": "albedo",
        "description": "ratio of reflected radiation to incident radiation",
        "data_type": "Quantity"
    },
    "P2376": {
        "label": "superpartner of",
        "description": "partner particle, in supersymmetry; inverse of \"has superpartner\"",
        "data_type": "WikibaseItem"
    },
    "P682": {
        "label": "biological process",
        "description": "is involved in the biological process",
        "data_type": "WikibaseItem"
    },
    "P7973": {
        "label": "quantity symbol (LaTeX)",
        "description": "symbol for a mathematical or physical quantity in LaTeX",
        "data_type": "Math"
    },
    "P2375": {
        "label": "has superpartner",
        "description": "partner particle, in supersymmetry; inverse of \"superpartner of\"",
        "data_type": "WikibaseItem"
    },
    "P128": {
        "label": "regulates (molecular biology)",
        "description": "process regulated by a protein or RNA in molecular biology",
        "data_type": "WikibaseItem"
    },
    "P2520": {
        "label": "UNESCO Biosphere Reserve URL",
        "description": "URL to the entry on the UNESCO biosphere reserve website",
        "data_type": "Url"
    },
    "P129": {
        "label": "physically interacts with",
        "description": "physical entity that the subject interacts with",
        "data_type": "WikibaseItem"
    },
    "P1990": {
        "label": "species kept",
        "description": "taxa, preferably species, present at a zoo, botanical garden,  collection, or other institution. NOT specific animals, not for any geographic location",
        "data_type": "WikibaseItem"
    },
    "P5668": {
        "label": "bulk modulus",
        "description": "measure of how resistant to compressibility that substance is",
        "data_type": "Quantity"
    },
    "P681": {
        "label": "cell component",
        "description": "component of the cell in which this item is present",
        "data_type": "WikibaseItem"
    },
    "P5589": {
        "label": "microstructure",
        "description": "very small scale structure of a material, defined as the structure of a prepared surface of material as revealed by a microscope above 25× magnification",
        "data_type": "WikibaseItem"
    },
    "P6753": {
        "label": "number of polling stations",
        "description": "number of polling stations, and correspondively of electoral precincts, in certain electoral district or during certain elections",
        "data_type": "Quantity"
    },
    "P3075": {
        "label": "official religion",
        "description": "official religion in this administrative entity",
        "data_type": "WikibaseItem"
    },
    "P9002": {
        "label": "liturgical rank",
        "description": "rank of a feast day in a religious calendar used for public worship",
        "data_type": "WikibaseItem"
    },
    "P2501": {
        "label": "results",
        "description": "results of a competition such as sports or elections",
        "data_type": "WikibaseItem"
    },
    "P7184": {
        "label": "Awards & Winners artist ID",
        "description": "identifier for an artist on Awards & Winners site",
        "data_type": "ExternalId"
    },
    "P5510": {
        "label": "DEFA film database ID",
        "description": "identifier in the film database of the DEFA foundation",
        "data_type": "ExternalId"
    },
    "P2354": {
        "label": "has list",
        "description": "Wikimedia list related to this subject",
        "data_type": "WikibaseItem"
    },
    "P7214": {
        "label": "Allcinema person ID",
        "description": "identifier of person on Allcinema database",
        "data_type": "ExternalId"
    },
    "P9146": {
        "label": "CITWF title ID",
        "description": "identifier for a film in the Complete Index to World Film (www.citwf.com)",
        "data_type": "ExternalId"
    },
    "P2925": {
        "label": "domain of saint or deity",
        "description": "domain(s) which this saint or deity controls or protects",
        "data_type": "WikibaseItem"
    },
    "P6780": {
        "label": "LGBTFansDB character ID",
        "description": "identifier for a character entry on LGBT fans deserve better database",
        "data_type": "ExternalId"
    },
    "P8879": {
        "label": "Kinorium person ID",
        "description": "identifier for a person on kinorium.com",
        "data_type": "ExternalId"
    },
    "P5045": {
        "label": "number of blank votes",
        "description": "number of blank votes in a voting round for a specific position in a election",
        "data_type": "Quantity"
    },
    "P2964": {
        "label": "right to vote",
        "description": "people who have right to vote at this election, referendum, etc.",
        "data_type": "WikibaseItem"
    },
    "P541": {
        "label": "office contested",
        "description": "title of office which election will determine the next holder of",
        "data_type": "WikibaseItem"
    },
    "P1867": {
        "label": "eligible voters",
        "description": "number of eligible voters for a particular election",
        "data_type": "Quantity"
    },
    "P1868": {
        "label": "ballots cast",
        "description": "total number of ballot(s) cast, including invalid or blank ballots",
        "data_type": "Quantity"
    },
    "P7215": {
        "label": "Eiga.com person ID",
        "description": "identifier of person on Eiga.com database",
        "data_type": "ExternalId"
    },
    "P8682": {
        "label": "number of negative votes",
        "description": "number of votes that a referendum, proposal, bill, etc. has been rejected in a vote",
        "data_type": "Quantity"
    },
    "P8683": {
        "label": "number of support votes",
        "description": "number of votes that a referendum, proposal, bill, etc. has received support in a vote. Other vote-counting properties are aimed at general elections where voting translates into support, but there is no explicit option to vote for / against",
        "data_type": "Quantity"
    },
    "P8035": {
        "label": "m3db.com film ID",
        "description": "identifier for a Malayalam film (movie) in the m3db.com archive",
        "data_type": "ExternalId"
    },
    "P7553": {
        "label": "Sinemalar person ID",
        "description": "identifier for a person on the Sinemalar site (cinema)",
        "data_type": "ExternalId"
    },
    "P5576": {
        "label": "EDb company ID",
        "description": "identifier for a company at the Israeli movie database EDb",
        "data_type": "ExternalId"
    },
    "P5151": {
        "label": "Israel Film Fund ID",
        "description": "ID for movies backed by the Israel Film Fund",
        "data_type": "ExternalId"
    },
    "P5044": {
        "label": "number of spoilt votes",
        "description": "number of spoilt votes in a voting round for a specfic position in a election",
        "data_type": "Quantity"
    },
    "P5043": {
        "label": "number of abstentions",
        "description": "number of abstentions in a voting round of a election",
        "data_type": "Quantity"
    },
    "P5377": {
        "label": "Étonnants voyageurs participant ID",
        "description": "identifier for a participant on the Étonnants voyageurs website",
        "data_type": "ExternalId"
    },
    "P4185": {
        "label": "iconographic symbol",
        "description": "identifying element typically depicted as accompanying or worn by this religious figure, hero, fictional or historical character",
        "data_type": "WikibaseItem"
    },
    "P8706": {
        "label": "ritual object",
        "description": "ceremonial or ritual objects associated with, and used as part of, rites and rituals practiced in everyday life and in rarer cultic and communal rites",
        "data_type": "WikibaseItem"
    },
    "P611": {
        "label": "religious order",
        "description": "order of monks or nuns to which an individual or religious house belongs",
        "data_type": "WikibaseItem"
    },
    "P2998": {
        "label": "age of candidacy",
        "description": "minimum age at which a person can legally qualify to hold certain elected government offices",
        "data_type": "Quantity"
    },
    "P8233": {
        "label": "Augustins de l'Assomption ID",
        "description": "identifier for an Assumptionist on the website of the order",
        "data_type": "ExternalId"
    },
    "P360": {
        "label": "is a list of",
        "description": "common element between all listed items",
        "data_type": "WikibaseItem"
    },
    "P991": {
        "label": "successful candidate",
        "description": "person(s) elected after the election",
        "data_type": "WikibaseItem"
    },
    "P8796": {
        "label": "FilmPolski press collection ID",
        "data_type": "ExternalId"
    },
    "P1697": {
        "label": "total valid votes",
        "description": "vote count for the elections (excluding invalid votes)",
        "data_type": "Quantity"
    },
    "P9203": {
        "label": "CineFAN.ro title ID",
        "description": "identifier for a movie on the CineFAN.ro website",
        "data_type": "ExternalId"
    },
    "P9204": {
        "label": "CinemaRX title ID",
        "description": "identifier of a movie on cinemarx.ro",
        "data_type": "ExternalId"
    },
    "P9205": {
        "label": "Sinemalar title ID",
        "description": "identifier of a movie on sinemalar.com",
        "data_type": "ExternalId"
    },
    "P8414": {
        "label": "AlloCiné theater ID",
        "description": "identifier for a movie theater in the AlloCiné database",
        "data_type": "ExternalId"
    },
    "P6741": {
        "label": "Kinosozvezdie biography ID",
        "description": "ID of biographical article at Sergei Nikolaev's project Kinosozvezdie",
        "data_type": "ExternalId"
    },
    "P5873": {
        "label": "place of devotion",
        "description": "place where a religious devotion was practiced and for which this devotional image (or text) was produced",
        "data_type": "WikibaseItem"
    },
    "P7105": {
        "label": "LezWatch.TV actor ID",
        "description": "IDs for actors listed in the LezWatch.TV actor database",
        "data_type": "ExternalId"
    },
    "P8456": {
        "label": "ASCRG 2016 ID",
        "description": "identifier in the Australian Standard Classification of Religious Groups 2016",
        "data_type": "ExternalId"
    },
    "P9229": {
        "label": "DzygaMDB person ID",
        "description": "identifier for a person in the DzygaMDB database",
        "data_type": "ExternalId"
    },
    "P2817": {
        "label": "appears in the heritage monument list",
        "description": "heritage monument is a part of the list of heritage monuments",
        "data_type": "WikibaseItem"
    },
    "P4247": {
        "label": "Tagesschau election ID",
        "description": "identifier of a parliamentary election in the elections archive on tagesschau.de",
        "data_type": "ExternalId"
    },
    "P9102": {
        "label": "CineFanRo person ID",
        "description": "identifier for a person on the cinefan.ro site",
        "data_type": "ExternalId"
    },
    "P8013": {
        "label": "Trakt.tv ID",
        "description": "Identifier for a work or person on the website Trakt.tv",
        "data_type": "ExternalId"
    },
    "P1750": {
        "label": "name day",
        "description": "day of the year associated with a first/given name. A qualifier should be used to identify the calendar that is being used. Distinguish from \"feast day\" (P:P841)",
        "data_type": "WikibaseItem"
    },
    "P6762": {
        "label": "FilmFreeway ID",
        "description": "identifier for film festivals in the FilmFreeway.com database",
        "data_type": "ExternalId"
    },
    "P8636": {
        "label": "McClintock and Strong Biblical Cyclopedia ID",
        "description": "identifier for an article in McClintock and Strong Biblical Cyclopedia",
        "data_type": "ExternalId"
    },
    "P726": {
        "label": "candidate",
        "description": "person or party that is an option for an office in this election",
        "data_type": "WikibaseItem"
    },
    "P8817": {
        "label": "NCAA organization code",
        "description": "identifier for an organization designated by the National Collegiate Athletic Association",
        "data_type": "ExternalId"
    },
    "P7948": {
        "label": "GreatSchools ID",
        "description": "identifier for an American school on the GreatSchools.org website",
        "data_type": "ExternalId"
    },
    "P588": {
        "label": "coolant",
        "description": "substance used by the subject to dissipate excess thermal energy",
        "data_type": "WikibaseItem"
    },
    "P8801": {
        "label": "MSRI institution ID",
        "description": "identifier for a mathematics organization, university, department, research institute, etc. on the Mathematical Sciences Research Institute website",
        "data_type": "ExternalId"
    },
    "P6744": {
        "label": "FragDenStaat public body slug",
        "description": "identifier for public bodies from FragDenStaat.de",
        "data_type": "ExternalId"
    },
    "P516": {
        "label": "powered by",
        "description": "equipment or engine used by the subject to convert a source or energy into mechanical energy",
        "data_type": "WikibaseItem"
    },
    "P7945": {
        "label": "ArchiWebture ID",
        "description": "identifier for an architect on the ArchiWebture website",
        "data_type": "ExternalId"
    },
    "P7856": {
        "label": "museum in Salzburg (AT) ID",
        "description": "identifier to link museums and other GLAM-institutions in the Austrian federal state Salzbourg",
        "data_type": "ExternalId"
    },
    "P8053": {
        "label": "Latvian unified registration number",
        "description": "identifier for a legal entity in Latvia",
        "data_type": "ExternalId"
    },
    "P8562": {
        "label": "JSTOR publisher ID",
        "data_type": "ExternalId"
    },
    "P8291": {
        "label": "AusStage organization ID",
        "description": "identifier for an organisation at AusStage",
        "data_type": "ExternalId"
    },
    "P6782": {
        "label": "ROR ID",
        "description": "identifier for the Research Organization Registry",
        "data_type": "ExternalId"
    },
    "P8825": {
        "label": "Sports-Reference.com college basketball school ID",
        "description": "identifier for a NCAA college men's basketball team on the Sports-Reference.com college basketball website",
        "data_type": "ExternalId"
    },
    "P8126": {
        "label": "Club Netherlands Handball Association ID",
        "description": "official number given by the NHA to official clubs",
        "data_type": "ExternalId"
    },
    "P6773": {
        "label": "HAL structure ID",
        "description": "numerical code for a research institution as stored in HAL author affiliations",
        "data_type": "ExternalId"
    },
    "P2031": {
        "label": "work period (start)",
        "description": "start of period during which a person or group flourished (fl. = \"floruit\") in their professional activity",
        "data_type": "Time"
    },
    "P6820": {
        "label": "Australian bank branch ID",
        "description": "unique identifier of a bank branch in Australia",
        "data_type": "ExternalId"
    },
    "P8761": {
        "label": "Sports-Reference.com college football school ID",
        "description": "identifier for an NCAA Division I college football team on Sports-Reference.com",
        "data_type": "ExternalId"
    },
    "P8200": {
        "label": "BeWeb cultural institution ID",
        "description": "identifier for a cultural institution, in the Italian Episcopal Conference database",
        "data_type": "ExternalId"
    },
    "P6691": {
        "label": "GruCultura ID",
        "description": "identifier of cultural relevant places in the Brazilian city Guarulhos",
        "data_type": "ExternalId"
    },
    "P8167": {
        "label": "WhatDoTheyKnow organisation ID",
        "description": "identifier for a British public organisation in the WhatDoTheyKnow database",
        "data_type": "ExternalId"
    },
    "P9019": {
        "label": "Medieval Manuscripts in Oxford Libraries organization ID",
        "description": "identifier of an organization in the online database Medieval Manuscripts in Oxford Libraries",
        "data_type": "ExternalId"
    },
    "P8129": {
        "label": "IDU theatre name unique ID",
        "description": "identifier for theatres (not theatre buildings) in Czechia in the Theatre Institute database",
        "data_type": "ExternalId"
    },
    "P8716": {
        "label": "Mir@bel publisher ID",
        "description": "numerical identifier for a journal publisher on Mir@bel",
        "data_type": "ExternalId"
    },
    "P9201": {
        "label": "Norwegian State Administration Database ID",
        "description": "identifier for data from the Norwegian civil service/state administration after the Second World War",
        "data_type": "ExternalId"
    },
    "P6869": {
        "label": "Hoopla publisher ID",
        "description": "identifier for a publisher on Hoopla",
        "data_type": "ExternalId"
    },
    "P6989": {
        "label": "Hungarian National Namespace organisation ID",
        "description": "identifier for an organisation in the Hungarian National Namespace",
        "data_type": "ExternalId"
    },
    "P8118": {
        "label": "Forest Stewardship Council Certificate Code",
        "description": "unique identifier of a certificate issued by the Forest Stewardship Council",
        "data_type": "ExternalId"
    },
    "P8128": {
        "label": "Forest Stewardship Council License Code",
        "description": "unique identifier of a license issued by the Forest Stewardship Council",
        "data_type": "ExternalId"
    },
    "P8174": {
        "label": "Slovak Registration ID",
        "description": "identifier for an organization in Slovakia assigned by the Statistical Office of the Slovak Republic",
        "data_type": "ExternalId"
    },
    "P2032": {
        "label": "work period (end)",
        "description": "end of period during which a person or group flourished (fl. = \"floruit\") in their professional activity",
        "data_type": "Time"
    },
    "P8545": {
        "label": "Nasjonalt skoleregister ID",
        "description": "identifier of an educational institution in the Nasjonalt skoleregister (National School Registry) from the Norwegian Directorate for Education and Training",
        "data_type": "ExternalId"
    },
    "P6309": {
        "label": "historical Répertoire national des associations identifier",
        "description": "identifier of an association in the historical database of the Répertoire national des associations",
        "data_type": "ExternalId"
    },
    "P9255": {
        "label": "Resident Advisor club ID",
        "description": "identifier for a club or venue listed on the Resident Advisor website",
        "data_type": "ExternalId"
    },
    "P8063": {
        "label": "BookBrainz publisher ID",
        "description": "identifier for a publisher or imprint on the BookBrainz open book encyclopedia",
        "data_type": "ExternalId"
    },
    "P7011": {
        "label": "Russian organisation number",
        "description": "identifier in the Uniform State Register of Legal Entities (USRLE) of the Russian Federation",
        "data_type": "ExternalId"
    },
    "P8182": {
        "label": "South African Company Registration Number",
        "description": "registration number of South African companies issued by the Companies and Intellectual Property Commission of South Africa",
        "data_type": "ExternalId"
    },
    "P6696": {
        "label": "PRELIB organization ID",
        "description": "identifier for an organization on the Projet de recherche en littérature de langue bretonne website",
        "data_type": "ExternalId"
    },
    "P9416": {
        "label": "ScoreStorybook company ID",
        "description": "identifier for a company or nonprofit organization in ScoreStorybook",
        "data_type": "ExternalId"
    },
    "P9372": {
        "label": "NHS Organisation Data Service ID",
        "description": "identifier for a component organisation of the UK's National Health Service",
        "data_type": "ExternalId"
    },
    "P9418": {
        "label": "Biblio.com ID",
        "description": "alphabetical identifier for a bookstore on the Biblio.com website",
        "data_type": "ExternalId"
    },
    "P1086": {
        "label": "atomic number",
        "description": "number of protons found in the nucleus of the atom",
        "data_type": "Quantity"
    },
    "P2407": {
        "label": "short-term exposure limit",
        "description": "recommended or required concentration limit for chemical exposure during a brief time window",
        "data_type": "Quantity"
    },
    "P2993": {
        "label": "partition coefficient water/octanol",
        "description": "logarithm of the partition coefficient (n-octanol/water)",
        "data_type": "Quantity"
    },
    "P2240": {
        "label": "median lethal dose (LD50)",
        "description": "the dose required to kill half the members of a tested population after a specified test duration (LC₅₀ → P2712)",
        "data_type": "Quantity"
    },
    "P2203": {
        "label": "upper flammable limit",
        "description": "upper bound of the concentration range over which a flammable mixture of gas or vapour in air can be ignited at a given temperature and pressure (0-1)",
        "data_type": "Quantity"
    },
    "P4239": {
        "label": "vocalized name",
        "description": "item name with full vowel marks",
        "data_type": "Monolingualtext"
    },
    "P3321": {
        "label": "male form of label",
        "description": "male form of name of an element",
        "data_type": "Monolingualtext"
    },
    "P1117": {
        "label": "pKa",
        "description": "acid dissociation constant",
        "data_type": "Quantity"
    },
    "P661": {
        "label": "ChemSpider ID",
        "description": "identifier in a free chemical database, owned by the Royal Society of Chemistry",
        "data_type": "ExternalId"
    },
    "P2405": {
        "label": "ceiling exposure limit",
        "description": "recommended or required maximum concentration for chemical exposure in a given work day",
        "data_type": "Quantity"
    },
    "P5056": {
        "label": "patronym or matronym for this person",
        "description": "part of a person's name that comes from the given name of a parent, in naming traditions that require it",
        "data_type": "WikibaseItem"
    },
    "P2665": {
        "label": "alcohol by volume",
        "description": "percentage of alcohol in a fluid",
        "data_type": "Quantity"
    },
    "P660": {
        "label": "EC enzyme classification",
        "description": "the Enzyme Commission (EC)-based accepted name of any enzyme classifications of the protein or RNA molecule",
        "data_type": "WikibaseItem"
    },
    "P592": {
        "label": "ChEMBL ID",
        "description": "identifier from a chemical database of bioactive molecules with drug-like properties",
        "data_type": "ExternalId"
    },
    "P652": {
        "label": "UNII",
        "description": "identifier issued by the FDA / Unique Ingredient Identifier",
        "data_type": "ExternalId"
    },
    "P2926": {
        "label": "InterPro ID",
        "description": "InterPro unique identifier",
        "data_type": "ExternalId"
    },
    "P2374": {
        "label": "natural abundance",
        "description": "relative proportion of an isotope as found in nature (Earth's crust, or other location with qualifiers)",
        "data_type": "Quantity"
    },
    "P876": {
        "label": "UN packaging group",
        "description": "packaging code according to UN transportation rules",
        "data_type": "String"
    },
    "P874": {
        "label": "UN class",
        "description": "UN hazard classification code",
        "data_type": "String"
    },
    "P2275": {
        "label": "World Health Organisation International Nonproprietary Name",
        "description": "identifier for a drug",
        "data_type": "Monolingualtext"
    },
    "P2521": {
        "label": "female form of label",
        "description": "female form of name or title",
        "data_type": "Monolingualtext"
    },
    "P1705": {
        "label": "native label",
        "description": "label for item in its official or original language",
        "data_type": "Monolingualtext"
    },
    "P1668": {
        "label": "ATCvet",
        "description": "ATCvet code",
        "data_type": "ExternalId"
    },
    "P1579": {
        "label": "Reaxys registry number",
        "description": "identifier for chemical substance in Reaxys",
        "data_type": "ExternalId"
    },
    "P2084": {
        "label": "ZINC ID",
        "description": "identifier for a chemical compound in the ZINC database",
        "data_type": "ExternalId"
    },
    "P1108": {
        "label": "electronegativity",
        "description": "tendency of an atom or functional group to attract electrons",
        "data_type": "Quantity"
    },
    "P873": {
        "label": "phase point",
        "description": "phase point to describe critical point and triple point (see talk page for an example)",
        "data_type": "WikibaseItem"
    },
    "P1121": {
        "label": "oxidation state",
        "description": "this atom has or can have the oxidation number",
        "data_type": "Quantity"
    },
    "P2562": {
        "label": "married name",
        "description": "name or surname adopted by a person upon marriage",
        "data_type": "Monolingualtext"
    },
    "P875": {
        "label": "UN code classification",
        "description": "UN code classification",
        "data_type": "String"
    },
    "P5278": {
        "label": "gender inflection of surname",
        "description": "surname version for other gender",
        "data_type": "WikibaseItem"
    },
    "P2976": {
        "label": "patronym or matronym for this name",
        "description": "patronym or matronym based on this given name",
        "data_type": "WikibaseItem"
    },
    "P1148": {
        "label": "neutron number",
        "description": "number of neutrons of an isotope",
        "data_type": "Quantity"
    },
    "P2204": {
        "label": "minimum explosive concentration",
        "description": "minimum concentration at which a chemical can explode",
        "data_type": "Quantity"
    },
    "P2658": {
        "label": "Scoville grade",
        "description": "scale measuring pungency of chili peppers (values from 0 to 16,000,000,000)",
        "data_type": "Quantity"
    },
    "P1813": {
        "label": "short name",
        "description": "short name of a place, organisation, person, Wikidata property, etc.",
        "data_type": "Monolingualtext"
    },
    "P6978": {
        "label": "Scandinavian middle family name",
        "description": "middle name of some people from Denmark or Norway where a family name, usually the mother's, is taken on as a \"middle name\". For given names based on family names, use \"given name\" (P735). For Spanish names, use P734/P1950.",
        "data_type": "WikibaseItem"
    },
    "P2712": {
        "label": "median lethal concentration (LC50)",
        "description": "statistically derived median concentration of a substance in an environmental medium expected to kill 50% of organisms in a given population under a defined set of conditions",
        "data_type": "Quantity"
    },
    "P556": {
        "label": "crystal system",
        "description": "type of crystal for minerals and/or for crystal compounds",
        "data_type": "WikibaseItem"
    },
    "P2406": {
        "label": "maximum peak exposure limit",
        "description": "recommended or required maximum concentration for chemical exposure during a five-minute excursion within a certain period of hours",
        "data_type": "Quantity"
    },
    "P2414": {
        "label": "substrate of",
        "description": "substrate that an enzyme acts upon to create a product",
        "data_type": "WikibaseItem"
    },
    "P4566": {
        "label": "awarded for period",
        "description": "period during which the achievement must have been made to be eligible for an award",
        "data_type": "Time"
    },
    "P1411": {
        "label": "nominated for",
        "description": "award nomination received by a person, organisation or creative work (inspired from \"award received\" (Property:P166))",
        "data_type": "WikibaseItem"
    },
    "P9375": {
        "label": "Sociologická encyklopedie ID",
        "description": "identifier for an article in Sociologická encyklopedie",
        "data_type": "ExternalId"
    },
    "P2121": {
        "label": "prize money",
        "description": "amount in a specific currency",
        "data_type": "Quantity"
    },
    "P3733": {
        "label": "MOOMA artist ID",
        "description": "ID of an artist at the Israeli music website MOOMA",
        "data_type": "ExternalId"
    },
    "P4444": {
        "label": "reward",
        "description": "reward, bonus, or prize received as part of a membership or loyalty program",
        "data_type": "WikibaseItem"
    },
    "P9407": {
        "label": "Megale Kypriake Enkyklopaideia ID",
        "description": "identifier for an article in the Cypriot encyclopedia Megale Kypriake Enkyklopaideia (Great Cypriot Encyclopedia)",
        "data_type": "ExternalId"
    },
    "P5645": {
        "label": "Académie française award winner ID",
        "description": "identifier for a prize winner on the Académie française website",
        "data_type": "ExternalId"
    },
    "P2173": {
        "label": "BBC News Democracy Live ID",
        "description": "identifer in the BBC News Democracy Live database of British MPs",
        "data_type": "ExternalId"
    },
    "P2962": {
        "label": "title of chess person",
        "description": "title awarded by a chess federation to a person",
        "data_type": "WikibaseItem"
    },
    "P9288": {
        "label": "EncyclopediaPR ID",
        "description": "identifier for an article in the EncyclopediaPR",
        "data_type": "ExternalId"
    },
    "P3809": {
        "label": "YerelNET district ID",
        "description": "identifier for a district of Turkey, in the YerelNET database",
        "data_type": "ExternalId"
    },
    "P5533": {
        "label": "Kopaliński Online ID",
        "description": "identifier for an entry in the online version of 'Dictionary of Foreign Words and Foreign-language Phrases' by Władysław Kopaliński",
        "data_type": "ExternalId"
    },
    "P1713": {
        "label": "biography at the Bundestag of Germany URL",
        "description": "link to the biography pages of the German parliament",
        "data_type": "Url"
    },
    "P101": {
        "label": "field of work",
        "description": "specialization of a person or organization; see P106 for the occupation",
        "data_type": "WikibaseItem"
    },
    "P9365": {
        "label": "Modernist Commons ID",
        "description": "identifier for an article in the encyclopedia Modernist Commons",
        "data_type": "ExternalId"
    },
    "P2678": {
        "label": "Russiancinema.ru film ID",
        "description": "id number for films in Russiancinema.ru",
        "data_type": "ExternalId"
    },
    "P5460": {
        "label": "grants",
        "description": "confers degree, honor, award, prize, title, certificate or medal denoting achievement to a person or organization",
        "data_type": "WikibaseItem"
    },
    "P1253": {
        "label": "BCU Ecrivainsvd ID",
        "description": "identifier in the database of writers from the canton of Vaud",
        "data_type": "ExternalId"
    },
    "P3183": {
        "label": "Wall Street Journal topic ID",
        "description": "identifier for a topic, at the Wall Street Journal website",
        "data_type": "ExternalId"
    },
    "P530": {
        "label": "diplomatic relation",
        "description": "diplomatic relations of the country",
        "data_type": "WikibaseItem"
    },
    "P2847": {
        "label": "Google Currents ID",
        "description": "Google Currents, account identifier of this person or organization: either starting with a \"+\" or consisting of 21 digits",
        "data_type": "ExternalId"
    },
    "P9336": {
        "label": "Das wissenschaftliche Bibellexikon im Internet ID",
        "description": "identifier in the German Bible encyclopedia online, written by academics and theologians",
        "data_type": "ExternalId"
    },
    "P9281": {
        "label": "Encyclopedia of Greater Philadelphia ID",
        "description": "alphabetical identifier for an article in the Encyclopedia of Greater Philadelphia",
        "data_type": "ExternalId"
    },
    "P4622": {
        "label": "trophy awarded",
        "description": "trophy awarded at the end of a selection process or of a competition, usually to winners or finishers under the form of a cup, a medal, a diploma",
        "data_type": "WikibaseItem"
    },
    "P9304": {
        "label": "Encyclopedia of Melbourne ID",
        "description": "identifier for an article in the Encyclopedia of Melbourne",
        "data_type": "ExternalId"
    },
    "P2172": {
        "label": "Parliamentary record identifier",
        "description": "Parliamentary record identifier for British MPs",
        "data_type": "ExternalId"
    },
    "P5318": {
        "label": "César Award film ID",
        "description": "identifier for a film on the César Awards website",
        "data_type": "ExternalId"
    },
    "P8115": {
        "label": "eligible recipient",
        "description": "party that is eligible to receive something, like an award, grant or prize",
        "data_type": "WikibaseItem"
    },
    "P6145": {
        "label": "Academy Awards Database film ID",
        "description": "identifier for a film in the official Academy Awards database",
        "data_type": "ExternalId"
    },
    "P3143": {
        "label": "elFilm film ID",
        "description": "identifier for a movie at the Arabic movie database elFilm",
        "data_type": "ExternalId"
    },
    "P4199": {
        "label": "Google Play Music album ID (former scheme)",
        "description": "identifier for an album, single, other music release or other publication on the Google Play Music website. Obsolete starting December 2020",
        "data_type": "ExternalId"
    },
    "P4198": {
        "label": "Google Play Music artist ID (former scheme)",
        "description": "identifier of an artist on the Google Play Music website. Obsolete starting December 2020",
        "data_type": "ExternalId"
    },
    "P9291": {
        "label": "Eesti entsüklopeedia ID",
        "description": "identifier for an article in the online version of the Eesti entsüklopeedia (Estonia Encyclopedia)",
        "data_type": "ExternalId"
    },
    "P9256": {
        "label": "Diccionari de la traducció catalana ID",
        "description": "identifier for an article in the Diccionari de la traducció catalana",
        "data_type": "ExternalId"
    },
    "P9285": {
        "label": "Stadtlexikon Karlsruhe ID",
        "description": "alphanumerical identifier for a person, a places, an institution or an event in the Stadtlexikon Karlsruhe, a reference work on the German city",
        "data_type": "ExternalId"
    },
    "P8800": {
        "label": "European Film Awards ID",
        "description": "numerical identifier for a film on the European Film Awards website",
        "data_type": "ExternalId"
    },
    "P9286": {
        "label": "Encyclopaedia Herder concept ID",
        "description": "identifier for a concept in the Encyclopaedia Herder",
        "data_type": "ExternalId"
    },
    "P689": {
        "label": "afflicts",
        "description": "type of organism or organ which a condition or disease afflicts",
        "data_type": "WikibaseItem"
    },
    "P970": {
        "label": "neurological function",
        "description": "the function of a neurological structure",
        "data_type": "WikibaseItem"
    },
    "P492": {
        "label": "OMIM ID",
        "description": "Online \"Mendelian Inheritance in Man\" catalogue codes for diseases, genes, or phenotypes",
        "data_type": "ExternalId"
    },
    "P637": {
        "label": "RefSeq protein ID",
        "description": "identifier for a protein",
        "data_type": "ExternalId"
    },
    "P962": {
        "label": "MycoBank taxon name ID",
        "description": "identifier per MycoBank - a database that provides an authoritative and comprehensive list of names of fungi",
        "data_type": "ExternalId"
    },
    "P672": {
        "label": "MeSH tree code",
        "description": "Medical Subject Headings (MeSH) codes are an index and thesaurus for the life sciences (≠ MeSH ID, P486)",
        "data_type": "ExternalId"
    },
    "P8702": {
        "label": "parallel street",
        "description": "closest street(s) that run parallel to the item",
        "data_type": "WikibaseItem"
    },
    "P1930": {
        "label": "DSM-5 classification",
        "description": "identifier for a mental disorder in the 5th edition of Diagnostic and Statistical Manual of Mental Disorders",
        "data_type": "ExternalId"
    },
    "P1603": {
        "label": "number of cases",
        "description": "cumulative number of confirmed, probable and suspected occurrences",
        "data_type": "Quantity"
    },
    "P8701": {
        "label": "has census",
        "description": "general population census of the country or region",
        "data_type": "WikibaseItem"
    },
    "P1055": {
        "label": "NLM Unique ID",
        "description": "identifier in the catalog of the National Library of Medicine",
        "data_type": "ExternalId"
    },
    "P1924": {
        "label": "vaccine for",
        "description": "disease that a vaccine is for",
        "data_type": "WikibaseItem"
    },
    "P673": {
        "label": "eMedicine ID",
        "description": "identifier in the online clinical medical knowledge base",
        "data_type": "ExternalId"
    },
    "P595": {
        "label": "Guide to Pharmacology Ligand ID",
        "description": "ligand identifier of the Guide to Pharmacology database",
        "data_type": "ExternalId"
    },
    "P1917": {
        "label": "posttranslational modification",
        "description": "This property should link a protein",
        "data_type": "WikibaseItem"
    },
    "P780": {
        "label": "symptoms",
        "description": "possible symptoms of a medical condition",
        "data_type": "WikibaseItem"
    },
    "P593": {
        "label": "HomoloGene ID",
        "description": "identifier in the HomoloGene database",
        "data_type": "String"
    },
    "P1057": {
        "label": "chromosome",
        "description": "chromosome on which an entity is localized",
        "data_type": "WikibaseItem"
    },
    "P1391": {
        "label": "Index Fungorum ID",
        "description": "identifier for a fungus taxon in Index Fungorum",
        "data_type": "ExternalId"
    },
    "P665": {
        "label": "KEGG ID",
        "description": "identifier from databases dealing with genomes, enzymatic pathways, and biological chemicals",
        "data_type": "ExternalId"
    },
    "P7254": {
        "label": "Pilze Deutschland ID",
        "description": "identifier for a taxon in the database of German Mycological Society (voluntary association - DGfM)",
        "data_type": "ExternalId"
    },
    "P636": {
        "label": "route of administration",
        "description": "path by which a drug, fluid, poison, or other substance is taken into the body",
        "data_type": "WikibaseItem"
    },
    "P786": {
        "label": "stipe character",
        "description": "indicates whether a mushroom has a universal or partial veil",
        "data_type": "WikibaseItem"
    },
    "P1060": {
        "label": "pathogen transmission process",
        "description": "process by which a pathogen is transmitted, equivalent to \"transmitted by\" in the relation ontology http://purl.obolibrary.org/obo/RO_0002451",
        "data_type": "WikibaseItem"
    },
    "P668": {
        "label": "GeneReviews ID",
        "description": "collection of peer-reviewed articles that describe specific gene-releated diseases",
        "data_type": "ExternalId"
    },
    "P1909": {
        "label": "side effect",
        "description": "effect of a medication or procedure, that occurs next to the desired effect",
        "data_type": "WikibaseItem"
    },
    "P6101": {
        "label": "Mushroom Observer ID",
        "description": "identifier of a taxon name at Mushroom Observer",
        "data_type": "ExternalId"
    },
    "P688": {
        "label": "encodes",
        "description": "the product of a gene (protein or RNA)",
        "data_type": "WikibaseItem"
    },
    "P8890": {
        "label": "power consumption index",
        "description": "value of the relative energy consumption from the energy certificate of a building",
        "data_type": "Quantity"
    },
    "P8891": {
        "label": "heating energy consumption index",
        "description": "value of the relative heating consumption from the energy certificate of a building",
        "data_type": "Quantity"
    },
    "P1604": {
        "label": "biosafety level",
        "description": "level of the biocontainment precautions required to isolate dangerous biological agents",
        "data_type": "WikibaseItem"
    },
    "P924": {
        "label": "possible treatment",
        "description": "health treatment used to resolve or ameliorate a medical condition",
        "data_type": "WikibaseItem"
    },
    "P926": {
        "label": "postsynaptic connection",
        "description": "neuron connects on its postsynaptic end to",
        "data_type": "WikibaseItem"
    },
    "P6268": {
        "label": "Danmarks svampeatlas ID",
        "description": "identifier of fungus taxa in Danmarks svampeatlas",
        "data_type": "ExternalId"
    },
    "P1199": {
        "label": "mode of inheritance",
        "description": "manner in which a particular genetic trait or disorder is passed from one generation to the next",
        "data_type": "WikibaseItem"
    },
    "P639": {
        "label": "RefSeq RNA ID",
        "description": "RNA Identifier",
        "data_type": "ExternalId"
    },
    "P925": {
        "label": "presynaptic connection",
        "description": "neuron connects on its presynaptic end to",
        "data_type": "WikibaseItem"
    },
    "P1995": {
        "label": "health specialty",
        "description": "main specialty that diagnoses, prevent human illness, injury and other physical and mental impairments",
        "data_type": "WikibaseItem"
    },
    "P1605": {
        "label": "has natural reservoir",
        "description": "host species for the pathogen in which it is endemic",
        "data_type": "WikibaseItem"
    },
    "P769": {
        "label": "significant drug interaction",
        "description": "clinically significant interaction between two pharmacologically active substances (i.e., drugs and/or active metabolites) where concomitant intake can lead to altered effectiveness or adverse drug events.",
        "data_type": "WikibaseItem"
    },
    "P1691": {
        "label": "operations and procedures key (OPS)",
        "description": "official classification of operational procedures",
        "data_type": "ExternalId"
    },
    "P783": {
        "label": "hymenium type",
        "description": "type of spore-bearing surface or that mushroom",
        "data_type": "WikibaseItem"
    },
    "P789": {
        "label": "edibility",
        "description": "whether a mushroom can be eaten or not",
        "data_type": "WikibaseItem"
    },
    "P785": {
        "label": "hymenium attachment",
        "description": "how the hymenium of the mushroom attaches to the stem",
        "data_type": "WikibaseItem"
    },
    "P784": {
        "label": "mushroom cap shape",
        "description": "property classifying the shape of the cap of a mushroom",
        "data_type": "WikibaseItem"
    },
    "P788": {
        "label": "mushroom ecological type",
        "description": "property classifying the ecological type of a mushroom",
        "data_type": "WikibaseItem"
    },
    "P787": {
        "label": "spore print color",
        "description": "color of a mushroom spore print (see documentation for allowed values)",
        "data_type": "WikibaseItem"
    },
    "P9241": {
        "label": "demographics of topic",
        "description": "item that deals with demographics of the subject. Sample: \"Andorra\" uses this property with value \"demographics of Andorra\" (Q2087350).",
        "data_type": "WikibaseItem"
    },
    "P1660": {
        "label": "has index case",
        "description": "initial patient in the population of an epidemiological investigation",
        "data_type": "WikibaseItem"
    },
    "P1606": {
        "label": "natural reservoir of",
        "description": "pathogen of which this species is a long-term host",
        "data_type": "WikibaseItem"
    },
    "P1193": {
        "label": "prevalence",
        "description": "portion in percent of a population with a given disease or disorder",
        "data_type": "Quantity"
    },
    "P715": {
        "label": "DrugBank ID",
        "description": "identifier in the bioinformatics and cheminformatics database from the University of Alberta",
        "data_type": "ExternalId"
    },
    "P923": {
        "label": "medical examinations",
        "description": "examinations that might be used to diagnose the medical condition",
        "data_type": "WikibaseItem"
    },
    "P2974": {
        "label": "habitat",
        "description": "the natural environment in which an organism lives, or the physical environment that surrounds a species population",
        "data_type": "WikibaseItem"
    },
    "P4552": {
        "label": "mountain range",
        "description": "range or subrange to which the geographical item belongs",
        "data_type": "WikibaseItem"
    },
    "P5588": {
        "label": "invasive to",
        "description": "region which a taxon is not endemic to where the taxon has a tendency to spread to a degree believed to cause damage to the environment, human economy or human health",
        "data_type": "WikibaseItem"
    },
    "P2975": {
        "label": "host",
        "description": "an organism harboring another organism or organisms on or in itself",
        "data_type": "WikibaseItem"
    },
    "P1334": {
        "label": "coordinates of easternmost point",
        "description": "easternmost point of a location",
        "data_type": "GlobeCoordinate"
    },
    "P1335": {
        "label": "coordinates of westernmost point",
        "description": "westernmost point of a location",
        "data_type": "GlobeCoordinate"
    },
    "P524": {
        "label": "temporal range end",
        "description": "the end of a process such as a geological unit or extinction of a life form relative to the geologic time scale",
        "data_type": "WikibaseItem"
    },
    "P105": {
        "label": "taxon rank",
        "description": "level in a taxonomic hierarchy",
        "data_type": "WikibaseItem"
    },
    "P504": {
        "label": "home port",
        "description": "home port of the vessel (if different from \"ship registry\"): For civilian ships, the primary port from which the ship operates. Port of registry →P532 should be listed in \"Ship registry\". For warships, this will be the ship's assigned naval base",
        "data_type": "WikibaseItem"
    },
    "P376": {
        "label": "located on astronomical body",
        "description": "astronomical body on which features or places are situated",
        "data_type": "WikibaseItem"
    },
    "P613": {
        "label": "OS grid reference",
        "description": "grid location reference from the Ordnance Survey National Grid reference system used in Great Britain",
        "data_type": "String"
    },
    "P1843": {
        "label": "taxon common name",
        "description": "common or vernacular name of a biological taxon",
        "data_type": "Monolingualtext"
    },
    "P1420": {
        "label": "taxon synonym",
        "description": "name listed as synonym of a taxon name",
        "data_type": "WikibaseItem"
    },
    "P183": {
        "label": "endemic to",
        "description": "sole location or habitat type where the taxon lives",
        "data_type": "WikibaseItem"
    },
    "P5998": {
        "label": "distributary",
        "description": "stream that branches off and flows away from the river",
        "data_type": "WikibaseItem"
    },
    "P3137": {
        "label": "parent peak",
        "description": "parent is the peak whose territory this peak resides in, based on the contour of the lowest col",
        "data_type": "WikibaseItem"
    },
    "P1425": {
        "label": "ecoregion (WWF)",
        "description": "ecoregion of the item (choose from WWF's list)",
        "data_type": "WikibaseItem"
    },
    "P3842": {
        "label": "located in present-day administrative territorial entity",
        "description": "the item was located in the territory of this present-day administrative unit; however the two did not at any point coexist in time",
        "data_type": "WikibaseItem"
    },
    "P3096": {
        "label": "KML file",
        "description": "item with sitelinks to wikimedia page containing text of a relevant KML file",
        "data_type": "WikibaseItem"
    },
    "P206": {
        "label": "located in or next to body of water",
        "description": "sea, lake, river or stream",
        "data_type": "WikibaseItem"
    },
    "P566": {
        "label": "basionym",
        "description": "the legitimate, previously published name on which a new combination or name at new rank is based",
        "data_type": "WikibaseItem"
    },
    "P694": {
        "label": "replaced synonym (for nom. nov.)",
        "description": "previously published name on which a replacement name (avowed substitute, nomen novum) is based.",
        "data_type": "WikibaseItem"
    },
    "P36": {
        "label": "capital",
        "description": "seat of government of a country, province, state or other type of administrative territorial entity",
        "data_type": "WikibaseItem"
    },
    "P3337": {
        "label": "generation time",
        "description": "average time from one generation to another within the same population",
        "data_type": "Quantity"
    },
    "P427": {
        "label": "taxonomic type",
        "description": "the type genus of this family (or subfamily, etc), or the type species of this genus (or subgenus, etc)",
        "data_type": "WikibaseItem"
    },
    "P523": {
        "label": "temporal range start",
        "description": "the start of a process or appearance of a life form or geological unit relative to the geologic time scale",
        "data_type": "WikibaseItem"
    },
    "P1302": {
        "label": "primary destinations",
        "description": "major cities that a road serves",
        "data_type": "WikibaseItem"
    },
    "P1332": {
        "label": "coordinates of northernmost point",
        "description": "northernmost point of a location. For an administrative entity this includes offshore islands",
        "data_type": "GlobeCoordinate"
    },
    "P3512": {
        "label": "means of locomotion",
        "description": "method that the subject uses to move from one place to another",
        "data_type": "WikibaseItem"
    },
    "P703": {
        "label": "found in taxon",
        "description": "the taxon in which the item can be found",
        "data_type": "WikibaseItem"
    },
    "P141": {
        "label": "IUCN conservation status",
        "description": "conservation status assigned by the International Union for Conservation of Nature",
        "data_type": "WikibaseItem"
    },
    "P4214": {
        "label": "highest observed lifespan",
        "description": "age of the oldest specimen of a taxon for which the age can be verified",
        "data_type": "Quantity"
    },
    "P4565": {
        "label": "electoral district number",
        "description": "number of the constituency/electoral district established by law. Only to be used if the number is established by law, regulation, or other legally-binding decision; not to be used for database identifier numbers which lack legal force, even if contained in a government database.",
        "data_type": "String"
    },
    "P1672": {
        "label": "this taxon is source of",
        "description": "links a taxon to natural products it produces. Note that it does not say \"this taxon is the source of\" or \"this taxon is a source of\" as this may vary. Some products may be yielded by more than one taxon.",
        "data_type": "WikibaseItem"
    },
    "P3648": {
        "label": "NatureServe conservation status",
        "description": "conservation status assigned by NatureServe",
        "data_type": "WikibaseItem"
    },
    "P2786": {
        "label": "aerodrome reference point",
        "description": "official coordinates for airport",
        "data_type": "GlobeCoordinate"
    },
    "P6800": {
        "label": "sequenced genome URL",
        "description": "sequenced and assembled genome for this taxon",
        "data_type": "Url"
    },
    "P5841": {
        "label": "Status in the Red List of Threatened Species in the Czech Republic",
        "description": "endangerment status of species in the national Red List of the Czech Republic",
        "data_type": "WikibaseItem"
    },
    "P944": {
        "label": "Code of nomenclature",
        "description": "the Code that governs the scientific name of this taxon",
        "data_type": "WikibaseItem"
    },
    "P5248": {
        "label": "medical evacuation to",
        "description": "site to which those injured are evacuated immediately after a catastrophic event or a battle",
        "data_type": "WikibaseItem"
    },
    "P7153": {
        "label": "significant place",
        "description": "significant or notable places associated with the subject",
        "data_type": "WikibaseItem"
    },
    "P1703": {
        "label": "is pollinated by",
        "description": "how pollen from the male anthers is transferred to the female stigma of a flower",
        "data_type": "WikibaseItem"
    },
    "P2795": {
        "label": "directions",
        "description": "describe how to find the subject - directions, objects along way, comments",
        "data_type": "Monolingualtext"
    },
    "P30": {
        "label": "continent",
        "description": "continent of which the subject is a part",
        "data_type": "WikibaseItem"
    },
    "P47": {
        "label": "shares border with",
        "description": "countries or administrative subdivisions, of equal level, that this item borders, either by land or water. A single common point is enough.",
        "data_type": "WikibaseItem"
    },
    "P1704": {
        "label": "is pollinator of",
        "description": "plant that this animal visits for pollen (and that it pollinates)",
        "data_type": "WikibaseItem"
    },
    "P7108": {
        "label": "location of the point of view",
        "description": "geographic location of point of view, may or may not be depicted",
        "data_type": "WikibaseItem"
    },
    "P8485": {
        "label": "distribution map of taxon",
        "description": "dynamic distribution map of taxon",
        "data_type": "GeoShape"
    },
    "P1403": {
        "label": "original combination",
        "description": "for animals: the combination (binomen or trinomen) where the species-group name used in this taxon name was first published",
        "data_type": "WikibaseItem"
    },
    "P840": {
        "label": "narrative location",
        "description": "the narrative of the work is set in this location",
        "data_type": "WikibaseItem"
    },
    "P1376": {
        "label": "capital of",
        "description": "country, state, department, canton or other administrative division of which the municipality is the governmental seat",
        "data_type": "WikibaseItem"
    },
    "P625": {
        "label": "coordinate location",
        "description": "geocoordinates of the subject. For Earth, please note that only WGS84 coordinating system is supported at the moment",
        "data_type": "GlobeCoordinate"
    },
    "P5326": {
        "label": "publication in which this taxon name was established",
        "description": "individual publication, in which the unit of biological nomenclature was first formally established",
        "data_type": "WikibaseItem"
    },
    "P4091": {
        "label": "Irish Grid Reference",
        "description": "grid location reference from the Irish Grid reference system used in Ireland",
        "data_type": "String"
    },
    "P1589": {
        "label": "lowest point",
        "description": "point with lowest elevation in the country, region, city or area",
        "data_type": "WikibaseItem"
    },
    "P3018": {
        "label": "located in protected area",
        "description": "protected area where a place or geographical feature is physically located",
        "data_type": "WikibaseItem"
    },
    "P2433": {
        "label": "gender of a scientific name of a genus",
        "description": "determines the correct form of some names of species and subdivisions of species, also subdivisions of a genus",
        "data_type": "WikibaseItem"
    },
    "P931": {
        "label": "place served by transport hub",
        "description": "territorial entity or entities served by this transport hub (airport, train station, etc.)",
        "data_type": "WikibaseItem"
    },
    "P4688": {
        "label": "geomorphological unit",
        "description": "topographic or bathymetric feature to which this geographical item belongs",
        "data_type": "WikibaseItem"
    },
    "P3032": {
        "label": "adjacent building",
        "description": "building adjacent to the item",
        "data_type": "WikibaseItem"
    },
    "P1333": {
        "label": "coordinates of southernmost point",
        "description": "southernmost point of a place. For administrative entities this includes offshore islands",
        "data_type": "GlobeCoordinate"
    },
    "P7598": {
        "label": "ascending scale",
        "description": "sequence of notes starting from a tonic in one octave and ending with the tonic one octave above",
        "data_type": "MusicalNotation"
    },
    "P6686": {
        "label": "musical motif",
        "description": "short recurring melodic or rhythmic musical phrase in this musical work, in LilyPond syntax",
        "data_type": "MusicalNotation"
    },
    "P8454": {
        "label": "Vmusic.bg artist ID",
        "description": "identifier of an artist at Bulgarian website Vmusic.bg",
        "data_type": "ExternalId"
    },
    "P3040": {
        "label": "SoundCloud ID",
        "description": "identifier for a person, band, radio station or other entity, on SoundCloud",
        "data_type": "ExternalId"
    },
    "P1762": {
        "label": "Hornbostel-Sachs classification",
        "description": "Hornbostel-Sachs classification of a musical instrument",
        "data_type": "String"
    },
    "P8700": {
        "label": "Scottish Brick History Brick & Tileworks ID",
        "description": "Scottish Brick History project identifier for a brick or tile works in Scotland, without any trailing slash",
        "data_type": "ExternalId"
    },
    "P3763": {
        "label": "MIMO instrument ID",
        "description": "identifier for a musical instrument, on the instrument keyword thesaurus from Musical Instruments Museums Online",
        "data_type": "ExternalId"
    },
    "P6883": {
        "label": "LilyPond notation",
        "description": "musical notation in LilyPond syntax",
        "data_type": "MusicalNotation"
    },
    "P8577": {
        "label": "ICP artist ID",
        "description": "identifier for a photographer on the International Center of Photography website",
        "data_type": "ExternalId"
    },
    "P1330": {
        "label": "MusicBrainz instrument ID",
        "description": "identifier for an instrument on the music encyclopedia MusicBrainz",
        "data_type": "ExternalId"
    },
    "P3997": {
        "label": "Bait La Zemer Ha-Ivri artist ID",
        "description": "ID in the israeli database for Israeli music, owned by the National Library of Israel",
        "data_type": "ExternalId"
    },
    "P8435": {
        "label": "AnyDecentMusic album ID",
        "description": "identifier for album reviews at AnyDecentMusic",
        "data_type": "ExternalId"
    },
    "P7599": {
        "label": "descending scale",
        "description": "sequence of notes starting from a tonic in one octave and ending with the tonic one octave below",
        "data_type": "MusicalNotation"
    },
    "P8645": {
        "label": "Energy Identification Code",
        "description": "code used in Europe to uniquely identify entities and objects related to the electricity and gas sector",
        "data_type": "ExternalId"
    },
    "P1898": {
        "label": "lowest note",
        "description": "lowest note that an instrument can play or singer can sing",
        "data_type": "WikibaseItem"
    },
    "P8236": {
        "label": "LiederNet song cycle ID",
        "description": "song cycle identifier used by the LiederNet Archive",
        "data_type": "ExternalId"
    },
    "P6250": {
        "label": "iTunes music movie ID",
        "description": "identifier for a concert film or other music-related film in iTunes",
        "data_type": "ExternalId"
    },
    "P5798": {
        "label": "Thomson Reuters Business Classification",
        "description": "use for companies, industries, products",
        "data_type": "String"
    },
    "P8432": {
        "label": "Österreichisches Musiklexikon Online ID",
        "description": "identifier for entries in the Österreichisches Musiklexikon Online, published by Austrian Academy of Sciences",
        "data_type": "ExternalId"
    },
    "P3224": {
        "label": "NAICS code",
        "description": "classification in the North American Industry Classification System",
        "data_type": "ExternalId"
    },
    "P8228": {
        "label": "lighting",
        "description": "the way the photo got the light or the kind of natural light",
        "data_type": "WikibaseItem"
    },
    "P8692": {
        "label": "ACER code",
        "description": "code of energy market player by the Agency for the Cooperation of Energy Regulators",
        "data_type": "ExternalId"
    },
    "P8331": {
        "label": "strike tone",
        "description": "dominant note of bell",
        "data_type": "WikibaseItem"
    },
    "P8490": {
        "label": "ANZSIC 2006 ID",
        "description": "identifier of division/subdivision/group/class in the Australian and New Zealand Standard Industrial Classification 2006",
        "data_type": "ExternalId"
    },
    "P8535": {
        "label": "tala",
        "description": "musical meter for Indian music",
        "data_type": "WikibaseItem"
    },
    "P4104": {
        "label": "Carnegie Hall agent ID",
        "description": "identifier for a person or ensemble in the Carnegie Hall Linked Open Data (LOD)",
        "data_type": "ExternalId"
    },
    "P6670": {
        "label": "musical quotation or excerpt",
        "description": "quotation or excerpt from this musical work, in LilyPond syntax",
        "data_type": "MusicalNotation"
    },
    "P3242": {
        "label": "SIC code",
        "description": "U.S. Standard Industrial Classification number",
        "data_type": "ExternalId"
    },
    "P4496": {
        "label": "NACE code",
        "description": "code in the statistical classification of economic activities in the European Community (NACE)",
        "data_type": "String"
    },
    "P865": {
        "label": "BMLO ID",
        "description": "identifier of Bayerisches Musiker-Lexikon Online",
        "data_type": "ExternalId"
    },
    "P8516": {
        "label": "Library of Congress Medium of Performance Thesaurus ID",
        "description": "identifier for a term in the Library of Congress Medium of Performance Thesaurus for Music",
        "data_type": "ExternalId"
    },
    "P1069": {
        "label": "Statistics Denmarks classification of occupation (DISCO-08)",
        "description": "Statistics Denmarks classification of occupation, derived from ISCO",
        "data_type": "ExternalId"
    },
    "P1897": {
        "label": "highest note",
        "description": "highest note that an instrument can play or singer can sing",
        "data_type": "WikibaseItem"
    },
    "P1796": {
        "label": "International Standard Industrial Classification code Rev.4",
        "description": "code of industry by ISIC (International Standard Industrial Classification of All Economic Activities)",
        "data_type": "String"
    },
    "P7185": {
        "label": "eska.pl topic ID",
        "description": "identifier for an topic on eska.pl site",
        "data_type": "ExternalId"
    },
    "P5437": {
        "label": "EuroVoc ID",
        "description": "identifier for an area of government activity, in the European Union's EuroVoc thesaurus",
        "data_type": "ExternalId"
    },
    "P3610": {
        "label": "fare zone",
        "description": "fare zone that the station is in",
        "data_type": "WikibaseItem"
    },
    "P35": {
        "label": "head of state",
        "description": "official with the highest formal authority in a country/state",
        "data_type": "WikibaseItem"
    },
    "P1876": {
        "label": "vehicle",
        "description": "vessel involved in this mission, voyage or event",
        "data_type": "WikibaseItem"
    },
    "P8771": {
        "label": "Vélib' Métropole station ID",
        "description": "identifier of a Vélib' Métropole bicycle-sharing station located in Paris or in its suburbs",
        "data_type": "ExternalId"
    },
    "P6": {
        "label": "head of government",
        "description": "head of the executive power of this town, city, municipality, state, country, or other governmental body",
        "data_type": "WikibaseItem"
    },
    "P1029": {
        "label": "crew member",
        "description": "person that participated operating or serving aboard this vehicle",
        "data_type": "WikibaseItem"
    },
    "P1142": {
        "label": "political ideology",
        "description": "political ideology of this organization or person",
        "data_type": "WikibaseItem"
    },
    "P8753": {
        "label": "annual average daily traffic",
        "description": "daily traffic through a certain point, as yearly average",
        "data_type": "Quantity"
    },
    "P4243": {
        "label": "candidate number",
        "description": "number of the candidate in an election",
        "data_type": "String"
    },
    "P1192": {
        "label": "connecting service",
        "description": "service stopping at a station",
        "data_type": "WikibaseItem"
    },
    "P833": {
        "label": "interchange station",
        "description": "station to which passengers can transfer to from this station, normally without extra expense",
        "data_type": "WikibaseItem"
    },
    "P210": {
        "label": "party chief representative",
        "description": "chief representative of a party in an institution or an administrative unit (use qualifier to identify the party)",
        "data_type": "WikibaseItem"
    },
    "P3349": {
        "label": "designed to carry",
        "description": "what the vehicle or class of vehicles is or was designed to carry",
        "data_type": "WikibaseItem"
    },
    "P8453": {
        "label": "public transport stop",
        "description": "general (e.g. nearest) public transport stop (bus, tram, train, subway)",
        "data_type": "WikibaseItem"
    },
    "P3437": {
        "label": "people or cargo transported",
        "description": "the type of passengers or cargo a vehicle actually carries/carried",
        "data_type": "WikibaseItem"
    },
    "P1831": {
        "label": "electorate",
        "description": "number of persons qualified to vote during elections",
        "data_type": "Quantity"
    },
    "P2505": {
        "label": "carries",
        "description": "item (e.g. road, railway, canal) carried by a bridge, a tunnel or a mountain pass",
        "data_type": "WikibaseItem"
    },
    "P122": {
        "label": "basic form of government",
        "description": "subject's government",
        "data_type": "WikibaseItem"
    },
    "P5832": {
        "label": "political coalition",
        "description": "agreement for cooperation between different political parties",
        "data_type": "WikibaseItem"
    },
    "P4217": {
        "label": "UK Electoral Commission ID",
        "description": "identifier issued by the Electoral Commission in the UK to identify a political party",
        "data_type": "ExternalId"
    },
    "P2319": {
        "label": "elector",
        "description": "people or other entities which are qualified to participate in the subject election",
        "data_type": "WikibaseItem"
    },
    "P8196": {
        "label": "voting system",
        "description": "system used for this election or vote",
        "data_type": "WikibaseItem"
    },
    "P8594": {
        "label": "ride height",
        "description": "ground clearance of vehicle or distance between the base of a vehicle tire and the lowest point",
        "data_type": "Quantity"
    },
    "P1103": {
        "label": "number of platform tracks",
        "description": "number of tracks served by the platforms at a railway station",
        "data_type": "Quantity"
    },
    "P1906": {
        "label": "office held by head of state",
        "description": "political office that is fulfilled by the head of state of this item",
        "data_type": "WikibaseItem"
    },
    "P4906": {
        "label": "Bargeton ID",
        "description": "identifier of a French prefect on the ''Dictionnaire biographique des préfets'', said \"Bargeton\" (1870-1980)",
        "data_type": "ExternalId"
    },
    "P4724": {
        "label": "Maitron ID",
        "description": "identifier for a person on the online version of Le Maitron, a biographical dictionary of the French labour movement",
        "data_type": "ExternalId"
    },
    "P8328": {
        "label": "Democracy Index",
        "description": "Democracy Index rating of a country",
        "data_type": "Quantity"
    },
    "P674": {
        "label": "characters",
        "description": "characters which appear in this item (like plays, operas, operettas, books, comics, films, TV series, video games)",
        "data_type": "WikibaseItem"
    },
    "P4434": {
        "label": "LesBiographies.com ID",
        "description": "note on LesBiographies.com, a biographical dictionary edited by the Société générale de presse",
        "data_type": "ExternalId"
    },
    "P1387": {
        "label": "political alignment",
        "description": "political position within the political spectrum",
        "data_type": "WikibaseItem"
    },
    "P4100": {
        "label": "parliamentary group",
        "description": "parliamentary group which a member of a parliament belongs to",
        "data_type": "WikibaseItem"
    },
    "P4811": {
        "label": "BWSA ID",
        "description": "article in the biographical lexicon of socialism and labor unions in the Netherlands",
        "data_type": "ExternalId"
    },
    "P1373": {
        "label": "daily patronage",
        "description": "number of daily passengers, patrons or visitors in specified time period",
        "data_type": "Quantity"
    },
    "P4527": {
        "label": "UK Parliament thesaurus ID",
        "description": "identifier in the thesaurus of subject headings maintained by the librarians in the House of Commons in the UK",
        "data_type": "ExternalId"
    },
    "P945": {
        "label": "allegiance",
        "description": "country (or other power) that the person or organization serves",
        "data_type": "WikibaseItem"
    },
    "P3872": {
        "label": "patronage",
        "description": "number of passengers, patrons or visitors in specified time period",
        "data_type": "Quantity"
    },
    "P4519": {
        "label": "payload mass",
        "description": "amount of mass that can be transported by the vehicle",
        "data_type": "Quantity"
    },
    "P4253": {
        "label": "number of constituencies",
        "description": "number of constituencies related to a legislative body",
        "data_type": "Quantity"
    },
    "P2802": {
        "label": "fleet or registration number",
        "description": "the fleet number or registration of the vehicle",
        "data_type": "String"
    },
    "P3858": {
        "label": "route diagram",
        "description": "Wikimedia route diagram template",
        "data_type": "WikibaseItem"
    },
    "P8001": {
        "label": "endorsed by",
        "description": "public support given by a person or organization to a political campaign",
        "data_type": "WikibaseItem"
    },
    "P6414": {
        "label": "DIZIE ID",
        "description": "identifier for a topic on the online version of the ''Dizionario dell'integrazione europea''",
        "data_type": "ExternalId"
    },
    "P8181": {
        "label": "SNCF Station Identifier",
        "description": "identifier for train stations used by SNCF",
        "data_type": "ExternalId"
    },
    "P2595": {
        "label": "maximum gradient",
        "description": "maximum gradient on a route, expressed as a percentage",
        "data_type": "Quantity"
    },
    "P2790": {
        "label": "net tonnage",
        "description": "dimensionless index calculated from the total moulded volume of the ship's cargo spaces by using a mathematical formula",
        "data_type": "Quantity"
    },
    "P2979": {
        "label": "maritime identification digits",
        "description": "identifer used by radio communication facilities to identify their home country or base area in messages as part of their Maritime Mobile Service Identities",
        "data_type": "String"
    },
    "P1043": {
        "label": "IDEO Job ID",
        "description": "identifier for an occupation on IDEO",
        "data_type": "ExternalId"
    },
    "P1023": {
        "label": "SBC-2010 occupation code",
        "description": "Dutch classification of occupations SBC maintained by the Dutch CBS (Centraal Bureau voor de Statistiek), 2010 version",
        "data_type": "ExternalId"
    },
    "P289": {
        "label": "vessel class",
        "description": "series of vessels built to the same design of which this vessel is a member",
        "data_type": "WikibaseItem"
    },
    "P1624": {
        "label": "MarineTraffic Port ID",
        "description": "identifier for a port assigned by the MarineTraffic database",
        "data_type": "ExternalId"
    },
    "P2981": {
        "label": "UIC alphabetical country code",
        "description": "alphabetical country code issued by the International Union of Railways",
        "data_type": "ExternalId"
    },
    "P2982": {
        "label": "UIC numerical country code",
        "description": "numerical country code issued by International Union of Railways",
        "data_type": "ExternalId"
    },
    "P3994": {
        "label": "racon signal",
        "description": "signal that a radar beacon responds in morse code, when triggered by radar",
        "data_type": "String"
    },
    "P425": {
        "label": "field of this occupation",
        "description": "activity corresponding to this occupation or profession (use only for occupations/professions - for people use Property:P101, for companies use P452)",
        "data_type": "WikibaseItem"
    },
    "P5606": {
        "label": "station class",
        "description": "class of a railway station, assigned by its operator; \"issued by\" is a mandatory qualifier.",
        "data_type": "WikibaseItem"
    },
    "P434": {
        "label": "MusicBrainz artist ID",
        "description": "identifier for an artist in the MusicBrainz open music encyclopedia",
        "data_type": "ExternalId"
    },
    "P436": {
        "label": "MusicBrainz release group ID",
        "description": "identifier for a release group per the MusicBrainz open music encyclopedia (album, single, etc.)",
        "data_type": "ExternalId"
    },
    "P8621": {
        "label": "turning radius",
        "description": "radius of smallest circle a particular automobile or similar vehicle can turn along",
        "data_type": "Quantity"
    },
    "P5658": {
        "label": "railway traffic side",
        "description": "indicates for a country or a railway line whether rail traffic usually runs on the left or right hand side",
        "data_type": "WikibaseItem"
    },
    "P1024": {
        "label": "SBFI occupation code",
        "description": "Swiss classification of occupations SBFI/SEFRI maintained by the Swiss SBFI/SEFRI",
        "data_type": "ExternalId"
    },
    "P5931": {
        "label": "bazakolejowa.pl railway line ID",
        "description": "ID of Polish railway line in database \"bazakolejowa.pl\"",
        "data_type": "ExternalId"
    },
    "P6856": {
        "label": "average speed",
        "description": "average speed of a transport line, service or network, or of a vehicle when used on such a line, service or network",
        "data_type": "Quantity"
    },
    "P6017": {
        "label": "CanalPlan AC place ID",
        "description": "identifier for a feature on or near a British canal, in the CanalPlan AC database",
        "data_type": "ExternalId"
    },
    "P1064": {
        "label": "track gauge",
        "description": "spacing of the rails on a railway track",
        "data_type": "WikibaseItem"
    },
    "P435": {
        "label": "MusicBrainz work ID",
        "description": "identifier for a work per the MusicBrainz open music encyclopedia",
        "data_type": "ExternalId"
    },
    "P8470": {
        "label": "order number",
        "description": "identifier given to rolling stock at the point of purchase, sale, or manufacturing",
        "data_type": "String"
    },
    "P1099": {
        "label": "number of masts",
        "description": "number of masts a ship has",
        "data_type": "Quantity"
    },
    "P532": {
        "label": "port of registry",
        "description": "ship's port of registry. This is generally painted on the ship's stern (for the \"home port\", see Property:P504)",
        "data_type": "WikibaseItem"
    },
    "P6708": {
        "label": "departure angle",
        "description": "angle of a ramp from which a vehicle can descend, use degree (Q28390) as unit",
        "data_type": "Quantity"
    },
    "P8898": {
        "label": "road name formatter",
        "description": "template from which “$1” can be automatically replaced by a bare road number to obtain the full name of a numbered road",
        "data_type": "Monolingualtext"
    },
    "P9135": {
        "label": "WreckSite ID",
        "description": "identifier of a wreck towards the site wrecksite.eu",
        "data_type": "ExternalId"
    },
    "P1021": {
        "label": "KldB-2010 occupation code",
        "description": "German classification of occupations 2010",
        "data_type": "ExternalId"
    },
    "P8764": {
        "label": "port calls",
        "description": "harbor or port at which the ship stopped; port of call",
        "data_type": "WikibaseItem"
    },
    "P2673": {
        "label": "next crossing upstream",
        "description": "next crossing of this river, canal, etc. upstream of this subject",
        "data_type": "WikibaseItem"
    },
    "P2674": {
        "label": "next crossing downstream",
        "description": "next crossing of this river, canal, etc. downstream of this subject",
        "data_type": "WikibaseItem"
    },
    "P5595": {
        "label": "number of platform faces",
        "description": "number of continuous platform edges facing a track or vehicle guideway in a station (often, but not always, the same as P1103 and the number of platform numbers). See property talk page for illustrated samples",
        "data_type": "Quantity"
    },
    "P9227": {
        "label": "Encyclopedia Titanica ID",
        "description": "Encyclopedia Titanica identifier for a passenger or crew member of the Titanic",
        "data_type": "ExternalId"
    },
    "P3086": {
        "label": "speed limit",
        "description": "maximum speed allowed on a transport route or in a certain area",
        "data_type": "Quantity"
    },
    "P8458": {
        "label": "ANZSCO 2013 ID",
        "description": "the Australian and New Zealand Standard Classification of Occupations, 2013, by the Australian Bureau of Statistics and Stats NZ",
        "data_type": "ExternalId"
    },
    "P2317": {
        "label": "call sign",
        "description": "short form identifier for a radio operator or broadcaster and (as flag signal) previous for watercrafts",
        "data_type": "String"
    },
    "P2978": {
        "label": "wheel arrangement",
        "description": "wheel/axle arrangement for locomotives, railcars and other rolling stock",
        "data_type": "WikibaseItem"
    },
    "P1093": {
        "label": "gross tonnage",
        "description": "unitless index related to a ship's overall internal volume",
        "data_type": "Quantity"
    },
    "P8180": {
        "label": "Indian Railways train number",
        "description": "identifier of a train service in a particular direction operated on the Indian rail network",
        "data_type": "ExternalId"
    },
    "P834": {
        "label": "train depot",
        "description": "depot which serves this railway line, not public stations",
        "data_type": "WikibaseItem"
    },
    "P8654": {
        "label": "SSYK 2012 The Swedish Standard Classification of Occupations",
        "description": "Swedish occupation code based on ISCO–08",
        "data_type": "ExternalId"
    },
    "P5330": {
        "label": "United Kingdom Department for Transport railway station category",
        "description": "A category for stations in the UK (more specifically Great Britain) applied by National Rail to every mainline station. See [[:w:en:United Kingdom railway station categories]]",
        "data_type": "WikibaseItem"
    },
    "P3019": {
        "label": "railway signalling system",
        "description": "type of signalling used on this railway line",
        "data_type": "WikibaseItem"
    },
    "P930": {
        "label": "type of electrification",
        "description": "electrification system scheme and/or voltage",
        "data_type": "WikibaseItem"
    },
    "P9277": {
        "label": "LFChistory.net player ID",
        "description": "identifier for football player on the LFChistory.net website",
        "data_type": "ExternalId"
    },
    "P8735": {
        "label": "Opta football competition ID",
        "description": "identifier of a football competition in Opta",
        "data_type": "ExternalId"
    },
    "P8737": {
        "label": "Opta football team ID",
        "description": "identifier of an association football team in Opta Sports",
        "data_type": "ExternalId"
    },
    "P3306": {
        "label": "ICAA rating",
        "description": "rating in the Spanish motion picture content rating system",
        "data_type": "WikibaseItem"
    },
    "P5152": {
        "label": "KAVI rating",
        "description": "rating in the Finnish film and television rating system",
        "data_type": "WikibaseItem"
    },
    "P5150": {
        "label": "IGAC rating",
        "description": "rating of a film or theatrical production in the Portuguese classification system",
        "data_type": "WikibaseItem"
    },
    "P8912": {
        "label": "Kicker.de player ID (actual scheme)",
        "description": "text identifier for an association football player at kicker.de",
        "data_type": "ExternalId"
    },
    "P1470": {
        "label": "maximum glide ratio",
        "description": "maximum glide ratio of an aircraft",
        "data_type": "Quantity"
    },
    "P1087": {
        "label": "Elo rating",
        "description": "quantitative measure of one's game-playing ability, particularly in classical chess",
        "data_type": "Quantity"
    },
    "P8889": {
        "label": "MTRCB rating",
        "description": "rating of a film or television program in the Philippines",
        "data_type": "WikibaseItem"
    },
    "P8779": {
        "label": "USL League One ID",
        "description": "identifier for soccer player profile on website of USL League One",
        "data_type": "ExternalId"
    },
    "P9264": {
        "label": "FFF player ID (new scheme)",
        "description": "identifier for a player on the French Football Federation website",
        "data_type": "ExternalId"
    },
    "P426": {
        "label": "aircraft registration",
        "description": "identifier assigned to an individual aircraft by civil aircraft registry",
        "data_type": "String"
    },
    "P7327": {
        "label": "OFLC classification",
        "description": "rating in the media content rating system of New Zealand",
        "data_type": "WikibaseItem"
    },
    "P9086": {
        "label": "BAMID film rating",
        "description": "rating in the Taiwanese movie content rating system",
        "data_type": "WikibaseItem"
    },
    "P916": {
        "label": "GSRR rating",
        "description": "Taiwanese video game content rating system",
        "data_type": "WikibaseItem"
    },
    "P9266": {
        "label": "Ligue 1 player ID",
        "description": "identifier for a football player on the Ligue 1 website",
        "data_type": "ExternalId"
    },
    "P1956": {
        "label": "takeoff and landing capability",
        "description": "type of takeoff and landing the aircraft is designed to operate",
        "data_type": "WikibaseItem"
    },
    "P3216": {
        "label": "ClassInd rating",
        "description": "rating of an audiovisual work, video game or RPG in the Brazilian Advisory Rating System (ClassInd)",
        "data_type": "WikibaseItem"
    },
    "P878": {
        "label": "avionics",
        "description": "notable sensors, electronics, and other avionics installed in the subject aircraft",
        "data_type": "WikibaseItem"
    },
    "P9028": {
        "label": "Consejo de Calificación Cinematográfica rating",
        "description": "video game content rating in Chile, according to the Decree 51",
        "data_type": "WikibaseItem"
    },
    "P8874": {
        "label": "Hong Kong film rating",
        "description": "Hong Kong film classification system",
        "data_type": "WikibaseItem"
    },
    "P8901": {
        "label": "ELSPA rating",
        "description": "rating of a video game in the British ELSPA rating system (1994 - Spring 2003)",
        "data_type": "WikibaseItem"
    },
    "P8652": {
        "label": "Netflix maturity rating",
        "description": "content rating used and determined by Netflix in countries where local ratings do not apply to streaming media",
        "data_type": "WikibaseItem"
    },
    "P852": {
        "label": "ESRB rating",
        "description": "North American video game content rating",
        "data_type": "WikibaseItem"
    },
    "P6452": {
        "label": "CBFC rating",
        "description": "rating in the Indian film rating system",
        "data_type": "WikibaseItem"
    },
    "P7573": {
        "label": "IFCO rating",
        "description": "rating certificate of a film in the Irish content classification system",
        "data_type": "WikibaseItem"
    },
    "P2857": {
        "label": "WWE.com superstar ID",
        "description": "identifier for a professional wrestling performer or tag team at WWE.com",
        "data_type": "ExternalId"
    },
    "P5201": {
        "label": "IMDA rating",
        "description": "rating in the Singaporean film and video game rating system",
        "data_type": "WikibaseItem"
    },
    "P914": {
        "label": "USK rating",
        "description": "German video game content rating - see talk page for appropriate values",
        "data_type": "WikibaseItem"
    },
    "P853": {
        "label": "CERO rating",
        "description": "Japanese video game rating system - see talk page for appropriate values",
        "data_type": "WikibaseItem"
    },
    "P908": {
        "label": "PEGI rating",
        "description": "European video game content rating system - see talk page for appropriate values",
        "data_type": "WikibaseItem"
    },
    "P7663": {
        "label": "Scienza a due voci ID",
        "description": "identifier for a female scientist in Italy in the project Scienza a due voci",
        "data_type": "ExternalId"
    },
    "P7267": {
        "label": "Guia dos Quadrinhos publishing house ID (Brazilian)",
        "description": "identifier for a Brazilian comic book publishing house",
        "data_type": "ExternalId"
    },
    "P7266": {
        "label": "Guia dos Quadrinhos comic ID (Brazilian)",
        "description": "identifier for a Brazilian comic book or graphic novel",
        "data_type": "ExternalId"
    },
    "P9030": {
        "label": "solution to",
        "description": "a mathematical object that satisfies the criteria for a mathematical problem",
        "data_type": "WikibaseItem"
    },
    "P3752": {
        "label": "worst-case time complexity",
        "description": "time complexity of an algorithm at most",
        "data_type": "Math"
    },
    "P6488": {
        "label": "Enciclopedia delle donne ID",
        "description": "identifier for a subject on the ''Enciclopedia delle donne''",
        "data_type": "ExternalId"
    },
    "P3753": {
        "label": "best-case time complexity",
        "description": "time complexity of an algorithm at least",
        "data_type": "Math"
    },
    "P3756": {
        "label": "best-case space complexity",
        "description": "space complexity of an algorithm at least",
        "data_type": "Math"
    },
    "P6731": {
        "label": "Commons quality assessment",
        "description": "assessment on Wikimedia Commons of the file: featured picture, valued image, quality images, picture of the day, picture of the year etc. Use on media entities only",
        "data_type": "WikibaseItem"
    },
    "P3754": {
        "label": "average time complexity",
        "description": "time complexity of an algorithm on average",
        "data_type": "Math"
    },
    "P3757": {
        "label": "average space complexity",
        "description": "space complexity of an algorithm on average",
        "data_type": "Math"
    },
    "P7269": {
        "label": "Guia dos Quadrinhos comic ID",
        "description": "identifier for a non-Brazilian comic book or graphic novel that was published in Brazil",
        "data_type": "ExternalId"
    },
    "P4632": {
        "label": "Bechdel Test Movie List ID",
        "description": "identifier for a film on bechdeltest.com",
        "data_type": "ExternalId"
    },
    "P2159": {
        "label": "computes solution to",
        "description": "problem that this algorithm or method solves",
        "data_type": "WikibaseItem"
    },
    "P8558": {
        "label": "Butcher tableau",
        "description": "table containing the coefficients of a Runge-Kutta method",
        "data_type": "Math"
    },
    "P4449": {
        "label": "Shirat Nashim person ID",
        "description": "identifier of a person in the Israeli female singers database, Shirat Nashim",
        "data_type": "ExternalId"
    },
    "P4479": {
        "label": "INDUCKS character ID",
        "description": "identifier for a character of the Disney Universe",
        "data_type": "ExternalId"
    },
    "P6637": {
        "label": "AWARE ID",
        "description": "identifier for a female artist on the Archives of Women Artists, Research and Exhibitions",
        "data_type": "ExternalId"
    },
    "P9310": {
        "label": "pHash checksum",
        "description": "perceptual hash for file on Commons, value which tells whether two images look nearly identical",
        "data_type": "String"
    },
    "P8644": {
        "label": "Lambiek comic magazines ID",
        "description": "identifier for an entry about comic magazines from Lambiek",
        "data_type": "ExternalId"
    },
    "P5665": {
        "label": "Cartooning for Peace cartoonist ID",
        "description": "identifier for a cartoonist on the Cartooning for Peace website",
        "data_type": "ExternalId"
    },
    "P7009": {
        "label": "extracted from",
        "description": "source this image was cropped from. Use on Wikimedia Commons only.",
        "data_type": "CommonsMedia"
    },
    "P2498": {
        "label": "Catalan Biographical Dictionary of Women ID",
        "description": "identifier in the biographical dictionary of Catalan women",
        "data_type": "ExternalId"
    },
    "P8264": {
        "label": "attribution text",
        "description": "the attribution string as specified by the licensor, that re-users are obliged to name",
        "data_type": "String"
    },
    "P2533": {
        "label": "WomenWriters ID",
        "description": "identifier in the Women Writers database of the Huygens Institute for Dutch History",
        "data_type": "ExternalId"
    },
    "P9410": {
        "label": "placeholder text element ID",
        "description": "unique identifier (id attribute) of the <text> or <tspan> element in the SVG file whose contents should be replaced by text described by the value of the claim",
        "data_type": "String"
    },
    "P7270": {
        "label": "Guia dos Quadrinhos publishing house ID",
        "description": "identifier for a non-Brazilian comic book publishing house that has its comics published in Brazil",
        "data_type": "ExternalId"
    },
    "P7384": {
        "label": "geograph.org.uk image ID",
        "description": "identifier for an image from Geograph Britain and Ireland",
        "data_type": "ExternalId"
    },
    "P9320": {
        "label": "ComiXology publisher ID",
        "description": "identifier for a publisher at ComiXology",
        "data_type": "ExternalId"
    },
    "P828": {
        "label": "has cause",
        "description": "underlying cause, thing that ultimately resulted in this effect",
        "data_type": "WikibaseItem"
    },
    "P7418": {
        "label": "image of frontside",
        "description": "image of obverse (front side) of this 2D object",
        "data_type": "CommonsMedia"
    },
    "P8715": {
        "label": "Lambiek Special pages ID",
        "description": "identifier for an entry about comics from Lambiek",
        "data_type": "ExternalId"
    },
    "P5905": {
        "label": "Comic Vine ID",
        "description": "identifier at the Comic Vine database of comic books, fictional characters, people, films and television series/episodes",
        "data_type": "ExternalId"
    },
    "P4963": {
        "label": "Svenskt kvinnobiografiskt lexikon ID",
        "description": "identifier for an article in the Biographical Dictionary of Swedish Women",
        "data_type": "ExternalId"
    },
    "P7417": {
        "label": "image of reverse",
        "description": "image of the secondary side of this \"2D\" object",
        "data_type": "CommonsMedia"
    },
    "P5621": {
        "label": "Marvel character ID",
        "description": "identifier for a character on the Marvel.com website",
        "data_type": "ExternalId"
    },
    "P1539": {
        "label": "female population",
        "description": "number of female people inhabiting the place; number of female people of subject",
        "data_type": "Quantity"
    },
    "P1615": {
        "label": "CLARA-ID",
        "description": "identifier from database about women visual artists",
        "data_type": "ExternalId"
    },
    "P3755": {
        "label": "worst-case space complexity",
        "description": "space complexity of an algorithm at most",
        "data_type": "Math"
    },
    "P8048": {
        "label": "Women of Scotland memorial ID",
        "description": "The Women of Scotland project identifier for a memorial to a woman in Scotland",
        "data_type": "ExternalId"
    },
    "P9353": {
        "label": "does not have cause",
        "description": "expected cause, assumed cause or usual cause absent here",
        "data_type": "WikibaseItem"
    },
    "P110": {
        "label": "illustrator",
        "description": "person drawing the pictures in a book",
        "data_type": "WikibaseItem"
    },
    "P9361": {
        "label": "external georeferencing URL",
        "description": "URL where external georeferencing is available for a map",
        "data_type": "Url"
    },
    "P3612": {
        "label": "Le Monde diplomatique subject ID",
        "description": "identifier for a topic in the French newspaper Le Monde diplomatique",
        "data_type": "ExternalId"
    },
    "P3076": {
        "label": "Open Beauty Facts category ID",
        "description": "category on the Open Beauty Facts website",
        "data_type": "ExternalId"
    },
    "P2026": {
        "label": "Avibase ID",
        "description": "identifier for a species, subspecies, or genus in Avibase - the world bird database",
        "data_type": "ExternalId"
    },
    "P3277": {
        "label": "KANTL member ID",
        "description": "identifier for a member of the Royal Academy of Dutch language and literature",
        "data_type": "ExternalId"
    },
    "P3304": {
        "label": "NGS pumping station ID",
        "description": "identifier for a pumping station described on the website of the Nederlandse Gemalen Stichting",
        "data_type": "ExternalId"
    },
    "P1816": {
        "label": "National Portrait Gallery (London) person ID",
        "description": "identifier for sitters and artists represented in the National Portrait Gallery, London",
        "data_type": "ExternalId"
    },
    "P3067": {
        "label": "GS1 country code",
        "description": "GS1 Prefix, the first three digits, usually identifying the national GS1 Member Organization to which the manufacturer is registered (not necessarily where the product is actually made)",
        "data_type": "String"
    },
    "P3055": {
        "label": "NAQ elected person ID",
        "description": "official identifier of a person elected to the National Assembly of Quebec (NAQ) or one of its predecessors",
        "data_type": "ExternalId"
    },
    "P3083": {
        "label": "SIMBAD ID",
        "description": "identifier for an astronomical object, in the University of Strasbourg's SIMBAD database",
        "data_type": "ExternalId"
    },
    "P2944": {
        "label": "Plarr ID",
        "description": "identifier of a person, in the biographical website of the Royal College of Surgeons, London",
        "data_type": "ExternalId"
    },
    "P3060": {
        "label": "ButMoth ID",
        "description": "identifier for a butterfly or moth genus in the UK Natural History Museum's 'Butterflies and Moths of the World' database",
        "data_type": "ExternalId"
    },
    "P1800": {
        "label": "Wikimedia database name",
        "description": "identifier for a Wikimedia project database",
        "data_type": "ExternalId"
    },
    "P2815": {
        "label": "ESR station code",
        "description": "unique railway station/operation point code used in USSR and later in former USSR countries",
        "data_type": "ExternalId"
    },
    "P3102": {
        "label": "Plantarium ID",
        "description": "identifier for a plant taxon, in the Plantarium database",
        "data_type": "ExternalId"
    },
    "P1820": {
        "label": "Open Food Facts food additive ID",
        "description": "represents a food additive on Open Food Facts",
        "data_type": "ExternalId"
    },
    "P2986": {
        "label": "aircraft registration prefix",
        "description": "national prefix used in civil aircraft registration numbers",
        "data_type": "ExternalId"
    },
    "P3099": {
        "label": "Internet Bird Collection species ID",
        "description": "identifier for a species of bird, in the Internet Bird Collection database",
        "data_type": "ExternalId"
    },
    "P3130": {
        "label": "NSW Flora ID",
        "description": "identifier for a plant taxon, in the NSW Flora Online",
        "data_type": "ExternalId"
    },
    "P3100": {
        "label": "Flora of Australia ID (old)",
        "description": "identifier for a plant taxon, in the Flora of Australia Online",
        "data_type": "ExternalId"
    },
    "P2741": {
        "label": "Tate artist ID",
        "description": "identifier of an artist in the Tate collection",
        "data_type": "ExternalId"
    },
    "P3064": {
        "label": "LepIndex ID",
        "description": "identifier for a Lepidoptera taxon, in the UK Natural History Museum's 'Global Lepidoptera Names Index'",
        "data_type": "ExternalId"
    },
    "P2108": {
        "label": "Kunstindeks Danmark artwork ID",
        "description": "identifier for artwork in the Danish art database",
        "data_type": "ExternalId"
    },
    "P3208": {
        "label": "New General Catalogue ID",
        "description": "identifier for an astronomical object, on the New General Catalogue of Nebulae and Clusters of Stars, or its supplements",
        "data_type": "ExternalId"
    },
    "P3160": {
        "label": "Minnesota legislator ID",
        "description": "official identifier for a Minnesota legislator",
        "data_type": "ExternalId"
    },
    "P3213": {
        "label": "Indian census area code (2001)",
        "description": "code used for an area for calculations in the 2001 census of India",
        "data_type": "ExternalId"
    },
    "P1887": {
        "label": "vice-county",
        "description": "geographical delineation utilized for biological records in the British Isles",
        "data_type": "WikibaseItem"
    },
    "P2171": {
        "label": "TheyWorkForYou ID",
        "description": "identifier in the 'TheyWorkForYou' database of British MPs",
        "data_type": "ExternalId"
    },
    "P3288": {
        "label": "World Spider Catalog ID",
        "description": "identifier of a taxon in the World Spider Catalog",
        "data_type": "ExternalId"
    },
    "P3229": {
        "label": "South Dakota legislator ID",
        "description": "official identifier for a South Dakota legislator",
        "data_type": "ExternalId"
    },
    "P3404": {
        "label": "The Vogue List ID",
        "description": "identifier for an actor of the fashion scene in the database maintained by Vogue Paris",
        "data_type": "ExternalId"
    },
    "P3466": {
        "label": "Israeli CBS municipal ID",
        "description": "The Israeli municipal code (Hebrew סמל) is an outline encoding used by the Israel Central Bureau of Statistics to finely fine-tune their statistics. It's a unique identifier given to each municipality in Israel.",
        "data_type": "ExternalId"
    },
    "P3468": {
        "label": "National Inventors Hall of Fame ID",
        "description": "identifier for an inductee in the United States National Inventors Hall of Fame",
        "data_type": "ExternalId"
    },
    "P3591": {
        "label": "WCSPF ID",
        "description": "identifier of a plant taxon, in the World Checklist of Selected Plant Families",
        "data_type": "ExternalId"
    },
    "P3101": {
        "label": "FloraBase ID",
        "description": "identifier for a plant taxon, in the Government of Western Australia's FloraBase database",
        "data_type": "ExternalId"
    },
    "P3104": {
        "label": "Gares & Connexions ID",
        "description": "ID for a railway station on the official website for French railway stations",
        "data_type": "ExternalId"
    },
    "P3209": {
        "label": "Heritage Railway Station of Canada ID",
        "description": "identifier of the Heritage Railway Station designated by the Historic Sites and Monuments Board of Canada",
        "data_type": "ExternalId"
    },
    "P2833": {
        "label": "ARKive ID",
        "description": "identifier for a taxon, in the ARKive database",
        "data_type": "ExternalId"
    },
    "P3322": {
        "label": "Vlinderstichting-ID",
        "description": "identifier for lepidoptera species in the vlinderstichting database",
        "data_type": "ExternalId"
    },
    "P3211": {
        "label": "Heritage Lighthouse of Canada ID",
        "description": "identifier of the Heritage Lighthouse designated by the Historic Sites and Monuments Board of Canada",
        "data_type": "ExternalId"
    },
    "P2980": {
        "label": "ARLHS lighthouse ID",
        "description": "identifier on the ARLHS World List of Lights",
        "data_type": "ExternalId"
    },
    "P3031": {
        "label": "EPPO Code",
        "description": "identifier for a taxon in the EPPO Global Database",
        "data_type": "ExternalId"
    },
    "P3054": {
        "label": "Ontario MPP ID",
        "description": "official identifier for a Member of Provincial Parliament of Ontario",
        "data_type": "ExternalId"
    },
    "P2589": {
        "label": "Statistics Indonesia ethnicity code",
        "description": "ethnicity code in Indonesia by Statistics Indonesia",
        "data_type": "ExternalId"
    },
    "P8416": {
        "label": "Dowker-Thistlethwaite name",
        "description": "unambiguous systematic identifier scheme for mathematical knots",
        "data_type": "Math"
    },
    "P5350": {
        "label": "Alexander polynomial",
        "description": "invariant of a knot or link. Use 't' as variable and list monomials in decreasing order.",
        "data_type": "Math"
    },
    "P8418": {
        "label": "Oberwolfach mathematician ID",
        "description": "identifier for a mathematician in the Oberwolfach Photo Collection of Photographs",
        "data_type": "ExternalId"
    },
    "P2695": {
        "label": "type locality (geology)",
        "description": "the locality where a particular rock type, stratigraphic unit or mineral species is defined from (can coincide but not the same as p189)",
        "data_type": "WikibaseItem"
    },
    "P9235": {
        "label": "seismic classification",
        "description": "seismic risk zone which the administrative entity that receives the seismic classification is in",
        "data_type": "WikibaseItem"
    },
    "P1336": {
        "label": "territory claimed by",
        "description": "administrative divisions that claim control of a given area",
        "data_type": "WikibaseItem"
    },
    "P8426": {
        "label": "Knot Atlas identifier",
        "description": "link from mathematical knot/link articles to authoritative information source Knot Atlas",
        "data_type": "ExternalId"
    },
    "P8864": {
        "label": "identity element",
        "description": "value of the identity element of the mathematical operation",
        "data_type": "WikibaseItem"
    },
    "P1304": {
        "label": "central bank",
        "description": "country's central bank",
        "data_type": "WikibaseItem"
    },
    "P6830": {
        "label": "swMATH work ID",
        "description": "identifier of information service for mathematical software",
        "data_type": "ExternalId"
    },
    "P5351": {
        "label": "Conway polynomial",
        "description": "invariant of a knot. Use z as variable and list monomials in decreasing order.",
        "data_type": "Math"
    },
    "P8489": {
        "label": "SmallGroup ID",
        "description": "identifier of a specific small group, in the SmallGrp library and Magma",
        "data_type": "ExternalId"
    },
    "P2098": {
        "label": "substitute/deputy/replacement of office/officeholder",
        "description": "function that serves as deputy/replacement of this function/office (scope/conditions vary depending on office)",
        "data_type": "WikibaseItem"
    },
    "P5761": {
        "label": "SAC ID",
        "description": "identifier for a hut, a summit, etc. on the Swiss Alpine Club website",
        "data_type": "ExternalId"
    },
    "P8378": {
        "label": "Dowker-Thistlethwaite notation",
        "description": "descriptive property of mathematical knots, also known as Dowker notation",
        "data_type": "Math"
    },
    "P733": {
        "label": "DINOloket",
        "description": "identifier from database of geologic units in the Netherlands (Data Informatie Nederlandse Ondergrond)",
        "data_type": "ExternalId"
    },
    "P6438": {
        "label": "Euler characteristic",
        "description": "topological invariant of a space; the alternating sum of the dimensions of the (co)homology of a space",
        "data_type": "Quantity"
    },
    "P2659": {
        "label": "topographic isolation",
        "description": "minimum distance to a point of higher  elevation",
        "data_type": "Quantity"
    },
    "P568": {
        "label": "overlies",
        "description": "stratigraphic unit that this unit lies over (i.e. the underlying unit)",
        "data_type": "WikibaseItem"
    },
    "P5236": {
        "label": "prime factor",
        "description": "one of the prime numbers that can be multiplied to give this number",
        "data_type": "WikibaseItem"
    },
    "P208": {
        "label": "executive body",
        "description": "branch of government for the daily administration of the state",
        "data_type": "WikibaseItem"
    },
    "P514": {
        "label": "interleaves with",
        "description": "stratigraphic relation in which two units overlap each other marginally",
        "data_type": "WikibaseItem"
    },
    "P5867": {
        "label": "Mathematical Reviews journal ID",
        "description": "number of a journal in Mathematical Reviews website",
        "data_type": "ExternalId"
    },
    "P194": {
        "label": "legislative body",
        "description": "legislative body governing this entity; political institution with elected representatives, such as a parliament/legislature or council",
        "data_type": "WikibaseItem"
    },
    "P8427": {
        "label": "Knotinfo ID",
        "description": "identifier used to link to the KnotInfo website entry for a knot",
        "data_type": "ExternalId"
    },
    "P6432": {
        "label": "Alexander–Briggs notation",
        "description": "common notation of abstract knots and links",
        "data_type": "Math"
    },
    "P8865": {
        "label": "mathematical inverse",
        "description": "inverse element with respect to binary operation given as a qualifier",
        "data_type": "WikibaseItem"
    },
    "P3815": {
        "label": "volcano observatory",
        "description": "institution that monitors this volcanic landform or phenomenon",
        "data_type": "WikibaseItem"
    },
    "P5352": {
        "label": "Jones polynomial",
        "description": "invariant of a knot or link. Use q as variable and list monomials in decreasing order.",
        "data_type": "Math"
    },
    "P8866": {
        "label": "has operator",
        "description": "mathematical operator associated with this algebraic structure",
        "data_type": "WikibaseItem"
    },
    "P567": {
        "label": "underlies",
        "description": "stratigraphic unit that this unit lies under (i.e. the overlying unit)",
        "data_type": "WikibaseItem"
    },
    "P2327": {
        "label": "ProCyclingStats race ID",
        "data_type": "ExternalId"
    },
    "P2660": {
        "label": "topographic prominence",
        "description": "height of a mountain or hill relative to the lowest contour line encircling it (on Earth, maximum 8,848 m)",
        "data_type": "Quantity"
    },
    "P7419": {
        "label": "gender educated",
        "description": "genders educated at this educational institution",
        "data_type": "WikibaseItem"
    },
    "P8480": {
        "label": "Ofsted inspection rating",
        "description": "grade for a school inspection by Ofsted",
        "data_type": "WikibaseItem"
    },
    "P5005": {
        "label": "National Thesis Number (France)",
        "description": "identifier usually assigned by the dissertation's library for a published thesis in France",
        "data_type": "ExternalId"
    },
    "P7095": {
        "label": "end grade",
        "description": "highest educational year (aka grade) taught by this institution",
        "data_type": "WikibaseItem"
    },
    "P5353": {
        "label": "school district",
        "description": "school district to which a settlement or other item belongs",
        "data_type": "WikibaseItem"
    },
    "P6498": {
        "label": "illiterate population",
        "description": "number of illiterate people within a territorial entity (typically determined during a census)",
        "data_type": "Quantity"
    },
    "P6095": {
        "label": "INEP IGC discrete grade",
        "data_type": "WikibaseItem"
    },
    "P4930": {
        "label": "Julien ID",
        "description": "identifier of a person on the biographical dictionary of the Lycée Clemenceau alumni, said \"Julien\"",
        "data_type": "ExternalId"
    },
    "P8263": {
        "label": "curriculum topics",
        "description": "topic or subject included in the curriculum",
        "data_type": "WikibaseItem"
    },
    "P3991": {
        "label": "Austrian Textbook ID",
        "description": "This identifier is used for textbooks in Austria that are allowed to be used in public schools.",
        "data_type": "ExternalId"
    },
    "P6069": {
        "label": "INEP IGC continuous score",
        "data_type": "Quantity"
    },
    "P8186": {
        "label": "DPE school code",
        "description": "identifier for an educational institution issued by Bangladesh's Directorate of Primary Education",
        "data_type": "ExternalId"
    },
    "P3271": {
        "label": "compulsory education (maximum age)",
        "description": "maximum age for children when compulsory education ends. Qualifiers: \"subject of\" to link to more detail. If applicable, use qualifier P31 with value \"varies by state/province\" (Q27145860). Use \"start time\" or \"point in time\" for historic data.",
        "data_type": "Quantity"
    },
    "P6897": {
        "label": "literacy rate",
        "description": "percentage of the population that is not illiterate",
        "data_type": "Quantity"
    },
    "P3270": {
        "label": "compulsory education (minimum age)",
        "description": "minimum age for children when compulsory education starts. Qualifiers: \"subject of\" to link to more detail. If applicable, use qualifier P31 with value \"varies by state/province\" (Q27145860). Use \"start time\" or \"point in time\" for historic data.",
        "data_type": "Quantity"
    },
    "P7374": {
        "label": "educational stage",
        "description": "educational stage of a specific school; qualifier of \"educated at\" (P69): the education stage of someone's attendance of school",
        "data_type": "WikibaseItem"
    },
    "P8481": {
        "label": "Described and Captioned Media Program producer ID",
        "description": "identifier code from Described and Captioned Media Program, a program fully funded by the United States Department of Education which provides resources for students with disabilities",
        "data_type": "ExternalId"
    },
    "P1075": {
        "label": "rector",
        "description": "rector of a university",
        "data_type": "WikibaseItem"
    },
    "P7086": {
        "label": "start grade",
        "description": "lowest educational year (aka grade) taught by this institution",
        "data_type": "WikibaseItem"
    },
    "P8495": {
        "label": "South Africa EMIS code",
        "description": "code of a South African institution in the Education Management Information System",
        "data_type": "ExternalId"
    },
    "P2573": {
        "label": "number of out-of-school children",
        "description": "number of out-of-school children reported for a place",
        "data_type": "Quantity"
    },
    "P5894": {
        "label": "tuition fee",
        "description": "the tuition fee at an educational instition (default duration:1 year, use duration qualifier to specify)",
        "data_type": "Quantity"
    },
    "P6499": {
        "label": "literate population",
        "description": "number of literate people within a territorial entity (typically determined during a census)",
        "data_type": "Quantity"
    },
    "P2027": {
        "label": "Colour Index International constitution ID",
        "description": "identifier issued by Colour Index International, for manufactured colour products",
        "data_type": "ExternalId"
    },
    "P9145": {
        "label": "Xinjiang Data Project identifier",
        "description": "identifier for internment camps or cultural sites in Xinjiang, China within the ASPI Xinjiang Data Project database",
        "data_type": "ExternalId"
    },
    "P2065": {
        "label": "NIAID ChemDB ID",
        "description": "ID in NIAID ChemDB",
        "data_type": "ExternalId"
    },
    "P9327": {
        "label": "Cartigli di Bologna ID",
        "description": "identifier for a building, park, garden, or canal in Bologna on the Cartigli di Bologna website",
        "data_type": "ExternalId"
    },
    "P8661": {
        "label": "Lower Sorbian place name ID",
        "description": "identifier used in the database for Lower Sorbian place names on dolnoserbski.de maintained by the Sorbian institute",
        "data_type": "ExternalId"
    },
    "P2062": {
        "label": "HSDB ID",
        "description": "ID in Hazardous Substances Data Bank",
        "data_type": "ExternalId"
    },
    "P3890": {
        "label": "MetaboLights Compound ID",
        "description": "identifiers for compound entries in the MetaboLights databases.",
        "data_type": "ExternalId"
    },
    "P1931": {
        "label": "NIOSH Pocket Guide ID",
        "description": "Identifier for a chemical in the NIOSH Pocket Guide to Chemical Hazards",
        "data_type": "String"
    },
    "P2840": {
        "label": "NSC number",
        "description": "numeric identifier for substances submitted to the National Cancer Institute (NCI) for testing and evaluation",
        "data_type": "ExternalId"
    },
    "P8502": {
        "label": "SÚKL code",
        "description": "identifier for substances assigned by the Czech State Institute for Drug Control",
        "data_type": "ExternalId"
    },
    "P235": {
        "label": "InChIKey",
        "description": "A hashed version of the full standard InChI - designed to create an identifier that encodes structural information and can also be practically used in web searching.",
        "data_type": "ExternalId"
    },
    "P234": {
        "label": "InChI",
        "description": "International Chemical Identifier",
        "data_type": "ExternalId"
    },
    "P3717": {
        "label": "MEROPS enzyme ID",
        "description": "ID for a proteolytic enzyme in MEROPS database",
        "data_type": "ExternalId"
    },
    "P9170": {
        "label": "Taiwan River Code",
        "description": "official code for the administrative divisions in Taiwan, and managed by the Water Resources Agency (MOEA)",
        "data_type": "ExternalId"
    },
    "P9330": {
        "label": "Washington Place Names ID",
        "description": "numerical identifier for a place in Washington Place Names",
        "data_type": "ExternalId"
    },
    "P695": {
        "label": "UN number",
        "description": "four-digit numbers that identify hazardous substances, and articles (such as explosives, flammable liquids, toxic substances, etc.) in the framework of international transport",
        "data_type": "ExternalId"
    },
    "P3640": {
        "label": "National Drug Code",
        "description": "A pharmaceutical code issued by the Food and Drug Administration for every drug product (formulation) on the U.S. market. Includes a labeler code, product code and package code, unique for every drug product.",
        "data_type": "ExternalId"
    },
    "P9314": {
        "label": "Latin Place Names ID",
        "description": "identifier for the Latin name of a place in the database Latin Place Names",
        "data_type": "ExternalId"
    },
    "P2086": {
        "label": "CDD Public ID",
        "description": "identifier for a chemical compound in the Collaborative Drug Discovery database",
        "data_type": "ExternalId"
    },
    "P2057": {
        "label": "Human Metabolome Database ID",
        "description": "ID in Human Metabolome Database",
        "data_type": "ExternalId"
    },
    "P2085": {
        "label": "Nikkaji ID",
        "description": "identifier for a chemical compound in the Japan Chemical Substance Dictionary (Nikkaji)",
        "data_type": "ExternalId"
    },
    "P683": {
        "label": "ChEBI ID",
        "description": "identifier from database and ontology of molecular entities focused on 'small' chemical compounds",
        "data_type": "ExternalId"
    },
    "P267": {
        "label": "ATC code",
        "description": "therapeutic chemical identification code per ATC",
        "data_type": "ExternalId"
    },
    "P2115": {
        "label": "NDF-RT ID",
        "description": "identifier for the National Drug File Reference Terminology",
        "data_type": "ExternalId"
    },
    "P111": {
        "label": "measured physical quantity",
        "description": "value of a physical property expressed as number multiplied by a unit",
        "data_type": "WikibaseItem"
    },
    "P8828": {
        "label": "AAA attraction ID",
        "description": "identifier for a tourist attraction on the American Automobile Association website",
        "data_type": "ExternalId"
    },
    "P2877": {
        "label": "SureChEMBL ID",
        "description": "the chemical compound identifier for the EBI SureChEMBL 'chemical compounds in patents' database",
        "data_type": "ExternalId"
    },
    "P2072": {
        "label": "CDB Chemical ID",
        "description": "ID in UCI chemical database",
        "data_type": "ExternalId"
    },
    "P2083": {
        "label": "Leadscope ID",
        "description": "identifier for a chemical compound in leadscope.com",
        "data_type": "ExternalId"
    },
    "P8710": {
        "label": "Dresden Street ID",
        "description": "official identifier for streets in Dresden, Saxony, Germany",
        "data_type": "ExternalId"
    },
    "P2063": {
        "label": "LIPID MAPS ID",
        "description": "identifier used in the LIPID MAPS database for lipids",
        "data_type": "ExternalId"
    },
    "P5926": {
        "label": "Spectral Database for Organic Compounds ID",
        "data_type": "ExternalId"
    },
    "P1122": {
        "label": "spin quantum number",
        "description": "a quantum number; a dimensionless rational number;  the intrinsic spin angular momentum is this times the Dirac or reduced Planck constant",
        "data_type": "Quantity"
    },
    "P3117": {
        "label": "DSSTox substance ID",
        "description": "DSSTox substance identifier (DTXSID) used in the Environmental Protection Agency CompTox Dashboard",
        "data_type": "ExternalId"
    },
    "P9381": {
        "label": "Enslaved place ID",
        "description": "alphanumerical identifier for a place on the Enslaved website",
        "data_type": "ExternalId"
    },
    "P1978": {
        "label": "USDA NDB number",
        "description": "identifier for a food item in the United States Department of Agriculture National Nutrient Database",
        "data_type": "ExternalId"
    },
    "P9213": {
        "label": "Swedish Literature Bank place ID",
        "description": "identifier for a location in bookd at the Swedish Literature Bank",
        "data_type": "ExternalId"
    },
    "P3519": {
        "label": "Pfam ID",
        "description": "identifier in the Pfam database of protein families",
        "data_type": "ExternalId"
    },
    "P4393": {
        "label": "Anvisa drug ID",
        "description": "identifier provided by Anvisa, a regulatory body of the Brazilian government responsible for the regulation and approval of pharmaceutical drugs, sanitary standards and regulation of the food industry",
        "data_type": "ExternalId"
    },
    "P8691": {
        "label": "SwissLipids identifier",
        "description": "identifier for lipids used by the SwissLipids database",
        "data_type": "ExternalId"
    },
    "P8741": {
        "label": "Great Places ID",
        "data_type": "ExternalId"
    },
    "P8508": {
        "label": "ChemSynthesis ID",
        "description": "identifier of a chemical compound in ChemSynthesis",
        "data_type": "ExternalId"
    },
    "P2646": {
        "label": "mirTarBase ID",
        "description": "identifier for the mirTarBase database, a database for microRNAs and their targets",
        "data_type": "ExternalId"
    },
    "P231": {
        "label": "CAS Registry Number",
        "description": "identifier for a chemical substance or compound per Chemical Abstract Service's Registry database",
        "data_type": "ExternalId"
    },
    "P9018": {
        "label": "Medieval Manuscripts in Oxford Libraries place ID",
        "description": "identifier of a place in the online database Medieval Manuscripts in Oxford Libraries",
        "data_type": "ExternalId"
    },
    "P2064": {
        "label": "KNApSAcK ID",
        "description": "identifier in the KNApSAcK Core System",
        "data_type": "ExternalId"
    },
    "P8714": {
        "label": "GADM ID",
        "description": "identifier in the Global Administrative Area database",
        "data_type": "ExternalId"
    },
    "P9200": {
        "label": "English Placenames MADS ID",
        "description": "identifier for a place in the Digital Exposure of English Place-names dataset",
        "data_type": "ExternalId"
    },
    "P3637": {
        "label": "European Medicines Agency product number",
        "description": "identifier issued by the European Medicines Agency for treatments approved in the European Union",
        "data_type": "ExternalId"
    },
    "P9383": {
        "label": "The Mountaineers Routes & Places ID",
        "description": "identifier for a place for outdoor recreation on The Mountaineers Routes & Places website",
        "data_type": "ExternalId"
    },
    "P1578": {
        "label": "Gmelin number",
        "description": "identifier for inorganic chemical compounds, in the Gmelin database",
        "data_type": "ExternalId"
    },
    "P3978": {
        "label": "IECIC 2015 ID",
        "description": "Inventory of Existing Cosmetic Ingredients in China (2015)",
        "data_type": "ExternalId"
    },
    "P795": {
        "label": "located on linear feature",
        "description": "linear feature along which distance is specified from a specified datum point",
        "data_type": "WikibaseItem"
    },
    "P8988": {
        "label": "Mapy.cz ID",
        "description": "identifier for a point of interest in Mapy.cz",
        "data_type": "ExternalId"
    },
    "P232": {
        "label": "EC number",
        "description": "identifier for a chemical compound per EINECS or ELINCS",
        "data_type": "ExternalId"
    },
    "P3550": {
        "label": "Australian Register of Therapeutic Goods ID",
        "description": "identifier of a medicine or medical device listed in the Australian Register of Therapeutic Goods",
        "data_type": "ExternalId"
    },
    "P8867": {
        "label": "AAA campground ID",
        "description": "identifier for a campground on the American Automobile Association website",
        "data_type": "ExternalId"
    },
    "P9370": {
        "label": "Place Names NI ID",
        "description": "identifier for a place in Place Names NI",
        "data_type": "ExternalId"
    },
    "P8755": {
        "label": "VA facility ID",
        "description": "identifier for a facility on the U.S. Department of Veteran Affairs website",
        "data_type": "ExternalId"
    },
    "P9293": {
        "label": "AIATSIS Place Thesaurus ID",
        "description": "identifier for a place in the AIATSIS Place Thesaurus",
        "data_type": "ExternalId"
    },
    "P657": {
        "label": "RTECS number",
        "description": "chemical identifier from the Registry of Toxic Effects of Chemical Substances",
        "data_type": "ExternalId"
    },
    "P2566": {
        "label": "ECHA InfoCard ID",
        "description": "identifier of a chemical substance used by the European Chemicals Agency (ECHA)",
        "data_type": "ExternalId"
    },
    "P3350": {
        "label": "WHO international non-proprietary names ID",
        "description": "a numeric ID for the WHO international non-proprietary drug names collection",
        "data_type": "ExternalId"
    },
    "P3073": {
        "label": "CosIng number",
        "description": "Number for a chemical substance in the European CosIng database",
        "data_type": "ExternalId"
    },
    "P9417": {
        "label": "Camptocamp ID",
        "description": "numerical identifier for a waypoint on the Camptocamp.org website",
        "data_type": "ExternalId"
    },
    "P9221": {
        "label": "Danmarks Adresseregister named street ID",
        "description": "identifier for a street or square in Danish Address Register (Danmarks Adresseregister), entity type ''named street'' (''navngiven vej'').",
        "data_type": "ExternalId"
    },
    "P2017": {
        "label": "isomeric SMILES",
        "description": "dedicated SMILES for isomer",
        "data_type": "String"
    },
    "P4567": {
        "label": "BTO five-letter code",
        "description": "identifier for a bird species, issued by the British Trust for Ornithology (BTO)",
        "data_type": "ExternalId"
    },
    "P5010": {
        "label": "Territographie ID",
        "description": "identifier for a rural chapel in the Provence-Alpes-Côte-d'Azur region on the Territography platform developed by the CNRS",
        "data_type": "ExternalId"
    },
    "P4432": {
        "label": "AKL Online artist ID",
        "description": "artist identifier in the Artists of the World Online database",
        "data_type": "ExternalId"
    },
    "P4569": {
        "label": "BPH journal ID",
        "description": "identifier for a botanical journal in the Hunt Institute's 'Botanico Periodicum Huntianum'",
        "data_type": "ExternalId"
    },
    "P4641": {
        "label": "Welsh Chapels ID",
        "description": "identifier for a chapel building in Wales, in the Welsh Religious Buildings Trust chapels database",
        "data_type": "ExternalId"
    },
    "P4798": {
        "label": "BTO Birds of Britain ID",
        "description": "identifier for a bird species or sub-species in the BTO Birds of Britain/ BirdFacts database",
        "data_type": "ExternalId"
    },
    "P4616": {
        "label": "UniProt journal ID",
        "description": "identifier for a scientific journal, in the UniProt database",
        "data_type": "ExternalId"
    },
    "P4931": {
        "label": "Todotango.com person ID",
        "description": "identifier for a person from Todotango.com",
        "data_type": "ExternalId"
    },
    "P4651": {
        "label": "Welsh assembly ID",
        "description": "identifier issued by the National Assembly for Wales",
        "data_type": "ExternalId"
    },
    "P4286": {
        "label": "Nominis saint ID",
        "description": "saint identifier in Nominis database",
        "data_type": "ExternalId"
    },
    "P4814": {
        "label": "Inventories of American Painting and Sculpture control number",
        "description": "control number for a work, in 'Inventories of American Painting and Sculpture'",
        "data_type": "ExternalId"
    },
    "P4824": {
        "label": "org-id.guide ID",
        "description": "identifier in the register of organization databases org-id.guide",
        "data_type": "ExternalId"
    },
    "P4881": {
        "label": "Maison d'écrivain ID",
        "description": "ID for a place adhering to the Federation of Writer's Houses and Literary Heritage",
        "data_type": "ExternalId"
    },
    "P4929": {
        "label": "AINM ID",
        "description": "іdentifier for a biographical page on the site of the National Database of Irish-Language Biographies",
        "data_type": "ExternalId"
    },
    "P4769": {
        "label": "GameFAQs game ID",
        "description": "identifier for a game on GameRankings and GameFAQs",
        "data_type": "ExternalId"
    },
    "P3761": {
        "label": "IPv4 routing prefix",
        "description": "range of IPv4 addresses",
        "data_type": "String"
    },
    "P4902": {
        "label": "CONABIO ID",
        "description": "identifier for a taxon, issued by the National Commission for the Knowledge and Use of Biodiversity (aka CONABIO), a Federal Agency of the Mexican Government",
        "data_type": "ExternalId"
    },
    "P4904": {
        "label": "Tropicos publication ID",
        "description": "identifer for a journal or other publication in Tropicos",
        "data_type": "ExternalId"
    },
    "P5050": {
        "label": "UMAC ID",
        "description": "identifier for a museum or collection in the Worldwide Database of University Museums and Collections",
        "data_type": "ExternalId"
    },
    "P4380": {
        "label": "Sandrart.net artwork ID",
        "description": "artwork identifier in the web-based edition of Joachim von Sandrart’s \"Teutscher Academie der Edlen Bau, Bild- und Mahlerey-Künste\" (1675–80)",
        "data_type": "ExternalId"
    },
    "P4758": {
        "label": "MONA ID",
        "description": "identifier for a lepidoptera species, in the Hodges List (i.e. the 'Check List of the Lepidoptera of America North of Mexico', with subsequent revisions)",
        "data_type": "ExternalId"
    },
    "P4928": {
        "label": "Ricorso author ID",
        "description": "identifier for an author page on the ricorso.net site",
        "data_type": "ExternalId"
    },
    "P4754": {
        "label": "Harvard botanical journal ID",
        "description": "identifier of a publication, in Harvard University Herbaria & Libraries' 'Index of Botanical Publications'",
        "data_type": "ExternalId"
    },
    "P4977": {
        "label": "IDA place ID",
        "description": "identifier for a dark-sky place on the International Dark-Sky Association website",
        "data_type": "ExternalId"
    },
    "P4431": {
        "label": "Google Doodle",
        "description": "Google Doodle celebrating this entity or event",
        "data_type": "ExternalId"
    },
    "P3796": {
        "label": "Bureau of Meteorology station ID",
        "description": "identifier of an Australian meteorological station issued by the Bureau of Meteorology",
        "data_type": "ExternalId"
    },
    "P4991": {
        "label": "Biographical Dictionary of Georgia ID",
        "description": "identifier for a person in the Biographical Dictionary of Georgia",
        "data_type": "ExternalId"
    },
    "P4869": {
        "label": "Operabase ID",
        "description": "identifier for an opera artist in Operabase",
        "data_type": "ExternalId"
    },
    "P4102": {
        "label": "Atlas of Hillforts ID",
        "description": "identifier in the Atlas of Hillforts database",
        "data_type": "ExternalId"
    },
    "P3758": {
        "label": "DOCOMOMO Ibérico ID",
        "description": "identifier of a building, structure or group of buildings in Registros del Movimiento Moderno database, DOCOMOMO Ibérico",
        "data_type": "ExternalId"
    },
    "P5003": {
        "label": "Amphibians of India ID",
        "description": "identifier for a species of amphibian, in the 'Amphibians of India' database",
        "data_type": "ExternalId"
    },
    "P4267": {
        "label": "Landtag of Liechtenstein ID",
        "description": "identifier of a member of the Landtag of Liechtenstein",
        "data_type": "ExternalId"
    },
    "P4755": {
        "label": "UK railway station code",
        "description": "official, three-letter identifier for a railway station in the United Kingdom",
        "data_type": "ExternalId"
    },
    "P4910": {
        "label": "AlloCiné company ID",
        "description": "identifier for a company on the AlloCiné film database",
        "data_type": "ExternalId"
    },
    "P4912": {
        "label": "century breaks",
        "description": "in snooker, the number of career century breaks at officіal professional tournaments",
        "data_type": "Quantity"
    },
    "P4530": {
        "label": "Bangladesh administrative division code (pre-2017)",
        "description": "code used for an area for census calculations in Bangladesh",
        "data_type": "ExternalId"
    },
    "P3261": {
        "label": "anatomical branch of",
        "description": "main stem of this blood vessel, lymphatic vessel or nerve",
        "data_type": "WikibaseItem"
    },
    "P3357": {
        "label": "negative diagnostic predictor",
        "description": "the presence of the genetic variant helps to diagnose the absence of disease, used as exclusion criteria",
        "data_type": "WikibaseItem"
    },
    "P3359": {
        "label": "negative prognostic predictor",
        "description": "the presence of the genetic variant helps to prognose poor outcome for the disease",
        "data_type": "WikibaseItem"
    },
    "P5446": {
        "label": "reference value",
        "description": "value or range of values for a medical test (subject item), used to evaluate whether the results of the test are normal or not",
        "data_type": "WikibaseItem"
    },
    "P2239": {
        "label": "first aid measures",
        "description": "actions to take to help a person in the case of accidents, injuries and accidental exposures to harmful chemicals",
        "data_type": "WikibaseItem"
    },
    "P2176": {
        "label": "drug used for treatment",
        "description": "drug, procedure, or therapy that can be used to treat a medical condition",
        "data_type": "WikibaseItem"
    },
    "P2841": {
        "label": "age of onset",
        "description": "age group in which disease manifestations appear",
        "data_type": "WikibaseItem"
    },
    "P3457": {
        "label": "case fatality rate",
        "description": "proportion of patients who die of a particular medical condition out of all who have this condition within a given time frame",
        "data_type": "Quantity"
    },
    "P3358": {
        "label": "positive prognostic predictor",
        "description": "the presence of the genetic variant helps to prognose good outcome for the disease",
        "data_type": "WikibaseItem"
    },
    "P3781": {
        "label": "has active ingredient",
        "description": "has part biologically active component. Inverse of \"active ingredient in\"",
        "data_type": "WikibaseItem"
    },
    "P2844": {
        "label": "incidence",
        "description": "probability of occurrence of a given condition in a population within a specified period of time",
        "data_type": "Quantity"
    },
    "P2789": {
        "label": "connects with",
        "description": "item with which the item is physically connected",
        "data_type": "WikibaseItem"
    },
    "P2329": {
        "label": "antagonist muscle",
        "data_type": "WikibaseItem"
    },
    "P4425": {
        "label": "mtDNA haplogroup",
        "description": "mitochondrial DNA haplogroup of a person or organism",
        "data_type": "WikibaseItem"
    },
    "P4954": {
        "label": "may prevent",
        "description": "disease which may be prevented by this substance",
        "data_type": "WikibaseItem"
    },
    "P3464": {
        "label": "medicine marketing authorization",
        "description": "medicinal product's marketing authorization status",
        "data_type": "WikibaseItem"
    },
    "P3489": {
        "label": "pregnancy category",
        "description": "official categorisation of safety of medicine in pregnancy",
        "data_type": "WikibaseItem"
    },
    "P2854": {
        "label": "disease burden",
        "description": "impact of a health problem as measured by financial cost, mortality, morbidity, or other indicators. It is often quantified in terms of quality-adjusted life years (QALYs) or disability-adjusted life years (DALYs)",
        "data_type": "Quantity"
    },
    "P4426": {
        "label": "Y-DNA Haplogroup",
        "description": "Y-DNA haplogroup of a person or organism",
        "data_type": "WikibaseItem"
    },
    "P3354": {
        "label": "positive therapeutic predictor",
        "description": "the presence of the genetic variant helps to predict response to a treatment",
        "data_type": "WikibaseItem"
    },
    "P6884": {
        "label": "target muscle",
        "description": "muscle that is targeted in a physical exercise",
        "data_type": "WikibaseItem"
    },
    "P4044": {
        "label": "therapeutic area",
        "description": "disease area in which a medical intervention is applied",
        "data_type": "WikibaseItem"
    },
    "P3190": {
        "label": "innervates",
        "description": "anatomical structures innervated by this nerve",
        "data_type": "WikibaseItem"
    },
    "P3355": {
        "label": "negative therapeutic predictor",
        "description": "the presence of the genetic variant helps to predict no response or resistance to a treatment",
        "data_type": "WikibaseItem"
    },
    "P3356": {
        "label": "positive diagnostic predictor",
        "description": "the presence of the genetic variant helps to diagnose the presence of disease, used as inclusion criteria",
        "data_type": "WikibaseItem"
    },
    "P3492": {
        "label": "basic reproduction number",
        "description": "number of infections caused by one infection within an uninfected population",
        "data_type": "Quantity"
    },
    "P5450": {
        "label": "Biographisches Archiv der Psychiatrie  ID",
        "description": "identifier for a person on the Biographisches Archiv der Psychiatrie website",
        "data_type": "ExternalId"
    },
    "P2288": {
        "label": "lymphatic drainage",
        "description": "lymphatic drainage of an anatomical structure",
        "data_type": "WikibaseItem"
    },
    "P4545": {
        "label": "sexually homologous with",
        "description": "body part that originates from the same tissue or cell during fetal development in the opposing sex",
        "data_type": "WikibaseItem"
    },
    "P5131": {
        "label": "possible medical findings",
        "description": "possible medical findings of a medical condition",
        "data_type": "WikibaseItem"
    },
    "P5642": {
        "label": "risk factor",
        "description": "factor associated with a high prevalence of a particular gene, disease or characteristic. These factors can be country of origin, country of citizenship, race, gender, occupation, anamnesis, etc.",
        "data_type": "WikibaseItem"
    },
    "P5134": {
        "label": "evaluation of",
        "description": "the subject finding is an evaluation or interpretation of the object attribute or test",
        "data_type": "WikibaseItem"
    },
    "P3189": {
        "label": "innervated by",
        "description": "nerves which innervate this anatomical structure",
        "data_type": "WikibaseItem"
    },
    "P5133": {
        "label": "has evaluation",
        "description": "evaluation or interpretation corresponding to the subject attribute or examination",
        "data_type": "WikibaseItem"
    },
    "P5132": {
        "label": "suggests the existence of",
        "description": "medical conditions associated with the subject medical finding",
        "data_type": "WikibaseItem"
    },
    "P2175": {
        "label": "medical condition treated",
        "description": "disease that this pharmaceutical drug, procedure, or therapy is used to treat",
        "data_type": "WikibaseItem"
    },
    "P3262": {
        "label": "has anatomical branch",
        "description": "branches of this blood vessel, lymphatic vessel or nerve",
        "data_type": "WikibaseItem"
    },
    "P1098": {
        "label": "number of speakers",
        "description": "number of people who speak a language",
        "data_type": "Quantity"
    },
    "P8049": {
        "label": "number of hospitalized cases",
        "description": "number of cases that are hospitalized",
        "data_type": "Quantity"
    },
    "P59": {
        "label": "constellation",
        "description": "the area of the celestial sphere of which the subject is a part (from a scientific standpoint, not an astrological one)",
        "data_type": "WikibaseItem"
    },
    "P2989": {
        "label": "has grammatical case",
        "description": "case used in this language",
        "data_type": "WikibaseItem"
    },
    "P196": {
        "label": "minor planet group",
        "description": "is in grouping of minor planets according to similar orbital characteristics",
        "data_type": "WikibaseItem"
    },
    "P8026": {
        "label": "LiverTox likelihood score",
        "description": "categorization of the likelihood that a medication is associated with drug induced liver injury that was developed by the Drug-Induced Liver Injury Network (DILIN) Network and published in NCBI's LiverTox",
        "data_type": "WikibaseItem"
    },
    "P8991": {
        "label": "ALA-LC romanization",
        "description": "transliteration schemes for non-Roman scripts approved by the Library of Congress and the American Library Association",
        "data_type": "String"
    },
    "P9107": {
        "label": "number of vaccinations",
        "description": "number of people vaccinated, qualify with \"point in time\" (P585)",
        "data_type": "Quantity"
    },
    "P8824": {
        "label": "attraction to",
        "description": "sexual attraction or arousal arising from an entity",
        "data_type": "WikibaseItem"
    },
    "P1999": {
        "label": "UNESCO language status",
        "description": "degree of endangerment of a language conferred by the UNESCO Atlas of World Languages in Danger",
        "data_type": "WikibaseItem"
    },
    "P7581": {
        "label": "IAST transliteration",
        "description": "transliteration of Devanagari and other related Brahmic scripts",
        "data_type": "String"
    },
    "P8046": {
        "label": "Latvian transcription",
        "description": "transcription of name in Latvian orthography",
        "data_type": "String"
    },
    "P4225": {
        "label": "IPA Braille",
        "description": "representation of a IPA phoneme in the Braille system",
        "data_type": "String"
    },
    "P4913": {
        "label": "dialect of",
        "description": "language of which an item with this property is a dialect. Use in addition to \"subclass of\" (P279) if a languoid is also considered a dialect.",
        "data_type": "WikibaseItem"
    },
    "P1018": {
        "label": "language regulatory body",
        "description": "regulatory body of a language",
        "data_type": "WikibaseItem"
    },
    "P5110": {
        "label": "has grammatical person",
        "description": "grammatical feature relating to the identity of participants in an event",
        "data_type": "WikibaseItem"
    },
    "P4187": {
        "label": "Wylie transliteration",
        "description": "transliteration from Tibetan to Latin script with the Wylie method",
        "data_type": "String"
    },
    "P3103": {
        "label": "has tense",
        "description": "grammatical category expressing time reference of the language. To include a sample, use qualifier \"quote\" (P1683), sample: \"He writes\". If an activity before is needed \"He sees\". If an activity afterwards is needed: \"He reads\".",
        "data_type": "WikibaseItem"
    },
    "P5913": {
        "label": "has inflection class",
        "description": "inflectional pattern for groups of lexemes in the language",
        "data_type": "WikibaseItem"
    },
    "P8010": {
        "label": "number of recoveries",
        "description": "number of cases that recovered from disease",
        "data_type": "Quantity"
    },
    "P5825": {
        "label": "ISO 15919 transliteration",
        "description": "representation according to the named ISO standard for transliterating Brahmic scripts",
        "data_type": "String"
    },
    "P8011": {
        "label": "number of clinical tests",
        "description": "cumulative number of clinical tests",
        "data_type": "Quantity"
    },
    "P7605": {
        "label": "Hunterian transliteration",
        "description": "transliteration of different Indic scripts",
        "data_type": "String"
    },
    "P5206": {
        "label": "has conjugation class",
        "description": "conjugation pattern for group of verbs in the language. Use on items for languages. For lexemes, use \"conjugation class\" (P5186).",
        "data_type": "WikibaseItem"
    },
    "P4132": {
        "label": "linguistic typology",
        "description": "classification of languages according to their linguistic trait (as opposed to historical families like romance languages)",
        "data_type": "WikibaseItem"
    },
    "P8339": {
        "label": "entry receptor",
        "description": "receptor protein or protein whose receptor is being used by this subject virus/microorganism group to enter cells",
        "data_type": "WikibaseItem"
    },
    "P3161": {
        "label": "has grammatical mood",
        "description": "language has this grammatical mood/mode for signaling modality",
        "data_type": "WikibaseItem"
    },
    "P5139": {
        "label": "Möllendorff transliteration",
        "description": "transliteration from the Manchu alphabet to Latin script following the Möllendorff  system",
        "data_type": "String"
    },
    "P2587": {
        "label": "has phoneme",
        "description": "the language's phonology includes this sound",
        "data_type": "WikibaseItem"
    },
    "P6106": {
        "label": "uses capitalization for",
        "description": "lexical classes and subclasses of words as well as other uses in the language where the first letter is written as a capital letter",
        "data_type": "WikibaseItem"
    },
    "P3969": {
        "label": "signed form",
        "description": "manually coded form of this language",
        "data_type": "WikibaseItem"
    },
    "P282": {
        "label": "writing system",
        "description": "alphabet, character set or other system of writing used by a language, supported by a typeface",
        "data_type": "WikibaseItem"
    },
    "P2591": {
        "label": "grammatical option indicates",
        "description": "what a particular option for a grammatical category indicates",
        "data_type": "WikibaseItem"
    },
    "P5109": {
        "label": "has grammatical gender",
        "description": "specific form of noun class system of a language. Use qualifier \"statement is subject of\" (P805) to link to language specific item if present",
        "data_type": "WikibaseItem"
    },
    "P3823": {
        "label": "Ethnologue language status",
        "description": "language status identifier by Ethnologue.com using EGIDS scale",
        "data_type": "WikibaseItem"
    },
    "P8881": {
        "label": "ITRANS",
        "description": "ASCII transliteration with ITRANS scheme for Indic scripts, particularly for Devanagari script",
        "data_type": "String"
    },
    "P1252": {
        "label": "AUSTLANG code",
        "description": "identifier for a language in the Australian Indigenous Languages Database",
        "data_type": "ExternalId"
    },
    "P5475": {
        "label": "CJKV variant character",
        "description": "equivalent forms of Han characters used in different regions or writing systems",
        "data_type": "WikibaseItem"
    },
    "P1406": {
        "label": "script directionality",
        "description": "direction that a writing system goes in",
        "data_type": "WikibaseItem"
    },
    "P7500": {
        "label": "comorbidity",
        "description": "presence of one or more additional conditions co-occurring with (that is, concomitant or concurrent with) a primary condition",
        "data_type": "WikibaseItem"
    },
    "P8656": {
        "label": "Symptom Ontology ID",
        "description": "identifier in the Symptom Ontology",
        "data_type": "ExternalId"
    },
    "P2336": {
        "label": "Swedish Film Database soundtrack ID",
        "description": "soundtrack identifier in the Swedish Film Database (SFDb)",
        "data_type": "ExternalId"
    },
    "P1002": {
        "label": "engine configuration",
        "description": "configuration of an engine's cylinders",
        "data_type": "WikibaseItem"
    },
    "P2557": {
        "label": "stroke",
        "description": "length of the motion in reciprocating engines and other mechanisms",
        "data_type": "Quantity"
    },
    "P1210": {
        "label": "supercharger",
        "description": "supercharger or turbocharger used by an engine",
        "data_type": "WikibaseItem"
    },
    "P9192": {
        "label": "sensors",
        "description": "notable sensors in the electronic device",
        "data_type": "WikibaseItem"
    },
    "P1078": {
        "label": "valvetrain configuration",
        "description": "configuration of the valvetrain utilized by this engine",
        "data_type": "WikibaseItem"
    },
    "P3439": {
        "label": "angular resolution",
        "description": "angular resolution of an optical or pointing instrument",
        "data_type": "Quantity"
    },
    "P1654": {
        "label": "wing configuration",
        "description": "configuration of wing(s) used by an aircraft",
        "data_type": "WikibaseItem"
    },
    "P3375": {
        "label": "GECD Firmen-ID",
        "description": "identifier of a film company in the German Early Cinema Database",
        "data_type": "ExternalId"
    },
    "P4140": {
        "label": "energy storage capacity",
        "description": "storage capacity of a battery or energy storage system",
        "data_type": "Quantity"
    },
    "P2556": {
        "label": "bore",
        "description": "cylinder bore represents the size, in terms of diameter, of the cylinder in which a piston travels",
        "data_type": "Quantity"
    },
    "P1211": {
        "label": "fuel system",
        "description": "fuel system that an engine uses",
        "data_type": "WikibaseItem"
    },
    "P5146": {
        "label": "Cinema Project (Israel) ID",
        "description": "ID for movies backed by the Cinema Project / Yehoshua Rabinovich Foundation (Israel)",
        "data_type": "ExternalId"
    },
    "P2335": {
        "label": "Swedish Film Database company ID",
        "description": "company identifier in the Swedish Film Database (SFDb)",
        "data_type": "ExternalId"
    },
    "P8506": {
        "label": "Power plant operating licence (Turkey)",
        "description": "operating licence for a power plant in Turkey",
        "data_type": "ExternalId"
    },
    "P2019": {
        "label": "AllMovie person ID",
        "description": "identifier for a person on the AllMovie film database",
        "data_type": "ExternalId"
    },
    "P1247": {
        "label": "compression ratio",
        "description": "compression ratio of an engine",
        "data_type": "Quantity"
    },
    "P1637": {
        "label": "undercarriage",
        "description": "type of aircraft landing gear the item is equipped with",
        "data_type": "WikibaseItem"
    },
    "P1221": {
        "label": "compressor type",
        "data_type": "WikibaseItem"
    },
    "P3458": {
        "label": "CNC authorization number",
        "description": "authorization number to operate a movie theater in France",
        "data_type": "ExternalId"
    },
    "P1100": {
        "label": "number of cylinders",
        "description": "number of cylinders in a piston engine or compressor",
        "data_type": "Quantity"
    },
    "P4129": {
        "label": "Cinema Treasures ID",
        "description": "identifier for a cinema in the Cinema Treasures database",
        "data_type": "ExternalId"
    },
    "P2604": {
        "label": "Kinopoisk person ID",
        "description": "identifier for a person in the Kinopoisk.ru database",
        "data_type": "ExternalId"
    },
    "P4666": {
        "label": "CineMagia person ID",
        "description": "identifier for a person on the Cinemagia.ro website",
        "data_type": "ExternalId"
    },
    "P4981": {
        "label": "Kinoliste ID",
        "description": "ID of a cinema described on the website Kinoliste.com",
        "data_type": "ExternalId"
    },
    "P3804": {
        "label": "TV Guide show ID (former scheme)",
        "description": "ID of a television program at TV Guide",
        "data_type": "ExternalId"
    },
    "P2109": {
        "label": "installed capacity",
        "description": "power produced by the engine or plant (use with unit of power)",
        "data_type": "Quantity"
    },
    "P5071": {
        "label": "boiler pressure",
        "description": "pressure at which a steam boiler operates. Use P1013 as qualifier to indicate if it is maximum pressure.",
        "data_type": "Quantity"
    },
    "P1829": {
        "label": "Roud Folk Song Index number",
        "description": "number assigned to a folk song in the Roud Folk Song Index",
        "data_type": "ExternalId"
    },
    "P2502": {
        "label": "classification of race",
        "description": "race for which this classification applies",
        "data_type": "WikibaseItem"
    },
    "P2500": {
        "label": "league level below",
        "description": "the league below this sports league",
        "data_type": "WikibaseItem"
    },
    "P4097": {
        "label": "MuseScore ID",
        "description": "identifier in the MuseScore database",
        "data_type": "ExternalId"
    },
    "P8591": {
        "label": "Grove Music Online ID",
        "description": "identifier of a topic in Grove Music Online",
        "data_type": "ExternalId"
    },
    "P1390": {
        "label": "match time of event",
        "description": "time into a game when an event occurs (use with a unit)",
        "data_type": "Quantity"
    },
    "P6887": {
        "label": "match interval",
        "description": "the part of a sporting match when an event takes place",
        "data_type": "WikibaseItem"
    },
    "P3736": {
        "label": "Eurovision Song Contest song ID",
        "description": "identifier for a song in the Eurovision Song Contest",
        "data_type": "ExternalId"
    },
    "P545": {
        "label": "torch lit by",
        "description": "person that lit the torch at an event, like the Olympic Games",
        "data_type": "WikibaseItem"
    },
    "P1363": {
        "label": "points/goal scored by",
        "description": "person who scored a point or goal in a game",
        "data_type": "WikibaseItem"
    },
    "P3085": {
        "label": "qualifies for event",
        "description": "this event qualifies for that event",
        "data_type": "WikibaseItem"
    },
    "P3996": {
        "label": "Bait La Zemer Ha-Ivri song ID",
        "description": "ID in the israeli database for Israeli music, owned by the National Library of Israel",
        "data_type": "ExternalId"
    },
    "P9197": {
        "label": "TlF artist ID",
        "description": "identifier for a musical artist on the Tous Les Festivals website",
        "data_type": "ExternalId"
    },
    "P1407": {
        "label": "MusicBrainz series ID",
        "description": "identifier for a series per the MusicBrainz open music encyclopedia",
        "data_type": "ExternalId"
    },
    "P4794": {
        "label": "season starts",
        "description": "month or season when sports competition begins",
        "data_type": "WikibaseItem"
    },
    "P9237": {
        "label": "reissue of",
        "description": "reissue of musical work",
        "data_type": "WikibaseItem"
    },
    "P3983": {
        "label": "sports league level",
        "description": "the level of the sport league in the sport league system",
        "data_type": "Quantity"
    },
    "P9172": {
        "label": "playDB artist ID",
        "description": "identifier for music artists in playDB",
        "data_type": "ExternalId"
    },
    "P2499": {
        "label": "league level above",
        "description": "the league above this sports league",
        "data_type": "WikibaseItem"
    },
    "P2881": {
        "label": "promoted",
        "description": "competitor or team relegated during competition",
        "data_type": "WikibaseItem"
    },
    "P4035": {
        "label": "Shironet song ID",
        "description": "ID of a song in the israeli database for lyrics, Shironet",
        "data_type": "ExternalId"
    },
    "P3195": {
        "label": "league points system",
        "description": "point system of an e.g. sports league",
        "data_type": "WikibaseItem"
    },
    "P3838": {
        "label": "Tab4u song ID",
        "description": "ID in the Israeli song database \"Tab4u\"",
        "data_type": "ExternalId"
    },
    "P2882": {
        "label": "relegated",
        "description": "competitor or team relegated during competition",
        "data_type": "WikibaseItem"
    },
    "P2206": {
        "label": "Discogs release ID",
        "description": "identifier for a musical work (release) in the Discogs database, if there is no master ID (P1954)",
        "data_type": "ExternalId"
    },
    "P1652": {
        "label": "referee",
        "description": "referee or umpire of a match",
        "data_type": "WikibaseItem"
    },
    "P3022": {
        "label": "flag bearer",
        "description": "person who carries the national flag of their country at an opening or closing ceremony",
        "data_type": "WikibaseItem"
    },
    "P1339": {
        "label": "number of injured",
        "description": "number of people with non-fatal injuries in an event",
        "data_type": "Quantity"
    },
    "P3967": {
        "label": "final event",
        "description": "final event of a competition",
        "data_type": "WikibaseItem"
    },
    "P765": {
        "label": "surface played on",
        "description": "the surface on which a sporting event is played",
        "data_type": "WikibaseItem"
    },
    "P6118": {
        "label": "season ends",
        "description": "to indicate what month of the year a sports season ends",
        "data_type": "WikibaseItem"
    },
    "P9055": {
        "label": "Olympedia event ID",
        "description": "identifier for a sporting event at www.olympedia.org",
        "data_type": "ExternalId"
    },
    "P5249": {
        "label": "sports competition competed at",
        "description": "edition of sports competitions at which the club or team that played a season competed at. Use this to link items for team seasons to items for specific seasons of competitions.Sample: 2012–13 FC Barcelona season → 2012–13 UEFA Champions League",
        "data_type": "WikibaseItem"
    },
    "P8536": {
        "label": "raga",
        "description": "melodic framework for Indian music",
        "data_type": "WikibaseItem"
    },
    "P543": {
        "label": "oath made by",
        "description": "person(s) that made the oath at an event, like the Olympic Games",
        "data_type": "WikibaseItem"
    },
    "P4027": {
        "label": "National Discography of Italian Song work ID",
        "description": "identifier for a musical work in National Discography of Italian Song website",
        "data_type": "ExternalId"
    },
    "P5349": {
        "label": "laps completed",
        "description": "number of laps completed by a race participant",
        "data_type": "Quantity"
    },
    "P4072": {
        "label": "Zemereshet song ID",
        "description": "ID of a song in the israeli database for lyrics, Zemereshet",
        "data_type": "ExternalId"
    },
    "P3832": {
        "label": "Europeana Fashion Vocabulary ID",
        "description": "identifier for clothing and fashion terms",
        "data_type": "ExternalId"
    },
    "P3352": {
        "label": "musipedia tune ID",
        "description": "identification of tune in musipedia database",
        "data_type": "ExternalId"
    },
    "P8284": {
        "label": "motorsportstats.com event ID",
        "description": "identifier for a motorsport event at motorsportstats.com",
        "data_type": "ExternalId"
    },
    "P3364": {
        "label": "stereoisomer of",
        "description": "target item is a stereoisomer of this item",
        "data_type": "WikibaseItem"
    },
    "P2294": {
        "label": "balance of trade",
        "description": "exports minus imports. not to be confused with current account balance and balance of payments",
        "data_type": "Quantity"
    },
    "P489": {
        "label": "currency symbol description",
        "description": "item with description of currency symbol",
        "data_type": "WikibaseItem"
    },
    "P4147": {
        "label": "conjugate acid",
        "description": "species formed by accepting a proton (H⁺)",
        "data_type": "WikibaseItem"
    },
    "P2232": {
        "label": "cash",
        "description": "the value of cash and securities held by a state government",
        "data_type": "Quantity"
    },
    "P2219": {
        "label": "real GDP growth rate",
        "description": "percentage change in nominal gross domestic product from one year to the next minus the inflation rate",
        "data_type": "Quantity"
    },
    "P4951": {
        "label": "INCI name",
        "description": "uniform, nonproprietary, systematic name internationally recognized to identify a cosmetic ingredient",
        "data_type": "String"
    },
    "P3772": {
        "label": "agonist of",
        "description": "protein for which this chemical acts as an agonist",
        "data_type": "WikibaseItem"
    },
    "P1198": {
        "label": "unemployment rate",
        "description": "portion of a workforce population that is not employed",
        "data_type": "Quantity"
    },
    "P2140": {
        "label": "foreign direct investment net outflow",
        "description": "net outflow of equity capital, reinvestment of earnings, other long-term capital, and short-term capital",
        "data_type": "Quantity"
    },
    "P2132": {
        "label": "nominal GDP per capita",
        "description": "country's total GDP divided by the population",
        "data_type": "Quantity"
    },
    "P3771": {
        "label": "activator of",
        "description": "protein for which this chemical acts as an activator",
        "data_type": "WikibaseItem"
    },
    "P3774": {
        "label": "blocker of",
        "description": "protein (e.g. channel/pore protein) for which this chemical compound acts as a blocker",
        "data_type": "WikibaseItem"
    },
    "P3775": {
        "label": "disrupting agent for",
        "description": "protein/protein complex for which this chemical compound acts as a disrupting agent",
        "data_type": "WikibaseItem"
    },
    "P2292": {
        "label": "consumption rate",
        "description": "rate at which a product is consumed",
        "data_type": "Quantity"
    },
    "P562": {
        "label": "central bank/issuer",
        "description": "central bank or other issuing authority for the currency",
        "data_type": "WikibaseItem"
    },
    "P6185": {
        "label": "tautomer of",
        "description": "target item is a tautomer of this one",
        "data_type": "WikibaseItem"
    },
    "P3779": {
        "label": "negative allosteric modulator of",
        "description": "protein for which this chemical compound acts as a negative allosteric modulator",
        "data_type": "WikibaseItem"
    },
    "P3778": {
        "label": "positive allosteric modulator of",
        "description": "protein for which this chemical acts as a positive allosteric modulator",
        "data_type": "WikibaseItem"
    },
    "P3780": {
        "label": "active ingredient in",
        "description": "is part of and forms biologically active component. Inverse of \"has active ingredient\"",
        "data_type": "WikibaseItem"
    },
    "P2295": {
        "label": "net profit",
        "description": "private entity profit after accounting for all costs",
        "data_type": "Quantity"
    },
    "P1689": {
        "label": "central government debt as a percent of GDP",
        "data_type": "Quantity"
    },
    "P4952": {
        "label": "safety classification and labelling",
        "description": "classification and labelling data for risk identification about chemicals",
        "data_type": "WikibaseItem"
    },
    "P2135": {
        "label": "total exports",
        "description": "amount of goods and services sold to other countries",
        "data_type": "Quantity"
    },
    "P2134": {
        "label": "total reserves",
        "description": "total reserves comprise holdings of monetary gold, special drawing rights, reserves of IMF members held by the IMF, and holdings of foreign exchange under the control of monetary authorities",
        "data_type": "Quantity"
    },
    "P4770": {
        "label": "hydrated form of",
        "description": "given compound is an anhydrous form of this compound",
        "data_type": "WikibaseItem"
    },
    "P2141": {
        "label": "foreign direct investment net inflow",
        "description": "net inflow of equity capital, reinvestment of earnings, other long-term capital, and short-term capital",
        "data_type": "Quantity"
    },
    "P3776": {
        "label": "inhibitor of",
        "description": "protein for which this chemical compound acts as an inhibitor; only specify the inhibited function if the protein has more than one function (qualifier \"of\")",
        "data_type": "WikibaseItem"
    },
    "P38": {
        "label": "currency",
        "description": "currency used by item",
        "data_type": "WikibaseItem"
    },
    "P5929": {
        "label": "limiting oxygen index",
        "description": "the minimum concentration of oxygen, expressed as a percentage, that will support combustion of a polymer",
        "data_type": "Quantity"
    },
    "P1081": {
        "label": "Human Development Index",
        "description": "HDI value of a country",
        "data_type": "Quantity"
    },
    "P2220": {
        "label": "household wealth",
        "description": "net worth of consumers",
        "data_type": "Quantity"
    },
    "P2138": {
        "label": "total liabilities",
        "description": "sum of all debts and other future sacrifices of economic benefits that an entity is obliged to make",
        "data_type": "Quantity"
    },
    "P2284": {
        "label": "price",
        "description": "published price listed or paid for a product (use with unit of currency)",
        "data_type": "Quantity"
    },
    "P1716": {
        "label": "brand",
        "description": "commercial brand associated with the item",
        "data_type": "WikibaseItem"
    },
    "P8465": {
        "label": "van der Waals constant a",
        "description": "constant used in the van der Waals equation in the pressure factor",
        "data_type": "Quantity"
    },
    "P8466": {
        "label": "van der Waals constant b",
        "description": "constant used in the van der Waals equation in the volume factor",
        "data_type": "Quantity"
    },
    "P2226": {
        "label": "market capitalization",
        "description": "total value of issued shares by a publicly traded company",
        "data_type": "Quantity"
    },
    "P8224": {
        "label": "molecular model or crystal lattice model",
        "description": "graphical representation of a molecule or crystal lattice",
        "data_type": "CommonsMedia"
    },
    "P2136": {
        "label": "total imports",
        "description": "amount of goods and services bought from other countries",
        "data_type": "Quantity"
    },
    "P2131": {
        "label": "nominal GDP",
        "description": "market value of all officially recognized final goods and services produced within a country in a given period of time",
        "data_type": "Quantity"
    },
    "P2197": {
        "label": "production rate",
        "description": "amount of a given material produced in a given time",
        "data_type": "Quantity"
    },
    "P1125": {
        "label": "Gini coefficient",
        "description": "measure of statistical dispersion intended to represent the income distribution of a nation's residents",
        "data_type": "Quantity"
    },
    "P1092": {
        "label": "total produced",
        "description": "quantity of item produced",
        "data_type": "Quantity"
    },
    "P3777": {
        "label": "antisense inhibitor of",
        "description": "protein/gene for which this chemical (e.g. a miRNA/siRNA) acts as an antisense inhibitor",
        "data_type": "WikibaseItem"
    },
    "P4599": {
        "label": "monomer of",
        "description": "polymer composed of this monomer subunits",
        "data_type": "WikibaseItem"
    },
    "P4600": {
        "label": "polymer of",
        "description": "monomer of which this polymer compose",
        "data_type": "WikibaseItem"
    },
    "P3773": {
        "label": "antagonist of",
        "description": "protein for which this chemical compound acts as an antagonist",
        "data_type": "WikibaseItem"
    },
    "P1279": {
        "label": "inflation rate",
        "description": "percent change in the consumer price index (CPI)",
        "data_type": "Quantity"
    },
    "P9275": {
        "label": "inverse agonist of",
        "description": "substance whose binding to a protein produces the opposite response as an agonist binding to that protein",
        "data_type": "WikibaseItem"
    },
    "P8459": {
        "label": "descriptive solubility",
        "description": "(qualifier for P2177) qualitative level of solubility, when a numerical value is not available",
        "data_type": "WikibaseItem"
    },
    "P4149": {
        "label": "conjugate base",
        "description": "species formed by losing a proton (H⁺)",
        "data_type": "WikibaseItem"
    },
    "P1951": {
        "label": "investor",
        "description": "individual or organization which invests money in the item for the purpose of obtaining financial return on their investment",
        "data_type": "WikibaseItem"
    },
    "P7075": {
        "label": "mod of",
        "description": "video game this mod is based on",
        "data_type": "WikibaseItem"
    },
    "P6554": {
        "label": "Represent Me ID",
        "description": "identifier of games on Represent Me",
        "data_type": "ExternalId"
    },
    "P5107": {
        "label": "Behind The Voice Actors character ID",
        "description": "identifier of a character from the Behind The Voice Actors website",
        "data_type": "ExternalId"
    },
    "P5247": {
        "label": "Giant Bomb ID",
        "description": "identifier at the Giant Bomb database of video games, fictional characters and voice actors",
        "data_type": "ExternalId"
    },
    "P5379": {
        "label": "GAMECIP platform ID",
        "description": "identifier of a computer game platform in the thesaurus of the Game Metadata and Citation Project",
        "data_type": "ExternalId"
    },
    "P5371": {
        "label": "GAMECIP media format ID",
        "description": "identifier of a media format in the thesaurus of the Game Metadata and Citation Project",
        "data_type": "ExternalId"
    },
    "P2894": {
        "label": "day of week",
        "description": "day of the week on which this item occurs, applies to or is valid",
        "data_type": "WikibaseItem"
    },
    "P7584": {
        "label": "age estimated by a dating method",
        "description": "estimation of the age of a star, geological feature, fossil or object",
        "data_type": "Quantity"
    },
    "P9032": {
        "label": "AVN movie ID",
        "description": "identifier for a pornographic movie in the AVN.com database",
        "data_type": "ExternalId"
    },
    "P5461": {
        "label": "model year code",
        "description": "letter or digit in position 10 of a vehicle identification number (VIN) indicating the model year (MY) for years 1980 to 2009, repeating for 2010 to 2039, etc. Format: not I,O,Q,U,Z,0. Year 2018 is J.",
        "data_type": "String"
    },
    "P1151": {
        "label": "topic's main Wikimedia portal",
        "description": "Wikimedia portal associated with this topic",
        "data_type": "WikibaseItem"
    },
    "P4174": {
        "label": "Wikimedia username",
        "description": "user name of a person across all Wikimedia projects",
        "data_type": "ExternalId"
    },
    "P837": {
        "label": "day in year for periodic occurrence",
        "description": "when a specific holiday or periodic event occurs. Can be used as property or qualifier",
        "data_type": "WikibaseItem"
    },
    "P2922": {
        "label": "month of the year",
        "description": "month of the year during which this item occurs, applies to or is valid in",
        "data_type": "WikibaseItem"
    },
    "P6259": {
        "label": "epoch",
        "description": "epoch of an astronomical object coordinate",
        "data_type": "WikibaseItem"
    },
    "P3453": {
        "label": "IPI base code",
        "description": "the international code for a legal entity or artist for copyright administration around Interested Parties Information",
        "data_type": "ExternalId"
    },
    "P5098": {
        "label": "IAFD film ID",
        "description": "identifier for a film in the Internet Adult Film Database",
        "data_type": "ExternalId"
    },
    "P1362": {
        "label": "Theaterlexikon der Schweiz ID",
        "description": "identifier in the online edition of the Theater Dictionary of Switzerland",
        "data_type": "ExternalId"
    },
    "P5083": {
        "label": "Adult Film Database film ID",
        "description": "identifier for a film in the Adult Film Database",
        "data_type": "ExternalId"
    },
    "P4895": {
        "label": "time index",
        "description": "indicate when something happens relative to its beginning",
        "data_type": "Quantity"
    },
    "P2896": {
        "label": "publication interval",
        "description": "standard publication interval for daily or periodical publications",
        "data_type": "Quantity"
    },
    "P3215": {
        "label": "SIRET number",
        "description": "number of a French establishment or company",
        "data_type": "ExternalId"
    },
    "P1204": {
        "label": "Wikimedia portal's main topic",
        "description": "primary topic of this Wikimedia portal",
        "data_type": "WikibaseItem"
    },
    "P1385": {
        "label": "Enciclopédia Açoriana ID",
        "description": "identifier for an item in Enciclopédia Açoriana",
        "data_type": "ExternalId"
    },
    "P4316": {
        "label": "kinship equivalent in SPARQL at Wikidata",
        "description": "part of a SPARQL query that lists this type of ?relative of a ?person. Use with relationship properties (except \"relative\" (P1038)). See instructions for applicable format",
        "data_type": "String"
    },
    "P3125": {
        "label": "EDRPOU code",
        "description": "Ukrainian state registry legal entity identifier",
        "data_type": "ExternalId"
    },
    "P830": {
        "label": "Encyclopedia of Life ID",
        "description": "eol.org item reference number",
        "data_type": "ExternalId"
    },
    "P5332": {
        "label": "OpenDOAR ID",
        "description": "identifier in the Directory of Open Access Repositories (OpenDOAR)",
        "data_type": "ExternalId"
    },
    "P5285": {
        "label": "Firmenbuchnummer",
        "description": "Austrian company register ID",
        "data_type": "ExternalId"
    },
    "P2803": {
        "label": "Wikidata time precision",
        "description": "numeric value used by Wikidata to describe the corresponding time/date precision. There are 15 items with this property with values from 0 to 14 (second), year is 9.",
        "data_type": "Quantity"
    },
    "P1296": {
        "label": "Gran Enciclopèdia Catalana ID",
        "description": "identifier for an item in the Gran Enciclopèdia Catalana",
        "data_type": "ExternalId"
    },
    "P8687": {
        "label": "social media followers",
        "description": "number of subscribers on a particular social media website (use as main statement only; see P3744 instead for qualifier). Qualify with \"point in time\" and property for account. For Twitter, use numeric id.",
        "data_type": "Quantity"
    },
    "P2377": {
        "label": "MediaWiki hooks used",
        "description": "MediaWiki hooks used by this extension",
        "data_type": "WikibaseItem"
    },
    "P3182": {
        "label": "FANTOIR code",
        "description": "unique code to identify French thoroughfares and places",
        "data_type": "ExternalId"
    },
    "P4329": {
        "label": "template or module that populates category",
        "description": "this category is populated by the following template or module",
        "data_type": "WikibaseItem"
    },
    "P8393": {
        "label": "QUDT quantity kind ID",
        "description": "identifier for a quantity in the QUDT ontology",
        "data_type": "ExternalId"
    },
    "P2348": {
        "label": "time period",
        "description": "time period (historic period or era, sports season, theatre season, legislative period etc.) in which the subject occurred",
        "data_type": "WikibaseItem"
    },
    "P9077": {
        "label": "number of aid beneficiaries",
        "description": "number of people reached with international aid programmes, actions and humanitarian activities. Qualify with \"point in time\" (P585) where appropriate",
        "data_type": "Quantity"
    },
    "P8757": {
        "label": "maximum current",
        "description": "maximum current for this item, specify unit (e.g. Ampere)",
        "data_type": "Quantity"
    },
    "P5464": {
        "label": "A Church Near You church ID",
        "description": "identifier for a church at the A Church Near You website",
        "data_type": "ExternalId"
    },
    "P5547": {
        "label": "CICLIC author ID",
        "description": "identifier for an author on the 'Centre pour le livre, l'image et la culture numérique'﻿ website",
        "data_type": "ExternalId"
    },
    "P5473": {
        "label": "The Reptile Database ID",
        "description": "identifier for all living reptile species in The Reptile Database",
        "data_type": "ExternalId"
    },
    "P5449": {
        "label": "Inventory of French sanctuaries ID",
        "description": "identifier for items in the inventory of Christian shrines and pilgrimage sites in France",
        "data_type": "ExternalId"
    },
    "P5160": {
        "label": "Thesaurus for Graphic Materials ID",
        "description": "identifier for indexing visual (graphic) materials by subject and genre/format, maintained by the United States Library of Congress, Prints and Photographs Division",
        "data_type": "ExternalId"
    },
    "P5455": {
        "label": "SGJP Online ID",
        "description": "identifier for an entry in the online version of Grammatical Dictionary of Polish (pl Słownik gramatyczny języka polskiego)",
        "data_type": "ExternalId"
    },
    "P5470": {
        "label": "Scottish Poetry Library poet ID",
        "description": "identifier for a poet on the Scottish Poetry Library website",
        "data_type": "ExternalId"
    },
    "P5537": {
        "label": "Hangul of a Chinese character",
        "description": "The modern Korean pronunciation(s) for this character in Hangul",
        "data_type": "WikibaseItem"
    },
    "P5532": {
        "label": "L'île author ID",
        "description": "identifier for an author on the Infocentre littéraire des écrivains website",
        "data_type": "ExternalId"
    },
    "P5542": {
        "label": "Literature Ireland ID",
        "description": "identifier for an author on the Literature Ireland website",
        "data_type": "ExternalId"
    },
    "P5538": {
        "label": "Auteurs en Auvergne-Rhône-Alpes ID",
        "description": "identifier for an author on the Auteurs en Auvergne-Rhône-Alpes website",
        "data_type": "ExternalId"
    },
    "P5539": {
        "label": "Eulalie ID",
        "description": "identifier for an author on the Eulalie website",
        "data_type": "ExternalId"
    },
    "P5535": {
        "label": "Prison History ID",
        "description": "identifier for a prison or lock-up in the 'Prison History' database of 19th century prisons in the UK",
        "data_type": "ExternalId"
    },
    "P5543": {
        "label": "Livre et lecture en Bretagne ID",
        "description": "identifier for an author on the Livre et lecture en Bretagne website",
        "data_type": "ExternalId"
    },
    "P5344": {
        "label": "Printemps des poètes poet ID",
        "description": "identifier for a poet on the Printemps des poètes website",
        "data_type": "ExternalId"
    },
    "P5522": {
        "label": "Unicode block",
        "description": "Unicode block that a character is in",
        "data_type": "WikibaseItem"
    },
    "P5454": {
        "label": "USNPL ID",
        "description": "identifier in the US Newspaper Links (USNPL) database for newspapers in the United States",
        "data_type": "ExternalId"
    },
    "P5208": {
        "label": "BAG building ID",
        "description": "BAG building ID for Dutch buildings (\"panden\")",
        "data_type": "ExternalId"
    },
    "P5263": {
        "label": "Czech NDOP taxon ID",
        "description": "identifier for a taxon in the Conservancy Species Occurrence Finding Database, managed by the Nature Conservation Agency of the Czech Republic",
        "data_type": "ExternalId"
    },
    "P5268": {
        "label": "MNAV work ID",
        "description": "identifier assigned to an artwork by the National Museum of Visual Arts of Uruguay",
        "data_type": "ExternalId"
    },
    "P5213": {
        "label": "Armenian Parliamentary ID",
        "description": "identifier for a Member of the National Assembly of the Republic of Armenia, in its official website",
        "data_type": "ExternalId"
    },
    "P5216": {
        "label": "GONIAT taxon ID",
        "description": "identifier for a taxon, in the GONIAT database",
        "data_type": "ExternalId"
    },
    "P5207": {
        "label": "BAG public space ID",
        "description": "BAG openbare ruimtecode for Dutch public spaces",
        "data_type": "ExternalId"
    },
    "P5497": {
        "label": "Doroszewski Online ID",
        "description": "identifier for an entry in the online version of the Polish Dictionary by Witold Doroszewski (Pl Słownik języka polskiego)",
        "data_type": "ExternalId"
    },
    "P5210": {
        "label": "National Gallery of Armenia work ID",
        "description": "identifier for an artwork, in the National Gallery of Armenia database",
        "data_type": "ExternalId"
    },
    "P5225": {
        "label": "Argentine biography deputy ID",
        "description": "biography of an Argentine deputy from the Chamber of Deputies' website",
        "data_type": "ExternalId"
    },
    "P5354": {
        "label": "Amphibian Species of the World ID",
        "description": "identifier for a taxon in the Amphibian Species of the World database",
        "data_type": "ExternalId"
    },
    "P5226": {
        "label": "BRAHMS artist ID",
        "description": "identifier for a composer in the B.R.A.H.M.S. database, by the IRCAM",
        "data_type": "ExternalId"
    },
    "P5321": {
        "label": "Museo del Prado artist ID",
        "description": "identifier of an artist in the Museo del Prado web page",
        "data_type": "ExternalId"
    },
    "P5272": {
        "label": "CDMC musician ID",
        "description": "identifier for a musician in the Centre de documentation de la musique contemporaine database",
        "data_type": "ExternalId"
    },
    "P5274": {
        "label": "SJP Online ID",
        "description": "identifier for an entry in the online version of Polish Dictionary (pl Słownik Języka Polskiego)",
        "data_type": "ExternalId"
    },
    "P5394": {
        "label": "Poets & Writers author ID",
        "description": "identifier for an author on the Poets & Writers website",
        "data_type": "ExternalId"
    },
    "P5299": {
        "label": "AntWeb ID",
        "description": "identifier for a taxon in the AntWeb database",
        "data_type": "ExternalId"
    },
    "P5302": {
        "label": "BRAHMS work ID",
        "description": "identifier for a musical work in the B.R.A.H.M.S. database, by the IRCAM",
        "data_type": "ExternalId"
    },
    "P5465": {
        "label": "Australian Poetry Library poet ID",
        "description": "identifier for a poet on the Australian Poetry Library website",
        "data_type": "ExternalId"
    },
    "P5430": {
        "label": "Poetry International Web poet ID",
        "description": "identifier for a poet on the Poetry International Web website",
        "data_type": "ExternalId"
    },
    "P5515": {
        "label": "Znicenekostely ID",
        "description": "identifier for damaged and destroyed churches in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P5383": {
        "label": "archINFORM project ID",
        "description": "identifier for a project on the archINFORM website",
        "data_type": "ExternalId"
    },
    "P5393": {
        "label": "cipM poet ID",
        "description": "identifier for a poet on the Centre international de poésie Marseille website",
        "data_type": "ExternalId"
    },
    "P5397": {
        "label": "Tierstimmenarchiv ID",
        "description": "identifier for a species in the Animal Sound Archive Berlin",
        "data_type": "ExternalId"
    },
    "P6434": {
        "label": "Amsterdam code",
        "description": "code for municipalities in the Netherlands",
        "data_type": "ExternalId"
    },
    "P6314": {
        "label": "WorldFootball.net referee ID",
        "description": "WorldFootball.net identifier for an association football (soccer) referee",
        "data_type": "ExternalId"
    },
    "P6425": {
        "label": "LGD local body code",
        "description": "code used by the Local Government Directory, a website run by the Government of India, which maintains directory of rural and urban local governments in India and their political territorial entities",
        "data_type": "ExternalId"
    },
    "P6605": {
        "label": "Plant Illustrations artist ID",
        "description": "identifier for a botanical artist, on the Plant Illustrations website",
        "data_type": "ExternalId"
    },
    "P6420": {
        "label": "Georgian National Filmography ID",
        "description": "identifier for a film in the Georgian National Filmography database",
        "data_type": "ExternalId"
    },
    "P6574": {
        "label": "Libraries & Information Centers in the Czech Republic ID",
        "description": "identifier for libraries and information centers in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P6315": {
        "label": "PlaymakerStats.com referee ID",
        "description": "identifier for an association football (soccer) referee at PlaymakerStats.com (formerly TheFinalBall.com)",
        "data_type": "ExternalId"
    },
    "P6319": {
        "label": "Italian Footballers' Association player ID",
        "description": "identifier for football player of AIC",
        "data_type": "ExternalId"
    },
    "P6585": {
        "label": "Directorio Legislativo ID",
        "description": "identiifer for a deputy in the National Congress.of  Argentina",
        "data_type": "ExternalId"
    },
    "P6506": {
        "label": "Eliseu Visconti Project ID",
        "description": "Catalog of works of Eliseu Visconti",
        "data_type": "ExternalId"
    },
    "P6594": {
        "label": "Guggenheim fellows ID",
        "description": "identifier for a person awarded a Guggenheim fellowship in the arts",
        "data_type": "ExternalId"
    },
    "P6467": {
        "label": "Hulu series ID",
        "description": "TV series identifier in Hulu",
        "data_type": "ExternalId"
    },
    "P6369": {
        "label": "Media Arts Database manga ID (former scheme)",
        "description": "identifier for a manga in the Media Art database created by the Japanese government department Bunkacho (obsolete, use P7886 for new identifier with 'C/M' prefix)",
        "data_type": "ExternalId"
    },
    "P6359": {
        "label": "Crew United title ID",
        "description": "identifier for films and series on Crew united",
        "data_type": "ExternalId"
    },
    "P6340": {
        "label": "Uppsala General Catalogue ID",
        "description": "identifier of a galaxy in the Uppsala General Catalogue",
        "data_type": "ExternalId"
    },
    "P6385": {
        "label": "Krugosvet article",
        "description": "link to article of Russian-language online encyclopedia",
        "data_type": "ExternalId"
    },
    "P6559": {
        "label": "Official Charts artist ID",
        "description": "identifier for musical acts at Official Charts Company's website",
        "data_type": "ExternalId"
    },
    "P6601": {
        "label": "Resident Advisor label ID",
        "description": "identifier for record labels at the Resident Advisor website",
        "data_type": "ExternalId"
    },
    "P6417": {
        "label": "Homosaurus ID",
        "description": "authorized term in the International Thesaurus of Lesbian, Gay, Bisexual and Transgender Index Terms (Homosaurus)",
        "data_type": "ExternalId"
    },
    "P6610": {
        "label": "Ashmolean museum ID",
        "description": "number of the object on online  collections",
        "data_type": "ExternalId"
    },
    "P6335": {
        "label": "MDWS place ID",
        "description": "code used by the Ministry of Drinking Water and Sanitation, Government of India to identify village, panchayats, community development blocks, districts and states of India.",
        "data_type": "ExternalId"
    },
    "P6382": {
        "label": "Hungarian public thesaurus ID",
        "description": "authorized term in the OSZK Thesaurus / Public thesaurus",
        "data_type": "ExternalId"
    },
    "P6466": {
        "label": "Hulu movie ID",
        "description": "movie identifier in Hulu",
        "data_type": "ExternalId"
    },
    "P6368": {
        "label": "Media Arts Database anime ID (former scheme)",
        "description": "identifier for an anime in the Media Art database created by the Japanese government department Bunkacho (former scheme, use P7886 for new identifier with 'C/M' prefix)",
        "data_type": "ExternalId"
    },
    "P6260": {
        "label": "galactic longitude",
        "description": "galactic longitude of an astronomical object",
        "data_type": "Quantity"
    },
    "P6343": {
        "label": "urban population",
        "description": "number of people living within the territorial entity who live in its urban parts",
        "data_type": "Quantity"
    },
    "P2584": {
        "label": "Australian Wetlands Code",
        "description": "identifier of a wetland listed in the Australian Wetlands Database Directory of Important Wetlands managed by the Australian Government Department of the Environment",
        "data_type": "ExternalId"
    },
    "P3592": {
        "label": "Saros cycle of eclipse",
        "data_type": "WikibaseItem"
    },
    "P8763": {
        "label": "birth rate",
        "description": "the total number of live births per 1,000 population",
        "data_type": "Quantity"
    },
    "P1046": {
        "label": "discovery method",
        "description": "way an exoplanet was discovered",
        "data_type": "WikibaseItem"
    },
    "P1082": {
        "label": "population",
        "description": "number of people inhabiting the place; number of people of subject",
        "data_type": "Quantity"
    },
    "P205": {
        "label": "basin country",
        "description": "country that have drainage to/from or border the body of water",
        "data_type": "WikibaseItem"
    },
    "P5982": {
        "label": "annual number of weddings",
        "description": "number of marriages per year in a location",
        "data_type": "Quantity"
    },
    "P2213": {
        "label": "longitude of ascending node",
        "description": "property of one of the orbital elements used to specify the orbit of an object in space",
        "data_type": "Quantity"
    },
    "P6257": {
        "label": "right ascension",
        "description": "astronomical equivalent of longitude",
        "data_type": "Quantity"
    },
    "P1538": {
        "label": "number of households",
        "description": "total number of households in this place, includes dwellings of all types",
        "data_type": "Quantity"
    },
    "P490": {
        "label": "provisional designation",
        "description": "designation of an astronomical body after its discovery and before its official name",
        "data_type": "String"
    },
    "P6261": {
        "label": "galactic latitude",
        "description": "galactic latitude of an astronomical object",
        "data_type": "Quantity"
    },
    "P2895": {
        "label": "maximum sustained winds",
        "description": "maximum sustained wind evaluated by agency",
        "data_type": "Quantity"
    },
    "P2325": {
        "label": "mean anomaly",
        "description": "element of orbital definition",
        "data_type": "Quantity"
    },
    "P881": {
        "label": "type of variable star",
        "description": "type of variable star",
        "data_type": "WikibaseItem"
    },
    "P2516": {
        "label": "Australian Ramsar site ID",
        "description": "identifier of a Ramsar wetland listed in the Australian Wetlands Database managed by the Australian Government Department of the Environment",
        "data_type": "ExternalId"
    },
    "P469": {
        "label": "lakes on river",
        "description": "lakes, reservoirs, waterfalls the river flows through",
        "data_type": "WikibaseItem"
    },
    "P1090": {
        "label": "redshift",
        "description": "the redshift measure of an astronomical object",
        "data_type": "Quantity"
    },
    "P3036": {
        "label": "precipitation height",
        "description": "maximum height of snow or rain during an event or space of time",
        "data_type": "Quantity"
    },
    "P6879": {
        "label": "effective temperature",
        "description": "the temperature of a black body that would emit the same total amount of electromagnetic radiation as measured star or planet",
        "data_type": "Quantity"
    },
    "P1227": {
        "label": "astronomical filter",
        "description": "passband used to isolate particular sections of the electromagnetic spectrum",
        "data_type": "WikibaseItem"
    },
    "P6258": {
        "label": "declination",
        "description": "astronomical equivalent of latitude",
        "data_type": "Quantity"
    },
    "P398": {
        "label": "child astronomical body",
        "description": "minor body that belongs to the item",
        "data_type": "WikibaseItem"
    },
    "P399": {
        "label": "companion of",
        "description": "two or more astronomic bodies of the same type relating to each other",
        "data_type": "WikibaseItem"
    },
    "P2225": {
        "label": "discharge",
        "description": "volume rate of water flow which is transported through a given cross-sectional area",
        "data_type": "Quantity"
    },
    "P2147": {
        "label": "rotation period",
        "description": "the time that astronomic objects takes to complete one revolution around its axis of rotation",
        "data_type": "Quantity"
    },
    "P744": {
        "label": "asteroid family",
        "description": "population of asteroids that share similar proper orbital elements",
        "data_type": "WikibaseItem"
    },
    "P2221": {
        "label": "flux",
        "description": "the observed flux in a particular wavelength band of an astronomical object",
        "data_type": "Quantity"
    },
    "P2564": {
        "label": "Köppen climate classification",
        "description": "indicates the characteristic climate of a place",
        "data_type": "WikibaseItem"
    },
    "P1215": {
        "label": "apparent magnitude",
        "description": "measurement of the brightness of an astronomic object, as seen from the Earth",
        "data_type": "Quantity"
    },
    "P397": {
        "label": "parent astronomical body",
        "description": "major astronomical body the item belongs to",
        "data_type": "WikibaseItem"
    },
    "P2215": {
        "label": "proper motion",
        "description": "proper motion of a star",
        "data_type": "Quantity"
    },
    "P223": {
        "label": "galaxy morphological type",
        "description": "galaxy morphological classification code",
        "data_type": "String"
    },
    "P201": {
        "label": "lake outflow",
        "description": "rivers and other outflows waterway names. If evaporation or seepage are notable outflows, they may be included. Some terms may not be place names, e.g. evaporation",
        "data_type": "WikibaseItem"
    },
    "P1145": {
        "label": "Lagrangian point",
        "description": "libration point of an object (default: object-sun-earth)",
        "data_type": "WikibaseItem"
    },
    "P3864": {
        "label": "suicide rate",
        "description": "amount of suicides per 100,000 people in the place or group",
        "data_type": "Quantity"
    },
    "P1540": {
        "label": "male population",
        "description": "number of male people inhabiting the place; number of male people of subject",
        "data_type": "Quantity"
    },
    "P720": {
        "label": "asteroid spectral type",
        "description": "spectral classifications of asteroids based on  spectral shape, color, and albedo",
        "data_type": "WikibaseItem"
    },
    "P6344": {
        "label": "rural population",
        "description": "number of people living within the territorial entity who live in its rural parts",
        "data_type": "Quantity"
    },
    "P4841": {
        "label": "total fertility rate",
        "description": "average number of children that would be born to a woman over her lifetime",
        "data_type": "Quantity"
    },
    "P1096": {
        "label": "orbital eccentricity",
        "description": "amount of the deviation of an orbit from a perfect circle",
        "data_type": "Quantity"
    },
    "P1457": {
        "label": "absolute magnitude",
        "description": "absolute magnitude of an astronomic object",
        "data_type": "Quantity"
    },
    "P2214": {
        "label": "parallax",
        "description": "parallax of nearest stars",
        "data_type": "Quantity"
    },
    "P1458": {
        "label": "color index",
        "description": "color index of an astronomical object",
        "data_type": "Quantity"
    },
    "P885": {
        "label": "origin of the watercourse",
        "description": "main source of a river, stream or lake",
        "data_type": "WikibaseItem"
    },
    "P974": {
        "label": "tributary",
        "description": "stream or river that flows into a larger river (for lake inflows use P200)",
        "data_type": "WikibaseItem"
    },
    "P4341": {
        "label": "synodic period",
        "description": "time between two passes of an object in front of a distant star seen from another orbiting object",
        "data_type": "Quantity"
    },
    "P1879": {
        "label": "income classification (Philippines)",
        "description": "classification grade of a Philippine local government unit based on income",
        "data_type": "WikibaseItem"
    },
    "P9149": {
        "label": "coordinates of depicted place",
        "description": "coordinates of the place depicted in the work (or Commons image)",
        "data_type": "GlobeCoordinate"
    },
    "P1548": {
        "label": "maximum Strahler number",
        "description": "highest Strahler number of a stream, creek, or river",
        "data_type": "Quantity"
    },
    "P4442": {
        "label": "mean age",
        "description": "mean age in a given place; qualify with P585",
        "data_type": "Quantity"
    },
    "P3025": {
        "label": "open days",
        "description": "days in the week subject item is open within the opening season",
        "data_type": "WikibaseItem"
    },
    "P501": {
        "label": "enclave within",
        "description": "territory is entirely surrounded by the other (enclaved)",
        "data_type": "WikibaseItem"
    },
    "P37": {
        "label": "official language",
        "description": "language designated as official by this item",
        "data_type": "WikibaseItem"
    },
    "P3335": {
        "label": "associated hazard",
        "description": "hazards implicit in the subject item: that a sign warns of; that one may encounter in this place, on this thoroughfare; that are inherent properties of the subject; etc.",
        "data_type": "WikibaseItem"
    },
    "P2633": {
        "label": "geography of topic",
        "description": "item that deals with the geography of the subject. Sample: \"Rio de Janeiro\" uses this property with value \"geography of Rio de Janeiro\" (Q10288853). For the location of a subject, use \"location\" (P276).",
        "data_type": "WikibaseItem"
    },
    "P1456": {
        "label": "list of monuments",
        "description": "link to the list of heritage monuments in the place/area",
        "data_type": "WikibaseItem"
    },
    "P78": {
        "label": "top-level Internet domain",
        "description": "Internet domain name system top-level code",
        "data_type": "WikibaseItem"
    },
    "P6591": {
        "label": "maximum temperature record",
        "description": "maximum temperature reached at this location",
        "data_type": "Quantity"
    },
    "P4238": {
        "label": "webcam page URL",
        "description": "URL of a page broadcasting pictures by a webcam situated in this location or mounted on this element",
        "data_type": "Url"
    },
    "P7422": {
        "label": "minimum temperature record",
        "description": "lowest temperature reached at this location",
        "data_type": "Quantity"
    },
    "P3173": {
        "label": "offers view on",
        "description": "things, places another place offers views on",
        "data_type": "WikibaseItem"
    },
    "P1083": {
        "label": "maximum capacity",
        "description": "number of people allowed for a venue or vehicle",
        "data_type": "Quantity"
    },
    "P1174": {
        "label": "visitors per year",
        "description": "number of people visiting a location or an event each year",
        "data_type": "Quantity"
    },
    "P8546": {
        "label": "recording location",
        "description": "location where a media item (sound/video/etc) was recorded",
        "data_type": "WikibaseItem"
    },
    "P7959": {
        "label": "historic county",
        "description": "traditional, geographical division of Great Britain and Ireland",
        "data_type": "WikibaseItem"
    },
    "P2927": {
        "label": "water as percent of area",
        "description": "which percentage of the territory of this item inside coast line and international bounderies is water. Use \"percent\" (Q11229)  as unit",
        "data_type": "Quantity"
    },
    "P4856": {
        "label": "conscription number",
        "description": "number identifying a building in one cadastral area/village/neighborhood",
        "data_type": "String"
    },
    "P4345": {
        "label": "director of archaeological fieldwork",
        "description": "person leading the archaeological or anthropological investigation at a given site",
        "data_type": "WikibaseItem"
    },
    "P5140": {
        "label": "coordinates of geographic center",
        "description": "coordinates of the center of an area. Use qualifier \"determination method\" (P459) to indicate how",
        "data_type": "GlobeCoordinate"
    },
    "P1383": {
        "label": "contains settlement",
        "description": "settlement which an administrative division contains",
        "data_type": "WikibaseItem"
    },
    "P814": {
        "label": "IUCN protected areas category",
        "description": "protected areas category by the World Commission on Protected Areas. Used with dedicated items for each category.",
        "data_type": "WikibaseItem"
    },
    "P2936": {
        "label": "language used",
        "description": "language widely used (spoken or written) in this place or at this event",
        "data_type": "WikibaseItem"
    },
    "P7532": {
        "label": "room number",
        "description": "number or identifier of a room",
        "data_type": "String"
    },
    "P4595": {
        "label": "post town",
        "description": "town part of the postal address, for example in the UK or Sweden",
        "data_type": "String"
    },
    "P150": {
        "label": "contains administrative territorial entity",
        "description": "(list of) direct subdivisions of an administrative territorial entity",
        "data_type": "WikibaseItem"
    },
    "P8981": {
        "label": "lunar coordinates",
        "description": "coordinates of a place on Earth's Moon",
        "data_type": "GlobeCoordinate"
    },
    "P190": {
        "label": "twinned administrative body",
        "description": "twin towns, sister cities, twinned municipalities and other localities that have a partnership or cooperative agreement, either legally or informally acknowledged by their governments",
        "data_type": "WikibaseItem"
    },
    "P7938": {
        "label": "associated electoral district",
        "description": "constituencies/electoral districts in which a place is located or is part of. If a municipality/county is split into or part of several districts: add several values. Use only if distinct from administrative entities (P131) in predefined countries",
        "data_type": "WikibaseItem"
    },
    "P500": {
        "label": "exclave of",
        "description": "territory is legally or politically attached to a main territory with which it is not physically contiguous because of surrounding alien territory. It may also be an enclave.",
        "data_type": "WikibaseItem"
    },
    "P912": {
        "label": "has facility",
        "description": "the subject item has this type of facility, e.g. toilet, car park",
        "data_type": "WikibaseItem"
    },
    "P8138": {
        "label": "located in the statistical territorial entity",
        "description": "statistical territorial entity in which a place is located or is part of. If a municipality or county is split into or part of several regions: add several values",
        "data_type": "WikibaseItem"
    },
    "P4179": {
        "label": "tabular population",
        "description": "same as population (P1082); used when there is data for many different points in time",
        "data_type": "TabularData"
    },
    "P2581": {
        "label": "BabelNet ID",
        "description": "ID in BabelNet encyclopedic dictionary",
        "data_type": "ExternalId"
    },
    "P5169": {
        "label": "Norwegian historical lexicon ID",
        "description": "identifier of an article in the online historic encyclopedia Norsk historisk leksikon",
        "data_type": "ExternalId"
    },
    "P3724": {
        "label": "USHMM Holocaust Encyclopedia ID",
        "description": "identifier in the Holocaust Encyclopedia of the United States Holocaust Memorial Museum",
        "data_type": "ExternalId"
    },
    "P4255": {
        "label": "Banglapedia (English version) ID",
        "description": "identifier for an English article on Banglapedia, the national online encyclopedia of Bangladesh",
        "data_type": "ExternalId"
    },
    "P4613": {
        "label": "Encyclopedia of Modern Ukraine ID",
        "description": "identifier for an entry on the official website of the Encyclopedia of Modern Ukraine",
        "data_type": "ExternalId"
    },
    "P3021": {
        "label": "Iranica ID",
        "description": "identifier of an article, on iranicaonline.org",
        "data_type": "ExternalId"
    },
    "P5019": {
        "label": "Brockhaus Enzyklopädie online ID",
        "description": "identifier for an article in the online version of Brockhaus Enzyklopädie",
        "data_type": "ExternalId"
    },
    "P6228": {
        "label": "RegiowikiAT ID",
        "description": "numeric identifier of a page in the Austria regional wiki RegiowikiAT",
        "data_type": "ExternalId"
    },
    "P2924": {
        "label": "Great Russian Encyclopedia Online ID",
        "description": "identifier for an entry on the official website of the Great Russian Encyclopedia",
        "data_type": "ExternalId"
    },
    "P6520": {
        "label": "Lokalhistoriewiki.no ID",
        "description": "identifier for an article in lokalhistoriewiki.no",
        "data_type": "ExternalId"
    },
    "P3794": {
        "label": "Dictionary of Sydney ID",
        "description": "identifier in the Dictionary of Sydney",
        "data_type": "ExternalId"
    },
    "P1631": {
        "label": "China Vitae ID",
        "description": "identifier for a person in the China Vitae database",
        "data_type": "ExternalId"
    },
    "P2843": {
        "label": "Benezit ID",
        "description": "identifier in Benezit Dictionary of Artists",
        "data_type": "ExternalId"
    },
    "P1988": {
        "label": "Le Delarge ID",
        "description": "identifier for an artist in the Delarge dictionary",
        "data_type": "ExternalId"
    },
    "P5081": {
        "label": "Norsk kunstnerleksikon ID",
        "description": "identifier of an article in the online visual artist encyclopedia nkl.snl.no",
        "data_type": "ExternalId"
    },
    "P1565": {
        "label": "Enciclopedia de la Literatura en México ID",
        "description": "identifier",
        "data_type": "ExternalId"
    },
    "P5088": {
        "label": "Internet Encyclopedia of Philosophy ID",
        "description": "identifier for an article in the Internet Encyclopedia of Philosophy",
        "data_type": "ExternalId"
    },
    "P1417": {
        "label": "Encyclopædia Britannica Online ID",
        "description": "identifer for an article in the online version of Encyclopædia Britannica",
        "data_type": "ExternalId"
    },
    "P4228": {
        "label": "Encyclopedia of Australian Science ID",
        "description": "identifier for a person or organisation in the Encyclopedia of Australian Science, an online compilation of biographical data about Australian scientists and their organisations",
        "data_type": "ExternalId"
    },
    "P3365": {
        "label": "Treccani ID",
        "description": "identifier for the Treccani website",
        "data_type": "ExternalId"
    },
    "P1453": {
        "label": "catholic.ru ID",
        "description": "identifier on the site catholic.ru",
        "data_type": "ExternalId"
    },
    "P1807": {
        "label": "Great Aragonese Encyclopedia ID",
        "description": "identifier in the Spanish language, Aragon-themed, Great Aragonese Encyclopedia (GEA)",
        "data_type": "ExternalId"
    },
    "P3123": {
        "label": "Stanford Encyclopedia of Philosophy ID",
        "description": "identifier of a topic in the online Stanford Encyclopedia of Philosophy",
        "data_type": "ExternalId"
    },
    "P5567": {
        "label": "National Research Institute for Cultural Properties artist ID",
        "description": "biography identifier from the Tokyo Research Institute for Cultural Properties in Japan",
        "data_type": "ExternalId"
    },
    "P3710": {
        "label": "Jewish Encyclopedia Daat ID",
        "description": "identifier for an entry at Jewish Encyclopedia Daat",
        "data_type": "ExternalId"
    },
    "P4223": {
        "label": "Treccani's Enciclopedia Italiana ID",
        "description": "identifier for the Enciclopedia Italiana on Treccani website",
        "data_type": "ExternalId"
    },
    "P4351": {
        "label": "Cravo Albin artist ID",
        "description": "identifier for an artist or group, in the Cravo Albin Dictionary of Brazilian Popular Music",
        "data_type": "ExternalId"
    },
    "P6706": {
        "label": "De Agostini ID",
        "description": "identifier for a subject on the Enciclopedia De Agostini online",
        "data_type": "ExternalId"
    },
    "P3218": {
        "label": "Auñamendi ID",
        "description": "identifier of an item in Auñamendi Encyclopaedia",
        "data_type": "ExternalId"
    },
    "P5557": {
        "label": "Interbibly author ID",
        "description": "identifier for an author on the Interbibly website",
        "data_type": "ExternalId"
    },
    "P5640": {
        "label": "Read NZ Te Pou Muramura writer ID",
        "description": "identifier for a writer on the Read NZ Te Pou Muramura website",
        "data_type": "ExternalId"
    },
    "P5693": {
        "label": "Fandango film ID",
        "description": "identifier for a film on Fandango.com",
        "data_type": "ExternalId"
    },
    "P5748": {
        "label": "Basisklassifikation",
        "description": "corresponding class in the Basisklassifikation library classification",
        "data_type": "ExternalId"
    },
    "P5757": {
        "label": "DAV hut ID",
        "description": "identifier for a hut on the German Alpine Club website",
        "data_type": "ExternalId"
    },
    "P5774": {
        "label": "Historic Hotels of Europe ID",
        "description": "identifier for an hotel on the Historic Hotels of Europe website",
        "data_type": "ExternalId"
    },
    "P5634": {
        "label": "Theatrum ID",
        "description": "identifier for an ancient theatre or odeon on the theatrum.de website",
        "data_type": "ExternalId"
    },
    "P5599": {
        "label": "Inventory of Quebec's Places of Worship ID",
        "description": "identifier of a place of worship in Quebec, built before 1975",
        "data_type": "ExternalId"
    },
    "P5696": {
        "label": "Indian Railways station code",
        "description": "code to identify a railway station operated by Indian Railways (this should NOT be used on items for metro stations; use P296 in those instances)",
        "data_type": "ExternalId"
    },
    "P5561": {
        "label": "Institut culturel de Bretagne ID",
        "description": "identifier for a writer on the Institut culturel de Bretagne website",
        "data_type": "ExternalId"
    },
    "P5609": {
        "label": "Split This Rock poet ID",
        "description": "identifier for a poet on the Split This Rock website, in its The Quarry database",
        "data_type": "ExternalId"
    },
    "P5578": {
        "label": "Indian census area code (2011)",
        "description": "code used for an area for calculations in the 2011 census of India",
        "data_type": "ExternalId"
    },
    "P5570": {
        "label": "NooSFere author ID",
        "description": "identifier for a person in the NooSFere database",
        "data_type": "ExternalId"
    },
    "P5758": {
        "label": "PZS hut ID",
        "description": "identifier for a hut on the Alpine Association of Slovenia website",
        "data_type": "ExternalId"
    },
    "P5571": {
        "label": "NooSFere book ID",
        "description": "identifier for a book in the NooSFere database",
        "data_type": "ExternalId"
    },
    "P5603": {
        "label": "Trove newspaper ID",
        "description": "identifier for a newspaper, in the National Library of Australia's Trove database",
        "data_type": "ExternalId"
    },
    "P5643": {
        "label": "RSL author ID",
        "description": "identifier for an author on the Royal Society of Literature website",
        "data_type": "ExternalId"
    },
    "P5625": {
        "label": "Vietnamese reading",
        "description": "reading of Han character in Quốc Ngữ",
        "data_type": "String"
    },
    "P5627": {
        "label": "WSO Online ID",
        "description": "identifier for an entry in the online version of Polish orthographic dictionary (pl Wielki słownik ortograficzny)",
        "data_type": "ExternalId"
    },
    "P5641": {
        "label": "PRELIB person ID",
        "description": "identifier for a person on the Projet de recherche en littérature de langue bretonne website",
        "data_type": "ExternalId"
    },
    "P5829": {
        "label": "Television Academy Foundation show ID",
        "description": "identifier for a television show discussed on the Television Academy Foundation's The Interviews website",
        "data_type": "ExternalId"
    },
    "P5700": {
        "label": "La Lorraine des écrivains writer ID",
        "description": "identifier for a writer on the Lorraine des écrivains website",
        "data_type": "ExternalId"
    },
    "P5747": {
        "label": "CWA writer ID",
        "description": "identifier for a writer on the Crime Writers' Association website",
        "data_type": "ExternalId"
    },
    "P5745": {
        "label": "Pacific Coast Architecture Database person ID",
        "description": "identifier for people involved in architecture on the West Coast of the United States",
        "data_type": "ExternalId"
    },
    "P5759": {
        "label": "ÖAV hut ID",
        "description": "identifier for a hut on the Austrian Alpine Club website",
        "data_type": "ExternalId"
    },
    "P5834": {
        "label": "Leading Hotels of the World hotel ID",
        "description": "identifier for an hotel on the Leading Hotels of the World website",
        "data_type": "ExternalId"
    },
    "P5864": {
        "label": "Enciclovida ID",
        "description": "identification in Enciclovida, an Encyclopedia of plants, fungi and animals of Mexico",
        "data_type": "ExternalId"
    },
    "P5890": {
        "label": "Preferred Hotels & Resorts hotel ID",
        "description": "identifier for an hotel on the Preferred Hotels & Resorts website",
        "data_type": "ExternalId"
    },
    "P5876": {
        "label": "SJPXVII ID",
        "description": "identifier for an entry in the electronic dictionary of the Polish language of the 17th and 18th centuries",
        "data_type": "ExternalId"
    },
    "P5877": {
        "label": "SPXVI ID",
        "description": "identifier for an entry in the Polish dictionary of the 16th century (pl: Słownik polszczyzny XVI wieku)",
        "data_type": "ExternalId"
    },
    "P5904": {
        "label": "ASCE Historical Civil Engineering Landmark ID",
        "description": "identifier for a landmark on the American Society of Civil Engineers website",
        "data_type": "ExternalId"
    },
    "P5598": {
        "label": "Places of Worship Inventory ID",
        "description": "identifier of a place of worship in Ontario",
        "data_type": "ExternalId"
    },
    "P5752": {
        "label": "Syndikat ID",
        "description": "identifier for an author on the Syndikat website",
        "data_type": "ExternalId"
    },
    "P5633": {
        "label": "Amphi-Theatrum ID",
        "description": "identifier for an amphitheatre on the amphi-theatrum.de website",
        "data_type": "ExternalId"
    },
    "P5652": {
        "label": "Carthalia ID",
        "description": "ID of a page at \"Carthalia - Theatres on Postcards\"",
        "data_type": "ExternalId"
    },
    "P5666": {
        "label": "Quais du polar writer ID",
        "description": "identifier for a writer on the Quais du polar website",
        "data_type": "ExternalId"
    },
    "P5720": {
        "label": "WSJP ID",
        "description": "identifier for an entry in the online dictionary of Polish language",
        "data_type": "ExternalId"
    },
    "P5782": {
        "label": "DBUnico MIBACT ID",
        "description": "database identifier from the Italian Ministry MIBACT (DBUnico 2.0)",
        "data_type": "ExternalId"
    },
    "P5793": {
        "label": "Dobry słownik ID",
        "description": "identifier for an entry in the online dictionary of Polish language (pl Dobry słownik)",
        "data_type": "ExternalId"
    },
    "P5809": {
        "label": "Alternativa Teatral place ID",
        "data_type": "ExternalId"
    },
    "P5836": {
        "label": "Relais & Châteaux ID",
        "description": "identifier for an hotel or restaurant on the Relais & Châteaux website",
        "data_type": "ExternalId"
    },
    "P423": {
        "label": "shooting handedness",
        "description": "whether the hockey player passes or shoots left- or right-handed",
        "data_type": "WikibaseItem"
    },
    "P3530": {
        "label": "par",
        "description": "predetermined number of strokes a 0-handicap golfer should require to complete a hole, a round or a tournament",
        "data_type": "Quantity"
    },
    "P9225": {
        "label": "triples hit",
        "description": "number of triples hit by a baseball player in a career or game(s)",
        "data_type": "Quantity"
    },
    "P9184": {
        "label": "number of hits",
        "description": "number of hits earned in a baseball career or game(s)",
        "data_type": "Quantity"
    },
    "P9188": {
        "label": "bases on balls",
        "description": "number of walks in a baseball career or game(s)",
        "data_type": "Quantity"
    },
    "P9190": {
        "label": "runs batted in",
        "description": "number of RBIs attributed to a baseball player in a career or game(s)",
        "data_type": "Quantity"
    },
    "P9217": {
        "label": "stolen bases",
        "description": "number of bases stolen by a baseball player in a career or game(s)",
        "data_type": "Quantity"
    },
    "P9338": {
        "label": "NFL.com ID (new scheme)",
        "description": "identifier for American football players at NFL.com",
        "data_type": "ExternalId"
    },
    "P9180": {
        "label": "number of at bats",
        "description": "number of batting attempts in a baseball career or game(s)",
        "data_type": "Quantity"
    },
    "P3840": {
        "label": "slope rating",
        "description": "the slope rating of a golf course is a measure of its difficulty for bogey golfers",
        "data_type": "Quantity"
    },
    "P9220": {
        "label": "doubles hit",
        "description": "number of doubles hit by a baseball player in a career or game(s)",
        "data_type": "Quantity"
    },
    "P8025": {
        "label": "VGMRips system ID",
        "description": "identifier of a video game system in the VGMRips database",
        "data_type": "ExternalId"
    },
    "P8878": {
        "label": "HLTV player ID",
        "description": "identifiant for a player in HLTV",
        "data_type": "ExternalId"
    },
    "P8267": {
        "label": "Mod DB mod ID",
        "description": "identifier for a mod on Mod D",
        "data_type": "ExternalId"
    },
    "P8268": {
        "label": "Ubisoft Store game ID",
        "description": "identifier for a game on Ubisoft Store site",
        "data_type": "ExternalId"
    },
    "P8056": {
        "label": "VGMRips composer ID",
        "description": "identifier of a video game composer in the VGMRips database",
        "data_type": "ExternalId"
    },
    "P8646": {
        "label": "expansion of",
        "description": "expansion pack of a game (video game, tabletop or board game)",
        "data_type": "WikibaseItem"
    },
    "P9297": {
        "label": "TapTap application ID",
        "description": "identifier for an application available from the tap distribution platform",
        "data_type": "ExternalId"
    },
    "P8018": {
        "label": "IGCD game ID",
        "description": "identifier of a video game in the Internet Game Cars Database",
        "data_type": "ExternalId"
    },
    "P8727": {
        "label": "Minecraft UUID",
        "description": "the UUID of a Minecraft account [lowercase with hyphens]",
        "data_type": "ExternalId"
    },
    "P8968": {
        "label": "OpenReview.net submission ID",
        "description": "identifier for submitted article to the OpenReview.net system",
        "data_type": "ExternalId"
    },
    "P8099": {
        "label": "I-Revues ID",
        "description": "identifier for a scholarly article in the I-Revues bibliographic database",
        "data_type": "ExternalId"
    },
    "P8101": {
        "label": "MUSE article ID",
        "description": "identifier for a scholarly article in the Project MUSE bibliographic database",
        "data_type": "ExternalId"
    },
    "P8875": {
        "label": "indexed in bibliographic review",
        "description": "bibliographic review(s) and/or database(s) which index this academic journal",
        "data_type": "WikibaseItem"
    },
    "P7864": {
        "label": "HAL article ID",
        "data_type": "ExternalId"
    },
    "P8758": {
        "label": "Persée article ID",
        "description": "identifier for an article on Persée",
        "data_type": "ExternalId"
    },
    "P1574": {
        "label": "exemplar of",
        "description": "property for manuscripts, autographs, incunabula, distinct printed copies",
        "data_type": "WikibaseItem"
    },
    "P9038": {
        "label": "Calaméo ID",
        "description": "unique identifier for publications on online publishing platform Calaméo",
        "data_type": "ExternalId"
    },
    "P3768": {
        "label": "Medieval Libraries of Great Britain ID",
        "description": "identifier for books known to have been in British libraries during the Middle Ages",
        "data_type": "ExternalId"
    },
    "P137": {
        "label": "operator",
        "description": "person, profession, or organization that operates the equipment, facility, or service",
        "data_type": "WikibaseItem"
    },
    "P2008": {
        "label": "IPNI publication ID",
        "description": "identifier for a publication in the International Plant Names Index",
        "data_type": "ExternalId"
    },
    "P2679": {
        "label": "author of foreword",
        "description": "person who wrote the preface, foreword, or introduction of the book but who isn't an author of the rest of the book",
        "data_type": "WikibaseItem"
    },
    "P2680": {
        "label": "author of afterword",
        "description": "person who wrote the postface, afterword, or conclusion of the book but who isn't an author of the rest of the book",
        "data_type": "WikibaseItem"
    },
    "P1535": {
        "label": "used by",
        "description": "item or concept that makes use of the subject (use sub-properties when appropriate)",
        "data_type": "WikibaseItem"
    },
    "P8109": {
        "label": "Pascal et Francis ID",
        "description": "identifier for a scholarly publication in the Pascal and Francis bibliographic databases",
        "data_type": "ExternalId"
    },
    "P1830": {
        "label": "owner of",
        "description": "entities owned by the subject",
        "data_type": "WikibaseItem"
    },
    "P1577": {
        "label": "Gregory-Aland-Number",
        "description": "identifier for Greek manuscripts of the New Testament",
        "data_type": "ExternalId"
    },
    "P2007": {
        "label": "ZooBank publication ID",
        "description": "identifier for a publication at ZooBank",
        "data_type": "ExternalId"
    },
    "P126": {
        "label": "maintained by",
        "description": "person or organization in charge of keeping the subject (for instance an infrastructure) in functioning order",
        "data_type": "WikibaseItem"
    },
    "P3950": {
        "label": "narrower external class",
        "description": "more specific subclass included in other ontology but not in Wikidata",
        "data_type": "Url"
    },
    "P7916": {
        "label": "Mendeley publication ID",
        "description": "identifier for a scientific publication on the Mendeley.com website",
        "data_type": "ExternalId"
    },
    "P4752": {
        "label": "Manus Online manuscript ID",
        "description": "identifier for a manuscript held by an Italian library",
        "data_type": "ExternalId"
    },
    "P9015": {
        "label": "Medieval Manuscripts in Oxford Libraries manuscript ID",
        "description": "identifier of a manuscript in the online database Medieval Manuscripts in Oxford Libraries",
        "data_type": "ExternalId"
    },
    "P1833": {
        "label": "number of registered users/contributors",
        "description": "number of registered users on a website",
        "data_type": "Quantity"
    },
    "P121": {
        "label": "item operated",
        "description": "equipment, installation or service operated by the subject",
        "data_type": "WikibaseItem"
    },
    "P4700": {
        "label": "Cairn publication ID",
        "description": "identifier for a journal or an article in Cairn, an online library of French-language scholarly journals",
        "data_type": "ExternalId"
    },
    "P366": {
        "label": "use",
        "description": "main use of the subject (includes current and former usage)",
        "data_type": "WikibaseItem"
    },
    "P8695": {
        "label": "Slovník českých nakladatelství 1848-1949 publishing house ID",
        "description": "identifier of publishing houses in Czech database of publishing houses (1848-1949)",
        "data_type": "ExternalId"
    },
    "P7989": {
        "label": "Mirabile manuscript ID",
        "description": "identifier for medieval manuscripts",
        "data_type": "ExternalId"
    },
    "P3702": {
        "label": "Catalogue of Illuminated Manuscripts ID",
        "description": "manuscript identifier in the British Library's Catalogue of Illuminated Manuscripts",
        "data_type": "ExternalId"
    },
    "P3181": {
        "label": "OpenCitations bibliographic resource ID",
        "description": "identifier for an article or publication, in the OpenCitations Corpus",
        "data_type": "ExternalId"
    },
    "P7409": {
        "label": "ERIC publication ID",
        "description": "ID of an article or book in ERIC",
        "data_type": "ExternalId"
    },
    "P2283": {
        "label": "uses",
        "description": "item or concept used by the subject or in the operation (see also instrument [P1303] and armament [P520])",
        "data_type": "WikibaseItem"
    },
    "P1709": {
        "label": "equivalent class",
        "description": "equivalent class in other ontologies (use property URI)",
        "data_type": "Url"
    },
    "P1104": {
        "label": "number of pages",
        "description": "number of pages in an edition of a written work. \"Roman-numbered page\" (Q56761382) can be used as unit for such pages.",
        "data_type": "Quantity"
    },
    "P6088": {
        "label": "beer bitterness",
        "description": "bitterness of beer in International Bittering Units scale",
        "data_type": "Quantity"
    },
    "P1034": {
        "label": "main food source",
        "description": "species, genus or family that an organism depends on for nutrition",
        "data_type": "WikibaseItem"
    },
    "P2371": {
        "label": "FAO risk status",
        "description": "UN Food and Agriculture Organization designation of status for a domesticated breed",
        "data_type": "WikibaseItem"
    },
    "P1821": {
        "label": "Open Food Facts food category ID",
        "description": "represents a food category on Open Food Facts",
        "data_type": "ExternalId"
    },
    "P9034": {
        "label": "BG School ID",
        "description": "Official identifier for schools and kindergartens used by Ministry of Education, Youth and Science of Bulgaria",
        "data_type": "ExternalId"
    },
    "P8937": {
        "label": "theses.cz ID",
        "description": "identifier of thesis in theses.cz",
        "data_type": "ExternalId"
    },
    "P3090": {
        "label": "flight number",
        "description": "identifier for a specific flight",
        "data_type": "String"
    },
    "P1873": {
        "label": "maximum number of players",
        "description": "maximum numbers of players of a game",
        "data_type": "Quantity"
    },
    "P4543": {
        "label": "has listed ingredient",
        "description": "substance that's listed as ingredient on the packaging of the product; use P1545 to qualify the order of the listed ingredients",
        "data_type": "WikibaseItem"
    },
    "P5703": {
        "label": "airline accounting code",
        "description": "number, referenced by IATA and unique among all the airlines, used to identify the airline in various accounting activities such as ticketing",
        "data_type": "String"
    },
    "P6322": {
        "label": "vehicle identification number",
        "description": "identifier of an individual vehicle. Format: 17 alphanumeric characters excluding letters I, O, Q",
        "data_type": "ExternalId"
    },
    "P799": {
        "label": "Air Ministry specification ID",
        "description": "identifier for an aircraft specification issued by the United Kingdom Air Ministry",
        "data_type": "String"
    },
    "P1872": {
        "label": "minimum number of players",
        "description": "minimum numbers of players of a game",
        "data_type": "Quantity"
    },
    "P8675": {
        "label": "BoardGameGeek family ID",
        "description": "identifier for a game series on the BoardGameGeek website",
        "data_type": "ExternalId"
    },
    "P4242": {
        "label": "maximal rate of climb",
        "description": "maximum rate of positive altitude change with respect to time for an aircraft",
        "data_type": "Quantity"
    },
    "P529": {
        "label": "runway",
        "description": "name (direction) of runway at airport/airfield/airstrip",
        "data_type": "String"
    },
    "P6089": {
        "label": "beer color",
        "description": "color of beer in Standard Reference Method",
        "data_type": "Quantity"
    },
    "P1622": {
        "label": "driving side",
        "description": "side of the road that vehicles drive on in a given jurisdiction",
        "data_type": "WikibaseItem"
    },
    "P4851": {
        "label": "maximum food additive use level",
        "description": "maximum allowed level of food additive permitted in a quantity of food",
        "data_type": "Quantity"
    },
    "P5167": {
        "label": "vehicles per thousand people",
        "description": "number of vehicles per 1000 people. Use qualifier determination method (P459) to describe how it's done.",
        "data_type": "Quantity"
    },
    "P6707": {
        "label": "approach angle",
        "description": "angle of a ramp on which a vehicle can climb, use degree (Q28390) as unit",
        "data_type": "Quantity"
    },
    "P7667": {
        "label": "Hellenic Civil Aviation Authority airport code",
        "data_type": "ExternalId"
    },
    "P8222": {
        "label": "rating.chgk.info ID",
        "description": "identifier of player at rating.chgk.info",
        "data_type": "ExternalId"
    },
    "P8431": {
        "label": "course",
        "description": "course type of a food dish (appetizer, dessert, drink, main course, salad, side dish)",
        "data_type": "WikibaseItem"
    },
    "P4030": {
        "label": "PLU code",
        "description": "identification numbers affixed to fruits and vegetables for sale, used in checkout and inventory",
        "data_type": "ExternalId"
    },
    "P4850": {
        "label": "permitted food additive",
        "description": "food additive which is permitted to be present within a food item",
        "data_type": "WikibaseItem"
    },
    "P7971": {
        "label": "food energy",
        "description": "amount of chemical energy that animals (including humans) derive from this item",
        "data_type": "Quantity"
    },
    "P16": {
        "label": "highway system",
        "description": "system (or specific country specific road type) of which the highway is a part",
        "data_type": "WikibaseItem"
    },
    "P868": {
        "label": "foods traditionally associated",
        "description": "foods usually during the ceremony or associated with a certain settlement",
        "data_type": "WikibaseItem"
    },
    "P4151": {
        "label": "game mechanics",
        "description": "constructs of rules or methods designed for interaction with the game state",
        "data_type": "WikibaseItem"
    },
    "P6134": {
        "label": "Thaifilm ID",
        "data_type": "ExternalId"
    },
    "P5965": {
        "label": "LUBW Protected Area No",
        "description": "unique number of a protected area in the German state of Baden-Württemberg",
        "data_type": "ExternalId"
    },
    "P5966": {
        "label": "SAGE journal ID",
        "description": "ID for a journal published by SAGE",
        "data_type": "ExternalId"
    },
    "P6141": {
        "label": "À nos grands hommes ID",
        "description": "identifier for a statue on 'À nos grands hommes'",
        "data_type": "ExternalId"
    },
    "P5963": {
        "label": "Elsevier journal ID",
        "description": "ID for a journal published by Elsevier",
        "data_type": "ExternalId"
    },
    "P5939": {
        "label": "Middelnederlandsch Woordenboek GTB ID",
        "description": "ID of entry in Middelnederlandsch Woordenboek on Geïntegreerde Taalbank",
        "data_type": "ExternalId"
    },
    "P5945": {
        "label": "VicFlora ID",
        "description": "identifier for a plant taxon, in the Australian 'Flora of Victoria' database",
        "data_type": "ExternalId"
    },
    "P6007": {
        "label": "Salons ID",
        "description": "identifier for an artwork, in the Paris Salons database",
        "data_type": "ExternalId"
    },
    "P6155": {
        "label": "Sotheby's Museum Network ID",
        "description": "identifier for a museum on the Sotheby's Museum Network website",
        "data_type": "ExternalId"
    },
    "P6173": {
        "label": "Bitraga author ID",
        "description": "identifier for an author in the Digital Library of Galician Literary Translation database (Bitraga) of the University of Vigo",
        "data_type": "ExternalId"
    },
    "P6158": {
        "label": "ALCA author ID",
        "description": "identifier for an author on the ALCA Nouvelle-Aquitaine website",
        "data_type": "ExternalId"
    },
    "P6156": {
        "label": "MNAV artist ID",
        "description": "identifier assigned to an artist by the National Museum of Visual Arts of Uruguay",
        "data_type": "ExternalId"
    },
    "P6028": {
        "label": "Hypericum MySpecies ID",
        "description": "identifier for a taxon in the database at Hypericum MySpecies",
        "data_type": "ExternalId"
    },
    "P6182": {
        "label": "GameFAQs company ID",
        "description": "identifier of a company in the GameFAQs database",
        "data_type": "ExternalId"
    },
    "P6189": {
        "label": "Sabinet journal ID",
        "description": "identifier for an academic journal on the Sabinet website",
        "data_type": "ExternalId"
    },
    "P5985": {
        "label": "Literaturaren Zubitegia ID",
        "description": "Armiarma basque literature on-line database identifier",
        "data_type": "ExternalId"
    },
    "P5983": {
        "label": "Springer journal ID",
        "description": "Springer journal number",
        "data_type": "ExternalId"
    },
    "P6183": {
        "label": "African Journals Online  journal ID",
        "description": "identifier for an academic journal on the African Journals Online website",
        "data_type": "ExternalId"
    },
    "P5990": {
        "label": "Movies Anywhere ID",
        "description": "identifier for a film on the Movies Anywhere service",
        "data_type": "ExternalId"
    },
    "P6215": {
        "label": "Vesti.kz person ID",
        "description": "identifier for a athlete at the Vesti.kz",
        "data_type": "ExternalId"
    },
    "P6006": {
        "label": "Museum Universe Data File ID",
        "description": "IMLS identifier for a museum, in the United States' 'Museum Universe Data File' (MUDF)",
        "data_type": "ExternalId"
    },
    "P6081": {
        "label": "RIA Novosti reference",
        "description": "background information on historical and current events, biographies of famous people",
        "data_type": "ExternalId"
    },
    "P6227": {
        "label": "Flora of Wisconsin ID",
        "description": "identifier for a taxon on the Flora of Wisconsin website",
        "data_type": "ExternalId"
    },
    "P6240": {
        "label": "MMLO ID",
        "description": "identifier in the Biographical Dictionary of Mintmasters, Wardens, Coin Engravers and Medallists",
        "data_type": "ExternalId"
    },
    "P5937": {
        "label": "Oudnederlands Woordenboek GTB ID",
        "description": "ID of entry in Oudnederlands Woordenboek on Geïntegreerde Taalbank",
        "data_type": "ExternalId"
    },
    "P6180": {
        "label": "Dimensions Source ID",
        "description": "unique identifier for a journal in the Dimensions database",
        "data_type": "ExternalId"
    },
    "P6133": {
        "label": "Siamzone film ID",
        "description": "identifier for a movie at siamzone.com",
        "data_type": "ExternalId"
    },
    "P6070": {
        "label": "BirdLife International IBA ID",
        "description": "identifier for an area officially-designated an 'Important Bird & Biodiversity Area' (IBA) by BirdLife International",
        "data_type": "ExternalId"
    },
    "P5938": {
        "label": "Vroegmiddelnederlands Woordenboek GTB ID",
        "description": "ID of entry in Vroegmiddelnederlands Woordenboek on Geïntegreerde Taalbank",
        "data_type": "ExternalId"
    },
    "P6219": {
        "label": "TASS reference",
        "description": "background information on historical and current events, locations, biographies of famous people",
        "data_type": "ExternalId"
    },
    "P6004": {
        "label": "Brasiliana Iconográfica ID",
        "description": "identifier for an artwork, in the \"Brasiliana Infográfica\" database",
        "data_type": "ExternalId"
    },
    "P6120": {
        "label": "Argentinean NCAA Airport code",
        "description": "airport code issued by the Argentinean National Civil Aviation Administration. Not to be confused with FAA (USA) code",
        "data_type": "ExternalId"
    },
    "P6194": {
        "label": "Austrian Biographical Encylopedia ID",
        "description": "іdentifier for people in the Austrian Biographical Encylopedia edited by the Austrian Academy of Science",
        "data_type": "ExternalId"
    },
    "P6127": {
        "label": "Letterboxd film ID",
        "description": "identifier for a film on the website Letterboxd",
        "data_type": "ExternalId"
    },
    "P6126": {
        "label": "Santiebeati ID",
        "description": "identifier of a person from the website Santiebeati.it",
        "data_type": "ExternalId"
    },
    "P5953": {
        "label": "NT Flora ID",
        "description": "identifier for a plant taxon, in the Northern Territory (Australia) 'NT Flora' database",
        "data_type": "ExternalId"
    },
    "P4384": {
        "label": "DAMIT asteroid ID",
        "description": "identifier for an asteroid in the Database of Asteroid Models from Inversion Techniques (DAMIT)",
        "data_type": "ExternalId"
    },
    "P2956": {
        "label": "NAIF ID",
        "description": "integer code assigned by NASA's Navigation and Ancillary Information Facility to solar system objects, reference frames, and instruments that it tracks",
        "data_type": "ExternalId"
    },
    "P7767": {
        "label": "serving temperature",
        "description": "serving temperature of a food dish",
        "data_type": "WikibaseItem"
    },
    "P705": {
        "label": "Ensembl protein ID",
        "description": "identifier for a protein issued by Ensembl database",
        "data_type": "ExternalId"
    },
    "P2167": {
        "label": "UNSPSC Code",
        "description": "identifier for taxonomy of products and services for use in eCommerce",
        "data_type": "ExternalId"
    },
    "P5519": {
        "label": "Cangjie input",
        "description": "the cangjie input code for the character",
        "data_type": "String"
    },
    "P717": {
        "label": "Minor Planet Center observatory code",
        "description": "observational data for minor planets (asteroids) and comets",
        "data_type": "ExternalId"
    },
    "P5650": {
        "label": "Exoplanet Data Explorer exoplanet ID",
        "description": "identifier for an exoplanet on the Exoplanet Data Explorer website",
        "data_type": "ExternalId"
    },
    "P5667": {
        "label": "NASA Exoplanet Archive exoplanet ID",
        "description": "identifier for an exoplanet on the NASA Exoplanet Archive website",
        "data_type": "ExternalId"
    },
    "P5518": {
        "label": "four-corner method",
        "description": "The four-corner code(s) for the character",
        "data_type": "String"
    },
    "P8777": {
        "label": "NCAA Statistics coach ID",
        "description": "identifier for a college sports coach on stats.ncaa.org",
        "data_type": "ExternalId"
    },
    "P5736": {
        "label": "Minor Planet Center body ID",
        "description": "identifier for an astronomical body in the Minor Planet Center database",
        "data_type": "ExternalId"
    },
    "P3916": {
        "label": "UNESCO Thesaurus ID",
        "description": "identifier for a concept in the UNESCO Thesaurus",
        "data_type": "ExternalId"
    },
    "P2444": {
        "label": "homoglyph",
        "description": "letter, number, or character which appears similar to another",
        "data_type": "WikibaseItem"
    },
    "P2983": {
        "label": "UNDP country code",
        "description": "country code by the United Nations Development Programme (UNDP)",
        "data_type": "ExternalId"
    },
    "P2082": {
        "label": "M.49 code",
        "description": "United Nations M.49 code for the subject item",
        "data_type": "ExternalId"
    },
    "P5425": {
        "label": "Han character in this lexeme",
        "description": "Han character(s) this lexeme consists of",
        "data_type": "WikibaseItem"
    },
    "P5738": {
        "label": "Open Exoplanet Catalogue exoplanet ID",
        "description": "identifier for an exoplanet on the Open Exoplanet Catalogue website",
        "data_type": "ExternalId"
    },
    "P5765": {
        "label": "The Park Database ID",
        "description": "identifier for an amusement park on the Park Database website",
        "data_type": "ExternalId"
    },
    "P4212": {
        "label": "PACTOLS thesaurus ID",
        "description": "identifier in the PACTOLS archaeology thesaurus by FRANTIQ",
        "data_type": "ExternalId"
    },
    "P5205": {
        "label": "stroke count",
        "description": "number of strokes in a Chinese, Japanese, Korean or Vietnamese character",
        "data_type": "Quantity"
    },
    "P716": {
        "label": "JPL Small-Body Database ID",
        "description": "identifier in the JPL Small-Body Database",
        "data_type": "ExternalId"
    },
    "P4095": {
        "label": "Principal Galaxies Catalogue ID",
        "description": "identifier for an astronomical object in the Principal Galaxies Catalogue",
        "data_type": "ExternalId"
    },
    "P8257": {
        "label": "Wikilengua article",
        "description": "name of an article on Wikilengua wiki",
        "data_type": "ExternalId"
    },
    "P5883": {
        "label": "Tourisme & Handicap ID",
        "description": "identifier for a place on the Tourisme & Handicap website",
        "data_type": "ExternalId"
    },
    "P4466": {
        "label": "Unified Astronomy Thesaurus ID",
        "description": "identifier for a term in the Unified Astronomy Thesaurus (UAT)",
        "data_type": "ExternalId"
    },
    "P4078": {
        "label": "SKIP code",
        "description": "kanji classification according to the System of Kanji Indexing by Patterns",
        "data_type": "String"
    },
    "P2824": {
        "label": "Gazetteer of Planetary Nomenclature ID",
        "description": "identifier of a feature on an astronomical body in the Gazetteer of Planetary Nomenclature",
        "data_type": "ExternalId"
    },
    "P238": {
        "label": "IATA airport code",
        "description": "three-letter identifier for designating airports or cities (for airlines, see P229)",
        "data_type": "String"
    },
    "P5426": {
        "label": "Japanese pitch accent type",
        "description": "type of an pitch accent of a Japanese lexeme",
        "data_type": "WikibaseItem"
    },
    "P5523": {
        "label": "fanqie",
        "description": "the pronunciation of a monosyllabic Chinese character represented by two other Chinese characters",
        "data_type": "WikibaseItem"
    },
    "P5323": {
        "label": "attested in",
        "description": "document where the lemma or form of a Lexeme entity, word or name is attested",
        "data_type": "WikibaseItem"
    },
    "P5653": {
        "label": "Extrasolar Planets Encyclopaedia exoplanet ID",
        "description": "identifier for an exoplanet on the Extrasolar Planets Encyclopaedia website",
        "data_type": "ExternalId"
    },
    "P5281": {
        "label": "residual stroke count",
        "description": "(qualifier) number of residual stroke in addition to radical of Chinese character",
        "data_type": "Quantity"
    },
    "P8486": {
        "label": "DATAtourisme ID",
        "description": "identifier (RDF URI) for the subject issued by DATAtourisme database",
        "data_type": "ExternalId"
    },
    "P352": {
        "label": "UniProt protein ID",
        "description": "identifier for a protein per the UniProt database.",
        "data_type": "ExternalId"
    },
    "P3524": {
        "label": "Simple Modular Architecture Research Tool ID",
        "description": "identifier for a protein domain in the SMART database",
        "data_type": "ExternalId"
    },
    "P5277": {
        "label": "grade of kanji",
        "description": "degree of difficulty of kanji",
        "data_type": "WikibaseItem"
    },
    "P8756": {
        "label": "Sports-Reference.com college basketball box score ID",
        "description": "identifier for a college basketball game on Sports-Reference.com",
        "data_type": "ExternalId"
    },
    "P966": {
        "label": "MusicBrainz label ID",
        "description": "identifier for a label in the MusicBrainz open music encyclopedia",
        "data_type": "ExternalId"
    },
    "P2893": {
        "label": "Skype username",
        "description": "username on the Skype instant messaging service",
        "data_type": "String"
    },
    "P6465": {
        "label": "Democracy Club candidate ID",
        "description": "identifier in the Democracy Club database of political candidates",
        "data_type": "ExternalId"
    },
    "P2002": {
        "label": "Twitter username",
        "description": "this item's username on Twitter; do not include the “@” symbol",
        "data_type": "ExternalId"
    },
    "P3417": {
        "label": "Quora topic ID",
        "description": "identifier for a topic on Quora (English language version)",
        "data_type": "ExternalId"
    },
    "P8745": {
        "label": "check-in time",
        "description": "the earliest time of day at which a reserved room in a hotel, inn or lodge is available to an arriving guest",
        "data_type": "WikibaseItem"
    },
    "P8746": {
        "label": "check-out time",
        "description": "the latest time of day a guest may check out of a hotel or lodge without incurring additional cost",
        "data_type": "WikibaseItem"
    },
    "P1617": {
        "label": "BBC Things ID",
        "description": "identifier in the BBC Things database",
        "data_type": "ExternalId"
    },
    "P3308": {
        "label": "lib.reviews ID",
        "description": "ID for 'Things' (review subjects) on lib.reviews",
        "data_type": "ExternalId"
    },
    "P2851": {
        "label": "payment types accepted",
        "description": "types of payment accepted by a venue",
        "data_type": "WikibaseItem"
    },
    "P2708": {
        "label": "CQ Ranking women's race ID",
        "description": "identifier for a women's cycling race at cqranking.com",
        "data_type": "ExternalId"
    },
    "P2013": {
        "label": "Facebook ID",
        "description": "identifier for an official (preferably) Facebook person, product or organization page (everything that follows URL part 'https://www.facebook.com/')",
        "data_type": "ExternalId"
    },
    "P3789": {
        "label": "Telegram username",
        "description": "this item's username, channel or group on Telegram",
        "data_type": "ExternalId"
    },
    "P2014": {
        "label": "Museum of Modern Art work ID",
        "description": "identifier for a work held at the Museum of Modern Art (MoMA)",
        "data_type": "ExternalId"
    },
    "P3167": {
        "label": "Allbreedpedigree ID",
        "description": "identifier for a horse on Allbreedpedigree database",
        "data_type": "ExternalId"
    },
    "P3169": {
        "label": "Infochevaux ID",
        "description": "identifier for a horse on the official French Infochevaux database",
        "data_type": "ExternalId"
    },
    "P1248": {
        "label": "KulturNav-ID",
        "description": "identifier for people, places, events, etc. used by some Nordic museums",
        "data_type": "ExternalId"
    },
    "P2848": {
        "label": "Wi-Fi access",
        "description": "internet access availability through Wi-Fi",
        "data_type": "WikibaseItem"
    },
    "P4404": {
        "label": "MusicBrainz recording ID",
        "description": "identifier for a recording in the MusicBrainz open music encyclopedia",
        "data_type": "ExternalId"
    },
    "P2607": {
        "label": "BookBrainz author ID",
        "description": "identifier for a creator per the BookBrainz open book encyclopedia",
        "data_type": "ExternalId"
    },
    "P3166": {
        "label": "Webpedigrees ID",
        "description": "identifier for a horse on Webpedigrees database",
        "data_type": "ExternalId"
    },
    "P8582": {
        "label": "InciWeb ID",
        "description": "identifier for a wildfire in the InciWeb database",
        "data_type": "ExternalId"
    },
    "P3165": {
        "label": "Horsetelex ID",
        "description": "identifier for a horse on Horsetelex database",
        "data_type": "ExternalId"
    },
    "P3180": {
        "label": "Visual Novel Database ID",
        "description": "identifier at the Visual Novel Database",
        "data_type": "ExternalId"
    },
    "P2846": {
        "label": "wheelchair accessibility",
        "description": "describes wheelchair accessibility of location or event",
        "data_type": "WikibaseItem"
    },
    "P2648": {
        "label": "CQ Ranking men's race ID",
        "description": "identifier for a men's cycling race at cqranking.com",
        "data_type": "ExternalId"
    },
    "P3245": {
        "label": "OKPD code of the good or service",
        "description": "product code in the Russian classification \"OKPD\" ОК 034-2007 (КПЕС 2002)",
        "data_type": "ExternalId"
    },
    "P3250": {
        "label": "OKPD2 code of the good or service",
        "description": "product code in the Russian classification \"OKPD2\" ОК 034-2014 (КПЕС 2008)",
        "data_type": "ExternalId"
    },
    "P3266": {
        "label": "Library of Congress Format Description Document ID",
        "description": "identifier for file formats in the Library of Congress database",
        "data_type": "ExternalId"
    },
    "P8141": {
        "label": "motorsportstats.com series ID",
        "description": "identifier for motorsport series at motorsportstats.com",
        "data_type": "ExternalId"
    },
    "P2330": {
        "label": "Cycling Archives race ID",
        "description": "identifier in the Cycling Archives",
        "data_type": "ExternalId"
    },
    "P6806": {
        "label": "racing-reference race ID",
        "description": "identifier of a motor race, in the racing-reference.com database",
        "data_type": "ExternalId"
    },
    "P3217": {
        "label": "Dictionary of Swedish National Biography ID",
        "description": "entry in the Dictionary of Swedish National Biography",
        "data_type": "ExternalId"
    },
    "P4472": {
        "label": "Czech Jockey Club horse ID",
        "description": "ID of a race horse in the database of the Jockey Club of the Czech Republic",
        "data_type": "ExternalId"
    },
    "P3168": {
        "label": "Sporthorse data ID",
        "description": "identifier for a horse on Sporthorse data database",
        "data_type": "ExternalId"
    },
    "P3248": {
        "label": "OKP ID of the good or service",
        "description": "numeric ID of this group or kind of goods according to OK 005-93 \"Russian classification of products\"",
        "data_type": "ExternalId"
    },
    "P2900": {
        "label": "fax number",
        "description": "telephone number of a facsimile line",
        "data_type": "String"
    },
    "P2897": {
        "label": "Eldoblaje Movie ID",
        "description": "Identifier of dubbing in the website eldoblaje.com, a database of Spanish dubbings",
        "data_type": "ExternalId"
    },
    "P2970": {
        "label": "Kinopolis film ID",
        "description": "identifier for films in the German Kinopolis database",
        "data_type": "ExternalId"
    },
    "P3787": {
        "label": "danskefilm silent film ID",
        "description": "identifier for a silent film in danskefilm.dk",
        "data_type": "ExternalId"
    },
    "P3750": {
        "label": "KMDb documentary film ID",
        "description": "ID of a documentary film at Korean Movie Database",
        "data_type": "ExternalId"
    },
    "P3704": {
        "label": "KMDb film ID",
        "description": "ID of a film at Korean Movie Database",
        "data_type": "ExternalId"
    },
    "P4513": {
        "label": "Africultures movie ID",
        "description": "identifier for a movie at Africultures.com",
        "data_type": "ExternalId"
    },
    "P6151": {
        "label": "ANICA ID",
        "description": "identifier of a movie from the website Archiviodelcinemaitaliano.it",
        "data_type": "ExternalId"
    },
    "P2465": {
        "label": "Allcinema film ID",
        "description": "film IDs on Allcinema Movie Database",
        "data_type": "ExternalId"
    },
    "P2346": {
        "label": "Elonet movie ID",
        "description": "identifier for a movie in the Elonet database",
        "data_type": "ExternalId"
    },
    "P2636": {
        "label": "Minkultury film ID",
        "description": "identifier for a movie in the Russian Ministry of culture's public register of films",
        "data_type": "ExternalId"
    },
    "P2725": {
        "label": "GOG application ID",
        "description": "identifier for an application (or film) available from the distribution platform GOG.com",
        "data_type": "ExternalId"
    },
    "P3302": {
        "label": "Open Media Database film ID",
        "description": "Identifier for a movie at the Open Movie Database",
        "data_type": "ExternalId"
    },
    "P3107": {
        "label": "LdiF ID",
        "description": "identifier of a film (movie) in the Lexikon des Internationalen Films (Lexicon of International Films)",
        "data_type": "ExternalId"
    },
    "P1804": {
        "label": "DNF film ID",
        "description": "identifier for a film in the Danish National Filmography",
        "data_type": "ExternalId"
    },
    "P3593": {
        "label": "AFI Catalog of Feature Films ID",
        "description": "identifier of a film (movie) in the American Film Institute Catalog of Feature Film",
        "data_type": "ExternalId"
    },
    "P2755": {
        "label": "exploitation visa number",
        "description": "number of the exploitation visa of a movie in France",
        "data_type": "ExternalId"
    },
    "P3785": {
        "label": "danskefilm film ID",
        "description": "identifier for a film in danskefilm.dk",
        "data_type": "ExternalId"
    },
    "P4529": {
        "label": "Douban film ID",
        "description": "identifier for a film (movie) at the website Douban",
        "data_type": "ExternalId"
    },
    "P6196": {
        "label": "Badtaste ID",
        "description": "identifier of a movie from the website Badtaste.it",
        "data_type": "ExternalId"
    },
    "P3139": {
        "label": "SourehCinema film ID",
        "description": "identifier for a movie at the Iranian movie database SourehCinema",
        "data_type": "ExternalId"
    },
    "P5032": {
        "label": "Filmweb.pl film ID",
        "description": "identifier for a movie or series at filmweb.pl",
        "data_type": "ExternalId"
    },
    "P5987": {
        "label": "Indiancine.ma film ID",
        "description": "identifier for a film (movie) in the Indiancine.ma archive of Indian film",
        "data_type": "ExternalId"
    },
    "P2826": {
        "label": "Megogo ID",
        "description": "identifier for the film in the database Megogo",
        "data_type": "ExternalId"
    },
    "P1970": {
        "label": "MovieMeter film ID",
        "description": "identifier for a film on the website MovieMeter",
        "data_type": "ExternalId"
    },
    "P3367": {
        "label": "GECD film ID",
        "description": "identifier of a film (movie) in the German Early Cinema Database",
        "data_type": "ExternalId"
    },
    "P2883": {
        "label": "HKMDB film ID",
        "description": "identifier for a film at the HKMDB (Hong Kong Movie Database) website",
        "data_type": "ExternalId"
    },
    "P1934": {
        "label": "Animator.ru film ID",
        "description": "ID of animated film in Animator.ru database",
        "data_type": "ExternalId"
    },
    "P3129": {
        "label": "cine.gr film ID",
        "description": "identifier for a movie at the Greek movie database cine.gr",
        "data_type": "ExternalId"
    },
    "P3844": {
        "label": "Deutsche Synchronkartei film ID",
        "description": "Identifier for films in the synchronisation database Deutsche Synchronkartei",
        "data_type": "ExternalId"
    },
    "P3138": {
        "label": "OFDb film ID",
        "description": "identifier for a film (movie) in the German Online-Filmdatenbank website",
        "data_type": "ExternalId"
    },
    "P5128": {
        "label": "ICAA film catalogue ID",
        "description": "ID of a film in the film catalogue of Instituto de la Cinematografía y de las Artes Audiovisuales",
        "data_type": "ExternalId"
    },
    "P2509": {
        "label": "Movie Walker film ID",
        "description": "identifier of a film in the Movie Walker Database. Format: \"mv\" followed by digits",
        "data_type": "ExternalId"
    },
    "P3961": {
        "label": "Unifrance film ID",
        "description": "identifier of a film on unifrance.org",
        "data_type": "ExternalId"
    },
    "P3203": {
        "label": "Ciné-Ressources film ID",
        "description": "page of a movie on the website \"Ciné-Ressources\"",
        "data_type": "ExternalId"
    },
    "P2631": {
        "label": "TCM Movie Database film ID",
        "description": "identifier for a film in the Turner Classic Movies movie database",
        "data_type": "ExternalId"
    },
    "P2334": {
        "label": "Swedish Film Database film ID",
        "description": "identifier for a movie on the Swedish Film Database (SFDb)",
        "data_type": "ExternalId"
    },
    "P4947": {
        "label": "TMDb movie ID",
        "description": "identifier for a movie at The Movie Database",
        "data_type": "ExternalId"
    },
    "P3128": {
        "label": "CiNetMag film ID",
        "description": "identifier for a movie at the Iranian movie database CiNetMag",
        "data_type": "ExternalId"
    },
    "P3851": {
        "label": "Cinenacional.com movie ID",
        "description": "identifier for a movie at cinenacional.com database (from Argentina)",
        "data_type": "ExternalId"
    },
    "P5791": {
        "label": "Cineuropa film ID",
        "description": "identifier for a movie at Cineuropa",
        "data_type": "ExternalId"
    },
    "P4665": {
        "label": "CineMagia film ID",
        "description": "identifier for a movie on the Cinemagia.ro website",
        "data_type": "ExternalId"
    },
    "P5786": {
        "label": "Moviepilot.de film ID",
        "description": "identifier for films on Moviepilot.de",
        "data_type": "ExternalId"
    },
    "P4786": {
        "label": "cinematografo film ID",
        "description": "identifier for a film in cinematografo.it",
        "data_type": "ExternalId"
    },
    "P5820": {
        "label": "Film Indonesia ID",
        "description": "Indonesian Movie database ID",
        "data_type": "ExternalId"
    },
    "P4933": {
        "label": "Big Cartoon Database ID",
        "description": "identifier for the Big Cartoon Database",
        "data_type": "ExternalId"
    },
    "P6256": {
        "label": "SinemaTürk film ID",
        "description": "identifier for a movie, in the SinemaTürk database",
        "data_type": "ExternalId"
    },
    "P2529": {
        "label": "ČSFD film ID",
        "description": "identifier for a movie in the Czech film database ČSFD",
        "data_type": "ExternalId"
    },
    "P4277": {
        "label": "Daum movie ID",
        "description": "identifier for a film in the Daum database",
        "data_type": "ExternalId"
    },
    "P3868": {
        "label": "MyDramaList title ID",
        "description": "identifier for a Film and TV Series in the MyDramaList database",
        "data_type": "ExternalId"
    },
    "P4282": {
        "label": "LUMIERE film ID",
        "description": "ID of a film in the LUMIERE database of the European Audiovisual Observatory",
        "data_type": "ExternalId"
    },
    "P3077": {
        "label": "Cineplex film ID",
        "description": "identifier for films in the Cineplex database",
        "data_type": "ExternalId"
    },
    "P4783": {
        "label": "Movieplayer film ID",
        "description": "identifier for film in movieplayer.it",
        "data_type": "ExternalId"
    },
    "P5311": {
        "label": "Kino-teatr.ua film ID",
        "description": "identifier for a movie at Kino-teatr.ua",
        "data_type": "ExternalId"
    },
    "P2518": {
        "label": "Scope.dk film ID",
        "description": "identifier for a film on the website Scope.dk",
        "data_type": "ExternalId"
    },
    "P4021": {
        "label": "danskefilm animated film ID",
        "description": "identifier for a animated film in danskefilm.dk",
        "data_type": "ExternalId"
    },
    "P4780": {
        "label": "MYmovies movie ID",
        "description": "identifier for a film in MYmovies.it",
        "data_type": "ExternalId"
    },
    "P2508": {
        "label": "KINENOTE film ID",
        "description": "identifier of a film in the KINENOTE movie database",
        "data_type": "ExternalId"
    },
    "P3110": {
        "label": "ISzDb film ID",
        "description": "identifier of a film in the ISzDb (a Hungarian dub database)",
        "data_type": "ExternalId"
    },
    "P3673": {
        "label": "film-documentaire.fr film ID",
        "description": "identifier of a film (movie) in the French database film-documentaire.fr",
        "data_type": "ExternalId"
    },
    "P4606": {
        "label": "National Film Board of Canada film ID",
        "description": "film identifier used by the National Film Board of Canada website",
        "data_type": "ExternalId"
    },
    "P3141": {
        "label": "EDb film ID",
        "description": "identifier for a movie at the Israeli movie database EDb",
        "data_type": "ExternalId"
    },
    "P3135": {
        "label": "elCinema film ID",
        "description": "identifier for a movie at elCinema",
        "data_type": "ExternalId"
    },
    "P2603": {
        "label": "Kinopoisk film ID",
        "description": "identifier for a film/movie in the Russian Kinopoisk.ru database",
        "data_type": "ExternalId"
    },
    "P2400": {
        "label": "JMDb film ID",
        "description": "film IDs on Japanese Movie Database",
        "data_type": "ExternalId"
    },
    "P5338": {
        "label": "SilentEra.com film ID",
        "description": "identifier for a movie at Silentera.com",
        "data_type": "ExternalId"
    },
    "P4209": {
        "label": "danskfilmogtv title ID",
        "description": "identifier for a film or TV episode or series in danskfilmogtv.dk",
        "data_type": "ExternalId"
    },
    "P3340": {
        "label": "Kvikmyndir film ID",
        "description": "identifier of a film (movie) in the Icelandic film database kvikmyndir.is",
        "data_type": "ExternalId"
    },
    "P5384": {
        "label": "Behind The Voice Actors film ID",
        "description": "identifier of a film on the website Behind The Voice Actors",
        "data_type": "ExternalId"
    },
    "P2270": {
        "label": "Emporis building complex ID",
        "description": "identifier for a building complex, as opposed to a single building, in the Emporis architecture database",
        "data_type": "ExternalId"
    },
    "P2765": {
        "label": "blue-style.com ID",
        "description": "identifier at blue-style.com, database of skyscrapers and contemporary architecture in Japan",
        "data_type": "ExternalId"
    },
    "P9068": {
        "label": "IFPI Danmark work ID",
        "description": "identifier of a music work on the IFPI Danemark official website",
        "data_type": "ExternalId"
    },
    "P2917": {
        "label": "COAM structure ID",
        "description": "identifier of a building or structure of the city of Madrid in the Official Architects' Association of Madrid (COAM) database",
        "data_type": "ExternalId"
    },
    "P8813": {
        "label": "IMMuB album ID",
        "description": "identifier for an album in the Brazilian Musical Memory Institute database",
        "data_type": "ExternalId"
    },
    "P8549": {
        "label": "Opera Online work ID",
        "description": "identifier of a work on the Opera Online website",
        "data_type": "ExternalId"
    },
    "P8665": {
        "label": "Présence Compositrices work ID",
        "description": "identifier of a work on the website Présence Compositrices",
        "data_type": "ExternalId"
    },
    "P3371": {
        "label": "Observatoire du Patrimoine Religieux ID",
        "description": "ID for religious buildings on the OPR website",
        "data_type": "ExternalId"
    },
    "P2867": {
        "label": "Molenecho's ID",
        "description": "identifier of mills in Molenecho's, a database on windmills and watermills in Belgium",
        "data_type": "ExternalId"
    },
    "P3223": {
        "label": "Online List of Lights ID",
        "description": "identifier in the Online List of Lights database",
        "data_type": "ExternalId"
    },
    "P2489": {
        "label": "hram.by ID",
        "description": "internet page corresponding to Orthodox architectural sites in Belarus",
        "data_type": "ExternalId"
    },
    "P858": {
        "label": "ESPNscrum player ID",
        "description": "identifier for an international rugby union player, from ESPN.co.uk or ESPNscrum.com",
        "data_type": "ExternalId"
    },
    "P1217": {
        "label": "Internet Broadway Database venue ID",
        "description": "identifier for a theatre or other venue in the IBDB",
        "data_type": "ExternalId"
    },
    "P8856": {
        "label": "British Phonographic Industry ID",
        "description": "identifier of a release on the British Phonographic Industry official website",
        "data_type": "ExternalId"
    },
    "P6110": {
        "label": "AllMusic release ID",
        "description": "identifier for a release (an edition of an album) in the AllMusic database",
        "data_type": "ExternalId"
    },
    "P9083": {
        "label": "45cat 7\" release ID",
        "description": "identifier for an 7\" single or EP on 45cal",
        "data_type": "ExternalId"
    },
    "P3059": {
        "label": "Architectuurgids building ID",
        "description": "unique identifier for a building in the Dutch architectuurgids.nl database",
        "data_type": "ExternalId"
    },
    "P3563": {
        "label": "NGA lighthouse ID",
        "description": "identifier of a lighthouse in the USA's National Geospatial-Intelligence Agency database",
        "data_type": "ExternalId"
    },
    "P3562": {
        "label": "Admiralty number",
        "description": "identifier of lighthouses by United Kingdom Hydrographic Office",
        "data_type": "ExternalId"
    },
    "P455": {
        "label": "Emporis building ID",
        "description": "identifier for a building in the Emporis database (for building complexes, see P2270)",
        "data_type": "ExternalId"
    },
    "P3601": {
        "label": "MarineTraffic Lighthouse ID",
        "description": "identifier of a lighthouse or light beacon in the MarineTraffic database",
        "data_type": "ExternalId"
    },
    "P9089": {
        "label": "Songlexikon ID",
        "description": "alphabetical identifier for a song on the Songlexikon website",
        "data_type": "ExternalId"
    },
    "P2866": {
        "label": "Hollandsche Molen ID",
        "description": "identifier of mills in \"Vereniging de Hollandsche Molen\", an organization that focuses on preservation of wind- and watermills in the Netherlands.",
        "data_type": "ExternalId"
    },
    "P2863": {
        "label": "Nederlandse Molendatabase ID",
        "description": "identifier of mills in the Nederlandse Molendatabase, with records of all functioning mills, windmills and watermills, in the Netherlands",
        "data_type": "ExternalId"
    },
    "P5251": {
        "label": "Corago opera ID",
        "description": "identifier for an opera on the Corago website",
        "data_type": "ExternalId"
    },
    "P454": {
        "label": "Structurae structure ID",
        "description": "identifier for a building in the Structurae database",
        "data_type": "ExternalId"
    },
    "P2762": {
        "label": "CTBUH Skyscraper Center building complex ID",
        "description": "identifier for a building complex as used on the CTBUH's www.skyscrapercenter.com",
        "data_type": "ExternalId"
    },
    "P240": {
        "label": "FAA airport code",
        "description": "three-letter or four-letter alphanumeric code identifying United States airports",
        "data_type": "String"
    },
    "P1305": {
        "label": "CTBUH Skyscraper Center building ID",
        "description": "identifier for a building as used on the CTBUH's www.skyscrapercenter.com",
        "data_type": "ExternalId"
    },
    "P1699": {
        "label": "SkyscraperPage building ID",
        "description": "identifier for a building in the SkyscraperPage database",
        "data_type": "ExternalId"
    },
    "P6111": {
        "label": "MusicNotes product ID",
        "description": "identifier for a product in the MusicNotes store",
        "data_type": "ExternalId"
    },
    "P239": {
        "label": "ICAO airport code",
        "description": "four-character alphanumeric identifier for designating airports (for airlines, see P230)",
        "data_type": "String"
    },
    "P6122": {
        "label": "MusicNotes song ID",
        "description": "identifier for a composition in MusicNotes",
        "data_type": "ExternalId"
    },
    "P8212": {
        "label": "Gitabitan.net ID",
        "description": "identifier of Rabindra Sangeets on Gitabitan.net website",
        "data_type": "ExternalId"
    },
    "P5358": {
        "label": "Operone opera ID",
        "description": "identifier for an opera on the Operone website",
        "data_type": "ExternalId"
    },
    "P3572": {
        "label": "ESPNcricinfo playing ground ID",
        "description": "ID of cricket ground at ESPNcricinfo",
        "data_type": "ExternalId"
    },
    "P3586": {
        "label": "CricketArchive playing ground ID",
        "description": "identifier of a cricket ground at CricketArchive",
        "data_type": "ExternalId"
    },
    "P1400": {
        "label": "FCC Facility ID",
        "description": "U.S. ID number for broadcast stations",
        "data_type": "ExternalId"
    },
    "P8590": {
        "label": "Jewish Encyclopedia ID",
        "description": "article identifier in online version of 1906 Jewish Encyclopedia",
        "data_type": "ExternalId"
    },
    "P8935": {
        "label": "Encyclopedia of the History of Ukraine ID",
        "description": "ID for an entry on the official website of the Encyclopedia of the History of Ukraine",
        "data_type": "ExternalId"
    },
    "P8759": {
        "label": "Polski Słownik Judaistyczny ID",
        "description": "identifier for a topic in the Polski Słownik Judaistyczny (Poland)",
        "data_type": "ExternalId"
    },
    "P7820": {
        "label": "Punjabipedia ID",
        "description": "identifier for a topic in the Punjabipedia encyclopedia",
        "data_type": "ExternalId"
    },
    "P9106": {
        "label": "Oxford Classical Dictionary ID",
        "description": "identifier for a topic in the Oxford Classical Dictionary online",
        "data_type": "ExternalId"
    },
    "P9069": {
        "label": "International Standard Bible Encyclopedia ID",
        "description": "identifier for an article in the International Standard Bible Encyclopedia",
        "data_type": "ExternalId"
    },
    "P8603": {
        "label": "Istrapedia ID",
        "description": "identifier for an article in the Internet encyclopedia Istrapedia",
        "data_type": "ExternalId"
    },
    "P8373": {
        "label": "Encyclopedia Tatarica (tt) Online ID",
        "description": "Tatar version of online encyclopedia created by Institute of Tatar encyclopedia of Tatarstan Academy of Sciences",
        "data_type": "ExternalId"
    },
    "P9070": {
        "label": "Internet Encyclopedia of Ukraine ID",
        "description": "identifier for an article in the Internet Encyclopedia of Ukraine",
        "data_type": "ExternalId"
    },
    "P8885": {
        "label": "Namuwiki ID",
        "description": "identifier for content inside the Namuwiki",
        "data_type": "ExternalId"
    },
    "P9245": {
        "label": "Labour Australia ID",
        "description": "identifier for a person in Labour Australia",
        "data_type": "ExternalId"
    },
    "P8669": {
        "label": "Encyclopaedia Beliana ID",
        "description": "identifier of an article in Encyclopaedia Beliana",
        "data_type": "ExternalId"
    },
    "P8576": {
        "label": "Hrvatska tehnička enciklopedija ID",
        "description": "identifier for an entry in the Hrvatska tehnička enciklopedija",
        "data_type": "ExternalId"
    },
    "P9082": {
        "label": "Akademická encyklopedie českých dějin ID",
        "description": "identifier for an article in the Akademická encyklopedie českých dějin",
        "data_type": "ExternalId"
    },
    "P6870": {
        "label": "Latvian National Encyclopedia Online ID",
        "description": "identifier for an article in the website of the Latvian National Encyclopedia",
        "data_type": "ExternalId"
    },
    "P8569": {
        "label": "YIVO Encyclopedia of Jews in Eastern Europe ID",
        "description": "identifier of an article in the YIVO Encyclopedia of Jews in Eastern Europe",
        "data_type": "ExternalId"
    },
    "P8992": {
        "label": "1914-1918-Online ID",
        "description": "identifier for an article in the website 1914-1918-online. International Encyclopedia of the First World War",
        "data_type": "ExternalId"
    },
    "P7305": {
        "label": "Online PWN Encyclopedia ID",
        "description": "identifier for an entry in the online release of the Online PWN Encyclopedia",
        "data_type": "ExternalId"
    },
    "P9104": {
        "label": "Encyclopedia of Brno History object ID",
        "description": "identifier for object related to the city of Brno and its history",
        "data_type": "ExternalId"
    },
    "P8377": {
        "label": "Encyclopedia Tatarica (ru) Online ID",
        "description": "Russian version of online encyclopedia created by Institute of Tatar encyclopedia of Tatarstan Academy of Sciences",
        "data_type": "ExternalId"
    },
    "P9240": {
        "label": "Encyclopedia of Seventh-day Adventists ID",
        "description": "identifier of an entry in the Encyclopedia of Seventh-day Adventists",
        "data_type": "ExternalId"
    },
    "P8439": {
        "label": "SAH Archipedia architect ID",
        "description": "identifier for an architect on the website of the SAH Archipedia, the online encyclopedia by the Society of Architectural Historians",
        "data_type": "ExternalId"
    },
    "P8441": {
        "label": "Cistercian Biography Online ID",
        "description": "identifier for entries in the online database \"cistercian biographie online\" www.zisterzienserlexikon.de",
        "data_type": "ExternalId"
    },
    "P8896": {
        "label": "Concise Literary Encyclopedia ID",
        "description": "identifier in Concise Literary Encyclopedia",
        "data_type": "ExternalId"
    },
    "P8713": {
        "label": "Stadtwiki Dresden article",
        "description": "article in Wiki of the town Dresden, Saxony, Germany",
        "data_type": "ExternalId"
    },
    "P7655": {
        "label": "Speiderhistorisk leksikon ID",
        "description": "article on Norwegian Scout history lexicon",
        "data_type": "ExternalId"
    },
    "P8095": {
        "label": "Encyklopedia Solidarności ID",
        "description": "identifier for an article in the Encyklopedia Solidarności",
        "data_type": "ExternalId"
    },
    "P8861": {
        "label": "FINA Wiki ID",
        "description": "numeric identifier of a page in the FINA Wiki",
        "data_type": "ExternalId"
    },
    "P7012": {
        "label": "World Encyclopedia of Puppetry Arts ID",
        "description": "identifier for the online encyclopedia kept by the Union International de la Marionette",
        "data_type": "ExternalId"
    },
    "P8350": {
        "label": "Singapore Infopedia ID",
        "description": "identifier of article on Singapore Infopedia site",
        "data_type": "ExternalId"
    },
    "P8630": {
        "label": "Thomas Jefferson Encyclopedia ID",
        "description": "identifier for an entry on the Thomas Jefferson Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P8472": {
        "label": "Encyclopedie berbere keyword ID",
        "description": "ID for a keyword in the French encyclopedia about the Berbers Encyclopédie berbère",
        "data_type": "ExternalId"
    },
    "P7688": {
        "label": "Mississippi Encyclopedia ID",
        "description": "identifier for an entry on the Mississippi Encyclopedia website",
        "data_type": "ExternalId"
    },
    "P9001": {
        "label": "Literary Encyclopedia ID",
        "description": "identifier in Soviet Literary Encyclopedia (1929-1939)",
        "data_type": "ExternalId"
    },
    "P9103": {
        "label": "Enciclopedia del Museo del Prado ID",
        "description": "identifier for an article in the Enciclopedia del Museo del Prado",
        "data_type": "ExternalId"
    },
    "P9222": {
        "label": "bashenc.online ID",
        "description": "article code on the encyclopedic portal \"Bashkortostan\"",
        "data_type": "ExternalId"
    },
    "P2649": {
        "label": "CQ Ranking men's team URL",
        "description": "URL for a men's cycling team at cqranking.com",
        "data_type": "Url"
    },
    "P4321": {
        "label": "best combative classification",
        "description": "person ranked as \"best combative\" for this cycling stage or race",
        "data_type": "WikibaseItem"
    },
    "P4322": {
        "label": "best sprinter classification",
        "description": "person ranked as \"best sprinter\" for this cycling stage or race",
        "data_type": "WikibaseItem"
    },
    "P2446": {
        "label": "Transfermarkt player ID",
        "description": "identifier for an association football (soccer) player in the transfermarkt.com database",
        "data_type": "ExternalId"
    },
    "P4323": {
        "label": "young rider classification",
        "description": "person ranked as young participant for this cycling stage or race",
        "data_type": "WikibaseItem"
    },
    "P7112": {
        "label": "CUP2002.RU person ID",
        "description": "identifier for a participants of 2002 FIFA World Cup at CUP2002.RU site",
        "data_type": "ExternalId"
    },
    "P7113": {
        "label": "Euro04.Ru person ID",
        "description": "identifier for a participants of UEFA Euro 2004 at Euro04.Ru site",
        "data_type": "ExternalId"
    },
    "P7114": {
        "label": "Gazeta.Ru Euro 2012 person ID",
        "description": "identifier for a participants of UEFA Euro 2012 at Gazeta.Ru Euro 2012 site",
        "data_type": "ExternalId"
    },
    "P7115": {
        "label": "WorldCup10.Ru person ID",
        "description": "identifier for a participants of 2010 FIFA World Cup at WorldCup10.Ru site",
        "data_type": "ExternalId"
    },
    "P4958": {
        "label": "classification of the best at metas volantes",
        "description": "secondary classification for bicycle race statistics used in hispanic language countries",
        "data_type": "WikibaseItem"
    },
    "P7111": {
        "label": "08euro.Ru person ID",
        "description": "identifier for a participants of UEFA Euro 2008 at 08euro.Ru site",
        "data_type": "ExternalId"
    },
    "P4320": {
        "label": "mountains classification",
        "description": "person ranked for this cycling stage or race",
        "data_type": "WikibaseItem"
    },
    "P2912": {
        "label": "distinctive jersey",
        "description": "distinctive jersey held by a class or stage leader in a cycling competition",
        "data_type": "WikibaseItem"
    },
    "P4324": {
        "label": "combination classification",
        "description": "person ranked in \"best at combination\" for this cycling stage or race",
        "data_type": "WikibaseItem"
    },
    "P1608": {
        "label": "Dialnet book ID",
        "description": "identifier of a book in Dialnet",
        "data_type": "ExternalId"
    },
    "P1823": {
        "label": "BAnQ work ID",
        "description": "identifier for a work in the collection of Bibliothèque et Archives nationales du Québec",
        "data_type": "ExternalId"
    },
    "P8531": {
        "label": "Filmstarts title ID",
        "description": "identifier of a movie on www.filmstarts.de",
        "data_type": "ExternalId"
    },
    "P1025": {
        "label": "SUDOC editions",
        "description": "identifier in the French union catalog (see also P269)",
        "data_type": "ExternalId"
    },
    "P7777": {
        "label": "AdoroCinema film ID",
        "description": "identifier for a film in AdoroCinema database",
        "data_type": "ExternalId"
    },
    "P1044": {
        "label": "SWB editions",
        "description": "union catalog of the South-West German Library Network (SWB)",
        "data_type": "ExternalId"
    },
    "P8055": {
        "label": "Amazon.com Prime Video ID",
        "description": "identifier for a video in Amazon Prime Video (US)",
        "data_type": "ExternalId"
    },
    "P3155": {
        "label": "Runeberg book ID",
        "description": "identifier for book item in the Runeberg digital library collection",
        "data_type": "ExternalId"
    },
    "P6458": {
        "label": "Mtime movie ID",
        "description": "identifier for a movie/film at the website Mtime",
        "data_type": "ExternalId"
    },
    "P9067": {
        "label": "Hungarian Film Archive ID",
        "description": "identifier of a movie in the database of the Hungarian Film Archive",
        "data_type": "ExternalId"
    },
    "P7236": {
        "label": "Dove.org work ID",
        "description": "identifier for a movie, book, television show or album on Dove.org",
        "data_type": "ExternalId"
    },
    "P7593": {
        "label": "CinePT film ID",
        "description": "identifier for a film on the CinePT website",
        "data_type": "ExternalId"
    },
    "P8908": {
        "label": "CINE21 film ID",
        "description": "identifier for a film in the database of the film magazine CINE21",
        "data_type": "ExternalId"
    },
    "P5485": {
        "label": "SBN books ID",
        "description": "identifier for books issued by National Library Service (SBN) of Italy",
        "data_type": "ExternalId"
    },
    "P8823": {
        "label": "Xfinity Stream ID",
        "description": "identifier for a program on Xfinity Stream",
        "data_type": "ExternalId"
    },
    "P7978": {
        "label": "Decine21 film ID",
        "description": "identifier for a film or TV series on the website decine21.com",
        "data_type": "ExternalId"
    },
    "P3631": {
        "label": "Babelio work ID",
        "description": "identifier for a book on the literature website Babelio",
        "data_type": "ExternalId"
    },
    "P7285": {
        "label": "Writers Guild of America project ID",
        "description": "identifier for a film or television project in the Writers Guild of America's Find a Writer database",
        "data_type": "ExternalId"
    },
    "P7760": {
        "label": "Tubi movie ID",
        "description": "identifier for a film at video streaming site Tubi (tubitv.com)",
        "data_type": "ExternalId"
    },
    "P7522": {
        "label": "SK cinema authority ID",
        "description": "identifier for a person, organisation, topic, event, genre or work title in the SK cinema database",
        "data_type": "ExternalId"
    },
    "P6549": {
        "label": "NZ On Screen work ID",
        "description": "identifier for a film, video or TV show in the NZ On Screen database",
        "data_type": "ExternalId"
    },
    "P675": {
        "label": "Google Books ID",
        "description": "identifier for a book edition in Google Books",
        "data_type": "ExternalId"
    },
    "P8847": {
        "label": "Kinorium movie ID",
        "description": "movie identifier on kinorium.com",
        "data_type": "ExternalId"
    },
    "P5199": {
        "label": "British Library system number",
        "description": "nine digit identifier for an edition of a book held by the British Library",
        "data_type": "ExternalId"
    },
    "P7523": {
        "label": "SK cinema film ID",
        "description": "identifier for a film in the SK cinema database",
        "data_type": "ExternalId"
    },
    "P6584": {
        "label": "Scripts.com script ID",
        "description": "ID of film scripts, screenplays and transcripts from famous and not-so-famous screen writers from around the world",
        "data_type": "ExternalId"
    },
    "P7970": {
        "label": "FandangoNow ID",
        "description": "identifier for a film or other similar visual work on the FandangoNow website",
        "data_type": "ExternalId"
    },
    "P8673": {
        "label": "Naver movie ID",
        "description": "identifier for a film in the Naver database",
        "data_type": "ExternalId"
    },
    "P8674": {
        "label": "Bollywood Hungama movie ID",
        "description": "identifier for Indian film at Bollywood Hungama",
        "data_type": "ExternalId"
    },
    "P1815": {
        "label": "RSL scanned books identifier",
        "description": "Identifier of the Russian State Library",
        "data_type": "String"
    },
    "P8918": {
        "label": "Max Movie film ID",
        "description": "identifier for films in the database of the magazine Max Movie",
        "data_type": "ExternalId"
    },
    "P1143": {
        "label": "BN (Argentine) editions",
        "description": "catalogue number for books (editions) in the National Library of the Argentine Republic",
        "data_type": "ExternalId"
    },
    "P8920": {
        "label": "Movist film ID",
        "description": "identifier for a film in the Movist database",
        "data_type": "ExternalId"
    },
    "P3184": {
        "label": "Czech National Bibliography book ID",
        "description": "identifier for a book at the Czech National Library",
        "data_type": "ExternalId"
    },
    "P1292": {
        "label": "DNB editions",
        "description": "identifier in the German National Library catalogue (see also authority data P227)",
        "data_type": "ExternalId"
    },
    "P3004": {
        "label": "Galiciana work ID",
        "description": "identifier of a work in Galiciana",
        "data_type": "ExternalId"
    },
    "P1084": {
        "label": "EUL editions",
        "description": "identifier for a book (edition) in the Egyptian union catalog",
        "data_type": "ExternalId"
    },
    "P6474": {
        "label": "Swiss Films ID",
        "description": "Identifier for films on swissfilms.ch",
        "data_type": "ExternalId"
    },
    "P2623": {
        "label": "MEK ID",
        "description": "identifier of a book or translation in the Hungarian Electronic Library",
        "data_type": "ExternalId"
    },
    "P1054": {
        "label": "NDL bib ID",
        "description": "identifier of an edition in the National Diet Library",
        "data_type": "ExternalId"
    },
    "P7091": {
        "label": "Common Sense Media ID",
        "description": "identifier for a website or work of fiction on Common Sense Media",
        "data_type": "ExternalId"
    },
    "P1474": {
        "label": "Nupill Literatura Digital - Document",
        "description": "identifier for a document in Nupill Literatura Digital",
        "data_type": "ExternalId"
    },
    "P8344": {
        "label": "WikiTrek ID",
        "description": "identifier of a WikiTrek article",
        "data_type": "ExternalId"
    },
    "P2969": {
        "label": "Goodreads version/edition ID",
        "description": "identifier of a version or edition, in the GoodReads.com website",
        "data_type": "ExternalId"
    },
    "P8033": {
        "label": "Movie Review Query Engine ID",
        "description": "identifier for a movie on the Movie Review Query Engine website",
        "data_type": "ExternalId"
    },
    "P1575": {
        "label": "RISS catalog",
        "description": "union catalog of South Korea and Japan, maintained by RISS",
        "data_type": "ExternalId"
    },
    "P7519": {
        "label": "Filmový přehled film ID",
        "description": "identifier for a film in the Filmový přehled database",
        "data_type": "ExternalId"
    },
    "P8245": {
        "label": "Mediaqueer.ca movie ID",
        "data_type": "ExternalId"
    },
    "P9367": {
        "label": "Filmitalia film ID",
        "description": "identifier used for films in the database of filmitalia.org",
        "data_type": "ExternalId"
    },
    "P1739": {
        "label": "CiNii book ID",
        "description": "identifier for books in the Japanese CiNii bibliography by NII aka National institute of informatics",
        "data_type": "ExternalId"
    },
    "P7975": {
        "label": "Filmfront film ID",
        "description": "identifiers for films in the Filmfront, a Norwegian movie database",
        "data_type": "ExternalId"
    },
    "P6580": {
        "label": "National Book Foundation book ID",
        "description": "identifier for a book on the website of the National Book Foundation",
        "data_type": "ExternalId"
    },
    "P8921": {
        "label": "KOBIS ID",
        "description": "identifier for a film the database of the Korea Box-Office Information System (KOBIS) provided by the Korean Film Council",
        "data_type": "ExternalId"
    },
    "P4181": {
        "label": "GujLit Book ID",
        "description": "identifier for a book, in the GujLit database of literature of Gujarati language of India",
        "data_type": "ExternalId"
    },
    "P1182": {
        "label": "LIBRIS editions",
        "description": "identifier for the items in the libris-catalogue of the Royal Swedish Library about specific publications",
        "data_type": "ExternalId"
    },
    "P6398": {
        "label": "iTunes movie ID",
        "description": "identifier for a film in the iTunes Store",
        "data_type": "ExternalId"
    },
    "P8928": {
        "label": "Filmstriben film-ID",
        "description": "identifier for film on the Danish website https://fjernleje.filmstriben.dk/",
        "data_type": "ExternalId"
    },
    "P2034": {
        "label": "Project Gutenberg ebook ID",
        "description": "ebook identifier at Project Gutenberg",
        "data_type": "ExternalId"
    },
    "P7222": {
        "label": "Eiga.com movie ID",
        "description": "identifier of movie on Eiga.com database",
        "data_type": "ExternalId"
    },
    "P9022": {
        "label": "Mediafilm ID",
        "description": "identifier for Mediafilm for movies",
        "data_type": "ExternalId"
    },
    "P9283": {
        "label": "AaRC film ID",
        "description": "identifier of a film on AaRC.ro (All About Romanian Cinema)",
        "data_type": "ExternalId"
    },
    "P6383": {
        "label": "TV Spielfilm film ID",
        "description": "identifier for films on tv-spielfilm.de",
        "data_type": "ExternalId"
    },
    "P7334": {
        "label": "Vudu video ID",
        "description": "identifier for a film, television episode, or television series season at Vudu.com",
        "data_type": "ExternalId"
    },
    "P7340": {
        "label": "sixpackfilmdata film ID",
        "description": "identifier of a film in the cinema sixpackfilmdata database",
        "data_type": "ExternalId"
    },
    "P8958": {
        "label": "ClassInd audiovisual work ID",
        "description": "identifier of an audiovisual work in the Classificação Indicativa database",
        "data_type": "ExternalId"
    },
    "P5154": {
        "label": "KKBox artist ID",
        "description": "identifier for an artist on KKBox music site",
        "data_type": "ExternalId"
    },
    "P2207": {
        "label": "Spotify track ID",
        "description": "identifier for a track on Spotify",
        "data_type": "ExternalId"
    },
    "P2723": {
        "label": "Deezer album ID",
        "description": "identifier for an album or single on Deezer",
        "data_type": "ExternalId"
    },
    "P6517": {
        "label": "WhoSampled artist ID",
        "description": "identifier for an artist on WhoSampled.com",
        "data_type": "ExternalId"
    },
    "P1902": {
        "label": "Spotify artist ID",
        "description": "identifier for an artist on Spotify",
        "data_type": "ExternalId"
    },
    "P1728": {
        "label": "AllMusic artist ID",
        "description": "identifier for an artist or musical group in the AllMusic database",
        "data_type": "ExternalId"
    },
    "P5655": {
        "label": "iTunes music video ID",
        "description": "identifier for a music video in iTunes",
        "data_type": "ExternalId"
    },
    "P2088": {
        "label": "Crunchbase organization ID",
        "description": "Identifier for an organization, in the Crunchbase database of companies and start-ups, operated by Oath Tech",
        "data_type": "ExternalId"
    },
    "P1553": {
        "label": "Yandex Music artist ID",
        "description": "identifier for an artist on Yandex Music",
        "data_type": "ExternalId"
    },
    "P2205": {
        "label": "Spotify album ID",
        "description": "identifier for an album or single on Spotify",
        "data_type": "ExternalId"
    },
    "P4579": {
        "label": "Tidal music video ID",
        "description": "identifier of a music video on the Tidal website",
        "data_type": "ExternalId"
    },
    "P1953": {
        "label": "Discogs artist ID",
        "description": "identifier for a band or person in the Discogs database",
        "data_type": "ExternalId"
    },
    "P7511": {
        "label": "doujinshi.org author ID",
        "description": "artist ID on doujinshi.org",
        "data_type": "ExternalId"
    },
    "P6538": {
        "label": "Yandex Music label ID",
        "description": "identifier of a music label on the Yandex Music website",
        "data_type": "ExternalId"
    },
    "P4577": {
        "label": "Tidal album ID",
        "description": "identifier of an album on the Tidal website",
        "data_type": "ExternalId"
    },
    "P5431": {
        "label": "setlist.fm artist ID",
        "description": "identifier for a solo artist or group at setlist.fm",
        "data_type": "ExternalId"
    },
    "P4576": {
        "label": "Tidal artist ID",
        "description": "identifier of an artist on the Tidal website",
        "data_type": "ExternalId"
    },
    "P2963": {
        "label": "Goodreads author ID",
        "description": "identifier of an author on the Goodreads website",
        "data_type": "ExternalId"
    },
    "P7109": {
        "label": "Acharts.co artist ID",
        "description": "identifier for a musical artist on Acharts.co",
        "data_type": "ExternalId"
    },
    "P8445": {
        "label": "Our Campaigns candidate ID",
        "description": "identifier of a candidate in the Our Campaigns political candidate database",
        "data_type": "ExternalId"
    },
    "P6972": {
        "label": "Napster artist ID",
        "description": "identifier for an artist on Napster music site",
        "data_type": "ExternalId"
    },
    "P8060": {
        "label": "Glassdoor company ID",
        "description": "identifier for a company on the Glassdoor employee review website",
        "data_type": "ExternalId"
    },
    "P2724": {
        "label": "Deezer track ID",
        "description": "identifier for a track on Deezer",
        "data_type": "ExternalId"
    },
    "P5915": {
        "label": "Shazam track ID",
        "description": "identifier for an audio or audiovisual creative work on Shazam",
        "data_type": "ExternalId"
    },
    "P5917": {
        "label": "Shazam artist ID",
        "description": "identifier for an artist on Shazam",
        "data_type": "ExternalId"
    },
    "P8510": {
        "label": "Maitron des fusillés ID",
        "description": "identifier for a person on the Maitron des fusillés",
        "data_type": "ExternalId"
    },
    "P7175": {
        "label": "Gaana.com album ID",
        "description": "identifier for an album or single on Gaana.com",
        "data_type": "ExternalId"
    },
    "P7071": {
        "label": "Qobuz artist ID",
        "description": "identifier for an artist on Qobuz music site",
        "data_type": "ExternalId"
    },
    "P8415": {
        "label": "Maniadb artist ID",
        "description": "unique identifier for artists listed in the Maniadb database",
        "data_type": "ExternalId"
    },
    "P4578": {
        "label": "Tidal track ID",
        "description": "identifier of a track on the Tidal website",
        "data_type": "ExternalId"
    },
    "P2850": {
        "label": "iTunes artist ID",
        "description": "identifier for a musical artist or author in the iTunes Store",
        "data_type": "ExternalId"
    },
    "P8990": {
        "label": "World Rowing ID (UUID format)",
        "description": "identifier for a person in the database of World Rowing (FISA)",
        "data_type": "ExternalId"
    },
    "P8664": {
        "label": "USA Water Polo Hall of Fame ID",
        "description": "identifier for an inductee on the USA Water Polo Hall of Fame website",
        "data_type": "ExternalId"
    },
    "P2091": {
        "label": "World Rowing ID (numeric format)",
        "description": "identifier for a rower in the database of World Rowing (FISA)",
        "data_type": "ExternalId"
    },
    "P3764": {
        "label": "pole position",
        "description": "person, who starts race at first row (leader in the starting grid)",
        "data_type": "WikibaseItem"
    },
    "P8161": {
        "label": "CONI honoured ID",
        "description": "ID of an Italian sportperson that was awarded a sports award by the Italian Olympic Committe",
        "data_type": "ExternalId"
    },
    "P8725": {
        "label": "Hungarian Water Polo Federation player ID",
        "description": "identifier for a player on the Hungarian Water Polo Federation website",
        "data_type": "ExternalId"
    },
    "P5053": {
        "label": "fastest lap",
        "description": "person who does the fastest lap during a motorsport race",
        "data_type": "WikibaseItem"
    },
    "P3684": {
        "label": "Driver Database driver ID",
        "description": "identifier for a driver in the Driver Database",
        "data_type": "ExternalId"
    },
    "P9399": {
        "label": "athletics.lv athlete ID",
        "description": "identifier for an athlete at the Statistics Database of the Latvian Athletics Association",
        "data_type": "ExternalId"
    },
    "P8382": {
        "label": "Finales Rugby ID",
        "description": "identifier for a rugby player on finalesrugby.fr website",
        "data_type": "ExternalId"
    },
    "P2095": {
        "label": "co-driver",
        "description": "rally team member who performs as a co-driver or co-pilot",
        "data_type": "WikibaseItem"
    },
    "P9228": {
        "label": "U.S. Masters Swimming ID",
        "description": "U.S. Masters Swimming masters swimmer identifier",
        "data_type": "ExternalId"
    },
    "P1030": {
        "label": "light characteristic of lighthouse",
        "description": "description of a navigational light sequence or colour displayed on a nautical chart. Explicitly state white light (W)",
        "data_type": "String"
    },
    "P4815": {
        "label": "Technical Element Score",
        "description": "for figure skating scores breakdown",
        "data_type": "Quantity"
    },
    "P8548": {
        "label": "Proballers ID",
        "description": "identifier for a basketball player on the Proballers website",
        "data_type": "ExternalId"
    },
    "P3922": {
        "label": "light sector",
        "description": "visibility sector of a navigational light, if applicable specified by color, given in compass directions/azimuth from the viewing location",
        "data_type": "String"
    },
    "P4825": {
        "label": "deductions (in figure skating)",
        "description": "for figure skating scores breakdown",
        "data_type": "Quantity"
    },
    "P4826": {
        "label": "Program Component Score",
        "description": "for figure skating scores breakdown",
        "data_type": "Quantity"
    },
    "P4143": {
        "label": "Finnish List of Lights ID",
        "description": "identifier of a lighthouse or beacon in the Finnish List of Lights, issued by the Finnish Transport Agency",
        "data_type": "ExternalId"
    },
    "P3863": {
        "label": "Italian Navy Lighthouses and Beacons ID",
        "description": "identifier of a lighthouse or beacon in \"Fari e Segnalamenti\" list of the Italian Navy",
        "data_type": "ExternalId"
    },
    "P3920": {
        "label": "Canadian Coastguard Lighthouse ID",
        "description": "identifier of a Canadian lighthouse or beacon in the Canadian Coast Guard List of Lights, Buoys and Fog Signals",
        "data_type": "ExternalId"
    },
    "P8712": {
        "label": "FIBA Hall of Fame ID",
        "description": "identifier for a FIBA hall of famer on the fiba.basketball website",
        "data_type": "ExternalId"
    },
    "P4172": {
        "label": "America's Byways road ID",
        "description": "identifier for a United States road on the America's Byways website",
        "data_type": "ExternalId"
    },
    "P4423": {
        "label": "Portuguese lighthouse ID",
        "description": "national number of a lighthouse in Portugal",
        "data_type": "ExternalId"
    },
    "P4038": {
        "label": "Danish List of Lights and Fog signals ID",
        "description": "identifier in the Danish List of a Lighthouse, beacon or fog signal in the Kingdom of Denmark",
        "data_type": "ExternalId"
    },
    "P3609": {
        "label": "Recreation.gov gateway ID",
        "description": "identifier for a gateway on the Recreation.gov website",
        "data_type": "ExternalId"
    },
    "P3714": {
        "label": "Recreation.gov point of interest ID",
        "description": "identifier for a point of interest in an American recreation area on the Recreation.gov website",
        "data_type": "ExternalId"
    },
    "P3993": {
        "label": "Vitaskrá ID",
        "description": "identifier of an Icelandic lighthouse or beacon in Vitaskrá, a list of lights by the Icelandic Coast Guard",
        "data_type": "ExternalId"
    },
    "P3723": {
        "label": "USCG lighthouse ID",
        "description": "іdentifier for lighthouses in the USA by the US Coast Guard",
        "data_type": "ExternalId"
    },
    "P4055": {
        "label": "Norwegian List of Lights ID",
        "description": "identifier of lighthouses and beacons in the Norwegian List of Lights issued by the Norwegian Coastal Administration",
        "data_type": "ExternalId"
    },
    "P9166": {
        "label": "Curling.fi player ID",
        "description": "numerical identifier for a player at Finnish Curling Association official site",
        "data_type": "ExternalId"
    },
    "P2929": {
        "label": "lighthouse range",
        "description": "the distance at which the light can be seen. If the light has colors, there will be several ranges, one for each color. Use qualifier \"color\" (P462)",
        "data_type": "Quantity"
    },
    "P3871": {
        "label": "tributary orientation",
        "description": "specify if the stream confluence is a left bank or right bank tributary",
        "data_type": "WikibaseItem"
    },
    "P565": {
        "label": "crystal habit",
        "description": "the proportions of a crystal or mineral",
        "data_type": "WikibaseItem"
    },
    "P4792": {
        "label": "dam",
        "description": "construction impounding this watercourse or creating this reservoir",
        "data_type": "WikibaseItem"
    },
    "P537": {
        "label": "twinning",
        "description": "type of twins a crystal forms",
        "data_type": "WikibaseItem"
    },
    "P3376": {
        "label": "Enterprise number (Belgium)",
        "description": "Identifier from Crossroads Bank for Belgian Enterprises",
        "data_type": "ExternalId"
    },
    "P711": {
        "label": "Strunz 8th edition (series ID, updated)",
        "description": "mineral classification 8th edition, series identifier, silicate classification modified by Athena",
        "data_type": "String"
    },
    "P6775": {
        "label": "Finnish Trade Register ID",
        "description": "identifier for Finnish companies up to 31 March 2001, predecessor to the Business Identity Code",
        "data_type": "ExternalId"
    },
    "P713": {
        "label": "Nickel-Strunz '10th ed', review of (9th ed/ 2009 update)",
        "description": "mineral classification Nickel-Strunz version 10 (MinDat)",
        "data_type": "String"
    },
    "P6518": {
        "label": "Business Registry code (Estonia)",
        "description": "identifier for a juridical person registered in Estonia",
        "data_type": "ExternalId"
    },
    "P6204": {
        "label": "CNPJ",
        "description": "identification number issued to Brazilian companies by the Secretariat of the Federal Revenue of Brazil",
        "data_type": "ExternalId"
    },
    "P6569": {
        "label": "taxa especially protected in area",
        "description": "species and other taxa especially protected in a area, whether still actually present or not. Do not use for countries/subnational entities: use red lists properties instead.",
        "data_type": "WikibaseItem"
    },
    "P4002": {
        "label": "WFD Ecological status",
        "description": "Ecological status of a Water Body in accordance with the Water Framework directive.",
        "data_type": "WikibaseItem"
    },
    "P2156": {
        "label": "pseudo crystal habit",
        "description": "possible macrosopic appearance of a crystal different to its molecular symmetry (crystal cell, specimens with twinning or synthetic material)",
        "data_type": "WikibaseItem"
    },
    "P6460": {
        "label": "Swedish Organization Number",
        "description": "sequence of two numberpairs that is unique to a Swedish company or organization given out by the responsible government department",
        "data_type": "ExternalId"
    },
    "P690": {
        "label": "space group",
        "description": "symmetry classification for 2 and 3 dimensional patterns or crystals",
        "data_type": "WikibaseItem"
    },
    "P5687": {
        "label": "New Zealand Business Number",
        "description": "unique identifier for a business entity registered in New Zealand",
        "data_type": "ExternalId"
    },
    "P3220": {
        "label": "KvK company ID",
        "description": "identifier of a company in the Dutch KvK Business Register",
        "data_type": "ExternalId"
    },
    "P3432": {
        "label": "parent cell line",
        "description": "closest parent cell line of the cell line in question",
        "data_type": "WikibaseItem"
    },
    "P712": {
        "label": "Nickel-Strunz 9th edition (updated 2009)",
        "description": "mineral classification Nickel-Strunz version 9 (updated 2009, deprecated), a PDF format file. Superseeded by  Nickel-Strunz '10th ed'",
        "data_type": "String"
    },
    "P6799": {
        "label": "GLIMS ID",
        "description": "identifier assigned to glaciers by the Global Land Ice Measurements from Space service at NSIDC",
        "data_type": "ExternalId"
    },
    "P4202": {
        "label": "WFD Chemical status",
        "description": "chemical status of a Water Body in accordance with the Water Framework directive of the  European Environment Agency",
        "data_type": "WikibaseItem"
    },
    "P8894": {
        "label": "BG EIK",
        "description": "BG Unified Identification Code for companies, BULSTAT code for non-commercial orgs. 9-digit for organisations, 13-digit for branches",
        "data_type": "ExternalId"
    },
    "P538": {
        "label": "fracturing",
        "description": "type of fracture a crystal or mineral forms",
        "data_type": "WikibaseItem"
    },
    "P534": {
        "label": "streak color",
        "description": "color of a mineral or material when abraded",
        "data_type": "WikibaseItem"
    },
    "P1088": {
        "label": "Mohs' hardness",
        "description": "qualitative ordinal scale characterizing scratch resistance of various minerals",
        "data_type": "Quantity"
    },
    "P4614": {
        "label": "drainage basin",
        "description": "area where precipitation collects and drains off into a common outlet, such as into a river, bay, or other body of water",
        "data_type": "WikibaseItem"
    },
    "P6843": {
        "label": "Malaysia company number",
        "description": "identifier for companies in Malaysia",
        "data_type": "ExternalId"
    },
    "P693": {
        "label": "cleavage",
        "description": "way that crystals or minerals cleave",
        "data_type": "WikibaseItem"
    },
    "P579": {
        "label": "IMA status and/or rank",
        "description": "status given to each mineral by the IMA (International Mineralogical Association)",
        "data_type": "WikibaseItem"
    },
    "P1616": {
        "label": "SIREN number",
        "description": "identifer for an organization in the SIRENE register maintained by INSEE",
        "data_type": "ExternalId"
    },
    "P8860": {
        "label": "Canada Business Number",
        "description": "identifier for businesses and legal entities registered in Canada",
        "data_type": "ExternalId"
    },
    "P4661": {
        "label": "reservoir created",
        "description": "reservoir created upstream of a dam by this dam",
        "data_type": "WikibaseItem"
    },
    "P3578": {
        "label": "autologous cell line",
        "description": "cell line originating from the same individual",
        "data_type": "WikibaseItem"
    },
    "P4350": {
        "label": "salinity",
        "description": "amount of dissolved salt in a body of water",
        "data_type": "Quantity"
    },
    "P4829": {
        "label": "Swiss Enterprise Identification Number",
        "description": "government identifier (\"UID\") for a business entity in Switzerland",
        "data_type": "ExternalId"
    },
    "P4190": {
        "label": "Rivers.gov protected area ID",
        "description": "identifier for an American National Wild and Scenic River on the United States' Rivers.gov website",
        "data_type": "ExternalId"
    },
    "P2155": {
        "label": "solid solution series with",
        "description": "the mineral forms a continous (true) or discontinous \"solid solution series\" with another mineral",
        "data_type": "WikibaseItem"
    },
    "P3608": {
        "label": "EU VAT number",
        "description": "EU VAT number",
        "data_type": "ExternalId"
    },
    "P2622": {
        "label": "Companies House company ID",
        "description": "numeric identifier for company registered with Companies House in the United Kingdom",
        "data_type": "ExternalId"
    },
    "P1632": {
        "label": "Hermann-Mauguin notation",
        "description": "Hermann-Mauguin notation of the mineral's symmetry element (short form)",
        "data_type": "ExternalId"
    },
    "P2333": {
        "label": "Norwegian organisation number",
        "description": "organisation's purely numerical identifier in the Norwegian \"Central Coordinating Register for Legal Entities\"",
        "data_type": "ExternalId"
    },
    "P3548": {
        "label": "Australian Business Number",
        "description": "unique identifier for a business entity registered in Australia",
        "data_type": "ExternalId"
    },
    "P7283": {
        "label": "Spectrum Management System Company Number",
        "description": "unique identifier of a company registered with Innovation, Science and Economic Development Canada's Spectrum Management System database",
        "data_type": "ExternalId"
    },
    "P2391": {
        "label": "OKPO ID",
        "description": "identifier for a company, in the Russian National Classifier of Enterprises and Organizations (OKPO) database",
        "data_type": "ExternalId"
    },
    "P2619": {
        "label": "Hungarian company ID",
        "description": "identifier for a company in the company register of Hungary",
        "data_type": "ExternalId"
    },
    "P1297": {
        "label": "IRS Employer Identification Number",
        "description": "number given to businesses by the United States Internal Revenue Service",
        "data_type": "ExternalId"
    },
    "P3549": {
        "label": "Australian Company Number",
        "description": "unique identifier for a company registered in Australia",
        "data_type": "ExternalId"
    },
    "P1059": {
        "label": "CVR number",
        "description": "unique identifier for a business in Denmark's Central Business Register (CVR), the official database of Danish businesses.",
        "data_type": "ExternalId"
    },
    "P9072": {
        "label": "derived from organism type",
        "description": "the taxon that the organism from which the subject cell line was derived belonged to (different from found in taxon (P703) because cell lines are derived only once from a single individual)",
        "data_type": "WikibaseItem"
    },
    "P8672": {
        "label": "Twitter topic ID",
        "description": "identifier for a topic in Twitter",
        "data_type": "ExternalId"
    },
    "P8383": {
        "label": "Goodreads work ID",
        "description": "unique identifier for written works (Q47461344) on Goodreads",
        "data_type": "ExternalId"
    },
    "P8473": {
        "label": "Japanese magazine code",
        "description": "unique code for magazines used in Japan",
        "data_type": "ExternalId"
    },
    "P6886": {
        "label": "writing language",
        "description": "language in which the writer has written their work",
        "data_type": "WikibaseItem"
    },
    "P9409": {
        "label": "American University of Beirut's Libraries title ID",
        "description": "identifier of a work in the Library Catalog of American University of Beirut",
        "data_type": "ExternalId"
    },
    "P4749": {
        "label": "Dictionnaire des auteurs luxembourgeois ID",
        "description": "identifier in the Dictionnaire des auteurs luxembourgeois en ligne, that presents the life and work of authors who, since 1815, have participated in the literary life of Luxembourg",
        "data_type": "ExternalId"
    },
    "P8522": {
        "label": "Manioc book ID",
        "description": "identifier for a book on the website of the Manioc digital library for the Caribbean, the Amazon and the Guyane",
        "data_type": "ExternalId"
    },
    "P1482": {
        "label": "Stack Exchange tag",
        "description": "tag on the Stack Exchange websites",
        "data_type": "Url"
    },
    "P8632": {
        "label": "Book Marks ID",
        "description": "identifier for a book at Book Marks",
        "data_type": "ExternalId"
    },
    "P6721": {
        "label": "K10plus PPN ID",
        "description": "identifier in the K10plus union catalog of the Common Library Network (GBV) and the South-West German Library Network (SWB)",
        "data_type": "ExternalId"
    },
    "P647": {
        "label": "drafted by",
        "description": "which team the player was drafted by",
        "data_type": "WikibaseItem"
    },
    "P3483": {
        "label": "VGMdb album ID",
        "description": "identifier for a single or album in the Video Game Music database",
        "data_type": "ExternalId"
    },
    "P404": {
        "label": "game mode",
        "description": "a video game's available playing mode(s)",
        "data_type": "WikibaseItem"
    },
    "P2415": {
        "label": "personal best",
        "description": "an individual's best performance in a given discipline",
        "data_type": "Quantity"
    },
    "P3435": {
        "label": "VGMdb artist ID",
        "description": "identifier for a musician or group in the Video Game Music database",
        "data_type": "ExternalId"
    },
    "P741": {
        "label": "playing hand",
        "description": "hand used to play a racket sport",
        "data_type": "WikibaseItem"
    },
    "P7002": {
        "label": "Handball123 player ID",
        "description": "identifier for an handball player at Handball123",
        "data_type": "ExternalId"
    },
    "P8277": {
        "label": "AFL player ID",
        "description": "identifier for a Australian rules footballer from AFL club player profile pages",
        "data_type": "ExternalId"
    },
    "P6648": {
        "label": "position in Forsyth-Edwards Notation",
        "description": "board position in a chess game described in Forsyth–Edwards Notation (FEN)",
        "data_type": "String"
    },
    "P5286": {
        "label": "Portable Game Notation",
        "description": "move of a chess game provided as a string in Portable Game Notation",
        "data_type": "String"
    },
    "P2545": {
        "label": "bowling style",
        "description": "type of bowling employed by a cricketer",
        "data_type": "WikibaseItem"
    },
    "P1532": {
        "label": "country for sport",
        "description": "country a person or a team represents when playing a sport",
        "data_type": "WikibaseItem"
    },
    "P6289": {
        "label": "ODNR Division of Wildlife ID",
        "description": "identifier for a taxon in the Species Guide Index of the Ohio Department of Natural Resources website",
        "data_type": "ExternalId"
    },
    "P6285": {
        "label": "Weeds in Ontario ID",
        "description": "identifier for a taxon on the Weeds in Ontario website",
        "data_type": "ExternalId"
    },
    "P6758": {
        "label": "supported sports team",
        "description": "sports team for which a person or a group is a supporter",
        "data_type": "WikibaseItem"
    },
    "P4807": {
        "label": "Fauna Europaea New ID",
        "description": "identifier for a taxon at the Fauna Europaea portal launched end of 2016",
        "data_type": "ExternalId"
    },
    "P8811": {
        "label": "Twitch team ID",
        "description": "identifier for a team on Twitch",
        "data_type": "ExternalId"
    },
    "P1746": {
        "label": "ZooBank ID for name or act",
        "description": "identifier for a name or nomenclatural act at ZooBank",
        "data_type": "ExternalId"
    },
    "P6161": {
        "label": "Invasive Plant Atlas of the United States ID",
        "description": "identifier for a species on the Invasive Plant Atlas of the United States website",
        "data_type": "ExternalId"
    },
    "P5138": {
        "label": "season of club or team",
        "description": "club or team that played the season",
        "data_type": "WikibaseItem"
    },
    "P1348": {
        "label": "AlgaeBase URL",
        "description": "URL for a taxon in AlgaeBase.org",
        "data_type": "Url"
    },
    "P6042": {
        "label": "Echinoid Directory ID",
        "description": "identifier for a taxon on the Echinoid Directory website",
        "data_type": "ExternalId"
    },
    "P6049": {
        "label": "NOAA Fisheries Species Directory ID",
        "description": "identifier for a species on the NOAA Fisheries website",
        "data_type": "ExternalId"
    },
    "P6163": {
        "label": "NAS ID",
        "description": "identifier for a species in the Nonindigenous Aquatic Species database, on the U.S. Geological Survey website",
        "data_type": "ExternalId"
    },
    "P3395": {
        "label": "heart rate",
        "description": "speed of the heartbeat measured by the number of contractions of the heart per minute (bpm)",
        "data_type": "Quantity"
    },
    "P5036": {
        "label": "AmphibiaWeb Species ID",
        "description": "identifier for a species, in the AmphibiaWeb database",
        "data_type": "ExternalId"
    },
    "P6043": {
        "label": "Espèces Envahissantes Outre-mer ID",
        "description": "identifier for a species on the Espèces Envahissantes Outre-mer website",
        "data_type": "ExternalId"
    },
    "P6036": {
        "label": "Wild Herps ID",
        "description": "identifier for a taxon on the Wild Herps website",
        "data_type": "ExternalId"
    },
    "P8915": {
        "label": "ZOBODAT taxon ID",
        "description": "numerical identifier for a taxon in ZOBODAT, a bibliographic database of mostly German-language works on zoology",
        "data_type": "ExternalId"
    },
    "P5626": {
        "label": "Global Invasive Species Database ID",
        "description": "identifier for a taxon in the IUCN's Global Invasive Species Database",
        "data_type": "ExternalId"
    },
    "P3387": {
        "label": "minimum frequency of audible sound",
        "description": "minimum frequency of audible sound for this species",
        "data_type": "Quantity"
    },
    "P5698": {
        "label": "Invasive Species Compendium Datasheet ID",
        "description": "identifier for a taxon datasheet in the Invasive Species Compendium, produced by the Centre for Agriculture and Bioscience International",
        "data_type": "ExternalId"
    },
    "P4743": {
        "label": "animal breed",
        "description": "subject item belongs to a specific group of domestic animals, generally given by association",
        "data_type": "WikibaseItem"
    },
    "P6033": {
        "label": "CMS ID",
        "description": "identifier for a taxon on the 'Convention on the Conservation of Migratory Species of Wild Animals' website",
        "data_type": "ExternalId"
    },
    "P6039": {
        "label": "Australian Faunal Directory ID",
        "description": "identifier for a taxon on the Australian Faunal Directory website",
        "data_type": "ExternalId"
    },
    "P6054": {
        "label": "GT IBMA ID",
        "description": "identifier for a species on the Groupe de travail Invasions biologiques en milieux aquatiques website",
        "data_type": "ExternalId"
    },
    "P6051": {
        "label": "Sea Slug Forum ID",
        "description": "identifier for a taxon on the Sea Slug Forum website",
        "data_type": "ExternalId"
    },
    "P3485": {
        "label": "bite force quotient",
        "description": "regression of the quotient of an animal's bite force in newtons divided by its body mass in kilogrammes",
        "data_type": "Quantity"
    },
    "P6176": {
        "label": "Cal-IPC ID",
        "description": "identifier for a species on the California Invasive Plant Council website",
        "data_type": "ExternalId"
    },
    "P3380": {
        "label": "FAO 2007 genetic resource ID",
        "description": "(constructed) ID of a domestic animal species or breed in the FAO Animal Genetic resources report 2007 breed",
        "data_type": "ExternalId"
    },
    "P2839": {
        "label": "gait",
        "description": "gait of an animal",
        "data_type": "WikibaseItem"
    },
    "P4024": {
        "label": "ADW taxon ID",
        "description": "identifier for a taxon, in the Animal Diversity Web database",
        "data_type": "ExternalId"
    },
    "P3465": {
        "label": "maximum frequency of audible sound",
        "description": "maximum frequency of audible sound for this species",
        "data_type": "Quantity"
    },
    "P959": {
        "label": "MSW ID",
        "description": "identifier from Mammal Species of the World  database",
        "data_type": "ExternalId"
    },
    "P6018": {
        "label": "SeaLifeBase ID",
        "description": "identifier for a taxon on the SeaLifeBase website",
        "data_type": "ExternalId"
    },
    "P5317": {
        "label": "jockey",
        "description": "person riding the horse in the race",
        "data_type": "WikibaseItem"
    },
    "P3486": {
        "label": "normal respiratory rate",
        "description": "normal number of respiratory cycles (inspiration and exhalation) per minute, measured in an individual",
        "data_type": "Quantity"
    },
    "P850": {
        "label": "WoRMS-ID for taxa",
        "description": "identifier in the World Register of Marine Species",
        "data_type": "ExternalId"
    },
    "P3594": {
        "label": "Araneae Spider ID",
        "description": "identifier for a taxon in araneae.nmbe.ch",
        "data_type": "ExternalId"
    },
    "P4630": {
        "label": "DORIS ID",
        "description": "identifier of an underwater animal or plant species from French metropolitan and overseas waters on the participatory site DORIS",
        "data_type": "ExternalId"
    },
    "P938": {
        "label": "FishBase species ID",
        "description": "identifier for a species in FishBase",
        "data_type": "ExternalId"
    },
    "P2024": {
        "label": "German cattle breed ID",
        "description": "identifier of a cattle breed in German national law",
        "data_type": "ExternalId"
    },
    "P4403": {
        "label": "caliber",
        "description": "inner diameter of the gun barrel for a weapon",
        "data_type": "Quantity"
    },
    "P2188": {
        "label": "BiblioNet author ID",
        "description": "identifier in the BiblioNet database of authors, created by the National Book Centre of Greece, many Greek individual publishers and their professional associations",
        "data_type": "ExternalId"
    },
    "P1564": {
        "label": "At the Circulating Library ID",
        "description": "identifier in the At the Circulating Library Victorian literature database",
        "data_type": "ExternalId"
    },
    "P2324": {
        "label": "quantity buried",
        "description": "number of remains buried or interred",
        "data_type": "Quantity"
    },
    "P561": {
        "label": "NATO reporting name",
        "description": "official reporting name assigned by the ASCC for NATO use",
        "data_type": "String"
    },
    "P1056": {
        "label": "product or material produced",
        "description": "material or product produced by a government agency, business, industry, facility, or process",
        "data_type": "WikibaseItem"
    },
    "P5477": {
        "label": "Les Voix de la poésie poet ID",
        "description": "identifier for a poet on the Les Voix de la poésie website",
        "data_type": "ExternalId"
    },
    "P5009": {
        "label": "complies with",
        "description": "the product or work complies with a certain norm or passes a test",
        "data_type": "WikibaseItem"
    },
    "P6075": {
        "label": "embodied energy",
        "description": "sum of all the energy required to produce any goods or services, considered as if that energy was incorporated or 'embodied' in the product itself",
        "data_type": "Quantity"
    },
    "P520": {
        "label": "armament",
        "description": "equippable weapon item for the subject",
        "data_type": "WikibaseItem"
    },
    "P4137": {
        "label": "muzzle velocity",
        "description": "speed of a projectile at the moment it leaves the muzzle of a gun",
        "data_type": "Quantity"
    },
    "P5506": {
        "label": "Poetry In Voice poet ID",
        "description": "identifier for a poet on the Poetry In Voice website",
        "data_type": "ExternalId"
    },
    "P3792": {
        "label": "rate of fire",
        "description": "the frequency at which a specific weapon can fire or launch its projectiles",
        "data_type": "Quantity"
    },
    "P4176": {
        "label": "effective firing range",
        "description": "distance within which a weapon can be used effectively",
        "data_type": "Quantity"
    },
    "P9354": {
        "label": "Porcelana Brasil ID",
        "description": "identifier for a faience or porcelain manufacturer on the Porcelana Brasil website",
        "data_type": "ExternalId"
    },
    "P2351": {
        "label": "number of graves",
        "description": "number of burial places In a cemetery or necropolis",
        "data_type": "Quantity"
    },
    "P9355": {
        "label": "Slovník českých filosofů ID",
        "description": "identifier for a Czech philosopher in the online biographical dictionary Slovník českých filosofů",
        "data_type": "ExternalId"
    },
    "P1389": {
        "label": "product certification",
        "description": "certification for a product, qualify with P1001 (\"applies to jurisdiction\") if needed",
        "data_type": "WikibaseItem"
    },
    "P624": {
        "label": "guidance system",
        "description": "guidance system of a missile",
        "data_type": "WikibaseItem"
    },
    "P2821": {
        "label": "by-product",
        "description": "product of a chemical or industrial process, of secondary economic value",
        "data_type": "WikibaseItem"
    },
    "P6076": {
        "label": "biobased content weight percentage",
        "description": "weight percentage of content comming from the biomass",
        "data_type": "Quantity"
    },
    "P5710": {
        "label": "Recours au poème ID",
        "description": "identifier for a poet on 'Recours au poème'",
        "data_type": "ExternalId"
    },
    "P3236": {
        "label": "PhilPapers publication ID",
        "description": "an identifier for publications (journals) on PhilPapers",
        "data_type": "ExternalId"
    },
    "P4988": {
        "label": "test method",
        "description": "method in science, engineering or medicine, which determines the subject quantity, qualitative property or any other measurable attribute",
        "data_type": "WikibaseItem"
    },
    "P8216": {
        "label": "iDAI.chronontology ID",
        "description": "identifier for an epoch in the database iDAI.chronontology compiled by the German Archaeological Institute",
        "data_type": "ExternalId"
    },
    "P8220": {
        "label": "NMMA antiquities ID",
        "description": "identifier for antiquities of India which are documented by National Mission on Monuments and Antiquities",
        "data_type": "ExternalId"
    },
    "P8690": {
        "label": "Quebec Dams Directory ID",
        "description": "unique identifier of a dam in Quebec",
        "data_type": "ExternalId"
    },
    "P1291": {
        "label": "Association Authors of Switzerland ID (former scheme)",
        "description": "identifier in a Swiss database of modern writers",
        "data_type": "ExternalId"
    },
    "P5478": {
        "label": "RPO poet ID",
        "description": "identifier for a poet in the Representative Poetry Online database",
        "data_type": "ExternalId"
    },
    "P176": {
        "label": "manufacturer",
        "description": "manufacturer or producer of this product",
        "data_type": "WikibaseItem"
    },
    "P6885": {
        "label": "historical region",
        "description": "geographic area which at some point in time had a cultural, ethnic, linguistic or political basis, regardless of present-day borders",
        "data_type": "WikibaseItem"
    },
    "P4441": {
        "label": "hydraulic head",
        "description": "hydraulic head of a dam or hydroelectric power station",
        "data_type": "Quantity"
    },
    "P8450": {
        "label": "peak bagging classification",
        "description": "recognised peak bagging classification of a mountain or hill",
        "data_type": "WikibaseItem"
    },
    "P5343": {
        "label": "Poets.org poet ID",
        "description": "identifier for a poet on the Academy of American Poets website",
        "data_type": "ExternalId"
    },
    "P965": {
        "label": "burial plot reference",
        "description": "string of references which identify a specific plot in a burial area, e.g. cemetery",
        "data_type": "String"
    },
    "P739": {
        "label": "ammunition",
        "description": "cartridge or other ammunition used by the subject firearm",
        "data_type": "WikibaseItem"
    },
    "P2079": {
        "label": "fabrication method",
        "description": "method, process or technique used to grow, cook, weave, build, assemble, manufacture the item",
        "data_type": "WikibaseItem"
    },
    "P547": {
        "label": "commemorates",
        "description": "what the place, monument, memorial, or holiday, commemorates",
        "data_type": "WikibaseItem"
    },
    "P8887": {
        "label": "water area",
        "description": "area/surface of a geographical entity that is covered by water",
        "data_type": "Quantity"
    },
    "P2822": {
        "label": "by-product of",
        "description": "chemical or industrial process which produces the item as a by-product",
        "data_type": "WikibaseItem"
    },
    "P3235": {
        "label": "PhilPapers topic",
        "description": "identifier for topics on PhilPapers",
        "data_type": "ExternalId"
    },
    "P5392": {
        "label": "Poetry Archive poet ID",
        "description": "identifier for a poet on the Poetry Archive website",
        "data_type": "ExternalId"
    },
    "P9411": {
        "label": "USCG MIC",
        "description": "United States Coast Guard Manufacturer's (of boat) Identification Code (\"MIC\")",
        "data_type": "ExternalId"
    },
    "P6687": {
        "label": "IANA timezone ID",
        "description": "identifier of a time zone in the IANA database",
        "data_type": "ExternalId"
    },
    "P4558": {
        "label": "SNCZI-IPE dam ID",
        "description": "identifier of a dam in Spain, in the SNCZI-Inventario de Presas y Embalses database",
        "data_type": "ExternalId"
    },
    "P5341": {
        "label": "Poetry Foundation ID",
        "description": "identifier for a poet on the Poetry Foundation website",
        "data_type": "ExternalId"
    },
    "P186": {
        "label": "made from material",
        "description": "material the subject is made of or derived from",
        "data_type": "WikibaseItem"
    },
    "P9047": {
        "label": "archaeological site of",
        "description": "at this archeological site the object has been uncovered",
        "data_type": "WikibaseItem"
    },
    "P6630": {
        "label": "SNISB ID",
        "description": "unique identifier of a Brazilian dam, defined by the Brazilian National Information System on Dams Safety (SNISB)",
        "data_type": "ExternalId"
    },
    "P723": {
        "label": "Digitale Bibliotheek voor de Nederlandse Letteren author ID",
        "description": "identifier for an author on the DBNL-website for Dutch language authors",
        "data_type": "ExternalId"
    },
    "P6433": {
        "label": "Global Species ID",
        "description": "identifier for a taxon in Global Species database",
        "data_type": "ExternalId"
    },
    "P518": {
        "label": "applies to part",
        "description": "part, aspect, or form of the item to which the claim applies",
        "data_type": "WikibaseItem"
    },
    "P1743": {
        "label": "Bradley and Fletcher checklist number",
        "description": "reference for a species in the Bradley and Fletcher checklist of British lepidoptera",
        "data_type": "ExternalId"
    },
    "P1415": {
        "label": "Oxford Dictionary of National Biography ID",
        "description": "identifier used by the Oxford Dictionary of National Biography",
        "data_type": "ExternalId"
    },
    "P1367": {
        "label": "Art UK artist ID",
        "description": "authority control identifier for artists (creators of publicly owned oil paintings in the UK)",
        "data_type": "ExternalId"
    },
    "P7218": {
        "label": "Vox FM artist ID",
        "description": "identifier for an artist on Vox FM site",
        "data_type": "ExternalId"
    },
    "P8327": {
        "label": "intended subject of deprecated statement",
        "description": "(qualifier) this statement is deprecated as it is actually about another subject",
        "data_type": "WikibaseItem"
    },
    "P6080": {
        "label": "Discogs composition ID",
        "description": "identifier for a musical composition in Discogs",
        "data_type": "ExternalId"
    },
    "P1585": {
        "label": "Brazilian municipality code",
        "description": "identifier for municipalities in Brazil",
        "data_type": "ExternalId"
    },
    "P6849": {
        "label": "DR topic ID",
        "description": "identifier for a topic for news on DR's website",
        "data_type": "ExternalId"
    },
    "P474": {
        "label": "country calling code",
        "description": "identifier for a country - dialed on phone after the international dialing prefix (precede value by +)",
        "data_type": "String"
    },
    "P1439": {
        "label": "Norwegian filmography ID",
        "description": "identifier from Norsk filmografi, published by the National Library of Norway",
        "data_type": "ExternalId"
    },
    "P6079": {
        "label": "Discogs track ID",
        "description": "identifier for a music track in Discogs",
        "data_type": "ExternalId"
    },
    "P303": {
        "label": "EE breed number",
        "description": "breed identification number per the EE list of the breeds of fancy pigeons (ELFP)",
        "data_type": "ExternalId"
    },
    "P1555": {
        "label": "Executive Order number",
        "description": "identifier for an executive order of the US",
        "data_type": "ExternalId"
    },
    "P2349": {
        "label": "Stuttgart Database of Scientific Illustrators ID",
        "description": "identifier for a person, in the University of Stuttgart's Database of Scientific Illustrators, 1450–1950",
        "data_type": "ExternalId"
    },
    "P1011": {
        "label": "excluding",
        "description": "usually used as a qualifier",
        "data_type": "WikibaseItem"
    },
    "P1744": {
        "label": "Agassiz checklist number",
        "description": "Reference for a species in the 2013 checklist of over 2,700 species British lepidoptera",
        "data_type": "ExternalId"
    },
    "P8798": {
        "label": "for color scheme",
        "description": "which color scheme this statement is designed for",
        "data_type": "WikibaseItem"
    },
    "P3879": {
        "label": "Cologne phonetics",
        "description": "phonetic algorithm which assigns to words a sequence of digits, the phonetic code",
        "data_type": "String"
    },
    "P4443": {
        "label": "card network",
        "description": "credit or debit card network",
        "data_type": "WikibaseItem"
    },
    "P1752": {
        "label": "scale",
        "description": "Proportional ratio of a linear dimension of a model, map, etc,  to the same feature of the original - 1:n. Use 1 for lifesize",
        "data_type": "Quantity"
    },
    "P842": {
        "label": "Fossilworks taxon ID",
        "description": "identifier for an animal, plant, or microorganism in the Fossilworks database",
        "data_type": "ExternalId"
    },
    "P3880": {
        "label": "Caverphone",
        "description": "phonetic algorithm",
        "data_type": "String"
    },
    "P6025": {
        "label": "Oiseaux.net ID",
        "description": "identifier for a bird taxon on the Oiseaux.net website",
        "data_type": "ExternalId"
    },
    "P4867": {
        "label": "RFE symbol",
        "description": "symbol of the phonetic alphabet of the Revista de Filología Española corresponding to an AFI phoneme",
        "data_type": "String"
    },
    "P3878": {
        "label": "Soundex",
        "description": "phonetic algorithm for indexing names by sound, as pronounced in English. Format: initial of word followed by 3 digits (0–6)",
        "data_type": "String"
    },
    "P8337": {
        "label": "Cimetières de France ID",
        "description": "identifier for a cemetery in the Cimetières de France database",
        "data_type": "ExternalId"
    },
    "P5386": {
        "label": "JMA Seismic Intensity Scale",
        "description": "maximum intensity scale of this earthquake given by JMA Seismic Intensity Scale",
        "data_type": "WikibaseItem"
    },
    "P4664": {
        "label": "Wiki Aves bird ID",
        "description": "identifier for a bird species on Wiki Aves is a Brazilian catalogue website",
        "data_type": "ExternalId"
    },
    "P6045": {
        "label": "Global Raptor Information Network ID",
        "description": "identifier for a taxon on the Global Raptor Information Network website",
        "data_type": "ExternalId"
    },
    "P6041": {
        "label": "John J. Audubon's Birds of America ID",
        "description": "identifier for a taxon in John J. Audubon's 'Birds of America' on the National Audubon Society website",
        "data_type": "ExternalId"
    },
    "P6040": {
        "label": "BirdLife Australia ID",
        "description": "identifier for a species on the BirdLife Australia website",
        "data_type": "ExternalId"
    },
    "P200": {
        "label": "inflows",
        "description": "major inflow sources — rivers, aquifers, glacial runoff, etc. Some terms may not be place names, e.g. none",
        "data_type": "WikibaseItem"
    },
    "P6047": {
        "label": "Neotropical Birds ID",
        "description": "identifier for a taxon on the Neotropical Birds website",
        "data_type": "ExternalId"
    },
    "P9189": {
        "label": "Sauvons nos tombes cemetery ID",
        "description": "numerical identifier for a cemetery on the 'Sauvons nos tombes' project",
        "data_type": "ExternalId"
    },
    "P6048": {
        "label": "New Zealand Birds Online ID",
        "description": "identifier for a species on the New Zealand Birds Online website",
        "data_type": "ExternalId"
    },
    "P4715": {
        "label": "Guide to North American Birds ID",
        "description": "identifier for a bird taxon in the Guide to North American Birds on the National Audubon Society website",
        "data_type": "ExternalId"
    },
    "P4448": {
        "label": "cash back",
        "description": "Cash or Points received for spending money as part of a loyalty program",
        "data_type": "Quantity"
    },
    "P5412": {
        "label": "romanization of Belarusian (2007 system)",
        "description": "romanization of Belarusian (2007)",
        "data_type": "String"
    },
    "P5900": {
        "label": "JMA Magnitude",
        "description": "Magnitude of an earthquake according to Japan Meteorological Agency",
        "data_type": "Quantity"
    },
    "P2426": {
        "label": "Xeno-canto species ID",
        "description": "identifier of a bird species in the Xeno-canto database",
        "data_type": "ExternalId"
    },
    "P4526": {
        "label": "ABA bird ID",
        "description": "identifier for a bird taxon, issued by the American Birding Association",
        "data_type": "ExternalId"
    },
    "P3444": {
        "label": "eBird taxon ID",
        "description": "identifier for a species or subspecies, in the eBird database",
        "data_type": "ExternalId"
    },
    "P4445": {
        "label": "grace period",
        "description": "period immediately after the deadline for an obligation during which a fee is waived",
        "data_type": "Quantity"
    },
    "P5526": {
        "label": "valency",
        "description": "number of arguments controlled by a verbal predicate",
        "data_type": "Quantity"
    },
    "P3459": {
        "label": "Euring number",
        "description": "identifier for a species or subspecies, in the Euring database",
        "data_type": "ExternalId"
    },
    "P5095": {
        "label": "fault",
        "description": "geological faults involved in the event",
        "data_type": "WikibaseItem"
    },
    "P8387": {
        "label": "Foreign war churchyards in Norway ID",
        "description": "identifier for foreign war churchyards in Norway",
        "data_type": "ExternalId"
    },
    "P4447": {
        "label": "minimum spend bonus",
        "description": "reward for spending a sum of money",
        "data_type": "Quantity"
    },
    "P1137": {
        "label": "fossil found in this unit",
        "description": "fossils that are found in this stratigraphic unit",
        "data_type": "WikibaseItem"
    },
    "P5257": {
        "label": "BirdLife taxon ID",
        "description": "identifier for an avian taxon, in the BirdLife factsheet database",
        "data_type": "ExternalId"
    },
    "P6590": {
        "label": "highest break",
        "description": "highest break made by a snooker player in their career or in a tournament",
        "data_type": "Quantity"
    },
    "P9144": {
        "label": "National Portrait Gallery (London) artwork ID",
        "description": "identifier for an artwork on the website of the National Portrait Gallery, in London",
        "data_type": "ExternalId"
    },
    "P9121": {
        "label": "National Museum Norway artwork ID",
        "description": "Identifier for an artwork in the Norwegian National Museum collections",
        "data_type": "ExternalId"
    },
    "P4079": {
        "label": "Theatres Trust Database ID",
        "description": "identifier of the Theatres Trust Database",
        "data_type": "ExternalId"
    },
    "P4535": {
        "label": "EUTA theatre ID",
        "description": "identifier for a theatre in the European Theatre Architecture (EUTA) database",
        "data_type": "ExternalId"
    },
    "P9061": {
        "label": "National Museum in Warsaw ID",
        "description": "numerical identifier for an artwork on the website of the National Museum in Warsaw, in Poland",
        "data_type": "ExternalId"
    },
    "P6238": {
        "label": "Monument aux morts ID",
        "description": "identifier for a memorial in the University of Lille database",
        "data_type": "ExternalId"
    },
    "P9080": {
        "label": "JAANUS ID",
        "description": "identifier in Japanese Architecture and Art Net Users System",
        "data_type": "ExternalId"
    },
    "P8905": {
        "label": "Museo del Prado ID",
        "description": "identifier for an artwork on the Museo del Prado website",
        "data_type": "ExternalId"
    },
    "P8906": {
        "label": "National Gallery of Ireland ID",
        "description": "identifier for an artwork on the National Gallery of Ireland website",
        "data_type": "ExternalId"
    },
    "P9394": {
        "label": "Louvre Museum ARK ID",
        "description": "ARK identifier for works in the Louvre Museum",
        "data_type": "ExternalId"
    },
    "P8907": {
        "label": "Auckland Art Gallery artwork ID",
        "description": "identifier for an artwork on the Auckland Art Gallery website",
        "data_type": "ExternalId"
    },
    "P8820": {
        "label": "Monuments de mémoire ID",
        "description": "identifier for a French war memorial on the Monuments de mémoire website",
        "data_type": "ExternalId"
    },
    "P4534": {
        "label": "EUTA person ID",
        "description": "identifier for a person in the European Theatre Architecture (EUTA) database",
        "data_type": "ExternalId"
    },
    "P8909": {
        "label": "National Gallery of Australia ID",
        "description": "identifier for an artwork on the National Gallery of Australia website",
        "data_type": "ExternalId"
    },
    "P8910": {
        "label": "Fitzwilliam Museum ID",
        "description": "identifier for an artwork on the Fitzwilliam Museum website",
        "data_type": "ExternalId"
    },
    "P7430": {
        "label": "Dictionary of Sculptors in Britain ID",
        "description": "identifier for a sculptor in A Biographical Dictionary of Sculptors in Britain, 1660-1851",
        "data_type": "ExternalId"
    },
    "P8107": {
        "label": "bus",
        "description": "bus used on computer hardware, mainly relevant for supercomputer nodes interconnect",
        "data_type": "WikibaseItem"
    },
    "P4721": {
        "label": "MuBE Virtual ID",
        "description": "identifier for a sculpture in Brazil, in the MuBE database",
        "data_type": "ExternalId"
    },
    "P8962": {
        "label": "PBA artwork ID",
        "description": "identifier for an artwork on the Palais des Beaux-Arts de Lille website",
        "data_type": "ExternalId"
    },
    "P8970": {
        "label": "Chinese Painting Database ID",
        "description": "identifier for artworks in the Chinese Painting Database",
        "data_type": "ExternalId"
    },
    "P9167": {
        "label": "De Lakenhal artwork ID",
        "description": "identifier of an artwork of Museum De Lakenhal in Leiden",
        "data_type": "ExternalId"
    },
    "P8916": {
        "label": "DIA ID",
        "description": "identifier for an artwork on the Detroit Institute of Arts website",
        "data_type": "ExternalId"
    },
    "P8917": {
        "label": "SKD ID",
        "description": "identifier for an artwork on the Staatliche Kunstsammlungen Dresden website",
        "data_type": "ExternalId"
    },
    "P9131": {
        "label": "Royal Museums Greenwich artwork ID",
        "description": "numerical identifier for an artwork on the website of the Royal Museums Greenwich",
        "data_type": "ExternalId"
    },
    "P9132": {
        "label": "Saint Louis Art Museum artwork ID",
        "description": "numerical identifier for an artwork on the Saint Louis Art Museum website",
        "data_type": "ExternalId"
    },
    "P6889": {
        "label": "microarchitecture",
        "description": "processor microarchitecture",
        "data_type": "WikibaseItem"
    },
    "P3386": {
        "label": "French Sculpture Census work ID",
        "description": "identifier of a sculpture in the French Sculpture Census",
        "data_type": "ExternalId"
    },
    "P8922": {
        "label": "Amsterdam Museum ID",
        "description": "numerical identifier for an artwork on the Amsterdam Museum website",
        "data_type": "ExternalId"
    },
    "P8923": {
        "label": "SMB-digital ID",
        "description": "numerical identifier for an object on the online collections database of the National Museums in Berlin",
        "data_type": "ExternalId"
    },
    "P6113": {
        "label": "Playbill venue ID",
        "description": "identifier for a performance venue at Playbill.com",
        "data_type": "ExternalId"
    },
    "P8924": {
        "label": "State Catalogue of the Museum Fund of Russia artwork ID",
        "description": "identifier for an artwork in the State Catalogue of the Museum Fund of Russia",
        "data_type": "ExternalId"
    },
    "P9088": {
        "label": "KBR Catalogue ID",
        "description": "identifier for an item in the online public catalogue of the Royal Library of Belgium (KBR)",
        "data_type": "ExternalId"
    },
    "P2037": {
        "label": "GitHub username",
        "description": "username of this project, person or organization on GitHub",
        "data_type": "ExternalId"
    },
    "P9091": {
        "label": "MNBAQ ID",
        "description": "numerical identifier for an artwork on the website of the Musée national des beaux-arts du Québec",
        "data_type": "ExternalId"
    },
    "P8946": {
        "label": "National Galleries of Scotland ID",
        "description": "identifier for an artwork on the National Galleries of Scotland website",
        "data_type": "ExternalId"
    },
    "P9403": {
        "label": "Musée national de la Marine ID",
        "description": "identifier for an object or artwork in the collection of Musée national de la Marine (MnM)",
        "data_type": "ExternalId"
    },
    "P3711": {
        "label": "Vanderkrogt.net Statues ID",
        "description": "identifier for a public sculpture in the Vanderkrogt.net Statues – Hither & Thither online database",
        "data_type": "ExternalId"
    },
    "P7078": {
        "label": "typing discipline",
        "description": "specifies the typing discipline implemented, used, or mandated in a programming language",
        "data_type": "WikibaseItem"
    },
    "P9254": {
        "label": "Museo Nacional de la Estampa ID",
        "description": "identifier for an artwork image in the Museo Nacional de la Estampa database",
        "data_type": "ExternalId"
    },
    "P8948": {
        "label": "Pinakothek artwork ID",
        "description": "identifier for an artwork on the Bavarian State Painting Collections website",
        "data_type": "ExternalId"
    },
    "P9095": {
        "label": "Kunstmuseum Basel ID",
        "description": "numerical identifier for an artwork on the Kunstmuseum Basel website",
        "data_type": "ExternalId"
    },
    "P5802": {
        "label": "stage crew member",
        "description": "member of the technical crew who supervises and operates the various technical aspects of the production during a stage performance",
        "data_type": "WikibaseItem"
    },
    "P8959": {
        "label": "Georgia Museum of Art ID",
        "description": "identifier for an artwork on the Georgia Museum of Art website",
        "data_type": "ExternalId"
    },
    "P8960": {
        "label": "Kunsthistorisches Museum Wien ID",
        "description": "identifier for an artwork on the website of the Kunsthistorisches Museum, in Vienna, Austria",
        "data_type": "ExternalId"
    },
    "P5411": {
        "label": "CeBeDem composer ID",
        "description": "identifier for a composer on the CeBeDem website",
        "data_type": "ExternalId"
    },
    "P2546": {
        "label": "sidekick of",
        "description": "close companion of a fictional character",
        "data_type": "WikibaseItem"
    },
    "P4084": {
        "label": "MyAnimeList people ID",
        "description": "identifier for a person or group of people on MyAnimeList",
        "data_type": "ExternalId"
    },
    "P8449": {
        "label": "Churches in Limburg ID",
        "description": "identifier of an object in the Churches in Limburg database",
        "data_type": "ExternalId"
    },
    "P2089": {
        "label": "Library of Congress JukeBox ID (former scheme)",
        "description": "former identifier in the Library of Congress JukeBox database of singers",
        "data_type": "ExternalId"
    },
    "P2164": {
        "label": "SIGIC author ID",
        "description": "SIGIC - Slovenian Music Information Center (property for author/person)",
        "data_type": "ExternalId"
    },
    "P3478": {
        "label": "Songkick artist ID",
        "description": "identifier for an artist on Songkick",
        "data_type": "ExternalId"
    },
    "P3674": {
        "label": "Mutopia composer ID",
        "description": "identifier for a composer at Mutopia Project",
        "data_type": "ExternalId"
    },
    "P1952": {
        "label": "Encyclopaedia Metallum band ID",
        "description": "identifier for a band in the Encyclopaedia Metallum database",
        "data_type": "ExternalId"
    },
    "P1955": {
        "label": "Discogs label ID",
        "description": "identifier for a record label, studio or series in the Discogs database",
        "data_type": "ExternalId"
    },
    "P7371": {
        "label": "Beatport label ID",
        "description": "identifier for a label on Beatport.com music site",
        "data_type": "ExternalId"
    },
    "P4138": {
        "label": "Treasury of Lives ID",
        "description": "identifier for a person in the biographical encyclopedia of biographies related to Tibet, Inner Asia, and the Himalayan region",
        "data_type": "ExternalId"
    },
    "P6772": {
        "label": "Buddhist Author Authority Database ID",
        "description": "identifier of a person or an organization in the Buddhist Author Authority Database",
        "data_type": "ExternalId"
    },
    "P6965": {
        "label": "IFPI Austria artist ID",
        "description": "identifier for an artist on IFPI Austria music site",
        "data_type": "ExternalId"
    },
    "P3839": {
        "label": "Tab4u artist ID",
        "description": "ID in the Israeli artist database \"Tab4u\"",
        "data_type": "ExternalId"
    },
    "P5649": {
        "label": "AniDB person ID",
        "description": "identifier for a person in AniDB database (human or organization)",
        "data_type": "ExternalId"
    },
    "P4607": {
        "label": "Swedish Musical Heritage ID",
        "description": "identifier for the Swedish Musical Heritage database",
        "data_type": "ExternalId"
    },
    "P6882": {
        "label": "RIAA artist ID",
        "description": "identifier for an artist on RIAA music site",
        "data_type": "ExternalId"
    },
    "P3913": {
        "label": "MobyGames developer ID",
        "description": "ID of a developer at MobyGames",
        "data_type": "ExternalId"
    },
    "P7313": {
        "label": "Rate Your Music label ID",
        "description": "identifier for a record label at Rate Your Music",
        "data_type": "ExternalId"
    },
    "P5174": {
        "label": "Rockipedia label ID",
        "description": "identifier of a music label in the online service Rockipedia",
        "data_type": "ExternalId"
    },
    "P6926": {
        "label": "Nederlandse Top 40 artist ID",
        "description": "identifier for an artist on Nederlandse Top 40 music site",
        "data_type": "ExternalId"
    },
    "P2338": {
        "label": "Musopen composer ID",
        "description": "identifier for a composer in the musopen.org database",
        "data_type": "ExternalId"
    },
    "P6921": {
        "label": "MangaSeek person ID",
        "description": "identifier for a mangaka on the MangaSeek website",
        "data_type": "ExternalId"
    },
    "P8688": {
        "label": "CompositionToday composer ID",
        "description": "composer ID on the website CompositionToday",
        "data_type": "ExternalId"
    },
    "P839": {
        "label": "IMSLP ID",
        "description": "identifier on the International Music Score Library Project",
        "data_type": "ExternalId"
    },
    "P6969": {
        "label": "BVMI artist ID",
        "description": "identifier for an artist on BVMI music site",
        "data_type": "ExternalId"
    },
    "P1188": {
        "label": "Dharma Drum Buddhist College place ID",
        "description": "identifier for a place in DDBC",
        "data_type": "ExternalId"
    },
    "P2510": {
        "label": "National Discography of Italian Song artist/group ID",
        "description": "artist or music group identity code on the National Discography of Italian Song  website",
        "data_type": "ExternalId"
    },
    "P6971": {
        "label": "IFPI Norge artist ID",
        "description": "identifier for an artist on IFPI Norge music site",
        "data_type": "ExternalId"
    },
    "P6893": {
        "label": "Offizielle Deutsche Charts artist ID",
        "description": "identifier for an artist on Offizielle Deutsche Charts music site",
        "data_type": "ExternalId"
    },
    "P5796": {
        "label": "Internet Game Database person ID",
        "description": "identifier for a person on the Internet Game Database",
        "data_type": "ExternalId"
    },
    "P4071": {
        "label": "Zemereshet artist ID",
        "description": "identifier for an artist in the Israeli database for lyrics, Zemereshet",
        "data_type": "ExternalId"
    },
    "P8166": {
        "label": "Encyclopaedia Metallum label ID",
        "description": "identifier for a label in the Encyclopaedia Metallum database",
        "data_type": "ExternalId"
    },
    "P7138": {
        "label": "Acharts.co chart ID",
        "description": "identifier for a record chart on Acharts.co",
        "data_type": "ExternalId"
    },
    "P3952": {
        "label": "Stereo Ve Mono artist ID",
        "description": "ID in the Israeli artist database \"Stereo Ve Mono\"",
        "data_type": "ExternalId"
    },
    "P2514": {
        "label": "Jamendo artist ID",
        "description": "identifier for an artist per the Jamendo website",
        "data_type": "ExternalId"
    },
    "P8863": {
        "label": "Qobuz label ID",
        "description": "identifier for a label in the Qobuz Store",
        "data_type": "ExternalId"
    },
    "P8323": {
        "label": "Monasteries in the Netherlands until 1800 ID",
        "data_type": "ExternalId"
    },
    "P6983": {
        "label": "Sverigetopplistan artist ID",
        "description": "identifier for an artist on Sverigetopplistan music site",
        "data_type": "ExternalId"
    },
    "P9062": {
        "label": "BABEL author ID",
        "description": "identifier for an author in the database about Patristic sources BABEL",
        "data_type": "ExternalId"
    },
    "P2909": {
        "label": "SecondHandSongs artist ID",
        "description": "identifier for an artist or group in the SecondHandSongs database",
        "data_type": "ExternalId"
    },
    "P4034": {
        "label": "Shironet artist ID",
        "description": "ID of an artist in the Israeli Shironet database",
        "data_type": "ExternalId"
    },
    "P2373": {
        "label": "Genius artist ID",
        "description": "identifier for an artist or label on Genius",
        "data_type": "ExternalId"
    },
    "P6966": {
        "label": "IFPI Danmark artist ID",
        "description": "identifier for an artist on IFPI Danmark music site",
        "data_type": "ExternalId"
    },
    "P6943": {
        "label": "VG-lista artist ID",
        "description": "identifier for an artist on VG-lista music site",
        "data_type": "ExternalId"
    },
    "P2624": {
        "label": "MetroLyrics ID",
        "description": "identifier for a song or artist at the MetroLyrics website",
        "data_type": "ExternalId"
    },
    "P5359": {
        "label": "Operone composer ID",
        "description": "identifier for a composer on the Operone website",
        "data_type": "ExternalId"
    },
    "P1982": {
        "label": "Anime News Network person ID",
        "description": "identifier for a person or group of people on animenewsnetwork.com",
        "data_type": "ExternalId"
    },
    "P1989": {
        "label": "Encyclopaedia Metallum artist ID",
        "description": "Identifier for a person in the Encyclopaedia Metallum database",
        "data_type": "ExternalId"
    },
    "P3511": {
        "label": "VGMDb organization ID",
        "description": "identifier for a record label, game developer, or other organization in the Video Game Music database",
        "data_type": "ExternalId"
    },
    "P3017": {
        "label": "Rolling Stone artist ID",
        "description": "identifier for an artist (person or group) in Rolling Stone magazine's website",
        "data_type": "ExternalId"
    },
    "P7047": {
        "label": "enemy of",
        "description": "opponent character or group of this fictive character or group",
        "data_type": "WikibaseItem"
    },
    "P7193": {
        "label": "YesAsia author ID",
        "description": "identifier for an author on YesAsia site",
        "data_type": "ExternalId"
    },
    "P3307": {
        "label": "Galiciana author ID",
        "description": "identifier of an author in Galiciana",
        "data_type": "ExternalId"
    },
    "P5413": {
        "label": "Ricochet author ID",
        "description": "identifier for an author on the Ricochet website",
        "data_type": "ExternalId"
    },
    "P6745": {
        "label": "Orlando author ID",
        "description": "identifier for an author in the Orlando reference work",
        "data_type": "ExternalId"
    },
    "P3154": {
        "label": "Runeberg author ID",
        "description": "identifier for an author in the Runeberg digital library",
        "data_type": "ExternalId"
    },
    "P5408": {
        "label": "Fantastic Fiction author ID",
        "description": "identifier for an author on the Fantastic Fiction website",
        "data_type": "ExternalId"
    },
    "P5414": {
        "label": "The Paris Review interviewee ID",
        "description": "identifier for an interviewee on the Paris Review website",
        "data_type": "ExternalId"
    },
    "P5639": {
        "label": "LARB contributor ID",
        "description": "identifier for a contributor on the Los Angeles Review of Books website",
        "data_type": "ExternalId"
    },
    "P6784": {
        "label": "Harper's author ID",
        "description": "identifier for a person who wrote for Harper's Magazine",
        "data_type": "ExternalId"
    },
    "P5618": {
        "label": "LOA ID",
        "description": "identifier for a writer on the Library of America website",
        "data_type": "ExternalId"
    },
    "P6441": {
        "label": "Douban author ID",
        "description": "identifier for an author at the Douban book website",
        "data_type": "ExternalId"
    },
    "P6226": {
        "label": "ARLLFB member ID",
        "description": "identifier for a member of the 'Académie royale de langue et de littérature françaises de Belgique' on its website",
        "data_type": "ExternalId"
    },
    "P5498": {
        "label": "Base de datos de premiados person ID",
        "description": "identifier for a person in the Base de datos de premiados del Libro, Lectura y Letras database",
        "data_type": "ExternalId"
    },
    "P5635": {
        "label": "ANZL writer ID",
        "description": "identifier for a writer on the Academy of New Zealand Literature website",
        "data_type": "ExternalId"
    },
    "P5527": {
        "label": "Academia Brasileira de Letras ID",
        "description": "identifier for a member on the Academia Brasileira de Letras website",
        "data_type": "ExternalId"
    },
    "P2799": {
        "label": "BVMC person ID",
        "description": "identifier of an author on the Biblioteca Virtual Miguel de Cervantes",
        "data_type": "ExternalId"
    },
    "P2191": {
        "label": "Vegetti Catalog of Fantastic Literature NILF ID",
        "description": "numeric identifier within the Vegetti Catalog of Fantastic Literature",
        "data_type": "ExternalId"
    },
    "P5695": {
        "label": "Bibliopoche author ID",
        "description": "identifier for an author in the Bibliopoche database",
        "data_type": "ExternalId"
    },
    "P6862": {
        "label": "digilibLT author ID",
        "description": "identifier for an author in the digilibLT digital library",
        "data_type": "ExternalId"
    },
    "P4629": {
        "label": "Online Books Page author ID",
        "description": "identifier for an author, at the Online Books Page website",
        "data_type": "ExternalId"
    },
    "P7356": {
        "label": "DC Books author ID",
        "description": "external identifier for authors whose books were published by DC Books",
        "data_type": "ExternalId"
    },
    "P6941": {
        "label": "PHI Latin Texts author ID",
        "description": "identifier for an author in the PHI Latin Texts digital library",
        "data_type": "ExternalId"
    },
    "P5613": {
        "label": "Bibliothèque de la Pléiade ID",
        "description": "identifier for an author on the Bibliothèque de la Pléiade website",
        "data_type": "ExternalId"
    },
    "P5101": {
        "label": "Swedish Literature Bank Author ID",
        "description": "identifier for an author in the Swedish Literature Bank",
        "data_type": "ExternalId"
    },
    "P6791": {
        "label": "The Atlantic author ID",
        "description": "identifier for a person who wrote for the magazine 'The Atlantic'",
        "data_type": "ExternalId"
    },
    "P8122": {
        "label": "DLL Catalog author ID",
        "description": "identifier for a Latin author in the catalog Digital Latin Library",
        "data_type": "ExternalId"
    },
    "P8160": {
        "label": "Bibliotheca Augustana author ID",
        "description": "identifier for an author in the website Bibliotheca Augustana",
        "data_type": "ExternalId"
    },
    "P5637": {
        "label": "CNL author ID",
        "description": "identifier for an author on the Centre national du livre website",
        "data_type": "ExternalId"
    },
    "P5039": {
        "label": "Authorea author ID",
        "description": "identifier for an author on the Authorea writing service",
        "data_type": "ExternalId"
    },
    "P5714": {
        "label": "Tor.com author ID",
        "description": "content by an author on the Tor.com website",
        "data_type": "ExternalId"
    },
    "P5705": {
        "label": "Los Angeles Review of Books author ID",
        "description": "identifier for an author on the Los Angeles Review of Books website",
        "data_type": "ExternalId"
    },
    "P2734": {
        "label": "Unz Review author ID",
        "description": "author identifier at The Unz Review (unz.org), a content-archiving website",
        "data_type": "ExternalId"
    },
    "P6873": {
        "label": "IntraText author ID",
        "description": "identifier for an author in the IntraText digital library",
        "data_type": "ExternalId"
    },
    "P7400": {
        "label": "LibraryThing author ID",
        "description": "authority control for authors on LibraryThing",
        "data_type": "ExternalId"
    },
    "P6831": {
        "label": "Pinakes author ID",
        "description": "identifier for an author in the Pinakes database",
        "data_type": "ExternalId"
    },
    "P6984": {
        "label": "Rupa Publications author ID",
        "description": "external identifier for authors whose books were published by Rupa Publications",
        "data_type": "ExternalId"
    },
    "P6999": {
        "label": "Musisque Deoque author ID",
        "description": "identifier for an author in the Musisque Deoque digital library",
        "data_type": "ExternalId"
    },
    "P6660": {
        "label": "Rxivist author ID",
        "description": "identifier for the author of scientific or academic preprint papers, in the Rxivist database",
        "data_type": "ExternalId"
    },
    "P7543": {
        "label": "ZSI author ID",
        "description": "identifier for authors who wrote for ZSI publications",
        "data_type": "ExternalId"
    },
    "P7031": {
        "label": "Penguin India author ID",
        "description": "external identifier for authors whose books were published by Penguin India publishing house",
        "data_type": "ExternalId"
    },
    "P7041": {
        "label": "Perseus author ID",
        "description": "identifier for an author in the Perseus digital library of ancient texts",
        "data_type": "ExternalId"
    },
    "P5365": {
        "label": "Internet Book Database of Fiction writer ID",
        "description": "identifier for a writer on the Internet Book Database of Fiction website",
        "data_type": "ExternalId"
    },
    "P6579": {
        "label": "National Book Foundation author ID",
        "description": "identifier for an author on the website of the National Book Foundation",
        "data_type": "ExternalId"
    },
    "P7935": {
        "label": "Corpus Corporum author ID",
        "description": "identifier for a Latin author in the database Corpus Corporum",
        "data_type": "ExternalId"
    },
    "P4180": {
        "label": "GujLit Person ID",
        "description": "identifier for a person, in the GujLit database of literature of the Gujarati language of India",
        "data_type": "ExternalId"
    },
    "P7986": {
        "label": "Mirabile author ID",
        "description": "identifier for medieval Latin writers",
        "data_type": "ExternalId"
    },
    "P3436": {
        "label": "AELG ID",
        "description": "identifier for a writer in the Galician Writers Association website",
        "data_type": "ExternalId"
    },
    "P7208": {
        "label": "Liber Liber author ID",
        "description": "identifier for an author on the website Liber Liber",
        "data_type": "ExternalId"
    },
    "P5409": {
        "label": "Île en île author ID",
        "description": "identifier for an author on the 'Île en île' website",
        "data_type": "ExternalId"
    },
    "P5364": {
        "label": "British Council writer ID",
        "description": "identifier for a writer on the British Council website",
        "data_type": "ExternalId"
    },
    "P7042": {
        "label": "The Latin Library author ID",
        "description": "identifier for an author in The Latin Library digital library",
        "data_type": "ExternalId"
    },
    "P8065": {
        "label": "CIRIS author ID",
        "description": "identifier for ancient authors in the CIRIS database",
        "data_type": "ExternalId"
    },
    "P2886": {
        "label": "Shakeosphere person ID",
        "description": "identifier in the Shakeosphere database of the University of Iowa",
        "data_type": "ExternalId"
    },
    "P7331": {
        "label": "Letteraturaitaliana.net author ID",
        "description": "identifier for an author of Italian literature on the website Letteraturaitaliana.net",
        "data_type": "ExternalId"
    },
    "P6447": {
        "label": "Douban Read author ID",
        "description": "identifier for a celebrity at the Douban Read website",
        "data_type": "ExternalId"
    },
    "P9343": {
        "label": "temples.ru ID",
        "description": "identifier for temple on the temples.ru website",
        "data_type": "ExternalId"
    },
    "P6788": {
        "label": "messes.info Catholic parish ID",
        "description": "identifier for a Catholic parish on ''messes.info''",
        "data_type": "ExternalId"
    },
    "P1848": {
        "label": "protected areas INPN Code",
        "description": "identifier from the Inventaire national du patrimoine naturel for protected areas",
        "data_type": "ExternalId"
    },
    "P3412": {
        "label": "Canadian Environmental Sustainability Indicators ID (Protected areas)",
        "description": "identifier of a protected area of Canada used by the Canadian Environmental Sustainability Indicators",
        "data_type": "ExternalId"
    },
    "P3401": {
        "label": "Wilderness Connect ID",
        "description": "identifier for an American wilderness area in the Wilderness Connect database",
        "data_type": "ExternalId"
    },
    "P8422": {
        "label": "EHESS ID of a French commune",
        "description": "identifier with digits to describe of French commune by EHESS",
        "data_type": "ExternalId"
    },
    "P1732": {
        "label": "Naturbase ID",
        "description": "identifier in the Naturbase database published by the Norwegian Environment Agency",
        "data_type": "ExternalId"
    },
    "P5946": {
        "label": "SUCA code",
        "description": "іdentifier of a specific urban {{Q|83620}} in the Madrid Region ({{Q|5756}}) provided by the {{Q|52637792}} catalog group of postal directions of {{Q|52775017}}",
        "data_type": "ExternalId"
    },
    "P3613": {
        "label": "Naturvårdsregistret ID",
        "description": "identifier for an area protected under the Environmental Code of Sweden",
        "data_type": "ExternalId"
    },
    "P8322": {
        "label": "cadastral municipality ID in Austria",
        "data_type": "ExternalId"
    },
    "P809": {
        "label": "WDPA ID",
        "description": "identifier in World Database on Protected Areas",
        "data_type": "ExternalId"
    },
    "P6602": {
        "label": "Rhineland-Palatinate protected area ID",
        "description": "identifier for nature and landscape protected areas in Rhineland-Palatinate (Germany), issued by the Landschaftsinformationssystem Rheinland-Pfalz der Naturschutzverwaltung",
        "data_type": "ExternalId"
    },
    "P3422": {
        "label": "INSEE countries and foreign territories code",
        "description": "French identifier for countries and foreign territories",
        "data_type": "ExternalId"
    },
    "P4800": {
        "label": "EUAP ID",
        "description": "identifier for an Italian protected natural area in the official list of protected natural areas (EUAP - elenco ufficiale delle aree naturali protette)",
        "data_type": "ExternalId"
    },
    "P5180": {
        "label": "CIVICUS Monitor country entry",
        "description": "page for the country at the CIVICUS Monitor",
        "data_type": "ExternalId"
    },
    "P5611": {
        "label": "BeWeb church ID",
        "description": "identifier for an Italian church, in the Italian Episcopal Conference database",
        "data_type": "ExternalId"
    },
    "P4533": {
        "label": "Czech street ID",
        "description": "identifier for a named street (or other public space) in the official registry of the Czech Republic",
        "data_type": "ExternalId"
    },
    "P3425": {
        "label": "Natura 2000 site ID",
        "description": "identifier for protected areas within the Natura 2000 network of the European Union",
        "data_type": "ExternalId"
    },
    "P8911": {
        "label": "Sonneveld-index",
        "description": "identifier in the database of religious buildings assembled by Jan Sonneveld",
        "data_type": "ExternalId"
    },
    "P2194": {
        "label": "PSS-archi architect ID",
        "description": "identifier for an architect in the PSS-archi database",
        "data_type": "ExternalId"
    },
    "P3810": {
        "label": "Parks.it ID",
        "description": "identifier for an Italian protected area on the Parks.it website",
        "data_type": "ExternalId"
    },
    "P2467": {
        "label": "Global Geoparks Network ID",
        "data_type": "ExternalId"
    },
    "P6560": {
        "label": "Saxony-Anhalt protected area ID",
        "description": "identifier for nature and landscape protected areas in Saxony-Anhalt (Germany), issued by the Landesverwaltungsamt Sachsen-Anhalt",
        "data_type": "ExternalId"
    },
    "P4083": {
        "label": "Park Alpha Code",
        "description": "4-letter identifier for a U.S. National Park System unit on the website of the National Park Service",
        "data_type": "ExternalId"
    },
    "P677": {
        "label": "ÚSOP code",
        "description": "identifier in drusop.nature.cz, a national database of protected areas and trees",
        "data_type": "ExternalId"
    },
    "P3974": {
        "label": "nature preserve in North Rhine-Westphalia ID",
        "description": "ID of a nature preserve in North Rhine-Westphalia as defined by the State Agency for Nature, Environment and Consumer Protection North Rhine-Westphalia",
        "data_type": "ExternalId"
    },
    "P8114": {
        "label": "Wikiparques ID",
        "description": "identifier for a Brazilian protected area on Wikiparques",
        "data_type": "ExternalId"
    },
    "P4182": {
        "label": "National Natural Landmarks site ID",
        "description": "identifier an American National Natural Landmark on the United States' National Park Service website",
        "data_type": "ExternalId"
    },
    "P7308": {
        "label": "National Wildlife Refuge Trails ID",
        "description": "identifier for a refuge or one of its trails on the National Wildlife Refuge Trails website",
        "data_type": "ExternalId"
    },
    "P2621": {
        "label": "Site of Special Scientific Interest (England) ID",
        "description": "identifier for Sites of Special Scientific Interest (SSSIs) in England",
        "data_type": "ExternalId"
    },
    "P3009": {
        "label": "Conservatoire du littoral ID",
        "description": "identifier on the Conservatoire du littoral website",
        "data_type": "ExternalId"
    },
    "P3516": {
        "label": "National Park Foundation ID",
        "description": "identifier for a protected area on the US National Park Foundation's website",
        "data_type": "ExternalId"
    },
    "P3515": {
        "label": "NPCA ID",
        "description": "identifier for a protected area on the US National Parks Conservation Association's website",
        "data_type": "ExternalId"
    },
    "P2385": {
        "label": "French diocesan architects ID",
        "description": "identifier for a person in the French diocesan architects index",
        "data_type": "ExternalId"
    },
    "P1854": {
        "label": "Kyiv street code",
        "description": "identifier for a street of Kyiv",
        "data_type": "ExternalId"
    },
    "P8897": {
        "label": "Reliwiki page ID",
        "description": "identifier of the page in the Reliwiki database of Dutch places of worship",
        "data_type": "ExternalId"
    },
    "P8542": {
        "label": "Macedonian cadastral municipality ID",
        "description": "identifier of Macedonian cadastral municipalities",
        "data_type": "ExternalId"
    },
    "P2525": {
        "label": "Ramsar Sites Information Service ID",
        "description": "identifier on the Ramsar Sites Information Service of a Ramsar site",
        "data_type": "ExternalId"
    },
    "P6280": {
        "label": "Berlin protected area ID",
        "description": "identifier of protected areas in Berlin, Germany",
        "data_type": "String"
    },
    "P2763": {
        "label": "Danish protected area ID",
        "description": "identifier for Danish protected area",
        "data_type": "ExternalId"
    },
    "P4029": {
        "label": "Latvian Protected Nature Territory ID",
        "description": "identifier for geographical territories under special state-level protection in Latvia",
        "data_type": "ExternalId"
    },
    "P3296": {
        "label": "DRÚSOP ID",
        "description": "identifier used to format links in a database of Czech protected areas and trees",
        "data_type": "ExternalId"
    },
    "P6230": {
        "label": "Bavarian protected area ID",
        "description": "identifier for nature and landscape protected areas in Bavaria (Germany), issued by the Bayerisches Landesamt für Umwelt",
        "data_type": "ExternalId"
    },
    "P4154": {
        "label": "National Forest Foundation ID",
        "description": "identifier for an American national forest on the National Forest Foundation website",
        "data_type": "ExternalId"
    },
    "P7129": {
        "label": "PEPC park ID",
        "description": "identifier for a protected area on the Planning, Environment and Public Comment website of the National Park Service",
        "data_type": "ExternalId"
    },
    "P4001": {
        "label": "Latvian Protected Nature Territory URL",
        "description": "URL of official information pages about Latvian Protected Nature Territories from Nature Conservation Agency",
        "data_type": "Url"
    },
    "P4762": {
        "label": "Common Database on Designated Areas ID",
        "description": "identifier in Common Database on Designated Areas",
        "data_type": "ExternalId"
    },
    "P6700": {
        "label": "Saxony protected area ID",
        "description": "identifier for nature and landscape protected areas in Saxony (Germany), issued by the Sächsisches Staatsministerium für Umwelt und Landwirtschaft",
        "data_type": "ExternalId"
    },
    "P3327": {
        "label": "Réserves naturelles de France ID",
        "data_type": "ExternalId"
    },
    "P8651": {
        "label": "Open Civic Data Division ID",
        "description": "identifier scheme for assigning globally unique identifiers to divisions used in civic datasets and tools",
        "data_type": "ExternalId"
    },
    "P8384": {
        "label": "locality number in Austria",
        "data_type": "ExternalId"
    },
    "P8955": {
        "label": "Chapels of southern Bohemia ID",
        "description": "identifier on pages about small monuments of Southern Bohemia",
        "data_type": "ExternalId"
    },
    "P630": {
        "label": "Paris city digital code",
        "description": "identifier for a street in Paris",
        "data_type": "ExternalId"
    },
    "P2258": {
        "label": "mobile country code",
        "description": "identifier for countries in ITU-T Recommendation E.212 for use in describing mobile networks",
        "data_type": "String"
    },
    "P3498": {
        "label": "ZNIEFF ID",
        "description": "identifier for a 'Zone naturelle d'intérêt écologique' on the Muséum national d'histoire naturelle's official website",
        "data_type": "ExternalId"
    },
    "P6478": {
        "label": "Mecklenburg-Vorpommern protected area ID",
        "description": "identifier for nature and landscape protected areas in Mecklenburg-Vorpommern (Germany), issued by the Ministerium für Landwirtschaft und Umwelt Mecklenburg-Vorpommern",
        "data_type": "ExternalId"
    },
    "P3024": {
        "label": "ITU letter code",
        "description": "identifier for member countries of the  International Telecommunication Union radiocommunication division",
        "data_type": "ExternalId"
    },
    "P8573": {
        "label": "LSG localbody code",
        "description": "Localbody code (political territorial entity) in Local Self Government Department (Kerala) site",
        "data_type": "ExternalId"
    },
    "P8498": {
        "label": "road number formatter",
        "description": "template from which “$1” can be automatically replaced by a bare road number to obtain the alphanumeric abbreviation of a numbered road",
        "data_type": "String"
    },
    "P2418": {
        "label": "Structurae person ID",
        "description": "identifier for a person in the Structurae database",
        "data_type": "ExternalId"
    },
    "P901": {
        "label": "FIPS 10-4 (countries and regions)",
        "description": "Identifier for countries and regions per former Federal Information Processing Standard FIPS 10-4",
        "data_type": "ExternalId"
    },
    "P5862": {
        "label": "Lepidoptera of Belgium ID",
        "description": "identifier in the Catalogue of Lepidoptera of Belgium",
        "data_type": "ExternalId"
    },
    "P4855": {
        "label": "Phasmida Species File ID",
        "description": "identifier for a Phasmida taxon, in the Phasmida SpeciesFile website",
        "data_type": "ExternalId"
    },
    "P5984": {
        "label": "APNI ID",
        "description": "identifier for a plant, in the Australian Plant Name index",
        "data_type": "ExternalId"
    },
    "P6105": {
        "label": "Observation.org ID",
        "description": "identifier on the Observation.org websites",
        "data_type": "ExternalId"
    },
    "P3795": {
        "label": "Flora of Israel Online plant ID",
        "description": "identifier for a plant taxon or cultivar in the Flora of Israel Online database",
        "data_type": "ExternalId"
    },
    "P3606": {
        "label": "BOLD Systems taxon ID",
        "description": "identifier for a taxon in boldsystems.org",
        "data_type": "ExternalId"
    },
    "P6003": {
        "label": "Center for Biological Diversity ID",
        "description": "identifier for a taxon on the Center for Biological Diversity website",
        "data_type": "ExternalId"
    },
    "P6115": {
        "label": "Digital Flora of Central Africa ID",
        "description": "identifier for a species on the Digital Flora of Central Africa website",
        "data_type": "ExternalId"
    },
    "P5037": {
        "label": "Plants of the World online ID",
        "description": "identifier of a (vascular) plant name in the Plants of the World online database",
        "data_type": "ExternalId"
    },
    "P5179": {
        "label": "FloraCatalana ID",
        "description": "identifier for a plant taxon in the FloraCatalana database",
        "data_type": "ExternalId"
    },
    "P3398": {
        "label": "Butterflies and Moths of North America ID",
        "description": "identifier for taxa in the \"Butterflies and Moths of North America\" database",
        "data_type": "ExternalId"
    },
    "P6046": {
        "label": "MNHN taxon ID",
        "description": "identifier for a taxon on the Muséum national d'histoire naturelle website",
        "data_type": "ExternalId"
    },
    "P4728": {
        "label": "uBio ID",
        "description": "identifier for a taxon, in the uBio 'Namebank' database",
        "data_type": "ExternalId"
    },
    "P6021": {
        "label": "Hymenoptera Online taxon ID",
        "description": "identifier for a taxon on the Hymenoptera Online website",
        "data_type": "ExternalId"
    },
    "P6019": {
        "label": "Species at Risk public registry ID",
        "description": "identifier for a taxon on the Canadian 'Species at Risk' public registry website",
        "data_type": "ExternalId"
    },
    "P3186": {
        "label": "TAXREF ID",
        "description": "identifier for a biological taxon in TAXREF, the national taxonomic reference for fauna, flora and Fungi of metropolitan France and overseas developed by the National Museum of Natural History",
        "data_type": "ExternalId"
    },
    "P4125": {
        "label": "Titan ID",
        "description": "identifier for a taxon (Cerambycidae) in the Titan database",
        "data_type": "ExternalId"
    },
    "P6052": {
        "label": "Cockroach Species File ID",
        "description": "identifier for a taxon on the Cockroach Species File website",
        "data_type": "ExternalId"
    },
    "P6053": {
        "label": "Coreoidea Species File ID",
        "description": "identifier for a taxon on the Coreoidea Species File website",
        "data_type": "ExternalId"
    },
    "P5221": {
        "label": "Tree of Life Web Project ID",
        "description": "identifier in the Tree of Life Web Project",
        "data_type": "ExternalId"
    },
    "P6035": {
        "label": "C.V. Starr Virtual Herbarium ID",
        "description": "identifier for a taxon on the Steere Herbarium website",
        "data_type": "ExternalId"
    },
    "P6044": {
        "label": "FEIS ID",
        "description": "identifier for a taxon on the Fire Effects Information System website",
        "data_type": "ExternalId"
    },
    "P6034": {
        "label": "Plant Finder ID",
        "description": "identifier for a taxon in Plant Finder, on the Missouri Botanical Garden website",
        "data_type": "ExternalId"
    },
    "P5055": {
        "label": "IRMNG ID",
        "description": "identifier of a scientific name, in the Interim Register of Marine and Nonmarine Genera (IRMNG) database",
        "data_type": "ExternalId"
    },
    "P6061": {
        "label": "Plant Parasites of Europe ID",
        "description": "identifier for a taxon at the Plant Parasites of Europe database",
        "data_type": "ExternalId"
    },
    "P6030": {
        "label": "ECOS ID",
        "description": "identifier for a taxon on the United States' 'Environmental Conservation Online System' website",
        "data_type": "ExternalId"
    },
    "P6096": {
        "label": "FLOW ID",
        "description": "identifier for a species on the Fulgoromorpha Lists On the Web website",
        "data_type": "ExternalId"
    },
    "P6098": {
        "label": "Info Flora ID",
        "description": "identifier for a taxon on the Info Flora website",
        "data_type": "ExternalId"
    },
    "P6114": {
        "label": "Portal to the Flora of Italy ID",
        "description": "identifier for a species on the Portal to the Flora of Italy website",
        "data_type": "ExternalId"
    },
    "P6057": {
        "label": "Systematic Catalog of Culicidae ID",
        "description": "identifier for a taxon at the Systematic Catalog of Culicidae",
        "data_type": "ExternalId"
    },
    "P6050": {
        "label": "OSF Online ID",
        "description": "identifier for a taxon on the Orthoptera Species File Online website",
        "data_type": "ExternalId"
    },
    "P6056": {
        "label": "Red List of South African Plants ID",
        "description": "identifier for a species on the Red List of South African Plants website",
        "data_type": "ExternalId"
    },
    "P3420": {
        "label": "Calflora ID",
        "description": "identifier for a taxon in Calflora",
        "data_type": "ExternalId"
    },
    "P6128": {
        "label": "Threatened Species Link ID",
        "description": "identifier for a Tasmanian species on the Threatened Species Link website",
        "data_type": "ExternalId"
    },
    "P3405": {
        "label": "Nederlands Soortenregister ID",
        "description": "identifier for a taxon in the ''Nederlands Soortenregister'', a database of taxa in the Netherlands by Naturalis Biodiversity Center",
        "data_type": "ExternalId"
    },
    "P4433": {
        "label": "Indian Foundation for Butterflies ID",
        "description": "identifier from Indian organization of butterfly biologists working on the conservation of butterflies of India. They maintain an internet-based and peer-reviewed resource devoted to Indian butterflies.",
        "data_type": "ExternalId"
    },
    "P4194": {
        "label": "CNPS ID",
        "description": "identifier in the California Native Plant Society database",
        "data_type": "ExternalId"
    },
    "P6137": {
        "label": "APA ID",
        "description": "identifier for a species on the Alabama Plant Atlas website",
        "data_type": "ExternalId"
    },
    "P6094": {
        "label": "FloraWeb ID",
        "description": "identifier for a species on the FloraWeb website",
        "data_type": "ExternalId"
    },
    "P4122": {
        "label": "PalDat plant ID",
        "description": "identifier for a plant species in the Palynological Database (PalDat) for pollen of the University of Vienna",
        "data_type": "ExternalId"
    },
    "P4311": {
        "label": "FOIH taxon ID",
        "description": "identifier for a taxon in the thesaurus of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P6139": {
        "label": "Virtual Guide to the Flora of Mongolia ID",
        "description": "identifier for a species on the Virtual Guide to the Flora of Mongolia website",
        "data_type": "ExternalId"
    },
    "P6092": {
        "label": "Map of Life ID",
        "description": "identifier for a taxon on the Map of Life website",
        "data_type": "ExternalId"
    },
    "P6093": {
        "label": "AfroMoths ID",
        "description": "identifier for a species on the AfroMoths website",
        "data_type": "ExternalId"
    },
    "P6103": {
        "label": "Michigan Flora ID",
        "description": "identifier for a species on the Michigan Flora website",
        "data_type": "ExternalId"
    },
    "P3151": {
        "label": "iNaturalist taxon ID",
        "description": "identifier in iNaturalist",
        "data_type": "ExternalId"
    },
    "P4753": {
        "label": "Ecocrop ID",
        "description": "identifier of a plant used as an agricultural crop in the Ecocrop database",
        "data_type": "ExternalId"
    },
    "P3746": {
        "label": "Wildflowers of Israel ID",
        "description": "identifier of a taxon in the Wildflowers of Israel database",
        "data_type": "ExternalId"
    },
    "P6055": {
        "label": "Mantodea Species File ID",
        "description": "identifier for a taxon on the Mantodea Species File website",
        "data_type": "ExternalId"
    },
    "P4301": {
        "label": "PfaF ID",
        "description": "identifier for a plant taxon, in the Plants For A Future database of uses of plants and their parts",
        "data_type": "ExternalId"
    },
    "P1920": {
        "label": "CWGC burial ground ID",
        "description": "identifier for a cemetery, churchyard, or memorial, in the online database of Commonwealth War Graves Commission",
        "data_type": "ExternalId"
    },
    "P4843": {
        "label": "development of anatomical structure",
        "description": "biological development of this anatomical structure",
        "data_type": "WikibaseItem"
    },
    "P3310": {
        "label": "muscle action",
        "description": "action a muscle engages in",
        "data_type": "WikibaseItem"
    },
    "P6244": {
        "label": "Volksbund ID",
        "description": "German War Graves Commission cemetery identification",
        "data_type": "ExternalId"
    },
    "P7949": {
        "label": "Czech War Graves Register",
        "description": "identifier in a database of war graves and other related memorials of the Czech Ministry of Defence",
        "data_type": "ExternalId"
    },
    "P8286": {
        "label": "Olympedia sportsperson ID",
        "description": "identifier for an athlete, coach, referee, etc. at olympedia.org",
        "data_type": "ExternalId"
    },
    "P928": {
        "label": "activating neurotransmitter",
        "description": "which neurotransmitter activates the neuron",
        "data_type": "WikibaseItem"
    },
    "P3531": {
        "label": "AZBilliards ID",
        "description": "identifier of a person at AZBilliards",
        "data_type": "ExternalId"
    },
    "P2025": {
        "label": "Find A Grave cemetery ID",
        "description": "identifier assigned to a cemetery at Find A Grave",
        "data_type": "ExternalId"
    },
    "P2286": {
        "label": "arterial supply",
        "description": "arterial supply of an anatomical structure",
        "data_type": "WikibaseItem"
    },
    "P2697": {
        "label": "ESPNcricinfo.com player ID",
        "description": "identifier for a cricket player at ESPNcricinfo.com",
        "data_type": "ExternalId"
    },
    "P4352": {
        "label": "BillionGraves cemetery ID",
        "description": "identifier for a burial ground, on the BillionGraves website",
        "data_type": "ExternalId"
    },
    "P1447": {
        "label": "Sports-Reference.com Olympic athlete ID",
        "description": "identifier for an Olympic athlete at www.sports-reference.com/olympics/athletes/",
        "data_type": "ExternalId"
    },
    "P8797": {
        "label": "French Paralympic and Sports Committee athlete ID",
        "description": "identifier for a French athlete on the French Paralympic and Sports Committee's website",
        "data_type": "ExternalId"
    },
    "P8932": {
        "label": "Czech Gymnastics athlete ID",
        "description": "numerical identifier for a Czech gymnast on gymfed.cz",
        "data_type": "ExternalId"
    },
    "P8421": {
        "label": "Spanish Olympic Committee athlete ID",
        "description": "identifier for an athlete on the website of the Spanish Olympic Committee (''Comité Olímpico Español'')",
        "data_type": "ExternalId"
    },
    "P2289": {
        "label": "venous drainage",
        "description": "vein draining the anatomical structure",
        "data_type": "WikibaseItem"
    },
    "P2464": {
        "label": "BugGuide ID",
        "description": "identifier in BugGuide.net",
        "data_type": "ExternalId"
    },
    "P5137": {
        "label": "item for this sense",
        "description": "concept corresponding to this sense of a lexeme. Do not use on items or properties",
        "data_type": "WikibaseItem"
    },
    "P9312": {
        "label": "AWB lemma ID",
        "description": "identifier for this lexeme in the Old High German dictionary at awb.saw-leipzig.de",
        "data_type": "ExternalId"
    },
    "P6571": {
        "label": "collective noun for animals",
        "description": "applicable term to describe a group of such animals (e.g. \"swarm\" for \"bee\" as in \"a swarm of bees\")",
        "data_type": "WikibaseLexeme"
    },
    "P7486": {
        "label": "grammatical aspect",
        "description": "grammatical category that expresses how an action, event or state, denoted by a verb, relates to the flow of time: perfective, progressive, prospective, habitual, etc.",
        "data_type": "WikibaseItem"
    },
    "P5911": {
        "label": "inflection class",
        "description": "inflection class this lexeme belongs to, pattern of forms of the lexeme",
        "data_type": "WikibaseItem"
    },
    "P5560": {
        "label": "passive voice",
        "description": "specify the type of passive voice of (German) verbs",
        "data_type": "WikibaseItem"
    },
    "P5975": {
        "label": "troponym of",
        "description": "this verb describes more narrowly the action of the more general verb used as value",
        "data_type": "WikibaseSense"
    },
    "P5713": {
        "label": "requires grammatical feature",
        "description": "grammatical features required or generally used with lexeme or form of lexeme",
        "data_type": "WikibaseItem"
    },
    "P5402": {
        "label": "homograph lexeme",
        "description": "lexeme in the same language with the same spelling as this one",
        "data_type": "WikibaseLexeme"
    },
    "P6712": {
        "label": "precedes word-initial",
        "description": "letter that a following word starts with which determines the form of the subject lexeme",
        "data_type": "WikibaseLexeme"
    },
    "P6191": {
        "label": "language style",
        "description": "to denote of the way a sense of a word is used, e.g., slang, vulgarism, babytalk, colloquialism, etc.",
        "data_type": "WikibaseItem"
    },
    "P6593": {
        "label": "hyperonym",
        "description": "word whose meaning includes this one",
        "data_type": "WikibaseSense"
    },
    "P8530": {
        "label": "alternative form",
        "description": "link to spelling variant between two or more forms",
        "data_type": "WikibaseForm"
    },
    "P9295": {
        "label": "transitivity",
        "description": "possible grammatical property of a verb accepting a object complement",
        "data_type": "WikibaseItem"
    },
    "P5186": {
        "label": "conjugation class",
        "description": "conjugation class this lexeme belongs to. On items for languages, use \"has conjugation class\" (P5206)",
        "data_type": "WikibaseItem"
    },
    "P5974": {
        "label": "antonym",
        "description": "sense of a lexeme with the opposite meaning to this sense, in the same language",
        "data_type": "WikibaseSense"
    },
    "P8376": {
        "label": "Duden ID",
        "description": "identifier of a lexeme in duden.de",
        "data_type": "ExternalId"
    },
    "P9397": {
        "label": "Lëtzebuerger Online Dictionnaire ID",
        "description": "identifier for this lexeme in the Luxembourgish dictionary at lod.lu",
        "data_type": "ExternalId"
    },
    "P5831": {
        "label": "usage example",
        "description": "example of how a lexeme might be used",
        "data_type": "Monolingualtext"
    },
    "P5191": {
        "label": "derived from",
        "description": "most direct lexeme(s) this one has developed from",
        "data_type": "WikibaseLexeme"
    },
    "P5194": {
        "label": "officialized by",
        "description": "the item which officialized this lexeme",
        "data_type": "WikibaseItem"
    },
    "P7165": {
        "label": "noun class",
        "description": "noun class of the lexeme, use \"grammatical gender\" (P5185) where applicable",
        "data_type": "WikibaseItem"
    },
    "P7706": {
        "label": "nominalized form",
        "description": "property to link a verb or adjective that is used as a noun to the lexeme for the noun",
        "data_type": "WikibaseLexeme"
    },
    "P5978": {
        "label": "classifier",
        "description": "classifier word used with this sense",
        "data_type": "WikibaseLexeme"
    },
    "P6684": {
        "label": "first attested from",
        "description": "to indicate the first ''known'' usage of a word",
        "data_type": "WikibaseItem"
    },
    "P6271": {
        "label": "demonym of",
        "description": "item that this sense is a demonym of",
        "data_type": "WikibaseItem"
    },
    "P6084": {
        "label": "location of sense usage",
        "description": "for senses which are considered regional, the main locations where the sense is used",
        "data_type": "WikibaseItem"
    },
    "P8471": {
        "label": "pertainym",
        "description": "this lexeme sense is of or pertaining to the value lexeme sense",
        "data_type": "WikibaseSense"
    },
    "P5238": {
        "label": "combines",
        "description": "lexemes combined in this lexeme",
        "data_type": "WikibaseLexeme"
    },
    "P5923": {
        "label": "creates lexeme type",
        "description": "morpheme creates this type of lexeme",
        "data_type": "WikibaseItem"
    },
    "P5976": {
        "label": "false friend",
        "description": "word in another language whose spelling is almost identical but whose meaning is different",
        "data_type": "WikibaseSense"
    },
    "P5920": {
        "label": "root",
        "description": "part of word that does not have a prefix or a suffix",
        "data_type": "WikibaseLexeme"
    },
    "P9351": {
        "label": "MHDBDB lemma ID",
        "description": "identifier for this lexeme in the Middle High German Conceptual Database at mhdbdb.sbg.ac.at",
        "data_type": "ExternalId"
    },
    "P8202": {
        "label": "Reta Vortaro ID",
        "description": "entry of an Esperanto lexeme in Reta Vortaro",
        "data_type": "ExternalId"
    },
    "P5187": {
        "label": "word stem",
        "description": "word stem of the subject lexeme",
        "data_type": "Monolingualtext"
    },
    "P5185": {
        "label": "grammatical gender",
        "description": "grammatical gender of the word",
        "data_type": "WikibaseItem"
    },
    "P8394": {
        "label": "gloss quote",
        "description": "gloss for a lexeme, which originates from a citable source",
        "data_type": "Monolingualtext"
    },
    "P5401": {
        "label": "auxiliary verb",
        "description": "auxiliary verb to be used with this verb",
        "data_type": "WikibaseLexeme"
    },
    "P7243": {
        "label": "pronunciation",
        "description": "lexeme form pronunciation, usually identical to the form's representation, but may include stress position indicator ◌́. Pronunciation audio and IPA text should be attached as qualifiers. Duplicate values are OK if more than one pronunciation exists.",
        "data_type": "Monolingualtext"
    },
    "P7186": {
        "label": "Image-Line artist ID",
        "description": "identifier for an artist on Image-Line site",
        "data_type": "ExternalId"
    },
    "P7187": {
        "label": "Jango artist ID",
        "description": "identifier for an artist on Jango site",
        "data_type": "ExternalId"
    },
    "P7205": {
        "label": "jog.fm artist ID",
        "description": "identifier for an artist on jog.fm site",
        "data_type": "ExternalId"
    },
    "P7188": {
        "label": "LiveXLive artist ID",
        "description": "identifier for an artist on LiveXLive site",
        "data_type": "ExternalId"
    },
    "P7201": {
        "label": "SoundHound artist ID",
        "description": "identifier for an artist on SoundHound site",
        "data_type": "ExternalId"
    },
    "P7196": {
        "label": "musica.com artist ID",
        "description": "identifier for an artist on musica.com site",
        "data_type": "ExternalId"
    },
    "P7198": {
        "label": "RauteMusik artist ID",
        "description": "identifier for an artist on RauteMusik site",
        "data_type": "ExternalId"
    },
    "P7052": {
        "label": "Jaxsta profile ID",
        "description": "identifier for an artist or organization on the Jaxsta website",
        "data_type": "ExternalId"
    },
    "P7355": {
        "label": "See Tickets artist ID",
        "description": "identifier for an artist on See Tickets site",
        "data_type": "ExternalId"
    },
    "P7230": {
        "label": "AccuRadio artist ID",
        "description": "identifier for an artist on AccuRadio site",
        "data_type": "ExternalId"
    },
    "P6968": {
        "label": "hitparade.ch artist (certyfication) ID",
        "description": "identifier for an artist on hitparade.ch music site",
        "data_type": "ExternalId"
    },
    "P7212": {
        "label": "LyricsTranslate ID",
        "description": "identifier on LyricsTranslate",
        "data_type": "ExternalId"
    },
    "P6912": {
        "label": "Equipboard artist ID",
        "description": "identifier for an artist on Equipboard music site",
        "data_type": "ExternalId"
    },
    "P8812": {
        "label": "IMMuB artist ID",
        "description": "identifier for an artist in the Brazilian Musical Memory Institute database",
        "data_type": "ExternalId"
    },
    "P7190": {
        "label": "RMF FM artist ID",
        "description": "identifier for an artist on RMF FM site",
        "data_type": "ExternalId"
    },
    "P7200": {
        "label": "SongMeanings artist ID",
        "description": "identifier for an artist on SongMeanings site",
        "data_type": "ExternalId"
    },
    "P8819": {
        "label": "Pitchfork artist ID",
        "description": "identifier for musical artists on the Pitchfork website",
        "data_type": "ExternalId"
    },
    "P6909": {
        "label": "Charts in France artist ID",
        "description": "identifier for an artist on Charts in France music site",
        "data_type": "ExternalId"
    },
    "P6927": {
        "label": "NME artist ID",
        "description": "identifier for an artist on NME site",
        "data_type": "ExternalId"
    },
    "P6935": {
        "label": "The DJ List artist ID",
        "description": "identifier for an artist on The DJ List music site",
        "data_type": "ExternalId"
    },
    "P5145": {
        "label": "Bugs! artist ID",
        "description": "identifier for an artist on Bugs! music site",
        "data_type": "ExternalId"
    },
    "P7191": {
        "label": "Supraphonline artist ID",
        "description": "identifier for an artist on Supraphonline site",
        "data_type": "ExternalId"
    },
    "P7192": {
        "label": "TuneIn artist ID",
        "description": "identifier for an artist on TuneIn site",
        "data_type": "ExternalId"
    },
    "P5654": {
        "label": "France Musique person ID",
        "description": "identifier for a person on the France Musique website",
        "data_type": "ExternalId"
    },
    "P4407": {
        "label": "Opera Vivra singer ID",
        "description": "identifier for a singer, on the Opera Vivra website",
        "data_type": "ExternalId"
    },
    "P7354": {
        "label": "Ticketmaster artist ID",
        "description": "identifier for an artist on Ticketmaster site",
        "data_type": "ExternalId"
    },
    "P4457": {
        "label": "DAHR artist ID",
        "description": "Identifier for an artist in the Discography of American Historical Recordings (DAHR)",
        "data_type": "ExternalId"
    },
    "P7210": {
        "label": "GreatSong artist ID",
        "description": "identifier for an artist on GreatSong site",
        "data_type": "ExternalId"
    },
    "P7317": {
        "label": "iHeartRadio artist ID",
        "description": "identifier for musical artists at iHeartRadio",
        "data_type": "ExternalId"
    },
    "P6923": {
        "label": "Moov artist ID",
        "description": "identifier for an artist on Moov music site",
        "data_type": "ExternalId"
    },
    "P6925": {
        "label": "Musicalics composer ID",
        "description": "identifier for a composer in the Musicalics database",
        "data_type": "ExternalId"
    },
    "P6911": {
        "label": "Dutch Charts artist ID",
        "description": "identifier for an artist on Dutch Charts music site",
        "data_type": "ExternalId"
    },
    "P6920": {
        "label": "Juno Download artist ID",
        "description": "identifier for an artist on Juno Download music site",
        "data_type": "ExternalId"
    },
    "P6894": {
        "label": "Beatport artist ID",
        "description": "identifier for an artist on Beatport.com music site",
        "data_type": "ExternalId"
    },
    "P5240": {
        "label": "RollDaBeats artist ID",
        "description": "identifier for a musical artist on the RollDaBeats website",
        "data_type": "ExternalId"
    },
    "P6914": {
        "label": "Gaana.com artist ID",
        "description": "identifier for an artist on Gaana.com music site",
        "data_type": "ExternalId"
    },
    "P6918": {
        "label": "hitparade.ch artist ID",
        "description": "identifier for an artist on hitparade.ch music site",
        "data_type": "ExternalId"
    },
    "P6970": {
        "label": "Music Canada artist ID",
        "description": "identifier for an artist on Music Canada music site",
        "data_type": "ExternalId"
    },
    "P6967": {
        "label": "SNEP artist ID",
        "description": "identifier for an artist on SNEP music site",
        "data_type": "ExternalId"
    },
    "P6874": {
        "label": "Musixmatch artist ID",
        "description": "identifier for an artist on Musixmatch",
        "data_type": "ExternalId"
    },
    "P6955": {
        "label": "El portal de Música artist ID",
        "description": "identifier for an artist on El portal de Música music site",
        "data_type": "ExternalId"
    },
    "P6276": {
        "label": "Amazon Music artist ID",
        "description": "artist identifier at Amazon Music",
        "data_type": "ExternalId"
    },
    "P6924": {
        "label": "Murfie artist ID",
        "description": "identifier for an artist on Murfie music site",
        "data_type": "ExternalId"
    },
    "P5287": {
        "label": "Songfacts artist ID",
        "description": "identifier for an artist on the Songfacts website",
        "data_type": "ExternalId"
    },
    "P7164": {
        "label": "Raptekster.dk ID",
        "description": "identifier for a Danish hip hop performer, group or song on Raptekster.dk",
        "data_type": "ExternalId"
    },
    "P4757": {
        "label": "Songwriters Hall of Fame ID",
        "description": "ID for a person at Songwriters Hall of Fame",
        "data_type": "ExternalId"
    },
    "P8790": {
        "label": "Opera Online performer ID",
        "description": "identifier of a performer on the Opera Online website",
        "data_type": "ExternalId"
    },
    "P4747": {
        "label": "Line Music artist ID",
        "description": "identifier for an artist on Line Music",
        "data_type": "ExternalId"
    },
    "P6446": {
        "label": "Douban musician ID",
        "description": "identifier for a musician at the Douban music website",
        "data_type": "ExternalId"
    },
    "P5927": {
        "label": "Ultimate Guitar artist ID",
        "description": "identifier for an artist on Ultimate Guitar",
        "data_type": "ExternalId"
    },
    "P7195": {
        "label": "Bandsintown artist ID",
        "description": "Identifier for an artist on Bandsintown site",
        "data_type": "ExternalId"
    },
    "P5295": {
        "label": "Lafonoteca artist ID",
        "description": "identifier for an artist on the Lafonoteca website",
        "data_type": "ExternalId"
    },
    "P5291": {
        "label": "Taratata artist ID",
        "description": "identifier for an artist on the Taratata website",
        "data_type": "ExternalId"
    },
    "P7194": {
        "label": "AZLyrics.com artist ID",
        "description": "identifier for an artist on AZLyrics.com site",
        "data_type": "ExternalId"
    },
    "P7050": {
        "label": "Album of the Year artist ID",
        "description": "identifier for an artist on the Album of the Year (AOTY) website",
        "data_type": "ExternalId"
    },
    "P7206": {
        "label": "Lyrics007 artist ID",
        "description": "identifier for an artist on Lyrics007 site",
        "data_type": "ExternalId"
    },
    "P6896": {
        "label": "Djshop artist ID",
        "description": "identifier for an artist on Djshop music site",
        "data_type": "ExternalId"
    },
    "P5410": {
        "label": "Flanders Music Centre person ID",
        "description": "identifier for a person on the Flanders Music Centre website",
        "data_type": "ExternalId"
    },
    "P5882": {
        "label": "Muziekweb performer ID",
        "description": "identifier for a performer, composer or band at Muziekweb",
        "data_type": "ExternalId"
    },
    "P5261": {
        "label": "Les Enfoirés participant ID",
        "description": "identifier for a participating celebrity in the Enfoiréthèque, the database of Les Enfoirés website",
        "data_type": "ExternalId"
    },
    "P7217": {
        "label": "MTV UK artist ID",
        "description": "identifier for an artist on MTV UK site",
        "data_type": "ExternalId"
    },
    "P7216": {
        "label": "MTV Polska artist ID",
        "description": "identifier for an artist on MTV Polska site",
        "data_type": "ExternalId"
    },
    "P8842": {
        "label": "PromoDj ID",
        "description": "identifier for a person on PromoDj.com",
        "data_type": "ExternalId"
    },
    "P5292": {
        "label": "NRJ artist ID",
        "description": "identifier for an artist on the French NRJ website",
        "data_type": "ExternalId"
    },
    "P5121": {
        "label": "All About Jazz musician ID",
        "description": "identifier for a musician or musical group at All About Jazz website",
        "data_type": "ExternalId"
    },
    "P6964": {
        "label": "Musiikkituottajat artist ID (certification)",
        "description": "identifier for an artist on Musiikkituottajat music site",
        "data_type": "ExternalId"
    },
    "P5504": {
        "label": "RISM ID",
        "description": "identifier for a person or a work in the Répertoire International des Sources Musicales database",
        "data_type": "ExternalId"
    },
    "P5356": {
        "label": "Laut.de artist ID",
        "description": "identifier for an artist on the Laut.de website",
        "data_type": "ExternalId"
    },
    "P7159": {
        "label": "Plus Music artist ID",
        "description": "identifier for an artist on Plus Music site",
        "data_type": "ExternalId"
    },
    "P7156": {
        "label": "Interia Muzyka artist ID",
        "description": "identifier for an artist on Interia Muzyka site",
        "data_type": "ExternalId"
    },
    "P7151": {
        "label": "Wynk artist ID",
        "description": "identifier for an artist on Wynk music website",
        "data_type": "ExternalId"
    },
    "P7155": {
        "label": "WP Muzyka artist ID",
        "description": "identifier for an artist on WP Muzyka site",
        "data_type": "ExternalId"
    },
    "P5404": {
        "label": "Rate Your Music artist ID",
        "description": "identifier for an artist or group on the website Rate Your Music",
        "data_type": "ExternalId"
    },
    "P5366": {
        "label": "Operissimo artist ID",
        "description": "identifier for an artist on the Operissimo website",
        "data_type": "ExternalId"
    },
    "P7189": {
        "label": "mora artist ID",
        "description": "identifier for an artist on mora site",
        "data_type": "ExternalId"
    },
    "P8617": {
        "label": "Historic Montana ID",
        "description": "numerical identifier for a structure or building on the Historic Montana website",
        "data_type": "ExternalId"
    },
    "P7789": {
        "label": "Ten-Bruggencatenummer",
        "description": "identifier in the website allemolens.nl",
        "data_type": "ExternalId"
    },
    "P1314": {
        "label": "number of spans",
        "description": "number of spans a bridge has",
        "data_type": "Quantity"
    },
    "P5011": {
        "label": "Prazdne Domy building ID",
        "description": "identifier of a building in the Czech prazdnedomy.cz database",
        "data_type": "ExternalId"
    },
    "P4088": {
        "label": "Irish National Inventory of Architectural Heritage ID",
        "description": "identifier of a structure, in National Inventory of Architectural Heritage database of Ireland",
        "data_type": "ExternalId"
    },
    "P2787": {
        "label": "longest span",
        "description": "length of the longest span of a bridge, a natural arch",
        "data_type": "Quantity"
    },
    "P4694": {
        "label": "Arquivo Arq ID",
        "description": "identifier for an architect or building, in the Arquivo Arq database",
        "data_type": "ExternalId"
    },
    "P4358": {
        "label": "Dutch lost building register ID",
        "description": "identifier for former buildings in the Netherlands, connecting them to images in public collections",
        "data_type": "ExternalId"
    },
    "P4346": {
        "label": "T4T35 megalith ID",
        "description": "identifier for a European megalith, on the T4T35 website",
        "data_type": "ExternalId"
    },
    "P4141": {
        "label": "Gatehouse Gazetteer place ID",
        "description": "identifier for the castle sites in the Gatehouse Gazetteer website",
        "data_type": "ExternalId"
    },
    "P3824": {
        "label": "VTJ-PRT building ID",
        "description": "Persistent building identifier of the Population Information System in Finland",
        "data_type": "ExternalId"
    },
    "P8429": {
        "label": "Swiss Tunnel ID",
        "description": "identifier of a Swiss tunnel in the Swiss Tunnel-Database",
        "data_type": "ExternalId"
    },
    "P6486": {
        "label": "Pacific Coast Architecture Database building ID",
        "description": "identifier for a building in the Pacific Coast Architecture Database (PCAD)",
        "data_type": "ExternalId"
    },
    "P5746": {
        "label": "Directorate General of Civil Aeronautics (Mexico) code",
        "description": "airport code for Mexico",
        "data_type": "ExternalId"
    },
    "P8668": {
        "label": "Indonesian prison database ID",
        "description": "Indonesian prison database",
        "data_type": "ExternalId"
    },
    "P8227": {
        "label": "Lower Austrian Castle ID",
        "description": "identifier in the online database for castles in Lower Austria maintained by the Institut für Realienkunde des Mittelalters und der frühen Neuzeit (University of Salzburg)",
        "data_type": "ExternalId"
    },
    "P4658": {
        "label": "Coflein ID",
        "description": "identifier for an historic site, monument or building in the National Monuments Record of Wales (NMRW) database",
        "data_type": "ExternalId"
    },
    "P4009": {
        "label": "RKY national built heritage environment ID",
        "description": "identifier in the national built heritage environments listing RKY",
        "data_type": "ExternalId"
    },
    "P4136": {
        "label": "WIGOS station ID",
        "description": "identifier of a meteorological station, in the WIGOS database",
        "data_type": "ExternalId"
    },
    "P1265": {
        "label": "AlloCiné film ID",
        "description": "identifier for a film in the French AlloCiné database",
        "data_type": "ExternalId"
    },
    "P9042": {
        "label": "DiCamillo Database Country House ID",
        "description": "identifier for country houses in Britain and Ireland",
        "data_type": "ExternalId"
    },
    "P1562": {
        "label": "AllMovie title ID",
        "description": "identifier for a work on the website AllMovie",
        "data_type": "ExternalId"
    },
    "P8191": {
        "label": "UK Modern House Index building ID",
        "description": "identifier for a house, in the UK Modern House Index",
        "data_type": "ExternalId"
    },
    "P4219": {
        "label": "Tyrolean Art Cadastre inventory ID",
        "description": "identifier of a cadastre published by the regional government of Tyrol, Austria",
        "data_type": "ExternalId"
    },
    "P6082": {
        "label": "NYC Building Identification Number (BIN)",
        "description": "unique ID used by the City of New York to identify a building",
        "data_type": "ExternalId"
    },
    "P4356": {
        "label": "Megalithic Portal ID",
        "description": "identifier for an ancient site on The Megalithic Portal",
        "data_type": "ExternalId"
    },
    "P8949": {
        "label": "Delft municipal monument ID",
        "description": "identifier used for municipal monument in the city of Delft",
        "data_type": "ExternalId"
    },
    "P8078": {
        "label": "FHF hospital group ID",
        "description": "identifier for a French hospital group on the Fédération hospitalière de France website",
        "data_type": "ExternalId"
    },
    "P7547": {
        "label": "National Health Portal hospital ID",
        "description": "identifier for hospitals in India maintained by the 'National Health Portal'",
        "data_type": "ExternalId"
    },
    "P8463": {
        "label": "SAH Archipedia building ID",
        "description": "identifier for a building on the website of the SAH Archipedia, the online encyclopedia by the Society of Architectural Historians",
        "data_type": "ExternalId"
    },
    "P8062": {
        "label": "Ameli ID",
        "description": "identifier for a French medical facility on the Ameli website",
        "data_type": "ExternalId"
    },
    "P8355": {
        "label": "Helsinki persistent building ID Ratu",
        "description": "buildings identifier of the City of Helsinki, replaced by nationwide VTJ-PRT",
        "data_type": "ExternalId"
    },
    "P480": {
        "label": "FilmAffinity ID",
        "description": "FilmAffinity identification number of a creative work",
        "data_type": "ExternalId"
    },
    "P5129": {
        "label": "Stepwell Atlas ID",
        "description": "ID for Stepwell Atlas, a collaborative map and information resource for stepwells, stepped ponds and other notable stepped water architecture",
        "data_type": "ExternalId"
    },
    "P8082": {
        "label": "Spanish National Catalog of Hospitals ID",
        "description": "identifier for a Spanish health facility in the National Catalog of Hospitals",
        "data_type": "ExternalId"
    },
    "P8616": {
        "label": "Kansas Historic Resources Inventory ID",
        "description": "identifier for a structure or building in the Kansas Historic Resources Inventory database",
        "data_type": "ExternalId"
    },
    "P905": {
        "label": "PORT film ID",
        "description": "PORT-network film database: identifier for a film or television program",
        "data_type": "ExternalId"
    },
    "P8934": {
        "label": "mailing list archive",
        "description": "archive of the mailing list for the object",
        "data_type": "Url"
    },
    "P950": {
        "label": "Biblioteca Nacional de España ID",
        "description": "identifier from the authority file of the Biblioteca Nacional de España. Format for persons: \"XX\" followed by 4 to 7 digits",
        "data_type": "ExternalId"
    },
    "P6099": {
        "label": "clinical trial phase",
        "description": "phase designation for a clinical trial",
        "data_type": "WikibaseItem"
    },
    "P3000": {
        "label": "marriageable age",
        "description": "minimum age at which a person is generally allowed by law to marry. Use qualifier \"applies to part\" (P518), to specify if it applies only to males (Q6581097), females (Q6581072) or both genders (Q27177113)",
        "data_type": "Quantity"
    },
    "P9151": {
        "label": "Can I use feature ID",
        "description": "identifier for a browser feature, technology or web standard on Can I use...",
        "data_type": "ExternalId"
    },
    "P4452": {
        "label": "Thibaudeau classification",
        "description": "a classification (created by Francis Thibaudeau) for typefaces",
        "data_type": "WikibaseItem"
    },
    "P9098": {
        "label": "BiblioLMC ID",
        "description": "identifier for a subject (person, musical work, etc.) in the Italian website BiblioLMC",
        "data_type": "ExternalId"
    },
    "P2163": {
        "label": "FAST ID",
        "description": "authority control identifier in WorldCat's “FAST Linked Data” authority file",
        "data_type": "ExternalId"
    },
    "P2999": {
        "label": "age of consent",
        "description": "youngest age at which a person can legally consent to sexual activity",
        "data_type": "Quantity"
    },
    "P8467": {
        "label": "Wiki-Rennes ID",
        "description": "identifier for an article on Wiki-Rennes",
        "data_type": "ExternalId"
    },
    "P8131": {
        "label": "dialect of computer language",
        "description": "computer language of which an item with this property is a dialect",
        "data_type": "WikibaseItem"
    },
    "P2612": {
        "label": "TED topic ID",
        "description": "identifier of a topic, in the TED database of talks",
        "data_type": "ExternalId"
    },
    "P6153": {
        "label": "research site",
        "description": "place where this item was researched",
        "data_type": "WikibaseItem"
    },
    "P2997": {
        "label": "age of majority",
        "description": "threshold of adulthood as recognized or declared in law. Use qualifiers \"statement is subject of\" (P805) to link to item for articles with more detail. Use \"start time\" (P580) or \"point in time\" (P585) for historic data",
        "data_type": "Quantity"
    },
    "P2899": {
        "label": "minimum age",
        "description": "minimum age for, for example, movie certification",
        "data_type": "Quantity"
    },
    "P3847": {
        "label": "Open Library subject ID",
        "description": "identifier for a topic in the Open Library database",
        "data_type": "ExternalId"
    },
    "P1424": {
        "label": "topic's main template",
        "description": "the main template relating to a topic",
        "data_type": "WikibaseItem"
    },
    "P8410": {
        "label": "Oxford Bibliographies ID",
        "description": "identifier for a topic in Oxford Bibliographies",
        "data_type": "ExternalId"
    },
    "P3001": {
        "label": "retirement age",
        "description": "the age at which most people normally retire from work",
        "data_type": "Quantity"
    },
    "P6249": {
        "label": "narrative age",
        "description": "age of a fictional character (use qualifiers like P1441 (present in work) to restrict the age to a certain work)",
        "data_type": "Quantity"
    },
    "P3348": {
        "label": "National Library of Greece ID",
        "description": "authority ID from the National Library of Greece Authority Records",
        "data_type": "ExternalId"
    },
    "P4099": {
        "label": "metrically compatible typeface",
        "description": "typeface metrically compatible with this typeface: glyph position, width and height match",
        "data_type": "WikibaseItem"
    },
    "P1423": {
        "label": "template has topic",
        "description": "topic related to template",
        "data_type": "WikibaseItem"
    },
    "P9112": {
        "label": "IFLA value vocabularies ID",
        "description": "identifier for a term in the IFLA value vocabularies",
        "data_type": "ExternalId"
    },
    "P8203": {
        "label": "supported metadata",
        "description": "metadata, information or properties that can be stored by a system, message or structure",
        "data_type": "WikibaseItem"
    },
    "P3553": {
        "label": "Zhihu topic ID",
        "description": "numeric id of topics (tags) of questions on Zhihu, a Chinese question-and-answer website",
        "data_type": "ExternalId"
    },
    "P4135": {
        "label": "maximum age",
        "description": "highest age a person is eligible to do something, for example be member in youth organisation or sport teams, some positions like judges, clinical trials, etc",
        "data_type": "Quantity"
    },
    "P6611": {
        "label": "Semantic Scholar topic ID",
        "description": "identifier for a research topic, in the Semantic Scholar database",
        "data_type": "ExternalId"
    },
    "P269": {
        "label": "IdRef ID",
        "description": "identifier for authority control in the French collaborative library catalog (see also P1025). Format: 8 digits followed by a digit or \"X\"",
        "data_type": "ExternalId"
    },
    "P8615": {
        "label": "Oakeshott typology",
        "description": "catalogue of medieval swords",
        "data_type": "WikibaseItem"
    },
    "P8484": {
        "label": "PM20 subject code",
        "description": "code for a subject category in the 20th Century Press Archives' subject category system",
        "data_type": "ExternalId"
    },
    "P2332": {
        "label": "Dictionary of Art Historians ID",
        "description": "identifier for a person in the Dictionary of Art Historians",
        "data_type": "ExternalId"
    },
    "P4844": {
        "label": "research intervention",
        "description": "the focus of an experiment, also called the experimental variable, research activity, or study intervention",
        "data_type": "WikibaseItem"
    },
    "P9138": {
        "label": "service status information URL",
        "description": "URI where you can find information about the status of the service",
        "data_type": "Url"
    },
    "P9294": {
        "label": "AIATSIS Subject Thesaurus ID",
        "description": "identifier for a term in the AIATSIS Subject Thesaurus",
        "data_type": "ExternalId"
    },
    "P349": {
        "label": "National Diet Library ID",
        "description": "identifier for authority control per the National Diet Library of Japan",
        "data_type": "ExternalId"
    },
    "P1005": {
        "label": "Portuguese National Library ID",
        "description": "identifier for the Biblioteca Nacional de Portugal",
        "data_type": "ExternalId"
    },
    "P1878": {
        "label": "Vox-ATypI classification",
        "description": "classification for typefaces",
        "data_type": "WikibaseItem"
    },
    "P1648": {
        "label": "Dictionary of Welsh Biography ID",
        "description": "identifier per the Dictionary of Welsh Biography",
        "data_type": "ExternalId"
    },
    "P8259": {
        "label": "FlashScore.com player ID",
        "description": "identifier for a player at the FlashScore.com",
        "data_type": "ExternalId"
    },
    "P8736": {
        "label": "Opta football player ID",
        "description": "identifier of an association football player in Opta Sports",
        "data_type": "ExternalId"
    },
    "P8740": {
        "label": "The Boardr profile ID",
        "description": "identifier for a skateboarder or sport cyclist on The Boardr",
        "data_type": "ExternalId"
    },
    "P8321": {
        "label": "Canadian Football League player ID",
        "description": "identifier for a player on the Canadian Football League website",
        "data_type": "ExternalId"
    },
    "P2985": {
        "label": "DBSE ID",
        "description": "identifier in the Diccionario Biográfico del Socialismo Español",
        "data_type": "ExternalId"
    },
    "P2029": {
        "label": "Dictionary of Ulster Biography ID",
        "description": "identifer for a person in the Dictionary of Ulster Biography",
        "data_type": "ExternalId"
    },
    "P8845": {
        "label": "FIS grass skier ID",
        "description": "identifier for a grass skier in the International Ski Federation database",
        "data_type": "ExternalId"
    },
    "P1907": {
        "label": "Australian Dictionary of Biography ID",
        "description": "article about the person in the Australian Dictionary of Biography",
        "data_type": "ExternalId"
    },
    "P2180": {
        "label": "Kansallisbiografia ID",
        "description": "identifier for a person in the Kansallisbiografia (National Biography of Finland)",
        "data_type": "ExternalId"
    },
    "P8996": {
        "label": "Swiss Unihockey player ID",
        "description": "numerical identifier for a floorball player on the Swiss Unihockey website",
        "data_type": "ExternalId"
    },
    "P8787": {
        "label": "UAF person ID",
        "description": "identifier for a person at the site Ukrainian Athletic Federation",
        "data_type": "ExternalId"
    },
    "P8997": {
        "label": "IFF player ID",
        "description": "numerical identifier for a floorball player on the International Floorball Federation website",
        "data_type": "ExternalId"
    },
    "P9150": {
        "label": "FloGrappling ID",
        "description": "identifier for a Brazilian Jiu Jitsu athlete, in the FloGrappling database",
        "data_type": "ExternalId"
    },
    "P7551": {
        "label": "Plusliga player ID",
        "description": "identifier for a player on the Polish Plusliga website",
        "data_type": "ExternalId"
    },
    "P1254": {
        "label": "Slovenska biografija ID",
        "description": "identifier in the \"Slovenska biografija\" reference project",
        "data_type": "ExternalId"
    },
    "P2745": {
        "label": "Dictionary of New Zealand Biography ID",
        "description": "identifier in the Dictionary of New Zealand Biography",
        "data_type": "ExternalId"
    },
    "P781": {
        "label": "SIKART ID",
        "description": "identifier in SIKART, a biographical dictionary and a database on visual art in Switzerland and Liechtenstein",
        "data_type": "ExternalId"
    },
    "P9303": {
        "label": "svoyak.info player ID",
        "description": "identifier for a player at the Base of Sport SI",
        "data_type": "ExternalId"
    },
    "P2753": {
        "label": "Dictionary of Canadian Biography ID",
        "description": "entry of the item in the Dictionary of Canadian Biography",
        "data_type": "ExternalId"
    },
    "P8634": {
        "label": "SPoT skater ID",
        "description": "identifier of a skateboarder in the Skatepark of Tampa database",
        "data_type": "ExternalId"
    },
    "P1986": {
        "label": "Treccani's Dizionario biografico degli italiani ID",
        "description": "identifier of the Biographical Dictionary of Italian People",
        "data_type": "ExternalId"
    },
    "P2277": {
        "label": "Magdeburger Biographisches Lexikon",
        "description": "biographical article about people from the Magdeburg area",
        "data_type": "ExternalId"
    },
    "P708": {
        "label": "diocese",
        "description": "administrative division of the church to which the element belongs; use P5607 for other types of ecclesiastical territorial entities",
        "data_type": "WikibaseItem"
    },
    "P7136": {
        "label": "Goibibo Hotel ID",
        "description": "external identifiers for Goibibo hotels",
        "data_type": "ExternalId"
    },
    "P4803": {
        "label": "Amtrak station code",
        "description": "Station identifier code for the Amtrak system",
        "data_type": "ExternalId"
    },
    "P3898": {
        "label": "Hotels.com hotel ID",
        "description": "identifier for a hotel on the Hotels.com website",
        "data_type": "ExternalId"
    },
    "P6269": {
        "label": "API endpoint",
        "description": "base URL of a web service",
        "data_type": "Url"
    },
    "P7133": {
        "label": "Cleartrip hotel ID",
        "description": "external identifiers for Cleartrip hotels",
        "data_type": "ExternalId"
    },
    "P4206": {
        "label": "FOIH person ID",
        "description": "identifier for a person or an organization in the database of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P1377": {
        "label": "MTR station code",
        "description": "identifier for a station in Hong Kong issued by MTR",
        "data_type": "ExternalId"
    },
    "P5651": {
        "label": "Expedia hotel ID",
        "description": "identifier for an hotel on the Expedia website",
        "data_type": "ExternalId"
    },
    "P8671": {
        "label": "Deutsche Bahn station code",
        "description": "identifier for train stations and other operating points used by Deutsche Bahn",
        "data_type": "String"
    },
    "P3426": {
        "label": "ArbetSam ID",
        "description": "identifier in Working Life Museums Co-operation Council's and the Museum of Work's database of working-life museums in Sweden (Arbetslivsmuseer)",
        "data_type": "ExternalId"
    },
    "P2563": {
        "label": "superhuman feature or ability",
        "description": "superhuman, supernatural, or paranormal abilities that the fictional subject exhibits",
        "data_type": "WikibaseItem"
    },
    "P5734": {
        "label": "Historic Hotels of America ID",
        "description": "identifier for an hotel on the Historic Hotels of America website",
        "data_type": "ExternalId"
    },
    "P417": {
        "label": "patron saint",
        "description": "patron saint adopted by the subject",
        "data_type": "WikibaseItem"
    },
    "P4146": {
        "label": "Athenaeum museum ID",
        "description": "museum or other repository/source id in the Athenaeum artworks website",
        "data_type": "ExternalId"
    },
    "P8575": {
        "label": "Encyclopaedia of Architects, Vienna 1770–1945 ID",
        "description": "identifier for an architect who has worked in Vienna in the Architektenlexikon of the Architekturzentrum Wien",
        "data_type": "ExternalId"
    },
    "P3058": {
        "label": "Architectuurgids architect ID",
        "description": "identifier for an architect in the Dutch architectuurgids.nl database",
        "data_type": "ExternalId"
    },
    "P6818": {
        "label": "calendar feed URL",
        "description": "calendar feed (iCal, ICS, etc.) of this object",
        "data_type": "Url"
    },
    "P8402": {
        "label": "Open Data portal",
        "description": "the Open Data portal where more information can be found",
        "data_type": "WikibaseItem"
    },
    "P1165": {
        "label": "home world",
        "description": "home planet or natural satellite for a fictional character or species",
        "data_type": "WikibaseItem"
    },
    "P6008": {
        "label": "Agoda hotel ID",
        "description": "identifier for an hotel on the Agoda website",
        "data_type": "ExternalId"
    },
    "P722": {
        "label": "UIC station code",
        "description": "identifier for a railway station in Europe, CIS countries, China, Mongolia, North Africa and the Middle East",
        "data_type": "ExternalId"
    },
    "P6811": {
        "label": "Prazdne Domy architect ID",
        "description": "identifier of an architect in the Czech prazdnedomy.cz database",
        "data_type": "ExternalId"
    },
    "P5735": {
        "label": "Historic Hotels Worldwide ID",
        "description": "identifier for an hotel on the Historic Hotels Worldwide website",
        "data_type": "ExternalId"
    },
    "P7917": {
        "label": "Museums in Austria Code",
        "description": "Registration Code from the association of museums in Autria",
        "data_type": "ExternalId"
    },
    "P5763": {
        "label": "Directorio de Museos y Colecciones de España ID",
        "description": "identifier for a museum on the Directorio de Museos y Colecciones de España website",
        "data_type": "ExternalId"
    },
    "P8831": {
        "label": "Museum Day ID",
        "description": "identifier for a museum on the Smithsonian Magazine Museum Day website",
        "data_type": "ExternalId"
    },
    "P1983": {
        "label": "Anime News Network company ID",
        "description": "identifier of a company on Anime News Network",
        "data_type": "ExternalId"
    },
    "P9074": {
        "label": "NZ Museums ID",
        "description": "numerical identifier for New Zealand cultural organisations (museums, archives, art galleries, and their collections) listed on the NZ Museums website",
        "data_type": "ExternalId"
    },
    "P6985": {
        "label": "OYO Hotel ID",
        "description": "external identifiers for OYO hotels",
        "data_type": "ExternalId"
    },
    "P1885": {
        "label": "cathedral",
        "description": "principal church of a religious district",
        "data_type": "WikibaseItem"
    },
    "P5764": {
        "label": "IMA museum ID",
        "description": "identifier for a museum on the Irish Museums Association website",
        "data_type": "ExternalId"
    },
    "P4488": {
        "label": "COAM architect ID",
        "description": "identifier of a person that has built or projected relevant buildings or structures of the city of Madrid included in the Official Architects' Association of Madrid (COAM) database",
        "data_type": "ExternalId"
    },
    "P5308": {
        "label": "Dictionary of Scottish Architects ID",
        "description": "identifier for a person on the online Dictionary of Scottish Architects",
        "data_type": "ExternalId"
    },
    "P8448": {
        "label": "Benerail station ID",
        "description": "identifier for train stations used by Benerail/SNCB/Thalys",
        "data_type": "ExternalId"
    },
    "P5305": {
        "label": "SPARQL endpoint",
        "description": "URL of the SPARQL endpoint of the database/website",
        "data_type": "Url"
    },
    "P5105": {
        "label": "Deutsche Bahn station category",
        "description": "category a German railway station operated by DB Station&Service is assigned to",
        "data_type": "WikibaseItem"
    },
    "P1378": {
        "label": "China railway TMIS station code",
        "description": "identifier for a railway station used in China railway system",
        "data_type": "ExternalId"
    },
    "P1019": {
        "label": "web feed URL",
        "description": "news feed (RSS, Atom, etc.) of this person/organisation/project",
        "data_type": "Url"
    },
    "P539": {
        "label": "Museofile",
        "description": "identifer for a museum in the Museofile database of the French ministry of culture",
        "data_type": "ExternalId"
    },
    "P3960": {
        "label": "Base biographique AUTOR ID",
        "description": "notice of a personality linked to the funds of the Mediatheque of Architecture and Heritage (France)",
        "data_type": "ExternalId"
    },
    "P612": {
        "label": "mother house",
        "description": "principal house or community for a religious institute",
        "data_type": "WikibaseItem"
    },
    "P954": {
        "label": "IBNR ID",
        "description": "identifier for a railway station (Internationale Bahnhofsnummer)",
        "data_type": "ExternalId"
    },
    "P8868": {
        "label": "AAA hotel ID",
        "description": "identifier for an hotel on the American Automobile Association website",
        "data_type": "ExternalId"
    },
    "P2699": {
        "label": "URL",
        "description": "location of a resource",
        "data_type": "Url"
    },
    "P3607": {
        "label": "Booking.com hotel ID",
        "description": "identifier for a hotel on the Booking.com website",
        "data_type": "ExternalId"
    },
    "P4675": {
        "label": "appears in the form of",
        "description": "this fictional or mythical entity takes the form of that entity",
        "data_type": "WikibaseItem"
    },
    "P8729": {
        "label": "AniList anime ID",
        "description": "numerical identifier for an anime on AniList.co",
        "data_type": "ExternalId"
    },
    "P8098": {
        "label": "Biographical Dictionary of Architects in Canada 1800-1950 ID",
        "description": "identifier for an architect in the Biographical Dictionary of Architects in Canada website",
        "data_type": "ExternalId"
    },
    "P429": {
        "label": "dantai code",
        "description": "identifier for administrative division in 6-digit format assigned by the interior ministry of Japan",
        "data_type": "ExternalId"
    },
    "P4274": {
        "label": "Tunisian geographic code",
        "description": "Identifier for administrative regions of Tunisia",
        "data_type": "ExternalId"
    },
    "P1077": {
        "label": "KOATUU identifier",
        "description": "identifier for each of the Ukrainian administrative units (till 2020)",
        "data_type": "ExternalId"
    },
    "P964": {
        "label": "Austrian municipality key",
        "description": "identifier for municipalities in Austria",
        "data_type": "ExternalId"
    },
    "P1894": {
        "label": "Danish urban area code",
        "description": "official identifier for an urban area in Denmark",
        "data_type": "ExternalId"
    },
    "P4094": {
        "label": "Australian Standard Geographic Classification 2006 ID",
        "description": "Identifier of a geographic region defined in the Australian Standard Geographic Classification 2006",
        "data_type": "ExternalId"
    },
    "P8287": {
        "label": "Worlds Without End author ID",
        "description": "identifier for authors at the website Worlds Without End",
        "data_type": "ExternalId"
    },
    "P988": {
        "label": "Philippine Standard Geographic Code",
        "description": "systematic classification and coding of geographic areas of the Philippines.",
        "data_type": "ExternalId"
    },
    "P439": {
        "label": "German municipality key",
        "description": "identifier for municipalities and independent towns in Germany",
        "data_type": "ExternalId"
    },
    "P3012": {
        "label": "Statistics Canada Geographic code",
        "description": "system by Statistics Canada for categorizing and enumerating the census geographic units of Canada",
        "data_type": "ExternalId"
    },
    "P2788": {
        "label": "Czech neighbourhood ID code",
        "description": "identifier for a neighbourhood in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P635": {
        "label": "ISTAT ID",
        "description": "identifier for municipalities in Italy",
        "data_type": "ExternalId"
    },
    "P1567": {
        "label": "NIS/INS code",
        "description": "Belgian municipality codes maintained by Statistics Belgium",
        "data_type": "ExternalId"
    },
    "P4812": {
        "label": "Statistical Service of Cyprus Geocode",
        "description": "Statistical Codes of Municipalities, Communities and Quarters of Cyprus per the Statistical Service of Cyprus",
        "data_type": "ExternalId"
    },
    "P525": {
        "label": "Swedish municipality code",
        "description": "identifier for a municipality in Sweden \"kommunkod\" (four-digit)",
        "data_type": "ExternalId"
    },
    "P6471": {
        "label": "Den Digitale Byport ID",
        "description": "identifier for a Danish city in \"Den Digitale Byport\"",
        "data_type": "ExternalId"
    },
    "P2898": {
        "label": "Swedish county letter",
        "data_type": "ExternalId"
    },
    "P3731": {
        "label": "Serbia cadastral municipality ID",
        "description": "identifier of cadastral municipalities of Serbia",
        "data_type": "ExternalId"
    },
    "P4838": {
        "label": "CATO ID",
        "description": "Identifier in Classifier of administrative-territorial objects of Kazakhstan",
        "data_type": "ExternalId"
    },
    "P771": {
        "label": "Swiss municipality code",
        "description": "identifier for a municipality in Switzerland",
        "data_type": "ExternalId"
    },
    "P775": {
        "label": "Swedish urban area code",
        "description": "identifier for an urban area in Sweden",
        "data_type": "ExternalId"
    },
    "P440": {
        "label": "German district key",
        "description": "identifier for districts (Landkreise) and independent towns in Germany",
        "data_type": "ExternalId"
    },
    "P4335": {
        "label": "IDESCAT territorial code in Catalonia",
        "description": "official identifier for a Catalonian territorial unit, from Institut Estadístic de Catalunya (IDESCAT)",
        "data_type": "ExternalId"
    },
    "P3639": {
        "label": "KEPN ID",
        "description": "identifier for a place on the KEPN site",
        "data_type": "ExternalId"
    },
    "P9141": {
        "label": "Portal da Literatura ID",
        "description": "identifier for an author in the Portuguese website Portal da Literatura",
        "data_type": "ExternalId"
    },
    "P2585": {
        "label": "INSEE region code",
        "description": "number sequence for the identification of regions in France",
        "data_type": "ExternalId"
    },
    "P3423": {
        "label": "INSEE arrondissement code",
        "description": "number sequence for identification of departmental arrondissements in France",
        "data_type": "ExternalId"
    },
    "P772": {
        "label": "INE municipality code",
        "description": "identifier for Spanish municipalities, by the Spanish Statistical Office (INE)",
        "data_type": "ExternalId"
    },
    "P507": {
        "label": "Swedish county code",
        "description": "identifier for a county in Sweden \"länskod\" (two-digit)",
        "data_type": "ExternalId"
    },
    "P2586": {
        "label": "INSEE department code",
        "description": "number sequence for the identification of departments in France",
        "data_type": "ExternalId"
    },
    "P1115": {
        "label": "ATVK ID",
        "description": "identifier for each of the Latvian administrative units, i.e. municipality, parish, city under state jurisdiction and city under municipality jurisdiction. Based on the {{Q|15621843}}.",
        "data_type": "ExternalId"
    },
    "P4014": {
        "label": "Australian Statistical Geography 2011 ID",
        "description": "identifier of a geographic region defined in the Australian Statistical Geography Standard 2011",
        "data_type": "ExternalId"
    },
    "P382": {
        "label": "CBS municipality code",
        "description": "identifier for a Dutch municipality as defined by Statistics Netherlands (CBS)",
        "data_type": "ExternalId"
    },
    "P1653": {
        "label": "TERC municipality code",
        "description": "identifier for a municipality per the Główny Urząd Statystyczny in Poland",
        "data_type": "ExternalId"
    },
    "P9053": {
        "label": "Projecto Vercial author ID",
        "description": "identifier for an author in the website Projecto Vercial",
        "data_type": "ExternalId"
    },
    "P1203": {
        "label": "Finnish municipality number",
        "description": "identifier for a municipality in Finland",
        "data_type": "ExternalId"
    },
    "P8993": {
        "label": "JAD ID",
        "description": "numerical identifier for an author in the Journaux d'Alexandre Dumas database",
        "data_type": "ExternalId"
    },
    "P4046": {
        "label": "SIMC place ID",
        "description": "identifer for places in Poland in the SIMC system run by the Polish Central Statistical Office",
        "data_type": "ExternalId"
    },
    "P6895": {
        "label": "Bangladesh administrative division code (2017-)",
        "description": "code used for an area for census calculations in Bangladesh, based on a scheme introduced in 2017",
        "data_type": "ExternalId"
    },
    "P8504": {
        "label": "Science Fiction Awards Database author ID",
        "description": "identifier for a person in the Science Fiction Awards Database",
        "data_type": "ExternalId"
    },
    "P1116": {
        "label": "ELSTAT geographical code",
        "description": "identifier for administrative regions of Greece by the Greek Statistical Authority (starting 2010)",
        "data_type": "ExternalId"
    },
    "P5086": {
        "label": "FIPS 5-2 alpha code (US states)",
        "description": "Two-letter identifier for US states and other associated areas per former Federal Information Processing Standard FIPS 5-2 standard",
        "data_type": "ExternalId"
    },
    "P3856": {
        "label": "Quebec municipalities geographical code",
        "description": "geographical code of the municipalities, indigenous comunauties and unorganized areas in Quebec",
        "data_type": "ExternalId"
    },
    "P4119": {
        "label": "NLS Geographic Names Register Place ID",
        "description": "National Land Survey of Finland Geographic Names Register Named Place ID",
        "data_type": "ExternalId"
    },
    "P764": {
        "label": "OKTMO ID",
        "description": "identifier in Classification on Objects territory of municipal formations (Russia)",
        "data_type": "ExternalId"
    },
    "P939": {
        "label": "KSH code",
        "description": "identifier for every municipality (town and commune) in Hungary issued by the Hungarian Central Statistical Office",
        "data_type": "ExternalId"
    },
    "P442": {
        "label": "China administrative division code",
        "description": "identifier for administrative divisions of People's Republic of China (with spaces)",
        "data_type": "ExternalId"
    },
    "P1976": {
        "label": "INEGI locality ID",
        "description": "identifier for a locality of Mexico",
        "data_type": "ExternalId"
    },
    "P1172": {
        "label": "Geokod",
        "description": "identifier for historical administrative division of Sweden (1862-1951)",
        "data_type": "ExternalId"
    },
    "P1168": {
        "label": "municipality code (Denmark)",
        "description": "identifier for municipalities in Denmark",
        "data_type": "ExternalId"
    },
    "P9085": {
        "label": "Aracne author ID",
        "description": "identifier for an author in the database of the Italian academic publisher Aracne",
        "data_type": "ExternalId"
    },
    "P605": {
        "label": "NUTS code",
        "description": "identifier for a region per NUTS",
        "data_type": "ExternalId"
    },
    "P1388": {
        "label": "German regional key",
        "description": "identifier for municipalities and other areas in Germany",
        "data_type": "ExternalId"
    },
    "P3419": {
        "label": "Basic Unit of Settlement code (Czech)",
        "description": "code given to each basic unit of settlement (smallest administrative entity, below municipality) in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P6832": {
        "label": "IndicePA ID",
        "description": "identifier of public administrations in Italy",
        "data_type": "ExternalId"
    },
    "P782": {
        "label": "LAU",
        "description": "identifier for a local administrative unit, renamed from NUTS 4 and NUTS 5. Format: 2 letters followed by digits",
        "data_type": "ExternalId"
    },
    "P1370": {
        "label": "IHSI ID",
        "description": "identifier for an administrative territorial entity in Haiti",
        "data_type": "ExternalId"
    },
    "P4093": {
        "label": "Australian Statistical Geography 2016 ID",
        "description": "Identifier of a geographic region defined in the Australian Statistical Geography Standard 2016",
        "data_type": "ExternalId"
    },
    "P777": {
        "label": "Swedish civil parish code/ATA code",
        "description": "identifier for a civil parish in Sweden",
        "data_type": "ExternalId"
    },
    "P2504": {
        "label": "Norwegian municipality number",
        "description": "unique identifier for municipalities of Norway",
        "data_type": "ExternalId"
    },
    "P774": {
        "label": "FIPS 55-3 (locations in the US)",
        "description": "identifier for places in the United States per former Federal Information Processing Standard FIPS 55-3",
        "data_type": "ExternalId"
    },
    "P806": {
        "label": "Italian cadastre code",
        "description": "identifier for Italian comunes in the cadastre",
        "data_type": "ExternalId"
    },
    "P9196": {
        "label": "BookBub author ID",
        "description": "author identifier on BookBub website",
        "data_type": "ExternalId"
    },
    "P2672": {
        "label": "SOATO ID",
        "description": "identifier for the Belarusian Code System for Administrative-territory Division Objects and Settlements",
        "data_type": "ExternalId"
    },
    "P2123": {
        "label": "YerelNet village ID",
        "description": "identifier for a village, in YerelNet.org.tr, which contains detailed informations about all villages in Turkey",
        "data_type": "ExternalId"
    },
    "P776": {
        "label": "Swedish minor urban area code",
        "description": "identifier for a minor urban area in Sweden",
        "data_type": "ExternalId"
    },
    "P2290": {
        "label": "Danish parish code",
        "description": "identifier for an parish in Denmark",
        "data_type": "ExternalId"
    },
    "P3615": {
        "label": "Vision of Britain unit ID",
        "description": "identifier of an administrative unit in the University of Portsmouth's Vision of Britain database",
        "data_type": "ExternalId"
    },
    "P3727": {
        "label": "Serbia municipality ID",
        "description": "identifier of municipalities of Serbia",
        "data_type": "ExternalId"
    },
    "P882": {
        "label": "FIPS 6-4 (US counties)",
        "description": "Identifier for US counties per former Federal Information Processing Standard FIPS 6-4. Is a five-digit code in which the first 2 digits are the state code and the remaining 3 digits are the county code.",
        "data_type": "ExternalId"
    },
    "P3990": {
        "label": "BG EKATTE place ID",
        "description": "Bulgarian place ID (settlement, community=Kmetstvo, municipality=Obshtina, province=Oblast) by the BG National Statistical Institute (NSI)",
        "data_type": "ExternalId"
    },
    "P9063": {
        "label": "Base de Dados de Autores Portugueses ID",
        "description": "identifier for an author in the Base de Dados de Autores Portugueses",
        "data_type": "ExternalId"
    },
    "P9165": {
        "label": "Lexikon Literatur in Tirol ID",
        "description": "numerical identifier for a writer in the Lexikon Literatur in Tirol",
        "data_type": "ExternalId"
    },
    "P9046": {
        "label": "A*dS Encyclopedia ID",
        "description": "identifier for an author in the A*dS Encyclopedia",
        "data_type": "ExternalId"
    },
    "P374": {
        "label": "INSEE municipality code",
        "description": "identifier with 5 digits or letters for a municipality or a municipal arrondissement in France, per the National Institute of Statistics and Economic Studies",
        "data_type": "ExternalId"
    },
    "P1841": {
        "label": "Swedish district code",
        "description": "identifier for Swedish districts from 2016",
        "data_type": "ExternalId"
    },
    "P721": {
        "label": "OKATO ID",
        "description": "identifier for every administrative unit in Russia",
        "data_type": "ExternalId"
    },
    "P1067": {
        "label": "Thailand central administrative unit code",
        "description": "identifier for each of the Thai central administrative units",
        "data_type": "ExternalId"
    },
    "P843": {
        "label": "SIRUTA code",
        "description": "identifier for an area of Romania per National Institute of Statistics",
        "data_type": "ExternalId"
    },
    "P1588": {
        "label": "village code of Indonesia",
        "description": "village identifier by Statistics Indonesia",
        "data_type": "String"
    },
    "P9029": {
        "label": "Viceversa Letteratura author ID",
        "description": "identifier for an author of the Swiss literary magazine Viceversa Letteratura",
        "data_type": "ExternalId"
    },
    "P5020": {
        "label": "Code of Household Registration and Conscription Information System (Taiwan)",
        "description": "This code is official for the administrative divisions in Taiwan, and manages by the Department of Household Registration, Ministry of Interior.",
        "data_type": "ExternalId"
    },
    "P2588": {
        "label": "administrative code of Indonesia",
        "description": "unique administrative code for a place, issued by the Ministry of Home Affairs of Republic of Indonesia in 2015",
        "data_type": "ExternalId"
    },
    "P2506": {
        "label": "INSEE canton code",
        "description": "number sequence for the identification of canton in France",
        "data_type": "ExternalId"
    },
    "P1140": {
        "label": "EHAK id",
        "description": "Estonian Administrative and Settlement Classification",
        "data_type": "ExternalId"
    },
    "P8841": {
        "label": "Restaurant Guru restaurant ID",
        "description": "identifier for a restaurant on the Restaurant Guru website",
        "data_type": "ExternalId"
    },
    "P2390": {
        "label": "Ballotpedia ID",
        "description": "title of corresponding article on the Ballotpedia encyclopedia of American politics",
        "data_type": "ExternalId"
    },
    "P2281": {
        "label": "Apple Music album ID",
        "description": "identifier for a musical work in the iTunes website",
        "data_type": "ExternalId"
    },
    "P9290": {
        "label": "TikTok music ID",
        "description": "identifier for an audio track on TikTok",
        "data_type": "ExternalId"
    },
    "P9316": {
        "label": "MyMusic artist ID",
        "description": "identifier for an artist on the MyMusic website",
        "data_type": "ExternalId"
    },
    "P8862": {
        "label": "Foodlocate restaurant ID",
        "description": "identifier for restaurant on the Foodlocate website",
        "data_type": "ExternalId"
    },
    "P8704": {
        "label": "Spotify playlist ID",
        "description": "identifier for a playlist on Spotify",
        "data_type": "ExternalId"
    },
    "P6641": {
        "label": "Zomato ID",
        "description": "international dining directory and review service",
        "data_type": "ExternalId"
    },
    "P9321": {
        "label": "Inforegister ID",
        "description": "identifier for a business in Inforegister",
        "data_type": "ExternalId"
    },
    "P6381": {
        "label": "iTunes TV season ID",
        "description": "identifier for a show or a season/series of a show in the iTunes Store",
        "data_type": "ExternalId"
    },
    "P7940": {
        "label": "Adventure Gamers company ID",
        "description": "identifier of a company in the Adventure Gamers database of adventure video games",
        "data_type": "ExternalId"
    },
    "P3699": {
        "label": "Transfermarkt referee ID",
        "description": "identifier for an association football (soccer) referee in the transfermarkt.com database",
        "data_type": "ExternalId"
    },
    "P7997": {
        "label": "Gry Online company ID",
        "description": "identifier of a company in the Gry-Online database",
        "data_type": "ExternalId"
    },
    "P7755": {
        "label": "ScreenScraper company ID",
        "description": "identifier of a video game in the ScreenScraper database",
        "data_type": "ExternalId"
    },
    "P7241": {
        "label": "SCTrails trail ID",
        "description": "identifier for a trail on the South Carolina Trails website",
        "data_type": "ExternalId"
    },
    "P7233": {
        "label": "WTA trail ID",
        "description": "identifier for a Washington state trail on the Washington Trails Association website",
        "data_type": "ExternalId"
    },
    "P7634": {
        "label": "TheGamesDB developer ID",
        "description": "identifier of a video game developer in TheGamesDB database",
        "data_type": "ExternalId"
    },
    "P7435": {
        "label": "EU-Football.info referee ID",
        "description": "identifier for an association football referee on the EU-Football.info website",
        "data_type": "ExternalId"
    },
    "P9183": {
        "label": "uta-net.com artist ID",
        "description": "identifier for an artist on uta-net.com",
        "data_type": "ExternalId"
    },
    "P9328": {
        "label": "Universal Music France artist ID",
        "description": "identifier for a band or person in the site UniversalMusic.fr",
        "data_type": "ExternalId"
    },
    "P8027": {
        "label": "VGMRips company ID",
        "description": "identifier of a company in the VGMRips database",
        "data_type": "ExternalId"
    },
    "P7708": {
        "label": "SNESmusic.org company ID",
        "description": "identifier of a company in the SNESmusic.org database",
        "data_type": "ExternalId"
    },
    "P4085": {
        "label": "MyAnimeList character ID",
        "description": "identifier for a fictional character on MyAnimeList",
        "data_type": "ExternalId"
    },
    "P7521": {
        "label": "Gaming-History company ID",
        "description": "identifier of a company in the Gaming-History database",
        "data_type": "ExternalId"
    },
    "P7465": {
        "label": "Soccerbase referee ID",
        "description": "Soccerbase identifier for an association football (soccer) referee",
        "data_type": "ExternalId"
    },
    "P7321": {
        "label": "Trail Finder ID",
        "description": "identifier for a New Hampshire or Vermont trail on the Trail Finder website",
        "data_type": "ExternalId"
    },
    "P6945": {
        "label": "Steam profile ID",
        "description": "identifier for this entity on the Steam digital distribution platform",
        "data_type": "ExternalId"
    },
    "P7436": {
        "label": "WorldReferee.com referee ID",
        "description": "WorldReferee.com identifier for an association football (soccer) referee",
        "data_type": "ExternalId"
    },
    "P8254": {
        "label": "PC Games Database.de company ID",
        "description": "identifier of a company in the PC Games Database.de",
        "data_type": "ExternalId"
    },
    "P9171": {
        "label": "RILM ID",
        "description": "identifier for an entity in the Répertoire International de Littérature Musicale through VIAF",
        "data_type": "ExternalId"
    },
    "P5648": {
        "label": "AniDB character ID",
        "description": "identifier for a character in AniDB database, if multiple identifiers are available for one character, use the smallest number one",
        "data_type": "ExternalId"
    },
    "P2004": {
        "label": "NALT ID",
        "description": "identifier in the subject categories of the Agricultural Thesaurus of United States National Agricultural Library",
        "data_type": "ExternalId"
    },
    "P7310": {
        "label": "Maine Trail Finder ID",
        "description": "identifier for a trail on the Maine Trail Finder website",
        "data_type": "ExternalId"
    },
    "P7911": {
        "label": "Gamekult company ID",
        "description": "identifier of a company in the Gamekult database",
        "data_type": "ExternalId"
    },
    "P6296": {
        "label": "bgm.tv character identifier",
        "description": "identifier for a character or group of characters on bgm.tv (Bangumi/Chobits)",
        "data_type": "ExternalId"
    },
    "P8072": {
        "label": "CAB ID",
        "description": "identifier for a subject in CAB Thesaurus",
        "data_type": "ExternalId"
    },
    "P8073": {
        "label": "Cult of Saints in Late Antiquity ID",
        "description": "Identifier of an item in the Cult of Saints in Late Antiquity database",
        "data_type": "ExternalId"
    },
    "P7810": {
        "label": "LaunchBox Games Database publisher ID",
        "description": "identifier of a video game publisher in the LaunchBox Games Database",
        "data_type": "ExternalId"
    },
    "P9216": {
        "label": "Opera Online conductor ID",
        "description": "identifier of a conductor on Opera Online website",
        "data_type": "ExternalId"
    },
    "P7642": {
        "label": "TheGamesDB publisher ID",
        "description": "identifier of a video game publisher in TheGamesDB database",
        "data_type": "ExternalId"
    },
    "P7319": {
        "label": "ExplorePaTrails ID",
        "description": "identifier for a trail on the Explore Pennsylvania Trails website",
        "data_type": "ExternalId"
    },
    "P8113": {
        "label": "COTREX trail ID",
        "description": "identifier for a trail on the Colorado Trail Explorer website",
        "data_type": "ExternalId"
    },
    "P7784": {
        "label": "LaunchBox Games Database developer ID",
        "description": "identifier of a video game developer in the LaunchBox Games Database",
        "data_type": "ExternalId"
    },
    "P5847": {
        "label": "AnimeClick character ID",
        "description": "identifier of character from AnimeClick database",
        "data_type": "ExternalId"
    },
    "P9402": {
        "label": "GameSpot developer ID",
        "description": "identifier of a video game developer in the GameSpot database",
        "data_type": "ExternalId"
    },
    "P7013": {
        "label": "Anime Characters Database character ID",
        "description": "identifier of a character in the Anime Characters Database",
        "data_type": "ExternalId"
    },
    "P7560": {
        "label": "UVL company ID",
        "description": "identifier of a company in the Universal Videogame List",
        "data_type": "ExternalId"
    },
    "P7661": {
        "label": "GameFAQs credit ID",
        "description": "identifier of a game credit in the GameFAQs database",
        "data_type": "ExternalId"
    },
    "P4427": {
        "label": "GACS ID",
        "description": "identifier of a concept in the Global Agricultural Concept Scheme",
        "data_type": "ExternalId"
    },
    "P7324": {
        "label": "iHeartRadio podcast ID",
        "description": "identifier for podcasts on iHeartRadio",
        "data_type": "ExternalId"
    },
    "P6327": {
        "label": "Goodreads character ID",
        "description": "identifier for a fictional character on Goodreads",
        "data_type": "ExternalId"
    },
    "P8061": {
        "label": "AGROVOC ID",
        "description": "identifier for a subject in AGROVOC thesaurus",
        "data_type": "ExternalId"
    },
    "P7116": {
        "label": "Hiking Project trail ID",
        "description": "identifier for a trail on the Hiking Project website",
        "data_type": "ExternalId"
    },
    "P8080": {
        "label": "Ökumenisches Heiligenlexikon ID",
        "description": "identifier for a saint in the database Ökumenisches Heiligenlexikon",
        "data_type": "ExternalId"
    },
    "P7990": {
        "label": "Mirabile saint ID",
        "description": "identifier for Christian saints",
        "data_type": "ExternalId"
    },
    "P7127": {
        "label": "AllTrails trail ID",
        "description": "identifier for a trail on the AllTrails website",
        "data_type": "ExternalId"
    },
    "P4773": {
        "label": "MobyGames company ID",
        "description": "identifier for a company at MobyGames",
        "data_type": "ExternalId"
    },
    "P7570": {
        "label": "OGDB company ID",
        "description": "identifier of a company in the Online Games-Datenbank",
        "data_type": "ExternalId"
    },
    "P6725": {
        "label": "CatholicSaints.info ID",
        "description": "identifier of a catholic saint in the website CatholicSaints.info",
        "data_type": "ExternalId"
    },
    "P8660": {
        "label": "Native Plants Database ID",
        "description": "identifier for a taxon in the Native Plants Database, on the Lady Bird Johnson Wildflower Center website",
        "data_type": "ExternalId"
    },
    "P9093": {
        "label": "Australian Lichen ID",
        "description": "identifier for a taxon (a lichen), in the Australian Lichen database",
        "data_type": "ExternalId"
    },
    "P1747": {
        "label": "Flora of China ID",
        "description": "identifier for a taxon in Flora of the People's Republic of China",
        "data_type": "ExternalId"
    },
    "P6341": {
        "label": "IFPNI species ID",
        "description": "identifier for a species at the International Fossil Plant Names Index",
        "data_type": "ExternalId"
    },
    "P6347": {
        "label": "The White-files species ID",
        "description": "identifier for a species in The White-files",
        "data_type": "ExternalId"
    },
    "P6516": {
        "label": "ScaleNet ID",
        "description": "identifier for a taxon in ScaleNet",
        "data_type": "ExternalId"
    },
    "P6159": {
        "label": "Atlas of Florida Plants ID",
        "description": "identifier for a species on the Atlas of Florida Plants website",
        "data_type": "ExternalId"
    },
    "P6376": {
        "label": "Psyl'list species ID",
        "description": "identifier for a species in the Psyl'list database",
        "data_type": "ExternalId"
    },
    "P2809": {
        "label": "Australasian Pollen and Spore Atlas Code",
        "description": "identifier for a pollen or spore in the Australasian Pollen and Spore Atlas, managed by the Australian National University",
        "data_type": "ExternalId"
    },
    "P2794": {
        "label": "Index Hepaticarum ID",
        "description": "identifier in the Index Hepaticarum, a nomenclatural database",
        "data_type": "ExternalId"
    },
    "P7224": {
        "label": "Insects (Insecta) of the World ID",
        "description": "identifier for a taxon in the Insects (Insecta) of the World database",
        "data_type": "ExternalId"
    },
    "P9408": {
        "label": "Washington Flora Checklist ID",
        "description": "identifier in the Washington Flora Checklist",
        "data_type": "ExternalId"
    },
    "P1940": {
        "label": "conifers.org ID",
        "description": "identifier for a taxon in the conifers.org database",
        "data_type": "ExternalId"
    },
    "P1772": {
        "label": "USDA PLANTS ID",
        "description": "identifier in the United States Department of Agriculture PLANTS Database",
        "data_type": "ExternalId"
    },
    "P4000": {
        "label": "has fruit type",
        "description": "morphology of the fruit of this taxon, as defined in botany",
        "data_type": "WikibaseItem"
    },
    "P6209": {
        "label": "SEINet ID",
        "description": "identifier for a species on the SEINet website",
        "data_type": "ExternalId"
    },
    "P6626": {
        "label": "Index to American Botanical Literature ID",
        "description": "identifier in Index to American Botanical Literature",
        "data_type": "ExternalId"
    },
    "P2036": {
        "label": "African Plant Database ID",
        "description": "identifier for a plant taxon, in the Conservatoire et Jardin botaniques de Genève's African Plant Database of scientific names",
        "data_type": "ExternalId"
    },
    "P7255": {
        "label": "Moose Deutschland ID",
        "description": "identifier for a taxon in the Moose Deutschland database",
        "data_type": "ExternalId"
    },
    "P6904": {
        "label": "ATRF ID",
        "description": "identifier for a plant taxon, in the Australian Tropical Rainforest plants database",
        "data_type": "ExternalId"
    },
    "P6933": {
        "label": "SA Flora ID",
        "description": "identifier for a plant taxon, in the Australian 'South Australian electronic Flora' database",
        "data_type": "ExternalId"
    },
    "P6487": {
        "label": "Illustrated catalog of Tessaratomidae species ID",
        "description": "identifier for a species in the Illustrated catalog of Tessaratomidae",
        "data_type": "ExternalId"
    },
    "P2827": {
        "label": "flower color",
        "description": "colour of flowers of a plant species, hybrid or cultivar",
        "data_type": "WikibaseItem"
    },
    "P1727": {
        "label": "Flora of North America taxon ID",
        "description": "identifier for a taxon in the Flora of North America database",
        "data_type": "ExternalId"
    },
    "P1745": {
        "label": "VASCAN ID",
        "description": "identifier for a taxon in the Database of Vascular Plants of Canada",
        "data_type": "ExternalId"
    },
    "P8765": {
        "label": "Royal Horticultural Society plant ID",
        "description": "identifier for plants listed on the Royal Horticultural Society website",
        "data_type": "ExternalId"
    },
    "P1070": {
        "label": "PlantList-ID",
        "description": "identifier in 'The Plant List' database",
        "data_type": "ExternalId"
    },
    "P3105": {
        "label": "Tela Botanica ID",
        "description": "identifier for a plant taxon in Tela Botanica's 'base des trachéophytes de France métropolitaine' - BDTFX - (Metropolitan France Tracheophyte Database)",
        "data_type": "ExternalId"
    },
    "P6756": {
        "label": "Flora of Australia ID (new)",
        "description": "identifier for a plant taxon, in an Australian Commonwealth database",
        "data_type": "ExternalId"
    },
    "P7546": {
        "label": "Flowers of India ID",
        "description": "identifer for plant taxons",
        "data_type": "ExternalId"
    },
    "P8145": {
        "label": "Psocodea Species File ID",
        "description": "numerical identifier for a taxon on the Psocodea Species File project",
        "data_type": "ExternalId"
    },
    "P3739": {
        "label": "inflorescence",
        "description": "placement of flowers on the stem of a flower plant",
        "data_type": "WikibaseItem"
    },
    "P6349": {
        "label": "Lace bugs database ID",
        "description": "identifier for a species in the Lace bugs database",
        "data_type": "ExternalId"
    },
    "P7496": {
        "label": "NZPCN ID",
        "description": "ID of a plant taxon held by the New Zealand Plant Conservation Network",
        "data_type": "ExternalId"
    },
    "P960": {
        "label": "Tropicos ID",
        "description": "identifier for a name, in the Tropicos database",
        "data_type": "ExternalId"
    },
    "P6485": {
        "label": "Brentidae of the world species ID",
        "description": "identifier for a species in the Brentidae of the world database",
        "data_type": "ExternalId"
    },
    "P6481": {
        "label": "Strepsiptera database species ID",
        "description": "identifier for a species in the Strepsiptera database",
        "data_type": "ExternalId"
    },
    "P1832": {
        "label": "GrassBase ID",
        "description": "identifier in GrassBase - The Online World Grass Flora",
        "data_type": "ExternalId"
    },
    "P9076": {
        "label": "Australian Fungi ID",
        "description": "identifier for a taxon (a fungus), in the Australian 'Fungi' database",
        "data_type": "ExternalId"
    },
    "P1761": {
        "label": "Watson & Dallwitz family ID",
        "description": "familyID in Watson & Dallwitz: The families of flowering plants",
        "data_type": "ExternalId"
    },
    "P2434": {
        "label": "Panarctic Flora ID",
        "description": "identifier for a taxon in the Panarctic Flora",
        "data_type": "ExternalId"
    },
    "P6408": {
        "label": "COOL species ID",
        "description": "identifier for a species in the COOL (Cercopoidea Organised On Line) database",
        "data_type": "ExternalId"
    },
    "P961": {
        "label": "IPNI plant ID",
        "description": "numerical identifier for a plant name in the International Plant Names Index",
        "data_type": "ExternalId"
    },
    "P8341": {
        "label": "Dansk Biografisk Leksikon ID",
        "description": "identifier of a person in the online encyclopedia biografiskleksikon at lex.dk",
        "data_type": "ExternalId"
    },
    "P5505": {
        "label": "WorldCat Registry ID",
        "description": "identifier for a library, library consortium or similar institution in the Worldcat Registry",
        "data_type": "ExternalId"
    },
    "P9105": {
        "label": "Dictionnaire des journalistes ID",
        "description": "alphanumerical identifier for a person in the Dictionnaire des journalistes (1600-1789)",
        "data_type": "ExternalId"
    },
    "P5165": {
        "label": "Norsk pop- og rockleksikon ID",
        "description": "identifier of an article in the online biographic encyclopedia Norsk pop- og rocklexicon",
        "data_type": "ExternalId"
    },
    "P9363": {
        "label": "Tacoma-Pierce County Obituary Index ID",
        "description": "identifier for an entry in the Tacoma Public Library's Tacoma-Pierce County Obituary Index",
        "data_type": "ExternalId"
    },
    "P9232": {
        "label": "Obituaries Australia ID",
        "description": "identifier for a person in Obituaries Australia",
        "data_type": "ExternalId"
    },
    "P9212": {
        "label": "Dizionario biografico online degli anarchici italiani ID",
        "description": "identifier for a person in the Dizionario biografico online degli anarchici italiani",
        "data_type": "ExternalId"
    },
    "P4645": {
        "label": "RCR number",
        "description": "national identifier of a library in France assigned by Abes",
        "data_type": "ExternalId"
    },
    "P8936": {
        "label": "CollEx-Persée ID",
        "description": "identifier for an organization in the CollEx-Persée database",
        "data_type": "ExternalId"
    },
    "P8686": {
        "label": "Biographical Dictionary of Fourierism ID",
        "description": "identifier for a person on the Biographical Dictionary of Fourierism, a dictionary of actors of the Fourierism movement",
        "data_type": "ExternalId"
    },
    "P7941": {
        "label": "Biografisch Woordenboek van Nederland: 1880-2000 ID",
        "description": "identifier for an article in the Biografisch Woordenboek van Nederland: 1880-2000",
        "data_type": "ExternalId"
    },
    "P9163": {
        "label": "Obrazi slovenskih pokrajin ID",
        "description": "identifier for a person in the Obrazi slovenskih pokrajin",
        "data_type": "ExternalId"
    },
    "P5602": {
        "label": "Chercheurs d'Asie ID",
        "description": "identifier of a researcher on the online version of Chercheurs d'Asie",
        "data_type": "ExternalId"
    },
    "P6234": {
        "label": "Biographie nationale de Belgique ID",
        "description": "identifier for a person on the ''Biographie nationale de Belgique''",
        "data_type": "ExternalId"
    },
    "P6342": {
        "label": "Dictionnaire des Wallons ID",
        "description": "identifier for a person on the online ''Dictionnaire des Wallons''",
        "data_type": "ExternalId"
    },
    "P6059": {
        "label": "Biographie vosgienne ID",
        "description": "identifier for a person on 'Biographie vosgienne",
        "data_type": "ExternalId"
    },
    "P8838": {
        "label": "LibraryThing venue ID",
        "description": "identifier for a venue on the LibraryThing website",
        "data_type": "ExternalId"
    },
    "P4823": {
        "label": "American National Biography ID",
        "description": "identifier of a person in the scholarly biographical encyclopedia",
        "data_type": "ExternalId"
    },
    "P7203": {
        "label": "Dizionario biografico dei Friulani ID",
        "description": "identifier for a person in the Dizionario biografico dei Friulani",
        "data_type": "ExternalId"
    },
    "P9244": {
        "label": "Women Australia ID",
        "description": "identifier for a person in Women Australia",
        "data_type": "ExternalId"
    },
    "P8760": {
        "label": "Base constructions bibliothèques ID",
        "description": "identifier for a public library in the 'Base constructions bibliothèques' of the ENSSIB",
        "data_type": "ExternalId"
    },
    "P6829": {
        "label": "Dictionary of Irish Biography ID",
        "description": "identifier in the Dictionary of Irish Biography",
        "data_type": "ExternalId"
    },
    "P3946": {
        "label": "Dictionary Grierson ID",
        "description": "ID for argentinian scientists and researchers from the Diccionario de científicos argentinos Dra. Grierson",
        "data_type": "ExternalId"
    },
    "P4110": {
        "label": "Crunchyroll ID",
        "description": "identifier for an anime or drama topic on Crunchyroll",
        "data_type": "ExternalId"
    },
    "P4992": {
        "label": "DBA ID",
        "description": "identifier of a person in the online 'Nouveau dictionnaire de biographie alsacienne'",
        "data_type": "ExternalId"
    },
    "P6851": {
        "label": "Biographical Dictionary of Iowa ID",
        "description": "ID of entry in Biographical Dictionary of Iowa",
        "data_type": "ExternalId"
    },
    "P6858": {
        "label": "Réunionnais du monde ID",
        "description": "identifier for a person on \"Réunionnais du monde\", the online version of the biographical dictionary ''1000 célébrités de la Réunion''",
        "data_type": "ExternalId"
    },
    "P5646": {
        "label": "AniDB anime ID",
        "description": "identifier for an anime in AniDB database",
        "data_type": "ExternalId"
    },
    "P6618": {
        "label": "Federal-State Cooperative System ID",
        "description": "identifier for libraries in the United States, assigned by the Institute of Museum and Library Services",
        "data_type": "ExternalId"
    },
    "P5605": {
        "label": "DOLF ID",
        "description": "identifier of an orientalist on the online version of the Dictionnaire des orientalistes de langue française",
        "data_type": "ExternalId"
    },
    "P8888": {
        "label": "AniDB episode ID",
        "description": "identifier of this episode on AniDB",
        "data_type": "ExternalId"
    },
    "P8130": {
        "label": "Internetowy Polski Słownik Biograficzny ID",
        "description": "identifier for an article in the Internetowy Polski Słownik Biograficzny",
        "data_type": "ExternalId"
    },
    "P7023": {
        "label": "Biyografya ID",
        "description": "identifier for a person in the Internet Encyclopedia of Turkey's Famous People",
        "data_type": "ExternalId"
    },
    "P8581": {
        "label": "Hrvatski biografski leksikon ID",
        "description": "identifier for a person in the Hrvatski biografski leksikon (Croatian biographical dictionary)",
        "data_type": "ExternalId"
    },
    "P1311": {
        "label": "lostbridges.org ID",
        "description": "identifier from database of covered bridges in the United States and Canada",
        "data_type": "ExternalId"
    },
    "P6937": {
        "label": "SNBP ID",
        "description": "identifier of the Brazilian Sistema Nacional de Bibliotecas Públicas",
        "data_type": "ExternalId"
    },
    "P8985": {
        "label": "Dizionario bio-bibliografico dei bibliotecari italiani del XX secolo ID",
        "description": "identifier for a person in the Dizionario bio-bibliografico dei bibliotecari italiani del XX secolo",
        "data_type": "ExternalId"
    },
    "P7646": {
        "label": "IISG ID",
        "description": "identifier for the biographical dictionary of the  International Institute of Social History",
        "data_type": "ExternalId"
    },
    "P4943": {
        "label": "Dominicains ID",
        "description": "identifier of a Dominican monk on the online ''Dictionnaire biographique des frères prêcheurs''",
        "data_type": "ExternalId"
    },
    "P5080": {
        "label": "Norsk biografisk leksikon ID",
        "description": "identifier of an article in the online biographic encyclopedia nbl.snl.no",
        "data_type": "ExternalId"
    },
    "P9248": {
        "label": "EEVA author ID",
        "description": "identifier for an author in the Estonian database EEVA",
        "data_type": "ExternalId"
    },
    "P7744": {
        "label": "Mille Anni di Scienza in Italia ID",
        "description": "identifier for a scientist in the project Mille Anni di Scienza in Italia",
        "data_type": "ExternalId"
    },
    "P5845": {
        "label": "AnimeClick anime ID",
        "description": "identifier of anime from AnimeClick database",
        "data_type": "ExternalId"
    },
    "P8749": {
        "label": "Dicionário de Historiadores Portugueses ID",
        "description": "identifier for a subject (person, topic, institution, journal) in the Dicionário de Historiadores Portugueses",
        "data_type": "ExternalId"
    },
    "P8509": {
        "label": "Dizionario Biografico dei Protestanti in Italia ID",
        "description": "identifier for a person in the Dizionario Biografico dei Protestanti in Italia",
        "data_type": "ExternalId"
    },
    "P7017": {
        "label": "Anime Characters Database anime ID",
        "description": "identifier of an anime in the Anime Characters Database",
        "data_type": "ExternalId"
    },
    "P7342": {
        "label": "Dirección General de Bibliotecas ID",
        "description": "identifier of a Mexican public library, in the National Network of Public Libraries website",
        "data_type": "ExternalId"
    },
    "P3595": {
        "label": "Biografiskt Lexikon för Finland ID",
        "description": "identifier in the Biografiskt Lexikon för Finland released by Svenska litteratursällskapet i Finland",
        "data_type": "ExternalId"
    },
    "P7928": {
        "label": "BioLexSOE ID",
        "description": "identifier for a persona in the Biographisches Lexikon zur Geschichte Südosteuropas",
        "data_type": "ExternalId"
    },
    "P4459": {
        "label": "Spanish Biographical Dictionary ID",
        "description": "identifier for a person in the online version of the Diccionario Biográfico Español (Dictionary for Spanish Biography)",
        "data_type": "ExternalId"
    },
    "P4848": {
        "label": "Libraries.org ID",
        "description": "identifier for a library in the Libraries.org database",
        "data_type": "ExternalId"
    },
    "P8950": {
        "label": "Dizionario degli artisti ID",
        "description": "identifier for a person in the Dizionario degli artisti",
        "data_type": "ExternalId"
    },
    "P8998": {
        "label": "Magyar életrajzi lexikon ID",
        "description": "identifier for a person in the “Magyar életrajzi lexikon” (Hungarian biographical dictionary)",
        "data_type": "ExternalId"
    },
    "P9415": {
        "label": "ÖCV Biolex ID",
        "description": "identifier for a person in the ÖCV Biolex biographical dictionary",
        "data_type": "ExternalId"
    },
    "P9159": {
        "label": "People Australia ID",
        "description": "identifier of someone with a biography in People Australia",
        "data_type": "ExternalId"
    },
    "P4086": {
        "label": "MyAnimeList anime ID",
        "description": "identifier for an anime on MyAnimeList",
        "data_type": "ExternalId"
    },
    "P8719": {
        "label": "CCFr library ID",
        "description": "identifier for a library on the union catalog of France",
        "data_type": "ExternalId"
    },
    "P4124": {
        "label": "Who's Who in France biography ID",
        "description": "unique identifier in the Who's Who in France online biography dictionary",
        "data_type": "ExternalId"
    },
    "P5147": {
        "label": "Dictionary of Swedish Translators ID",
        "description": "article in the Dictionary of Swedish Translators",
        "data_type": "ExternalId"
    },
    "P4687": {
        "label": "Dictionnaire des peintres belges ID",
        "description": "identifier for a Belgian painter on the Dictionnaire des peintres belges website",
        "data_type": "ExternalId"
    },
    "P4730": {
        "label": "Mir@bel journal ID",
        "description": "identifier for a journal in the Mir@bel database",
        "data_type": "ExternalId"
    },
    "P1042": {
        "label": "ZDB ID",
        "description": "identifier for serials German National Serials Database",
        "data_type": "ExternalId"
    },
    "P1250": {
        "label": "Danish Bibliometric Research Indicator (BFI) SNO/CNO",
        "description": "serial & conference identifier",
        "data_type": "ExternalId"
    },
    "P3127": {
        "label": "Latindex ID",
        "description": "identifier of a journal in Latindex",
        "data_type": "ExternalId"
    },
    "P4722": {
        "label": "Érudit journal ID",
        "description": "identifier for a journal in Érudit",
        "data_type": "ExternalId"
    },
    "P8089": {
        "label": "EZB ID",
        "description": "identifier for a scientific journal in the Electronic Journals Library bibliographic database",
        "data_type": "ExternalId"
    },
    "P5783": {
        "label": "Cranach Digital Archive artwork ID",
        "description": "database for 1700 paintings by Cranach and his circle",
        "data_type": "ExternalId"
    },
    "P1609": {
        "label": "Dialnet journal ID",
        "description": "identifier of a journal in Dialnet",
        "data_type": "ExternalId"
    },
    "P5559": {
        "label": "MUSE journal ID",
        "description": "identifier of an academic journal in the Project MUSE",
        "data_type": "ExternalId"
    },
    "P1230": {
        "label": "JSTOR journal ID",
        "description": "identifier for a journal in JSTOR",
        "data_type": "ExternalId"
    },
    "P7368": {
        "label": "JournalBase ID",
        "description": "identifier for a journal on JournalBase",
        "data_type": "ExternalId"
    },
    "P3434": {
        "label": "ERIH PLUS ID",
        "description": "identifier of a journal in ERIH PLUS",
        "data_type": "ExternalId"
    },
    "P1272": {
        "label": "Norway Import Service and Registration Authority periodical code",
        "description": "Wikidata property for authority control for academic journals",
        "data_type": "ExternalId"
    },
    "P4741": {
        "label": "OpenEdition journal ID",
        "description": "identifier for a journal in OpenEdition Journals",
        "data_type": "ExternalId"
    },
    "P1161": {
        "label": "Z39.5 abbreviation",
        "description": "Z39.5 bibliographic code/abbreviation established in 1969 by National Information Standards Organization and used widely in the US and elsewhere",
        "data_type": "ExternalId"
    },
    "P7473": {
        "label": "Neliti Journal ID",
        "description": "journal ID/URL with Neliti, an Indonesian academic repository",
        "data_type": "ExternalId"
    },
    "P7461": {
        "label": "Publons journals/conferences ID",
        "description": "identifier for journals and conferences on the Publons website",
        "data_type": "ExternalId"
    },
    "P3131": {
        "label": "Redalyc journal ID",
        "description": "identifier of a journal in Red de Revistas Científicas de América Latina y el Caribe, España y Portugal (Redalyc)",
        "data_type": "ExternalId"
    },
    "P2733": {
        "label": "Persée journal ID",
        "description": "identifier for a journal in Persée, an open access digital library of French-language scholarly journals, established by the Ministry of National Education of France",
        "data_type": "ExternalId"
    },
    "P1156": {
        "label": "Scopus source ID",
        "description": "identifier for a source/publication venue (journal, conference, etc) in Scopus",
        "data_type": "ExternalId"
    },
    "P5115": {
        "label": "Directory of Open Access Journals ID",
        "description": "ISSN identifier of a journal in the Directory of Open Access Journals. When statements with this property need correcting, property \"ISSN\" (P236) are likely to be wrong as well",
        "data_type": "ExternalId"
    },
    "P7983": {
        "label": "ACM Journal ID",
        "description": "identifier for a journal published by ACM",
        "data_type": "ExternalId"
    },
    "P8462": {
        "label": "Political Graveyard politician ID",
        "description": "identifier for a politician in the Political Graveyard database",
        "data_type": "ExternalId"
    },
    "P6981": {
        "label": "ACNP journal ID",
        "description": "identifier of a journal in the Italian catalog of periodicals",
        "data_type": "ExternalId"
    },
    "P1277": {
        "label": "JUFO ID",
        "description": "identifier in the Julkaisufoorumi (Finnish Publication Forum)",
        "data_type": "ExternalId"
    },
    "P1159": {
        "label": "CODEN",
        "description": "CODEN bibliographic code",
        "data_type": "ExternalId"
    },
    "P1058": {
        "label": "ERA Journal ID",
        "description": "identifier for academic journals",
        "data_type": "ExternalId"
    },
    "P1270": {
        "label": "Norwegian Register journal ID",
        "description": "identifier for academic journals",
        "data_type": "ExternalId"
    },
    "P3853": {
        "label": "Rat Genome Database ID",
        "description": "Identifier for a genetic element in the Rat Genome Database",
        "data_type": "ExternalId"
    },
    "P6142": {
        "label": "Verspreidingsatlas.nl ID",
        "description": "identifier for a species on the verspreidingsatlas.nl website",
        "data_type": "ExternalId"
    },
    "P6550": {
        "label": "Pacific Coast Architecture Database firm ID",
        "description": "Identifier for a firm (company, partnership, sole proprietorship) in the Pacific Coast Architecture Database (PCAD)",
        "data_type": "ExternalId"
    },
    "P7743": {
        "label": "Microworld ID",
        "description": "identifier for a taxon on the Microworld website",
        "data_type": "ExternalId"
    },
    "P7905": {
        "label": "AnimalBase ID",
        "description": "identifier of a taxon in AnimalBase website",
        "data_type": "ExternalId"
    },
    "P2236": {
        "label": "external subproperty",
        "description": "all resources related by the property from external vocabularies are also related to this property",
        "data_type": "Url"
    },
    "P9243": {
        "label": "NParks Flora & Fauna Web ID",
        "description": "identifier for a species in NParks Flora & Fauna Web",
        "data_type": "ExternalId"
    },
    "P6177": {
        "label": "EUNIS ID for species",
        "description": "identifier for a species on the European Nature Information System website",
        "data_type": "ExternalId"
    },
    "P1212": {
        "label": "Atlas ID",
        "description": "identifier in the Atlas database of the Louvre Museum",
        "data_type": "ExternalId"
    },
    "P8468": {
        "label": "Queensland Biota ID",
        "description": "identifier for a taxon, in a Queensland Government database of species profiles",
        "data_type": "ExternalId"
    },
    "P8469": {
        "label": "Australian Weed ID",
        "description": "identifier for a taxon, in an Atlas of Living Australia database of weed profiles",
        "data_type": "ExternalId"
    },
    "P3852": {
        "label": "FlyBase Gene ID",
        "description": "Identifier for a gene in the FlyBase database of Drosophila genes and genomes",
        "data_type": "ExternalId"
    },
    "P8563": {
        "label": "Emporis company ID",
        "data_type": "ExternalId"
    },
    "P8164": {
        "label": "DPVweb ID",
        "description": "identifier of plant virus in DPV (Descriptions of Plant Viruses) database",
        "data_type": "ExternalId"
    },
    "P8892": {
        "label": "edition humboldt digital Flora ID",
        "description": "identifier for the register of plants in edition humboldt digital",
        "data_type": "ExternalId"
    },
    "P2393": {
        "label": "NCBI locus tag",
        "description": "unique gene identifier that can be used for any species of organism",
        "data_type": "ExternalId"
    },
    "P350": {
        "label": "RKDimages ID",
        "description": "identifier per RKDimages of the Netherlands Institute for Art History",
        "data_type": "ExternalId"
    },
    "P9157": {
        "label": "Open Tree of Life ID",
        "description": "identifier for an entity in Open Tree of Life, a comprehensive, dynamic and digitally-available tree of life by synthesizing published phylogenetic trees along with taxonomic data",
        "data_type": "ExternalId"
    },
    "P2092": {
        "label": "Bildindex der Kunst und Architektur ID",
        "description": "property for Bildindex",
        "data_type": "ExternalId"
    },
    "P2249": {
        "label": "RefSeq genome ID",
        "description": "ID in the RefSeq Genome database",
        "data_type": "ExternalId"
    },
    "P594": {
        "label": "Ensembl gene ID",
        "description": "identifier for a gene as per the Ensembl (European Bioinformatics Institute and the Wellcome Trust Sanger Institute) database",
        "data_type": "ExternalId"
    },
    "P1628": {
        "label": "equivalent property",
        "description": "equivalent property in other ontologies (use in statements on properties, use property URI)",
        "data_type": "Url"
    },
    "P4980": {
        "label": "Thésaurus de la désignation des œuvres architecturales et des espaces aménagés ID",
        "description": "identifier for terms in this French architectural thesaurus",
        "data_type": "ExternalId"
    },
    "P347": {
        "label": "Joconde work ID",
        "description": "identifier in the Joconde database of the French Ministry of Culture",
        "data_type": "ExternalId"
    },
    "P6864": {
        "label": "eBiodiversity ID",
        "description": "identifier for a species in the Estonian eBiodiversity portal",
        "data_type": "ExternalId"
    },
    "P354": {
        "label": "HGNC ID",
        "description": "A unique ID provided by the HGNC for each gene with an approved symbol. HGNC IDs remain stable even if a name or symbol changes.",
        "data_type": "ExternalId"
    },
    "P8792": {
        "label": "Bioweb Ecuador ID",
        "data_type": "ExternalId"
    },
    "P7552": {
        "label": "Finnish Biodiversity Information Facility's Species List ID",
        "description": "identifier for species found in Finland maintained by the Finnish Biodiversity Information Facility",
        "data_type": "ExternalId"
    },
    "P1679": {
        "label": "Art UK artwork ID",
        "description": "identifier for artworks (publicly owned oil paintings in the UK)",
        "data_type": "ExternalId"
    },
    "P7066": {
        "label": "Taxonomicon ID",
        "description": "identifier of a taxon at Taxonomicon website",
        "data_type": "ExternalId"
    },
    "P4304": {
        "label": "FOIH materials ID",
        "description": "identifier for a type of material in the thesaurus of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P7090": {
        "label": "Australian Marine Algal Name Index ID",
        "description": "identifier of a name in the Australian Marine Algal Name Index",
        "data_type": "ExternalId"
    },
    "P2394": {
        "label": "MGI Gene Symbol",
        "description": "official gene symbol for a mouse gene",
        "data_type": "ExternalId"
    },
    "P671": {
        "label": "Mouse Genome Informatics ID",
        "description": "identifier for a gene in the Mouse Genome Informatics database",
        "data_type": "ExternalId"
    },
    "P8724": {
        "label": "BHL name ID",
        "description": "identifier for a scientific name in the Biodiversity Heritage Library",
        "data_type": "ExternalId"
    },
    "P8707": {
        "label": "NBIC taxon ID",
        "description": "identifier for a taxon in the Norwegian Biodiversity Information Centre database, a database on species and ecosystems in Norway",
        "data_type": "ExternalId"
    },
    "P2235": {
        "label": "external superproperty",
        "description": "all resources related by this property are also related to that property from an external vocabulary",
        "data_type": "Url"
    },
    "P353": {
        "label": "HGNC gene symbol",
        "description": "The official gene symbol approved by the HGNC, which is typically a short form of the gene name.",
        "data_type": "ExternalId"
    },
    "P7202": {
        "label": "Belgian Species List ID",
        "description": "identifier for a taxon in the Belgian Species List",
        "data_type": "ExternalId"
    },
    "P3258": {
        "label": "LiveJournal ID",
        "description": "username of a person or organisation, on LiveJournal",
        "data_type": "ExternalId"
    },
    "P4265": {
        "label": "Reddit username",
        "description": "username on the social news website Reddit",
        "data_type": "ExternalId"
    },
    "P5163": {
        "label": "Odnoklassniki ID",
        "description": "identifier for a person or organization in Odnoklassniki",
        "data_type": "ExternalId"
    },
    "P2397": {
        "label": "YouTube channel ID",
        "description": "ID of the YouTube channel of a person or organisation (not to be confused with the name of the channel) The ID can also be used for music.youtube.com IDs.",
        "data_type": "ExternalId"
    },
    "P1381": {
        "label": "bridgehunter.com ID",
        "description": "identifier from a database of historic bridges",
        "data_type": "ExternalId"
    },
    "P1380": {
        "label": "BridgeReports.com ID",
        "description": "identifier from database based on the National Bridge Inventory. Lists every U.S. bridge over 20 feet (6 meters) long",
        "data_type": "ExternalId"
    },
    "P3676": {
        "label": "National Bridge Inventory Number",
        "description": "identifer of a bridge, in the USA's National Bridge Inventory",
        "data_type": "ExternalId"
    },
    "P3836": {
        "label": "Pinterest username",
        "description": "username of the Pinterest account of a person or an organization",
        "data_type": "ExternalId"
    },
    "P3185": {
        "label": "VK ID",
        "description": "identifier for a person or organization in VKontakte",
        "data_type": "ExternalId"
    },
    "P6459": {
        "label": "QQ user ID",
        "description": "this item's user ID on QQ",
        "data_type": "ExternalId"
    },
    "P2003": {
        "label": "Instagram username",
        "description": "item's username on Instagram",
        "data_type": "ExternalId"
    },
    "P4003": {
        "label": "Facebook page ID",
        "description": "official Facebook page of this entity (only for use with URLs containing \"/pages/\")",
        "data_type": "ExternalId"
    },
    "P4033": {
        "label": "Mastodon address",
        "description": "address on the Mastodon decentralized social network. The form is: 'user@server.domain' there is no leading '@' as sometimes written to distinguish Mastodon addresses from email addresses.",
        "data_type": "ExternalId"
    },
    "P2942": {
        "label": "Dailymotion channel ID",
        "description": "identifier of the Dailymotion channel of a person, or organization",
        "data_type": "ExternalId"
    },
    "P3502": {
        "label": "Ameblo username",
        "description": "this item's username on Ameblo",
        "data_type": "ExternalId"
    },
    "P8412": {
        "label": "Swiss Timber Bridges ID",
        "description": "identifier of a bridge on Swiss Timber Bridges",
        "data_type": "ExternalId"
    },
    "P8475": {
        "label": "IVS ID",
        "description": "identifier of objects and ways in the Inventory of historical ways Switzerland (IVS)",
        "data_type": "ExternalId"
    },
    "P3899": {
        "label": "Medium username",
        "description": "username of the Medium account of a person or an organization",
        "data_type": "ExternalId"
    },
    "P2984": {
        "label": "Snapchat username",
        "description": "this item's username on Snapchat",
        "data_type": "ExternalId"
    },
    "P5797": {
        "label": "Twitch channel ID",
        "description": "ID of a Twitch channel",
        "data_type": "ExternalId"
    },
    "P3555": {
        "label": "World Guide to Covered Bridges ID",
        "description": "identifier for a covered bridge, in the World Guide to Covered Bridges",
        "data_type": "ExternalId"
    },
    "P3943": {
        "label": "Tumblr username",
        "description": "account name on Tumblr",
        "data_type": "ExternalId"
    },
    "P4175": {
        "label": "Patreon ID",
        "description": "identifier for a person on Patreon",
        "data_type": "ExternalId"
    },
    "P4017": {
        "label": "Ustream username",
        "description": "this item's username on Ustream",
        "data_type": "ExternalId"
    },
    "P4016": {
        "label": "SlideShare username",
        "description": "this item's username on SlideShare",
        "data_type": "ExternalId"
    },
    "P8650": {
        "label": "MnDOT Historic Bridges ID",
        "description": "identifier for a historic bridge in the database of the Minnesota Department of Transportation",
        "data_type": "ExternalId"
    },
    "P3207": {
        "label": "Vine user ID",
        "description": "ID of the Vine account of a person or organisation",
        "data_type": "ExternalId"
    },
    "P8931": {
        "label": "Encyklopedie mostů v Čechách, na Moravě a ve Slezsku ID",
        "description": "numerican identifier for a bridge in the Czech Republic",
        "data_type": "ExternalId"
    },
    "P4226": {
        "label": "Cyworld ID",
        "description": "identifier of the subject at Cyworld",
        "data_type": "ExternalId"
    },
    "P3267": {
        "label": "Flickr user ID",
        "description": "identifier for a person or organisation, with an account at Flickr",
        "data_type": "ExternalId"
    },
    "P4328": {
        "label": "Brueckenweb ID",
        "description": "identifier of a bridge, in the Brückenweb database",
        "data_type": "ExternalId"
    },
    "P4690": {
        "label": "Oireachtas member ID",
        "description": "identifier issued by the Oireachtas for parliament members (both Dáil and Séanad) in Ireland",
        "data_type": "ExternalId"
    },
    "P9296": {
        "label": "Astro Awani topic ID",
        "description": "identifier for a topic at the Astro Awani news website",
        "data_type": "ExternalId"
    },
    "P8551": {
        "label": "Opera Online composer ID",
        "description": "identifier of a composer or librettist on Opera Online website",
        "data_type": "ExternalId"
    },
    "P6828": {
        "label": "Czech parliament ID",
        "description": "identifier in the public database of the deputies of the Czech Parliament",
        "data_type": "ExternalId"
    },
    "P7349": {
        "label": "Gazetteer for Scotland person ID",
        "description": "identifier of a Scottish personality on www.scottish-places.info",
        "data_type": "ExternalId"
    },
    "P9332": {
        "label": "Oxford African American Studies Center ID",
        "description": "identifiant of an entity in the Oxford African American Studies Center",
        "data_type": "ExternalId"
    },
    "P6892": {
        "label": "Fundamental.org Baptist Church ID",
        "description": "identifier for an independent Baptist church at Fundamental.org",
        "data_type": "ExternalId"
    },
    "P6619": {
        "label": "XING company ID",
        "description": "identifier for a company on XING",
        "data_type": "ExternalId"
    },
    "P6631": {
        "label": "Tainacan MHN ID",
        "description": "identifier for collection items from the collection of the National Historical Museum",
        "data_type": "ExternalId"
    },
    "P8658": {
        "label": "Syro-Malabar Catholic Church ID",
        "description": "Syro-Malabar Catholic Church website identifier",
        "data_type": "ExternalId"
    },
    "P8529": {
        "label": "ANZSRC 2020 FoR ID",
        "description": "Australia and New Zealand Standard Research Classification 2020 identifier for a field of research",
        "data_type": "ExternalId"
    },
    "P1613": {
        "label": "IRC channel",
        "description": "official IRC channel of an institution or project",
        "data_type": "Url"
    },
    "P4200": {
        "label": "Christie's creator ID",
        "description": "identifier for an artist in the Christie's online database",
        "data_type": "ExternalId"
    },
    "P8100": {
        "label": "JournalTOCs ID",
        "description": "identifier for a scientific journal in the JournalTOCs bibliographic database",
        "data_type": "ExternalId"
    },
    "P3782": {
        "label": "Artnet artist ID",
        "description": "identifier of an artist in the Artnet database of auction results",
        "data_type": "ExternalId"
    },
    "P8104": {
        "label": "Paperity journal ID",
        "description": "identifier for a scientific journal in the Paperity bibliographic database",
        "data_type": "ExternalId"
    },
    "P8501": {
        "label": "Gateway to Research organisation ID",
        "description": "identifier for an organisation in the UK Research and Innovation (UKRI)'s Gateway to Research (GtR) database of research funding",
        "data_type": "ExternalId"
    },
    "P1416": {
        "label": "affiliation",
        "description": "organization that a person or organization is affiliated with (not necessarily member of or employed by)",
        "data_type": "WikibaseItem"
    },
    "P7816": {
        "label": "textove.com song ID",
        "description": "Song ID at Bulgarian website textove.com",
        "data_type": "ExternalId"
    },
    "P8926": {
        "label": "DBLP venue ID",
        "description": "identifier for venue entries in the DBLP computer science bibliography (use portion of DBLP stream key after streams/)",
        "data_type": "ExternalId"
    },
    "P8527": {
        "label": "Unsplash user ID",
        "description": "identifier for a person or organisation with an account on Unsplash.com",
        "data_type": "ExternalId"
    },
    "P8774": {
        "label": "MTMT journal ID",
        "description": "journal identifier in the Hungarian Scientific Bibliography",
        "data_type": "ExternalId"
    },
    "P8375": {
        "label": "Crossref journal ID",
        "description": "identifier of a journal in Crossref",
        "data_type": "ExternalId"
    },
    "P6578": {
        "label": "MutualArt artist ID",
        "description": "identifier for an artist in the MutualArt database",
        "data_type": "ExternalId"
    },
    "P8553": {
        "label": "CTHS publication ID",
        "description": "identifier for a publication on the CTHS website (ENC)",
        "data_type": "ExternalId"
    },
    "P8776": {
        "label": "IMVDb music video ID",
        "description": "identifier for a music video the on IMVDb online database",
        "data_type": "ExternalId"
    },
    "P5197": {
        "label": "Melon song ID",
        "description": "identifier for a song on Melon",
        "data_type": "ExternalId"
    },
    "P8987": {
        "label": "ISDS ID",
        "description": "identifier of an electronic mailbox assigned by the Czech Republic government to businesses, individuals and government agencies of the Czech Republic",
        "data_type": "ExternalId"
    },
    "P9133": {
        "label": "AZLyrics.com song ID",
        "description": "identifier for an song on AZLyrics.com site",
        "data_type": "ExternalId"
    },
    "P7056": {
        "label": "Jaxsta recording ID",
        "description": "identifier for a recording on the Jaxsta website",
        "data_type": "ExternalId"
    },
    "P6364": {
        "label": "official color or colors",
        "description": "official color or colors chosen to represent or identify an item",
        "data_type": "WikibaseItem"
    },
    "P8541": {
        "label": "BHF magazine ID",
        "description": "identifier for a magazine on the online Bibliographie de l'histoire de France",
        "data_type": "ExternalId"
    },
    "P7166": {
        "label": "Acharts.co song ID",
        "description": "identifier for a composition, track or single on the Acharts.co music site",
        "data_type": "ExternalId"
    },
    "P7106": {
        "label": "LezWatch.TV character ID",
        "description": "IDs for characters listed in the LezWatch.TV character database",
        "data_type": "ExternalId"
    },
    "P8243": {
        "label": "500 Queer Scientists profile",
        "description": "profile on 500 Queer Scientists database",
        "data_type": "ExternalId"
    },
    "P1730": {
        "label": "AllMusic song ID",
        "description": "identifier for a song in AllMusic database",
        "data_type": "ExternalId"
    },
    "P7107": {
        "label": "LezWatch.TV show ID",
        "description": "ID for shows listed in LezWatch.TV show database",
        "data_type": "ExternalId"
    },
    "P6124": {
        "label": "Sotheby's person ID",
        "description": "identifier for an artist on the Sotheby's website",
        "data_type": "ExternalId"
    },
    "P8903": {
        "label": "HAL journal id",
        "description": "numerical identifier for a journal on HAL, an open archive of scholarly documents",
        "data_type": "ExternalId"
    },
    "P8653": {
        "label": "Museen Dresden article ID",
        "description": "identifier of persons and organisations on Museen der Stadt Dresden",
        "data_type": "ExternalId"
    },
    "P6629": {
        "label": "Artcurial lot ID",
        "description": "identifier for a lot on the website of Artcurial, a French auction house",
        "data_type": "ExternalId"
    },
    "P3479": {
        "label": "Omni topic ID",
        "description": "identifier for a topic, used by Omni and Aftonbladet",
        "data_type": "ExternalId"
    },
    "P6664": {
        "label": "Le Vif tag ID",
        "data_type": "ExternalId"
    },
    "P5161": {
        "label": "Trustpilot company ID",
        "description": "identifier of a company or website on the Trustpilot customer reviews website",
        "data_type": "ExternalId"
    },
    "P2042": {
        "label": "Artsy artist ID",
        "description": "identifier at the Artsy website",
        "data_type": "ExternalId"
    },
    "P4293": {
        "label": "PM20 folder ID",
        "description": "identifier for a folder in the ZBW Pressemappe 20. Jahrhundert (20th century press archive)",
        "data_type": "ExternalId"
    },
    "P5551": {
        "label": "Courrier international topic ID",
        "description": "identifier for a topic on the Courrier international website",
        "data_type": "ExternalId"
    },
    "P6922": {
        "label": "Metro topic ID",
        "description": "identifier for an topic on Metro site",
        "data_type": "ExternalId"
    },
    "P8052": {
        "label": "MusicBrainz genre ID",
        "description": "identifier for a genre on the MusicBrainz open music encyclopedia",
        "data_type": "ExternalId"
    },
    "P9347": {
        "label": "France24 topic ID (English)",
        "description": "identifier of a topic on the English version of the France24 website",
        "data_type": "ExternalId"
    },
    "P9346": {
        "label": "France24 topic ID (French)",
        "description": "identifier of a topic on the French version of the France24 website",
        "data_type": "ExternalId"
    },
    "P9348": {
        "label": "France24 topic ID (Spanish)",
        "description": "identifier of a topic on the Spanish version of the France24 website",
        "data_type": "ExternalId"
    },
    "P9349": {
        "label": "France24 topic ID (Arabic)",
        "description": "identifier of a topic on the Arabic version of the France24 website",
        "data_type": "ExternalId"
    },
    "P7610": {
        "label": "Taiwan Cinema company ID",
        "description": "identifier for a company on Taiwan Cinema",
        "data_type": "ExternalId"
    },
    "P1882": {
        "label": "Web Gallery of Art ID",
        "description": "identifier for an artist in the Web Gallery of Art",
        "data_type": "ExternalId"
    },
    "P2268": {
        "label": "Musée d'Orsay artist ID",
        "description": "numeric ID for an artist in the Répertoire des artistes documentation database of the Orsay Museum",
        "data_type": "ExternalId"
    },
    "P7284": {
        "label": "The Independent topic ID",
        "description": "identifier for an topic on The Independent site",
        "data_type": "ExternalId"
    },
    "P650": {
        "label": "RKDartists ID",
        "description": "identifier in the RKDartists database (Rijksbureau voor Kunsthistorische Documentatie)",
        "data_type": "ExternalId"
    },
    "P9185": {
        "label": "AllMusic genre/style ID",
        "description": "ID of a music genre or style in AllMusic",
        "data_type": "ExternalId"
    },
    "P5558": {
        "label": "Le Monde ID",
        "description": "identifier for a topic on the Le Monde website",
        "data_type": "ExternalId"
    },
    "P6963": {
        "label": "BaFin Institute ID",
        "description": "identifier for an institute in the BaFin company database",
        "data_type": "ExternalId"
    },
    "P6200": {
        "label": "BBC News topic ID",
        "description": "identifier for a topic, on the BBC News website",
        "data_type": "ExternalId"
    },
    "P9219": {
        "label": "Discogs style ID",
        "description": "identifier of a music style in Discogs",
        "data_type": "ExternalId"
    },
    "P7301": {
        "label": "Al-Jazeera topic ID",
        "description": "identifier for a topic at Al-Jazeera.com's website",
        "data_type": "ExternalId"
    },
    "P9173": {
        "label": "Rate Your Music genre ID",
        "description": "identifier for a music genre or style in Rate Your Music",
        "data_type": "ExternalId"
    },
    "P4167": {
        "label": "Dagens Næringsliv topic ID",
        "description": "identifier for a topic, used by the Norwegian daily newspaper Dagens Næringsliv",
        "data_type": "ExternalId"
    },
    "P6537": {
        "label": "Yandex Music genre ID",
        "description": "identifier of a music genre on the Yandex Music website",
        "data_type": "ExternalId"
    },
    "P1138": {
        "label": "Kunstindeks Danmark Artist ID",
        "description": "unique artist identifier used by Kunstindeks Danmark",
        "data_type": "ExternalId"
    },
    "P8919": {
        "label": "Gab username",
        "description": "this item's username on Gab",
        "data_type": "ExternalId"
    },
    "P7737": {
        "label": "DeviantArt username",
        "description": "identifier for an account on deviantArt",
        "data_type": "ExternalId"
    },
    "P3115": {
        "label": "ISzDb company ID",
        "description": "identifier of a company in the ISzDb (a Hungarian dub database)",
        "data_type": "ExternalId"
    },
    "P4204": {
        "label": "The Times of India topic ID",
        "description": "identifier for a topic for news and information aggregation on The Times of India website",
        "data_type": "ExternalId"
    },
    "P8904": {
        "label": "Parler username",
        "description": "this item's username on Parler",
        "data_type": "ExternalId"
    },
    "P3221": {
        "label": "New York Times topic ID",
        "description": "identifier for a topic, at the New York Times' website",
        "data_type": "ExternalId"
    },
    "P3509": {
        "label": "Dagens Nyheter topic ID",
        "description": "identifier for a topic, used by the Swedish daily newspaper Dagens Nyheter",
        "data_type": "ExternalId"
    },
    "P6621": {
        "label": "Le Figaro tag ID",
        "description": "identifier for a topic on the website of Le Figaro",
        "data_type": "ExternalId"
    },
    "P9084": {
        "label": "ABC News topic ID",
        "description": "identifier of a topic of content on the Australian Broadcasting Corporation website",
        "data_type": "ExternalId"
    },
    "P6622": {
        "label": "Le Parisien tag ID",
        "description": "identifier for a topic on the website of Le Parisien",
        "data_type": "ExternalId"
    },
    "P1707": {
        "label": "DAAO ID",
        "description": "identifier per Design & Art Australia Online",
        "data_type": "ExternalId"
    },
    "P7529": {
        "label": "dp.ru company ID",
        "description": "dp.ru identifier for a company",
        "data_type": "ExternalId"
    },
    "P7022": {
        "label": "Australian Suppliers Directory ID",
        "description": "identifier for a company in the Australian Suppliers Directory",
        "data_type": "ExternalId"
    },
    "P9318": {
        "label": "Ávvir topic ID",
        "description": "identifier of a topic in the Northern Sámi publication Ávvir",
        "data_type": "ExternalId"
    },
    "P5232": {
        "label": "D&B Hoovers company profile",
        "description": "identifier for a company in the D&B Hoovers database ID",
        "data_type": "ExternalId"
    },
    "P9218": {
        "label": "Discogs genre ID",
        "description": "identifier for a music genre in Discogs",
        "data_type": "ExternalId"
    },
    "P9271": {
        "label": "Kickstarter username",
        "description": "this item's username on  Kickstarter",
        "data_type": "ExternalId"
    },
    "P5256": {
        "label": "OpenCorporates corporate grouping",
        "description": "companies grouped together at OpenCorporates",
        "data_type": "ExternalId"
    },
    "P9101": {
        "label": "Discord username",
        "description": "identifier for a person or bot on Discord",
        "data_type": "ExternalId"
    },
    "P9368": {
        "label": "CNA topic ID",
        "description": "identifier for a topic on CNA's website",
        "data_type": "ExternalId"
    },
    "P7729": {
        "label": "GUI number",
        "description": "a number of a company in Taiwan given by Ministry of Economic Affairs, Taiwan",
        "data_type": "ExternalId"
    },
    "P8754": {
        "label": "OpenStreetMap numeric user ID",
        "description": "numeric identifier for a user on OpenStreetMap",
        "data_type": "ExternalId"
    },
    "P4679": {
        "label": "MAVISE company ID",
        "description": "identifier for a company in the MAVISE database of the European Audiovisual Observatory",
        "data_type": "ExternalId"
    },
    "P5337": {
        "label": "Google News topics ID",
        "description": "identifier for a subject in the news on Google News",
        "data_type": "ExternalId"
    },
    "P6813": {
        "label": "Mediapart tag ID",
        "data_type": "ExternalId"
    },
    "P1736": {
        "label": "Information Center for Israeli Art artist ID",
        "description": "Identifier for artists in the Information Center for Israeli Art",
        "data_type": "ExternalId"
    },
    "P6328": {
        "label": "Politika topic ID",
        "description": "identifier for a topic on Politika.rs website",
        "data_type": "ExternalId"
    },
    "P8599": {
        "label": "Nelson-Atkins Museum of Art artwork ID",
        "description": "identifier for an artwork on the Nelson-Atkins Museum of Art website",
        "data_type": "ExternalId"
    },
    "P6152": {
        "label": "National Portrait Gallery (United States) object ID",
        "description": "object identifier for artworks at the National Portrait Gallery (US)",
        "data_type": "ExternalId"
    },
    "P4610": {
        "label": "ARTIC artwork ID",
        "description": "identifier for an artwork on the Art Institute of Chicago website",
        "data_type": "ExternalId"
    },
    "P4373": {
        "label": "National Trust Collections ID",
        "description": "identifier for an artwork or heritage object, in the catalogue of the United Kingdom's National Trust",
        "data_type": "ExternalId"
    },
    "P6565": {
        "label": "Musenor artwork ID",
        "description": "identifier for an artwork on the website of the Association des Conservateurs des Musées des Hauts-de-France",
        "data_type": "ExternalId"
    },
    "P4704": {
        "label": "Smithsonian American Art Museum artwork ID",
        "description": "identifier for an artwork or other object on the Smithsonian American Art Museum website",
        "data_type": "ExternalId"
    },
    "P4737": {
        "label": "Solomon R. Guggenheim Foundation artwork ID",
        "description": "identifier for an artwork or other object on the Solomon R. Guggenheim Foundation website",
        "data_type": "ExternalId"
    },
    "P5407": {
        "label": "MHK object ID",
        "description": "identifier for an object (painting, sculpture, instrument) in the collection of the Museumslandschaft Hessen Kassel",
        "data_type": "ExternalId"
    },
    "P4674": {
        "label": "Indianapolis Museum of Art artwork ID",
        "description": "identifier for an artwork or other object on the Indianapolis Museum of Art collection website",
        "data_type": "ExternalId"
    },
    "P4673": {
        "label": "Museum of Fine Arts, Houston object ID",
        "description": "identifier for an artwork or other object on the Museum of Fine Arts, Houston website",
        "data_type": "ExternalId"
    },
    "P8639": {
        "label": "Cincinnati Art Museum ID",
        "description": "identifier for an artwork on the Cincinnati Art Museum website",
        "data_type": "ExternalId"
    },
    "P8640": {
        "label": "Dallas Museum of Art ID",
        "description": "identifier for an artwork on the Dallas Museum of Art website",
        "data_type": "ExternalId"
    },
    "P4625": {
        "label": "Museum of Fine Arts, Boston object ID",
        "description": "identifier for an artwork or other object on the Museum of Fine Arts, Boston website",
        "data_type": "ExternalId"
    },
    "P8601": {
        "label": "Barcelona Public art ID",
        "description": "identifier for public art in Barcelona",
        "data_type": "ExternalId"
    },
    "P6489": {
        "label": "Joan Miró Online Image Bank ID",
        "description": "identifier for an artwork by Joan Miró in the Joan Miró Online Image Bank",
        "data_type": "ExternalId"
    },
    "P4692": {
        "label": "American Art Collaborative object ID",
        "description": "identifier for an artwork or any relevant object on the American Art Collaborative website",
        "data_type": "ExternalId"
    },
    "P4564": {
        "label": "Art Museum of Estonia artwork ID",
        "description": "Artwork identifier for the Art Museum of Estonia",
        "data_type": "ExternalId"
    },
    "P5265": {
        "label": "Dordrechts Museum artwork ID",
        "description": "identifier for an artwork or other object on the Dordrechts Museum website",
        "data_type": "ExternalId"
    },
    "P6372": {
        "label": "Interpol WOA artwork ID (OBSOLETE)",
        "description": "identifier for a stolen artwork in the dedicated Interpol database",
        "data_type": "ExternalId"
    },
    "P6374": {
        "label": "MAMVP artwork ID",
        "description": "identifier for an artwork on the website of the Musée d'Art Moderne de la Ville de Paris",
        "data_type": "ExternalId"
    },
    "P8335": {
        "label": "Uffizi artwork ID",
        "description": "identifier for an object or place in the website of Gallerie degli Uffizi",
        "data_type": "ExternalId"
    },
    "P4627": {
        "label": "EK number",
        "description": "identifier for an artwork in the Entartete Kunst database of the Freie Universität Berlin",
        "data_type": "ExternalId"
    },
    "P6246": {
        "label": "Paris Musées work ID",
        "description": "identifier for a work on the website of the museums of the city of Paris",
        "data_type": "ExternalId"
    },
    "P2582": {
        "label": "J. Paul Getty Museum object ID",
        "description": "identifier for an object in the J. Paul Getty Museum",
        "data_type": "ExternalId"
    },
    "P5823": {
        "label": "Belvedere object ID",
        "description": "identifier for an artwork in the Österreichische Galerie Belvedere in Vienna",
        "data_type": "ExternalId"
    },
    "P6576": {
        "label": "Art Fund artwork ID",
        "description": "identifier for an artwork on the website of the Art Fund",
        "data_type": "ExternalId"
    },
    "P8574": {
        "label": "NYARC Discovery ID",
        "description": "record for a publication held by the New York Art Resources Consortium (NYARC) in New York City",
        "data_type": "ExternalId"
    },
    "P4709": {
        "label": "Barnes Foundation ID",
        "description": "identifier for an artwork or other object on the Barnes Foundation website",
        "data_type": "ExternalId"
    },
    "P4701": {
        "label": "Google Arts & Culture asset ID",
        "description": "identifier for an artwork or other object on the Google Arts & Culture website",
        "data_type": "ExternalId"
    },
    "P2539": {
        "label": "Nationalmuseum Sweden artwork ID",
        "description": "artwork identifier for Nationalmuseum in Sweden",
        "data_type": "ExternalId"
    },
    "P6595": {
        "label": "Paintings by Salvador Dalí ID",
        "description": "identifier for an artwork in the catalogue raisonné of paintings by Salvador Dalí",
        "data_type": "ExternalId"
    },
    "P6633": {
        "label": "Cini Foundation ID",
        "description": "identifier for an artwork on the Cini Foundation website",
        "data_type": "ExternalId"
    },
    "P4740": {
        "label": "Brooklyn Museum artwork ID",
        "description": "identifier for an artwork or other object on the Brooklyn Museum website",
        "data_type": "ExternalId"
    },
    "P4739": {
        "label": "Musée des Augustins artwork ID (former scheme)",
        "description": "identifier for an artwork or other object on the Musée des Augustins website",
        "data_type": "ExternalId"
    },
    "P3855": {
        "label": "LombardiaBeniCulturali artwork ID",
        "description": "identifier of an artwork in the cultural heritage database of the Lombardy Region of Italy",
        "data_type": "ExternalId"
    },
    "P4144": {
        "label": "Athenaeum artwork ID",
        "description": "artwork id in the Athenaeum artworks website",
        "data_type": "ExternalId"
    },
    "P4738": {
        "label": "Yale Center for British Art artwork ID",
        "description": "identifier for an artwork or other object on the Yale Center for British Art website",
        "data_type": "ExternalId"
    },
    "P6020": {
        "label": "d'Art d'Art ! ID",
        "description": "identifier for an artwork on the d'Art d'Art ! website",
        "data_type": "ExternalId"
    },
    "P4643": {
        "label": "Philadelphia Museum of Art work ID",
        "description": "identifier for an artwork or other object on the Philadelphia Museum of Art website",
        "data_type": "ExternalId"
    },
    "P3272": {
        "label": "Zeri image ID",
        "description": "number in the Zeriimages database",
        "data_type": "ExternalId"
    },
    "P4712": {
        "label": "Minneapolis Institute of Art artwork ID",
        "description": "identifier for an artwork or other object on the Minneapolis Institute of Art website",
        "data_type": "ExternalId"
    },
    "P4713": {
        "label": "Walters Art Museum ID",
        "description": "identifier for an artwork or other object on the Walters Art Museum website",
        "data_type": "ExternalId"
    },
    "P4905": {
        "label": "KMSKA work PID",
        "description": "Persistent identifier for an artwork from the collection of the Royal Museum of Fine Arts Antwerp in Antwerp, Belgium",
        "data_type": "ExternalId"
    },
    "P2344": {
        "label": "AGORHA work ID",
        "description": "Identifier for historical art resources",
        "data_type": "ExternalId"
    },
    "P4659": {
        "label": "Musée d'Orsay artwork ID",
        "description": "identifier for an artwork or other object on the Musée d'Orsay website",
        "data_type": "ExternalId"
    },
    "P6625": {
        "label": "Salvador Dali Museum ID",
        "description": "identifier for an artwork on the Salvador Dali Museum website",
        "data_type": "ExternalId"
    },
    "P3929": {
        "label": "V&A item ID",
        "description": "identifier for items in the collection of the Victoria and Albert Museum",
        "data_type": "ExternalId"
    },
    "P5499": {
        "label": "Boijmans work ID",
        "description": "identifier for an artwork in the Museum Boijmans Van Beuningen",
        "data_type": "ExternalId"
    },
    "P5891": {
        "label": "Bpk-ID",
        "description": "Bpk-identificator, identifier for art photos by picture agency, Image Bank of cultural institutions",
        "data_type": "ExternalId"
    },
    "P6358": {
        "label": "Musée Picasso artwork ID",
        "description": "identifier for an artwork on the website of the Musée Picasso, in Paris",
        "data_type": "ExternalId"
    },
    "P4257": {
        "label": "National Museums of Japan e-museum ID",
        "description": "identifier of a work, in the National Museums of Japan's e-museum website",
        "data_type": "ExternalId"
    },
    "P5223": {
        "label": "Information Center for Israeli Art artwork ID",
        "description": "identifier assigned to an artwork by the Information Center for Israeli Art",
        "data_type": "ExternalId"
    },
    "P3293": {
        "label": "BALaT object ID",
        "description": "identifier for creative works in the Photo Library database of BALaT (Belgian Art Links & Tools), maintained by KIK-IRPA, Belgium's Royal Institute for Cultural Heritage",
        "data_type": "ExternalId"
    },
    "P2282": {
        "label": "Groeningemuseum work PID",
        "description": "persistent identifier for an artwork from the collection of the Groeningemuseum in Bruges, Belgium",
        "data_type": "ExternalId"
    },
    "P5269": {
        "label": "Web umenia work ID",
        "description": "identifier for an artwork or other object on the Web umenia website",
        "data_type": "ExternalId"
    },
    "P8583": {
        "label": "Yale University Art Gallery ID",
        "description": "identifier for an artwork on the Yale University Art Gallery website",
        "data_type": "ExternalId"
    },
    "P6310": {
        "label": "Muséosphère work ID",
        "description": "identifier for a work on the Muséosphère website",
        "data_type": "ExternalId"
    },
    "P4683": {
        "label": "National Gallery of Art artwork ID",
        "description": "identifier for an artwork or other object on the United States' National Gallery of Art website",
        "data_type": "ExternalId"
    },
    "P4684": {
        "label": "National Gallery of Victoria artwork ID",
        "description": "identifier for an artwork or other object on the National Gallery of Victoria website",
        "data_type": "ExternalId"
    },
    "P8390": {
        "label": "PMSA ID",
        "description": "identifier for an artwork in the National Recording Project database of the Public Monuments & Sculpture Association, UK",
        "data_type": "ExternalId"
    },
    "P4764": {
        "label": "Base Arcade artwork ID",
        "description": "identifier for an artwork or other object in the Arcade database",
        "data_type": "ExternalId"
    },
    "P4686": {
        "label": "Carnegie Museum of Art ID",
        "description": "identifier for an artwork or other object on the Carnegie Museum of Art website",
        "data_type": "ExternalId"
    },
    "P6332": {
        "label": "Panorama de l'art ID",
        "description": "identifier for an entry on the Panorama de l'art website",
        "data_type": "ExternalId"
    },
    "P4525": {
        "label": "MuIS object ID",
        "description": "identifier for an object in the Estonian museum database MuIS",
        "data_type": "ExternalId"
    },
    "P4761": {
        "label": "Images d'Art artwork ID",
        "description": "identifier for an artwork or other object on the Images d'Art website",
        "data_type": "ExternalId"
    },
    "P7229": {
        "label": "Fundación Goya en Aragón ID",
        "description": "identifier for an artwork by Francisco Goya on the Fundación Goya en Aragón website",
        "data_type": "ExternalId"
    },
    "P6355": {
        "label": "MNAM artwork ID",
        "description": "identifier for an artwork on the website of the collections of the Musée national d'Art moderne, in Paris",
        "data_type": "ExternalId"
    },
    "P3504": {
        "label": "Florentine Inventario Palatina art ID",
        "description": "identifier of an artwork in the inventory of the Palatine Gallery, Palazzo Pitti of Florentine museums",
        "data_type": "ExternalId"
    },
    "P6738": {
        "label": "Whitney Museum of American Art artwork ID",
        "description": "identifier for an artwork on the Whitney Museum of American Art website",
        "data_type": "ExternalId"
    },
    "P6356": {
        "label": "IHOI work ID",
        "description": "identifier for a work on the website of the Iconothèque historique de l'océan Indien",
        "data_type": "ExternalId"
    },
    "P4582": {
        "label": "Kulturelles Erbe Köln object ID",
        "description": "object identifier in Kulturelles Erbe Köln",
        "data_type": "ExternalId"
    },
    "P2511": {
        "label": "MSK Gent work PID",
        "description": "identifier for an artwork from the collection of the Museum of Fine Arts, Ghent",
        "data_type": "ExternalId"
    },
    "P8598": {
        "label": "Seattle Art Museum ID",
        "description": "identifier for an artwork on the Seattle Art Museum website",
        "data_type": "ExternalId"
    },
    "P2971": {
        "label": "GCatholic church ID",
        "description": "ID of a church on GCatholic.org",
        "data_type": "ExternalId"
    },
    "P6801": {
        "label": "number of hospital beds",
        "description": "number of inpatient beds",
        "data_type": "Quantity"
    },
    "P4340": {
        "label": "PubsHistory pub ID",
        "description": "identifier for a public house in the United Kingdom, as used by the 'I Am Pub History' website",
        "data_type": "ExternalId"
    },
    "P2908": {
        "label": "SecondHandSongs work ID",
        "description": "identifier for a creative work, in the SecondHandSongs database about covers and samplings",
        "data_type": "ExternalId"
    },
    "P6855": {
        "label": "emergency services",
        "description": "emergencies treated by hospital or health care organization",
        "data_type": "WikibaseItem"
    },
    "P3877": {
        "label": "HappyCow restaurant ID",
        "description": "identifier for a vegan or vegetarian restaurant, on the HappyCow reviews website",
        "data_type": "ExternalId"
    },
    "P4657": {
        "label": "The Numbers person ID",
        "description": "identifier of an actor at The Numbers",
        "data_type": "ExternalId"
    },
    "P4160": {
        "label": "Michelin Restaurants ID",
        "description": "identifier for a restaurant in the Michelin Restaurants website",
        "data_type": "ExternalId"
    },
    "P5294": {
        "label": "Church of Norway building ID",
        "description": "identifier for a church building listed by Church of Norway (\"Kirkebyggdatabasen\")",
        "data_type": "ExternalId"
    },
    "P2688": {
        "label": "Box Office Mojo person ID",
        "description": "former identifier of a person in the Box Office Mojo database; data has been moved to IMDBPro",
        "data_type": "ExternalId"
    },
    "P2531": {
        "label": "Box Office Mojo studio ID",
        "description": "former identifier for a film studio in the Box Office Mojo database; data has been moved to IMDBPro",
        "data_type": "ExternalId"
    },
    "P1994": {
        "label": "AllMusic composition ID",
        "description": "identifier for a composition in AllMusic",
        "data_type": "ExternalId"
    },
    "P1237": {
        "label": "Box Office Mojo film ID (former scheme)",
        "description": "identifier for a movie at the website Box Office Mojo",
        "data_type": "ExternalId"
    },
    "P3808": {
        "label": "The Numbers movie ID",
        "description": "ID of a film at The Numbers",
        "data_type": "ExternalId"
    },
    "P8869": {
        "label": "AAA restaurant ID",
        "description": "identifier for a restaurant on the American Automobile Association website",
        "data_type": "ExternalId"
    },
    "P3152": {
        "label": "Findsmiley ID",
        "description": "identifier for Danish companies serving food",
        "data_type": "ExternalId"
    },
    "P2530": {
        "label": "Box Office Mojo franchise ID (former scheme)",
        "description": "former identifier of a franchise in the Box Office Mojo database",
        "data_type": "ExternalId"
    },
    "P6647": {
        "label": "Zagat ID",
        "description": "ID for restaurants on the Zagat website",
        "data_type": "ExternalId"
    },
    "P6402": {
        "label": "Gault et Millau ID",
        "description": "identifier for a restaurant in the Gault et Millau Restaurants website",
        "data_type": "ExternalId"
    },
    "P846": {
        "label": "GBIF taxon ID",
        "description": "taxon identifier in GBIF",
        "data_type": "ExternalId"
    },
    "P1076": {
        "label": "ICTV virus ID",
        "description": "identifier in the Universal Virus Database (ICTVdB)",
        "data_type": "ExternalId"
    },
    "P2752": {
        "label": "New Zealand Organisms Register ID",
        "description": "identifier for a taxon names in the New Zealand Organisms Register",
        "data_type": "ExternalId"
    },
    "P1992": {
        "label": "Plazi ID",
        "description": "identifier for a taxon treatment at Plazi.org",
        "data_type": "ExternalId"
    },
    "P1421": {
        "label": "GRIN URL",
        "description": "URL for a taxon in the GRIN website",
        "data_type": "Url"
    },
    "P2946": {
        "label": "BacDive ID",
        "description": "identifier for a microorganism, in the BacDive database",
        "data_type": "ExternalId"
    },
    "P838": {
        "label": "BioLib taxon ID",
        "description": "identifier for a taxon in the biological encyclopedia BioLib",
        "data_type": "ExternalId"
    },
    "P2040": {
        "label": "CITES Species+ ID",
        "description": "identifier for a taxon in the Species+ database (CITES)",
        "data_type": "ExternalId"
    },
    "P2455": {
        "label": "Species Profile and Threats Database ID",
        "description": "identifier from Species Profile and Threats Database managed by Australian Government Department of the Environment",
        "data_type": "ExternalId"
    },
    "P3970": {
        "label": "channel number",
        "description": "channel number of given TV channel; use with the mandatory qualifier P137",
        "data_type": "String"
    },
    "P1895": {
        "label": "Fauna Europaea ID",
        "description": "identifier for a taxon in Fauna Europaea",
        "data_type": "ExternalId"
    },
    "P815": {
        "label": "ITIS TSN",
        "description": "identifier for a taxon in the Integrated Taxonomic Information System",
        "data_type": "ExternalId"
    },
    "P687": {
        "label": "BHL Page ID",
        "description": "identifier in the Biodiversity Heritage Library (BHL)",
        "data_type": "ExternalId"
    },
    "P1939": {
        "label": "Dyntaxa ID",
        "description": "identifier for a taxon in the Swedish Taxonomic Database",
        "data_type": "ExternalId"
    },
    "P627": {
        "label": "IUCN taxon ID",
        "description": "identifier for a taxon in the International Union for Conservation of Nature database; source for conservation status (P141)",
        "data_type": "String"
    },
    "P685": {
        "label": "NCBI taxonomy ID",
        "description": "identifer for a taxon in the Taxonomy Database by the National Center for Biotechnology Information",
        "data_type": "ExternalId"
    },
    "P2919": {
        "label": "label in sign language",
        "description": "media file showing label of this item in sign language. Use \"language of work or name\" (P407) as qualifier to indicate which language",
        "data_type": "CommonsMedia"
    },
    "P3342": {
        "label": "significant person",
        "description": "person linked to the item in any possible way",
        "data_type": "WikibaseItem"
    },
    "P8979": {
        "label": "DataTrek ID",
        "description": "identifier of a DataTrek entity",
        "data_type": "ExternalId"
    },
    "P8854": {
        "label": "Salzburgwiki ID",
        "description": "numeric identifier of a page in the Austrian regional wiki Salzburgwiki",
        "data_type": "ExternalId"
    },
    "P6547": {
        "label": "career plus-minus rating",
        "description": "sports statistic",
        "data_type": "Quantity"
    },
    "P6942": {
        "label": "animator",
        "description": "person creating animated sequences out of still images",
        "data_type": "WikibaseItem"
    },
    "P8900": {
        "label": "HuijiWiki Wiki ID",
        "description": "identifier of a topic's wiki, in huijiwiki.com",
        "data_type": "ExternalId"
    },
    "P6735": {
        "label": "Watercolour World artist ID",
        "description": "identifier for an artist, in the Watercolour World database",
        "data_type": "ExternalId"
    },
    "P6394": {
        "label": "ELNET ID",
        "description": "Identifier assigned by the ELNET consortium of Estonian libraries. Format: \"a\", 7 digits, \"x\" or another digit.",
        "data_type": "ExternalId"
    },
    "P3372": {
        "label": "Auckland Art Gallery artist ID",
        "description": "identifier assigned to an artist by the Auckland Art Gallery in New Zealand",
        "data_type": "ExternalId"
    },
    "P6714": {
        "label": "Whitney Museum of American Art artist ID",
        "description": "identifier for an artist on the Whitney Museum of American Art website",
        "data_type": "ExternalId"
    },
    "P1036": {
        "label": "Dewey Decimal Classification",
        "description": "use with qualifier \"edition (P747)\" with item value \"DDC 23\" or create new item to represent the corresponding DDC edition",
        "data_type": "ExternalId"
    },
    "P7369": {
        "label": "National Library of Chile ID",
        "description": "identifier for authority control managed by the National Library of Chile",
        "data_type": "ExternalId"
    },
    "P7029": {
        "label": "National Library of Russia ID",
        "description": "identifier for authority control used by the National Library of Russia, Saint-Petersburg",
        "data_type": "ExternalId"
    },
    "P1150": {
        "label": "Regensburg Classification",
        "description": "identifier used for this class in the Regensburg Classification system",
        "data_type": "ExternalId"
    },
    "P3965": {
        "label": "Bridgeman artist ID",
        "description": "identifier for an artist in Bridgeman images",
        "data_type": "ExternalId"
    },
    "P5597": {
        "label": "Artcyclopedia artist ID",
        "description": "identifier for an artist on the Artcyclopedia website",
        "data_type": "ExternalId"
    },
    "P3888": {
        "label": "Boijmans artist ID",
        "description": "identifier for an artist in the Museum Boijmans Van Beuningen",
        "data_type": "ExternalId"
    },
    "P7026": {
        "label": "Lebanese National Library ID",
        "description": "authority control identifier of the Lebanese National Library",
        "data_type": "ExternalId"
    },
    "P4887": {
        "label": "Web umenia creator ID",
        "description": "identifier assigned to an artist for the online database Webumenia (joint project of Slovak art galleries)",
        "data_type": "ExternalId"
    },
    "P6295": {
        "label": "ARTIC artist ID",
        "description": "identifier for an artist on the Art Institute of Chicago website",
        "data_type": "ExternalId"
    },
    "P4580": {
        "label": "Berlinische Galerie artist ID",
        "description": "artist identifier for the Berlinische Galerie",
        "data_type": "ExternalId"
    },
    "P7024": {
        "label": "Flemish Public Libraries ID",
        "description": "authority control identifier of the Flemish Public Libraries",
        "data_type": "ExternalId"
    },
    "P7028": {
        "label": "National Library of Luxembourg ID",
        "description": "identifier for authority control managed by the National Library of Luxembourg",
        "data_type": "ExternalId"
    },
    "P6764": {
        "label": "V&A person ID",
        "description": "identifier for artists and makers or subjects of items in the collection of the Victoria and Albert Museum",
        "data_type": "ExternalId"
    },
    "P1190": {
        "label": "Universal Decimal Classification",
        "description": "type of library classification",
        "data_type": "ExternalId"
    },
    "P1189": {
        "label": "Chinese Library Classification",
        "description": "identifier used for this class in the CLC",
        "data_type": "ExternalId"
    },
    "P3421": {
        "label": "Belvedere artist ID",
        "description": "identifier assigned to an artist by the Österreichische Galerie Belvedere in Vienna",
        "data_type": "ExternalId"
    },
    "P4936": {
        "label": "SFMOMA artist ID",
        "description": "identifier assigned to an artist by the San Francisco Museum of Modern Art",
        "data_type": "ExternalId"
    },
    "P6645": {
        "label": "Stedelijk Museum Amsterdam ID",
        "description": "identifier for an artist on the Stedelijk Museum Amsterdam website",
        "data_type": "ExternalId"
    },
    "P5239": {
        "label": "Artists in Canada record number",
        "description": "authority control maintained by National Gallery of Canada Library listing biographical data for artists who were born in Canada or worked in Canada",
        "data_type": "ExternalId"
    },
    "P2431": {
        "label": "Thyssen-Bornemisza artist ID",
        "description": "identifier assigned to an artist by the Thyssen-Bornemisza Museum",
        "data_type": "ExternalId"
    },
    "P5271": {
        "label": "Saint Louis Art Museum person ID",
        "description": "identifier for a person on the Saint Louis Art Museum website",
        "data_type": "ExternalId"
    },
    "P2915": {
        "label": "ECARTICO person ID",
        "description": "identifier for a person, in the ECARTICO biographical database of the Dutch and Flemish Golden Ages",
        "data_type": "ExternalId"
    },
    "P4439": {
        "label": "MNCARS artist ID",
        "description": "identifier of a person in the MNCARS (Museo Nacional Centro de Arte Reina Sofía) database",
        "data_type": "ExternalId"
    },
    "P7058": {
        "label": "BNRM ID",
        "description": "identifier for the National Library of the Kingdom of Morocco",
        "data_type": "ExternalId"
    },
    "P6667": {
        "label": "MARGS ID",
        "description": "unique identifier for artists in the collection of the Rio Grande do Sul Art Museum",
        "data_type": "ExternalId"
    },
    "P4663": {
        "label": "DACS ID",
        "description": "code to identify 50,000 artists as members of the British collective rights management organisation DACS and sister organisations worldwide",
        "data_type": "ExternalId"
    },
    "P1149": {
        "label": "Library of Congress Classification",
        "description": "subject classification identifier used in the Library of Congress Classification system",
        "data_type": "ExternalId"
    },
    "P4772": {
        "label": "compArt person ID",
        "description": "identifier for a person in the compArt database Digital Art",
        "data_type": "ExternalId"
    },
    "P3544": {
        "label": "Te Papa agent ID",
        "description": "identifier assigned to a person or organisation by the Museum of New Zealand Te Papa Tongarewa",
        "data_type": "ExternalId"
    },
    "P4025": {
        "label": "Pinakothek artist ID",
        "description": "identifier in the Pinakothek collection",
        "data_type": "ExternalId"
    },
    "P3791": {
        "label": "Art Renewal Center artist ID",
        "description": "identifier of an artist at the Art Renewal Center's ARC Museum website",
        "data_type": "ExternalId"
    },
    "P5273": {
        "label": "Nelson-Atkins Museum of Art person ID",
        "description": "identifier for a person on the Nelson-Atkins Museum of Art website",
        "data_type": "ExternalId"
    },
    "P7039": {
        "label": "National Library of Iceland ID",
        "description": "identifier for authority control managed by the National Library of Iceland",
        "data_type": "ExternalId"
    },
    "P5489": {
        "label": "artist-info artist ID",
        "description": "identifier in the artist-info artist database of exhibition records",
        "data_type": "ExternalId"
    },
    "P5363": {
        "label": "Académie des beaux-arts member ID",
        "description": "identifier of a member of the Académie des beaux-arts on its website",
        "data_type": "ExternalId"
    },
    "P4177": {
        "label": "Finnish National Gallery artist ID",
        "description": "identifier for people in the Finnish National Gallery",
        "data_type": "ExternalId"
    },
    "P7699": {
        "label": "National Library of Lithuania ID",
        "description": "authority id at the National Library of Lithuania, part of VIAF (code LIH)",
        "data_type": "ExternalId"
    },
    "P2399": {
        "label": "British Council artist ID",
        "description": "identifier for an artist, in the catalogue of the British Council",
        "data_type": "ExternalId"
    },
    "P5368": {
        "label": "National Gallery of Canada artist ID",
        "description": "identifier of an artist at the National Gallery of Canada's web site",
        "data_type": "ExternalId"
    },
    "P4531": {
        "label": "ChinesePosters artist ID",
        "description": "identifier for a Chinese propaganda poster artist in the database of chineseposters.net",
        "data_type": "ExternalId"
    },
    "P1045": {
        "label": "Sycomore ID",
        "description": "identifer in the Sycomore database of French MPs, National Assembly (France)",
        "data_type": "ExternalId"
    },
    "P1331": {
        "label": "PACE member ID",
        "description": "identifier for a member of the Parliamentary Assembly of the Council of Europe",
        "data_type": "ExternalId"
    },
    "P3298": {
        "label": "Belgian Senate person ID",
        "description": "identifier for a person on the website of the Belgian Senate",
        "data_type": "ExternalId"
    },
    "P4454": {
        "label": "Argentine Chamber of Deputies ID",
        "description": "identifier for a member of the Chamber of Deputies of Argentina (only incumbent)",
        "data_type": "ExternalId"
    },
    "P5451": {
        "label": "House of Representatives of Morocco ID",
        "description": "identifier for a person on the House of Representatives of Morocco website",
        "data_type": "ExternalId"
    },
    "P4139": {
        "label": "National Assembly of Nigeria ID",
        "description": "identifier for a person on the National Assembly of Nigeria website, both for Representatives and Senators",
        "data_type": "ExternalId"
    },
    "P3344": {
        "label": "Vote Smart ID",
        "description": "identifies candidates in US elections for national and state government within Project Vote Smart's database",
        "data_type": "ExternalId"
    },
    "P4467": {
        "label": "Twitch category ID",
        "description": "identifier for a category on the twitch.tv website",
        "data_type": "ExternalId"
    },
    "P8333": {
        "label": "DLive game ID",
        "description": "identifier for a video game on the dlive.tv website",
        "data_type": "ExternalId"
    },
    "P4453": {
        "label": "Argentine Senate member ID",
        "description": "identifier for a member of the Senate of Argentina (incumbent)",
        "data_type": "ExternalId"
    },
    "P2686": {
        "label": "OpenSecrets people ID",
        "description": "identifier used by Opensecrets for people involved in US elections",
        "data_type": "ExternalId"
    },
    "P5142": {
        "label": "Chinese Political Elites Database ID",
        "description": "identifer for a Chinese politician in the Chinese Political Elites Database",
        "data_type": "ExternalId"
    },
    "P3954": {
        "label": "Italian Senate ID",
        "description": "identifier of a member of the Senate of the Kingdom of Sardinia or of the Senate of the Kingdom of Italy on the Italian Senate website",
        "data_type": "ExternalId"
    },
    "P4693": {
        "label": "Argentine deputy votations ID",
        "description": "identifier for votations and absentism statistics of a member of the Argentine Chamber of Deputies",
        "data_type": "ExternalId"
    },
    "P4703": {
        "label": "HATVP person ID",
        "description": "identifier for a French politician on the High Authority for Transparency in Public Life website",
        "data_type": "ExternalId"
    },
    "P3391": {
        "label": "Verkhovna Rada MP ID",
        "description": "identifier for a member of Verkhovna Rada (Parliament of Ukraine) on the official website",
        "data_type": "ExternalId"
    },
    "P6213": {
        "label": "UK Parliament identifier",
        "description": "identifier used by the UK Parliament linked-data system",
        "data_type": "ExternalId"
    },
    "P1186": {
        "label": "MEP directory ID",
        "description": "identifier for a past or present MEP in a directory of all members of the European Parliament",
        "data_type": "ExternalId"
    },
    "P2190": {
        "label": "C-SPAN person ID",
        "description": "identifier for a person's appearances on C-SPAN",
        "data_type": "ExternalId"
    },
    "P1614": {
        "label": "History of Parliament ID",
        "description": "identifier on the History of Parliament website",
        "data_type": "ExternalId"
    },
    "P1341": {
        "label": "Italian Chamber of Deputies dati ID",
        "description": "identifier for incumbent and former deputies from the Italian Chamber of Deputies",
        "data_type": "ExternalId"
    },
    "P2280": {
        "label": "Austrian Parliament ID",
        "description": "identifier for an individual, in the Austrian Parliament's \"Who's Who\" database",
        "data_type": "ExternalId"
    },
    "P1883": {
        "label": "Declarator.org ID",
        "description": "Person ID on Declarator.org database",
        "data_type": "ExternalId"
    },
    "P3297": {
        "label": "Flemish Parliament person ID",
        "description": "identifier for a person on the website of the Flemish Parliament",
        "data_type": "ExternalId"
    },
    "P2181": {
        "label": "Finnish MP ID",
        "description": "code of the entry on the Finnish parliament database",
        "data_type": "ExternalId"
    },
    "P2170": {
        "label": "Hansard (2006–March 2016) ID",
        "description": "identifier for a British MP in Hansard (2006–March 2016)",
        "data_type": "ExternalId"
    },
    "P4978": {
        "label": "PRS Legislative Research MP ID",
        "description": "identifier for Member of Parliaments of India tracked in PRS Legislative Research website",
        "data_type": "ExternalId"
    },
    "P4667": {
        "label": "Buenos Aires legislator ID",
        "description": "identifier for a member of the Buenos Aires City Legislature",
        "data_type": "ExternalId"
    },
    "P5303": {
        "label": "DLCM ID",
        "description": "identifier for a past or present member of the Legislative Council of Hong Kong in the Database on Legislative Council Members",
        "data_type": "ExternalId"
    },
    "P2549": {
        "label": "Italian Senate of the Republic ID",
        "description": "identifier for incumbent and former senators from the Italian Senate of the Republic",
        "data_type": "ExternalId"
    },
    "P2182": {
        "label": "Finnish Ministers database ID",
        "description": "code of the entry on the Finnish ministers database",
        "data_type": "ExternalId"
    },
    "P2169": {
        "label": "PublicWhip ID",
        "description": "identifer in the PublicWhip database of British MPs",
        "data_type": "ExternalId"
    },
    "P3935": {
        "label": "Chamber of Deputies of Italy storia ID",
        "description": "identifier of a member of the Chamber of Deputies of the Kingdom of Sardinia, Chamber of Deputies of the Kingdom of Italy, or of the Chamber of Deputies of Italy, on the Chamber of Deputies of Italy website",
        "data_type": "ExternalId"
    },
    "P4997": {
        "label": "National Governors Association biography URL",
        "description": "biographical entry in the website of the National Governors Association",
        "data_type": "Url"
    },
    "P1959": {
        "label": "Dutch Senate person ID",
        "description": "identifier for a person on the website of the Dutch Senate",
        "data_type": "ExternalId"
    },
    "P1214": {
        "label": "Riksdagen person-ID",
        "description": "identifier at data.riksdagen.se",
        "data_type": "ExternalId"
    },
    "P1307": {
        "label": "Swiss parliament ID",
        "description": "identifier for a member of the Swiss Parliament since 1848",
        "data_type": "ExternalId"
    },
    "P4966": {
        "label": "Hungarian National Assembly identifier",
        "description": "identifier for a past or present member or office-holder of the Hungarian Parliament",
        "data_type": "ExternalId"
    },
    "P4123": {
        "label": "French National Assembly ID",
        "description": "identifier for a member of the French National Assembly on the official website (do not confuse with Sycomore ID)",
        "data_type": "ExternalId"
    },
    "P5440": {
        "label": "Elysee.fr president ID",
        "description": "identifier of a president of the French Republic on ''elysee.fr''",
        "data_type": "ExternalId"
    },
    "P3290": {
        "label": "biography at the Landtag of Mecklenburg-Vorpommern",
        "description": "link to the biography pages at the Landtag of Mecklenburg-Vorpommern",
        "data_type": "ExternalId"
    },
    "P1839": {
        "label": "US Federal Election Commission ID",
        "description": "identifier assigned by the US Federal Election Commission for federal candidates, parties, and other committees",
        "data_type": "ExternalId"
    },
    "P5355": {
        "label": "abgeordnetenwatch.de politician ID",
        "description": "identifier for a German politician, in the abgeordnetenwatch.de database",
        "data_type": "ExternalId"
    },
    "P4797": {
        "label": "member of the Assembly of Madrid ID",
        "description": "identifier for a person in the Assembly of Madrid webpage",
        "data_type": "ExternalId"
    },
    "P2278": {
        "label": "Member of the Hellenic Parliament ID",
        "description": "an identifier for members, or former members, of the national parliament of Greece",
        "data_type": "ExternalId"
    },
    "P1996": {
        "label": "parliament.uk biography pages",
        "description": "link to an MP or Peer's biography on parliament.uk",
        "data_type": "ExternalId"
    },
    "P1808": {
        "label": "senat.fr ID",
        "description": "identifier on the website of the French senate",
        "data_type": "ExternalId"
    },
    "P7145": {
        "label": "CHESNO politician ID",
        "description": "identifier for politician on CHESNO site",
        "data_type": "ExternalId"
    },
    "P9325": {
        "label": "Gedong Kirtya Classification",
        "description": "classification of Balinese palm-leaf manuscript based by Nyoman Kadjeng",
        "data_type": "WikibaseItem"
    },
    "P7885": {
        "label": "Joconde time period ID",
        "description": "identifier for time periods in the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P4520": {
        "label": "SUNCAT ID",
        "description": "identifier for periodicals from the UK serial union catalogue",
        "data_type": "ExternalId"
    },
    "P8370": {
        "label": "UNBIS Thesaurus ID",
        "description": "identifier for a concept in the UNBIS Thesaurus",
        "data_type": "ExternalId"
    },
    "P4270": {
        "label": "Daum TV series ID",
        "description": "identifier for a TV Series in the Daum database",
        "data_type": "ExternalId"
    },
    "P8374": {
        "label": "Criminological Thesaurus ID",
        "description": "identifier for a concept in the United Nations Interregional Crime and Justice Research Institute's Criminological Thesaurus",
        "data_type": "ExternalId"
    },
    "P4979": {
        "label": "Thésaurus de la désignation des objets mobiliers ID",
        "description": "ID in the Thésaurus de la désignation des objets mobiliers",
        "data_type": "ExternalId"
    },
    "P7462": {
        "label": "graph diameter",
        "description": "maximum eccentricity of any vertex in the graph",
        "data_type": "Quantity"
    },
    "P7960": {
        "label": "Joconde discovery ID",
        "description": "identifier for a discovery environment in the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P7961": {
        "label": "Joconde creation ID",
        "description": "identifier for stage or type of creation from the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P3121": {
        "label": "Epguides ID",
        "description": "identifier for a television programme or series, at epguides.com",
        "data_type": "ExternalId"
    },
    "P8248": {
        "label": "Colon Classification",
        "description": "type of library classification, to be used for topics, not for written works; use with qualifier \"edition (P747)\" with item value \"CC 6\" or \"CC 7\"",
        "data_type": "ExternalId"
    },
    "P7907": {
        "label": "Canmore object-type ID",
        "description": "Identifier for an archaeological object-type in the Canmore thesaurus",
        "data_type": "ExternalId"
    },
    "P9162": {
        "label": "EMS ID",
        "description": "identifier for a subject in the Estonian Subject Thesaurus (EMS)",
        "data_type": "ExternalId"
    },
    "P5198": {
        "label": "ASC Leiden Thesaurus ID",
        "description": "identifier for a topic, in the African Studies Centre (ASC) Leiden's Thesaurus",
        "data_type": "ExternalId"
    },
    "P8539": {
        "label": "ERIC Thesaurus ID",
        "description": "identifier for a term in the Education Resources Information Center (ERIC) Thesaurus",
        "data_type": "ExternalId"
    },
    "P4706": {
        "label": "Ent'revues ID",
        "description": "identifier for a magazine on Ent'revues, directory service of French cultural magazines",
        "data_type": "ExternalId"
    },
    "P8540": {
        "label": "American Folklore Society Ethnographic Thesaurus ID",
        "description": "controlled vocabulary of terms about folklore, ethnomusicology, ethnology, and related fields",
        "data_type": "ExternalId"
    },
    "P8503": {
        "label": "heraldic attitude",
        "description": "the attitude or posture of a charge on a coat of arms (e.g. lion rampant)",
        "data_type": "WikibaseItem"
    },
    "P7922": {
        "label": "Canmore monument-type ID",
        "description": "Identifier for a heritage site-type in the Canmore thesaurus",
        "data_type": "ExternalId"
    },
    "P7884": {
        "label": "Joconde inscription ID",
        "description": "identifier for a type of inscription from the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P8365": {
        "label": "Thesaurus Sozialwissenschaften ID",
        "description": "identifier for a concept in the Thesaurus Sozialwissenschaften (Thesaurus for the Social Sciences)",
        "data_type": "ExternalId"
    },
    "P9231": {
        "label": "Archivo Linz de la Transición Española ID",
        "description": "identifier for an entity in the thesaurus of the Archivo Linz de la Transición Española",
        "data_type": "ExternalId"
    },
    "P5255": {
        "label": "FilmTv.it TV series ID",
        "description": "identifier for a TV serie at FilmTv.it",
        "data_type": "ExternalId"
    },
    "P7906": {
        "label": "Canmore maritime-type ID",
        "description": "Identifier for a type of boat or ship or other maritime craft in the Canmore maritime thesaurus",
        "data_type": "ExternalId"
    },
    "P8986": {
        "label": "graph girth",
        "description": "length of a shortest cycle contained in the graph",
        "data_type": "Quantity"
    },
    "P4309": {
        "label": "FOIH event types ID",
        "description": "identifier for an event type in the thesaurus of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P4835": {
        "label": "TheTVDB.com series ID",
        "description": "identifier for a television series at TheTVDB.com",
        "data_type": "ExternalId"
    },
    "P8359": {
        "label": "Dewey Decimal Classification (works and editions)",
        "description": "DDC number assigned to a publication",
        "data_type": "String"
    },
    "P8360": {
        "label": "Library of Congress Classification (works and editions)",
        "description": "Library of Congress Classification number assigned to a publication",
        "data_type": "String"
    },
    "P8361": {
        "label": "Universal Decimal Classification (works and editions)",
        "description": "UDC number assigned to a publication",
        "data_type": "String"
    },
    "P8362": {
        "label": "Regensburg Classification (works and editions)",
        "description": "RVK number assigned to a publication",
        "data_type": "String"
    },
    "P5091": {
        "label": "HanCinema drama ID",
        "description": "identifier for a TV Series in the HanCinema database",
        "data_type": "ExternalId"
    },
    "P4308": {
        "label": "FOIH value types ID",
        "description": "identifier for a value type in the thesaurus of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P6293": {
        "label": "YSA ID",
        "description": "identifier for a concept in the General Finnish Thesaurus YSA",
        "data_type": "ExternalId"
    },
    "P2768": {
        "label": "BNE journal ID",
        "description": "identifier for a journal in BNE's journals archive, an open access digital library of Spanish-language periodical publications, established by Biblioteca Nacional de España",
        "data_type": "ExternalId"
    },
    "P4784": {
        "label": "Movieplayer TV-series ID",
        "description": "identifier for TV-series in movieplayer.it",
        "data_type": "ExternalId"
    },
    "P8519": {
        "label": "RKD thesaurus ID",
        "description": "identifier in the RKD thesaurus (Rijksbureau voor Kunsthistorische Documentatie)",
        "data_type": "ExternalId"
    },
    "P8717": {
        "label": "ILO Thesaurus ID",
        "description": "identifier for a term in the ILO Thesaurus",
        "data_type": "ExternalId"
    },
    "P4022": {
        "label": "danskefilm TV series ID",
        "description": "identifier for a TV serie in danskefilm.dk",
        "data_type": "ExternalId"
    },
    "P8391": {
        "label": "BBK (library and bibliographic classification)",
        "description": "library and bibliographic classification is the national classification system of Russia, it codifies themes of books published in Russia and some countries of the former USSR",
        "data_type": "ExternalId"
    },
    "P7712": {
        "label": "Joconde domain ID",
        "description": "identifier for the general context (domain) of a cultural artifact from the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P4983": {
        "label": "TMDb TV series ID",
        "description": "identifier for a TV show at The Movie Database",
        "data_type": "ExternalId"
    },
    "P7844": {
        "label": "Joconde object type ID",
        "description": "identifier for object types from the Service des Musées de France Joconde authority file",
        "data_type": "ExternalId"
    },
    "P7391": {
        "label": "graph radius",
        "description": "the minimum eccentricity of any vertex of a graph",
        "data_type": "Quantity"
    },
    "P4898": {
        "label": "Chronicling America newspaper ID",
        "description": "identifier of a newspaper issued by the Library of Congress",
        "data_type": "ExternalId"
    },
    "P1267": {
        "label": "AlloCiné series ID",
        "description": "identifier for a serie in the AlloCiné film database",
        "data_type": "ExternalId"
    },
    "P4518": {
        "label": "Melon album ID",
        "description": "identifier for a music album on Melon",
        "data_type": "ExternalId"
    },
    "P9182": {
        "label": "uta-net.com song ID",
        "description": "identifier for a song on uta-net.com",
        "data_type": "ExternalId"
    },
    "P3501": {
        "label": "Catholic rite",
        "description": "Catholic rite associated with this item",
        "data_type": "WikibaseItem"
    },
    "P8663": {
        "label": "Maniadb album ID",
        "description": "identifier for albums listed in the Maniadb database",
        "data_type": "ExternalId"
    },
    "P5153": {
        "label": "KKBox album ID",
        "description": "identifier for an album on KKBox music site",
        "data_type": "ExternalId"
    },
    "P5144": {
        "label": "Bugs! album ID",
        "description": "identifier for an album on Bugs! music site",
        "data_type": "ExternalId"
    },
    "P2721": {
        "label": "Encyclopaedia Metallum release ID",
        "description": "identifier for a release in the Encyclopaedia Metallum database",
        "data_type": "ExternalId"
    },
    "P6218": {
        "label": "Genius song ID",
        "description": "identifier for a song, musical composition or written work on Genius",
        "data_type": "ExternalId"
    },
    "P5171": {
        "label": "Rockipedia album ID",
        "description": "identifier of an artists or bands album release in the online service Rockipedia",
        "data_type": "ExternalId"
    },
    "P2819": {
        "label": "Yandex Music album ID",
        "description": "identifier for an album on Yandex Music",
        "data_type": "ExternalId"
    },
    "P1954": {
        "label": "Discogs master ID",
        "description": "identifier for a musical work in the Discogs database",
        "data_type": "ExternalId"
    },
    "P4041": {
        "label": "Rock.com.ar album ID",
        "description": "identifier for an album at Rock.com.ar",
        "data_type": "ExternalId"
    },
    "P6217": {
        "label": "Genius album ID",
        "description": "identifier for an album, single, other music release or other publication on Genius",
        "data_type": "ExternalId"
    },
    "P4748": {
        "label": "Line Music album ID",
        "description": "identifier for an album on Line Music",
        "data_type": "ExternalId"
    },
    "P8389": {
        "label": "GCatholic diocese ID",
        "description": "identifier for a diocese on GCatholic.org",
        "data_type": "ExternalId"
    },
    "P6361": {
        "label": "Genius song numeric ID",
        "description": "numeric ID for a song, musical composition or written work on Genius",
        "data_type": "ExternalId"
    },
    "P6742": {
        "label": "Musixmatch track ID",
        "description": "identifier for a song on Musixmatch",
        "data_type": "ExternalId"
    },
    "P7055": {
        "label": "Jaxsta release ID",
        "description": "identifier for a release on the Jaxsta website",
        "data_type": "ExternalId"
    },
    "P7143": {
        "label": "Acharts.co album ID",
        "description": "identifier for an album on the Acharts.co music site",
        "data_type": "ExternalId"
    },
    "P1729": {
        "label": "AllMusic album ID",
        "description": "identifier for an album in AllMusic database",
        "data_type": "ExternalId"
    },
    "P2513": {
        "label": "Jamendo album ID",
        "description": "identifier for an album per the Jamendo website",
        "data_type": "ExternalId"
    },
    "P9033": {
        "label": "Archivo Histórico de diputados de España ID",
        "description": "identifier of a member of Parliament in the Historical Archive of Deputies of Spain",
        "data_type": "ExternalId"
    },
    "P8151": {
        "label": "Sicilian Regional Assembly ID",
        "description": "identifier of a deputy of the Sicilian Regional Assembly",
        "data_type": "ExternalId"
    },
    "P8152": {
        "label": "Sicilian Regional Assembly numeric ID",
        "description": "numeric identifier of a deputy of the Sicilian Regional Assembly",
        "data_type": "ExternalId"
    },
    "P8602": {
        "label": "University of Ghana Digital Collections (UGSpace) ID",
        "description": "authority control identifier for the University of Ghana Digital Collections (UGSpace)",
        "data_type": "ExternalId"
    },
    "P7882": {
        "label": "ft.dk politician identifier",
        "description": "identifier for a member of the danish Folketing at ft.dk",
        "data_type": "ExternalId"
    },
    "P8893": {
        "label": "Sistema de Información Legislativa ID",
        "description": "identifier for a representative or senator in the Congreso de la Unión",
        "data_type": "ExternalId"
    },
    "P8666": {
        "label": "Regional Council of Tuscany ID",
        "data_type": "ExternalId"
    },
    "P8198": {
        "label": "regional government of Sicily ID",
        "description": "identifier of a member of the Government of Sicily, called \"assessore\"",
        "data_type": "ExternalId"
    },
    "P8677": {
        "label": "Région Île-de-France ID",
        "description": "identifier of an elected official on the regional council of Île-de-France website",
        "data_type": "ExternalId"
    },
    "P6493": {
        "label": "CCBAE publication ID",
        "description": "authority identifier in the Catálogo Colectivo de la Red de Bibliotecas de los Archivos Estatales y del CIDA",
        "data_type": "ExternalId"
    },
    "P8474": {
        "label": "lobbywatch.ch ID of a member of the Swiss parliament",
        "description": "identifier for a Swiss MP within lobbywatch.ch 's database",
        "data_type": "ExternalId"
    },
    "P8388": {
        "label": "Swedish Parliament person GUID",
        "description": "identifier for a Riksdagen member or other person on riksdagen.se with format UUID",
        "data_type": "ExternalId"
    },
    "P8731": {
        "label": "AniList manga ID",
        "description": "identifier for AniList.co manga and light novels",
        "data_type": "ExternalId"
    },
    "P4087": {
        "label": "MyAnimeList manga ID",
        "description": "identifier for a manga on MyAnimeList",
        "data_type": "ExternalId"
    },
    "P5008": {
        "label": "on focus list of Wikimedia project",
        "description": "property to indicate that an item is of particular interest for a Wikimedia project. This property does not add notability. Items should not be created with this property if they are not notable for Wikidata. See also P6104, P972, P2354.",
        "data_type": "WikibaseItem"
    },
    "P8871": {
        "label": "Southwest Harbor Public Library item ID",
        "description": "identifier of a item in the catalog of Southwest Harbor Public Library",
        "data_type": "ExternalId"
    },
    "P6453": {
        "label": "Bilibili bangumi ID",
        "description": "identifier of animation episodes on Bilibili",
        "data_type": "ExternalId"
    },
    "P7043": {
        "label": "TheTVDB episode ID",
        "description": "identifier for an episode of a television series in TheTVDB",
        "data_type": "ExternalId"
    },
    "P5849": {
        "label": "AnimeClick manga ID",
        "description": "identifier of manga from AnimeClick database",
        "data_type": "ExternalId"
    },
    "P1984": {
        "label": "Anime News Network manga ID",
        "description": "identifier of a manga (or a light novel) in Anime News Network",
        "data_type": "ExternalId"
    },
    "P8278": {
        "label": "serial identifier on SerialZone",
        "description": "identifier for a series or episode in the Czech database SerialZone",
        "data_type": "ExternalId"
    },
    "P9258": {
        "label": "Austrian Newspapers Online ID",
        "description": "newspaper identifier in the Austrian Newspapers Online (ANNO) project website",
        "data_type": "ExternalId"
    },
    "P9035": {
        "label": "Muck Rack media outlet ID",
        "description": "alphabetical identifier for a news publication on Muckrack.com",
        "data_type": "ExternalId"
    },
    "P6157": {
        "label": "Google News publication ID",
        "description": "identifier for a news source in the news on Google News",
        "data_type": "ExternalId"
    },
    "P8816": {
        "label": "Yandex Zen ID",
        "description": "identifier for a news source on Yandex Zen",
        "data_type": "ExternalId"
    },
    "P8336": {
        "label": "MNBAQ artist ID",
        "description": "unique identifier for an artist in the Musée national des beaux-arts du Québec",
        "data_type": "ExternalId"
    },
    "P8840": {
        "label": "Union des artistes ID",
        "description": "identifiant on Union des artistes website",
        "data_type": "ExternalId"
    },
    "P6136": {
        "label": "Newseum newspaper front page ID",
        "description": "identifier for a newspaper in the Newseum's front page gallery",
        "data_type": "ExternalId"
    },
    "P7944": {
        "label": "DoME artist ID",
        "description": "identifier of an artist in the Database of Modern Exhibitions (1905-1915)",
        "data_type": "ExternalId"
    },
    "P8434": {
        "label": "Artprice artist ID",
        "description": "identifier of an artist in Artprice",
        "data_type": "ExternalId"
    },
    "P7332": {
        "label": "NMM artist ID",
        "description": "authority control for artists' collections at the National Maritime Museum website",
        "data_type": "ExternalId"
    },
    "P7259": {
        "label": "Newspapers.com paper ID",
        "description": "identifier for a newspaper on the Newspapers.com website",
        "data_type": "ExternalId"
    },
    "P6868": {
        "label": "Hoopla artist ID",
        "description": "identifier for an individual on Hoopla",
        "data_type": "ExternalId"
    },
    "P6067": {
        "label": "ARD Mediathek ID",
        "description": "identifier for a show on ARD Mediathek",
        "data_type": "ExternalId"
    },
    "P8108": {
        "label": "Papers Past ID",
        "description": "identifier for a periodical on the Papers Past website",
        "data_type": "ExternalId"
    },
    "P8110": {
        "label": "Retronews ID",
        "description": "identifier for a newspaper on the Retronews website",
        "data_type": "ExternalId"
    },
    "P8234": {
        "label": "LiederNet author ID",
        "description": "author identifier used by the LiederNet Archive",
        "data_type": "ExternalId"
    },
    "P6643": {
        "label": "TV Spielfilm series ID",
        "description": "identifier for a television series, on tv-spielfilm.de",
        "data_type": "ExternalId"
    },
    "P8600": {
        "label": "TV Maze series ID",
        "description": "identifier number in the TV Maze database of shows",
        "data_type": "ExternalId"
    },
    "P6903": {
        "label": "ArtBrokerage artist ID",
        "description": "identifier for an artist, on the ArtBrokerage website",
        "data_type": "ExternalId"
    },
    "P5844": {
        "label": "Treccani Vocabulary ID",
        "description": "ID for lexemes",
        "data_type": "ExternalId"
    },
    "P8943": {
        "label": "Artfacts ID",
        "description": "unique identifier for an artist on Artfacts.net",
        "data_type": "ExternalId"
    },
    "P5554": {
        "label": "Courrier international source ID",
        "description": "identifier for a source on the Courrier international website",
        "data_type": "ExternalId"
    },
    "P1900": {
        "label": "EAGLE id",
        "description": "identifier of an epigraphic concept in the Europeana Network of Ancient Greek and Latin Epigraphy (EAGLE) vocabulary",
        "data_type": "ExternalId"
    },
    "P8705": {
        "label": "Artist ID of the department of Prints and Drawings of the Louvre",
        "description": "Artist ID of the online database of the Department of Prints and Drawings of the Louvre",
        "data_type": "ExternalId"
    },
    "P7681": {
        "label": "Israel Museum Jerusalem artist ID",
        "description": "identifier of a person (artist) in the Israel Museum Jerusalem collection",
        "data_type": "ExternalId"
    },
    "P8572": {
        "label": "Spanish Artists from the Fourth to the Twentieth Century ID",
        "description": "identifier for an artist in Spanish Artists from the Fourth to the Twentieth Century: A Critical Dictionary",
        "data_type": "ExternalId"
    },
    "P9152": {
        "label": "CollectieGelderland creator ID",
        "description": "identifier for a creator in the collaborative platform CollectieGelderland",
        "data_type": "ExternalId"
    },
    "P7761": {
        "label": "Tubi series ID",
        "description": "identifier for a TV series at video streaming site Tubi (tubitv.com)",
        "data_type": "ExternalId"
    },
    "P7849": {
        "label": "Panoptikum podcast episode ID",
        "description": "identifier of a podcast episode in the Panoptikum database",
        "data_type": "ExternalId"
    },
    "P6472": {
        "label": "GameFAQs franchise ID",
        "description": "identifier of a franchise in GameFAQs",
        "data_type": "ExternalId"
    },
    "P8005": {
        "label": "research subject recruitment status",
        "description": "categoric way to express whether a research project is or was open for enrolling research subjects meeting certain characteristics",
        "data_type": "WikibaseItem"
    },
    "P7763": {
        "label": "copyright status as a creator",
        "description": "states if the body of work published during the lifetime of this creator is still copyrighted or in the public domain",
        "data_type": "WikibaseItem"
    },
    "P8329": {
        "label": "principal investigator",
        "description": "person(s) in charge of conducting a research project",
        "data_type": "WikibaseItem"
    },
    "P7921": {
        "label": "TrueAchievements series ID",
        "description": "identifier of a video game series in the TrueAchievements database",
        "data_type": "ExternalId"
    },
    "P7535": {
        "label": "scope and content",
        "description": "a summary statement providing an overview of the archival collection",
        "data_type": "Monolingualtext"
    },
    "P8363": {
        "label": "study type",
        "description": "classification system for clinical trial, experiment, research",
        "data_type": "WikibaseItem"
    },
    "P7912": {
        "label": "Gamekult franchise ID",
        "description": "identifier of a video game franchise in the Gamekult database",
        "data_type": "ExternalId"
    },
    "P8012": {
        "label": "Adventure Gamers series ID",
        "description": "identifier of a video game series in the Adventure Gamers database of adventure video games",
        "data_type": "ExternalId"
    },
    "P7562": {
        "label": "UVL group ID",
        "description": "identifier of a group in the Universal Videogame List",
        "data_type": "ExternalId"
    },
    "P5382": {
        "label": "Behind The Voice Actors franchise ID",
        "description": "identifier of a franchise on the website",
        "data_type": "ExternalId"
    },
    "P5360": {
        "label": "MobyGames group ID",
        "description": "identifier of a group in the MobyGames database",
        "data_type": "ExternalId"
    },
    "P7765": {
        "label": "DES ID",
        "description": "identifier for a lexeme on the Dictionnaire électronique des synonymes",
        "data_type": "ExternalId"
    },
    "P7860": {
        "label": "word in DPEJ of RAE ID",
        "description": "identifier for a word in the Juridical Spanish Dictionary of the Spanish Royal Academy",
        "data_type": "ExternalId"
    },
    "P7790": {
        "label": "word in DLE dictionary of Spanish",
        "description": "word in the Spanish Language Dictionary of the Spanish Royal Academy",
        "data_type": "ExternalId"
    },
    "P9158": {
        "label": "Wurdboek fan de Fryske taal GTB ID",
        "description": "identifier of an entry in the Wurdboek fan de Fryske taal in the Geïntegreerde Taalbank",
        "data_type": "ExternalId"
    },
    "P5438": {
        "label": "signum",
        "description": "familiar nickname of the ancient Romans of Greek origin developed during the 3rd century",
        "data_type": "WikibaseItem"
    },
    "P7732": {
        "label": "DAF ID",
        "description": "identifier for a lexeme on the 9th edition of the Dictionnaire de l'Académie française on the Académie française website",
        "data_type": "ExternalId"
    },
    "P7766": {
        "label": "Bob ID",
        "description": "identifier for a lexeme on Bob",
        "data_type": "ExternalId"
    },
    "P6168": {
        "label": "Bantu Lexical Reconstructions ID",
        "description": "ID of entry in Bantu Lexical Reconstructions",
        "data_type": "ExternalId"
    },
    "P5025": {
        "label": "gens",
        "description": "a clan or group of families from Ancient Rome who shared the same nomen",
        "data_type": "WikibaseItem"
    },
    "P4305": {
        "label": "FOIH styles and cultures ID",
        "description": "identifier for a style or culture in the thesaurus of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P4306": {
        "label": "FOIH periods ID",
        "description": "identifier for a period in the thesaurus of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P5988": {
        "label": "Deezer show ID",
        "description": "identifier for a show or podcast on Deezer",
        "data_type": "ExternalId"
    },
    "P3632": {
        "label": "British Museum thesaurus ID",
        "description": "identifier in the British Museum thesaurus",
        "data_type": "ExternalId"
    },
    "P508": {
        "label": "BNCF Thesaurus ID",
        "description": "identifier in the subject indexing tool of the National Central Library of Florence",
        "data_type": "ExternalId"
    },
    "P7583": {
        "label": "fyyd podcast ID",
        "description": "identifier of a podcast in the fyyd database",
        "data_type": "ExternalId"
    },
    "P7268": {
        "label": "Guia dos Quadrinhos character ID",
        "description": "identifier for a comic book character",
        "data_type": "ExternalId"
    },
    "P1741": {
        "label": "GTAA ID",
        "description": "identifier for GTAA, a thesaurus used in audiovisual archives (NISV, EYE)",
        "data_type": "ExternalId"
    },
    "P5842": {
        "label": "Apple Podcasts podcast ID",
        "description": "identifier for a podcast in iTunes",
        "data_type": "ExternalId"
    },
    "P4818": {
        "label": "Panoptikum podcast ID",
        "description": "identifier in the Panoptikum podcast database",
        "data_type": "ExternalId"
    },
    "P9003": {
        "label": "Google Podcasts show ID",
        "description": "identifier for a podcast on Google Podcasts",
        "data_type": "ExternalId"
    },
    "P9004": {
        "label": "Stitcher show ID",
        "description": "identifier for a podcast on Stitcher",
        "data_type": "ExternalId"
    },
    "P9005": {
        "label": "Castbox show ID",
        "description": "identifier for a podcast on Castbox",
        "data_type": "ExternalId"
    },
    "P9006": {
        "label": "Pocket Casts show ID",
        "description": "identifier for a podcast on Pocket Casts",
        "data_type": "ExternalId"
    },
    "P5840": {
        "label": "NPR podcast ID",
        "description": "identifier for a podcast on National Public Radio (NPR)",
        "data_type": "ExternalId"
    },
    "P7857": {
        "label": "fyyd podcast episode ID",
        "description": "identifier of a podcast episode in the ffyd database",
        "data_type": "ExternalId"
    },
    "P4307": {
        "label": "FOIH heritage types ID",
        "description": "identifier for a heritage type in the thesaurus of the Flemish organization for Immovable Heritage",
        "data_type": "ExternalId"
    },
    "P5863": {
        "label": "CharacTour character ID",
        "data_type": "ExternalId"
    },
    "P5916": {
        "label": "Spotify show ID",
        "description": "identifier for a show or podcast on Spotify",
        "data_type": "ExternalId"
    },
    "P2347": {
        "label": "YSO ID",
        "description": "identifier for a concept in the General Finnish Ontology YSO",
        "data_type": "ExternalId"
    },
    "P2950": {
        "label": "Nomisma ID",
        "description": "identifier in Nomisma.org, a linked open data thesaurus of numismatic concepts",
        "data_type": "ExternalId"
    },
    "P9214": {
        "label": "FAQ URL",
        "description": "FAQ (frequently asked questions) page of the subject",
        "data_type": "Url"
    },
    "P9007": {
        "label": "Podcast Addict show ID",
        "description": "identifier for a podcast on Podcast Addict",
        "data_type": "ExternalId"
    },
    "P9008": {
        "label": "TuneIn Podcasts show ID",
        "description": "identifier for a podcast on TuneIn Podcasts",
        "data_type": "ExternalId"
    },
    "P9009": {
        "label": "RadioPublic show ID",
        "description": "identifier for a podcast on RadioPublic",
        "data_type": "ExternalId"
    },
    "P9010": {
        "label": "Player.fm show ID",
        "description": "identifier for a podcast on Player.fm",
        "data_type": "ExternalId"
    },
    "P9011": {
        "label": "PodBean show ID",
        "description": "identifier for a podcast on PodBean",
        "data_type": "ExternalId"
    },
    "P9012": {
        "label": "Podtail show ID",
        "description": "identifier for a podcast on the podcasting hosting website Podtail",
        "data_type": "ExternalId"
    },
    "P9014": {
        "label": "The Podcast App show ID",
        "description": "identifier for a podcast on The Podcast App",
        "data_type": "ExternalId"
    },
    "P9023": {
        "label": "Radio.com Podcasts show ID",
        "description": "identifier for a podcast on Radio.com Podcasts",
        "data_type": "ExternalId"
    },
    "P9024": {
        "label": "Libsyn show ID",
        "description": "identifier for a podcast on Libsyn",
        "data_type": "ExternalId"
    }
}